#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : dtmf.tcl                                                    #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : Set up the proceedures and data structures to support DTMF  #
#                                                                           #
#############################################################################

package require Parse_list

namespace eval DTMF \
{
  #***************************************************************************
  #
  # Name:        Compose_digit
  #
  # Parameters:  digit - digit to send
  #              dur   - duration of the tone
  #
  # Description: Return the tone sequence list for a given DTMF digit
  #
  #****************************************************************************
  proc Compose_digit { digit { dur 0 } } \
  {
    variable FREQS
  
    set digit  [string tolower $digit]
  
    if { ![info exists FREQS($digit)] } \
    {
      return -code error "BAD DTMF DIGIT $dur"
    }

    set x $FREQS($digit)
    
    lappend x DTMF $dur 
    
    return $x
  }
  
  
  #***************************************************************************
  #
  # Name:        Compose
  #
  # Parameters:  Digits to send
  #
  # Description: Return list of sequence commands for a DTMF string
  #
  #***************************************************************************
  proc Compose { args } \
  {
    Parse_list DTMF::Compose_core $args
  }


  #***************************************************************************
  #
  # Name:        Compose_core
  #
  # Parameters:  zero or more digit strings
  #
  # Description: Compose the list for a single string of digits
  #
  #***************************************************************************
  proc Compose_core { args } \
  {
    variable Timing
  
    set seq {}
  
    foreach { arg } $args \
    {
      set arg  [string tolower $arg]
    
      set digits  [split $arg {}]
    
      foreach { d } $digits \
      {
        # look for special timing chars..
        if { [info exists Timing($d)] } \
        {
          if {$Timing($d) != 0 } \
          {
            lappend seq [Compose_digit $d $Timing($d)]
          }
        } \
        else \
        {
          lappend seq [Compose_digit $d $Timing(MARK)]
          lappend seq [Compose_digit W  $Timing(SPACE)]
        }
      }
    
    }
  
    # Remove the last SPACE TIMING item in order to add the
    # END TIMING to the end of the list
    set last_index  [expr [llength  $seq] - 1]
    
    set seq  [lreplace  $seq  $last_index  $last_index]
    
    lappend seq  [Compose_end]

    return $seq
  }


  #***************************************************************************
  #
  # Name:        Compose_end
  #
  # Parameters:
  #
  # Description: Create the end of digit string delay
  #
  #***************************************************************************
  proc Compose_end { } \
  {
    variable Timing

    return [Compose_digit W $Timing(END) ]
  }
  

  #***************************************************************************
  #
  # Name:        Compose_busy
  #
  # Parameters:  count - number of iterations
  #
  # Description: Return list of sequence commands for US busy signal
  #
  #****************************************************************************
  proc Compose_busy { { count 3 } } \
  {
    set seq {}
  
    for {set i 0} { $i < $count } { incr i } \
    {
       lappend seq [Compose_digit Z 500]
       lappend seq [Compose_digit W 500]
    }
    
    return $seq  
  }
  
  
  #***************************************************************************
  #
  # Name:        Compose_SIT
  #
  # Parameters:
  #
  # Description: Return list of sequence commands for SIT (number not available)
  #
  #***************************************************************************
  proc Compose_SIT {} \
  {
    return { {950 {} SINE 330} { 0 {} SINE 20 } {1400 {} SINE 330} {0 {} SINE 20 } {1800 {} SINE 330} }
  }

};  # End of DTMF: namespace

