#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : to_dvm.tcl                                                  #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Johnson                                               #
# DESCRIPTION : Interface to DVM via Message Bus                            #
#                                                                           #
#############################################################################

#
#  DVM System Number
#
set DVM_ENGINE(SYSTEM_ID)         4

#
#  DVM Sub-Systems
#
set DVM_ENGINE(MTR_READING)       1
set DVM_ENGINE(MTR_MODE)          2
set DVM_ENGINE(MTR_ULVAL)         3
set DVM_ENGINE(MTR_LLVAL)         4
set DVM_ENGINE(MTR_PHRST)         5
set DVM_ENGINE(MTR_AVERAGE)       6 
set DVM_ENGINE(MTR_TRIG)          7
set DVM_ENGINE(MTR_TRIG_ARM)      8
set DVM_ENGINE(MTR_MATH)          9
set DVM_ENGINE(MTR_IMPEDANCE)    10
set DVM_ENGINE(MTR_REF)          11
set DVM_ENGINE(MTR_INFO)         12
set DVM_ENGINE(MTR_ENABLE)       13
set DVM_ENGINE(MTR_RANGE)        14
set DVM_ENGINE(MTR_HOLD)         15
set DVM_ENGINE(MTR_SOURCE)       16
set DVM_ENGINE(MTR_50HZ)         17
set DVM_ENGINE(MTR_RUN)          18
set DVM_ENGINE(MTR_ALARM)        19
set DVM_ENGINE(MTR_PEAKS)        20


font create dvm_font -family helvetica -size 10

#
#  Receiver Call-back Data
#
set DVM_STATUS(reading)     -1
set DVM_STATUS(alarm)       -1
set DVM_STATUS(holding)     -1
set DVM_STATUS(zero)        -1
set DVM_STATUS(range)       -1
set DVM_STATUS(overvoltage) -1
set DVM_STATUS(cmdchange)   -1


#
#  Dvm Helpful globals
#
set DVM_ENGINE(Handler_alarm)      0
set DVM_ENGINE(Dvm_live)           0




#############################################################################
#                                                                           #
#                       Message Bus Interfaces                              #
#                                                                           #
#############################################################################


#
#  dvm_stat_continuity - high if input voltage below continuity threshold (100 mV)
#
proc dvm_stat_continuity { stat } { 
  if { $stat } {
    return "Continuity"
  } 

  return "Open Circuit"
}


#
#  dvm_stat_holding - high if live reading, else hold mode
#
proc dvm_stat_holding { stat } { 
  if { $stat } {
    return "Live Reading"
  } 

  return "Hold Reading"
}


#
#  dvm_stat_lowbattery - high if battery condition is low, else batttery ok
#
proc dvm_stat_lowbattery { stat } { 
  if { $stat } {
    return "Low"
  } 

  return "OK"
}

#
#  dvm_stat_zero - indicate when the DVM is doing a zero operation
#
proc dvm_stat_zero { stat } {
  if { $stat } {
    return "Zero Active"
  } 

  return "Zero Disable"
}

#
#  dvm_cmd_change - has changed to indicate that HW settings have changed
#
proc dvm_cmd_change { stat } {
  if { $stat } {
    return "HW Setting Changed"
  } 

  return "HW Stable"
}


#
#  dvm_stat_zero - indicate when the DVM is doing a zero operation
#
proc dvm_stat_range { stat } {
  if { $stat == 0x1 } {
    return "400 mV"
  } 

  if { $stat == 0x2 } {
    return "4 V"
  }

  if { $stat == 0x3 } {
    return "40 V"
  }

  if { $stat == 0x4 } {
    return "400 V"
  }

  return "Auto"
}


#
#  dvm_stat_zero - indicate when the DVM is doing a zero operation
#
proc dvm_stat_overvolt { stat } {
  if { $stat } {
    return "Over Voltage!"
  } 

  return "Within Range"
}

#
#  dvm_load_disconn - indicate when the DVM is disconnected
#
proc dvm_load_disconn { stat } {
  global DvmButton4

  if { $stat } {
    return "Load Overvoltage!"
  } 

  return "Load Voltage Ok"
}



#
#  Message bus callback - fill in supplied structure
#
#       0x000007 = Range Setting:
#                   1=400mV, 2=4V, 3=40V, 4=400V.
#       0x000010 = Input OverVoltage:
#                   0=Good input voltage, 1=Overvoltage condition!
#       0x000f00 = Hold Reading (inverse of $80000D bit1):
#                   0=Live reading, 1=Holding reading.
#                   0x100 = low battery
#                   0x200 = holding
#                   0x400 = latched continuity
#                   0x800 = always '1'
#       0x001000 = Changed Command Settings:
#                   0=No changes, 1=Setting(s) changed since last reading.
#       0x010000 = Start/Stop Readings:
#                   0=Stop, 1=Start.
#       0x100000 = Zeroing Reading
#                   0=Normal reading, 1=zeroing reading
#       0x200000 = Input Load Disconnected (OverVoltage from $800002 bit7)
#                   0=load connected, 1=load disconnected!
# 
#
proc handle_dvm_reading {sys sub lun flags uid volts status } {
  global DVM_STATUS
  global DvmButton4

#puts "dvm reading: $volts $status"

  if { $status & 0x100000 } { 
    # do not update reading when DVM is doing zero operation
  } else {
    set DVM_STATUS(reading)     $volts
  }

  set DVM_STATUS(holding)     [ dvm_stat_holding  [ expr { $status & 0x200     } ] ]
  set DVM_STATUS(zero)        [ dvm_stat_zero     [ expr { $status & 0x100000  } ] ]
  set DVM_STATUS(range)       [ dvm_stat_range    [ expr { $status & 0x7       } ] ]
  set DVM_STATUS(overvoltage) [ dvm_stat_overvolt [ expr { $status & 0x10      } ] ]
  set DVM_STATUS(cmdchange)   [ dvm_cmd_change    [ expr { $status & 0x1000    } ] ]
  set DVM_STATUS(disconnect)  [ dvm_load_disconn  [ expr { $status & 0x200000  } ] ]

  if { $status & 0x200000 } {
    if { $DvmButton4 != "LOADOFF" } {
      set DvmButton4 LOADOFF
      set_dvm_load LOADOFF
    }
  }
}

#
#  Message bus callback - fill in supplied structure
#
proc handle_dvm_alarm {sys sub lun flags uid alarm } {
  global DVM_STATUS

puts "dvm msg:  $alarm"

  set DVM_STATUS(alarm)  $alarm
}

#
#  Message bus interfaces for messages being generated FROM rcvr engine
#
proc create_dvm_callback { } {
  global DVM_ENGINE

  set DVM_ENGINE(Dvm_live)  [tclMsgbus rcv handle \
                            -sys $DVM_ENGINE(SYSTEM_ID) \
                            -sub $DVM_ENGINE(MTR_READING) \
                            -lun 1 -cmd "handle_dvm_reading" ]
  set DVM_ENGINE(Handler_alarm)  [tclMsgbus rcv handle \
                            -sys $DVM_ENGINE(SYSTEM_ID) \
                            -sub $DVM_ENGINE(MTR_ALARM) \
                            -lun 1 -cmd "handle_dvm_alarm" ]
}


#
#  Message bus interfaces - removed from use
#
proc destroy_dvm_callback { } {
  global DVM_ENGINE

  if {$DVM_ENGINE(Handler_alarm) != 0 } {
    tclMsgbus $DVM_ENGINE(Handler_alarm) destroy
  }

  if {$DVM_ENGINE(Dvm_live) != 0 } {
    tclMsgbus $DVM_ENGINE(Dvm_live) destroy
  }

  set DVM_ENGINE(Handler_alarm) 0
  set DVM_ENGINE(Dvm_live)      0
}


#############################################################################
#                                                                           #
#                       Engine Interfaces                                   #
#                                                                           #
#############################################################################

#
# toggle Zero
#
proc set_dvm_zero { x } {
  global DVM_ENGINE
 
  switch $x {
    ZEROENABLE  { set zero 1 }
    ZEROENABLE0 { set zero 0 }
   }

  message_bus_int $DVM_ENGINE(SYSTEM_ID) $DVM_ENGINE(MTR_MATH) 1 $zero
}


#
# toggle Hold
#
proc set_dvm_hold { x } {
  global DVM_ENGINE
 
  switch $x {
    HOLDENABLE { set hold 1 }
    HOLDNORMAL { set hold 0 }
   }

  message_bus_int $DVM_ENGINE(SYSTEM_ID) $DVM_ENGINE(MTR_HOLD) 1 $hold
}


#
# set DVM load
#
proc set_dvm_load { x } {
  global DVM_ENGINE
  
  switch $x {
    LOADOFF { set load 0   }
    LOAD150 { set load 150 }
    LOAD600 { set load 600 }
  }

  message_bus_int $DVM_ENGINE(SYSTEM_ID) $DVM_ENGINE(MTR_IMPEDANCE) 1 $load
}


#
# set dvm range
#
proc set_dvm_range { x } {
  global DVM_ENGINE

  switch $x {
    RAUTO  { set range 0    }
    R400MV { set range -400 }
    R4V    { set range 4    }
    R40V   { set range 40   }
    R400V  { set range 400  }
  }

  message_bus_int $DVM_ENGINE(SYSTEM_ID) $DVM_ENGINE(MTR_RANGE) 1 $range
}

#
# set dvm mode
#
proc set_dvm_mode { x } {
  global DVM_ENGINE

  switch $x {
    DC { set mode 2 }
    AC { set mode 1 }
  }

  message_bus_int $DVM_ENGINE(SYSTEM_ID) $DVM_ENGINE(MTR_MODE) 1 $mode
}


#
# set dvm signal source
#
proc set_dvm_source { x } {
  global DVM_ENGINE

  switch $x {
    DVM    { set source 0 }
    SCOPE1 { set source 1 }
    SCOPE2 { set source 2 }
  }

  message_bus_int $DVM_ENGINE(SYSTEM_ID) $DVM_ENGINE(MTR_SOURCE) 1 $source
}

#
# set dvm 50/60 Hz line signal
#
proc set_dvm_50hz { x } {
  global DVM_ENGINE

  switch $x {
    HZ50   { set hz 50 }
    HZ60   { set hz 60 }
  }

  message_bus_int $DVM_ENGINE(SYSTEM_ID) $DVM_ENGINE(MTR_50HZ) 1 $hz
}


#
# start,stop DVM engine/DSP
#
proc dvm_run { x } {
  global DVM_ENGINE

  switch $x {
    ON  { 
          message_bus_int $DVM_ENGINE(SYSTEM_ID) $DVM_ENGINE(MTR_ENABLE) 1 3 
          message_bus_int $DVM_ENGINE(SYSTEM_ID) $DVM_ENGINE(MTR_RUN)    1 1
        }

    OFF { message_bus_int $DVM_ENGINE(SYSTEM_ID) $DVM_ENGINE(MTR_RUN)    1 0 }
  }
}





