#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : afcounter.tcl                                               #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the AFCounter command is   #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::AFCounter::State ruw "::AFCounter::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
if {"[info command AFCounter]" != "" } { return }

# Call the base class dummy command to force a load
package require Engtclbase
package require BBAudio
Engtclbase

# Load the object file
::load $::AFCounter::object

package provide AFCounter 1.0

#****************************************************************************
# Add functions to the AFCounter namespace
#****************************************************************************
#    Status of commands sent to AFCounter
#       ::AFCounter::State(Engine_run)          {0 | 1}
#       ::AFCounter::State(run)                 {OFF | ON}
#       ::AFCounter::State(average)             {1 - 100}
#       ::AFCounter::State(Engine_enabled)      {0 | 1}
#       ::AFCounter::State(enabled)             {OFF | ON}
#       ::AFCounter::State(Engine_gate)         {0 | 1}
#       ::AFCounter::State(gate)                {OFF | ON}
#       ::AFCounter::State(trig)                {0 | 1}
#       ::AFCounter::State(driverId)            Driver ID information
#       ::AFCounter::State(engineId)            Engine ID information
#    Status of counter data received from the engine
#       ::AFCounter::State(count)               Current counter value
#       ::AFCounter::State(reading)             Current counter value
#       ::AFCounter::State(precision)           Current precision of count
#       ::AFCounter::State(accuracy)            Eventual precision of count
#       ::AFCounter::State(status)              Status flags
#    Status of alert limits and peak values
#       ::AFCounter::State(peakHi)              Maximum peak reading
#       ::AFCounter::State(peakLo)              Minimum peak reading
#       ::AFCounter::State(peakFlags)           Peak Hold status flags
#       ::AFCounter::State(peakEnable)          Peak Enable settings
#       ::AFCounter::State(alertHi)             Maximum valid reading limit
#       ::AFCounter::State(alertLo)             Minimum valid reading limit
#       ::AFCounter::State(alerts)              Alert flag status
#****************************************************************************
namespace eval ::AFCounter {
   variable SaveSortOrder AFTER(BBAudio)
#****************************************************************************
# Name:        set_afctr_callbacks
# Parameters:  none
# Description: Sets up the callbacks to update the AF Counter data
#****************************************************************************
 proc set_afctr_callbacks { } {

   if { $::AFCounter::callbackStatus != 0 } {
       return
   }

   AFCounter -onchange alerts   { ::AFCounter::updateAlerts %V}

   AFCounter -onchange peak     { ::AFCounter::updatePeakEnable %V}

   AFCounter -onchange avg      { set ::AFCounter::State(average) %V}
   AFCounter -onchange trig     { set ::AFCounter::State(trig) %V}

   AFCounter -onchange reading  { ::AFCounter::updateReading %V}
   AFCounter -onchange reading2 { ::AFCounter::updateReading2 %V}
   AFCounter -onchange readpeak { ::AFCounter::updatePeaks %V}
   AFCounter -onchange run      { ::AFCounter::updateMode %V}
   AFCounter -onchange run2     { ::AFCounter::updateMode2 %V}
   AFCounter -onchange gate     { ::AFCounter::updateGate %V}
   AFCounter -onchange en       { ::AFCounter::updateEnable %V}
   AFCounter -onchange en2      { ::AFCounter::updateEnable2 %V}
   AFCounter -onchange ulimit   { ::AFCounter::updateAlertHi %V}
   AFCounter -onchange llimit   { ::AFCounter::updateAlertLo %V}

   trace variable ::BBAudio::State(Route_15)  w  "AFCounter::updateInput"
   trace variable ::BBAudio::State(micbias)   w  "AFCounter::updateInput"
   trace variable ::BBAudio::State(sinadbal)  w  "AFCounter::updateInput"

   set ::AFCounter::callbackStatus 1
 }
 
 
#****************************************************************************
# Name:        updateInput
# Parameters:  args from trace (ignored)
# Description: Track changes in the Baseband to update our input source
#****************************************************************************
 proc updateInput { args } \
 {
    set cmd "AFCounter::updateInput_core"
    after cancel $cmd
    after idle $cmd
 }
 proc updateInput_core { } \
 {
    variable State
    switch -- $::BBAudio::State(Route_15) \
    {
       0   -
       10  { set State(input)  "DEMOD" }
       
       23  { set State(input)  "SQLCH" }
       
       1   { if { $::BBAudio::State(micbias) == 0 } \
             { 
               set State(input)  "MIC dyn"
             } \
             else \
             {
               set State(input)  "MIC elec"
             }
           }
        
       2   { if { $::BBAudio::State(sinadbal) == 0 } \
             { 
               set State(input)  "Aud Bal"
             } \
             else \
             {
               set State(input)  "AUDIO"
             }
           }
        
       3   { set State(input) "FILE" }
    }
    
    return
 }


#****************************************************************************
# Name:        updateMode
# Parameters:  runState         0 = Off, 1 = On
# Description: Sets up the current run state of the AF Counter
#****************************************************************************
 proc updateMode { runState } \
 {
   variable  State

   switch $runState \
   {
     0       { set State(run) OFF }
     1       { set State(run) ON }
     default { return }
   }
   
   set State(Engine_run) $runState
   
   return
 }
#****************************************************************************
# Name:        updateMode
# Parameters:  runState         0 = Off, 1 = On
# Description: Sets up the current run state of the AF Counter
#****************************************************************************
 proc updateMode2 { runState } \
 {
   variable  State

   switch $runState \
   {
     0       { set State(run2) OFF }
     1       { set State(run2) ON }
     default { return }
   }
   
   set State(Engine_run2) $runState
   
   return
 }


#****************************************************************************
# Name:        updateEnable
# Parameters:  enableBits       1 = Off, 3 = Enabled
# Description: Sets up the current enabled state of the AF Counter
#****************************************************************************
 proc updateEnable { enableBits } \
 {
   variable  State
   if {$enableBits & 2} \
   {
      set State(enabled) Enabled
   } \
   else \
   {
     set State(enabled) Disabled
   }
   
   set State(Engine_enabled) $enableBits
   
   return
 }


#****************************************************************************
# Name:        updateEnable
# Parameters:  enableBits       1 = Off, 3 = Enabled
# Description: Sets up the current enabled state of the AF Counter
#****************************************************************************
 proc updateEnable2 { enableBits } \
 {
   variable  State

   if {$enableBits & 4} \
   {
     set State(enabled2) Enabled 
   } \
   else \
   {
      set State(enabled2) Disabled
   }
   
   set State(Engine_enabled2) $enableBits
   
   return
 }
 
#****************************************************************************
# Name:        updateGate
# Parameters:  runState         0 = 0.1 second, 1 = 1.0 second
# Description: Sets up the current state of the AF Counter gate time
#****************************************************************************
 proc updateGate { gateState } \
 {
   variable  State

   switch $gateState \
   {
     0       { set State(gate) TenthSec }
     1       { set State(gate) OneSec }
     default { return }
   }
   
   set State(Engine_gate) $gateState
   
   return
 }


#****************************************************************************
# Name:        updateReading
# Parameters:  count            Frequency count from the engine
#              count_precision  Current digits of accuracy
#              status           Status flags
#              count_accuracy   Current MAXIMUM digits of accuracy
# Description: Sets up the current state of the AF Counter gate time
#****************************************************************************
 proc updateReading { readingList } {

   variable State

   # Get the data from the meter reading
   set count            [lindex $readingList 0]
   set State(precision) [lindex $readingList 1]
   set State(status)    [lindex $readingList 2]
   set State(accuracy)  [lindex $readingList 3]

   # Set the raw data reading variable
   # Used for bargraph, limit, peak, range, and alarm updates
   # where a valid number is necessary
   set State(reading_raw)  $count
   
   
   # Truncate information for low accuracy readings
   set temp10 [expr (int($count + 4.99) / 10) * 10 ]
   set temp1 [expr int($count)]

   # Format the count based on accuracy
   switch -- $State(accuracy) \
   {
     -1      { set tempCnt [format "%5d" $temp10 ] }
     0       { set tempCnt [format "%5d" $temp1 ] }
     1       { set tempCnt [format "%5.1f" $count] }
     2       { set tempCnt [format "%5.1f" $count] }
     default { return }
   }

   # If the measurement has not reached the available accuracy for 0.1 hz
   # readings, dash the last digit
   if { $State(accuracy) > 0 } \
   {
     if { $State(precision) < 1 } \
     {
       set idx [string first "." $tempCnt]
       set front [string range $tempCnt 0 $idx]

       set State(count)  $front-
         
       # Set the conditioned reading variable
       # Used for the display of the reading on the meter
       set State(reading_cond)  $front-
     } \
     else \
     {
       set State(count)  $tempCnt

       # Set the conditioned reading variable
       # Used for the display of the reading on the meter
       set State(reading_cond)  $tempCnt
     }
   } \
   else \
   {
     set State(count)  $tempCnt

     # Set the conditioned reading variable
     # Used for the display of the reading on the meter
     set State(reading_cond) $tempCnt
   }
   
   return
 }

#****************************************************************************
# Name:        updateReading2
# Parameters:  count            Frequency count from the engine
#              count_precision  Current digits of accuracy
#              status           Status flags
#              count_accuracy   Current MAXIMUM digits of accuracy
# Description: Sets up the current state of the AF Counter gate time
#****************************************************************************
 proc updateReading2 { readingList } {

   variable State

   # Get the data from the meter reading
   set count            [lindex $readingList 0]
   set State(precision2) [lindex $readingList 1]
   set State(status2)    [lindex $readingList 2]
   set State(accuracy2)  [lindex $readingList 3]

   # Set the raw data reading variable
   # Used for bargraph, limit, peak, range, and alarm updates
   # where a valid number is necessary
   set State(reading_raw2)  $count
   
   
   # Truncate information for low accuracy readings
   set temp10 [expr (int($count + 4.99) / 10) * 10 ]
   set temp1 [expr int($count)]

   # Format the count based on accuracy
   switch -- $State(accuracy2) \
   {
     -1      { set tempCnt [format "%5d" $temp10 ] }
     0       { set tempCnt [format "%5d" $temp1 ] }
     1       { set tempCnt [format "%5.1f" $count] }
     2       { set tempCnt [format "%5.1f" $count] }
     default { return }
   }

   # If the measurement has not reached the available accuracy for 0.1 hz
   # readings, dash the last digit
   if { $State(accuracy2) > 0 } \
   {
     if { $State(precision2) < 1 } \
     {
       set idx [string first "." $tempCnt]
       set front [string range $tempCnt 0 $idx]

       set State(count2)  $front-
         
       # Set the conditioned reading variable
       # Used for the display of the reading on the meter
       set State(reading_cond2)  $front-
     } \
     else \
     {
       set State(count2)  $tempCnt

       # Set the conditioned reading variable
       # Used for the display of the reading on the meter
       set State(reading_cond2)  $tempCnt
     }
   } \
   else \
   {
     set State(count2)  $tempCnt

     # Set the conditioned reading variable
     # Used for the display of the reading on the meter
     set State(reading_cond2) $tempCnt
   }
   #now try to find the code, given the freq. First, the direct approach...
   if [info exists ::CTCSS::Codes($count)] \
   {
      set State(reed_code) $::CTCSS::Codes($count)
   } \
   else \
   {
     # OK, not an exact match, so find the closest match.
     set delta 1000
     set err 1
     set code "unknown"
     foreach freq [array names ::CTCSS::Codes] \
     {
        set new_delta [expr abs($freq-$count)]
        if {$new_delta < $delta} \
        {
           set delta $new_delta
           set code $::CTCSS::Codes($freq)
           set err [expr $delta/$freq]
        }
     }
     if {$err > .2} \
     {
        set State(reed_code) "---"
     } \
     else \
     {
        set State(reed_code) $code
     }
   }
   # now, set desired freq & freq error
   if ![info exists ::CTCSS::Freqs($State(reed_code))] \
   {
      set State(reed_freq) "----"
      set State(reed_freq,error) "----"
   } \
   else \
   {
      set State(reed_freq) $::CTCSS::Freqs($State(reed_code))
      set State(reed_freq,error) [format %5.2f [expr 100.0*($State(reed_freq)-$count)/$State(reed_freq)]]
   }
   
   return
 }

#****************************************************************************
# Name:        updatePeaks
# Parameters:  readingList      List of data from the engine
#                 Peak HI       Max peak value
#                 HI Precision  Max peak precision
#                 Peak LO       Min peak value
#                 LO Precision  Min peak precision
#                 Flags         Status flags
# Description: Sets up the current state of the peak readings
#****************************************************************************
 proc updatePeaks { readingList } \
 {
   variable  State

   # Get the data from the peak readings
   set peakHi    [lindex $readingList 0]
   set peakLo    [lindex $readingList 2]

   set State(peakFlags) [lindex $readingList 4]

   set State(peakHi)    [format "%0.1f" $peakHi]
   set State(peakLo)    [format "%0.1f" $peakLo]
   
   return
 }


#****************************************************************************
# Name:        updatePeakEnable
# Parameters:  
# Description: Sets up the current state of the peak enable
#****************************************************************************
 proc updatePeakEnable { peakEnable } \
 {
   variable  State

   set State(peakEnable)  $peakEnable

   if { $State(peakEnable) == 1 } \
   {
     set State(peakEnable_text)  "ON"
   }

   if { $State(peakEnable) == 0 } \
   {
     set State(peakEnable_text)  "OFF"
   }

 }


#****************************************************************************
# Name:        updateAlerts
# Parameters:  
# Description: Update the Alerts flags
#****************************************************************************
 proc updateAlerts { alerts } \
 {
   variable State
 
   set State(alerts)  $alerts
   if {$alerts & 2} \
   {
     set State(alertHi_text) "ON"
   } \
   else \
   {
     set State(alertHi_text) "OFF"
   }
   if {$alerts & 1} \
   {
     set State(alertLo_text) "ON"
   } \
   else \
   {
     set State(alertLo_text) "OFF"
   }
   if {$alerts & 4} \
   {
     set State(alarm_text)       "ON"
     set State(BellEnabled_text) "ON"

     set State(BellEnabled) 1
   } \
   else \
   {
     set State(alarm_text)       "OFF"
     set State(BellEnabled_text) "OFF"

     set State(BellEnabled) 0
   }
 }


#****************************************************************************
# Name:        updateAlertHi
# Parameters:  
# Description: Formats the Alert High value
#****************************************************************************
 proc updateAlertHi { alertHi } \
 {
   variable  State

   set State(alertHi) [format "%0.1f" $alertHi]

 }


#****************************************************************************
# Name:        updateAlertLo
# Parameters:  
# Description: Formats the Alert Low value
#****************************************************************************
 proc updateAlertLo { alertLo } \
 {
   variable  State

   set State(alertLo) [format "%0.1f" $alertLo]

 }


#********************************************************************
#
#       Name: setLimits
#
# Parameters:
#
# Decsription: Sets the limit values
#
#********************************************************************
proc setLimits { limit_type } \
{
  variable  State
  
  if { $limit_type == "high" } \
  {
    set temp  [expr $State(alerts) & 0x02]
    
    # If the limit alert is turned OFF, turn it ON
    if { $temp == 0 } \
    {
      set temp  [expr $State(alerts) | 0x02]
      
      AFCounter alerts  $temp  -nowait
    }
    
    AFCounter ulimit  $State(alertHi)  -nowait
  }

  if { $limit_type == "low" } \
  {
    set temp  [expr $State(alerts) & 0x01]
    
    # If the limit alert is turned OFF, turn it ON
    if { $temp == 0 } \
    {
      set temp  [expr $State(alerts) | 0x01]
      
      AFCounter alerts  $temp  -nowait
    }
    
    AFCounter llimit  $State(alertLo)  -nowait
  }

  return
}  


#********************************************************************
#
#       Name: toggleAlerts
#
# Parameters:
#
# Decsription: Toggles the state of the alerts
#
#********************************************************************
proc toggleAlerts { limit_type } \
 {
   variable  State
   
   if { $limit_type == "high" } \
   {
     set new  [expr $State(alerts) ^ 2]
    
     AFCounter alerts  $new  -nowait
   }

   if { $limit_type == "low" } \
   {
     set new  [expr $State(alerts) ^ 1]
    
     AFCounter alerts  $new  -nowait
   }
  
   return
 }


#********************************************************************
#
#       Name: toggleAlarm
#
# Parameters:
#
# Decsription: Toggles the state of the alarm
#
#********************************************************************
proc toggleAlarm { } \
 {
   variable State
   AFCounter alerts [expr $State(alerts) ^ 4] -nowait
 }


#********************************************************************
#
#       Name: togglePeakEnable
#
# Parameters: 
#
# Decsription: 
#
#********************************************************************
proc togglePeakEnable { } \
 {
   variable  State

   if { $State(peakEnable) == 1 } \
   {
     after idle "AFCounter peak 0 -nowait"
   }
  
   if { $State(peakEnable) == 0 } \
   {
     after idle "AFCounter phreset  -nowait"
     after idle "AFCounter peak 1   -nowait"
   }

   return    
 }


#********************************************************************
#
#       Name: toggleResolution
#
# Parameters: 
#
# Decsription: 
#
#********************************************************************
proc toggleResolution { } \
 {
   variable  State
  
   if { $State(resolution) == 0.1 } \
   {
     set State(resolution)  1.0
   } \
   else \
   {
     set State(resolution)  0.1
   }


   if { $State(resolution) == 0.1 } \
   {
     # Set the meter resolution (gate time) to 0.1
     # Set average to 100
     after idle "AFCounter gate  TENTH  -nowait"
     after idle "AFCounter average 100  -nowait"
   }
  
   if { $State(resolution) == 1.0 } \
   {
     # Set the meter resolution (gate time) to 0.1
     # Set average to 10
     after idle "AFCounter gate  TENTH  -nowait"
     after idle "AFCounter average  20  -nowait"
   }

   return
 }


#********************************************************************
#
#       Name: setAverage
#
# Parameters: new_value   - requested new value 
#                           -1 - no new value requested
#
# Description: set the meter average
#
#********************************************************************
proc setAverage { } \
 {
   # Set the meter engine
   after idle "AFCounter  average  $::AFCounter::State(average)  -nowait"
    
   return
 }


#****************************************************************************
# Name:        query_afctr_state
# Parameters:  none
# Description: This function sends down a request to the AF Counter engine to 
#            : obtain the engine state 
#****************************************************************************
 proc query_afctr_state { } {

   AFCounter run  
   AFCounter gate 
   AFCounter avg
   AFCounter en
   AFCounter trig
   AFCounter ulim
   AFCounter llim
   AFCounter al
   AFCounter peak
   AFCounter alerts

   set ::AFCounter::State(engineId) [format_version [AFCounter ver 0] ]
   set ::AFCounter::State(driverId) [format_version [AFCounter ver 1] ]
 }


#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the AF Counter engine to its current state
#****************************************************************************
 proc getSaveState { } {
   
   variable State
 
   set returnList \
   {
     AFCounter run    $State(Engine_run)      -nowait
     AFCounter run2   $State(Engine_run2)     -nowait
     AFCounter gate   $State(Engine_gate)     -nowait
     AFCounter avg    $State(average)         -nowait 
     AFCounter trig   $State(trig)            -nowait 
     AFCounter en     $State(Engine_enabled)  -nowait
     AFCounter en2    $State(Engine_enabled2) -nowait
     AFCounter peak   $State(peakEnable)      -nowait
     AFCounter alerts $State(alerts)          -nowait 
     AFCounter ulim   $State(alertHi)         -nowait 
     AFCounter llim   $State(alertLo)         -nowait

     set ::AFCounter::State(BellEnabled)   {$State(BellEnabled)}
     set ::AFCounter::State(resolution)    {$State(resolution)}
     set ::AFCounter::State(units)         {$State(units)}

     AFCounter::Input {$State(input)}
   }
       
   return [subst -nocommands $returnList]
 }

#****************************************************************************
# Name:        Input
# Parameters:  input source
# Description: Select input source
#****************************************************************************
 proc Input { source } \
 {
    set cmd [list AFCounter::Input_core $source]
    after cancel $cmd
    after idle $cmd
 }
 proc Input_core { source } \
 {
    variable State
    switch -- $source \
    {
       "DEMOD"    {BBAudio route L1        AFCTR}
       "SQLCH"    {BBAudio route SQUELCHED AFCTR}
       "MIC dyn"  {BBAudio route MIC   AFCTR; BBAudio micbias dyn}
       "MIC elec" {BBAudio route MIC   AFCTR; BBAudio micbias elec}
       "Aud Bal"  {BBAudio route SINAD AFCTR; BBAudio sinadbalance balanced}
       "AUDIO"    {BBAudio route SINAD AFCTR; BBAudio sinadbalance unbalanced}
       "FILE"     {BBAudio route FILE  AFCTR}
    }
 }

#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to hardware
#****************************************************************************
 proc Sync { } {
    after 1000 { eval [AFCounter::getSaveState]}
 }

#****************************************************************************
# Name:        DefaultSetup
# Parameters:  none
# Description: Sets the AFCounter engine with default values
#****************************************************************************
 proc DefaultSetup { } {
    AFCounter gate TenthSec     ;# Set gate time to 0.1 second
    AFCounter avg 20            ;# Set average to give 1 Hz accuracy
    AFCounter trig continuous   ;# Enable continuous readings
 }

#****************************************************************************
# Name:        QuickStart
# Parameters:  none
# Description: Turns on the AFCounter engine but does not set up parameters 
#****************************************************************************
 proc QuickStart { } {
    AFCounter en 3 -nowait       ;# Enable data to engine
    AFCounter run ON -nowait     ;# Start the engine
 }
 proc QuickStart2 { } {
    AFCounter en2 5 -nowait     ;# Enable data to engine
    AFCounter run2 ON -nowait    ;# Start the engine
 }

#****************************************************************************
# Name:        QuickStop
# Parameters:  none
# Description: Handles a normal stop of the AFCounter engine 
#****************************************************************************
 proc QuickStop { } {
    AFCounter en 1  -nowait             ;# Disable data to engine
    AFCounter run OFF -nowait           ;# Stop the engine
 }
proc QuickStop2 { } {
    AFCounter en2 1  -nowait             ;# Disable data to engine
    AFCounter run2 OFF -nowait           ;# Stop the engine
 }


#****************************************************************************
# Name:        SyncAll
# Parameters:  none
# Description: Insures that TCL, the engine and hardware are in sync
#****************************************************************************
 proc SyncAll { } {
    set_afctr_callbacks
    query_afctr_state
    if ![interp issafe] \
    {
       ::AFCounter::Sync
    }
 }
 
#****************************************************************************
# Name:        Get_HTML_reading
# Parameters:  base_url - the base URL for the containing page
# Description: Return an string containing HTML showing the current state
#              of this system
#****************************************************************************
 proc Get_HTML_reading { {base_url ""} } \
 {
    variable State
    set x \
{<tr>
<td nowrap>$State(input) frequency</td><td nowrap>$State(reading_cond) Hz</td>
</tr>}
    if {$State(Engine_run2) != 0}\
    {
       append x \
{<tr>
<td nowrap>RX CTCSS audio frequency</td><td nowrap>$State(reading_cond2) Hz</td>
</tr>}
    }
    return [subst $x]
 }
 
#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
   Load::on_system_done { ::AFCounter::SyncAll }

}  ;# End of the AF Counter namespace

#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines AFCounter
}

#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
     Afctr_Eng "AF Counter Engine"
     Afctr_Drv "AF Counter Driver"
   }
   set version(Afctr_Eng) [AFCounter version 0]
   set version(Afctr_Drv) [AFCounter version 1]
}

namespace eval ::HTTPD \
{
   variable main_screen_readings
   lappend  main_screen_readings(1) AFCounter
}

