#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : analyzer.tcl                                                #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the Analyzer command is    #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::Analyzer::State ruw "::Analyzer::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
if {"[info command Analyzer]" != "" } { return }

# Call the base class dummy command to force a load
package require Engtclbase
Engtclbase

package require Generator

# Load the object file
::load $::Analyzer::object

package provide Analyzer 1.0

#****************************************************************************
# Add functions to the Analyzer namespace
#****************************************************************************
#    Status of commands sent to Analyzer
#       ::Analyzer::State(Engine_run)          {1 | 2 | 3}
#       ::Analyzer::State(run)                 {START_CHAN | START_FULL | STOP}
#       ::Analyzer::State(Engine_enabled)      {0 | 1}
#       ::Analyzer::State(driverId)            Driver ID information
#       ::Analyzer::State(engineId)            Engine ID information
#    Status of Analyzer data received from the engine
#       ::Analyzer::State(mode_string)         current Analyzer mode
#       ::Analyzer::State(scale)               0.1 to 30.0
#       ::Analyzer::State(scale$i) {i=1-8}     Analyzer scale label values
#       ::Analyzer::State(tos)                 -99.9 to 80.0
#       ::Analyzer::State(sweep_time)
#       ::Analyzer::State(sweep_type)
#       ::Analyzer::State(sweep)
#       ::Analyzer::State(rbw)             
#       ::Analyzer::State(Engine_rbw)
#       ::Analyzer::State(vbw)             
#       ::Analyzer::State(Engine_vbw)
#       ::Analyzer::State(span)             
#       ::Analyzer::State(span_f1)
#       ::Analyzer::State(span_f2)          
#       ::Analyzer::State(tgstate)             
#       ::Analyzer::State(normalize)             
#****************************************************************************
namespace eval ::Analyzer {

# define the state number to string translation
   variable normalize_states
   array set normalize_states \
   {
     0 "Idle/Complete"
     1 "Reconfig Analyzer"
     2 "Setup RF Gen"
     4 "Adjusting RF Gen for Reference Level"
     5 "Adjust TOS for Cal Point"
     6 "Saving Compensation Value"
     7 "Doing Next RBW"
     8 "Cal Complete"
     9 "Cal Error"
     10 "User Aborted"
     13 "Adjusting Trace to match Ref Level"
     14 "Power Term Reading Zero Err"
     15 "IOCTL Error"
     16 "Power Term Reading Converge Err"
     17 "Gain Adjust Ref Error"
     18 "Tweak IF Gain"
     20 "Config Analyzer"
     21 "Set Analyzer TOS"
     22 "Find 0 dB DAC Step"
     23 "Find 1 dB DAC Steps"
     24 "Gain Adjust done"
     25 "Adjusting Gain"
     26 "Gain Adjustment Complete"
     27 "Gain Calibration Skipped"
     28 "Save state requested"
   }
   
   variable span_strings
   array set span_strings \
   {
      0.000     "0 SPAN"
      0.001     "1 kHz"
      0.002     "2 kHz"
      0.005     "5 kHz"
      0.010     "10 kHz"
      0.020     "20 kHz"
      0.050     "50 kHz"
      0.100     "100 kHz"
      0.200     "200 kHz"
      0.500     "500 kHz"
      1.000     "1 MHz"
      2.000     "2 MHz"
      5.000     "5 MHz"
      10.000    "10 MHz"
      20.000    "20 MHz"
      50.000    "50 MHz"
      100.000   "100 MHz"
      200.000   "200 MHz"
      500.000   "500 MHz"
      1000.000  "1 GHz"
      2000.000  "2 GHz"
      3000.000  "3 GHz"
   }

   variable span_values
   variable spans
   foreach spanval [lsort -real [array names span_strings]] \
   {
      set span_string $span_strings($spanval)
      set span_values($span_string) $spanval
      lappend spans $span_string
   }

   set State(span) $span_strings(0.000)
   
   # Maximum allowable loss due to oversweeping the filters (dB)
   set State(max_uncal_loss) .5
   
   # allowed sweep rates, in seconds
   set State(sweep_list)  { .005 .01 .02 .05 .1 .2 .5 1 2 5 10} 
   
   # possible video bandwidths, in Hertz
   set State(vbw_list) { 10 30 100 300 1000 3000 10000 30000 100000 300000 1000000 3000000 0}

#****************************************************************************
# Name:        toggle_auto_couple
# Parameters:  none
# Description: toggle the analyzer auto couple mode
#****************************************************************************
  proc toggle_auto_couple { } \
  {
     variable State
     if {$State(auto_couple) == 0} \
     {
        Analyzer autocpl 1 -nowait
     } \
     else \
     {
        Analyzer autocpl 0 -nowait
     }
  }
  
#****************************************************************************
# Name:        update_auto_couple
# Parameters:  none
# Description: update the analyzer auto couple mode
#****************************************************************************
  proc update_auto_couple { {state ""}} \
  {
     variable State
     if {$state != ""} { set State(auto_couple) $state}
     
     if {$State(auto_couple) == 0}\
     {
        set State(auto_couple,text) MAN
     } \
     else \
     {
        set State(auto_couple,text) AUTO
     }
  }
  
#****************************************************************************
# Name:        set_analyzer_callbacks
# Parameters:  none
# Description: Sets up the callbacks to update the Analyzer data
#****************************************************************************
 proc set_analyzer_callbacks { } \
 {
   Analyzer -onchange run                  { ::Analyzer::updateRunMode %V }

   Analyzer -onchange enable               { ::Analyzer::updateEnable %V }

   Analyzer -onchange scale                { ::Analyzer::updateScale %V }
   #Analyzer -onchange topofscreen          { puts %V }
   Analyzer -onchange sweep                { ::Analyzer::updateSweep %V }
   Analyzer -onchange resolutionbandwidth  { ::Analyzer::updateRBW %V }
   Analyzer -onchange videobandwidth       { ::Analyzer::updateVBW %V }
   Analyzer -onchange span                 { ::Analyzer::updateSpan %V }
   Analyzer -onchange center               { ::Analyzer::updateCenter %V }
   Analyzer -onchange tos_info             { ::Analyzer::updateTOSinfo %V }

   Analyzer -onchange trigmode   { ::Analyzer::updateTriggerMode %V }
   Analyzer -onchange triglevel  { ::Analyzer::updateTriggerLevel %V }
   Analyzer -onchange trigedge   { ::Analyzer::updateTriggerSlope %V }
   Analyzer -onchange trigdepth  { ::Analyzer::updateTriggerHorizPos %V }
   Analyzer -onchange autocpl    { ::Analyzer::update_auto_couple %V }

   set ::Analyzer::callbackStatus 1
 }
 proc toggle_tgen { } \
 {
    variable State
    if {$State(tgstate)} \
    {
      Analyzer tgstate 0 -nowait
    }\
    else \
    {
      Analyzer tgstate 1 -nowait
    }
 }
 
#****************************************************************************
# Name:        updateTOSinfo
# Parameters:  TOS info
# Description: top of screen info
#****************************************************************************
proc updateTOSinfo { v } \
{
  set vv $v               ;# v dosen't exist after being used once, so copy

  variable State

  set tos     [ lindex $vv 0 ]
  set db_step [ lindex $vv 1 ]
  set atten   [ lindex $vv 2 ]
  set ifgain  [ lindex $vv 3 ]
  set cal     [ lindex $vv 4 ]

  set State(tos)           [ format "%3.1f" $tos     ]
  set State(stat_db_step)  [ format "%3.1f" $db_step ]
  set State(stat_ifgain)   [ format "%3.1f" $ifgain  ]
  set State(stat_atten)    [ expr int($atten)        ]

  if { $cal & 1 } \
  {
     set State(stat_cal) CALed
  } \
  else \
  {
     set State(stat_cal) UnCAL
  }

  updateScaleTOS

  #puts "updateTOSinfo $State(tos) $State(stat_db_step) $State(stat_ifgain) $State(stat_atten)"
}


#****************************************************************************
# Name:        updateCenter
# Parameters:  value of center of trace
# Description: save the value of the center of the trace
#****************************************************************************
  proc updateCenter { state } \
  {
    variable State
    set State(center_dBm,raw)  $state

    set State(center_dBm)  [format "% 6.1f" $state]
  }
  
  
#****************************************************************************
# Name:        Restore
# Parameters:  none
# Description: Restore state after normalize or TOS
#****************************************************************************
  proc Restore { } \
  {
     #puts "Restoring state"

     variable State
     
     if ![info exists State(saved_state)] { return }
     
     set cmd $State(saved_state)
    
     unset State(saved_state)
     
     if [catch {eval $cmd } err] \
     {
        puts "Error restoring state: $err $::errorInfo"
     }
  }
  
  

#****************************************************************************
# Name:        updateScale
# Parameters:  Engine scale data
# Description: This formats the analyzer scale data
#****************************************************************************
  proc updateScale { scale_value } \
  {
    variable State
    set State(scale)  [expr int($scale_value)]
    
    updateScaleTOS
  }


#****************************************************************************
# Name:        updateTOS
# Parameters:  Engine top of screen data
# Description: This formats the analyzer top of screen data
#****************************************************************************
  proc updateTOS { tos_value } \
  {
    variable State
    set State(tos)  [format "%.1f" $tos_value]

    updateScaleTOS
  }


#****************************************************************************
# Name:        updateScaleTOS
# Parameters:  none
# Description: This updates the Analyzer scale markers when the top of screen
#              or the scale changes
#****************************************************************************
  proc updateScaleTOS { } \
  {
    variable State
    set scale $State(scale)
    set tos   $State(tos)

    for { set i 1 } { $i <= 8 } { incr i 1 } \
    {
      set diff  [expr $i * $scale]
    
      set temp_scale  [expr $tos - $diff]
    
      set State(scale${i})  [format "%.1f" $temp_scale]
    }
  }

#****************************************************************************
# Name:        updateSweep
# Parameters:  Engine sweep data
# Description: This formats the analyzer sweep data
#****************************************************************************
  proc updateSweep { sweep_value } \
  {
    variable State

    set sweep_time         [lindex $sweep_value 0]
    set State(sweep_type)  [lindex $sweep_value 1]
    set State(sweep_time) $sweep_time

    set State(sweep_time,sec)  [expr $sweep_time/1000000.0]

    # recompute the freq rate

    if {$sweep_time < 1000} \
    {
        set State(sweep)  "$State(sweep_time) us"
    } \
    else \
    {
        set temp [expr $sweep_time / 1000]
        if {$temp < 1000} \
        {
            set State(sweep)  "$temp ms"
        } \
        else \
        {
            set temp [expr $temp / 1000]
            set State(sweep)  "$temp s"
        }
    }
  }


#****************************************************************************
# Name:        updateRBW
# Parameters:  Engine resolution bandwidth data
# Description: This formats the analyzer resolution bandwidth data
#****************************************************************************
  proc updateRBW { rbw_value } \
  {
    variable State
    #puts "updateRBW:  $rbw_value"

    set ::Analyzer::State(Engine_rbw)  $rbw_value

    switch -- $rbw_value \
    {
      300     { set rbw_string "300 Hz"  }
      3000    { set rbw_string "3 kHz"   }
      60000   { set rbw_string "60 kHz"  }
      30000   { set rbw_string "30 kHz"  }
      300000  { set rbw_string "300 kHz" }
      6000000 { set rbw_string "6 MHz"   }
    }

    set State(rbw)  $rbw_string
  }


#****************************************************************************
# Name:        updateVBW
# Parameters:  Engine video bandwidth data
# Description: This formats the analyzer video bandwidth data
#****************************************************************************
  proc updateVBW { vbw_value } \
  {
    variable State

    set State(Engine_vbw)  $vbw_value

    switch -- $vbw_value \
    {
      10      { set vbw_string "10 Hz"   }
      30      { set vbw_string "30 Hz"   }
      100     { set vbw_string "100 Hz"  }
      300     { set vbw_string "300 Hz"  }
      1000    { set vbw_string "1 kHz"   }
      3000    { set vbw_string "3 kHz"   }
      10000   { set vbw_string "10 kHz"  }
      30000   { set vbw_string "30 kHz"  }
      100000  { set vbw_string "100 kHz" }
      300000  { set vbw_string "300 kHz" }
      1000000 { set vbw_string "1 MHz"   }
      3000000 { set vbw_string "3 MHz"   }
      0       { set vbw_string "NONE"    }
    }   

    set State(vbw)  $vbw_string
  }


#****************************************************************************
# Name:        updateRunMode
# Parameters:  run_mode_value The Analyzer "run" engine data
# Description: Format and save the Analyzer execution mode
#****************************************************************************
  proc updateRunMode { run_mode_value } \
  {
    #puts "updateRunMode:  $run_mode_value"

    variable State

    set State(Engine_run)  $run_mode_value

    switch -- $run_mode_value \
    {
      1  {
           set State(run) "START_CHAN"
           check_valid_span
         }

      2  {
           set State(run) "START_FULL"
           check_valid_span
           if {$State(tgstate) == 1} \
           {
             Receiver freq ;#update tg freq at need
           }
         }

      3  { set State(run) "STOP" }
    }
  }
  
  
#****************************************************************************
# Name:        set_span
# Parameters:  none
# Description: update the engine from the span value
#****************************************************************************
  proc set_span { } \
  {
     variable State
     variable span_values
  
     upvar 0 State(span) span
  
     if [info exists span_values($span) ] \
     {
       after idle "Analyzer span $span_values($span) -1.0 -nowait"
     }
  }
  
  
#****************************************************************************
# Name:        updateSpan
# Parameters:  span_value The Analyzer span engine data
# Description: Format and save the Analyzer execution mode
#****************************************************************************
  proc updateSpan { span_value } \
  {
    #puts "updateSpan:  $span_value"

    variable State
    variable span_strings
    
    set State(span,raw) $span_value
    set State(span_f1)  [lindex $span_value 0]
    set State(span_f2)  [lindex $span_value 1]
    
    if { $State(span_f2) == "" } \
    {
      set State(span_f2)  -1.0
    }
    
    if { $State(span_f2) < 0 } \
    {
       set span $State(span_f1)
    } \
    else \
    {
       set span [expr abs($State(span_f1) - $State(span_f2))]
    }

    set temp_span  [expr $span * 1000.0]
    set State(span,MHz)  $span
    set State(span_kHz)  [expr $temp_span]
    set State(span,Hz)   [expr $temp_span*1000.0]
    
    # recompute the freq rate

    set span [format %0.3f $span]
    
    if { [info exists span_strings($span) ] } \
    {
      set State(span) $span_strings($span)
    } \
    else \
    {
      set State(span) "USER"
    }

  }


#****************************************************************************
# Name:        updateTriggerMode
# Parameters:  trig_mode - the Analyzer trigger mode
# Description: Sets the State variable for the trigger mode
#****************************************************************************
  proc updateTriggerMode { trig_mode } \
  {
    #puts "updateTriggerMode:  $trig_mode"

    variable State
    
    set State(trig_mode)  $trig_mode
           
    switch -- $trig_mode \
    {
      0  { set State(trig_mode_text)  "AUTO"     }

      1  { set State(trig_mode_text)  "NORMAL"   }

      2  { set State(trig_mode_text)  "ONE SHOT" }
    }
    
    return
  }


#****************************************************************************
# Name:        updateTriggerLevel
# Parameters:  trig_level - the Analyzer trigger level
# Description: Sets the State variable for the trigger level
#****************************************************************************
  proc updateTriggerLevel { trig_level } \
  {
    #puts "updateTriggerLevel:  $trig_level"

    variable State
    
    set State(trig_level)  [format  "%.1f"  $trig_level]
    
    return
  }


#****************************************************************************
# Name:        updateTriggerSlope 
# Parameters:  trig_slope - the Analyzer trigger slope
# Description: Sets the State variable for the trigger slope
#****************************************************************************
  proc updateTriggerSlope  { trig_slope } \
  {
    #puts "updateTriggerSlope :  $trig_slope"

    variable State
    
    set State(trig_slope)  $trig_slope
           
    switch -- $trig_slope \
    {
      1  { set State(trig_slope_text)  "RISING"  }

      2  { set State(trig_slope_text)  "FALLING" }
    }
    
    return
  }


#****************************************************************************
# Name:        updateTriggerHorizPos
# Parameters:  trig_hpos - the Analyzer trigger horizontal position
# Description: Sets the State variable for the trigger horizontal position
#****************************************************************************
  proc updateTriggerHorizPos { trig_hpos } \
  {
    #puts "updateTriggerHorizPos:  $trig_hpos"

    variable State
    
    set State(trig_hpos)  [format  "%.1f"  $trig_hpos]
    
    return
  }

#****************************************************************************
# Name:        set_average
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the Analyzer engine to its current state
#****************************************************************************
 proc set_average { {avg ""} } \
 {
    variable State
 
    if {$avg != "" } { set State(average) $avg}
 
    Analyzer math 2 $State(average) -nowait
 }
 
 

#****************************************************************************
# Name:        updateEnable
# Parameters:  none
# Description: This function returns a string to describe the state of the 
#            : Analyzer
#****************************************************************************
 proc updateEnable { modes } \
 {
   variable State
   set State(trace_length) [lindex $modes 0]
   set State(enable)  [lindex $modes 1]
 }

 proc Enable { args } \
 {
   variable State
    
   foreach arg $args \
   {
      set bits 0
      switch $arg \
      {
        LIVE    { set bits 0x02}
        PEAK    { set bits 0x08}
        AVERAGE { set bits 0x10}
        SGLPT   { set bits 0x20}
        TOS     { set bits 0x40}
        REF     { set bits 0x80}
      }
      Analyzer enable $State(trace_length) [expr $State(enable)|$bits] -nowait
   }
 }
 proc Disable { args } \
 {
   variable State
    
   foreach arg $args \
   {
      set bits 0
      switch $arg \
      {
        LIVE    { set bits  0x02 }
        PEAK    { set bits  0x08 }
        AVERAGE { set bits  0x10 }
        SGLPT   { set bits  0x20 }
        TOS     { set bits  0x40 }
        REF     { set bits  0x80 }
      }
      Analyzer enable $State(trace_length) [expr $State(enable)&~$bits] -nowait
   }
 }
 proc Toggle { args } \
 {
   variable State
    
   foreach arg $args \
   {
      set bits 0
      switch $arg \
      {
        LIVE    { set bits  0x02 }
        PEAK    { set bits  0x08 }
        AVERAGE { set bits  0x10 }
        SGLPT   { set bits  0x20 }
        TOS     { set bits  0x40 }
        REF     { set bits  0x80 }
      }
      Analyzer enable $State(trace_length) [expr $State(enable)^$bits] -nowait
   }
 }
#****************************************************************************
# Name:        setEnable
# Parameters:  modes
# Description: This function is used in place of the 'Analyzer enable' 
#            : command temporarily.
#****************************************************************************
 proc setEnable { modes } \
 {
   #puts "Analyzer::setEnable:  $modes"
   variable State
   set state 0

   foreach { el } $modes \
   {
     switch -- $el \
     {
       DISABLE { set state  0
       
                 set State(mode)         "DISABLE"
                 set State(mode_string)  "Trace Disabled" }
               
       LIVE    { set state  [expr $state | 0x02]
       
                 set State(mode)         "LIVE"
                 set State(mode_string)  "Displaying Live Trace" }
               
       PEAK    { set state  [expr $state | 0x08]
       
                 set State(mode)         "PEAK"
                 set State(mode_string)  "Displaying Peak Trace" }
               
       AVERAGE { set state  [expr $state | 0x10]
       
                 set State(mode)         "AVERAGE"
                 set State(mode_string)  "Displaying Avg Trace" }
               
       SGLPT   { set state  [expr $state | 0x20] }
       
       TOS     { set state  [expr $state | 0x40] }
       
       REF     { set state  [expr $state | 0x80] }
       
       ACP     { set State(mode)         "ACP"
                 set State(mode_string)  "ACP Measurement" }
    
       default { puts "Analyzer::setEnable: invalid mode $el" } 
     }
   }

   #puts "Analyzer::setEnable:  $state"
   
   Analyzer enable $State(trace_length) $state  -nowait
   
   return
 }


#****************************************************************************
# Name:        setMath
# Parameters:  modes
# Description: This function is used to set the math of the Analyzer
#****************************************************************************
 proc setMath { type {value 0.0} } \
 {
   switch -- $type \
   {
     CLEAR-UP { set t 0 }   ; # for average
     CLEAR-DN { set t 1 }   ; # for peak
     AVERAGE  { set t 2 }
    
     default { puts "set_anlzr_math err: $type"; return; }
   }

   Analyzer math $t $value -nowait
 }

 
#****************************************************************************
# Name:        check_valid_span
# Parameters:  none
# Description: see if current is valid for current run state - adjust if
#               required
#**************************************************************************** 
 proc check_valid_span { } \
 {
    variable State

    switch -- $State(run) \
    {
      "START_FULL" { }
    
      "START_CHAN"
      {
         if {$State(span,Hz) > 5000000} \
         {
            Analyzer span 5000000 -1 -nowait
         }
         Analyzer rbw $State(Engine_rbw) -nowait
      }
    
      default      { return }
    } 
 }
 
#****************************************************************************
# Name:        X_to_freq
# Parameters:  widget - trace widget name
#              x      - position within that widget
# Description: Given a trace widget and an offset in pixels from the left
#              edge of that widget, compute the frequency.
#****************************************************************************
 proc X_to_freq { widget  x } \
 {
   variable State
   
   if { ![winfo exists $widget] } { return 0 }
   
   if { [winfo class $widget] != "TraceWidget" } { return 0 }
   
   set width  [$widget cget -width]
   
   # Convert to a 'double' value for calculations
   set width  [expr double($width)]

   # Get the Analyzer center frequency
   set center_freq  $::Receiver::State(freq)
   
   # If the value is nulled out because of the Spinentry behavior
   # return a frequency of 0.0
   if { $center_freq == "" } \
   {
     return  0.0
   }
   
   # Get the Analyzer span
   set span1 $::Analyzer::State(span,MHz)
   
   # If the span is nulled out or zero, return a frequency of 0.0
   if { $span1 == "" || $span1 == 0.0 } \
   {
     return  0.0
   }

   # Convert the X position to a frequency value
   set maxx  [expr $center_freq + ($span1 / 2.0)]
   set minx  [expr $center_freq - ($span1 / 2.0)]

   set freq  [expr ( ( ($maxx - $minx) * $x ) / $width ) + $minx]
   
   return  $freq
 }
 

#****************************************************************************
# Name:        freq_to_X
# Parameters:  widget - trace widget path
#              freq   - frequency value
# Description: Given a trace widget and a frequency, compute the offset in 
#              pixels from the left edge of that widget.
#****************************************************************************
 proc freq_to_X { widget  freq } \
 {
   variable State
    
   if { ![winfo exists $widget] } { return 0 }
    
   if { [winfo class $widget] != "TraceWidget" } { return 0 }
    
   set width  [$widget cget -width]
   
   # Convert to a 'double' value for calculations
   set width  [expr double($width)]
   
   # Convert to a 'double' value for calculations
   if { $freq == "" } \
   {
     return 0
   }
   
   set freq   [expr double($freq)]

   # Get the Analyzer center frequency
   set center_freq  $::Receiver::State(freq)
   
   # If the value is nulled out because of the Spinentry behavior
   # return a position of 0
   if { $center_freq == "" } \
   {
     return  0
   }

   # Get the Analyzer span
   set span1  $::Analyzer::State(span,MHz)
   
   # If the span is nulled out or zero, return a position of 0
   if { $span1 == "" || $span1 == 0.0 } \
   {
     return  0
   }
   
   # Convert the frequency value to an X position
   set maxx  [expr $center_freq + ($span1 / 2.0)]
   set minx  [expr $center_freq - ($span1 / 2.0)]
   
   set x  [expr ( ($freq - $minx) / ($maxx - $minx) ) * $width]
   
   # Convert to an 'int' value for display
   set x  [expr int($x)]
   
   # Limit the X position
   if { $x < 0 } \
   {
     set x  0
   }
    
   if { $x > $width } \
   {
     set x  $width
   }

   return  $x
 }


#****************************************************************************
# Name:        X_to_time
# Parameters:  widget - trace widget name
#              x      - position within that widget
# Description: Given a trace widget and an offset in pixels from the left
#              edge of that widget, compute the time value.
#****************************************************************************
 proc X_to_time { widget  x } \
 {
   variable  State
   variable  sweep_values
      
   if { ![winfo exists $widget] } { return 0 }
   
   if { [winfo class $widget] != "TraceWidget" } { return 0 }
   
   set width  [$widget cget -width]
   
   # Convert to a 'double' value for calculations
   set width  [expr double($width)]

   # Get the Analyzer sweep time
   set sweep_msec [expr $State(sweep_time,sec)*1000]

   # Convert the X position to a time value
   set maxx  $sweep_msec
   set minx  0.0

   set time  [expr ( ( ($maxx - $minx) * $x ) / $width ) + $minx]
   
   return  $time
 }

 
#****************************************************************************
# Name:        time_to_X
# Parameters:  widget - trace widget path
#              time   - frequency value
# Description: Given a trace widget and a time value, compute the offset in 
#              pixels from the left edge of that widget
#****************************************************************************
 proc time_to_X { widget  time } \
 {
   variable  State
   variable  sweep_values
    
   if { ![winfo exists $widget] } { return 0 }
    
   if { [winfo class $widget] != "TraceWidget" } { return 0 }
    
   set width  [$widget cget -width]
   
   # Convert to 'double' values for the ratio calculations
   set width  [expr double($width)]
   
   # Convert to a 'double' value for calculations
   if { $time == "" } \
   {
     return 0
   }

   set time   [expr double($time)]
   set sweep_msec [expr $State(sweep_time,sec)*1000]

   # Convert the time value to an X position
   set maxx  $sweep_msec
   set minx  0.0

   set x  [expr ( ($time - $minx) / ($maxx - $minx) ) * $width]
   
   # Convert to an 'int' value for display
   set x  [expr int($x)]
   
   # Limit the X position
   if { $x < 0 } \
   {
     set x  0
   }
    
   if { $x > $width } \
   {
     set x  $width
   }

   return  $x
 }


#****************************************************************************
# Name:        Y_to_db
# Parameters:  Trace widget name, position within that widget
# Description: Given a trace widget and an offset in pixels from the top
#              edge of that widget, compute the level, ASSUMING that
#              widget really is showing the FFT trace
#****************************************************************************
 proc Y_to_db { widget  y } \
 {
   variable State
   
   if { ![winfo exists $widget] } { return 0 }
   
   if { [winfo class $widget] != "TraceWidget" } { return 0 }
   
   set height  [$widget cget -height]
   
   # Convert to a 'double' value for calculations
   set height  [expr double($height)]

   # Convert to a 'double' value for calculations
   set tos  [expr double($State(tos))]

   # If the value is nulled out because of the Spinentry behavior
   # return a level of 0.0
   if { $tos == "" } \
   {
     return  0.0
   }

   # Convert the Y position to a level value
   set maxy    [expr $tos]
   set miny    [expr $tos - ($State(scale) * 8.0)]

   set db  [expr $maxy - ( ( ( $maxy - $miny ) * $y ) / $height )]
   
   return  $db
 }


#****************************************************************************
# Name:        db_to_Y
# Parameters:  widget - trace widget name
#              db     - magnitude value
# Description: Given a trace widget and a level, compute the offset in 
#              pixels from the top edge of that widget, ASSUMING that
#              widget really is showing the FFT trace
#****************************************************************************
 proc db_to_Y { widget  db } \
 {
   variable State
    
   if { ![winfo exists $widget] } { return 0 }
    
   if { [winfo class $widget] != "TraceWidget" } { return 0 }
    
   set height  [$widget cget -height]
   
   # Convert to a 'double' value for calculations
   set db   [expr double($db)]
   
   # Convert to a 'double' value for calculations
   set height  [expr double($height)]

   # Convert to a 'double' value for calculations
   set tos     [expr double($State(tos))]

   # If the value is nulled out because of the Spinentry behavior
   # return a position of 0
   if { $tos == "" } \
   {
     return  0
   }

   # Convert the level value to a Y position
   set maxy    [expr $tos]
   set miny    [expr $tos - ($State(scale) * 8.0)]
    
   set y  [expr ( ( $maxy - $db ) / ( $maxy - $miny ) ) * $height]
   
   # Convert to an 'int' value for display
   set y  [expr int($y)]

   # Limit the Y position
   if { $y < 0 } \
   {
     set y  0
   }
    
   if { $y > $height } \
   {
     set y  $height
   }

   return  $y
 }

#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the Analyzer engine to its current state
#****************************************************************************
 proc getSaveState_core { } \
 {
   #puts "Analyzer::getSaveState"
 
   variable State
   
   if { "$State(span_f2)" == "" } \
   {
     set State(span_f2)  -1.0
   }

   set returnList \
   {
      Analyzer run  $State(Engine_run) 
      catch {Analyzer autocpl $State(auto_couple)}
      
      Analyzer enable  $State(trace_length)  $State(enable) -nowait
      Analyzer scale  $State(scale)                              -nowait
      Analyzer sweep  $State(sweep_time)  $State(sweep_type)     -nowait
      Analyzer resolutionbandwidth  $State(rbw)                  -nowait
      Analyzer videobandwidth  $State(Engine_vbw)                -nowait
        
      Analyzer span  $State(span_f1)  $State(span_f2)  -nowait
        
      Analyzer tgstate  $State(tgstate)  -nowait
        
      set ::Analyzer::State(track_gen_level)  {$Analyzer::State(track_gen_level)}
        
      Analyzer::set_average  $::Analyzer::State(average)
        
      Analyzer run  $State(Engine_run)  -nowait
        
      Analyzer trigmode   $State(trig_mode)   -nowait
      Analyzer triglevel  $State(trig_level)  -nowait
      Analyzer trigedge   $State(trig_slope)  -nowait
      Analyzer trigdepth  $State(trig_hpos)   -nowait
      update idle
      Analyzer tos  $State(tos) -nowait
   }

   return [subst $returnList]
 }
 proc getSaveState { } \
 {
   #puts "Analyzer::getSaveState"
 
   variable State
   
   if { "$State(span_f2)" == "" } \
   {
     set State(span_f2)  -1.0
   }
        
   set returnList \
   {
       [getSaveState_core]
   }
   if {$State(tgstate) != 0} \
   {
       append returnList [::Generator::getSaveState]
   }

   return [subst $returnList]
 }
 # load the sweep coupling check IFF master
 if ![interp issafe] \
 {
    source [file join $::Analyzer::path master.tcl]
 }
 set_analyzer_callbacks
#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
   Load::on_system_done \
   {
      set State(engineId) [format_version [Analyzer ver 0] ]
      set State(driverId) [format_version [Analyzer ver 1] ]
      set State(HwId)     [format_version [Analyzer ver 2] ]
   }


}  ;# End of the Analyzer namespace
