namespace eval ::Analyzer \
{
  variable SaveSortOrder AFTER(Receiver)
   
  set State(rbw_group_delay_ratio) 1.0
  set State(max_delay) .01
  
#****************************************************************************
# Name:        auto_set_vbw
# Parameters:  none
# Description: Set VBW given sweep
#****************************************************************************
 proc auto_set_vbw {} \
 {
    variable State
    upvar 0 State(sweep_time,sec) sweep

    # pick a not-unreasonable VBW for this:
    if {$sweep == 0 } { return }
     
    set min_vbw [expr 1.0/(double($State(max_delay))*$sweep)]
     
    foreach  possible_vbw $State(vbw_list) \
    {
       if {$possible_vbw > $min_vbw} \
       {
          break
       }
    }
    Analyzer vbw $possible_vbw  -nowait
    Debug::Puts Analyzer 5 "auto_set_vbw - sweep is $sweep, vbw $possible_vbw"
 }
#****************************************************************************
# Name:        compute_delay
# Parameters:  sweep rate (defaults to current)
# Description: Given current RBW and sweep rate, compute (roughly) filter group delay
#              in terms of fractions of a sweep
#****************************************************************************
 proc compute_delay { {sweep ""}} \
 {
    variable State
    upvar 0 State(Engine_rbw)     rbw
    if {$sweep == ""} { set sweep $State(sweep_time,sec) } 
    set delay  [expr $State(rbw_group_delay_ratio)/double($rbw)]
    set frac [expr $delay/$sweep]
    
    # if in Full span mode, double the delay to allow for settling
    if {$State(Engine_run) == 2} \
    {
       set frac [expr $frac*2]
    }

    return $frac
 }
 
#****************************************************************************
# Name:        compute_loss
# Parameters:  sweep rate (defaults to current)
# Description: Given current RBW, span, and sweep, compute dB loss
#****************************************************************************
 proc compute_loss { {sweep ""}} \
 {
    variable State
    upvar 0 State(Engine_rbw)     rbw
    if {$sweep == ""} { set sweep $State(sweep_time,sec) } 
    
    set freq_rate [expr $State(span,Hz)/$sweep]
    if {$freq_rate < 1.0 } { return 0.0 }
    return [expr 5*log(1.0 + 0.195 * pow( $freq_rate/pow($rbw,2) ,2))]
 }
 
#****************************************************************************
# Name:        auto_set_sweep
# Parameters:  none
# Description: Set sweep and VBW given span and RBW
#****************************************************************************
 proc auto_set_sweep {} \
 {
    variable State
    upvar 0 State(Engine_rbw) rbw
    upvar 0 State(span,Hz)    span

    # don't auto-set in 0 span mode.
    if {$span < 1.0} { return }
    # OK, now pick a reasonable sweep, given our desired max error.
 
    foreach possible_sweep $State(sweep_list) \
    {
        set db_loss [compute_loss  $possible_sweep]
        set delay   [compute_delay $possible_sweep]
        Debug::Puts Analyzer 5 "auto_set_sweep: $possible_sweep loss $db_loss delay [expr $delay*10] div"
        if {$delay > $State(max_delay)} { continue }
        if {$db_loss <= $State(max_uncal_loss)} { break} 
    }
    Analyzer sweep [expr $possible_sweep*1000000] $State(sweep_type) -nowait
    Debug::Puts Analyzer 5 "auto_set_sweep - rbw $State(rbw), sweep $possible_sweep"
 }
 
#****************************************************************************
# Name:        auto_set_rbw
# Parameters:  none
# Description: Set RBW for a given span
#****************************************************************************
 proc auto_set_rbw {} \
 {
     variable State
     upvar 0 State(span,Hz) span

     set rbw  [expr $span/200.0]
     
     # first, set the resolution bandwidth
     if {$span == 0} \
     {
        # Zero span uses 60000 by default
        set rbw 60000
     }
     # NOTE: we will rely upon the engine to enforce filter selections for
     #       ch an mode.
     Analyzer rbw $rbw -nowait
     Debug::Puts Analyzer 5 "auto_set_rbw - span $span, rbw $rbw"
 }
 
#****************************************************************************
# Name:        check_cal_state
# Parameters:  none
# Description: Validate the sweep rate, span, RBW, and VBW against one another
#              and determine if the current setup is CAL or UNCAL
# Notes:       all equations and constants are from
#              Modern Spectrum Analyzer Theory And Applications, by
#              Morris Engelson
#****************************************************************************
   proc check_cal_state { } \
   {
      variable State
      # assume OK unless otherwise
      set rbw_cal 1
      set vbw_cal 1
      set sweep_cal 1
      
      # make some accesses a bit simpler
      upvar 0 State(Engine_rbw)     rbw
      upvar 0 State(Engine_vbw)     vbw
      upvar 0 State(sweep_time,sec) sweep
      upvar 0 State(span,Hz)        span

      # the single biggest factor is the frequency rate: the rate at which
      # frequency changes vs time (in Hz/sec).
      if {$State(sweep_time,sec) == 0} { return }
          
      # Step 1: Validate RBW. Our criterion is
      #   amplitude loss less than State(max_uncal_loss) dB
      # and
      #   group delay < State(max_delay)
      # And we don't do this for 0 span mode.
      if {$span > 1.0 } \
      {
        set db_loss [compute_loss]
        set shift   [compute_delay]

        Debug::Puts Analyzer 5 "check_cal_state - loss $db_loss max allowed $State(max_uncal_loss)"
        Debug::Puts Analyzer 5 "shift $shift max allowed $State(max_delay)"

        if {$db_loss > $State(max_uncal_loss)} \
        {
           set sweep_cal 0
        }
        if {$shift > $State(max_delay)} \
        {
           set sweep_cal 0
        }
      }
      
      # OK, now evaluate the VBW filter. This is based on the idea that we want the
      # filter delay to be << the sweep speed (say 1%) . So, approximate the VBW response as 1/vbw.
      if { $vbw != 0} \
      {
         set min_vbw [expr 1.0/(double($State(max_delay))*$sweep)]

         if {$vbw < $min_vbw } \
         {
            set vbw_cal 0
         }
         Debug::Puts Analyzer 5 " vbw_limit $min_vbw, sweep $sweep, cal $vbw_cal"
      }
      
      set State(rbw,cal)   $rbw_cal
      set State(vbw,cal)   $vbw_cal
      set State(sweep,cal) $sweep_cal
      if {$rbw_cal && $vbw_cal && $sweep_cal} \
      {
         set State(cal) 1
      } \
      else \
      {
         set State(cal) 0
      }
   }

   proc check_cal_stat_later { args } \
   {
      set cmd "Analyzer::check_cal_state"
      after cancel $cmd
      after idle $cmd
   }
   
   proc master_span_update { args } \
   {
      set cmd "Analyzer::master_span_update_core"
      after cancel $cmd
      after idle $cmd
   }
   proc master_span_update_core { } \
   {
      variable State
      if {$State(auto_couple) != 0} \
      {
         auto_set_rbw
      } \
      else \
      {
        #check for sane sweep speeds
        Analyzer sweep $State(sweep_time) $State(sweep_type) -forgive
      }
      check_cal_stat_later
      # if not in zero span, force trigger to auto
      if {$State(span,Hz) != 0} \
      {
         Analyzer trigmode 0 -nowait
      }
   }
   proc master_sweep_update { args } \
   {
      set cmd "Analyzer::master_sweep_update_core"
      after cancel $cmd
      after idle $cmd
   }
   proc master_sweep_update_core { } \
   {
      variable State
      if {$State(auto_couple) != 0} \
      {
         auto_set_vbw
      }
      check_cal_stat_later
   }
   proc master_rbw_update { args } \
   {
      set cmd "Analyzer::master_rbw_update_core"
      after cancel $cmd
      after idle $cmd
   }
   proc master_rbw_update_core { } \
   {
      variable State
      if {$State(auto_couple) != 0} \
      {
         auto_set_sweep
      }
      check_cal_stat_later
   }   
# watch the parameters of interest, and recalc if they change.
trace variable ::Analyzer::State(Engine_run)     w ::Analyzer::master_rbw_update
trace variable ::Analyzer::State(Engine_rbw)     w ::Analyzer::master_rbw_update
trace variable ::Analyzer::State(Engine_vbw)     w ::Analyzer::check_cal_stat_later
trace variable ::Analyzer::State(sweep_time,sec) w ::Analyzer::master_sweep_update
trace variable ::Analyzer::State(span,Hz)        w ::Analyzer::master_span_update
trace variable ::Analyzer::State(auto_couple)    w ::Analyzer::master_span_update

#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to hardware
#****************************************************************************
 proc Sync { } \
 {
    eval [getSaveState_core]
 }
 #****************************************************************************
# Name:        updateTGState
# Parameters:  tracking gen state
# Description: save the state of the tracking generator
#****************************************************************************
  proc updateTGState { state } \
  {
    variable State

    set State(tgstate)  $state

    # Bypass the update if being called due to a recall of a setup
    if { ![info exists ::Recall_SetupsUI::State(recall_in_process)] } \
    {
        return
    }
    if { $::Recall_SetupsUI::State(recall_in_process) == 1 } \
    {
        return
    }
    # If the track gen is ON, ask the receiver for its freq - this will
    # drive us to set the gen freq.
    # if the track gen is OFF, then reset the gen freq.
    if {$state} \
    {
       Receiver freq
    } \
    else \
    {
      # Retrieve the current frequency setting in the generator engine, and
      # drive this setting to the DSP. The Generator namespace variable
      # cannot be used for this during RECALL setups 
      Generator freq [Generator freq] -nowait
    }
  }
  
#****************************************************************************
# Name:        updateTGfreq
# Parameters:  new receiver freq
# Description: track the generator freq to the receiver freq
#****************************************************************************
 proc updateTGfreq { rx_freq } \
 {
    variable State

    # Bypass the update if being called due to a recall of a setup
    if { ![info exists ::Recall_SetupsUI::State(recall_in_process)] } \
    {
        return
    }
    if { $::Recall_SetupsUI::State(recall_in_process) == 1 } \
    {
        return
    }

    if { ($State(tgstate) == 1) && ($State(Engine_run) == 2) } \
    {
       Generator freq $rx_freq -nowait
    }
 }
 
#****************************************************************************
# Name:        updateNormalize
# Parameters:  normalization state
# Description: save the normalization state
#****************************************************************************
  proc updateNormalize { state } \
  {
    variable State
    variable normalize_states
    
    set State(normalize_state)  $state
  
    if [info exists normalize_states($state)] \
    {
       set State(normalize) $normalize_states($state)
    } \
    else \
    {
       set State(normalize) "Unknown state $state"
    }
  
    # look for state save request, and save is needed
    switch $State(normalize_state) \
    {
       28 {  if [info exists State(saved_state)] \
             {
                unset State(saved_state)
             }
             append State(saved_state) "Analyzer run 3"
             append State(saved_state) [::Generator::getSaveState]
             append State(saved_state) "update idle"
             append State(saved_state) [::Receiver::getSaveState]
             append State(saved_state) "update idle"
             append State(saved_state) [::Analyzer::getSaveState]
             append State(saved_state) "update idle"
             append State(saved_state) [::Power::getSaveState]
             append State(saved_state) "update idle"
             Analyzer autocpl 0
             Analyzer run 2
             Analyzer normalize 0x80
          }
          
       9 -
       14 -
       15 -
       16 -
       17 {
            after cancel "Analyzer::Restore"
            after 1000 "Analyzer::Restore"
            ErrLog::Log "Analyzer normalize error: $State(normalize)"
          }
       26 -
       10 -
       8  {
            after cancel "Analyzer::Restore"
            after 1000 "Analyzer::Restore"
          }
    }
    
    #puts "Analyzer normalize state is $state $State(normalize)"
  }
  
  
Analyzer -onchange status            { ::Analyzer::updateNormalize %V }
Analyzer -onchange tgstate           { ::Analyzer::updateTGState   %V }
Receiver -onchange freq              { ::Analyzer::updateTGfreq %V }

   HW_control::OnYigDone { ::Analyzer::Sync }
}



#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines Analyzer
   
   variable pre_restore_cmds
   
   lappend pre_restore_cmds {Analyzer run $Analyzer::State(Engine_run) -nowait}
}

#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
     Anlz_Eng "Spectrum Analyzer Engine"
     Anlz_Drv "Spectrum Analyzer Driver"
     Anlz_Hw "Spectrum Analyzer Hardware"
   }
   set version(Anlz_Eng)  [format_version [Analyzer  ver 0]]
   set version(Anlz_Drv)  [format_version [Analyzer  ver 1]]
   set version(Anlz_Hw)   [format_version [Analyzer  ver 2]]
}

