#############################################################################
# COPYRIGHT (C) 2003, 2005  Aeroflex, Inc.                                  #
# --------------------------------------------------------------------------#
# FILE        : Bit.tcl                                                     #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the BIT command is         #
#             : first used.  It provides the basic functions for the Built  #
#             : In Test (BIT) capabilities of the 2975                      #
#############################################################################
package provide BIT 1.0

#****************************************************************************
# BIT variables
#****************************************************************************
#  ReportLevel		Controls the detail of test results sent to the file
#       "Silent" - only writes a pass/fail status
#       "Errors" - writes information when an error occurs
#       "Manual" - writes error and manual verification data
#       "Full"   - writes any available information
#
#****************************************************************************
# BIT functions
#****************************************************************************
#
#  Global Functions:
#     addTest           Typically call this function at the end of a test file
#                       to adds the test namespace to the list of available
#                       BIT tests
#     getTestList       Returns the current list of tests
#     runSelectedTests  Call to execute all of the tests
#
#  Local Functions:
#     addTimestamp      This writes the current time into the test output file
#     Blanks            Returns a string of blanks of a desired length
#     closeResultsFile  Closes the standard test results file
#     compareDbls       Compares a list of double values
#     compareInts       Compares a list of integer values
#     MakeHeader        This takes a string and formats it into one of several
#                       standard formats
#     MakeString        This takes a string and creates a justified string 
#                       of a specified length
#     openResultsFile   Opens the standard test results file
#     putsTest          Writes a string to the standard test results file
#     resetTestStats    Resets all of the accumulated test information
#     resetTestList     Removes all tests from the test list
#     runCmdLineTests   Executes a test case in a given namespace
#
#****************************************************************************
namespace eval ::BIT {

#****************************************************************************
# Name:        addTest
# Parameters:  testNamespace		Namespace of test to add to available list
# Description: Adds the indicated namespace to the current list of available
#     tests so that the UI or user can run a query to find out what tests are
#     available
#****************************************************************************
 proc addTest { testNamespace } \
 {
    variable testList

    if { [lsearch -exact $testList $testNamespace] == -1 } \
    {
       lappend testList  $testNamespace
    }
 }

#****************************************************************************
# Name:        getTestList
# Parameters:  none
# Description: Returns a TCL list of the available tests
#****************************************************************************
 proc getTestList { } \
 {
    return $::BIT::testList
 }

#****************************************************************************
# Name:        MakeHeader
# Parameters:  header		String to be printed in the header
#              type		Selects type of header format to use
# Description: Returns a formatted string
#****************************************************************************
 proc MakeHeader {header type} \
 {
    switch $type \
    {
       1 {
            set lead "+ + + + "
            set trail " + + + +"
            set final $lead[MakeString $header 50 center]$trail
         }
       2 {
          # set lead "- - "
          # set trail " - -"
            set final [MakeString $header 66 center]
         }
       3 {
            set lead "* * * * * * * * "
            set trail " * * * * * * * *"
            set final $lead[MakeString $header 34 center]$trail
         }
    }

    return $final 	;# returns the value of final
 }

#****************************************************************************
# Name:        MakeString
# Parameters:  str		String to be justified
#              len		Desired length of new string
#              just		Type of justification desired (center, right, left)
# Description: Returns a justified string of the desired length, with spaces
#    padding the front or end as needed
#****************************************************************************
 proc MakeString {str len just} \
 {
    # Convert 'just' to one upper case character (Only the first character
    # is needed to determine centered, left, or right justification).
    set just [string toupper [string index $just 0]]
    set lengthy [string length $str]
    if {[expr {$len - $lengthy}] > 0} \
    {
       switch $just \
       {
          C { ;# Justify Center
              set str1 [Blanks [expr int(($len - $lengthy) / 2.0)]]
              if {[expr ($len - $lengthy) / 2.0] == [expr int(($len - $lengthy) / 2.0)]} \
              {
                 # string length is even, so spaces are equal
                 set str2 $str1
              } \
              else \
              {
                 # string length is odd, so add an extra space on the right
                 set str2 [Blanks [expr [string length $str1] + 1]]
              }
              set final $str1$str$str2
            }
          L { ;# Justify Left
              set final $str[Blanks [expr $len - $lengthy]]
            }
          R { ;# Justify Right
              set final [Blanks [expr $len - $lengthy]]$str
            }
          default { ;# Illegal justify parameter, return 'str' as is
              set final $str
            }
       }
    } \
    else \
    {
       # If str is equal to, or longer than, the requested field length
       # return str without doing anything to it.
       set final $str
    }
    return $final ;# returns the value of final
 }

#****************************************************************************
# Name:        Blanks
# Parameters:  strLen		Desired length of string
# Description: Generates a string of blanks of length "strLen"
#****************************************************************************
proc Blanks { strLen } \
{
   set space " "
   set temp $space
   for  {set x 1} {$x < $strLen} {incr x} {append temp $space}
   return $temp
}

#****************************************************************************
# Name:        openResultsFile
# Parameters:  none
# Description: Opens the test results file
#****************************************************************************
 proc openResultsFile { { mode "TRUNC" } } \
 {
    variable resultsFileID

    if [catch {open $::sys(system_path)/interface/BIT/results.txt [list WRONLY CREAT $mode]} resultsFileID ] \
    {
       set resultsFileID ""
    }
 }

#****************************************************************************
# Name:        closeResultsFile
# Parameters:  none
# Description: Closes the test results file
#****************************************************************************
 proc closeResultsFile { } \
 {
    if {$::BIT::resultsFileID != ""} \
    {
       close $::BIT::resultsFileID
       set ::BIT::resultsFileID ""
    }
 }

#****************************************************************************
# Name:        addTimestamp
# Parameters:  header		Indicates which header format to use
# Description: Adds a timestamp to the test results file
#****************************************************************************
 proc addTimestamp { { header 3 } } \
 {
    putsTest [MakeHeader [clock format [clock seconds] -format $::BIT::ClockFormat] $header]
 }

#****************************************************************************
# Name:        putsTest
# Parameters:  outStr		Test data string to output to the results file
# Description: Writes a string to the test results file
#****************************************************************************
 proc putsTest { outStr } \
 {
    if {$::BIT::resultsFileID != ""} \
    {
       puts $::BIT::resultsFileID $outStr
    }
 }

#****************************************************************************
# Name:        runSelectedTests
# Parameters:  none
# Description: Runs all of the selected tests
#****************************************************************************
 proc runSelectedTests { {resetTests 1} } \
 {
    variable testList
    variable selectedTestList
    variable ReportLevel
    variable State

    closeResultsFile
    openResultsFile
    putsTest [MakeHeader "2975 Self Test Results" 3]
    putsTest [MakeHeader "Running selected tests" 3]
    addTimestamp

    if {$resetTests == 1} \
    {
       set State(TotalCases) 0
       set State(TotalErrors) 0
    }

    set selectedTestList $testList
    foreach test $selectedTestList \
    {
       putsTest "\n\nAbout to run test: $test"
       set cmd "::"
       append cmd $test "::autoTest " $ReportLevel
       eval $cmd

       set status "FAILED"
       if {$State($test,ERRORS) == 0} {set status "PASSED"}
       putsTest "   Test $test $status:  $State($test,ERRORS) errors in $State($test,CASES) cases"
       addTimestamp
    }

    putsTest "\n\n"
    set status "Total Cases: $State(TotalCases),  Total Errors: $State(TotalErrors)"
    putsTest [MakeHeader $status 3]

    addTimestamp
    putsTest [MakeHeader "End of selected tests" 3]
    closeResultsFile
 }

#****************************************************************************
# Name:        runCmdLineTests
# Parameters:  testNameSpace		Namespace of test object
#              testCaseList		TCL list of test cases
# Description: Software Module Testing
#    This function expects to receive the namespace of a test object which
#    is designed for testing the remote commands for a specific engine
#****************************************************************************
 proc runCmdLineTests { testNameSpace testCaseList } \
 {
    variable State
    variable ReportLevel

    set LocalErrors 0
    set LocalCases 0
    set SentCaseName 0

    # Initialize array variables if they do not exist yet
    if { ![info exists State($testNameSpace,CASES)] } \
    {
       set State($testNameSpace,CASES) 0
    }
    if { ![info exists State($testNameSpace,ERRORS)] } \
    {
       set State($testNameSpace,ERRORS) 0
    }

    set temp "::"
    append temp $testNameSpace "::longDesc"

    upvar #0 $temp testDescription

    if {$ReportLevel == "Full"} \
    {
       set ShowCases 1
       set ShowErrors 1
       putsTest "Running $testDescription: $testNameSpace.$testCaseList"
       set SentCaseName 1
    } \
    elseif {$ReportLevel == "Manual"} \
    {
       set ShowCases 0
       set ShowErrors 1
    } \
    elseif {$ReportLevel == "Errors"} \
    {
       set ShowCases 0
       set ShowErrors 1
    } \
    elseif {$ReportLevel == "Silent"} \
    {
       set ShowCases 0
       set ShowErrors 0
    }

    set temp "::"
    append temp $testNameSpace "::" $testCaseList
puts "Test name: $temp"
    upvar #0 $temp fullName

    foreach testCase $fullName {
       set testCmd     [lindex $testCase 0]
       set testDesired [lindex $testCase 1]
       set testType    [lindex $testCase 2]
       set testRange   [lindex $testCase 3]

       incr LocalCases 1
       if {$ShowCases == 1 } {
          putsTest "-- Test Case $LocalCases: Cmd = $testCmd"
       }

       set cmdError 0
       set testResults ""
       if [catch {eval $testCmd} testResults] \
       {
          set cmdError 1
       }
       set numResults [llength $testDesired]

       if {$ShowCases == 1 } {
          putsTest "-- Desired = $testDesired, Actual = $testResults"
       }

       if {$cmdError == 1} {
          incr LocalErrors 1
          if {$ShowErrors == 1 } \
          {
             if {$SentCaseName == 0} \
             {
                putsTest "Running $testDescription: $testNameSpace.$testCaseList"
                set SentCaseName 1
             }
             putsTest "** Test Error: $LocalErrors $testCmd, TCL ERROR=$testResults"
          }
       } elseif {$testType == "compareInts"} {
          if { [compareInts $testDesired $testResults] } {
             incr LocalErrors 1
             if {$ShowErrors == 1 } \
             {
                if {$SentCaseName == 0} \
                {
                   putsTest "Running $testDescription: $testNameSpace.$testCaseList"
                   set SentCaseName 1
                }
                putsTest "** Test Error: $LocalErrors $testCmd, actual=$testResults"
             }
          }
       } elseif {$testType == "compareDbls"} {
          if { [compareDbls $testDesired $testResults $testRange] } {
             incr LocalErrors 1
             if {$ShowErrors == 1 } \
             {
                if {$SentCaseName == 0} \
                {
                   putsTest "Running $testDescription: $testNameSpace.$testCaseList"
                   set SentCaseName 1
                }
                putsTest "** Test Error: $LocalErrors $testCmd, actual=$testResults"
             }
          }
       } elseif {$testType == "INT"} {
     
          if { $numResults != 1 } \
          {
             puts "INT test, llength = $numResults, results = $testResults"
             putsTest "INT test, llength = $numResults, results = $testResults"
          }
          if {$testResults != $testDesired} {
             incr LocalErrors 1
             if {$ShowErrors == 1 } \
             {
                if {$SentCaseName == 0} \
                {
                   putsTest "Running $testDescription: $testNameSpace.$testCaseList"
                   set SentCaseName 1
                }
                putsTest "** Test Error: #$LocalErrors, $testCmd, actual=$testResults"
             }
          }
       } elseif {$testType == "DBL"} {
          if { $numResults != 1 } \
          {
             puts "DBL test, llength = $numResults, results = $testResults"
             putsTest "DBL test, llength = $numResults, results = $testResults"
          }
          if {$testResults > $testDesired } {
             set delta [expr $testResults - $testDesired]               
          } else {
             set delta [expr $testDesired - $testResults]               
          }
          if {$delta > $testRange} {
             incr LocalErrors 1
             if {$ShowErrors == 1 } \
             {
                if {$SentCaseName == 0} \
                {
                   putsTest "Running $testDescription: $testNameSpace.$testCaseList"
                   set SentCaseName 1
                }
                putsTest "** Test Error: #$LocalErrors, $testCmd, actual=$testResults"
             }
          }
       } elseif {$testType == "STR"} {
          if {$testResults != $testDesired} {
             incr LocalErrors 1
             if {$ShowErrors == 1 } \
             {
                if {$SentCaseName == 0} \
                {
                   putsTest "Running $testDescription: $testNameSpace.$testCaseList"
                   set SentCaseName 1
                }
                putsTest "** Test Error: $LocalErrors $testCmd, actual=$testResults"
             }
          }
       } elseif {$testType == "DELAY"} {
           puts "Test delay starting at: [clock format [clock seconds] -format $::BIT::ClockFormat]"
       } elseif {$testType == "NOCMP"} {
       } else {
          if {$SentCaseName == 0} \
          {
             putsTest "Running $testDescription: $testNameSpace.$testCaseList"
             set SentCaseName 1
          }
          putsTest "Unknown test compare type = $testType"
       }
    }

    set State($testNameSpace,CASES)  [expr $State($testNameSpace,CASES) + $LocalCases]
    set State($testNameSpace,ERRORS) [expr $State($testNameSpace,ERRORS) + $LocalErrors]

    set State(TotalCases) [expr $State(TotalCases) + $LocalCases]
    set State(TotalErrors) [expr $State(TotalErrors) + $LocalErrors]
 }

#****************************************************************************
# Name:        compareInts
# Parameters:  testDesired		List of desired test values 
#              testResults          List of actual test result values
# Description: Compares one or more test results with the expected values
#     and returns a status of the results
#****************************************************************************
 proc compareInts { testDesired testResults } \
 {
    set numDesired [llength $testDesired]
    set numResults [llength $testResults]

    set j 0
    incr numDesired -1
    set error 0
    for {set i 0} {$i < $numResults} {incr i 1} \
    {
        set desired [lindex $testDesired $j]
        set actual [lindex $testResults $i]
        if {$desired != $actual} \
        {
           set error 1
        }
        if {$j < $numDesired} \
        {
           incr j 1
        }
    }
    return $error
 }

#****************************************************************************
# Name:        compareDbls
# Parameters:  testDesired		List of desired test values 
#              testResults          List of actual test result values
# Description: Compares one or more test results with the expected values
#     and returns a status of the results
#****************************************************************************
 proc compareDbls { testDesired testResults deltaLimit } \
 {
    set numDesired [llength $testDesired]
    set numResults [llength $testResults]

    set j 0
    incr numDesired -1
    set error 0
    for {set i 0} {$i < $numResults} {incr i 1} \
    {
        set desired [lindex $testDesired $j]
        set actual [lindex $testResults $i]

        if {$actual > $desired } {
           set delta [expr $actual - $desired]               
        } else {
           set delta [expr $desired - $actual]               
        }
        if {$delta > $deltaLimit} \
        {
           set error 1
        }
        if {$j < $numDesired} \
        {
           incr j 1
        }
    }
    return $error
 }

#****************************************************************************
# Name:        resetTestStats
# Parameters:  testNameSpace		Namespace of test object
# Description: Zeros the accumulated cases and errors for a test
#****************************************************************************
 proc resetTestStats { testNameSpace } \
 {
    variable State

    set State($testNameSpace,CASES) 0
    set State($testNameSpace,ERRORS) 0
 }

#****************************************************************************
# Name:        resetTestList
# Parameters:  none
# Description: This function removes all tests from the list
#****************************************************************************
 proc resetTestList { } \
 {
    set ::BIT::testList { }
 }

}  ;# End of the BIT namespace


