#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : Civ_fm.tcl                                                  #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the C4fm command is        #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::C4fm::State ruw "::C4fm::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
# Call the base class dummy command to force a load
if {"[info command C4fm]" != "" } { return }
package require Engtclbase
Engtclbase
package require Alert

# Load the object file
::load $::C4fm::object

package provide C4fm 1.0

#****************************************************************************
# Add functions to the C4fm namespace
#    Status of commands sent to C4fm
#       ::C4fm::State(Engine_run)          {0 | 1}
#       ::C4fm::State(run)                 {OFF | ON}
#       ::C4fm::State(average)             {1 - 100}
#       ::C4fm::State(Engine_enabled)      {0 | 1}
#       ::C4fm::State(enabled)             {OFF | ON}
#       ::C4fm::State(Engine_trigger)      {0 | 1 | 2}
#       ::C4fm::State(trigger)             {OFF | CONTINUOUS | ONE_SHOT}
#    Status of counter data received from the engine
#       ::C4fm::State(reading)             Current C4 FM value
#       ::C4fm::State(driverId)            Driver ID information
#       ::C4fm::State(engineId)            Engine ID information
#    Status of alert limits and peak values
#       ::C4fm::State(peakHi)              Maximum peak reading
#       ::C4fm::State(peakLo)              Minimum peak reading
#       ::C4fm::State(peakFlags)           Peak Hold status flags
#       ::C4fm::State(peakEnable)          Peak Enable settings
#       ::C4fm::State(alertHi)             Maximum valid reading limit
#       ::C4fm::State(alertLo)             Minimum valid reading limit
#       ::C4fm::State(modacc,alerts)       Alert flag status
#****************************************************************************
namespace eval ::C4fm {
   variable SaveSortOrder AFTER(Receiver)
   variable name "Mod Fid Error"
   
   array set trigger_map { \
      CONTINUOUS  1 \
      ONE_SHOT    2 \
      1 CONTINUOUS  \
      2 ONE_SHOT    \
   }

#****************************************************************************
# Name:        set_C4fm_callbacks
# Parameters:  none
# Description: Sets up the callbacks to update the C4 FM data
#****************************************************************************
 proc set_C4fm_callbacks { } {

   C4fm -onchange reading       { ::C4fm::updateReading %V modacc }
   C4fm -onchange ferr_reading  { ::C4fm::updateReading %V freq_err }
   C4fm -onchange dev_reading   { ::C4fm::updateReading %V deviation }
   
   C4fm -onchange readpeakhold       { ::C4fm::updatePeaks %V modacc}
   C4fm -onchange ferr_readpeakhold  { ::C4fm::updatePeaks %V freq_err}
   C4fm -onchange dev_readpeakhold   { ::C4fm::updatePeaks %V deviation}
   
   C4fm -onchange ulimit      { ::C4fm::updateAlertHi %V modacc}
   C4fm -onchange ferr_ulimit { ::C4fm::updateAlertHi %V freq_err}
   C4fm -onchange dev_ulimit  { ::C4fm::updateAlertHi %V deviation}

   C4fm -onchange llimit      { ::C4fm::updateAlertLo %V modacc}
   C4fm -onchange ferr_llimit { ::C4fm::updateAlertLo %V freq_err}
   C4fm -onchange dev_llimit  { ::C4fm::updateAlertLo %V deviation}
  
   C4fm -onchange alerts      { ::C4fm::updateAlerts %V modacc}
   C4fm -onchange ferr_alerts { ::C4fm::updateAlerts %V freq_err}
   C4fm -onchange dev_alerts  { ::C4fm::updateAlerts %V deviation}

   C4fm -onchange peak          { ::C4fm::updatePeakEnable %V modacc}
   C4fm -onchange ferr_peakhold { ::C4fm::updatePeakEnable %V freq_err}
   C4fm -onchange dev_peakhold  { ::C4fm::updatePeakEnable %V deviation}

   C4fm -onchange run      { ::C4fm::updateMode %V }
   C4fm -onchange avg      { set ::C4fm::State(average) %V }
   C4fm -onchange en       { ::C4fm::updateEnable %V }
   C4fm -onchange trig     { ::C4fm::updateTrig %V }
 }


#****************************************************************************
# Name:        setTrig
# Parameters:  trigState         OFF | ONE_SHOT | CONTINUOUS
# Description: Sets up the current trigger mode of the C4 FM meter
#****************************************************************************
 proc setTrig { trigState } {
    if [info exists ::C4fm::trigger_map($trigState)] {
       C4fm trig $::C4fm::trigger_map($trigState) -nowait
    }
 }

#****************************************************************************
# Name:        updateTrig
# Parameters:  trigState         0 = Off, 1 = One Shot, 2 = Continuous
# Description: Sets up the current trigger mode of the C4 FM meter
#****************************************************************************
 proc updateTrig { trigState } {
    if [info exists ::C4fm::trigger_map($trigState)] {
       set ::C4fm::State(trigger) $::C4fm::trigger_map($trigState)
    }
    set ::C4fm::State(Engine_trigger) $trigState
 }

#****************************************************************************
# Name:        updateMode
# Parameters:  runState         0 = Off, 1 = On
# Description: Sets up the current run state of the C4 FM meter
#****************************************************************************
 proc updateMode { runState } {
   variable State
   set State(Engine_run) $runState
   switch $runState {
      0       { set State(run) OFF }
      1       { set State(run) ON }
      default { return }
   }
 }

#****************************************************************************
# Name:        updateEnable
# Parameters:  enableBits       1 = Off, 3 = Enabled
# Description: Sets up the current enabled state of the C4 FM
#****************************************************************************
 proc updateEnable { enableBits } {
   switch $enableBits {
      3       { set ::C4fm::State(enabled) Enabled }
      default { set ::C4fm::State(enabled) Disabled }
   }
   set ::C4fm::State(Engine_enabled) $enableBits
 }


#****************************************************************************
# Name:        updateReading
# Parameters:  readingList      List of engine data
#                reading        Current engine reading
#                status         Status flags
# Description: Sets up the current state of the C4fm reading
#****************************************************************************
 proc updateReading { readingList which} {

   variable State
   
   set reading [lindex $readingList 0]
   set prec    [lindex $readingList 1]
   set status  [lindex $readingList 2]
   
   set State($which,raw) $reading
   set State($which) [format "%2.${prec}f" $reading]
   set State($which,precision) $prec
   set State($which,status) $status
   after cancel "C4fm::updateAlarm"
   after idle "C4fm::updateAlarm"
 }

 proc updateAlarm { } \
 {
    variable State
    set alarm 0
    foreach reading { modacc freq_err deviation} \
    {
       if {$State($reading,status) & 0x2000 } \
       {
          set alarm 1
          break;
       }
    }
    if {$alarm} \
    {
       set State(alarm) "ACTIVE"
    } \
    else \
    {
      set State(alarm) "inactive"
    }
 }

#****************************************************************************
# Name:        updatePeaks
# Parameters:  readingList      List of data from the engine
#                 Peak HI       Max peak value
#                 HI Precision  Max peak precision
#                 Peak LO       Min peak value
#                 LO Precision  Min peak precision
#                 Flags         Status flags
#              count_accuracy   Current MAXIMUM digits of accuracy
# Description: Sets up the current state of the peak readings
#****************************************************************************
 proc updatePeaks { readingList which } \
 {
   variable State

   # Get the data from the peak readings
   set peakHi    [lindex $readingList 0]
   set peakLo    [lindex $readingList 2]
   
   set State($which,peakFlags) [lindex $readingList 4]

   set State($which,peakHi)    [format "%0.1f" $peakHi]
   set State($which,peakLo)    [format "%0.1f" $peakLo]
 }


#****************************************************************************
# Name:        updatePeakEnable
# Parameters:  
# Description: Sets up the current state of the peak enable
#****************************************************************************
 proc updatePeakEnable { peakEnable which } \
 {
   variable State

   set State($which,peakEnable)  $peakEnable

   switch -- $State($which,peakEnable) \
   {
      0       { set State($which,peakEnable_text)  "OFF"}
      default { set State($which,peakEnable_text)  "ON"}
   }
 }


#****************************************************************************
# Name:        updateAlerts
# Parameters:  
# Description: Update the Alerts flags
#****************************************************************************
 proc updateAlerts { alerts which } \
 {
   variable State
 
   set State($which,alerts)  $alerts
   
   if { $alerts & 2 } \
   {
     set State($which,alertHi_text) "ON"
   } \
   else \
   {
     set State($which,alertHi_text) "OFF"
   }
   
   if { $alerts & 1 } \
   {
     set State($which,alertLo_text) "ON"
   } \
   else \
   {
     set State($which,alertLo_text) "OFF"
   }
   if {$which == "modacc"} \
   {
     if { $alerts & 4 } \
     {
       set State(BellEnabled) 1

       set State(BellEnabled_text) "ON"
       set State(alarm_text)       "ON"
     } \
     else \
     {
       set State(BellEnabled) 0

       set State(BellEnabled_text) "OFF"
       set State(alarm_text)       "OFF"
     }
   }
   
   return
 }

#****************************************************************************
# Name:        updateAlertHi
# Parameters:  
# Description: Formats the Alert High value
#****************************************************************************
 proc updateAlertHi { alertHi  which } \
 {
   variable State
   
   set State($which,alertHi,raw) $alertHi
   
   set State($which,alertHi) [format "%0.$State($which,precision)f" $alertHi]
   
   return
 }

#****************************************************************************
# Name:        updateAlertLo
# Parameters:  
# Description: Formats the Alert Low value
#****************************************************************************
 proc updateAlertLo { alertLo  which } \
 {
   variable State
   
   set State($which,alertLo,raw) $alertLo
   
   set State($which,alertLo) [format "%0.$State($which,precision)f" $alertLo]
   
   return
 }


#********************************************************************
#
#       Name: setLimits
#
# Parameters:
#
# Decsription: Sets the limit values
#
#********************************************************************
proc setHiLimits { } \
{
  variable State

  C4fm alerts  [expr $State(modacc,alerts) | 0x02] -nowait

  C4fm ulimit  $State(modacc,alertHi)  -nowait
  
  return
}

proc setLoLimits { } \
{
  variable State
  
  C4fm alerts  [expr $State(modacc,alerts) | 0x01] -nowait

  C4fm llimit  $State(modacc,alertLo)  -nowait
  
  return
}  


#********************************************************************
#
#       Name: toggleAlerts
#
# Parameters:
#
# Decsription: Toggles the state of the alerts
#
#********************************************************************
proc toggleHiAlert {} \
{
  variable  State
  
  C4fm alerts [expr $State(modacc,alerts) ^ 2] -nowait
}

proc toggleLoAlert {} \
{
  variable  State
  
  C4fm alerts [expr $State(modacc,alerts) ^ 1] -nowait
}


#********************************************************************
#
#       Name: toggleAlarm
#
# Parameters:
#
# Decsription: Toggles the state of the alarm
#
#********************************************************************
proc toggleAlarm { } \
 {
   variable State
   
   C4fm alerts [expr $State(modacc,alerts) ^ 4] -nowait
 }


#********************************************************************
#
#       Name: togglePeakEnable
#
# Parameters: 
#
# Decsription: 
#
#********************************************************************
proc togglePeakEnable { } \
 {
   variable State

   if { $State(modacc,peakEnable) == 1 } \
   {
     after idle "C4fm peak 0 -nowait"
   }
  
   if { $State(modacc,peakEnable) == 0 } \
   {
     after idle "C4fm phreset  -nowait"
     after idle "C4fm peak 1   -nowait"
   }

   return    
 }
  
  
#********************************************************************
#
#       Name: setAverage
#
# Parameters: new_value   - requested new value 
#                           -1 - no new value requested
#
# Description: set the meter average
#
#********************************************************************
proc setAverage { } \
 {
   variable   State

   # Set the meter engine
   after idle "C4fm  average  $::C4fm::State(average)  -nowait"

   # Setting the AVG will not update the DSP. If the meter is running,
   # we need to turn it OFF and then ON.
   if { $State(Engine_run) == 1 } \
   {
      after idle "C4fm run 0  -nowait"
      after idle "C4fm run 1  -nowait"
   }
 }


#****************************************************************************
# Name:        query_C4fm_state
# Parameters:  none
# Description: This function sends down a request to the C4 FM engine to 
#            : obtain the engine state 
#****************************************************************************
 proc query_C4fm_state { } \
 {
   set State(engineId) [format_version [C4fm ver 0] ]
   set State(driverId) [format_version [C4fm ver 1] ]
 }
 

#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the C4 FM engine to its current state
#****************************************************************************
 proc getSaveState { } {
   variable State 
   
   set returnList \
   {
     C4fm run    $State(Engine_run)        -nowait
     C4fm avg    $State(average)           -nowait
     C4fm trig   $State(Engine_trigger)    -nowait
     C4fm en     $State(Engine_enabled)    -nowait
     C4fm peak   $State(modacc,peakEnable) -nowait
     C4fm alerts $State(modacc,alerts)     -nowait
     C4fm ulim   $State(modacc,alertHi)    -nowait
     C4fm llim   $State(modacc,alertLo)    -nowait

     set ::C4fm::State(BellEnabled)   {$State(BellEnabled)}
     set ::C4fm::State(resolution)    {$State(resolution)}
     set ::C4fm::State(units)         {$State(units)}
     
     catch \
     {
        C4fm ferr_alerts         $State(freq_err,alerts)      -nowait
        C4fm dev_alerts          $State(deviation,alerts)     -nowait
        C4fm ferr_peakhold       $State(freq_err,peakEnable)  -nowait
        C4fm dev_peakhold        $State(deviation,peakEnable) -nowait
        C4fm ferr_ulimit         $State(freq_err,alertHi)     -nowait
        C4fm ferr_llimit         $State(freq_err,alertLo)     -nowait
        C4fm dev_ulimit          $State(deviation,alertHi)    -nowait
        C4fm dev_llimit          $State(deviation,alertLo)    -nowait
     }
   }
   return [subst -nocommands $returnList]
 }

#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to hardware
#****************************************************************************
 proc Sync { } { 
    DefaultSetup
    QuickStart
    eval [getSaveState]
    C4fm value
    variable State
    set State(init) yes
 }

#****************************************************************************
# Name:        DefaultSetup
# Parameters:  none
# Description: Sets the C4 FM engine with default values
#****************************************************************************
 proc DefaultSetup { } {
    C4fm avg 200                ;# Set number of symbols to average over
    C4fm trig continuous        ;# Enable continuous readings
 }

#****************************************************************************
# Name:        QuickStart
# Parameters:  none
# Description: Turns on the C4 FM engine but does not set up parameters 
#****************************************************************************
 proc QuickStart { } {
    C4fm en 3                   ;# Enable data to engine
    C4fm run ON                 ;# Start the engine
 }

#****************************************************************************
# Name:        QuickStop
# Parameters:  none
# Description: Handles a normal stop of the C4 FM engine 
#****************************************************************************
 proc QuickStop { } {
    C4fm run OFF                ;# Stop the engine
 }

#****************************************************************************
# Name:        SyncAll
# Parameters:  none
# Description: Insures that TCL, the engine and hardware are in sync
#****************************************************************************
 proc SyncAll { } {
    set_C4fm_callbacks
    query_C4fm_state
    if ![interp issafe] \
    {
       after 100 {C4fm::Sync}
    }
 }


#****************************************************************************
# Name:        Get_HTML
# Parameters:  base_url - the base URL for the containing page
# Description: Return an string containing HTML showing the current state
#              of this system
#****************************************************************************
 proc Get_HTML { {base_url ""} } \
 {
    variable State
    # if not running skip it
    if {$State(Engine_run) == 0 } \
    {
       return
    }
    # if the reading is stale, skip this
    if {$State(modacc,status) & 2} \
    {
       return
    }
    
     return [subst \
{
<table align="left" border=1>
<tr><td nowrap colspan=2><center><h2>P25 C4FM Modulation fidelity</h2></center></td></tr>
<tr><td nowrap>Modulation fidelity error</td><td nowrap>$::C4fm::State(modacc)%</td></tr>
<tr><td nowrap>Frequency error at symbol time</td><td nowrap>$::C4fm::State(freq_err) Hz</td></tr>
<tr><td nowrap>Deviation at symbol time (nominal 1800 Hz)</td><td nowrap>$::C4fm::State(deviation) Hz</td></tr>
</table>
}]
}
 
#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
   Load::on_system_done { ::C4fm::SyncAll }
   Alert::AddMeter C4fm

}  ;# End of the C4fm namespace

#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines C4fm
}

#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
     C4fm_Eng  "C4FM Engine"
     C4fm_Drv  "C4FM Driver"
   }
   set version(C4fm_Eng)  [C4fm version 0]
   set version(C4fm_Drv)  [C4fm version 1]
}


#****************************************************************************
# This registers the engine with the list for generating an HTML page
#****************************************************************************
namespace eval ::HTTPD \
{
   variable main_screen_items
   lappend  main_screen_items(2) C4fm
}
