#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : dns.tcl                                                    #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : DNS interface routines                                     #
#############################################################################
package require Callback
Callback

namespace eval ::DNS \
{
   ::load $object
   variable cache
   variable mx_cache
   variable pending
   variable lookup_count 0
   
   array set cache \
   {
     localhost             {NEVER A 127.0.0.1}
     ftp.ifrsys.com        {0 A 65.196.219.168}
     www.p25.com           {0 A 63.90.162.208}
     dumas1.eng.ifrsys.com {0 A 10.200.126.250}
   }
   array set mx_cache {}
   array set pending {}
   variable timeout 4000

   proc Update_DNS { args } \
   {
      catch {DNS server $DHCP::State(DNS)}
   }
   trace variable DHCP::State(DNS) w [namespace code Update_DNS]
   Update_DNS
   
   proc Callback { server_ip server_port transaction flags queries answers servers additional} \
   {
      variable pending
      variable cache
      variable mxcache
      variable timeout
      
      if ![info exists pending($transaction)] \
      {
         Debug::Puts DNS 1 "$transaction not found in pending list"
         puts "$queries\n$answers\n$servers\n$additional"
         return
      }

      set hostname [lindex $pending($transaction) 0]
      set tries    [lindex $pending($transaction) 1]
      set sent_to  [lindex $pending($transaction) 2]
      set cmd      [lindex $pending($transaction) 3]
      set check    [lindex $pending($transaction) 4]
      
      if {$sent_to != $server_ip} \
      {
         Debug::Puts DNS 1 "$server_ip responded to $transaction sent to $sent_to - ignoring"
         return;
      }
      set type [lindex $flags 0]
      if {$type != "RESPONSE"} \
      {
         Debug::Puts DNS 1 "$server_ip sent a $type to us? Ignoring."
         return;
      }
      # cancel timeout
      after cancel [namespace code "DNS_timeout $transaction"]
      unset pending($transaction)
      
      set code [lindex $flags 10]
      switch -- $code \
      {
         "Name invalid" -
         "OK" \
         {
         }
         default \
         {
            Debug::Puts DNS 3 "Result code $code - Using next server"
            DNS next
            
         }
      }
      # OK, plug any results into the cache
      foreach result "$answers $additional $servers" \
      {
         Debug::Puts DNS 10 "Parsing $result"
         set name [lindex $result 0]
         set type [lindex $result 1]
         set ttl  [lindex $result 3]
         # trim final "." off the name
         set name [string trimright $name "."]
         set now  [clock seconds]
         if {$ttl == ""} { set ttl 0}
         set expires [expr $now + $ttl]
         switch -- $type \
         {
            "CNAME" -
            "PTR" \
            {
               set addr [lindex $result 4]
               set cache($name) [list $expires $type $addr]
               Debug::Puts DNS 10 "create $type record for $name: $addr"
            }
            "A" \
            {
               set addr [lindex $result 4]
               if [info exists cache($name)] \
               {
                  set addrs [lrange $cache($name) 2 end]
                  if {[lsearch $addrs $addr] == -1} \
                  {
                      set cache($name) [list $expires $type $addrs $addr]
                  } \
                  else \
                  {
                      set cache($name) [list $expires $type $addr]
                  }
                  Debug::Puts DNS 10 "add $type record for $name: $addr TTL $ttl expires $expires"
               } \
               else \
               {
                  set cache($name) [list $expires $type $addr]
                  Debug::Puts DNS 10 "create $type record for $name: $addr TTL $ttl expires $expires"
               }
            }
            "MX" \
            {
               set pref [lindex $result 4]
               set addr [lindex $result 5]
               lappend mxcache($name) [list $expires $pref $addr]
               Debug::Puts DNS 10 "create $type record for $name: $pref $addr TTL $ttl expires $expires"
            }
         }
      }
      if {$cmd == ""} { return }
      
      set addr [eval $check $hostname]
      if {$addr != ""} \
      {
         after idle "$cmd [list $addr]"
      } \
      else \
      {
         after idle "$cmd ERROR"
      }
   }
   
   # given a host in cache, return an address & rotate the list of addresses
   proc Rotate_addrs { hostname } \
   {
      variable cache
      set expire [lindex $cache($hostname) 0]
      set type   [lindex $cache($hostname) 1]
      set addr   [lindex $cache($hostname) 2]
      
      # if this is a CNAME, then look up the real name
      switch -- $type \
      {
         "CNAME" \
         {
            Debug::Puts DNS 3 "$hostname CNAME $addr"
            return [Check_cache $addr]
         }
         "PTR" \
         {
            return $addr
         }
         "A" \
         {
            # remove the address, and add to end
            set l [lrange $cache($hostname) 3 end]
            set cache($hostname) [concat $expire $type $l $addr]
            return $addr
         }
         default \
         {
            return
         }
      }
   }
   
      
   # check if hostname is in cache and if TTL not exceeded.
   proc Check_cache { hostname {allow_stale 0} } \
   {
      variable cache
      set hostname [string trimright $hostname "."]
      if ![info exists cache($hostname)] {return }
      # if in the cache, check TTL and rotate addresses
      set expire [lindex $cache($hostname) 0]
      if {$expire == "NEVER"}        {return [Rotate_addrs $hostname]}
      set now [clock seconds]
      if {$expire > $now} {return [Rotate_addrs $hostname]}
      Debug::Puts DNS 3 "Expiring $hostname as $expire < $now"
      if {$allow_stale != 0} \
      {
         return [Rotate_addrs $hostname]
      }
      return
   }
   proc sort_mx { a b } \
   {
      set p_a [lindex $a 1]
      set p_b [lindex $b 1]
      if {$p_a < $p_b} { return -1}
      if {$p_a > $p_b} { return 1}
      return 0
   }
   proc Check_MX_cache { hostname {allow_stale 0} } \
   {
      variable mxcache
      set hostname [string trimright $hostname "."]
      if ![info exists mxcache($hostname)] {return }
      # scan the list, purge any stale entries
      set now [clock seconds]
      foreach item $mxcache($hostname) \
      {
         set expire [lindex $item 0]
         set pref    [lindex $item 1]
         set host    [lindex $item 2]
         if {$now < $expire} \
         {
            lappend result [list $expire $pref $host]
         }
      }
      if [info exists result] \
      {
         set mxcache($hostname) [lsort -command sort_mx -increasing $result]
         return $result
      }
      unset mxcache($hostname)
      return {}
   }
   proc DNS_timeout { transaction {query Do_A_query}} \
   {
      variable pending
      Debug::Puts DNS 1 "$transaction timeout"
      if ![info exists pending($transaction)] \
      {
         Debug::Puts DNS 1 "Invalid/missing transaction $transaction"
         return
      }
      Retry $transaction $query
   }
   proc Retry { transaction {query Do_A_query}} \
   {
      variable pending
      variable cache
      set hostname [lindex $pending($transaction) 0]
      set tries    [lindex $pending($transaction) 1]
      set cmd      [lindex $pending($transaction) 3]
      set check    [lindex $pending($transaction) 4]
      unset pending($transaction)
      # tried all known servers?
      set max [llength $DHCP::State(DNS)]
      if {$tries >= $max} \
      {
         if {$cmd != ""} \
         {
            # if this is still in the cache, use the stale value. Else error
            set addr [eval $check $hostname 1]
            if {$addr != ""} \
            {
               after idle "$cmd [list $addr]"
            } \
            else \
            {
               after idle "$cmd ERROR"
            }
         }
         return
      }
      DNS next
      eval $query "$hostname" [incr tries] [list $cmd]
   }
   
   proc Do_MX_query { hostname {tries 0} {cmd {}} } \
   {
      variable pending
      variable timeout
      
      if [oldCatch \
      {
         set transaction [DNS mx $hostname]
         set sent_to     [lindex $transaction 1]
         set transaction [lindex $transaction 0]
         set pending($transaction) [list $hostname $tries $sent_to $cmd Check_MX_cache]
         Debug::Puts DNS 3 "set up pending $transaction to $pending($transaction)"
         after $timeout [namespace code "DNS_timeout $transaction Do_MX_query"]
      } err] \
      {
        if {$cmd != ""} \
         {
            # if this is still in the cache, use the stale value. Else error
            set addr [Check_MX_cache $hostname 1]
            if {$addr != ""} \
            {
               after idle "$cmd $addr"
            } \
            else \
            {
               after idle "$cmd ERROR"
            }
         }
      }
   }
   proc Do_A_query { hostname {tries 0} {cmd {}} } \
   {
      variable pending
      variable timeout
      
      if {$hostname == "localhost"} { return }
      
      if [oldCatch \
      {
         set transaction [DNS address $hostname]
         set sent_to     [lindex $transaction 1]
         set transaction [lindex $transaction 0]
         set pending($transaction) [list $hostname $tries $sent_to $cmd Check_cache]
         Debug::Puts DNS 3 "set up pending $transaction to $pending($transaction)"
         after $timeout [namespace code "DNS_timeout $transaction"]
      } err] \
      {
        if {$cmd != ""} \
         {
            # if this is still in the cache, use the stale value. Else error
            set addr [Check_cache $hostname 1]
            if {$addr != ""} \
            {
               after idle "$cmd $addr"
            } \
            else \
            {
               after idle "$cmd ERROR"
            }
         }
      }
   }
   
   proc Resolve { hostname {cmd {}} } \
   {
      variable cache
      variable pending
      variable timeout
      
      set element "Domain name"
      set domain $DHCP::State($element)
      
      # already an IP address?
      if [regexp {[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+} $hostname] \
      {
         if {$cmd != ""} {after idle "$cmd $hostname"}
         return
      }
      # need to append domain name?
      if ![string match *.* $hostname] \
      {
         append hostname .$domain
         Debug::Puts DNS 2 "Added $domain to $hostname"
      }
      # remove any trailing .
      set hostname [string trimright $hostname "."]
      # already in cache?
      set addr [Check_cache $hostname]
      Debug::Puts DNS 2 "Look for $hostname, found \"$addr\""
      if {$addr != ""} \
      {
         if {$cmd != ""} {after idle "$cmd $addr"}
         return
      }
      # nope. Try to look up the address
      Do_A_query $hostname 0 $cmd
   }
   
   # synchronous lookup
   proc Lookup { hostname }\
   {
      set addr [Check_cache $hostname]
      if {$addr != ""} { return $addr}
      
      variable lookup_count
      variable pending
      set me "lookup[incr lookup_count]"
      set ::DNS::pending($me) {}
      Resolve $hostname "set ::DNS::pending($me)"
      vwait ::DNS::pending($me)
      if {$pending($me) == "ERROR"} \
      {
         error "Cannot resolve $hostname"
      }
      set addr $::DNS::pending($me)
      unset ::DNS::pending($me)
      return $addr
   }
   proc Lookup_MX { hostname }\
   {
      set addr [Check_MX_cache $hostname]
      if {$addr != ""} { return $addr}
      
      variable lookup_count
      variable pending
      set me "lookup[incr lookup_count]"
      set ::DNS::pending($me) {}
      Do_MX_query $hostname 0 "set ::DNS::pending($me)"
      vwait ::DNS::pending($me)
      if {$pending($me) == "ERROR"} \
      {
         error "Cannot resolve $hostname"
      }
      set addr $::DNS::pending($me)
      unset ::DNS::pending($me)
      return $addr
   }
   
    
   # replacement for TCL "socket" command with hostname lookup
   proc Socket { args } \
   {
      foreach arg $args \
      {
         if [string match "-server" $arg] \
         {
            return [eval oldSocket $args]
         }
         if [string match "-*" $arg] \
         {
            lappend options $arg
         } \
         else \
         {
            # OK, -server not found, this must be the host name
            lappend values $arg
         }
      }
      # replace the first element of $values with the host IP
      if ![info exists values]\
      {
         return [eval oldSocket $args]
      }
      set host [lindex $values 0]
      set port [lindex $values 1]
      set addr [DNS::Lookup $host]
      if [info exists options] \
      {
         return [eval oldSocket $options $addr $port]
      } \
      else \
      {
         return [eval oldSocket $addr $port]
      }
   }
}
rename socket oldSocket
rename ::DNS::Socket socket
