#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : dvm.tcl                                                     #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the Dvm command is         #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::Dvm::State ruw "::Dvm::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
# Call the base class dummy command to force a load
if {"[info command Dvm]" != "" } { return }
package require Engtclbase
Engtclbase

# Load the object file
::load $::Dvm::object

package provide Dvm 1.0

#****************************************************************************
# Add functions to the Dvm namespace
#****************************************************************************
#    Status of commands sent to Dvm
#       ::Dvm::State(Engine_run)          {0 | 1}
#       ::Dvm::State(run)                 {OFF | ON}
#       ::Dvm::State(average)             {2 - 255}
#       ::Dvm::State(Engine_enabled)      {0 | 1}
#       ::Dvm::State(enabled)             {OFF | ON}
#       ::Dvm::State(driverId)            Driver ID information
#       ::Dvm::State(engineId)            Engine ID information
#    Status of counter data received from the engine
#       ::Dvm::State(reading)             Current meter value
#    Status of alert limits and peak values
#       ::Dvm::State(peakHi)              Maximum peak reading
#       ::Dvm::State(peakLo)              Minimum peak reading
#       ::Dvm::State(peakFlags)           Peak Hold status flags
#       ::Dvm::State(peakEnable)          Peak Enable settings
#       ::Dvm::State(alertHi)             Maximum valid reading limit
#       ::Dvm::State(alertLo)             Minimum valid reading limit
#       ::Dvm::State(alerts)              Alert flag status
#****************************************************************************
namespace eval ::Dvm {

#****************************************************************************
# Name:        setResistance
# Parameters:  none
# Description: sets the assumed resistance for User set impedance
#****************************************************************************
 proc setResistance { ohms } \
 {
    variable State
    set State(ohms) $ohms
    if {$State(load) == 0 } \
    {
       set State(current_ohms) $ohms
    }
    updatedBm
 }
 
 
#****************************************************************************
# Name:        set_dvm_callbacks
# Parameters:  none
# Description: Sets up the callbacks to update the Dvm data
#****************************************************************************
 proc set_dvm_callbacks { } {

   Dvm -onchange reading { ::Dvm::updateReading %V; }
   
   Dvm -onchange readpeak    { ::Dvm::updatePeaks %V }
   Dvm -onchange ulimit      { ::Dvm::updateAlertHi %V }
   Dvm -onchange llimit      { ::Dvm::updateAlertLo %V }
   
   Dvm -onchange alerts      { ::Dvm::updateAlerts %V }
   Dvm -onchange peak        { ::Dvm::updatePeakEnable %V }

   Dvm -onchange run         { ::Dvm::updateMode %V }
   Dvm -onchange avg         { set ::Dvm::State(average) %V }
   Dvm -onchange en          { ::Dvm::updateEnable %V }
   Dvm -onchange load        { ::Dvm::updateLoad %V }
   Dvm -onchange mode        { ::Dvm::updateDC %V }
   Dvm -onchange hwrange     { ::Dvm::updateHWrange %V }

 }
#****************************************************************************
# Name:        updateHWrange
# Parameters:  mode
# Description: Tracks changes in hardware range
#****************************************************************************
 proc updateHWrange { range } \
 {
    variable State
    set State(hwrange,raw) $range
    if {$range == 0} \
    {
       set State(hwrange) "AUTO"
    } \
    else \
    {
       set State(hwrange) $range
    }
 }
 
#****************************************************************************
# Name:        updateDC
# Parameters:  mode
# Description: Tracks changes in AC/DC mode
#****************************************************************************
 proc updateDC { mode } {
   variable State
   set State(mode,raw) $mode
   switch -- $mode \
   {
      1 { set State(mode) AC }
      2 { set State(mode) DC }
      default { set State(mode) invalid }
   }
 }
#****************************************************************************
# Name:        updateLoad
# Parameters:  ohms
# Description: Tracks changes in load setting
#****************************************************************************
 proc updateLoad { ohms } {
   variable State
   set State(load) $ohms
   if {$State(load) != 0 } \
   {
      set State(current_ohms) $ohms
   } \
   else \
   {
      set State(current_ohms) $State(ohms)
   }
 }
#****************************************************************************
# Name:        updateMode
# Parameters:  runState         0 = Off, 1 = On
# Description: Sets up the current run state of the Dvm
#****************************************************************************
 proc updateMode { runState } {
   switch $runState {
      0       { set ::Dvm::State(run) OFF }
      1       { set ::Dvm::State(run) ON }
      default { return }
   }
   set ::Dvm::State(Engine_run) $runState
 }

#****************************************************************************
# Name:        updateEnable
# Parameters:  enableBits       1 = Off, 3 = Enabled
# Description: Sets up the current enabled state of the Dvm
#****************************************************************************
 proc updateEnable { enableBits } {
   switch $enableBits {
      3       { set ::Dvm::State(enabled) Enabled }
      default { set ::Dvm::State(enabled) Disabled }
   }
   set ::Dvm::State(Engine_enabled) $enableBits
 }

#****************************************************************************
# Name:        to_dBm
# Parameters:  volts
# Description: Computes dBm given voltage and resistance
#****************************************************************************
 proc to_dBm { volts } \
 {
    variable State

    # set up units conversions
    if {($State(current_ohms) > 0) && ($volts != 0) }\
    {
      return [format %0.2f [expr 10 * log10(($volts*$volts)/$State(current_ohms))+30]]
    } \
    else \
    {
      return ---
    }
 }
#****************************************************************************
# Name:        to_V
# Parameters:  dBm
# Description: Computes voltage given dBm and resistance
#****************************************************************************
 proc to_V { dBm } \
 {
    variable State

    # set up units conversions
    if {($State(current_ohms) > 0) && ($dBm != "---") }\
    {
      return [expr sqrt(pow(10.0,$dBm/10.0 - 3)*$State(current_ohms))]
    }
    return ""
 }
#****************************************************************************
# Name:        ulimit
# Parameters:  value, units (defaults to V)
# Description: sets the upper limit
#****************************************************************************
 proc ulimit { value {units V} } \
 {
    switch -- $units \
    {
      "V" \
      {
      }
      "dBm" \
      {
         set value [to_V $value]
         if {"$value" == "" } \
         {
            # Try querying the current setting..
            Dvm ulimit
            return
         }
      }
      default \
      {
         error "bad units $units"
      }
   }
   Dvm ulimit $value -nowait
 }
 #****************************************************************************
# Name:        llimit
# Parameters:  value, units (defaults to V)
# Description: sets the lower limit
#****************************************************************************
 proc llimit { value {units V} } \
 {
    switch -- $units \
    {
      "V" \
      {
      }
      "dBm" \
      {
         set value [to_V $value]
         if {"$value" == "" } \
         {
            # Try querying the current setting..
            Dvm llimit
            return
         }
      }
      default \
      {
         error "bad units $units"
      }
   }
   Dvm llimit $value -nowait
 }
#****************************************************************************
# Name:        updatedBm
# Parameters:  none
# Description: Computes dBm given voltage and resistance
#****************************************************************************
 proc updatedBm {  } \
 {
    variable State
    set State(reading,dBm)  [to_dBm $State(reading)]
    set State(peak,dBm)     [to_dBm $State(peak)]
    set State(peakHi,dBm)   [to_dBm $State(peakHi)]
    set State(peakLo,dBm)   [to_dBm $State(peakLo)]
    set State(alertHi,dBm)  [to_dBm $State(peakHi)]
    set State(alertLo,dBm)  [to_dBm $State(peakLo)]
 }
 
#****************************************************************************
# Name:        updateReading
# Parameters:  readingList      List of data from the engine
#                reading        Voltage reading from the engine
#                status         Status flags
# Description: Sets up the current state of the Dvm reading
#****************************************************************************
 proc updateReading { readingList } {

   variable State
   # Get the data from the meter reading
   set State(raw)        [lindex $readingList 0]
   set State(precision)  [lindex $readingList 1]
   set State(status)     [lindex $readingList 2]

   set State(reading,dBm) [to_dBm $State(raw)]

   set State(reading)     [format %.2f $State(raw)]
   set State(reading,V)   [format %.2f $State(raw)]
   if [info exists State(reading,$State(units))] \
   {
      set State(reading,any) $State(reading,$State(units))
   } \
   else \
   {
      set State(reading,any) $State(reading,V)
   }
 }

#****************************************************************************
# Name:        updatePeaks
# Parameters:  readingList      List of data from the engine
#                 Peak HI       Max peak value
#                 HI Precision  Max peak precision
#                 Peak LO       Min peak value
#                 LO Precision  Min peak precision
#                 Flags         Status flags
# Description: Sets up the current state of the peak readings
#****************************************************************************
 proc updatePeaks { readingList } {
   variable State
   # Get the data from the peak readings
   foreach {peakHi foo peakLo foo2 flags } $readingList \
   {
   }
   set State(peakFlags) $flags
   
   set peak [expr abs($peakLo)]
   if {$peak < $peakHi } { set peak $peakHi }
   
   set State(peak)      $peak
   set State(peakHi)    $peakHi
   set State(peakLo)    $peakLo
   set State(peak,V)      [format "%0.2f" $peak]
   set State(peakHi,V)    [format "%0.2f" $peakHi]
   set State(peakLo,V)    [format "%0.2f" $peakLo]
   set State(peak,dBm)    [to_dBm $peak]
   set State(peakHi,dBm)  [to_dBm $peakHi]
   set State(peakLo,dBm)  [to_dBm $peakLo]
 }


#********************************************************************
#
#       Name: updatePeakEnable
#
# Parameters: 
#
# Description: Conditions the 1/0 value from the engine
#            : to a "ON"/"OFF" string for the user interface
#
#********************************************************************
proc updatePeakEnable { peakEnable } \
 {
   variable State
   
    set State(peakEnable)  $peakEnable

   if { $State(peakEnable) == 1 } \
   {    
     set State(peakEnable_text)  "ON"
  
     # The following maintains compatibility with the present 'CanvasWidget'
     set ::DvmUI::StateUI(peak_hold)  "ON"
   } \
   else \
   {
     set State(peakEnable_text)  "OFF"

     # The following maintains compatibility with the present 'CanvasWidget'
     set ::DvmUI::StateUI(peak_hold)  "OFF"
   }
    
   return
 }
  

#****************************************************************************
# Name:        updateAlerts
# Parameters:  
# Description: Update the Alerts flags
#****************************************************************************
 proc updateAlerts { alerts } \
 {
   variable State
 
   set State(alerts)  $alerts
   if {$alerts & 2} \
   {
     set State(alertHi_text) "ON"
   } \
   else \
   {
     set State(alertHi_text) "OFF"
   }
   if {$alerts & 1} \
   {
     set State(alertLo_text) "ON"
   } \
   else \
   {
     set State(alertLo_text) "OFF"
   }
   if {$alerts & 4} \
   {
     set State(alarm_text)       "ON"
     set State(BellEnabled_text) "ON"

     set State(BellEnabled) 1
   } \
   else \
   {
     set State(alarm_text)       "OFF"
     set State(BellEnabled_text) "OFF"

     set State(BellEnabled) 0
   }
 }


#****************************************************************************
# Name:        updateAlertHi
# Parameters:  
# Description: Formats the Alert High value
#****************************************************************************
 proc updateAlertHi { alertHi } \
 {
   variable State
   set State(alertHi)     $alertHi
   set State(alertHi,V)   [format "%0.2f" $alertHi]
   set State(alertHi,dBm) [to_dBm $alertHi]

 }

#****************************************************************************
# Name:        updateAlertLo
# Parameters:  
# Description: Formats the Alert Low value
#****************************************************************************
 proc updateAlertLo { alertLo } \
 {
   variable State
   set State(alertLo)     $alertLo
   set State(alertLo,V)   [format "%0.2f" $alertLo]
   set State(alertLo,dBm) [to_dBm $alertLo]

 }


#********************************************************************
#
#       Name: toggleAlarm
#
# Parameters:
#
# Decsription: Toggles the state of the alarm
#
#********************************************************************
proc toggleAlarm { } \
 {
   variable State
   Dvm alerts [expr $State(alerts) ^ 4] -nowait
 }


#********************************************************************
#
#       Name: togglePeakEnable
#
# Parameters:
# Decsription: Displays PH and sets peak hold variable
#
#********************************************************************
proc togglePeakEnable {  } \
 {
   variable  State
   
   if {$State(peakEnable) == 1 } \
   {
     after idle "Dvm peak 0 -nowait"
   } \
   else \
   {
     after idle "Dvm peak 1 -nowait"
     after idle "Dvm phreset  -nowait"
   }
 }
  
  
#********************************************************************
#
#       Name: toggleHiLimit
#
# Parameters: 
#
# Description: change the state of the high limit
#
#********************************************************************
proc toggleHiLimit { } \
 {
   variable  State
   
   Dvm alerts [expr $State(alerts) ^ 0x2]
 }

  
#********************************************************************
#
#       Name: toggleLoLimit
#
# Parameters: 
#
# Description: change the state of the low limit
#
#********************************************************************
proc toggleLoLimit { } \
 {
   variable  State

   Dvm alerts [expr $State(alerts) ^ 0x1]
 }  

  
#********************************************************************
#
#       Name: setAverage
#
# Parameters:
#
# Description: set the meter average
#
#********************************************************************
proc setAverage { } \
 {
   #puts "setAverage"
    
   variable  State
   
   # Set the meter engine
   after idle "Dvm  average  $State(average)  -nowait"
      
   return
 }


#********************************************************************
#
#       Name: setMode
#
# Parameters: 
#
# Decsription: sets meter mode
#
#********************************************************************
proc setMode {  } \
 {
   #puts "setMode:  $meter_type"
   
   variable  State

   if { $State(mode) == "AC" } \
   { 
     Dvm mode DC -nowait
   } \
   else \
   {
     Dvm mode AC -nowait
   }
 }


#****************************************************************************
# Name:        query_dvm_state
# Parameters:  none
# Description: This function sends down a request to the Dvm engine to 
#            : obtain the engine state 
#****************************************************************************
 proc query_dvm_state { } {

   set ::Dvm::State(engineId) [format_version [Dvm ver 0] ]
   set ::Dvm::State(driverId) [format_version [Dvm ver 1] ]

 }
 
#****************************************************************************
# Name:        Get_HTML_reading
# Parameters:  base_url - the base URL for the containing page
# Description: Return an string containing HTML showing the current state
#              of this system
#****************************************************************************
 proc Get_HTML_reading { {base_url ""} } \
 {
    variable State
    if {"$State(units)" != "V"} \
    {
       set x \
{<tr>
 <td nowrap>DVM</td>
 <td nowrap>$State(reading,any) $State(units) (@$State(current_ohms) ohms)</td>
</tr>}
    } \
    else \
    {
       set x \
{<tr><td nowrap>DVM</td><td nowrap>$State(reading,any) $State(units)</td></tr>}
    }
    return [subst $x]
 }
 
#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the Dvm engine to its current state
#****************************************************************************
 proc getSaveState { } {

   variable State
   set returnList \
   {
      Dvm run $State(Engine_run) -nowait
      Dvm avg $State(average) -nowait
      Dvm en $State(Engine_enabled) -nowait
      Dvm peak $State(peakEnable) -nowait
      Dvm alerts $State(alerts) -nowait
      Dvm::setResistance $State(ohms)
      Dvm ulim $State(alertHi) -nowait
      Dvm llim $State(alertLo) -nowait
      Dvm impedance $State(load) -nowait
      Dvm mode $State(mode) -nowait
      Dvm hwrange $State(hwrange,raw) -nowait

      set ::Dvm::State(BellEnabled)   {$State(BellEnabled)}
      set ::Dvm::State(units)         {$State(units)}
   }
   
   return [subst -nocommands $returnList]

 }

#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to hardware
#****************************************************************************
 proc Sync { } {
    after 100 {eval [::Dvm::getSaveState]}
 }

#****************************************************************************
# Name:        SyncAll
# Parameters:  none
# Description: Insures that TCL, the engine and hardware are in sync
#****************************************************************************
 proc SyncAll { } {
    set_dvm_callbacks
    query_dvm_state
    if ![interp issafe] \
    {
       Sync
       Dvm linefilter 60 -nowait
    }
 }

#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
   ::Load::on_UI_done ::Dvm::SyncAll 

}  ;# End of the Dvm namespace

#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines Dvm
}

#****************************************************************************
# This registers the engine with the list for generating an HTML page
#****************************************************************************
namespace eval ::HTTPD \
{
   variable main_screen_readings
   lappend  main_screen_readings(1) Dvm
}

#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
     Dvm_Eng   "DVM Engine"
     Dvm_Drv   "DVM Driver"
   }
   set version(Dvm_Eng)   [Dvm ver 0]
   set version(Dvm_Drv)   [Dvm ver 1]
}
