#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : fgen.tcl                                                    #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the FGen command is first  #
#             : used.  It loads the C++ interface file, provides an FGen    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################

trace vdelete ::FGen::State ruw "::FGen::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
# Call the base class dummy command to force a load
if {"[info command Fgen]" != "" } { return }
package require Engtclbase
Engtclbase

package require  Parse_list

package require  CTCSS

# Load the object file
::load $::FGen::object

interp alias {} Fgen {} FGen

package provide Fgen 1.0
package provide FGen 1.0


#****************************************************************************
# Set up a namespace for FGen
#****************************************************************************
namespace eval ::FGen \
{
  # This data is updated by the engines through callback routines, and it
  # indicates (in a screen format) the last setting received from an engine

  set eng_modes {OFF TONE}
  
  set shapes {SINE SQUARE TRIANGL RAMP}
   
  array set cmd_map \
  {
    OFF       -1
    SINE      0
    TRIANGLE  1
    SQUARE    2
    RAMP      3
    DC        4
    ARBITRARY 5
    ARB       5
    DTMF      6
    EVENT     7
    DCS       8
    SWEEP     10
 
    Sine   0
    Tri    1
    Sqr    2
    Rmp    3
    DC     4
    Arbit  5
    DTMF   6
    DCS    8
  }
   
   
  array set shape_map \
  {
    -1 OFF
    0 SINE 
    1 TRIANGLE
    2 SQUARE
    3 RAMP
    4 DC
    5 ARBITRARY
    6 DTMF
    8 DCS
    10 SWEEP
  }


  array set sname_shape_map \
  {
    -1 Off
    0 Sine
    1 Tri
    2 Sqr
    3 Rmp
    4 DC
    5 Arbit
    6 DTMF
    8 DCS
    10 SWEEP
  }

  array set engine_mode_map \
  {
    OFF         0
    TONE        1
    CONTINUOUS  1
    BURST       2
    TREM        3
    DTMF        4
    CTCSS       5
    DCS         6
    TSIG        7
    P25         8
    SINC        9
    ARB         10
    
    0   OFF
    1   TONE
    2   BURST
    3   TREM
    4   DTMF
    5   CTCSS
    6   DCS
    7   TSIG
    8   P25
    9   SINC
    10  ARB
  }
   



  #****************************************************************************
  # Name:        set_fgen_callbacks
  # Parameters:  none
  # Description: Sets up the callbacks to update the FGEN data
  #****************************************************************************
  proc set_fgen_callbacks { } \
  {
    variable State
    FGen -queue mode0   "::FGen::updateMode  %L %V"
    FGen -queue dur0    "set ::FGen::State(%L,dur) %V"
    FGen -queue shape0  "::FGen::updateShape %L %V"
    FGen -queue level0  "::FGen::updateLevel %L %V"
    FGen -queue freq0   "::FGen::updateFreq  %L %V"
    FGen -queue dev0    "::FGen::updateLevel %L %V"
    FGen -queue per0    "::FGen::updateLevel %L %V"
    FGen -queue status0 "::FGen::updateStatus %L %V"
    
    foreach lun { 0 1 2 3 4  } \
    {
      set State($lun,XOFF) 0
      set State($lun,sending) 0
      set State($lun,abort) 0
      FGen mode$lun
      FGen dur$lun
      FGen shape$lun
      FGen level$lun
      FGen freq$lun
      FGen dev$lun
      FGen per$lun
    }
  }
 

  #****************************************************************************
  # Name:        Mode
  # Parameters:  lun      which FGEN is affected
  #              state    0 = OFF, 1 = TONE
  # Description: Set the mode based on the string
  #****************************************************************************
  proc Mode { lun state } \
  {
    FGen mode$lun $::FGen::engine_mode_map($state) -nowait
  }
 
 
  #****************************************************************************
  # Name:        updateMode
  # Parameters:  lun      which FGEN is affected
  #              state    0 = OFF, 1 = TONE
  # Description: set the mode based on the string
  #****************************************************************************
  proc updateMode { lun  mode } \
  {
    #puts "updateMode:  $lun,  $mode"

    variable  State

    if { [info exists ::FGen::engine_mode_map($mode)] } \
    {
      set mode_text  $::FGen::engine_mode_map($mode)
    }
    set State($lun,fgen_mode) $mode_text

    initMode $lun $mode
  }
  
  
  #****************************************************************************
  # Name:        Shape
  # Parameters:  lun              which FGEN is affected
  #              waveShape        string for shape
  # Description: Set the shape based on the name of the shape
  #****************************************************************************
  proc Shape { lun  waveShape } \
  {
    if { [info exists ::FGen::cmd_map($waveShape)] } \
    {
      after idle "Fgen shape$lun $::FGen::cmd_map($waveShape) -nowait"
    }

    return
  }


  #****************************************************************************
  # Name:        updateShape
  # Parameters:  lun              which FGEN is affected
  #              waveShape        engine code for shape
  # Description: Creates (if needed) and packs a small FGEN tile in the container
  #****************************************************************************
  proc updateShape { lun  waveShape } \
  {
    #puts "updateShape:  $lun,  $waveShape"
    
    variable  State

    set State($lun,shape_raw)  $waveShape

    # Update the 'shape' string text, short string fgen3 & fgen4 and
    # long string fgen1 & fgen2
    if { $lun > 2 } \
    {
      if [info exists ::FGen::sname_shape_map($waveShape)] \
      {
        set State($lun,shape) $::FGen::sname_shape_map($waveShape)
      }

    } \
    else \
    {
      if [info exists ::FGen::shape_map($waveShape)] \
      {
        set State($lun,shape)  $::FGen::shape_map($waveShape)
      }
    }
   
    
    #########################################################################
    #
    # The following code section is used to maintain compatibility
    # with the newer Function Generator calls until the existing 
    # Function Generator RCI commands are deprecated
    #
    #########################################################################
    set mode  $State($lun,fgen_mode)

    switch -- $mode \
    {
      "OFF"         {  }
      
      "TONE"        { if { $State($lun,shape) == $State($lun,TONE,shape) } \
                      {
                        # This is a query, no action required
                        #puts "Query only"
                        
                        return
                      }
                      
                      # Update the TONE shape variable
                      set State($lun,TONE,shape)  $State($lun,shape)                         
                      
                      setShape  $lun  $State($lun,TONE,shape) }
      
      "CONTINUOUS"  {  }
      
      "BURST"       {  }
      
      "TREM"        {  }
      
      "DTMF"        {  }

      "CTCSS"       {  }

      "DCS"         {  }

      "TSIG"        {  }

      "P25"         { if { $State($lun,shape) == $State($lun,P25,shape) } \
                      {
                        # This is a query, no action required
                        #puts "Query only"
                        
                        return
                      }
                       
                      # Update the P25 shape variable
                      set State($lun,P25,shape)  $State($lun,shape)

                      setShape  $lun  $State($lun,P25,shape) }

      "SINC"        {  }

      "ARB"         {  }

      default       {  }
    }

    return
  }


  #****************************************************************************
  # Name:        updateLevel 
  # Parameters:  lun              which FGEN is affected
  #              level setting (in volts)
  # Description: update the level setting
  #****************************************************************************
  proc updateLevel { lun  values } \
  {
    #puts "updateLevel:  $lun,  $values"

    variable  State

    foreach { level percent dev } $values { }
 
    set State($lun,level,raw)    $level
    set State($lun,percent,raw)  $percent
    set State($lun,dev,raw)      $dev
    set State($lun,level)        [format "%2.3f" $level]
    if { $lun == 1 || $lun == 2 } \
    {
       set State($lun,TONE,level_rms) [format "%2.3f" [expr ($level * 0.3535534) + 0.0005] ]
    }
    set State($lun,percent)      [format "%3.2f" $percent]
    set State($lun,dev)          [format "%3.2f" $dev]


    #########################################################################
    #
    # The following code section is used to maintain compatibility
    # with the newer Function Generator calls until the existing 
    # Function Generator RCI commands are deprecated
    #
    #########################################################################
    set mode  $State($lun,fgen_mode)
    
    #puts "  mode:  $mode"

    if { $State($lun,level) == $State($lun,$mode,level) } \
    {
      # This is a query, no action required
      #puts "  Query only"
      
      return
    }
    
    #puts "  Set value"
    
    # Update the 'mode' level variable
    set State($lun,$mode,level)  $State($lun,level)
    
    #puts "  State($lun,$mode,level):  $State($lun,$mode,level)"
    
    # Update the 'mode' deviation variable
    set State($lun,$mode,dev)  $State($lun,dev)

    # Update the 'mode' percent deviation variable
    set State($lun,$mode,percent)  $State($lun,percent)

    if { $lun == 1 || $lun == 2 } \
    {
      switch -- $mode \
      {
        "OFF"         { FGen::setLevel  $lun  $State($lun,OFF,level)   }
        
        "TONE"        { FGen::setLevel  $lun  $State($lun,TONE,level)  }
        
        "CONTINUOUS"  { FGen::setLevel  $lun  $State($lun,TONE,level)  }
        
        "BURST"       { }
        
        "TREM"        { FGen::setLevel  $lun  $State($lun,TREM,level)  }
        
        "DTMF"        { FGen::setLevel  $lun  $State($lun,DTMF,level)  }
      
        "CTCSS"       { FGen::setLevel  $lun  $State($lun,CTCSS,level) }
      
        "DCS"         { FGen::setLevel  $lun  $State($lun,DCS,level)   }
      
        "TSIG"        { }
      
        "P25"         { FGen::setLevel  $lun  $State($lun,P25,level)   }
      
        "SINC"        { FGen::setLevel  $lun  $State($lun,SINC,level)  }
      
        "ARB"         { FGen::setLevel  $lun  $State($lun,ARB,level)   }
      
        default       { }
      }
    } \
    elseif { $lun == 3 || $lun == 4 } \
    {
      switch -- $mode \
      {
        "OFF"         { FGen::setDeviation  $lun  $State($lun,OFF,dev)   }
        
        "TONE"        { FGen::setDeviation  $lun  $State($lun,TONE,dev)  }
        
        "CONTINUOUS"  { FGen::setDeviation  $lun  $State($lun,TONE,dev)  }
        
        "BURST"       { }
        
        "TREM"        { FGen::setDeviation  $lun  $State($lun,TREM,dev)  }
        
        "DTMF"        { FGen::setDeviation  $lun  $State($lun,DTMF,dev)  }
      
        "CTCSS"       { FGen::setDeviation  $lun  $State($lun,CTCSS,dev) }
      
        "DCS"         { FGen::setDeviation  $lun  $State($lun,DCS,dev)   }
      
        "TSIG"        { }
      
        "P25"         { FGen::setDeviation  $lun  $State($lun,P25,dev)   }
      
        "SINC"        { FGen::setDeviation  $lun  $State($lun,SINC,dev)  }
      
        "ARB"         { FGen::setDeviation  $lun  $State($lun,ARB,dev)   }
      
        default       { }
      }
    }

    return
  }
  
  
  #****************************************************************************
  # Name:        updateFreq 
  # Parameters:  lun              which FGEN is affected
  #              frequency in hz
  # Description: update the frequency
  #****************************************************************************
  proc updateFreq { lun  freq } \
  {
    #puts "updateFreq:  $lun,  $freq"

    variable  State
    
    set State($lun,freq)  [format "%5.1f" $freq]
    

    #########################################################################
    #
    # The following code section is used to maintain compatibility
    # with the newer Function Generator calls until the existing 
    # Function Generator RCI commands are deprecated
    #
    #########################################################################
    set mode  $State($lun,fgen_mode)

    switch -- $mode \
    {
      "OFF"         {  }
      
      "TONE"        { if { $State($lun,freq) == $State($lun,TONE,freq) } \
                      {
                        # This is a query, no action required
                        #puts "Query only"
                        
                        return
                      }
    
                      # Update the TONE level variable
                      set State($lun,TONE,freq)  $State($lun,freq)                         
    
                      FGen::setFreq  $lun  $State($lun,freq) }
      
      "CONTINUOUS"  {  }
      
      "BURST"       {  }
      
      "TREM"        {  }
      
      "DTMF"        {  }

      "CTCSS"       { if { $State($lun,freq) == $State($lun,CTCSS,freq) } \
                      {
                        # This is a query, no action required
                        #puts "Query only"
                        
                        return
                      }
    
                      # Update the CTCSS level variable
                      set State($lun,CTCSS,freq)  $State($lun,freq)                         
    
                      FGen::setFreq  $lun  $State($lun,freq) }

      "DCS"         {  }

      "TSIG"        {  }

      "P25"         { if { $State($lun,freq) == $State($lun,P25,freq) } \
                      {
                        # This is a query, no action required
                        #puts "Query only"
                        
                        return
                      }
    
                      # Update the P25 frequency variable
                      set State($lun,P25,freq)  $State($lun,freq)                         
    
                      FGen::setFreq  $lun  $State($lun,freq) }

      "SINC"        { if { $State($lun,freq) == $State($lun,SINC,freq) } \
                      {
                        # This is a query, no action required
                        #puts "Query only"
                        
                        return
                      }
                      
                      # Update the SINC frequency and rate variables
                      set State($lun,SINC,freq)  $State($lun,freq)
                      set State($lun,SINC,rate)  \
                                          [expr int($State($lun,freq) * 10.0)]
                      
                      FGen::setFreq  $lun  $State($lun,freq) }

      "ARB"         { if { $State($lun,freq) == $State($lun,ARB,freq) } \
                      {
                        # This is a query, no action required
                        #puts "Query only"
                        
                        return
                      }
                      
                      # Update the ARB frequency and rate variables
                      set State($lun,ARB,freq)  $State($lun,freq)
                      set State($lun,ARB,rate)  \
                                          [expr int($State($lun,freq) * 10.0)]

                      }

      default       {  }
    }

    return
  } 
 
 
  #****************************************************************************
  # Name:        DevPercent
  # Parameters:  lun              which FGEN is affected
  #              dev_percent      deviation percentage
  # Description: Set the deviation based the percentage 
  #****************************************************************************
  proc DevPercent { lun  dev_percent } \
  {
    after idle "FGen per$lun $dev_percent"
  }


  #****************************************************************************
  # Name:        updateStatus
  # Parameters:  lun      which FGEN is affected
  #              {status value, ref code }
  # Description: update the status code of a gen
  #****************************************************************************
  proc updateStatus { lun  status } \
  {
    variable State
   
    set code [lindex $status 0]
   
    set ref  [lindex $status 1]
      
    switch -- $code \
    {
       3 { # We've been XOFF'ed
           Debug::Puts FGEN 4 "updateStatus - Got XOFF"
           set State($lun,XOFF) 1
         }

       4 { # We've been XON'ed
           Debug::Puts FGEN 4 "updateStatus - Got XON"
           set State($lun,XOFF) 0
         }

       5 { # Got a trigger response
           Debug::Puts FGEN 4 "updateStatus - Got trigger $ref"
           doResponse $lun $ref
         }
       6 {
           ErrLog::Log "FGen error from DSP: $status" 0
           puts "FGen error from DSP: $status"
         }

       7 { # Command queue FULL
           Debug::Puts FGEN 4 "updateStatus - Got FULL"
           set State($lun,XOFF) -1
         }

       default { Debug::Puts FGEN 0 "FGen::updateStatus $lun $status" }
    }
  }


  #****************************************************************************
  # Name:        setResponse
  # Parameters:  lun      which FGEN is affected
  #              ref code for message
  # Description: Register a response code handler for an FGEN
  #****************************************************************************
  proc setResponse { lun  ref  cmd } \
  {
     variable Response
  
     set Response($lun,$ref) $cmd
  }
  
  
  #****************************************************************************
  # Name:        doResponse
  # Parameters:  lun      which FGEN is affected
  #              ref code for message
  # Description: Handle calling back any response code for a message
  #****************************************************************************
  proc doResponse { lun  ref } \
  {
    variable Response
  
    if [info exists Response($lun,$ref)] \
    {
      eval $Response($lun,$ref)
    }
  }
  
  #****************************************************************************
  # Name:        Send_core
  # Parameters:  lun, data to send
  # Description: send one tone of a sequence. IF this fgen has been XOFF'ed,
  #              wait for XON!
  #****************************************************************************
  proc Send_core { lun freq1 {freq2 ""} {shape ""} {dur ""} {level ""} {rel_level ""} {ref ""} } \
  {
    #puts "Send_core:  $lun,  $freq1,  $freq2,  $shape,  $dur,  $level,  $rel_level,  $ref"
    
    variable State
    variable cmd_map 
    
    if { $State($lun,abort) != 0 } \
    { 
      return 
    }
   
     # Wait for the DSP to be ready, if we've been XOFFed
    while { $State($lun,XOFF) != 0 } \
    {
       Debug::Puts FGEN 4 "Send_core: Xoff'ed"
       tkwait variable ::FGen::State($lun,XOFF)
    }
       
    set mode  $State($lun,fgen_mode)
    
    Debug::Puts FGEN 5 "Send_core: mode = $mode"


    if { $freq1 == "" } \
    {
      if { [info exists  State($lun,$mode,freq)] } \
      {
        set freq1  $State($lun,$mode,freq)
      } \
      else \
      {
        set freq1  0
      }
    }
    
    if { $freq2 == "" } \
    {
      set freq2  0
    }
    
    if { $shape == "" } \
    {
      if { [info exists  State($lun,$mode,shape)] } \
      {
        set shape  $State($lun,$mode,shape)
      } \
      else \
      {
        set shape  OFF
      }
    }
    
    #puts "shape:  $shape"
    
    # Convert named shapes to numbers...
    if { [info exists cmd_map($shape)] } \
    {
      set shape_val  $cmd_map($shape)
    }

    #puts "shape_val:  $shape_val"

    if { $dur == "" } \
    {
      # Duration of '0' creates a continuous tone
      set dur  0
    }

    if { $level == "" } \
    {
      if { [info exists  State($lun,$mode,level)] } \
      {
        set level  $State($lun,$mode,level)
      } \
      else \
      {
        set level  0
      }
    }
    
    if { $rel_level == "" } \
    {
      set burst_level  $level
    } \
    else \
    {
      # See if this is a relative level (dB specified)
      if { [regexp "(.*) dB" $rel_level temp db] } \
      {
        # Relative level specified. Compute needed level
        set burst_level  [expr $level * pow(10,$db/20)]
      }
    }

    if { $ref == "" } \
    {
      set ref  0
    }

    Debug::Puts FGEN 4 "Send_Core - handing to DSP: seq$lun $freq1 $freq2 $shape_val $dur $burst_level $ref"
    
    set retcode  [FGen seq$lun $freq1 $freq2 $shape_val $dur $burst_level $ref -forgive]
    
    Debug::Puts FGEN 4 "Send_Core - retcode = $retcode"
    
    # If we got an XOFF from the driver, set the XOFF flag...
    if { $retcode == 0 } \
    {
      Debug::Puts FGEN 4 "Send_Core - Got XOFF"
      set State($lun,XOFF)  1
    } \
    elseif { $retcode == -1 } \
    {
      Debug::Puts FGEN 4 "Send_Core - Got XOFF FULL"
      set State($lun,XOFF)  -1
    } \
    elseif { $retcode != 1 } \
    {
      Debug::Puts FGEN 0 "Got return code $retcode from driver "
    }

    return
  }
 
 
  #****************************************************************************
  # Name:        Send
  # Parameters:  lun, sequence to send
  # Description: Send a sequence of tones. This is a list of lists (of lists...)
  #              of tone descriptions, where a tone description is
  #              {freq1 freq2 shape duration level}
  #              any of these may be null (""), in which case the last value
  #              is used
  #****************************************************************************
  proc Send { lun  args } \
  {   
    variable State
    
    if { $State($lun,abort) != 0 } \
    {
      return
    }
    Debug::Puts FGEN 4 "Send:  $lun,  $args"
  
    after idle "FGen::Send_after  $lun  $args"
  }
  
  
  proc Send_after { lun  args } \
  {
    Debug::Puts FGEN 4 "Send_after:  $lun,  $args"

    variable State
    
    if { $State($lun,abort) != 0 } \
    {
       Debug::Puts FGEN 4 "Send_after - abort"
       return
    }
      
    lappend State($lun,pending) [list $lun $args]
    
    # Check for another send running
    if { $State($lun,sending) != 0 } \
    {
      # retry later
      Debug::Puts FGEN 4 "Send_after - later"
      return
    }
    
    set State($lun,sending) 1
    catch \
    {
     Debug::Puts FGEN 4 "Send_after - sending"
         
     while {[llength $State($lun,pending)] != 0} \
     {
       set x [lindex $State($lun,pending) 0]
       set State($lun,pending) [lrange $State($lun,pending) 1 end]
       foreach {_lun _args} $x \
       {
          Debug::Puts FGEN 4 "Send_after - enacting $_lun $_args"
          Parse_list_of_lists "FGen::Send_core $_lun" $_args
       }
     }
       
     Debug::Puts FGEN 4 "Send_after - Done..."
    }
    # we are done sending
    set State($lun,sending) 0
 
    # clear any pending abort
    if {$State($lun,abort) != 0 }\
    {
       Debug::Puts FGEN 4 "Send_after - clearing abort"
       set State($lun,abort) 0
       set State($lun,pending) {}
    }
  }

 
  #****************************************************************************
  # Name:        Event
  # Parameters:  Reference code number
  # Description: Create an event record with a specified code number
  #****************************************************************************
  proc Event { {code 0} } \
  {
    return [list 0 0 EVENT 0 {} {} $code]
  }
  
  
  #****************************************************************************
  # Name:        Abort
  # Parameters:  none
  # Description: Abort all sends on a LUN
  #****************************************************************************
  proc Abort { lun } \
  {
    variable State
  
    Debug::Puts FGEN 3 "Abort"

    # terminate all pending sends at our level
    set State($lun,abort)  1
     
    # send the abort message down, no matter what
    Fgen seq$lun 0 0 -1 1 0 0 -nowait
     
    set State($lun,XOFF) 0
    
    if {$State($lun,sending) == 0} /
    {
      Debug::Puts FGEN 3 "Abort - clearing"
      # no send in progress - clear abort
      set State($lun,abort)  0
      set State($lun,pending) {}
    }
  }
  
  
  #****************************************************************************
  # Name:        Off
  # Parameters:  none
  # Description: Create an OFF record
  #****************************************************************************
  proc Off {  } \
  {
    return [list 0 0 OFF {}]
  }
 
  #***************************************************************************
  #
  # Name:        getMode
  #
  # Parameters:  lun  - Logical Unit Number of the Function Generator
  #
  # Description: Gets the mode of the Function Generators
  #
  #***************************************************************************
  proc getMode { lun } \
  {
    #puts "getMode:  $lun"
    
    variable  State
    
    return  $State($lun,fgen_mode)
  }

}

if ![interp issafe] \
{
    source [file join $::FGen::path master.tcl]
} \
else \
{
   FGen::set_fgen_callbacks
}

