namespace eval ::FGen \
{
  #***************************************************************************
  #
  # The following procedures are used to create a Tcl based engine
  # in order to bypass complications with using the existing 'C' based 
  # engine with the different modes of operation
  #
  #***************************************************************************
  #***************************************************************************
  #
  # Name:        setMode
  #
  # Parameters:  lun  - Logical Unit Number of the Function Generator
  #              mode - function generator mode
  #
  # Description: Sets the mode of the Function Generators
  #
  #***************************************************************************
  proc setMode { lun  mode } \
  {
    #puts "setMode:  $lun,  $mode"

    variable  engine_mode_map
    variable  State
    
    # Stop any existing sequence
    FGen::setResponse  $lun  1  {}

    # Send an 'OFF' sequence to clear the Function Generator
    FGen::Send  $lun  0  0  SINE  0
    
    #puts "  mode:  $mode"
    
    switch -- $mode \
    {
      "9"     -
      "SINC"   { 
                 FGen mode${lun} $mode -nowait  
                 FGen shape${lun} ARBITRARY -nowait
                 ::FGen::send_sinc_seq $lun
               }

      "11"     -
      "SWEEP"  { set State($lun,fgen_mode)        "SWEEP"
                 set State($lun,fgen_mode_value)  11 }

      default { FGen mode${lun} $mode -nowait }
    }
  }

  #***************************************************************************
  #
  # Name:        initMode
  #
  # Parameters:  lun  - Logical Unit Number of the Function Generator
  #              mode - function generator mode
  #
  # Description: Sets the mode of the Function Generators
  #
  #***************************************************************************
  proc initMode { lun  mode } \
  {
     puts "initMode:  $lun,  $mode"

     variable  engine_mode_map
     variable  State
    
     switch -- $mode \
     {
       "0"      -
       "OFF"    { set State($lun,fgen_mode)        "OFF"
                  set State($lun,fgen_mode_value)  0 }
      
      "1"      -
      "TONE"   { set State($lun,fgen_mode)        "TONE"
                 set State($lun,fgen_mode_value)  1
                 # Send out the TONE sequence
                 FGenUI::tone_send_seq  $lun }
      
      "3"      -
      "TREM"   { set State($lun,fgen_mode)        "TREM"
                 set State($lun,fgen_mode_value)  3
                 # Update the Tone Remote code
                 ToneRemoteUI::trem_set_code  $lun }
      
      "4"      -
      "DTMF"   { set State($lun,fgen_mode)        "DTMF"
                 set State($lun,fgen_mode_value)  4 
               }
      
      "5"      -
      "CTCSS"  { set State($lun,fgen_mode)        "CTCSS"
                 set State($lun,fgen_mode_value)  5
                 # Send out the CTCSS sequence
                 CTCSS_UI::ctcss_send_seq  $lun }
      
      "6"      -
      "DCS"    { set State($lun,fgen_mode)        "DCS"
                 set State($lun,fgen_mode_value)  6
                 # Validate the code
                 DCS_UI::dcs_validate  $lun }

      "7"      -
      "TSIG"   { set State($lun,fgen_mode)        "TSIG"
                 set State($lun,fgen_mode_value)  7
                 # Load the Tone Signaling packages
                 foreach { format } $::Tone_Signaling::formats \
                 {
                   package require $format
                 }
               }

      "8"      -
      "P25"    { set State($lun,fgen_mode)        "P25"
                 set State($lun,fgen_mode_value)  8
                 # Send out the P25 sequence
                 GeneratorUI::p25_send_seq  $lun }

      "9"      -
      "SINC"   { set State($lun,fgen_mode)        "SINC"
                 set State($lun,fgen_mode_value)  9
                 # Send out the SINC sequence
                 SincUI::setup_sinc_waveform  $lun }

      "10"     -
      "ARB"    { set State($lun,fgen_mode)        "ARB"
                 set State($lun,fgen_mode_value)  10
                 # Send out the ARB sequence
                 ArbitraryUI::setup_arb_waveform  $lun }

      default { }
    }
  }


  #***************************************************************************
  #
  # Name:        getMode
  #
  # Parameters:  lun  - Logical Unit Number of the Function Generator
  #
  # Description: Gets the mode of the Function Generators
  #
  #***************************************************************************
  proc getMode { lun } \
  {
    #puts "getMode:  $lun"
    
    variable  State
    
    return  $State($lun,fgen_mode)
  }


  #***************************************************************************
  #
  # Name:        setFreq
  #
  # Parameters:  lun  - Logical Unit Number of the Function Generator
  #              freq - new frequency value
  #
  # Description: Sets the frequency of the Function Generators 
  #              in the OFF, TONE, CTCSS, P25, SINC, and ARB modes
  #
  #***************************************************************************
  proc setFreq { lun  freq } \
  {
    #puts "setFreq:  $lun,  $freq"
    
    variable  State
    
    # Format the frequency value to one decimal place
    set freq  [format "%.1f"  $freq]
    
    #puts "  freq:  $freq"
    
    #puts "  State($lun,fgen_mode):  $State($lun,fgen_mode)"
    
    switch -- $State($lun,fgen_mode) \
    {
      "OFF"    { # Synchronize the Function Generator engine frequency
                 FGen freq${lun}  $freq  -nowait }
      
      "TONE"   { # Synchronize the Function Generator engine frequency
                 FGen freq${lun}  $freq  -nowait 
                 
                 # Update the TONE frequency variable
                 set State($lun,TONE,freq)  $freq 
                 
                 # Send out the TONE sequence
                 FGenUI::tone_send_seq  $lun }
      
      "TREM"   { }
      
      "DTMF"   { }
      
      "CTCSS"  { # Synchronize the Function Generator engine frequency
                 FGen freq${lun}  $freq  -nowait 
                 
                 # Update the CTCSS frequency variable
                 set State($lun,CTCSS,freq)  $freq 
                 
                 # Update the CTCSS code
                 set freq  [format "%.1f"  $freq]
                 
                 # Find the frequency value in the code list
                 foreach { temp_code } $::CTCSS::codes_and_freqs \
                 {
                   set str_length  [string length  $temp_code]
                                
                   set temp_freq  [string range  $temp_code  5  $str_length]
                   
                   if { $temp_freq == "$freq" } \
                   {
                     set State($lun,CTCSS,code)  [string range $temp_code  0  1]

                     # Send out the CTCSS sequence if the CTCSS freq exists
                     CTCSS_UI::ctcss_send_seq  $lun
                   } \
                   else \
                   {
                     # Get the first item in the list
                     set State($lun,CTCSS,code)  \
                       [string range  [lindex $::CTCSS::codes_and_freqs  0] 0 1]
                   }
                 }
               } 
      
      "DCS"    { }
      
      "TSIG"   { }

      "P25"    { # Synchronize the Function Generator engine frequency
                 FGen freq${lun}  $freq  -nowait 
                 
                 # Update the P25 frequency variable
                 set State($lun,P25,freq)  $freq

                 # Send out the P25 sequence
                 GeneratorUI::p25_send_seq  $lun } 
      
      "SINC"   { # Synchronize the Function Generator engine frequency
                 FGen freq${lun}  $freq  -nowait 
                 
                 # Update the SINC frequency and rate variables
                 set State($lun,SINC,freq)  $freq
                 set State($lun,SINC,rate)  [expr int($freq * 10.0)] 
                 
                 # Send out the SINC sequence
                 send_sinc_seq  $lun }

      "ARB"    { # Synchronize the Function Generator engine frequency
                 FGen freq${lun}  $freq  -nowait 
                 
                 # Update the ARB frequency and rate variables
                 set State($lun,ARB,freq)  $freq
                 set State($lun,ARB,rate)  [expr int($freq * 10.0)]
                 
                 # Send out the ARB sequence
                 ArbitraryUI::send_arb_seq  $lun }                 
    }
  
    return
  }


  #***************************************************************************
  #
  # Name:        getFreq
  #
  # Parameters:  lun  - Logical Unit Number of the Function Generator
  #
  # Description: Gets the frequency of the Function Generators 
  #              in the OFF, TONE, CTCSS, and P25 modes
  #
  #***************************************************************************
  proc getFreq { lun } \
  {
    #puts "getFreq:  $lun"
    
    variable  State
    
    switch -- $State($lun,fgen_mode) \
    {
      "OFF"    { return  $::FGen::State($lun,OFF,freq) }
      
      "TONE"   { return  $::FGen::State($lun,TONE,freq) }
      
      "TREM"   { }
      
      "DTMF"   { }
      
      "CTCSS"  { return  $::FGen::State($lun,CTCSS,freq) } 
      
      "DCS"    { }
      
      "TSIG"   { }

      "P25"    { return  $::FGen::State($lun,P25,freq) } 
      
      "SINC"   { return  $::FGen::State($lun,SINC,freq) }

      "ARB"    { return  $::FGen::State($lun,ARB,freq) }
    }
  
    return
  }


  #***************************************************************************
  #
  # Name:        setShape
  #
  # Parameters:  lun   - Logical Unit Number of the Function Generator
  #              shape - new shape value
  #
  # Description: Sets the shape of the waveform of the Function Generators 
  #              in OFF, TONE, CTCSS, and P25 modes
  #
  #***************************************************************************
  proc setShape { lun  shape } \
  {
    #puts "setShape:  $lun,  $shape"

    variable  State
    
    switch -- $State($lun,fgen_mode) \
    {
      "OFF"    { # Synchronize the Function Generator engine shape
                 FGen shape${lun}  $shape  -nowait  }
      
      "TONE"   { # Synchronize the Function Generator engine shape
                 FGen shape${lun}  $shape  -nowait 
                 
                 # Update the TONE shape variable
                 set State($lun,TONE,shape)  $shape 
                 
                 # Send out the TONE sequence
                 FGenUI::tone_send_seq  $lun }
      
      "TREM"   { }
      
      "DTMF"   { }
      
      "CTCSS"  { }
      
      "DCS"    { }
      
      "TSIG"   { }
      
      "P25"    { # Synchronize the Function Generator engine shape
                 FGen shape${lun}  $shape  -nowait 
                 
                 # Update the P25 frequency variable
                 set State($lun,P25,shape)  $shape 
                 
                 # Send out the P25 sequence
                 GeneratorUI::p25_send_seq  $lun } 

      "SINC"   { FGen shape${lun}  ARBITRARY  -nowait }

      "ARB"    { FGen shape${lun}  ARBITRARY  -nowait }
    }
  
    return
  }


  #***************************************************************************
  #
  # Name:        getShape
  #
  # Parameters:  lun   - Logical Unit Number of the Function Generator
  #
  # Description: Gets the shape of the waveform of the Function Generators 
  #              in OFF, TONE, CTCSS, and P25 modes
  #
  #***************************************************************************
  proc getShape { lun } \
  {
    #puts "getShape:  $lun"

    variable  State
    
    switch -- $State($lun,fgen_mode) \
    {
      "OFF"    { return  $::FGen::State($lun,OFF,shape) }
      
      "TONE"   { return  $::FGen::State($lun,TONE,shape) }
      
      "TREM"   { }
      
      "DTMF"   { }
      
      "CTCSS"  { }
      
      "DCS"    { }
      
      "TSIG"   { }
      
      "P25"    { return  $::FGen::State($lun,P25,shape) } 

      "SINC"   { return  "ARB" }

      "ARB"    { return  "ARB" }
    }
  
    return
  }


  #***************************************************************************
  #
  # Name:        setLevel
  #
  # Parameters:  lun   - Logical Unit Number of the Function Generator
  #              level - new level value
  #
  # Description: Sets the output level of the Function Generators
  #
  #***************************************************************************
  proc setLevel { lun  level } \
  {
    #puts "setLevel:  $lun,  $level"
    
    variable  State
    
    if { $lun == 1 || $lun == 2 } \
    {
      set mode  $State($lun,fgen_mode)
      
      #puts "  mode:  $mode"
      
      # Update the 'mode' level variable
      set State($lun,$mode,level)  [format "%.3f" $level]
      
      #puts "  State($lun,$mode,level):  $State($lun,$mode,level)

      # Update the 'mode' deviation variable
      set State($lun,$mode,dev)  [format "%.2f" [expr $level * 5.0]]

      # Update the 'mode' percent deviation variable
      set State($lun,$mode,percent)  [format "%.2f" [expr $level * 10.0]]

      switch -- $State($lun,fgen_mode) \
      {
        "OFF"    {  }
        
        "TONE"   { # Send out the TONE sequence
                   FGenUI::tone_send_seq  $lun  }
        
        "TREM"   {  }
        
        "DTMF"   {  }
        
        "CTCSS"  { # Send out the CTCSS sequence
                   CTCSS_UI::ctcss_send_seq  $lun } 
        
        "DCS"    { # Send out the DCS sequence if it is ON
                   if { $State($lun,DCS,state) == "ON" } \
                   {
                     DCS_UI::dcs_start  $lun
                   }
                 }
        
        "TSIG"   {  }
        
        "P25"    { # Send out the P25 sequence
                   GeneratorUI::p25_send_seq  $lun   }

        "SINC"   { # Send out the SINC sequence
                   send_sinc_seq  $lun }

        "ARB"    { # Send out the ARB sequence
                   ArbitraryUI::send_arb_seq  $lun }
                   
        "SWEEP"  {  }
      }
    

      # Synchronize the Function Generator engine level for RCI queries
      FGen level${lun}  $level  -nowait 
    }
  
    return
  }


  #***************************************************************************
  #
  # Name:        getLevel
  #
  # Parameters:  lun   - Logical Unit Number of the Function Generator
  #
  # Description: Gets the output level of the Function Generators
  #
  #***************************************************************************
  proc getLevel { lun } \
  {
    #puts "getLevel:  $lun"
    
    variable  State
    
    set mode  $State($lun,fgen_mode)
      
    return  $::FGen::State($lun,$mode,level)
  }


  #***************************************************************************
  #
  # Name:        setDeviation
  #
  # Parameters:  lun       - Logical Unit Number of the Function Generator
  #              deviation - new deviation value
  #
  # Description: Sets the deviation of the Function Generators
  #
  #***************************************************************************
  proc setDeviation { lun  deviation } \
  {
    #puts "setDeviation:  $lun,  $deviation"
    
    variable  State
    
    if { $lun == 3 || $lun == 4 } \
    {
      set mode  $State($lun,fgen_mode)
      
      #puts "  mode:  $mode"
      
      #puts "  State(dev_units):  $State(dev_units)"
      
      # Handle the deviation
      if { $State(dev_units) == "kHz" } \
      {
        # Update the 'mode' level variable
        set State($lun,$mode,level)  [format "%.3f" [expr $deviation / 5.0]]
      
        # Update the 'mode' percent deviation variable
        set State($lun,$mode,percent)  [format "%.2f" [expr $deviation * 2.0]]

        # Update the 'mode' deviation variable
        set State($lun,$mode,dev)  [format "%.2f" $deviation]
        
        #puts "  State($lun,$mode,dev):  $State($lun,$mode,dev)"
        
        switch -- $State($lun,fgen_mode) \
        {
          "OFF"    {  }
        
          "TONE"   { # Send out the TONE sequence
                     FGenUI::tone_send_seq  $lun  }
                                    
          "TREM"   {  }
        
          "DTMF"   {  }
        
          "CTCSS"  { # Send out the CTCSS sequence
                     CTCSS_UI::ctcss_send_seq  $lun } 
        
          "DCS"    { # Send out the DCS sequence if it is ON
                     if { $State($lun,DCS,state) == "ON" } \
                     {
                       DCS_UI::dcs_start  $lun
                     }
                   }
        
          "TSIG"   {  }
        
          "P25"    { # Send out the P25 sequence
                     GeneratorUI::p25_send_seq  $lun   }

          "SINC"   { # Send out the SINC sequence
                     send_sinc_seq  $lun }

          "ARB"    { # Send out the ARB sequence
                     ArbitraryUI::send_arb_seq  $lun }
                     
          "SWEEP"  {  }
        }

        # Synchronize the Function Generator engine level for RCI queries
        FGen deviation${lun}  $deviation  -nowait
      }
      
    }
     
    return
  }


  #***************************************************************************
  #
  # Name:        getDeviation
  #
  # Parameters:  lun   - Logical Unit Number of the Function Generator
  #
  # Description: Gets the output deviation of the Function Generators
  #
  #***************************************************************************
  proc getDeviation { lun } \
  {
    #puts "getDeviation:  $lun"
    
    variable  State
    
    set mode  $State($lun,fgen_mode)
      
    return  $::FGen::State($lun,$mode,dev)
  }


  #***************************************************************************
  #
  # Name:        setPercent
  #
  # Parameters:  lun     - Logical Unit Number of the Function Generator
  #              percent - new percent value
  #
  # Description: Sets the percent deviation of the Function Generators
  #
  #***************************************************************************
  proc setPercent { lun  percent } \
  {
    #puts "setPercent:  $lun,  $percent"
    
    variable  State
    
    if { $lun == 3 || $lun == 4 } \
    {
      set mode  $State($lun,fgen_mode)
      
      # Handle the percent deviation
      if { $State(dev_units) == "%" } \
      {
        # Update the 'mode' level variable
        set State($lun,$mode,level)  [format "%.3f" [expr $percent / 10.0]]

        # Update the 'mode' percent variable
        set State($lun,$mode,percent)  [format "%.2f" $percent]

        # Update the 'mode' deviation variable
        set State($lun,$mode,dev)  [format "%.2f" [expr $percent / 2.0]]

        switch -- $State($lun,fgen_mode) \
        {
          "OFF"    {  }
        
          "TONE"   { # Send out the TONE sequence
                     FGenUI::tone_send_seq  $lun  }
                                    
          "TREM"   {  }
        
          "DTMF"   {  }
        
          "CTCSS"  { # Send out the CTCSS sequence
                     CTCSS_UI::ctcss_send_seq  $lun } 
        
          "DCS"    { # Send out the DCS sequence if it is ON
                     if { $State($lun,DCS,state) == "ON" } \
                     {
                       DCS_UI::dcs_start  $lun
                     }
                   }
        
          "TSIG"   {  }
          
          "P25"    { # Send out the P25 sequence
                     GeneratorUI::p25_send_seq  $lun }

          "SINC"   { # Send out the SINC sequence
                     send_sinc_seq  $lun }

          "ARB"    { # Send out the ARB sequence
                     ArbitraryUI::send_arb_seq  $lun }
                     
          "SWEEP"  {  }
        }

        # Synchronize the Function Generator engine level for RCI queries
        FGen percent${lun}  $percent  -nowait 
      }
      
    }
     
    return
  }

  #**************************************************************************
  #  Name:         send_sinc_seq
  #  Parameters:   lun - Logical Unit Number for the function generator
  #  Description:  Set up a specific function generator with the parameters
  #      needed to generate the SINC waveform. 
  #  Notes:
  #    o  Do NOT set up the FGen mode in this routine. Only set the data
  #    o  freq1 = FGen::State($lun,SINC,freq)
  #    o  freq2 = 0
  #    o  shape = ARBITRARY
  #    o  duration = 0
  #    o  level = FGen::State($lun,SINC,level)
  #**************************************************************************
  proc send_sinc_seq { lun } \
  {
    #puts "send_sinc_seq:  $lun"
    after idle "FGen::Send  $lun  $::FGen::State($lun,SINC,freq)  \
                                  0                               \
                                  ARBITRARY                       \
                                  0                               \
                                  $::FGen::State($lun,SINC,level)"
  }


  #***************************************************************************
  #
  # Name:        getPercent
  #
  # Parameters:  lun   - Logical Unit Number of the Function Generator
  #
  # Description: Gets the output percent deviation of the Function Generators
  #
  #***************************************************************************
  proc getPercent { lun } \
  {
    #puts "getPercent:  $lun"
    
    variable  State
    
    set mode  $State($lun,fgen_mode)
      
    return  $::FGen::State($lun,$mode,percent)
  }


  #****************************************************************************
  # Name:        getSaveState
  # Parameters:  none
  # Description: return state of FGen system
  #****************************************************************************
  proc getSaveState { } \
  {
    variable State
 
    set returnList {}

    append returnList  "set ::FGen::State(level_units,1) $State(level_units,1)\n"
    append returnList  "set ::FGen::State(level_units,2) $State(level_units,2)\n"

    # Save the array
    foreach { lun } { 1 2 3 4 } \
    {
      foreach { mode } { TONE TREM DTMF CTCSS DCS TSIG P25 SINC ARB } \
      {
        foreach { item } [array names State $lun,$mode,*] \
        {
          append returnList "set ::FGen::State($item) {$State($item)} \n"
        }
      }
    }
    
    append returnList  "FGen::setMode  1  $State(1,fgen_mode_value)  \n"
    append returnList  "FGen::setMode  2  $State(2,fgen_mode_value)  \n"
    append returnList  "FGen::setMode  3  $State(3,fgen_mode_value)  \n"
    append returnList  "FGen::setMode  4  $State(4,fgen_mode_value)  \n"

    return [subst -nocommands $returnList]
  }

#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to hardware
#****************************************************************************
 proc Sync { } {
     setMode  1  0
     setMode  2  0
     setMode  3  0
     setMode  4  0
 }
 set_fgen_callbacks
}

#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
Load::on_system_done {::FGen::Sync}

#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines FGen
}


#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
     FGen_Eng "Function Generator Engine"
     FGen_Drv "Function Generator Driver"
   }
   set version(FGen_Eng)  [Fgen ver 0]
   set version(FGen_Drv)  [Fgen ver 1]
}

