#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : generator.tcl                                               #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the Generator command is   #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::Generator::State ruw "::Generator::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
# Call the base class dummy command to force a load
if {"[info command Generator]" != "" } { return }
package require Engtclbase
package require IFR_options
package require Units

Engtclbase

# Load the object file
::load $::Generator::object

package provide Generator 1.0

#****************************************************************************
# Generator namespace variables
#****************************************************************************
#    Status of commands sent to Generator
#       ::Generator::State(ptt)         Mhz
#****************************************************************************
namespace eval ::Generator {

   # This data is updated by the engines through callback routines, and it
   # indicates (in a screen format) the last setting received from an engine

   array set bw_map \
   {
      0 "NARROW"
      1 "WIDE"
   }

   array set p25mode_map \
   {
      0  "OFF"
      1  "VOICE"
      2  "SILENCE"
      3  "CALIBRATION"
      4  "1011"
      5  "SPEECH"
      6  "STD 1011"
      7  "STD SILENCE"
      8  "STD INTFRNC"
      9  "STD BUSY"
      10 "STD IDLE"
      11 "STD CAL"
      12 "STD LDU1TRG"
      13 "STD NOTRIG"
      14 "STD LDU2TRG"
      15 "STD 511"
      16 "STD SYMRATE"
      17 "STD LOWDEV"
      18 "STD FIDPAT"
      19 "STD FIDSPECT"
      20 "TRUNKING"
   }

   array set modtype_map \
   {
      0 "OFF"
      1 "FM"
      4 "AM"
      8 "P25"
      11 "P25 LSM"
      12 "P25 P2"
      13 "LTD"
      14 "P25 WP"
   }
   
   set State(P25_modes) {8 11 12 13 14}
   
#****************************************************************************
# Name:        set_generator_callbacks
# Parameters:  none
# Description: Sets up the callbacks to update the FGEN data
#****************************************************************************
 proc set_generator_callbacks { } {

   Generator -onchange ptt       { ::Generator::updatePtt            %V }
   Generator -onchange freq      { ::Generator::update_freq          %V }
   Generator -onchange rfoutput  { ::Generator::updateRFoutput       %V }
   Generator -onchange rflevel   { ::Generator::format_gen_level     %V }
   Generator -onchange hwbw      { ::Generator::updateBandwidth      %V }
   Generator -onchange dcpower   { ::Generator::updateDCpower        %V }
   Generator -onchange p25mode   { ::Generator::updateP25mode        %V }
   Generator -onchange mod       { ::Generator::updateModtype        %V }
   Generator -onchange lockstat  { ::Generator::updateLockStatus     %V }
   Generator -onchange yigstat   { ::Generator::updateYigStat        %V }
   Generator -onchange portmax   { ::Generator::updatePortLimitsMax  %V }
   Generator -onchange portmin   { ::Generator::updatePortLimitsMin  %V }
   Generator -onchange yigspeed  { ::Generator::updateYigSpeed       %V }

   Generator -onchange enable    { set ::Generator::State(enable)  %V }
   Generator -onchange hwalc     { set ::Generator::State(alc)     %V }
 }


#****************************************************************************
# Name:        get_valid_mod_names
# Parameters:  none
# Description: Return a list of valid modulation names
#****************************************************************************
 proc get_valid_mod_names {  } \
 {
   set rv { "OFF" "FM" "AM"  "P25"}
   if [IFR_options is_installed P25_LSM] \
   {
      lappend rv "P25 LSM" "P25 P2"
   }
   if [IFR_options is_installed LTD] \
   {
       lappend rv "LTD"
   }
   
   if [IFR_options is_installed P25_WIDEPULSE] \
   {
       lappend rv "P25 WP"
   }
   return $rv
 }
 
 
#****************************************************************************
# Name:        format_gen_level
# Parameters:  lev - level in dBm
# Description: create sensible number
#****************************************************************************
 proc format_gen_level { lev } \
 {
    #puts "format_gen_level:  $lev"
    
    variable State
 
    set State(rflev,raw)  $lev
    
    set State(rflev,dBm)  [format %.1f  $State(rflev,raw)]
    
    set State(rflev,mV)   [format %.4f  [expr 1000.0 * [convert_dBm_V  $State(rflev,raw)]]]
    
    set State(rflev,uV)   [format %.2f  [expr 1000000.0 * [convert_dBm_V  $State(rflev,raw)]]]
    
    return    

 }   
      
      
#****************************************************************************
# Name:        update_freq
# Parameters:  v - freq in MHz
# Description: 
#****************************************************************************
 proc update_freq { v } \
 {
   variable State
   set State(freq) [format %4.6f $v] 
   set State(freq,raw) $v
 }
 
 
#****************************************************************************
# Name:        updateDCpower
# Parameters:  state  0 = OFF, 1 = ON
# Description: Updates the DC power status
#****************************************************************************
 proc updateDCpower { state } {
   variable State
   switch $state {
      0       { set State(dcpower) OFF }
      1       { 
                 if {$State(dcpower) != "ON"} \
                 {
                    set State(dcpower) ON
                 }
              }
      default { return }
   }
 }
 
 
#****************************************************************************
# Name:        updateYigSpeed
# Parameters:  speed      Data from engine
# Description: Sets up the current state of the Yig Status
#****************************************************************************
 proc updateYigSpeed { speed } \
 {
     variable State
     set State(yigspeed) $speed
     switch -- $speed \
     {
        "0" { set State(yigspeed,text) "FAST"}
        "1" { set State(yigspeed,text) "SLOW"}
     }
 }


#****************************************************************************
# Name:        updateYigStat
# Parameters:  readingList      Data from engine
# Description: Sets up the current state of the Yig Status
#****************************************************************************
 proc updateYigStat { readingList } {
   variable State
   
   # Get the data from the meter reading
   set State(hold) [lindex $readingList 0]
   set State(dac)  [lindex $readingList 1]
   set State(alcstat)  [lindex $readingList 2]
 }


#****************************************************************************
# Name:        updateModtype
# Parameters:  state  0 = OFF, 1 = FM, 8 = P25
# Description: Updates the modulation status
#****************************************************************************
 proc updateModtype { state } {
    variable State
    variable modtype_map
    if {[lsearch $State(P25_modes) $state] != -1} \
    {
       set State(is_P25_mode) 1
    } \
    else \
    {
       set State(is_P25_mode) 0
    }
    set State(mod_raw) $state
    if [info exists modtype_map($state)] {
       set State(mod) $modtype_map($state)
    }
 }


#****************************************************************************
# Name:        updatePtt
# Parameters:  state  0 = OFF, 1 = ON
# Description: Updates the PTT status
#****************************************************************************
 proc updatePtt { state } {
   variable State
   set State(ptt,raw) $state
   switch $state \
   {
      0       { set State(ptt) OFF }
      1       { set State(ptt) ON }
      2       { set State(ptt) PTT }
      default { return }
   }
 }


#****************************************************************************
# Name:        updateBandwidth
# Parameters:  state  0 = NARROW, 1 = WIDE
# Description: Updates the generator variables from the engine transmission
#****************************************************************************
 proc updateBandwidth { state } {
    set ::Generator::State(hwbw_raw) $state
    if [info exists ::Generator::bw_map($state)] {
       set ::Generator::State(hwbw) $::Generator::bw_map($state)
    }
 }


#****************************************************************************
# Name:        updateP25mode
# Parameters:  state  0 = OFF, 1 = VOICE, 5 = SPEECH, 6 = 1011, 7 = SILENCE,
#                     11 = CALIBRATION
# Description: Updates P25 mode status 
#****************************************************************************
 proc updateP25mode { state } {
    set ::Generator::State(p25mode_raw) $state
    if [info exists ::Generator::p25mode_map($state)] {
       set ::Generator::State(p25mode) $::Generator::p25mode_map($state)
    }
 }


#****************************************************************************
# Name:        updateRFoutput
# Parameters:  state  0 = T/R, 1 = GEN 
# Description: Updates the RF output status
#****************************************************************************
 proc updateRFoutput { state } {
   switch $state {
      0       { set ::Generator::State(rfout) T/R }
      1       { set ::Generator::State(rfout) GEN }
      default { return }
   }
 }

 
#****************************************************************************
# Name:        updatePortLimitsMax
# Parameters:  db - max level in dBm
# Description: Updates the modulation status
#****************************************************************************
 proc updatePortLimitsMax { dBm } {
   set dBm [format "%3.1f" $dBm]
   
   variable State
   
   set State(portmax) $dBm
 }


#****************************************************************************
# Name:        updatePortLimitsMin
# Parameters:  db - max level in dBm
# Description: Updates the modulation status
#****************************************************************************
 proc updatePortLimitsMin { dBm } {
   set dBm [format "%3.1f" $dBm]

   variable State
   
   set State(portmin) $dBm
 }


#****************************************************************************
# Name:        updateLockStatus
# Parameters:  lockstat - engine lock status
# Description: Updates the lock status
#****************************************************************************
 proc is_locked { stat mask } \
 {
    if {$stat & $mask } \
    {
       return "UNLOCKED"
    }
    return "LOCKED"
 }
 
 proc updateLockStatus { lockstat } \
 {
   variable State
   
   set State(lockstat,raw) $lockstat
   
   # if the generator is OFF, we pretend it is locked...
   if {$State(dcpower) != "ON" } { set lockstat 0}
   
   set State(lock,main)     [is_locked $lockstat 1]
   set State(lock,clean)    [is_locked $lockstat 2]
   set State(lock,step)     [is_locked $lockstat 4]
   set State(lock,ref)      [is_locked $lockstat 8]

   set State(lock_unstable) [expr $lockstat & 0x8000]
   set State(lockstat)      [expr $lockstat & 0x7FFF]
   

 }


#***************************************************************************
# Name:        toggleRFoutput
# Parameters:  none
# Description: Switch the input for the Receiver
#***************************************************************************
 proc toggleRFoutput { } \
 {
    if { $::Generator::State(rfout) == "T/R" } \
    {
        Generator rfoutput "GEN"
    } else \
    {
        Generator rfoutput "T/R"
    }
 }


#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the RL power engine to its current state
#****************************************************************************
 proc getSaveState { } \
 {
   variable State
   
   set returnList \
   {
      Generator dcpower   $State(dcpower)     -nowait
      Generator enable    $State(enable)      -nowait
      Generator freq      $State(freq)        -nowait
      Generator rfoutput  $State(rfout)       -nowait
      Generator rflevel   $State(rflev,raw)   -nowait
      Generator hwalc     $State(alc)         -nowait
      Generator hwbw      $State(hwbw)        -nowait
      Generator mod       $State(mod_raw)     -nowait
      Generator yigspeed  $State(yigspeed)    -nowait
      Generator ptt       $State(ptt)         -nowait
   }
   
   if {$State(p25mode_raw) == 20}\
   {
      # trunking mode? then save as 1011
      append returnList \
      {
        Generator p25mode  4 -nowait
      }
   } \
   else \
   {
      append returnList \
      {
        Generator p25mode  $State(p25mode_raw) -nowait
      }
   }
   
   return [subst -nocommands $returnList ]
 }


#****************************************************************************
# Name:        getAlertProcedure
# Parameters:  none
# Description: This procedure returns the variable 'State(lockstat)' and the 
#            : procedure to call to display the alert message box.
#****************************************************************************
 proc getAlertProcedure { } \
 {
   #puts "getAlertProcedure"

   variable State
   
   set returnList { ::Generator::State(show_unlock)  ::Generator::displayAlert }

   return  $returnList
 }


#****************************************************************************
# Name:        SyncAll
# Parameters:  none
# Description: Insures that TCL, the engine and hardware are in sync
#****************************************************************************
 proc Sync { } { eval [getSaveState]}
 


#****************************************************************************
# Name:        Trunking_mode
# Parameters:  command to turn OFF new mode
# Description: Turn off any previous trunking mode, and register a new turn-off
#              command
#****************************************************************************
 proc Trunking_mode { {new_off {}}} \
 {
    variable last_trunking_off_cmd
    
    if {$last_trunking_off_cmd != ""} \
    {
       catch { eval $last_trunking_off_cmd }
    }
    set last_trunking_off_cmd $new_off
 }
 

 set_generator_callbacks
 
 if ![interp issafe] \
 {
    source [file join $::Generator::path master.tcl]
 }
 
}  ;# End of the Generator namespace

#****************************************************************************
# Define the Remote_cal namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::Remote_cal \
{
  variable InfoString

  set P25_version [string trimleft [P25 encode ver 0] "P25Mtr"]
  set InfoString [subst \
  {
         <br>
         This option permits external test software to remotely control and calibrate the 2975
  }]
}
