#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2002      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : hw_control.tcl                                              #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : This file is sourced by TCL when the HW_control command is  #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
foreach var $::HW_control::trace_list {
   trace vdelete ::HW_control::$var ruw "::HW_control::Load"
}

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
if {"[info command HW_control]" != "" } { return }

# Call the base class dummy command to force a load
package require Engtclbase
Engtclbase

# Load the object file
::load $::HW_control::object

package provide HW_control 1.0

namespace eval ::HW_control {

#****************************************************************************
# Name:        Init
# Parameters:  none
# Description: Sets up the callbacks to update the state data
#****************************************************************************
 proc Init { } {
   HW_control -onchange ref       { ::HW_control::updateRef   %V}
   HW_control -onchange backlight { ::HW_control::updateBack  %V}
   HW_control -onchange leds      { ::HW_control::updateLed   %V}
   HW_control -onchange tcxo      { ::HW_control::updateTcxo  %V}
   HW_control -onchange yig       { ::HW_control::updateYig   %V}
   HW_control -onchange yig_cal   { set ::HW_control::State(yig_running) %V}
   HW_control -onchange yig_res   { ::HW_control::updateYigRes   %V}
   HW_control -onchange enable    { ::HW_control::updateEnable   %V}
   HW_control -onchange router_cal { ::HW_control::updateRouteCal %V}
   HW_control -onchange temp      { ::HW_control::updateTemp   %V}
 }

#****************************************************************************
# Name:        updateRef
# Parameters:  Reference setting - 0 = external, 1 = internal
# Description: Sets up the current state of the 10 MHz ref source
#****************************************************************************
 proc updateRef { ref } {
    variable State
    if {"$ref" == 0 } { 
      set State(ref) "External"
    } else {
      set State(ref) "Internal"
    }
    set ::sys(10mhzref) $State(ref)
 }
 
#****************************************************************************
# Name:        updateBack
# Parameters:  Backlight level
# Description: Sets up the current state of the backlight
#****************************************************************************
 proc updateBack { level } {
    variable State
    set State(backlight) $level
 }
 
#****************************************************************************
# Name:        updateLed
# Parameters:  LED settings
# Description: Sets up the current state of the front panel LEDs
#****************************************************************************
 proc updateLed { state } {
    variable State
    set State(leds) $state
    if {[expr $state & 1] == 1 } {
      set State(SHIFT_LED) 1
    } else {
      set State(SHIFT_LED) 0
    }
 }
 
#****************************************************************************
# Name:        updateTcxo
# Parameters:  tcxo cal value
# Description: Sets up the current cal value for the TCXO
#****************************************************************************
 proc updateTcxo { value } {
    variable State
    set State(tcxo) $value
 }
#****************************************************************************
# Name:        Set_TCXO
# Parameters:  none
# Description: Sets up the current cal value for the TCXO
#****************************************************************************
 proc Set_TCXO {  } {
    variable State
    HW_control tcxo $State(tcxo)
 }
#****************************************************************************
# Name:        Set_RX_YIG
# Parameters:  none
# Description: Sets up the current cal value for the RX YIG
#****************************************************************************
 proc Set_RX_YIG {  } {
    variable State
    Receiver yigcal 0   $State(yig_rec0)
    Receiver yigcal 255 $State(yig_rec2800)
 }
#****************************************************************************
# Name:        Set_GEN_YIG
# Parameters:  none
# Description: Sets up the current cal value for the GEN YIG
#****************************************************************************
 proc Set_GEN_YIG {  } {
    variable State
    Generator yigcal 0   $State(yig_gen0)
    Generator yigcal 255 $State(yig_gen2800)
 }
#****************************************************************************
# Name:        updateYig
# Parameters:  YIG lock status
# Description: Sets up the lock status for the YIGS
#****************************************************************************
 proc updateYig { value } {
    variable State
    set State(yig) $value
 }
 
#****************************************************************************
# Name:        updateYigRes
# Parameters:  YIG lock status
# Description: Sets up the YIG cal results for the YIGS
#****************************************************************************
 proc updateYigRes { arg } {
    set status  [lindex $arg 0]
    set rcv0    [lindex $arg 1]
    set rcv2800 [lindex $arg 2]
    set gen0    [lindex $arg 3]
    set gen2800 [lindex $arg 4]
   
    variable State
    set gen_state         [ expr {$status & 4095}]
    set rec_state         [ expr {$status >> 12} ]

    set State(yigres) $status
    set State(yig_rec0)    $rcv0
    set State(yig_rec2800) $rcv2800

    set State(yig_gen0)    $gen0
    set State(yig_gen2800) $gen2800
    
    if {$rec_state == 0} \
    {
       set State(yigres_rec) "Complete"
       set ::RcvrYigCals(0)     $rcv0
       set ::RcvrYigCals(2800)  $rcv2800
    } \
    else \
    {
       set State(yigres_rec) "Err $rec_state"
    }
    if {$gen_state == 0} \
    {
       set State(yigres_gen) "Complete"
       set ::RfGenYigCals(0)    $gen0
       set ::RfGenYigCals(2800) $gen2800
    } \
    else \
    {
       set State(yigres_gen) "Err $gen_state"
    }
    if {$State(on_yig_done) != ""} \
    {
       after idle [subst "::HW_control::YigDoneAfter {$State(on_yig_done)}"]
    }
    set State(on_yig_done) ""
 }
 
#****************************************************************************
# Name:        updateEnable
# Parameters:  task enable
# Description: 
#****************************************************************************
 proc updateEnable { args } {
    variable State
    set State(enable) $args
 }
 
#****************************************************************************
# Name:        updateRouteCal
# Parameters:  task enable
# Description: 
#****************************************************************************
 proc updateRouteCal { value } {
    variable State
    set index [lindex $value 0]
    set val   [lindex $value 1]
    set State(route_cal$index) $val
 }
#****************************************************************************
# Name:        updateTemp
# Parameters:  task enable
# Description: 
#****************************************************************************
 proc updateTemp { temp } {
    variable State
    set State(if_temp) [format "%2.1f" $temp]
 }
#****************************************************************************
# Name:        WaitForYigDone
# Parameters:  none
# Description: Wait for the yig cal to complete
#****************************************************************************
 proc WaitForYigDone { } {
    variable State
    while {$State(yig) != 0 } \
    {
       vwait ::HW_control::State(yig)
    }
 }
 proc YigDoneAfter { {cmd ""} } \
 {
    variable State
    if {$cmd == ""} { return }
    if {$State(yig_running) != 0} \
    {
       append State(on_yig_done) "$cmd;"
    } \
    else \
    {
       eval $cmd
    }
 }
 
#****************************************************************************
# Name:        StartYigCal
# Parameters:  none
# Description: Start the yig cal routine
#****************************************************************************
 proc StartYigCal { } \
 {
    variable State
    if {$State(yig_running) == 1} { return }
    
    set State(yig_running)   1
    set State(yigres_rec) "Running"
    set State(yigres_gen) "Running"
    HW_control yig_cal
 }
 
#****************************************************************************
# Name:        ShiftLED
# Parameters:  none
# Description: Turn the SHIFT LED on or off
#****************************************************************************
 proc ShiftLED {state } {
    variable State
    switch -- $state {
      "0" -
      "off" -
      "OFF"   { set State(SHIFT_LED) 0 }
      "1" -
      "on" -
      "ON"    { set State(SHIFT_LED) 1 }
      default { set State(SHIFT_LED) 0 }
    }
    HW_control leds $State(SHIFT_LED) 1
 }

#****************************************************************************
# Name:        Set_clock
# Parameters:  Time string in the form of HH:MM:SS DD MM YYYY
# Description: Set the Real time clock chip's idea of the time
#****************************************************************************
 proc Set_clock { time } {
    tcl_rtc [clock scan $time]
 }
 
 #****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the HW_Utilities engine to its current state
#****************************************************************************
 proc getSaveState { } \
 {
   variable State
   set returnList \
   {
      HW_control ref $State(ref) -nowait      
   }

   return [subst -nocommands $returnList]
 }
 
#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to hardware
#****************************************************************************
 proc Sync { } {
     eval [getSaveState]
 }

#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
   Init

# Define items done only in the master interp
if ![interp issafe] \
{
   source $path/Master_only.tcl
}


}  ;# End of the Hardware control namespace
