#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2003      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : modacc.tcl                                                  #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood                                                #
# DESCRIPTION : This file is sourced by TCL when the ModAcc command is      #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::Modacc::State ruw "::Modacc::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
if {"[info command Modacc]" != "" } { return }

# Call the base class dummy command to force a load
package require Engtclbase
Engtclbase

# Load the object file
::load $::Modacc::object

package provide Modacc 1.0


namespace eval ::Modacc \
{

   variable SaveSortOrder AFTER(Receiver)
   proc set_symbols { } \
   {
      variable State
      Modacc symbols $State(symbols) -nowait
   }
   
#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
   proc clip_peak { value reading } \
   {
      variable State
      if [catch {
          set State($value) [format %.2f $reading]
          set State($value,raw) $reading
         } err ] \
      {
         set State($value) "NaN"
         set State($value,raw) "NaN"
      }
   }
   
#****************************************************************************
# clip values to precision
#****************************************************************************
   proc clip { which } \
   {
      variable State
      if {$State($which,precision) > 0} \
      {
         if [catch {
              set State($which,value) [format "%.*f" $State($which,precision) $State($which,raw)]
             } err ] \
         {
            set State($which,value) "NaN"
         }
      } \
      else \
      {
         if [catch {
              set State($which,value) [format "%.*f" 0 $State($which,raw)]
             } err ] \
         {
            set State($which,value) "NaN"
         }
      }      
   }
 
#****************************************************************************
# Snag the readings
#****************************************************************************
   proc updateReading { which  values } \
   {
     #puts "updateReading:  $which,  $values"

     variable State

     set State($which,raw)       [lindex $values 0]
     set State($which,precision) [lindex $values 1]
     set State($which,state)     [lindex $values 2]
  
     clip $which
   }
#****************************************************************************
# Snag the peak readings
#****************************************************************************
   proc updatePeak { which  values } \
   {
     #puts "updatePeak:  $which,  $values"

     variable State

     set State($which,hi,raw)       [lindex $values 0]
     set State($which,hi,precision) [lindex $values 1]
     set State($which,lo,raw)       [lindex $values 2]
     set State($which,lo,precision) [lindex $values 3]
     set State($which,peak,flags)   [lindex $values 4]
 
     clip $which,hi
     clip $which,lo
   }
#****************************************************************************
# Snag the alert limits
#****************************************************************************
   proc updateAlerts { which  values } \
   {
     #puts "updateAlerts:  $which,  $values"

     variable State

     set State($which,hi_limit,raw)       [lindex $values 0]
     set State($which,hi_limit,precision) [lindex $values 1]
     set State($which,lo_limit,raw)       [lindex $values 2]
     set State($which,lo_limit,precision) [lindex $values 3]
     set State($which,limit,flags)        [lindex $values 4]

     if {$State($which,limit,flags) & 1} \
     {
        set State($which,hi_limit,text) "ON"
     } \
     else \
     {
        set State($which,hi_limit,text) "OFF"
     }
     if {$State($which,limit,flags) & 2} \
     {
        set State($which,lo_limit,text) "ON"
     } \
     else \
     {
        set State($which,lo_limit,text) "OFF"
     }
     if {$State($which,limit,flags) & 4} \
     {
        set State($which,alarm,text) "ON"
        set State($which,alarm) 1
     } \
     else \
     {
        set State($which,alarm,text) "OFF"
        set State($which,alarm) 0
     }
 
     clip $which,hi_limit
     clip $which,lo_limit
   }
   
#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the AF Counter engine to its current state
#****************************************************************************
 proc getSaveState { } \
 {
    variable State
    
    foreach value { evm  mag  ph  freq  or } \
    {
      foreach cmd {average peakhold ulimit llimit alerts} \
      {
        append rv  "  Modacc $value $cmd  [Modacc $value $cmd]  -nowait \n"
      }
    }
    
    foreach value { enable  run  symbols } \
    {
      append rv  "  Modacc $value  [Modacc $value]  -nowait \n"
    }
    
    set returnList \
    {
      if { [IFR_options is_installed EVM] } \
      {
        $rv
      }
    }
    
    return  [subst -nocommand $returnList]
 }
 
 
#****************************************************************************
# Name:        Get_HTML
# Parameters:  base_url - the base URL for the containing page
# Description: Return an string containing HTML showing the current state
#              of this system
#****************************************************************************
 proc Get_HTML { {base_url ""} } \
 {
    variable State
    if ![IFR_options is_installed EVM] { return }
    if {$State(run) == 0 } { return }
    if {$State(evm,state) & 2 } \
    {
       # stale reading - don't do anything
       return
    }
    set x "<table align=left border=1>\n"
    append x "<tr><td nowrap colspan=2><center><h2>Error Vector Magnitude</h2></center></td></tr>\n"
    foreach {item name units } { evm "Error Vector Magnitude" %
                                 mag "Magnitude error" %
                                 ph  "Phase error" degrees
                                 freq "Frequency error" Hz
                                 or  "Origin offset" dBc
                               } \
    {
       append x "<tr><td nowrap>$name</td><td>$State($item,value) $units</td></tr>\n"
    }
    append x "<tr><td nowrap>Symbol count</td><td nowrap>$State(symbols)</td></tr>\n"
    append x "<tr><td nowrap>Filter alpha</td><td nowrap>$State(alpha)</td></tr>\n"
    append x "<tr><td nowrap>Maximum magnitude error</td><td nowrap>$State(pmag) %</td></tr>\n"
    append x "<tr><td nowrap>Maximum phase error</td><td nowrap>$State(pphase) degrees</td></tr>\n"
    append x "<tr><td nowrap>Maximum EVM</td><td nowrap>$State(pevm) %</td></tr>\n"
    append x "</table>"
    return $x
 }
 
#****************************************************************************
# hook up the callbacks
  proc callbacks {} \
  {
    variable State
   foreach value { evm  mag  ph  freq  or } \
   {
      Modacc $value -onchange reading     "::Modacc::updateReading $value %V"
      Modacc $value -onchange readpeak     "::Modacc::updatePeak    $value %V"
      Modacc $value -onchange readalerts   "::Modacc::updateAlerts  $value %V"
      Modacc $value -onchange average      "set ::Modacc::State($value,average) %V"
      Modacc $value  reading
      Modacc $value  readpeak
      Modacc $value  readalerts
      Modacc $value  average
   }
   foreach value { enable  run  symbols  alpha } \
   {
     Modacc -onchange $value    "set ::Modacc::State($value) %V"
     Modacc $value
   }

   foreach value { pphase pmag pevm} \
   {
     Modacc -onchange $value "::Modacc::clip_peak $value %V"
     Modacc $value
   }

   Modacc -onchange symmemerr "::Modacc::ReduceSymbols %V"
   set State(version) [Modacc version 1]
  }
  after idle ::Modacc::callbacks

 #****************************************************************************
 # Name:        ReduceSymbols
 # Parameters:  values - number of symbols when error occured
 # Description: This is called to reduce the number of symbols used for the
 #     I-Q plot, because there is not enough DSP space to handle it.
 #****************************************************************************
 proc ReduceSymbols { values } \
 {
     variable State

     set errSymbolCnt       [lindex $values 0]
     if { $errSymbolCnt <= $State(symbols) } \
     {
        set temp [expr $State(symbols) - 10]
        Modacc symbols $temp -forgive
     }
 }
 
}  ;# End of the Modacc namespace

#****************************************************************************
# Define the EVM namespace, and set up our HTML option info
#****************************************************************************
namespace eval EVM \
{
   variable InfoString

   set EVM_version [string trimleft [Modacc version] "ModAccEng"]
   set InfoString [subst \
   {
         <br>
         (using EVM version - $EVM_version)
         <br>
         This option enables modulation graphic and measurement capability.
   }]
}

#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines Modacc
}

#****************************************************************************
# This registers the engine with the list for generating an HTML page
#****************************************************************************
namespace eval ::HTTPD \
{
   variable main_screen_items
   lappend  main_screen_items(2) Modacc
}

#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   set names(Modacc)   "EVM engine"
   set version(Modacc)   [Modacc ver 0]
}
