#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : power.tcl                                                   #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the Power command is       #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::Power::State ruw "::Power::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
# Call the base class dummy command to force a load
if {"[info command Power]" != "" } { return }

package require Engtclbase
Engtclbase

# Load the object file
::load $::Power::object

package provide Power 1.0

#****************************************************************************
# Add functions to the Power namespace
#****************************************************************************
#    Status of commands sent to Power
#       ::Power::State(enabled)             {bitmap}
#       ::Power::State(reading)             Power reading
#       ::Power::State(range)               Power range
#       ::Power::State(accuracy)            Power reading accuracy
#       ::Power::State(over_range)          Power reading flags
#****************************************************************************
namespace eval ::Power {

  variable Fan_speeds
  array set Fan_speeds \
  {
    14 STOP
    12 SLOW
    10 MEDIUM1
    8  MEDIUM2
    6  FAST
  }
  
#****************************************************************************
# Name:        set_power_callbacks
# Parameters:  none
# Description: Sets up the callbacks to update the Power meter data
#****************************************************************************
 proc set_power_callbacks { } \
 {
   Power -onchange rate      { set ::Power::State(rate) %V}
   Power -onchange rawenable { ::Power::updateEnabled %V}
   Power -onchange barreading { ::Power::updateBarReading %V}
   Power -onchange -structure reading  { ::Power::updateReading %V}
   Power -onchange peakhold { ::Power::updatePeakhold %V}
   Power -onchange alerts   { ::Power::updateAlerts %V}
   Power -onchange ulimit   { ::Power::updateUlimit %V}
   Power -onchange llimit   { ::Power::updateLlimit %V}
   Power -onchange cable    {set ::Power::State(cableloss) [format %.2f %V]}
   Power -onchange temperature    {::Power::updateTemp %V}
   Power -onchange overtemp { set ::Power::State(temp_overload) %V}
   Power -onchange ovld     {::Power::updateOverload %V}
   Power -onchange prange   {set ::Power::State(prange) %V}
   Power -onchange readpeak { ::Power::updatePeaks %V }
   Power -onchange readpeakdbm { ::Power::updatePeaksdBm %V }
   Power -onchange samplerate { set ::Power::State(samplerate) %V }
   set State(engineId) [format_version [AFCounter ver 0] ]
   set State(driverId) [format_version [AFCounter ver 1] ]
 }


#****************************************************************************
# Name:        updatePeaks
# Parameters:  readingList      List of data from the engine
#                 Peak HI       Max peak value
#                 HI Precision  Max peak precision
#                 Peak LO       Min peak value
#                 LO Precision  Min peak precision
#                 Flags         Status flags
# Description: Sets up the current state of the peak readings
#****************************************************************************
 proc updatePeaks { readingList } \
 {
   variable  State

   # Get the data from the peak readings
   set peakHi            [lindex $readingList 0]
   set peakHiPrec        [lindex $readingList 1]
   set peakLo            [lindex $readingList 2]
   set peakLoPrec        [lindex $readingList 3]
   set State(peakFlags)  [lindex $readingList 4]

   set State(peakHi)  [format "%0.${peakHiPrec}f" $peakHi]
   set State(peakLo)  [format "%0.${peakLoPrec}f" $peakLo]
   
   return
 }


#****************************************************************************
# Name:        updatePeaksdBm
# Parameters:  readingList      List of data from the engine
#                 Peak HI       Max peak value
#                 HI Precision  Max peak precision
#                 Peak LO       Min peak value
#                 LO Precision  Min peak precision
#                 Flags         Status flags
# Description: Sets up the current state of the peak readings
#****************************************************************************
 proc updatePeaksdBm { readingList } \
 {
   variable  State

   # Get the data from the peak readings
   set peakHi            [lindex $readingList 0]
   set peakHiPrec        [lindex $readingList 1]
   set peakLo            [lindex $readingList 2]
   set peakLoPrec        [lindex $readingList 3]
   set State(peakFlags)  [lindex $readingList 4]

   set State(peakHi,dBm)  [format "%0.${peakHiPrec}f" $peakHi]
   set State(peakLo,dBm)  [format "%0.${peakLoPrec}f" $peakLo]
   
   return
 }
#****************************************************************************
# Name:        updateEnabled
# Parameters:  state
# Description: handle overload messages
#****************************************************************************
 proc updateEnabled { enable } \
 {
    variable State
    set State(enable,raw) $enable
    if {($enable & 2) == 0} \
    {
       set State(enable) off
    } \
    else \
    {
       set State(enable) on
    }
 }
  
#****************************************************************************
# Name:        updateOverload
# Parameters:  state
# Description: handle overload messages
#****************************************************************************
 proc updateOverload { overload } \
 {
   variable State
   set State(overload) $overload
 }
  
#****************************************************************************
# Name:        updateTemp
# Parameters:  state
# Description: update the temp reading
#****************************************************************************
 proc updateTemp { cmd } {
   variable State
   variable Fan_speeds
   
   foreach {temp1 temp2 fan } $cmd \
   {
      set State(temp1,raw) $temp1
      set State(temp2,raw) $temp2
      set State(temp1) [format %3.1f $temp1]
      set State(temp2) [format %3.1f $temp2]
      set State(fan_raw) $fan
      if [info exists Fan_speeds($fan)] {set State(fan) $Fan_speeds($fan)}
   }
 }
#****************************************************************************
# Name:        updateAlerts
# Parameters:  state
# Description: Sets up the current enabled state of the Power meter
#****************************************************************************
 proc updateAlerts { alerts } \
 {
   variable State
 
   set State(alerts)  $alerts
   if {$alerts & 2} \
   {
     set State(alertHi_text) "ON"
   } \
   else \
   {
     set State(alertHi_text) "OFF"
   }
   if {$alerts & 1} \
   {
     set State(alertLo_text) "ON"
   } \
   else \
   {
     set State(alertLo_text) "OFF"
   }
   if {$alerts & 4} \
   {
     set State(alarm_text) "ON"
     set State(BellEnabled) 1
   } \
   else \
   {
     set State(alarm_text) "OFF"
     set State(BellEnabled) 0
   }
 }

#****************************************************************************
# Name:        toggleAlerts
# Parameters:  alertType        {HI or LO}
# Description: Toggles either the hi or lo alert enable
#****************************************************************************
 proc toggleAlerts { alertType } \
 {
   variable State
   if { $alertType == "HI" } \
   {
     Power alerts [expr $State(alerts) ^ 2]
   } else \
   {
     Power alerts [expr $State(alerts) ^ 1]
   }
 }


#****************************************************************************
# Name:        toggleBellState
# Parameters:  none
# Description: Toggles whether there is an audible alert
#****************************************************************************
 proc toggleBellState { } \
 {
   variable State
   Power alerts [expr $State(alerts) ^ 4] -nowait
 }


#****************************************************************************
# Name:        updateUlimit
# Parameters:  state
# Description: 
#****************************************************************************
 proc updateUlimit { state } {
   variable State
   set State(alertHi) $state
   set State(alertHi,W) [format "%.4f" $state] 
   set State(alertHi,dBm) [format "%.2f" [Units_convert::W_to_dBm $state] ]
 }
#****************************************************************************
# Name:        updateLlimit
# Parameters:  state
# Description: 
#****************************************************************************
 proc updateLlimit { state } {
   variable State
   set State(alertLo) $state
   set State(alertLo,W) [format "%.4f" $state] 
   set State(alertLo,dBm) [format "%.2f" [Units_convert::W_to_dBm $state] ]
 }

#****************************************************************************
# Name:        ulimit
# Parameters:  none
# Description: This is called when the low alert limit is changed to trigger
#              an activation of the low alert
#****************************************************************************
 proc ulimit { units } {
    variable State

    if { $units == "dBm" } \
    {
       Power ulimit [Units_convert::dBm_to_W $State(alertHi,dBm)] -nowait
    } else \
    {
       Power ulimit $State(alertHi,W) -nowait
    }

    # Automatically activate the alert if the limit is changed
    set temp [expr $State(alerts) & 0x02]
    if {$temp == 0} \
    {
       toggleAlerts HI
    }
 }

#****************************************************************************
# Name:        llimit
# Parameters:  none
# Description: This is called when the low alert limit is changed to trigger
#              an activation of the low alert
#****************************************************************************
 proc llimit { units } {
    variable State

    if { $units == "dBm" } \
    {
       Power llimit [Units_convert::dBm_to_W $State(alertLo,dBm)] -nowait
    } else \
    {
       Power llimit $State(alertLo,W) -nowait
    }

    # Automatically activate the alert if the limit is changed
    set temp [expr $State(alerts) & 0x01]
    if {$temp == 0} \
    {
       toggleAlerts LO
    }
 }

#****************************************************************************
# Name:        updatePeakhold
# Parameters:  state
# Description: Sets up the current enabled state of the RL meter
#****************************************************************************
 proc updatePeakhold { peakEnable } \
 {
   variable  State

   set State(peakEnable)  $peakEnable

   if { $State(peakEnable) == 1 } \
   {
     set State(peakEnable_text)  "ON"
   } \
   else \
   {
     set State(peakEnable_text)  "OFF"
   }
 }
#****************************************************************************
# Name:        updateBarReading
# Parameters:  readingList      Data from engine: Reading, Accuracy, Status
# Description: Currently this is only used to set the audible alerts
#****************************************************************************
 proc updateBarReading { readingList } \
 {

   variable State
   # Get the data from the meter reading
   # set State(avg_value)     [format "%3.2f" [lindex $readingList 0] ]
   # set State(precision)     [lindex $readingList 1]

   set State(alertStatus) [lindex $readingList 2]
 }
 
#****************************************************************************
# Name:        updateReading
# Parameters:  readingList      Data from engine: Reading, Accuracy, Status
# Description: Sets up the current state of the RL meter reading
#****************************************************************************
 proc updateReading { readingList } \
 {
   variable State
   # Get the data from the meter reading
   foreach {readings range prec status State(ad_value) State(low_zero) State(high_zero)} $readingList {}
   foreach {State(reading_raw,max) State(reading_raw,min) State(reading_raw,max,dBm) State(reading_raw,min,dBm) } $readings {}
   
   if {$State(reading_raw,max) == "NaN" } { return }
      
   set State(precision) $prec
   set State(status)    $status
   set State(range)     $range
   set State(reading,max)    [format %0.${prec}f $State(reading_raw,max)]
   set State(reading,min)    [format %0.${prec}f $State(reading_raw,min)]
   set State(reading,max,dBm) [format %.2f $State(reading_raw,max,dBm)]
   set State(reading,min,dBm) [format %.2f $State(reading_raw,min,dBm)]
   
   set State(reading)     $State(reading,max)
   set State(reading,dBm) $State(reading,max,dBm)

   # update range string
   if {$range == 1 } \
   {
      set State(range_text) "Low"
   } \
   else \
   {
     set State(range_text) "High"
   }
 }

#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the AF Counter engine to its current state
#****************************************************************************
 proc getSaveState { } {

   variable State
   set returnList \
   {
      Power enable   $State(enabled)    -nowait
      Power peakhold $State(peakEnable) -nowait
      Power alerts   $State(alerts)     -nowait
      Power ulimit   $State(alertHi)    -nowait
      Power llimit   $State(alertLo)    -nowait
      Power cable    $State(cableloss)  -nowait
      Power prange   $State(prange)     -nowait
      catch {Power rate $State(rate) -nowait }
      catch {Power   samplerate 8000 -nowait }
      set ::Power::State(BellEnabled) {$State(BellEnabled)}
   }
   return [subst $returnList]
 }

#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to hardware
#****************************************************************************
 proc Sync { } {
    after 1000 {eval [Power::getSaveState]}
 }

#****************************************************************************
# Name:        SyncAll
# Parameters:  none
# Description: Insures that TCL, the engine and hardware are in sync
#****************************************************************************
 proc SyncAll { } {
    set_power_callbacks

    if ![interp issafe] \
    {
       variable path
       source [file join $path master.tcl]
    }
 }

#****************************************************************************
# Name:        Get_HTML_reading
# Parameters:  base_url - the base URL for the containing page
# Description: Return an string containing HTML showing the current state
#              of this system
#****************************************************************************
 proc Get_HTML_reading { {base_url ""} } \
 {
    variable State
    set x \
{<tr><td nowrap>Wideband RF power</td><td nowrap>$State(reading,max) Watts max</td></tr>
<tr><td nowrap></td><td nowrap>$State(reading,min) Watts min</td></tr>
<tr><td nowrap></td><td nowrap>$State(reading,max,dBm) dBm max</td></tr>
<tr><td nowrap></td><td nowrap>$State(reading,min,dBm) dBm min</td></tr>
}
    return [subst $x]
 }
 
#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
   Load::on_UI_done { ::Power::SyncAll }
   
}  ;# End of the Power namespace

#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines Power
}
#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
     Power_Eng "Power meter Engine"
     Power_Drv "Power meter Driver"
     Power_Hw  "Power Term Hardware"
   }
   set version(Power_Eng) [Power version 0]
   set version(Power_Drv) [Power version 1]
   set version(Power_Hw)  "Revision [Power version 2]"
}

#****************************************************************************
# This registers the engine with the list for generating an HTML page
#****************************************************************************
namespace eval ::HTTPD \
{
   variable main_screen_readings
   lappend  main_screen_readings(1) Power
}
