#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : rlmeter.tcl                                                 #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the RLmeter command is     #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::RLmeter::State ruw "::RLmeter::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
if {"[info command RLmeter]" != "" } { return }

# Call the base class dummy command to force a load
package require Engtclbase
Engtclbase

# Load the object file
::load $::RLmeter::object

package provide RLmeter 1.0

#****************************************************************************
# Add functions to the RLmeter namespace
#****************************************************************************
#    Status of commands sent to RLmeter
#       ::RLmeter::State(Engine_enabled)      {0 | 1}
#       ::RLmeter::State(enabled)             {OFF | ON}
#       ::RLmeter::State(driverId)            Driver ID information
#       ::RLmeter::State(engineId)            Engine ID information
#    Status of counter data received from the engine
#       ::RLmeter::State(reading)             Current RL reading
#       ::RLmeter::State(precision)           Current precision of reading
#       ::RLmeter::State(status)              Status flags
#       ::RLmeter::State(alertStatus)         Alert status flags
#       ::RLmeter::State(cableStateHistory)   Cable Loss state machine history
#****************************************************************************
namespace eval ::RLmeter {
   variable SaveSortOrder AFTER(Receiver)
#****************************************************************************
# Name:        set_rlmeter_callbacks
# Parameters:  none
# Description: Sets up the callbacks to update the RL meter data
#****************************************************************************
 proc set_rlmeter_callbacks { } {

   if { $::RLmeter::callbackStatus != 0 } {
       return
   }

   RLmeter -onchange barreading { ::RLmeter::updateBarReading %V}
   RLmeter -onchange reading    { ::RLmeter::updateReading %V}
   RLmeter -onchange readpeak   { ::RLmeter::updatePeaks %V}
   RLmeter -onchange en         { ::RLmeter::updateEnable %V}
   RLmeter -onchange run        { ::RLmeter::updateMode %V}
   RLmeter -onchange peakhold   { ::RLmeter::updatePeakhold %V}
   RLmeter -onchange alerts     { ::RLmeter::updateAlerts %V}
   RLmeter -onchange ulimit     { ::RLmeter::updateUlimit %V}
   RLmeter -onchange llimit     { ::RLmeter::updateLlimit %V}
   RLmeter -onchange cable      { ::RLmeter::updateCableloss %V}

   # Block slave interpreters from access to the calibration process
   if ![interp issafe] \
   {
      RLmeter -onchange status     { ::RLmeter::updateStatus %V}
   }
   
   set ::RLmeter::callbackStatus 1
 }

#****************************************************************************
# Name:        updateMode
# Parameters:  runState         0 = Off, 1 = On
# Description: Sets up the current run state of the RL meter
#****************************************************************************
 proc updateMode { runState } \
 {
   variable  State

   switch $runState \
   {
     0       { set State(run) OFF }
     1       { set State(run) ON }
     default { return }
   }
   
   set State(Engine_run) $runState
   
   return
 }

#****************************************************************************
# Name:        updatePeaks
# Parameters:  readingList      List of data from the engine
#                 Peak HI       Max peak value
#                 HI Precision  Max peak precision
#                 Peak LO       Min peak value
#                 LO Precision  Min peak precision
#                 Flags         Status flags
# Description: Sets up the current state of the peak readings
#****************************************************************************
 proc updatePeaks { readingList } \
 {
   variable  State

   # Get the data from the peak readings
   set peakHi    [lindex $readingList 0]
   set peakLo    [lindex $readingList 2]
   
   if {$peakHi == "NaN"} { return }
   if {$peakLo == "NaN"} { return }

   set State(peakFlags) [lindex $readingList 4]

   set State(dBm,peakHi)  [format "%0.2f" $peakHi]
   set State(dBm,peakLo)  [format "%0.2f" $peakLo]
   
   set State(W,peakHi)    [format "%0.2f" [Units_convert::dBm_to_W $peakHi] ]
   set State(W,peakLo)    [format "%0.2f" [Units_convert::dBm_to_W $peakLo] ]
   
   return
 }


#****************************************************************************
# Name:        llimit
# Parameters:  state
# Description: set the lower limit of the alarms, with optional units
#****************************************************************************
 proc llimit { units } \
 {
    variable State

    switch -- $units \
    {
       "W" \
       {
          RLmeter llimit [Units_convert::W_to_dBm $State(alertLo,W)] -nowait
       }
       default \
       {
          RLmeter llimit $State(alertLo,dBm) -nowait
       }
    }

    # Automatically activate the alert if the limit is modified
    set temp [expr $State(alerts) & 0x01]
    if {$temp == 0} \
    {
       toggleAlerts LO
    }

 }
#****************************************************************************
# Name:        ulimit
# Parameters:  state
# Description: set the upper limit of the alarms, with optional units
#****************************************************************************
 proc ulimit { units } \
 {
    variable State

    switch -- $units \
    {
       "W" \
       {
          RLmeter ulimit [Units_convert::W_to_dBm $State(alertHi,W)] -nowait
       }
       default \
       {
          RLmeter ulimit $State(alertHi,dBm) -nowait
       }
    }

    # Automatically activate the alert if the limit is changed
    set temp [expr $State(alerts) & 0x02]
    if {$temp == 0} \
    {
       toggleAlerts HI
    }
 }
#****************************************************************************
# Name:        updateCableloss
# Parameters:  state
# Description: Handle the calibration status of the RL meter
#****************************************************************************
 proc updateCableloss { loss } {
    variable State
    set State(cableloss) [format "%.1f" $loss]
 }
 
#****************************************************************************
# Name:        updateStatus
# Parameters:  state
# Description: Handle the calibration status of the RL meter
#****************************************************************************
 proc updateStatus { status } {
   variable State
   
  set lo [ expr ($status      ) & 0xff ]
  set hi [ expr ($status >> 8 ) & 0xFF ]
  
  set State(cal,status)  $hi
  set State(cal,current) [expr $State(cal,num_cals)-$lo]

  switch -- $hi \
  {
    1 { set State(cal,status_text)  "Cal Waiting" ; return }
    2 { set State(cal,status_text)  "Cal Running"
        return
      }
    5 { set State(cal,status_text)  "Cal Done"
        # Switch to calibrated readings...
        RLmeter calibration 4 0 0
      }
    6 { set State(cal,status_text)  "Cal Error"
      }
    7 { set State(cal,status_text)  "Err! (Cable?)"
      } 
    8 { set State(cal,status_text)  "Aborted Cal"
      }
    default {
         set State(cal,status_text)  "Unknown Status $status"
      }
    }
    Cal_finish
 }

#****************************************************************************
# Name:        updateAlerts
# Parameters:  state
# Description: Sets up the current enabled state of the RL meter
#****************************************************************************
 proc updateAlerts { alerts } \
 {
   variable State
 
   set State(alerts)  $alerts
   if {$alerts & 2} \
   {
     set State(alertHi_text) "ON"
   } \
   else \
   {
     set State(alertHi_text) "OFF"
   }
   if {$alerts & 1} \
   {
     set State(alertLo_text) "ON"
   } \
   else \
   {
     set State(alertLo_text) "OFF"
   }
   if {$alerts & 4} \
   {
     set State(alarm_text) "ON"
     set State(BellEnabled) 1
   } \
   else \
   {
     set State(alarm_text) "OFF"
     set State(BellEnabled) 0
   }
 }

#****************************************************************************
# Name:        toggleAlerts
# Parameters:  alertType        {HI or LO}
# Description: Toggles either the hi or lo alert enable
#****************************************************************************
 proc toggleAlerts { alertType } \
 {
   set status $::RLmeter::State(alerts)
   if { $alertType == "HI" } \
   {
     RLmeter alerts [expr $status ^ 2]
   } else \
   {
     RLmeter alerts [expr $status ^ 1]
   }
 }

#****************************************************************************
# Name:        toggleBellState
# Parameters:  none
# Description: Toggles whether there is an audible alert
#****************************************************************************
 proc toggleBellState { } \
 {
   variable State
   RLmeter alerts [expr $State(alerts) ^ 4]
 }


#****************************************************************************
# Name:        updateUlimit
# Parameters:  state
# Description: 
#****************************************************************************
 proc updateUlimit { state } {
   variable State
   set State(alertHi) $state
   set State(alertHi,dBm) [format %0.2f $state]
   set State(alertHi,W)   [format %0.2f [Units_convert::dBm_to_W $state]]
 }
#****************************************************************************
# Name:        updateLlimit
# Parameters:  state
# Description: 
#****************************************************************************
 proc updateLlimit { state } {
   variable State
   set State(alertLo) $state
   set State(alertLo,dBm) [format %0.2f $state]
   set State(alertLo,W)   [format %0.2f [Units_convert::dBm_to_W $state]]
 }
#****************************************************************************
# Name:        updatePeakhold
# Parameters:  state
# Description: Sets up the current enabled state of the RL meter
#****************************************************************************
 proc updatePeakhold { state } {
   variable  State

   set State(peakEnable) $state

   if { $State(peakEnable) == 1 } \
   {
     set State(peakEnable_text)  "ON"
   }

   if { $State(peakEnable) == 0 } \
   {
     set State(peakEnable_text)  "OFF"
   }
 }
 
#****************************************************************************
# Name:        updateEnable
# Parameters:  enableBits       1 = Off, 3 = Enabled
# Description: Sets up the current enabled state of the RL meter
#****************************************************************************
 proc updateEnable { enableBits } {
   variable State
   switch $enableBits {
      3       { set State(enabled) Enabled }
      default { set State(enabled) Disabled }
   }
   set State(Engine_enabled) $enableBits
 }

#****************************************************************************
# Name:        updateBarReading
# Parameters:  readingList      Data from engine: Reading, Accuracy, Status
# Description: Sets up the current state of the RL meter reading
#****************************************************************************
 proc updateBarReading { readingList } {
   variable State

   set State(alertStatus) [lindex $readingList 2]
 }
#****************************************************************************
# Name:        updateReading
# Parameters:  readingList      Data from engine: Reading, Accuracy, Status
# Description: Sets up the current state of the RL meter reading
#****************************************************************************
 proc updateReading { readingList } {
   variable State
   # Get the data from the meter reading
   set reading  [lindex $readingList 0]
   if {$reading > 50} { return }
   if {$reading < -120 } { return }
   switch -- $reading \
   {
     "Inf" -
     "NaN" \
     {
       return
     }
   }
   
   set status    [lindex $readingList 2]

   switch -- $status \
   {
      0  { set State(status_text) "Calibrated Level, dBm"  }
     -1  { set State(status_text) "RF HW Chg!, Reading dB" }
     -2  { set State(status_text) "Raw Readings, dB"       }
     -3  { set State(status_text) "No Spline, Raw Level (dB)" }
     -10 { set State(status_text) "Level Input Too Low"   }
     -11 { set State(status_text) "Level Input Too High"  }
     default { set State(status_text) "Unknown Message $status" }
   }

   set prec [lindex $readingList 1]
   
   set State(dBm,reading_raw) $reading
   set State(dBm,reading)     [format %0.${prec}f $reading]
   set State(dBm,precision)   $prec
   set State(status)          $status

   set watts [Units_convert::dBm_to_W $reading]
   set State(W,reading_raw) $watts
   # precision gets tricky since we are going from dBm to watts. We have roughly $prec+2 digits
   # of resolution in dBm. Since 0 dBm is 1 mW, that's +3 to the precision right there.
   # Since every 10 dBm is an order of magnitude change in watts, we
   # will compute our precision in watts by -$reading/10 + 5 + ($prec)
   # 
   set prec_raw [expr int((.5-$reading)/10 + 5 + $prec)]
   if {$prec_raw < 0 } { set prec_raw 0 }
   set prec $prec_raw
   if {$prec > 2 } { set prec 2 }
   
   set State(W,precision) $prec
   set State(W,reading) [format %0.${prec}f $watts]
   set prec [expr $prec_raw - 3]
   if {$prec < 0 } { set prec 0 }
   if {$prec > 2 } { set prec 2 }
   
   set State(mW,precision) $prec
   set State(mW,reading) [format %0.${prec}f [expr $watts*1000]]

   # BODGE compatibility
   set State(reading) $State(dBm,reading)
   set State(precision) $State(dBm,precision)
   
 }

#****************************************************************************
# Name:        query_rlmeter_state
# Parameters:  none
# Description: This function sends down a request to the RL meter to 
#            : obtain the engine state 
#****************************************************************************
 proc query_rlmeter_state { } {
   variable State
   set State(engineId) [format_version [RLmeter ver 0] ]
   set State(driverId) [format_version [RLmeter ver 1] ]
 }

#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the RL power engine to its current state
#****************************************************************************
 proc getSaveState { } {

   variable State
   
   set returnList \
   {
      RLmeter en       $State(Engine_enabled) -nowait
      RLmeter peakhold $State(peakEnable)     -nowait  
      RLmeter alerts   $State(alerts)         -nowait   
      RLmeter ulimit   $State(alertHi,dBm)    -nowait    
      RLmeter llimit   $State(alertLo,dBm)    -nowait   
      RLmeter cable    $State(cableloss)      -nowait 
      RLmeter run      $State(run)            -nowait
      
      set ::RLmeter::State(BellEnabled)   {$State(BellEnabled)}
   }

   return [subst -nocommands $returnList]
 }

#****************************************************************************
# Name:        Cal_start
# Parameters:  
# Description: starts the RL meter cal.
#****************************************************************************
 proc Cal_start { } \
 {
    variable State
    
    set State(cal_done) 0
   
    # Set the range we will cal over
    switch -- $::Receiver::State(rfinput) \
    {
      ANT \
      {
         set start_level     -20
         set end_level       -80
      }
      default \
      {
         set start_level     10
         set end_level      [expr -45 + [Receiver atten]]
      }
    }
    # save the generator state - we are going to be modifying it.
    set State(cal,old_gen_state) [::Generator::getSaveState]
    
    # initially, set our retries to -1, so we will try 10 times...
    set State(cal,retries) -1

    set delay 0
    # set up the gen hardware
    if {$::Generator::State(dcpower) != 1 } \
    {
       Generator dcpower ON
       puts "Powering on generator..."
       # the generator will be cold-starting.. give it a lot of time to start
       incr State(cal,retries) -100
    }
    
    if {"$::Generator::State(ptt)" != "ON"} \
    {
       Generator ptt on
       # the generator will be cold-starting.. give it a lot of time to start
       incr State(cal,retries) -50
    }
    
    Generator freq   [Receiver freq]
    Generator rfout  GEN
    Generator mod    OFF
    Generator rflevel $start_level
    Generator enable 27
    # set up the receiver state too
    set State(cal,old_rcv_state) [::Receiver::getSaveState]
    Receiver::Config_agc 10 -10.5 -9.5 16
    
    # BODGE QUESTION Should we reset the cable loss? or should we take it into account?
    
    # compute the # of steps to take
    set step_size -5
    set num_cals [expr ($end_level - $start_level)/$step_size]
    
    puts "RLmeter::Cal_start calibrating from $start_level to $end_level, in $step_size dB steps, for $num_cals points total"

    set State(cal,num_cals)  $num_cals
    set State(cal,step_size) $step_size
   
    # set up for the calibration run
    RLmeter calibration 1 $num_cals 0
    
    # request uncalibrated readings during the cal
    RLmeter calibration 4 1 0 
    
    set State(cal,status_text) "Waiting for RF Generator stable"
    Cal_wait_for_signal $start_level
 }
#****************************************************************************
# Name:        Cal_wait_for_signal
# Parameters:  starting cal level
# Description: wait for signal level ready
#****************************************************************************
 proc Cal_wait_for_signal { start_level} \
 {
    variable State
    if {[incr State(cal,retries)] > 60 } \
    {
        puts "RLmeter::Cal_wait_for_signal - retries exceeded"
        Cal_abort
        updateStatus 0x600
        return
    }
    if [info exists State(cal,after)] \
    {
       after cancel $State(cal,after)
    }
    set State(cal,after) [after 500 "RLmeter::Cal_signal_check $start_level"]
 }
#****************************************************************************
# Name:        Cal_signal_check
# Parameters:  starting cal level
# Description: verifies hardware is ready to cal
#****************************************************************************
 proc Cal_signal_check { start_level} \
 {
    variable State
   
    # if the generator hasn't locked yet, try try again
    if {$::Generator::State(lockstat) != 0 } \
    {
        puts "RLmeter::Cal_signal_check - gen not locked"
        Cal_wait_for_signal $start_level
        return
    }

    # Ready to begin cal
    set State(cal,retries) -1

    RLmeter calibration 2 $State(cal,num_cals) $State(cal,step_size)

    RLmeter calibration 3 0         $start_level
 }
 
#****************************************************************************
# Name:        Cal_abort
# Parameters:  none
# Description: aborts running cal
#****************************************************************************
 proc Cal_abort { } \
 {
    RLmeter calibration 5 0 0
 }
#****************************************************************************
# Name:        Cal_finish
# Parameters:  none
# Description: finish the cal - cancel any afters & restore state
#****************************************************************************
 proc Cal_finish { } \
 {
    variable State
    if [info exists State(cal,after)] \
    {
       after cancel $State(cal,after)
       unset State(cal,after)
    }
    # restore gen and receive states
    if [info exists State(cal,old_gen_state)] \
    {
       catch {eval $State(cal,old_gen_state)}
       unset State(cal,old_gen_state)
    }
    if [info exists State(cal,old_rcv_state)] \
    {
       catch {eval $State(cal,old_rcv_state)}
       unset State(cal,old_rcv_state)
    }
    
    set State(cal,current) $State(cal,num_cals)
    set State(cal_done) 1
 }

#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to hardware
#****************************************************************************
 proc Sync { } {
    eval [getSaveState]
 }

#****************************************************************************
# Name:        SyncAll
# Parameters:  none
# Description: Insures that TCL, the engine and hardware are in sync
#****************************************************************************
 proc SyncAll { } {
    set_rlmeter_callbacks
    query_rlmeter_state
    if ![interp issafe] \
    {
       after 100 {
         ::RLmeter::Sync
         RLmeter enable 3
         RLmeter run ON
       }
    }
 }

#****************************************************************************
# Name:        Get_HTML_reading
# Parameters:  base_url - the base URL for the containing page
# Description: Return an string containing HTML showing the current state
#              of this system
#****************************************************************************
 proc Get_HTML_reading { {base_url ""} } \
 {
    variable State
    set x \
{<tr>
<td nowrap>Narrowband RF power (RSSI)</td><td nowrap>$::RssiUI::StateUI(reading) $::RssiUI::StateUI(units)</td>
</tr>}
    return [subst $x]
 }
 
#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
   Load::on_UI_done { ::RLmeter::SyncAll }
   
}  ;# End of the RL Power namespace

#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines RLmeter
}

#****************************************************************************
# This registers the engine with the list for generating an HTML page
#****************************************************************************
namespace eval ::HTTPD \
{
   variable main_screen_readings
   lappend  main_screen_readings(1) RLmeter
}
#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
     Rlm_Eng   "Receive level meter Engine"
     Rlm_Drv   "Receive level meter Driver"
   }
   set version(Rlm_Eng)   [RLmeter ver 0]
   set version(Rlm_Drv)   [RLmeter ver 1]
}

