#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2002      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : rferror.tcl                                                 #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the RF Error command is    #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::RFError::State ruw "::RFError::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
if {"[info command RFError]" != "" } { return }
# Call the base class dummy command to force a load
package require Engtclbase
package require Receiver
Engtclbase

# Load the object file
::load $::RFError::object

package provide RFError 1.0

#****************************************************************************
# Set up a namespace for RFError
#****************************************************************************
#    Status of alert limits and peak values
#       ::RFError::State(peakHi)              Maximum peak reading
#       ::RFError::State(peakLo)              Minimum peak reading
#       ::RFError::State(peakFlags)           Peak Hold status flags
#       ::RFError::State(peakEnable)          Peak Enable settings
#       ::RFError::State(alertHi)             Maximum valid reading limit
#       ::RFError::State(alertLo)             Minimum valid reading limit
#       ::RFError::State(alerts)              Alert flag status
#****************************************************************************
namespace eval ::RFError {
   variable SaveSortOrder AFTER(Receiver)

#****************************************************************************
# Name:        set_rfctr_callbacks
# Parameters:  none
# Description: Sets up the callbacks to update the RF Error data
#****************************************************************************
 proc set_rfctr_callbacks { } {
   RFError -onchange reading { ::RFError::updateReading %V; }
   
   RFError -onchange readpeak { ::RFError::updatePeaks %V; }
   RFError -onchange ulimit   { ::RFError::updateAlertHi %V; }
   RFError -onchange llimit   { ::RFError::updateAlertLo %V; }
   
   RFError -onchange alerts { ::RFError::updateAlerts %V; }

   RFError -onchange peak { ::RFError::updatePeakEnable %V; }

   RFError -onchange run  { ::RFError::updateMode %V; }
   RFError -onchange gate { ::RFError::updateGate %V; }
   RFError -onchange avg  { ::RFError::updateAvgRes %V; }
   RFError -onchange en   { ::RFError::updateEnable %V; }
 }

#****************************************************************************
# Name:        updateAvgRes
# Parameters:  runState         0 = Off, 1 = On
# Description: Sets up the current run state of the RF Error
#****************************************************************************
 proc updateAvgRes { average } {
   variable State

   set State(average) $average
   if {$average < 10} \
   {
      set State(resolution) "10.0"
   } \
   elseif {$average < 100} \
   {
      set State(resolution) "1.0"
   } \
   else \
   {
      set State(resolution) "0.1"
   }
 }

#****************************************************************************
# Name:        updateMode
# Parameters:  runState         0 = Off, 1 = On
# Description: Sets up the current run state of the RF Error
#****************************************************************************
 proc updateMode { runState } {
   set ::RFError::State(Engine_run) $runState
   switch $runState {
      0       { set ::RFError::State(run) OFF }
      1       { set ::RFError::State(run) ON }
      default { return }
   }
 }

#****************************************************************************
# Name:        updateEnable
# Parameters:  enableBits       1 = Off, 3 = Enabled
# Description: Sets up the current enabled state of the RF Error
#****************************************************************************
 proc updateEnable { enableBits } {
   switch $enableBits {
      3       { set ::RFError::State(enabled) Enabled }
      default { set ::RFError::State(enabled) Disabled }
   }
   set ::RFError::State(Engine_enabled) $enableBits
 }

#****************************************************************************
# Name:        updateGate
# Parameters:  runState         0 = 0.1 second, 1 = 1.0 second
# Description: Sets up the current state of the RF Error gate time
#****************************************************************************
 proc updateGate { gateState } {
   set ::RFError::State(Engine_gate) $gateState
   switch $gateState {
      0       { set ::RFError::State(gate) TenthSec }
      1       { set ::RFError::State(gate) OneSec }
      default { return }
   }
 }

#****************************************************************************
# Name:        updateReading
# Parameters:  count            Frequency count from the engine
#              count_precision  Current digits of accuracy
#              status           Status flags
#              count_accuracy   Current MAXIMUM digits of accuracy
# Description: Sets up the current state of the RF Error gate time
#****************************************************************************
 proc updateReading { readingList } {

   variable State
   # Extract the basic data and store it
   set State(freq) [format "%4.6f" [expr [lindex $readingList 0] / 1000000.0] ]      
   set State(freqPrec)    [lindex $readingList 1]
   set FreqErr                       [lindex $readingList 2]
   set State(freqErrRaw)  $FreqErr
   set State(freqErrPrec) [lindex $readingList 3]
   set State(status)      [lindex $readingList 4]
   set State(freqErrAccuracy)  [lindex $readingList 5]

   set State(reading_raw)  $FreqErr

   # Set up formatted RF Error data
   set temp10 [expr (int($FreqErr + 4.99) / 10) * 10 ]
   set temp1 [expr int($FreqErr)]

   switch -- $State(freqErrAccuracy) {
      -1      { set tempError [format "%5d" $temp10 ] }
      0       { set tempError [format "%5d" $temp1 ] }
      1       { set tempError [format "%5.1f" $FreqErr] }
      2       { set tempError [format "%5.1f" $FreqErr] }
      default { return }
   }

   # If the measurement has not reached the available accuracy for 0.1 hz
   # readings, dash the last digit
   if { $State(freqErrAccuracy) > 0 } {
      if {$State(freqErrPrec) < 1 } {
         set idx [string first "." $tempError]
         set front [string range $tempError 0 $idx]
         set State(freqErrHz) $front-

         set State(reading_cond_Hz) $front-

      } else {
         set State(freqErrHz) $tempError

         set State(reading_cond_Hz) $tempError

      }
   } else {
      set State(freqErrHz) $tempError

      set State(reading_cond_Hz) $tempError
   }

   if {$FreqErr > 99999.9} {
      set State(freqErrMhz) +++++

      set State(reading_cond_MHz) +++++

   } elseif {$FreqErr < -99999.9} {
      set State(freqErrMhz) -----

      set State(reading_cond_MHz) -----

   } else {
      set FreqErr [expr $FreqErr / 1000000]
      set State(freqErrMhz) [format "%1.6f" $FreqErr] 

      set State(reading_cond_MHz) [format "%1.6f" $FreqErr] 

   }
 }

#****************************************************************************
# Name:        updatePeaks
# Parameters:  readingList      List of data from the engine
#                 Peak HI       Max peak value
#                 HI Precision  Max peak precision
#                 Peak LO       Min peak value
#                 LO Precision  Min peak precision
#                 Flags         Status flags
# Description: Sets up the current state of the peak readings
#****************************************************************************
 proc updatePeaks { readingList } {
   # Get the data from the peak readings
   set peakHi    [lindex $readingList 0]
   set peakLo    [lindex $readingList 2]

   set ::RFError::State(peakFlags) [lindex $readingList 4]

   set ::RFError::State(peakHi)    [format "%0.2f" $peakHi]
   set ::RFError::State(peakLo)    [format "%0.2f" $peakLo]
 }


#****************************************************************************
# Name:        updatePeakEnable
# Parameters:  
# Description: Sets up the current state of the peak enable
#****************************************************************************
 proc updatePeakEnable { peakEnable } \
 {
   set ::RFError::State(peakEnable)  $peakEnable

   if { $::RFError::State(peakEnable) == 1 } \
   {
     set ::RFError::State(peakEnable_text)  "ON"
   }

   if { $::RFError::State(peakEnable) == 0 } \
   {
     set ::RFError::State(peakEnable_text)  "OFF"
   }

 }


#****************************************************************************
# Name:        updateAlerts
# Parameters:  
# Description: Update the Alerts flags
#****************************************************************************
 proc updateAlerts { alerts } \
 {
   variable State
 
   set State(alerts)  $alerts
   if {$alerts & 2} \
   {
     set State(alertHi_text) "ON"
   } \
   else \
   {
     set State(alertHi_text) "OFF"
   }
   if {$alerts & 1} \
   {
     set State(alertLo_text) "ON"
   } \
   else \
   {
     set State(alertLo_text) "OFF"
   }
   if {$alerts & 4} \
   {
     set State(alarm_text)       "ON"
     set State(BellEnabled_text) "ON"

     set State(BellEnabled) 1
   } \
   else \
   {
     set State(alarm_text)       "OFF"
     set State(BellEnabled_text) "OFF"

     set State(BellEnabled) 0
   }
 }


#****************************************************************************
# Name:        updateAlertHi
# Parameters:  
# Description: Formats the Alert High value
#****************************************************************************
 proc updateAlertHi { alertHi } {
 
   set ::RFError::State(alertHi) [format "%0.2f" $alertHi]

 }

#****************************************************************************
# Name:        updateAlertLo
# Parameters:  
# Description: Formats the Alert Low value
#****************************************************************************
 proc updateAlertLo { alertLo } {
 
   set ::RFError::State(alertLo) [format "%0.2f" $alertLo]

 }


#********************************************************************
#
#       Name: setLimits
#
# Parameters:
#
# Decsription: Sets the limit values
#
#********************************************************************
proc setLimits { limit_type } \
{
  variable  State
  
  if { $limit_type == "high" } \
  {
    set temp  [expr $State(alerts) & 0x02]
    
    # If the limit alert is turned OFF, turn it ON
    if { $temp == 0 } \
    {
      set temp  [expr $State(alerts) | 0x02]
      
      RFError alerts  $temp  -nowait
    }
    
    RFError ulimit  $State(alertHi)  -nowait
  }

  if { $limit_type == "low" } \
  {
    set temp  [expr $State(alerts) & 0x01]
    
    # If the limit alert is turned OFF, turn it ON
    if { $temp == 0 } \
    {
      set temp  [expr $State(alerts) | 0x01]
      
      RFError alerts  $temp  -nowait
    }
    
    RFError llimit  $State(alertLo)  -nowait
  }

  return
}  


#********************************************************************
#
#       Name: toggleAlerts
#
# Parameters:
#
# Decsription: Toggles the state of the alerts
#
#********************************************************************
proc toggleAlerts { limit_type } \
 {
   variable  State
   
   if { $limit_type == "high" } \
   {
     set new  [expr $State(alerts) ^ 2]
    
     RFError alerts  $new  -nowait
   }

   if { $limit_type == "low" } \
   {
     set new  [expr $State(alerts) ^ 1]
    
     RFError alerts  $new  -nowait
   }
  
   return
 }


#********************************************************************
#
#       Name: toggleAlarm
#
# Parameters:
#
# Decsription: Toggles the state of the alarm
#
#********************************************************************
proc toggleAlarm { } \
{
  variable State
  RFError alerts [expr $State(alerts) ^ 4] -nowait
}


#********************************************************************
#
#       Name: togglePeakEnable
#
# Parameters: 
#
# Decsription: 
#
#********************************************************************
proc togglePeakEnable { } \
{
  if { $::RFError::State(peakEnable) == 1 } \
  {
    after idle "RFError peak 0 -nowait"
  }
  
  if { $::RFError::State(peakEnable) == 0 } \
  {
    after idle "RFError phreset  -nowait"
    after idle "RFError peak 1   -nowait"
  }

  return    
}


#********************************************************************
#
#       Name: setResolution
#
# Parameters: 
#
# Decsription: 
#
#********************************************************************
proc setResolution { } \
 {
   variable  State
  
   if { $State(resolution) == 0.1 } \
   {
     # Set the meter resolution (gate time) to 0.1
     # Set average to 100
     after idle "RFError gate  TENTH  -nowait"
     after idle "RFError average 100  -nowait"
   }
  
   if { $State(resolution) == 1.0 } \
   {
     # Set the meter resolution (gate time) to 0.1
     # Set average to 10
     after idle "RFError gate  TENTH  -nowait"
     after idle "RFError average  20  -nowait"
   }

   if { $State(resolution) == 10.0 } \
   {
     # Set the meter resolution (gate time) to 0.1
     # Set average to 10
     after idle "RFError gate  TENTH  -nowait"
     after idle "RFError average  2  -nowait"
   }

   return
 }


#********************************************************************
#
#       Name: setAverage
#
# Parameters: new_value   - requested new value 
#                           -1 - no new value requested
#
# Description: set the meter average
#
#********************************************************************
proc setAverage { } \
 {
   # Set the meter engine
   after idle "RFError  average  $::RFError::State(average)  -nowait"
    
   return
 }


#****************************************************************************
# Name:        query_rfctr_state
# Parameters:  none
# Description: This function sends down a request to the RF Error engine to 
#            : obtain the engine state so displays will be correct
#****************************************************************************
 proc query_rfctr_state { } {
   set ::RFError::State(engineId) [format_version [RFError ver 0] ]
   set ::RFError::State(driverId) [format_version [RFError ver 1] ]
 }

#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the FM Deviation engine to its current state
#****************************************************************************
 proc getSaveState { } {
   variable State
   
   set returnList \
   {
     RFError run    $State(Engine_run)     -nowait
     RFError gate   $State(Engine_gate)    -nowait
     RFError avg    $State(average)        -nowait
     RFError en     $State(Engine_enabled) -nowait
     RFError peak   $State(peakEnable)     -nowait
     RFError alerts $State(alerts)         -nowait
     RFError ulim   $State(alertHi)        -nowait
     RFError llim   $State(alertLo)        -nowait

     set ::RFError::State(BellEnabled)   {$State(BellEnabled)}
     set ::RFError::State(resolution)    {$State(resolution)}
     set ::RFError::State(units)         {$State(units)}
   }

   return [subst $returnList]
 }


#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to hardware
#****************************************************************************
 proc Sync { } {
    eval [getSaveState]
 }

#****************************************************************************
# Name:        DefaultSetup
# Parameters:  none
# Description: Sets the RF Error engine with default values
#****************************************************************************
 proc DefaultSetup { } {
    RFError gate TenthSec       ;# Set to get 10 readings a second
 }

#****************************************************************************
# Name:        QuickStart
# Parameters:  none
# Description: Turns on the RF Error engine but does not set up parameters 
#****************************************************************************
 proc QuickStart { } {
    RFError en 3                ;# Enable data to engine
    RFError run ON              ;# Start the engine
 }

#****************************************************************************
# Name:        QuickStop
# Parameters:  none
# Description: Handles a normal stop of the RF Error engine 
#****************************************************************************
 proc QuickStop { } {
    RFError en 1                ;# Disable data to engine
    RFError run OFF             ;# Stop the engine
 }

#****************************************************************************
# Name:        SyncAll
# Parameters:  none
# Description: Insures that TCL, the engine and hardware are in sync
#****************************************************************************
 proc SyncAll { } {
    set_rfctr_callbacks
    query_rfctr_state
    if ![interp issafe] \
    {
       after 100 {
       ::RFError::Sync
       RFError run on
       RFError en on
       }
    }
 }

#****************************************************************************
# Name:        Get_HTML_reading
# Parameters:  base_url - the base URL for the containing page
# Description: Return an string containing HTML showing the current state
#              of this system
#****************************************************************************
 proc Get_HTML_reading { {base_url ""} } \
 {
    variable State
    set x \
{<tr>
<td nowrap>Received frequency error</td><td nowrap>$State(reading_cond_Hz) Hz</td>
</tr>}
    return [subst $x]
 }
 
#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
   Load::on_system_done { ::RFError::SyncAll }

}  ;# End of the RF Error namespace

#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines RFError
}

#****************************************************************************
# This registers the engine with the list for generating an HTML page
#****************************************************************************
namespace eval ::HTTPD \
{
   variable main_screen_readings
   lappend  main_screen_readings(1) RFError
}
#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
      Rff_Eng   "RF Freq Error Engine"
      Rff_Drv   "RF Freq Error Driver"
   }
   set version(Rff_Eng)   [RFError version 0]
   set version(Rff_Drv)   [RFError version 1]
}


