#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2002      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : scope.tcl                                                   #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Hieu Tran                                                   #
# DESCRIPTION : This file is sourced by TCL when the Scope command is first #
#             : used.  It loads the C++ interface file, provides a          #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::Scope::State ruw "::Scope::Load"

set Scope::State(init) "started"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
if {"[info command Scope]" != "" } { return }

# Call the base class dummy command to force a load
package require Engtclbase
Engtclbase

# Load the object file
::load $::Scope::object

package provide Scope 1.0

#****************************************************************************
# Scope namespace variables
#****************************************************************************
namespace eval ::Scope {
   variable SaveSortOrder AFTER(BBAudio)
   # This data is updated by the engines through callback routines, and it
   # indicates (in a screen format) the last setting received from an engine

   array set trigtype_map { \
      0 "AUTO"  \
      1 "NORM"  \
      2 "ONE"   \
      "AUTO" 0  \
      "NORM" 1  \
      "ONE"  2  \
   }

   array set trigsource_map { \
      0 "Ext"   \
      1 "CH1"   \
      2 "CH2"   \
      "Ext" 0   \
      "CH1" 1   \
      "CH2" 2   \
   }

   array set sweep_map { \
      10       "1 us"   \
      20       "2 us"   \
      50       "5 us"   \
      100      "10 us"  \
      200      "20 us"  \
      500      "50 us"  \
      1000     "100 us" \
      2000     "200 us" \
      5000     "500 us" \
      10000    "1 ms"   \
      20000    "2 ms"   \
      50000    "5 ms"   \
      100000   "10 ms"  \
      200000   "20 ms"  \
      500000   "50 ms"  \
      1000000  "100 ms" \
      2000000  "200 ms" \
      5000000  "500 ms" \
      10000000 "1 s"    \
      "1 us"   10       \
      "2 us"   20       \
      "5 us"   50       \
      "10 us"  100      \
      "20 us"  200      \
      "50 us"  500      \
      "100 us" 1000     \
      "200 us" 2000     \
      "500 us" 5000     \
      "1 ms"   10000    \
      "2 ms"   20000    \
      "5 ms"   50000    \
      "10 ms"  100000   \
      "20 ms"  200000   \
      "50 ms"  500000   \
      "100 ms" 1000000  \
      "200 ms" 2000000  \
      "500 ms" 5000000  \
      "1 s"    10000000 \
   }

   variable coupling_strings

   array set coupling_strings \
   {
     0 AC
     1 DC
     2 GND
   }
   
#****************************************************************************
# Name:        set_scope_callbacks
# Parameters:  none
# Description: Sets up the callbacks to update the Scope data
#****************************************************************************
 proc set_scope_callbacks { } \
 {
       Scope -onchange -structure marker0  { ::Scope::updateMarker %L %V}
       Scope -onchange coupling0    { ::Scope::updateCoupling %L %V}
       Scope -onchange sweep0       { ::Scope::updateSweep %L %V}
       Scope -onchange trigpol0     { ::Scope::updateTrigpol %L %V}
       Scope -onchange trigtype0    { ::Scope::updateTrigtype %L %V}
       Scope -onchange trigsource0  { ::Scope::updateTrigsource %L %V}
       Scope -onchange status0      { ::Scope::updateStatus %L %V}
       Scope -onchange triglev0     { set ::Scope::State(triglev,%L) %V}
       Scope -onchange position0    { set ::Scope::State(hposition,%L) %V}
       Scope -onchange scale0       { set ::Scope::State(scale,%L) %V}
       Scope -onchange enable0      { set ::Scope::State(enable,%L) %V}
       Scope -onchange run0         { set ::Scope::State(run,%L) %V}

 }

#****************************************************************************
# Name:        updateMarker
# Parameters:  lun, marker data
# Description: updates variables based on the marker info
#****************************************************************************
 proc updateMarker { lun data } \
 {
    variable State
    set marker [lindex $data 0]
    set time    [lindex $data 1]
    set pos    [lindex $data 2]
    set levels [lindex $data 3]
    set live   [lindex $levels 0]
    set State(marker,$lun,$marker,pos)  $pos
    set State(marker,$lun,$marker,time)  $time
    set State(marker,$lun,$marker,live) $live
 }
 
#****************************************************************************
# Name:        updateCoupling
# Parameters:  lun, coupling
# Description: updates variables based on the status bits 
#****************************************************************************
 proc updateCoupling { lun coupling } \
 {
    variable State
    variable coupling_strings

    set State(coupling,$lun,raw) $coupling

    if [info exists coupling_strings($coupling)] \
    {
       set State(coupling,$lun) $coupling_strings($coupling)
    }
 }
 
#****************************************************************************
# Name:        updateStatus
# Parameters:  newStatus        New status info from the scope
# Description: updates variables based on the status bits 
#****************************************************************************
 proc updateStatus { lun newStatus } \
 {
    Debug::Puts Scope 5 "updateStatus:  [format %06x $newStatus]"
    
    variable State
    
    set State(status,$lun) $newStatus
    
    if {$lun == 3} { return [updateStatus3 $newStatus] }
    
    if {$newStatus == -1} \
    {
       # updates are happening too fast
       set State(trig_state,$lun)   "RUN"
       return
    }
    
    set state_string "DONE"
    
    if { $newStatus & 0x100 } \
    {
        set state_string   "PRE"
    }
        
    if { $newStatus & 0x200 } \
    {
        set state_string   "ARM"
    }
        
    if { $newStatus & 0x400  } \
    {
        set state_string   "TRIG'D"
    }
    if {$state_string != ""} \
    {
       set State(trig_state,$lun)  $state_string
    }
 }
#****************************************************************************
# Name:        updateStatus3
# Parameters:  newStatus        New status info from the scope
# Description: updates variables based on the status bits 
#****************************************************************************
 proc updateStatus3 { newStatus } \
 {
    Debug::Puts Scope 10 "updateStatus3:  $newStatus"

    variable State
    
    if {$newStatus == -1} \
    {
       # updates are happening too fast
       set State(trig_state,3)   "RUN"
       return
    }
    
    set State(trigstate3)  $newStatus
    
    set newStatus  [expr $newStatus & 0x7]
        
    switch $newStatus \
    {
      "0"  { set State(trig_state,3)   "DONE"
           }
           
      "1"  { set State(trig_state,3)   "PRE"
           }

      "2"  { set State(trig_state,3)   "ARM"
           }

      "3"  {set State(trig_state,3)   "TRIG'D"
           }

      "4"  {set State(trig_state,3)   "DONE"
           }
    }
    
    Debug::Puts Scope 10 "  State(trig_state):  $State(trig_state)"
    
    return    
 }

#****************************************************************************
# Name:        updateSweep
# Parameters:  lun  - which trigger type is affected
# Description: update sweep rate state variable 
#****************************************************************************
 proc updateSweep { lun rate } \
 {
    variable State

    set State(sweep,$lun,raw) $rate

    if [info exists ::Scope::sweep_map($rate)] {
       set State(sweep,$lun) $::Scope::sweep_map($rate)
    }
 }

#****************************************************************************
# Name:        Sweep
# Parameters:  rate - sweep rate 
# Description: Set scope sweep rate based on the string value 
#****************************************************************************
 proc Sweep { rate {lun ""}} \
 {
   if ![info exists ::Scope::sweep_map($rate)] { return }
   if {$lun == ""} \
   {
      Scope sweep3 $::Scope::sweep_map($rate) -nowait
      Scope sweep1 $::Scope::sweep_map($rate) -nowait
      return
   }
   Scope sweep$lun $::Scope::sweep_map($rate) -nowait
 }

#****************************************************************************
# Name:        updateTrigpol
# Parameters:  state 0 = FALL, 1 = RISE
# Description: update trigger slope polarity state variable 
#****************************************************************************
 proc updateTrigpol { lun state } \
 {
    variable State

    set State(trigpol,$lun,raw) $state

    switch $state {
       0       { set ::Scope::State(trigpol,$lun) FALL }
       1       { set ::Scope::State(trigpol,$lun) RISE }
       default { return }
    }
 } 

#****************************************************************************
# Name:        updateTrigtype
# Parameters:  type - engine code for type 
# Description: update trigger type state variable 
#****************************************************************************
 proc updateTrigtype { lun type } \
 {
   variable State

   set State(trigtype,$lun) $type
   
   if [info exists ::Scope::trigtype_map($type)] \
   {
      set State(trigtype) $::Scope::trigtype_map($type)
   }
   switch $type \
   {
      0 -
      1 \
      {
         set State(trig_state) "RUN"
      }
      2 \
      {
         set State(trig_state) "ARM"
      }
   }
 }
#****************************************************************************
# Name:        updateTrigsource
# Parameters:  lun    - which trigger type is affected
#              source - engine code for trig. source 
# Description: update trigger source state variable 
#****************************************************************************
 proc updateTrigsource { lun source } \
 {
   if [info exists ::Scope::trigsource_map($source)] {
      set ::Scope::State(trigsource,$lun) $::Scope::trigsource_map($source)
   }
 }

#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the Scope engine to its current state
#****************************************************************************
 proc getSaveState { } \
 {
   variable State

   set returnList \
   {
       Scope sweep$i      {$State(sweep,$i,raw)}  -nowait
       Scope trigpol$i    {$State(trigpol,$i,raw)}    -nowait
       Scope trigtype$i   {$State(trigtype,$i)}   -nowait
       Scope trigsource$i {$State(trigsource,$i)} -nowait
       Scope triglev$i    {$State(triglev,$i)}    -nowait
       Scope coupling$i   {$State(coupling,$i,raw)} -nowait
       Scope position$i   {$State(hposition,$i)}   -nowait
       Scope scale$i      {$State(scale,$i)}      -nowait
       Scope enable$i     {$State(enable,$i)}     -nowait
       Scope run$i        {$State(run,$i)}        -nowait
   }
   foreach i { 1 2 3 } \
   {
      append rv [subst -nocommands $returnList]
   }
   
   # save the markers
   append rv "#save markers \ncatch {\n"
   foreach name [lsort [array names State marker,*,*,time]] \
   {
      regexp "marker,(.*),(.*),time" $name foo lun marker
      append rv "Scope marker$lun $marker $State($name) 0.0 -nowait\n"
   }
   append rv "}"
   return $rv
 }

 set Scope::State(init) "loaded"
#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
    set_scope_callbacks
    if ![interp issafe] \
    {
       variable path
       source [file join $path master.tcl]
    }
 set Scope::State(init) "yes"
}  ;# End of the Scope namespace
