#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : ber.tcl                                                     #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the Ber command is         #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete  ::Ber::State    ruw  "::Ber::Load"
trace vdelete  ::Ber::Sources  ruw  "::Ber::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
# Call the base class dummy command to force a load
if {"[info command Ber]" != "" } { return }
package require Engtclbase
Engtclbase

# Load the object file
::load $::Ber::object

package provide Ber 1.0

#****************************************************************************
# Add functions to the BER namespace
#****************************************************************************
#    Status of commands sent to BER engine
#       ::Ber::State(Engine_run)          {0 | 1}
#       ::Ber::State(run)                 {OFF | ON}
#       ::Ber::State(Engine_enabled)      {0 | 1}
#       ::Ber::State(enabled)             {OFF | ON}
#       ::Ber::State(driverId)            Driver ID information
#       ::Ber::State(engineId)            Engine ID information
#       ::Ber::State(numFrames)       Number of frames to accumulate errors over
#    Status of counter data received from the engine
#       ::Ber::State(total_bits)          Total accumulated bit count
#       ::Ber::State(total_error_bits)    Total accumulated bit errors
#       ::Ber::State(total_frames)        Total accumulated frame count
#       ::Ber::State(total_error_frames)  Total accumualted frame errors
#       ::Ber::State(ber_raw)             Current BER rate value from the engine
#       ::Ber::State(ber_cond)            Formatted BER rate value 
#       ::Ber::State(ber_percent)         Current BER rate value multiplied by 100.0
#       ::Ber::State(fer_raw)             Current FER rate value from the engine
#       ::Ber::State(fer_cond)            Formatted FER rate value 
#       ::Ber::State(fer_percent)         Current FER rate value multiplied by 100.0
#****************************************************************************
namespace eval ::Ber \
{
   set State(Source) ::Ber::DEMOD
   
   array set stdPatterns \
   {
      0  "STD 1011"
      1  "STD CAL"
      2  "STD SILENCE"
      3  "STD INTERFERENCE"
      4  "STD BUSY"
      5  "STD IDLE"
      6  "STD 511"

      "STD 1011"	0
      "STD CAL"		1
      "STD SILENCE"	2
      "STD INTERFERENCE" 3
      "STD BUSY"	4
      "STD IDLE" 5
      "STD 511" 6
   }

   array set stdPatterns_short \
   {
      0  "S 1011"
      1  "S CAL"
      2  "S SILNC"
      3  "S ITNF"
      4  "S BUSY"
      5  "S IDLE"
      6  "S 511"

      "S 1011"   0
      "S CAL"    1
      "S SILNC"  2
      "S INTF"   3
      "S BUSY"   4
      "S IDLE"   5
      "S 511"    6
   }
   
#****************************************************************************
# Name:        set_ber_callbacks
# Parameters:  none
# Description: Sets up the callbacks to update the BER  data
#****************************************************************************
 proc set_ber_callbacks { } \
 {
   if { $::Ber::callbackStatus != 0 } \
   {
     return
   }

   Ber -onchange  reading   { ::Ber::updateReading  %V}
   Ber -onchange  run       { ::Ber::updateMode     %V}
   Ber -onchange  en        { ::Ber::updateEnable   %V}
   Ber -onchange  frames    { ::Ber::updateFrames   %V}
   Ber -onchange  pattern   { ::Ber::updatePattern  %V}
   Ber -onchange  readpeak  { ::Ber::updatePeaks %V}

   Ber -onchange  alerts    { ::Ber::updateAlerts     %V}
   Ber -onchange  peak      { ::Ber::updatePeakEnable %V}
   Ber -onchange  ulimit    { ::Ber::updateAlertHi    %V}
   Ber -onchange  llimit    { ::Ber::updateAlertLo    %V}

   set ::Ber::callbackStatus  1

   return
 }

#****************************************************************************
# Name:        SetFrames
# Parameters:  none
# Description: Set the number of frames
#****************************************************************************
 proc SetFrames { } \
 {
    variable State
    Ber frames $State(numFrames) -nowait
 }
#****************************************************************************
# Name:        updateFrames
# Parameters:  frameList         Data returned by the "Ber frames" command
# Description: Saves the state of the Ber pattern and number of frames 
#****************************************************************************
 proc updateFrames { frames } \
 {
   variable State

   set State(numFrames)	 $frames
   set bits 1728
   if {$State(test_pattern,raw) == 6} \
   {
     set bits 511
   }
   set time [expr $State(numFrames) * $bits/9600.0]
   set time_ms [expr $time * 1000.0]
   if { $time_ms < 1000 } \
   {
     set State(sample_time_units)  "ms"
     
     set State(sampleTime)  [format  "%.0f" $time_ms]
   } \
   else \
   {
     set State(sample_time_units)  "s"
     
     set State(sampleTime)  [format  "%.3f"  $time]
   }
 }
 
#****************************************************************************
# Name:        updatePattern
# Parameters:  pattern - test pattern setting
# Description: Saves the state of the Ber pattern
#****************************************************************************
 proc validatePattern { } \
 { 
    # stub - replaced in master
 }
 proc updatePattern { pattern } \
 {
   variable State
   variable stdPatterns
   variable stdPatterns_short

   set State(test_pattern,raw)  $pattern

   if [info exists stdPatterns($pattern)] \
   {
     set State(test_pattern)  $stdPatterns($pattern)
   }

   # Short pattern names for the small BER meter tile
   if [info exists  stdPatterns_short($pattern)] \
   {
     set State(test_pattern,short)  $stdPatterns_short($pattern)
   }
   validatePattern
   updateFrames $State(numFrames)
 }


#****************************************************************************
# Name:        updateMode
# Parameters:  runState         0 = Off, 1 = On
# Description: Sets up the current run state of the BER 
#****************************************************************************
 proc updateMode { runState } \
 {
   switch $runState \
   {
     0        { set ::Ber::State(run)  OFF }
      
     1        { set ::Ber::State(run)  ON }
      
     default  { return }
   }
   
   set ::Ber::State(Engine_run)  $runState
   
   return
 }


#****************************************************************************
# Name:        updateEnable
# Parameters:  enableBits       1 = Off, 3 = Enabled
# Description: Sets up the current enabled state of the BER 
#****************************************************************************
 proc updateEnable { enableBits } \
 {
   switch $enableBits \
   {
     3       { set ::Ber::State(enabled)  Enabled }
   
     default { set ::Ber::State(enabled)  Disabled }
   }
   
   set ::Ber::State(Engine_enabled)  $enableBits
   
   return
 }


#****************************************************************************
# Name:        updateReading
# Parameters:  count            Frequency count from the engine
#              count_precision  Current digits of accuracy
#              status           Status flags
#              count_accuracy   Current MAXIMUM digits of accuracy
# Description: Sets up the current state of the BER  reading
#****************************************************************************
 proc updateReading { readingList } \
 {
   variable  State
   
   set State(ber_raw)             [lindex $readingList 0]
   set State(fer_raw)             [lindex $readingList 1]
   set State(total_bits)          [lindex $readingList 2]
   set State(total_error_bits)    [lindex $readingList 3]
   set State(total_frames)        [lindex $readingList 4]
   set State(total_error_frames)  [lindex $readingList 5]
   set State(ber_avg)             [lindex $readingList 6]
   set State(precision)           [lindex $readingList 7]
   set State(status)              [lindex $readingList 8]
   set State(ber_avg_prec)        [lindex $readingList 9]
   
   # Get the data from the meter reading
   set State(ber_cond)     [format "%.2f" $State(ber_raw)]
   set State(ber_percent)  [format "%.1f" [expr $State(ber_raw) * 100.0]]
   
   set State(fer_cond)     [format "%.2f" $State(fer_raw)]
   set State(fer_percent)  [format "%.1f" [expr $State(fer_raw) * 100.0]]

   set State(ber_avgpercent)  [format "%.1f" [expr $State(ber_avg) * 100.0]]

   set State(precision,per)  [expr $State(precision) - 2]
   set State(precision,ppm)  [expr $State(precision) - 6]

   set State(ber_percent)  [format "%.*f" [expr $State(ber_avg_prec)-2] [expr $State(ber_avg) * 100.0]     ]
   set State(ber_ppm)      [format "%.*f" [expr $State(ber_avg_prec)-6] [expr $State(ber_avg) * 1000000.0] ]

   set State(rawber_percent)  [format "%.*f" [expr $State(ber_avg_prec)-2] [expr $State(ber_raw) * 100.0]     ]
   set State(rawber_ppm)      [format "%.*f" [expr $State(ber_avg_prec)-6] [expr $State(ber_raw) * 1000000.0] ]

   return
 }


#****************************************************************************
# Name:        updatePeaks
# Parameters:  readingList      List of data from the engine
#                 Peak HI       Max peak value
#                 HI Precision  Max peak precision
#                 Peak LO       Min peak value
#                 LO Precision  Min peak precision
#                 Flags         Status flags
# Description: Sets up the current state of the peak readings
#****************************************************************************
 proc updatePeaks { readingList } \
 {
   variable  State

   # Get the data from the peak readings
   set peakHi  [lindex $readingList 0]
   set peakLo  [lindex $readingList 2]

   set State(peakFlags) [lindex $readingList 4]

   set State(precision,per)  [expr $State(precision) - 2]
   set State(precision,ppm)  [expr $State(precision) - 6]

   set State(peakHi,per)  [format "%0.*f" $State(precision,per) [expr $peakHi * 100.0] ]

   # Convert the percent values from the engine to ppm
   set State(peakHi,ppm)  [format "%0.*f" $State(precision,ppm) [expr $peakHi * 1000000.0]]

   set State(peakLo,per)  [format "%0.*f" $State(precision,per) [expr $peakLo * 100.0] ]

   # Convert the percent values from the engine to ppm
   set State(peakLo,ppm)  [format "%0.*f" $State(precision,ppm) [expr $peakLo * 1000000.0]]
   
   return
 }


#****************************************************************************
# Name:        updatePeakEnable
# Parameters:  
# Description: Sets up the current state of the peak enable
#****************************************************************************
 proc updatePeakEnable { peakEnable } \
 {
   variable  State

   set State(peakEnable)  $peakEnable

   switch -- $State(peakEnable) \
   {
     1  { set State(peakEnable_text)  "ON"  }

     0  { set State(peakEnable_text)  "OFF" }
   }
   
   return
 }


#****************************************************************************
# Name:        updateAlerts
# Parameters:  
# Description: Update the Alerts flags
#****************************************************************************
 proc updateAlerts { alerts } \
 {
   variable State
 
   set State(alerts)  $alerts
   
   if {$alerts & 2} \
   {
     set State(alertHi_text) "ON"
   } \
   else \
   {
     set State(alertHi_text) "OFF"
   }
   
   if {$alerts & 1} \
   {
     set State(alertLo_text) "ON"
   } \
   else \
   {
     set State(alertLo_text) "OFF"
   }
   
   if {$alerts & 4} \
   {
     set State(BellEnabled,text)  "ON"
     set State(BellEnabled)       1
   } \
   else \
   {
     set State(BellEnabled,text)  "OFF"
     set State(BellEnabled)       0
   }
 }


#****************************************************************************
# Name:        updateAlertHi
# Parameters:  
# Description: Formats the Alert High value
#****************************************************************************
 proc updateAlertHi { alertHi } \
 {
   variable  State
   set State(alertHi,raw) $alertHi
   set State(alertHi,per)  [format "%.*f" [expr $State(precision)-2] [expr $alertHi * 100.0]]
   # Set the 'ppm' limit to 'per' limit times 10000
   set State(alertHi,ppm)  [format "%.*f" [expr $State(precision)-6] [expr $alertHi * 10000.0]]
   return
 }

#****************************************************************************
# Name:        updateAlertLo
# Parameters:  
# Description: Formats the Alert Low value
#****************************************************************************
 proc updateAlertLo { alertLo } \
 {
   variable  State
   set State(alertLo,raw) $alertLo
   set State(alertLo,per)  [format "%.*f" [expr $State(precision)-2] [expr $alertLo * 100.0]]

   # Set the 'ppm' limit to 'per' limit times 10000
   set State(alertLo,ppm)  [format "%.*f" [expr $State(precision)-6] [expr $alertLo * 10000.0]]
   
   return
 }

#********************************************************************
#
#        Name: setLimits
#
#  Parameters:
#
# Description: Sets the limit values
#
#********************************************************************
proc setHiLimit {  } \
{
  variable  State
  
  set temp  [expr $State(alerts) & 0x02]
    
  # If the limit alert is turned OFF, turn it ON
  if { $temp == 0 } \
  {
    set temp  [expr $State(alerts) | 0x02]
      
    Ber alerts  $temp  -nowait
  }
    
  switch -- $::Ber::State(units) \
  {
    "%"    { Ber ulimit [expr $State(alertHi,per)/100.0]  -nowait                  }

    "ppm"  { # Convert the percent values from the engine to ppm
             Ber ulimit [expr $State(alertHi,ppm) / 1000000.0]  -nowait }
  }
}
proc setLowLimit { } \
{
  variable  State
  
  set temp  [expr $State(alerts) & 0x01]
    
  # If the limit alert is turned OFF, turn it ON
  if { $temp == 0 } \
  {
    set temp  [expr $State(alerts) | 0x01]
      
    Ber alerts  $temp  -nowait
  }
    
  switch -- $::Ber::State(units) \
  {
    "%"    { Ber llimit  [expr $State(alertLo,per)/100.0]  -nowait                  }

    "ppm"  { # Convert the percent values from the engine to ppm
             Ber llimit  [expr $State(alertLo,ppm) / 1000000.0]  -nowait }
  }
}

#********************************************************************
#
#        Name: toggleAlerts
#
#  Parameters:
#
# Description: Toggles the state of the alerts
#
#********************************************************************
proc toggleHiAlert { } \
 {
   variable  State
   
   set new  [expr $State(alerts) ^ 2]
   
   Ber alerts  $new  -nowait
 }
 proc toggleLoAlert { } \
 {
   variable  State
   
   set new  [expr $State(alerts) ^ 1]
   
   Ber alerts  $new  -nowait
 }


#********************************************************************
#
#        Name: toggleAlarm
#
#  Parameters:
#
# Description: Toggles the state of the alarm
#
#********************************************************************
proc toggleAlarm { } \
 {
   variable State
   
   Ber alerts [expr $State(alerts) ^ 4] -nowait
 }


#********************************************************************
#
#        Name: togglePeakEnable
#
#  Parameters: 
#
# Description: 
#
#********************************************************************
proc togglePeakEnable { } \
 {
   variable  State

   switch -- $State(peakEnable) \
   {
     1  { Ber peak  0  -nowait }
     
     0  { after idle "Ber phreset  -nowait"
          after idle "Ber peak 1   -nowait" }
   }

   return    
 }


#********************************************************************
#
#        Name: resetTotals
#
#  Parameters: 
#
# Description: 
#
#********************************************************************
proc resetTotals { } \
 {
   variable  State

   Ber reseterrors
   
   return    
 }


#********************************************************************
#
#        Name: setTestPattern
#
#  Parameters: 
#
# Description: 
#
#********************************************************************
proc setTestPattern { } \
 {
   variable  State
   variable  stdPatterns
   
   if { [info exists  stdPatterns($State(test_pattern))] } \
   {
     set temp  $stdPatterns($State(test_pattern))
     
     Ber pattern  $temp  -nowait
   } \
   else \
   {
      Ber pattern -nowait
   }
 }

#********************************************************************
#
#        Name: toggleUnits
#
#  Parameters: 
#
# Description: Toggles the units between '%' and 'ppm'
#
#********************************************************************
proc toggleUnits { } \
 {
   variable  State
   
   if { $State(units) == "%" } \
   {
     set State(units)  "ppm"
   } \
   else \
   {
     set State(units)  "%"
   }
   
   # Update the limits
   switch -- $State(units) \
   {
     "%"    { # Limits
              Ber ulimit  $State(alertHi,per)  -nowait
              Ber llimit  $State(alertLo,per)  -nowait 
            }
              

     "ppm"  { # Limits
              # Convert the percent values from the engine to ppm
              Ber ulimit  [expr $State(alertHi,ppm) / 10000.0]  -nowait
              Ber llimit  [expr $State(alertLo,ppm) / 10000.0]  -nowait 
            }
   }
   
   return
 }


#****************************************************************************
# Name:        QuickStart
# Parameters:  none
# Description: Turns on the BER Meter engine but does not set up parameters 
#****************************************************************************
 proc QuickStart { } \
 {
   Ber en  3 -nowait
   Ber run 1 -nowait
 }


#****************************************************************************
# Name:        QuickStop
# Parameters:  none
# Description: Turns off the BER Meter engine but does not set up parameters 
#****************************************************************************
 proc QuickStop { } \
 {

   return
 }


#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the BER engine to its current state
#****************************************************************************
 proc getSaveState { } \
 {
   variable State
 
   set returnList  \
   {
     Ber run      $State(Engine_run) -nowait
     Ber frames   $State(numFrames) -nowait
     Ber pattern  $State(test_pattern,raw) -nowait
     Ber en       $State(Engine_enabled) -nowait
     
     Ber ulim   {$State(alertHi,raw)} -nowait
     Ber llim   {$State(alertLo,raw)} -nowait
     Ber alerts {$State(alerts)}      -nowait
     Ber peak   {$State(peakEnable)}  -nowait

     ::Ber::setPatternSource $State(Pattern)
   }
 
   return [subst -nocommands $returnList]
 }

#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
   set_ber_callbacks
   set State(engineId)  [format_version [Ber ver 0] ]
   set State(driverId)  [format_version [Ber ver 1] ]

   if ![interp issafe] { source [file join  $::Ber::path  master.tcl]}


#********************************************************************
#
#        Name: setPatternSource
#
#  Parameters: patt - pattern namespace
#
# Description: Sets the Pattern source
#
#********************************************************************
proc setPatternSource { patt } \
{
   Debug::Puts BER 3 "setPatternSource:  $patt"
   
   variable State
   if [info exists ::Ber::${patt}::ShortName] \
   {
      set patt ::Ber::${patt}
   }
   if [info exists ${patt}::ShortName] \
   {
      upvar ${patt}::ShortName name
   } \
   else \
   {
      set patt ::Ber::DEMOD
   }
   
   set State(pattern_source)  $name
   
   # Toss the command to our parent if we are a slave
   set State(Pattern)  [::Ber::source_core $patt]
}


};  # End of the BER namespace

#****************************************************************************
# Define the option namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::RX_BER \
{
  variable InfoString

  set RxBer_version [string trimleft [Ber ver 0] "BerMtr"]
  set InfoString [subst \
  {
         <br>
         (using BER system version - $RxBer_version)
         <br>
         This option extends the Bit Error Rate (BER) meter to allow it to
         receive direct input from the radio through the front panel test port
         of the 2975.
  }]
}
