#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001, 2002   DESIGN ENGINEERING     #
# --------------------------------------------------------------------------#
# FILE        : dcs.tcl                                                     #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the Dcs command is         #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
# Call the base class dummy command to force a load
if {"[info command Dcs]" != "" } { return }

package require Engtclbase

Engtclbase

package require Dcs

# Load the object file
::load $::Dcs::object

package provide Dcs 1.0


namespace eval ::Dcs \
{
  #****************************************************************************
  #
  # Name:        set_dcs_callbacks
  #
  # Parameters:  none
  #
  # Description: Sets up the callbacks to update the DTMF decode data
  #
  #****************************************************************************
  proc set_dcs_callbacks { } \
  {
    #puts "set_dcs_callbacks"

    if { $::Dcs::callbackStatus != 0 } \
    {
      return
    }

    Dcs -onchange run      { ::Dcs::updateMode %V; }
    Dcs -onchange en       { ::Dcs::updateEnable %V; }
    Dcs -onchange reading  { ::Dcs::updateDecodeData %V; }

    set ::Dcs::callbackStatus  1
  
    return
  }


  #****************************************************************************
  #
  # Name:        updateMode
  #
  # Parameters:  runState         0 = Off, 1 = On
  #
  # Description: Sets up the current run state of the DCS decode
  #
  #****************************************************************************
  proc updateMode { runState } \
  {
    variable  State

    switch $runState \
    {
      0       { set State(run)  "OFF" }
      
      1       { set State(run)  "ON"  }
      
      default { return }
    }
   
    set State(Engine_run)  $runState
   
    return
  }


  #****************************************************************************
  #
  # Name:        updateEnable
  #
  # Parameters:  enableBits       1 = Off, 3 = Enabled
  #
  # Description: Sets up the current enabled state of the DCS decode
  #
  #****************************************************************************
  proc updateEnable { enableBits } \
  {
    variable  State

    switch $enableBits \
    {
      3       { set State(enabled)  "Enabled"  }
      
      default { set State(enabled)  "Disabled" }
    }
   
    set State(Engine_enabled)  $enableBits
   
    return
  }


  #****************************************************************************
  #
  # Name:        updateDecodeData
  #
  # Parameters:  decode_data:
  #              (0) dcs_decode - value representing the complete
  #                               DCS decoding
  #              (1) dcs_decode - value representing the complete
  #                               DCS decoding
  #              (2) status - status of the current decoded digit
  #                           0 - valid
  #                           1 - not valid
  #                           2 - not valid
  #
  # Description: Gets the decoded data
  #
  #****************************************************************************
  proc updateDecodeData { decode_data } \
  { 
    #puts "updateDecodeData:  $decode_data"
    
    variable  State
    
    # Get the data from the decoded data
    set State(dcs_decode)  [lindex $decode_data  0]
    set State(status)      [lindex $decode_data  2]
    
    if { ![interp issafe] } \
    {
      # Not defined
    }
    
    switch $State(status) \
    {
      0  { set State(status_text)  "OK"            }
      
      1  { set State(status_text)  "Fixed"         }

      2  { set State(status_text)  "Could not fix" }

      default  { }
    }
        
    if { $State(status) == 0 } \
    {
      # Get the DCS code received from the decode
      set State(dcs_received)  [expr $State(dcs_decode) & 0777]
      
      # Convert to octal
      set State(dcs_received)  [format "%03o"  $State(dcs_received)]

      # Get the alternate DCS code
      # If the value received exists in the alternate array,
      # then the value received is an alternate
      # else set the DCS code to the value received
      if { [info exists ::DCS::alternate_codes($State(dcs_received))] } \
      {
        # Set the actual DCS code to the alternate value
        set State(dcs_code)  $::DCS::alternate_codes($State(dcs_received))
        
        # Set the indicator to show that the received is an alternate
        set State(dcs_alt_indicator)  "ALT"
      } \
      else \
      {
        # Set the actual DCS code to the value received
        set State(dcs_code)  $State(dcs_received)

        # Clear the indicator to show that the received is an alternate
        set State(dcs_alt_indicator)  ""
      }

      # Get the invert DCS code
      if { [info exists ::DCS::invert_codes($State(dcs_code))] } \
      {
        set State(dcs_invert_code)  $::DCS::invert_codes($State(dcs_code))
      } \
      else \
      {
        set State(dcs_invert_code)  ""
      }
    }
    
    return
  }


  #**************************************************************************
  #
  #       Name: clear
  #
  # Parameters:
  #
  #Description: Clears the existing data and the status
  #
  #**************************************************************************
  proc clear { } \
  {
    #puts "clear"
    
    variable  State
    
    set State(dcs_received)  ""
    set State(dcs_code)      ""
     
    set State(dcs_alt_indicator)  ""
    set State(dcs_invert_code)    ""
     
    set State(status)       ""
    set State(status_text)  ""
    
    return
  }
  
    
  #****************************************************************************
  #
  # Name:        query_dcs_state
  #
  # Parameters:  none
  #
  # Description: This function sends down a request to the DTMF decode 
  #              engine to obtain the engine state 
  #****************************************************************************
  proc query_dcs_state { } \
  {
    #puts "query_dcs_state"
    
    variable  State
    
    set State(engineId) [format_version [Dcs ver 0]]
    set State(driverId) [format_version [Dcs ver 1]]
    
    return
  }


  #****************************************************************************
  #
  # Name:        getSaveState
  #
  # Parameters:  none
  #
  # Description: This function returns a list of TCL commands which can be used
  #              to set the DTMF decode engine to its current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    variable State
 
    set returnList \
    {
 
    }
       
    return [subst -nocommands $returnList]
  }


  #****************************************************************************
  #
  # Name:        Sync
  #
  # Parameters:  none
  #
  # Description: Write state variables to hardware
  #
  #****************************************************************************
  proc Sync { } \
  {
    return
  }


  #****************************************************************************
  #
  # Name:        DefaultSetup
  #
  # Parameters:  none
  #
  # Description: Sets the AFCounter engine with default values
  #
  #****************************************************************************
  proc DefaultSetup { } \
  {
    #puts "DefaultSetup"

    return
  }


  #****************************************************************************
  #
  # Name:        SyncAll
  #
  # Parameters:  none
  #
  # Description: Insures that TCL, the engine and hardware are in sync
  #
  #****************************************************************************
  proc SyncAll { } \
  {
    set_dcs_callbacks 
    if { ![interp issafe] } \
    {
      Sync
    }
   
    return
  }

  #****************************************************************************
  # Initialize the namespace and activate the callbacks
  #****************************************************************************
  Load::on_system_done { Dcs::SyncAll }


};  # End of namespace Dcs


#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  #variable engines   
  #
  #lappend engines Dcs
}

#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
      Dcs_Eng   "DCS Engine"
      Dcs_Drv   "DCS Driver"
   }
   set version(Dcs_Eng)   [Dcs version 0]
   set version(Dcs_Drv)   [Dcs version 1]
}




