#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2003         DESIGN ENGINEERING     #
# --------------------------------------------------------------------------#
# FILE        : dtmf_decode.tcl                                             #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield/Brad Taylor                               #
# DESCRIPTION : This file is sourced by TCL when the Dcs command is         #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
# Call the base class dummy command to force a load
if {"[info command Dtmf]" != "" } { return }

package require Engtclbase

Engtclbase

package require Dtmf

# Load the object file
::load $::Dtmf::object

package provide Dtmf 1.0


namespace eval ::Dtmf \
{
  #****************************************************************************
  #
  # Name:        set_dtmf_callbacks
  #
  # Parameters:  none
  #
  # Description: Sets up the callbacks to update the DTMF decode data
  #
  #****************************************************************************
  proc set_dtmf_callbacks { } \
  {
    #puts "set_dtmf_callbacks"

    if { $::Dtmf::callbackStatus != 0 } \
    {
      return
    }

    Dtmf -onchange reading  { ::Dtmf::updateDecodeData %V; }

    set ::Dtmf::callbackStatus  1
  
    return
  }


  #****************************************************************************
  #
  # Name:        updateDecodeData
  #
  # Parameters:  decode_data:
  #              (0) status - status of the current decoded digit
  #                           0 - not valid
  #                           1 - valid
  #                           2 - repeat
  #                           3 - invalid, second harmonic incorrect
  #              (1) decoded digit decimal value
  #              (2) relative sample count
  #              (3) fundamental harmonic energy
  #              (4) highest fundamental second harmonic energy
  #
  # Description: Gets the decoded data
  #
  #****************************************************************************
  proc updateDecodeData { decode_data } \
  { 
    #puts "updateDecodeData:  $decode_data"
    
    variable  State
    
    # Get the data from the decoded data
    set State(status)         [lindex $decode_data  0]
    set State(digit_dec)      [lindex $decode_data  1]
    set State(sample_count)   [lindex $decode_data  2]
    #set State(first_energy)   [lindex $decode_data  3]
    #set State(second_energy)  [lindex $decode_data  4]
    
    if { ![interp issafe] } \
    {
      # Not defined
    }
    
    # Convert to a character
    set State(digit)  [format "%c" $State(digit_dec)]

    #puts "State(digit):  $State(digit)"

    # If the status is '1' or '3', add to the list of DTMF digits
    if { $State(status) == 1 || $State(status) == 3 } \
    {
      append  State(dtmf_received)  $State(digit)
      
      # Keep only the last 16 characters
      if { [string length $State(dtmf_received)] > 17 } \
      {
        set State(dtmf_received)  [string range $State(dtmf_received)  1  17]
      }
    }      
    
    switch $State(status) \
    {
      0  { set State(status_text)  "Invalid"          }
      
      1  { set State(status_text)  "Valid"            }

      2  { set State(status_text)  "Valid - Repeat"   }

      3  { set State(status_text)  "Valid - 2nd harm" }

      default  { }
    }
    
    return
  }


  #**************************************************************************
  #
  #       Name: clear
  #
  # Parameters:
  #
  #Description: Clears the existing data and the status
  #
  #**************************************************************************
  proc clear { } \
  {
    #puts "clear"
    
    variable  State
    
    set State(digit)          ""
    set State(dtmf_received)  ""
   
    set State(status)         ""
    set State(status_text)    ""

    return
  }
  
    
  #****************************************************************************
  #
  # Name:        query_dtmf_state
  #
  # Parameters:  none
  #
  # Description: This function sends down a request to the DTMF decode 
  #              engine to obtain the engine state 
  #****************************************************************************
  proc query_dtmf_state { } \
  {
    #puts "query_dtmf_state"
    
    variable  State
    
    set State(engineId) [format_version [Dtmf ver 0]]
    set State(driverId) [format_version [Dtmf ver 1]]
    
    return
  }


  #****************************************************************************
  #
  # Name:        getSaveState
  #
  # Parameters:  none
  #
  # Description: This function returns a list of TCL commands which can be used
  #              to set the DTMF decode engine to its current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    variable State
 
    set returnList \
    {
 
    }
       
    return [subst -nocommands $returnList]
  }


  #****************************************************************************
  #
  # Name:        Sync
  #
  # Parameters:  none
  #
  # Description: Write state variables to hardware
  #
  #****************************************************************************
  proc Sync { } \
  {
    return
  }


  #****************************************************************************
  #
  # Name:        DefaultSetup
  #
  # Parameters:  none
  #
  # Description: Sets the DTMF decode engine with default values
  #
  #****************************************************************************
  proc DefaultSetup { } \
  {
    return
  }


  #****************************************************************************
  #
  # Name:        SyncAll
  #
  # Parameters:  none
  #
  # Description: Insures that TCL, the engine and hardware are in sync
  #
  #****************************************************************************
  proc SyncAll { } \
  {
    #puts "SyncAll"

    set_dtmf_callbacks
   
    query_dtmf_state
 
    if { ![interp issafe] } \
    {
      Sync
    }
   
    return
  }

  #****************************************************************************
  # Initialize the namespace and activate the callbacks
  #****************************************************************************
  Load::on_system_done { Dtmf::SyncAll }


};  # End of namespace Dtmf


#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  #variable engines   
  #
  #lappend engines Dtmf
}


#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
     DTMF_DRV "DTMF decode driver"
     DTMF_ENG "DTMF decode engine"
   }
   set version(DTMF_DRV)  [Dtmf ver 1]
   set version(DTMF_ENG)  [Dtmf ver 0]
}

