#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : fmdev.tcl                                                   #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file is sourced by TCL when the FMDev command is       #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################
trace vdelete ::FMDev::State ruw "::FMDev::Load"

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
# Call the base class dummy command to force a load
if {"[info command FMDev]" != "" } { return }
package require Engtclbase
package require Receiver
Engtclbase

# Load the object file
::load $::FMDev::object

package provide FMDev 1.0


#****************************************************************************
# Add functions to the FM Deviation namespace
#****************************************************************************
#    Status of commands sent to FM Deviation engine
#       ::FMDev::State(Engine_run)          {0 | 1}
#       ::FMDev::State(run)                 {OFF | ON}
#       ::FMDev::State(average)             {1 - 100}
#       ::FMDev::State(Engine_enabled)      {0 | 1}
#       ::FMDev::State(enabled)             {OFF | ON}
#       ::FMDev::State(Engine_trigger)      {0 | 1}
#       ::FMDev::State(trigger)             {OFF | ON}
#       ::FMDev::State(Engine_math)         {0 | 1}
#       ::FMDev::State(math)                {PEAK | AVERAGE}
#       ::FMDev::State(driverId)            Driver ID information
#       ::FMDev::State(engineId)            Engine ID information
#    Status of counter data received from the engine
#       ::FMDev::State(pos_value)           Positive FM deviation value
#       ::FMDev::State(neg_value)           Negative FM deviation value
#       ::FMDev::State(avg_value)           Average  FM deviation value
#       ::FMDev::State(precision)           Data precision
#    Status of alert limits and peak values
#       ::FMDev::State(peakHi)              Maximum peak reading
#       ::FMDev::State(peakLo)              Minimum peak reading
#       ::FMDev::State(peakFlags)           Peak Hold status flags
#       ::FMDev::State(peakEnable)          Peak Enable settings
#       ::FMDev::State(alertHi)             Maximum valid reading limit
#       ::FMDev::State(alertLo)             Minimum valid reading limit
#       ::FMDev::State(alerts)              Alert flag status
#****************************************************************************
namespace eval ::FMDev {
   variable SaveSortOrder AFTER(Receiver)
   
   array set trigger_map { \
      CONTINUOUS  1 \
      ONE_SHOT    2 \
      1 CONTINUOUS  \
      2 ONE_SHOT    \
   }

   array set math_map { \
      PEAK    0 \
      AVERAGE 1 \
      PLUS    2 \
      MINUS   3 \
      0 PEAK    \
      1 AVERAGE \
      2 PLUS    \
      3 MINUS
   }



#****************************************************************************
# Name:        set_fmdev_callbacks
# Parameters:  none
# Description: Sets up the callbacks to update the FM Deviation data
#****************************************************************************
 proc set_fmdev_callbacks { } {
   FMDev -onchange reading { ::FMDev::updateReading %V }
   
   FMDev -onchange readpeak { ::FMDev::updatePeaks %V }
   FMDev -onchange ulimit   { ::FMDev::updateAlertHi %V }
   FMDev -onchange llimit   { ::FMDev::updateAlertLo %V }
   
   FMDev -onchange alerts   { ::FMDev::updateAlerts %V }

   FMDev -onchange peak     { ::FMDev::updatePeakEnable %V }

   FMDev -onchange run  { ::FMDev::updateMode %V }
   FMDev -onchange math { ::FMDev::updateMath %V }
   FMDev -onchange trig { ::FMDev::updateTrig %V }
   FMDev -onchange avg  { ::FMDev::updateAverage %V }
   FMDev -onchange en   { ::FMDev::updateEnable %V }
   FMDev -onchange squ  { ::FMDev::updateSquelch %V  }   
   trace variable ::Receiver::State(demod_raw)  w  "::FMDev::update_mod_format"
 }

#****************************************************************************
# Name:        toggleSquelch
# Parameters:  none
# Description: Change the use/ignore squelch state
#****************************************************************************
 proc toggleSquelch { } \
 {
    variable State
    if {$State(use_squelch,raw) == 0} \
    {
       FMDev squ 1
    } \
    else \
    {
        FMDev squ 0
   }
 }
#****************************************************************************
# Name:        updateSquelch
# Parameters:  new state
# Description: Update use/ignore squelch state
#****************************************************************************
 proc updateSquelch { state } \
 {
    variable State
    set State(use_squelch,raw) $state
    if {$state == 0 } \
    {
       set State(use_squelch) "OFF"
    } \
    else \
    {
       set State(use_squelch) "ON"
    }
 }
#****************************************************************************
# Name:        update_mod_format
# Parameters:  ignored
# Description: Track receiver demod
#****************************************************************************
 proc update_mod_format { args } \
 {
    after idle [namespace code update_mod_format_core]
 }
 proc update_mod_format_core { } \
 {
    variable State
    switch -- $::Receiver::State(demod_raw) \
    {
       0 -
       1 -
       2 -
       3 -
       5 -
       10 -
       12 \
       {
          set State(title)  "DEV"
          set State(units)  "kHz"
       }
    
       4 -
       15 -
       16 -
       17 -
       18 \
       {
          set State(title)  "MOD"
          set State(units)  "%"
       }
    }
 }

#****************************************************************************
# Name:        setTrig
# Parameters:  trigState         OFF | ONE_SHOT | CONTINUOUS
# Description: Sets up the current trigger mode of the FM deviation meter
#****************************************************************************
 proc setTrig { trigState } {
    if [info exists ::FMDev::trigger_map($trigState)] {
       FMDev trig $::FMDev::trigger_map($trigState)
    }
 }

#****************************************************************************
# Name:        updateTrig
# Parameters:  trigState         1 = One Shot, 2 = Continuous
# Description: Sets up the current trigger mode of the FM deviation meter
#****************************************************************************
 proc updateTrig { trigState } {
    if [info exists ::FMDev::trigger_map($trigState)] {
       set ::FMDev::State(trigger) $::FMDev::trigger_map($trigState)
    }
    set ::FMDev::State(Engine_trigger) $trigState
 }

#****************************************************************************
# Name:        setMath
# Parameters:  mathState         PEAK | AVERAGE
# Description: Sets up the current math mode of the FM deviation meter
#****************************************************************************
 proc setMath { mathState } {
    if [info exists ::FMDev::math_map($mathState)] {
       FMDev math $::FMDev::math_map($mathState)
    }
 }

#****************************************************************************
# Name:        updateMath
# Parameters:  trigState         0 = Peak, 1 = Average
# Description: Sets up the current math mode of the FM deviation meter
#****************************************************************************
 proc updateMath { mathState } \
 {
   #puts "updateMath:  $mathState"
   
   variable  State
   
   if { [info exists ::FMDev::math_map($mathState)] } \
   {
     set State(math)  $::FMDev::math_map($mathState)
   }
   
   set State(Engine_math)  $mathState
   

   set State(mode)  $mathState
   
   switch  $mathState \
   {
     0  { set State(mode,text,long)   "PEAK"
          set State(mode,text,short)  "PEAK" }

     1  { set State(mode,text,long)   "AVERAGE"
          set State(mode,text,short)  "AVG" }
     
     2  { set State(mode,text,long)   "POS PEAK"
          set State(mode,text,short)  "P PK" }
     
     3  { set State(mode,text,long)   "NEG PEAK"
          set State(mode,text,short)  "N PK" }
   }
   
   return
 }


#****************************************************************************
# Name:        set_fmdev_average
# Parameters:  average         approximate time (0.1 sec intervals) to average over
# Description: Sets up the current average of the FM Deviation
#
#    Minimum = 1, this sends 100 to engine
#    Maximum = 100, this sends 10k to engine
#
#****************************************************************************
proc set_fmdev_average { } {
    variable State
    set average [expr $State(average)*100]
    
    if {$average < 100 } { set average 100 }  
    if {$average > 10000 } { set average 10000 }   
    
    after idle "FMDev avg $average -nowait"
}

#****************************************************************************
# Name:        updateAverage
# Parameters:  average         Average setting from the engine
# Description: Sets up a meter displayed value for an average
#****************************************************************************
proc updateAverage { average } {
    variable State
    set State(Engine_average) $average

    set temp [expr $average/100]
    set State(average) $temp
}

#****************************************************************************
# Name:        updateMode
# Parameters:  runState         0 = Off, 1 = On
# Description: Sets up the current run state of the FM Deviation
#****************************************************************************
 proc updateMode { runState } {
   switch $runState {
      0       { set ::FMDev::State(run) OFF }
      1       { set ::FMDev::State(run) ON }
      default { return }
   }
   set ::FMDev::State(Engine_run) $runState
 }

#****************************************************************************
# Name:        updateEnable
# Parameters:  enableBits       1 = Off, 3 = Enabled
# Description: Sets up the current enabled state of the FM Deviation
#****************************************************************************
 proc updateEnable { enableBits } {
   switch $enableBits {
      3       { set ::FMDev::State(enabled) Enabled }
      default { set ::FMDev::State(enabled) Disabled }
   }
   set ::FMDev::State(Engine_enabled) $enableBits
 }


#****************************************************************************
# Name:        updateReading
# Parameters:  count            Frequency count from the engine
#              count_precision  Current digits of accuracy
#              status           Status flags
#              count_accuracy   Current MAXIMUM digits of accuracy
# Description: Sets up the current state of the FM Deviation reading
#****************************************************************************
 proc updateReading { readingList } {
   variable State
   # Get the data from the meter reading
   set State(avg_value)     [format "%3.3f" [lindex $readingList 0] ]
   set State(precision)     [lindex $readingList 1]
   set State(status)        [lindex $readingList 2]
 }

#****************************************************************************
# Name:        updatePeaks
# Parameters:  readingList      List of data from the engine
#                 Peak HI       Max peak value
#                 HI Precision  Max peak precision
#                 Peak LO       Min peak value
#                 LO Precision  Min peak precision
#                 Flags         Status flags
# Description: Sets up the current state of the peak readings
#****************************************************************************
 proc updatePeaks { readingList } {
   # Get the data from the peak readings
   set peakHi    [lindex $readingList 0]
   set peakLo    [lindex $readingList 2]

   set ::FMDev::State(peakFlags) [lindex $readingList 4]

   set ::FMDev::State(peakHi)    [format "%0.2f" $peakHi]
   set ::FMDev::State(peakLo)    [format "%0.2f" $peakLo]
 }


#****************************************************************************
# Name:        updatePeakEnable
# Parameters:  
# Description: Sets up the current state of the peak enable
#****************************************************************************
 proc updatePeakEnable { peakEnable } \
 {
   set ::FMDev::State(peakEnable)  $peakEnable

   if { $::FMDev::State(peakEnable) == 1 } \
   {
     set ::FMDev::State(peakEnable_text)  "ON"
   }

   if { $::FMDev::State(peakEnable) == 0 } \
   {
     set ::FMDev::State(peakEnable_text)  "OFF"
   }

 }


#****************************************************************************
# Name:        updateAlerts
# Parameters:  
# Description: Update the Alerts flags
#****************************************************************************
 proc updateAlerts { alerts } \
 {
   variable State
 
   set State(alerts)  $alerts
   if {$alerts & 2} \
   {
     set State(alertHi_text) "ON"
   } \
   else \
   {
     set State(alertHi_text) "OFF"
   }
   if {$alerts & 1} \
   {
     set State(alertLo_text) "ON"
   } \
   else \
   {
     set State(alertLo_text) "OFF"
   }
   if {$alerts & 4} \
   {
     set State(alarm_text)       "ON"
     set State(BellEnabled_text) "ON"

     set State(BellEnabled) 1
   } \
   else \
   {
     set State(alarm_text)       "OFF"
     set State(BellEnabled_text) "OFF"

     set State(BellEnabled) 0
   }
 }


#****************************************************************************
# Name:        updateAlertHi
# Parameters:  
# Description: Formats the Alert High value
#****************************************************************************
 proc updateAlertHi { alertHi } {
 
   set ::FMDev::State(alertHi) [format "%0.2f" $alertHi]

 }

#****************************************************************************
# Name:        updateAlertLo
# Parameters:  
# Description: Formats the Alert Low value
#****************************************************************************
 proc updateAlertLo { alertLo } {
 
   set ::FMDev::State(alertLo) [format "%0.2f" $alertLo]

 }


#********************************************************************
#
#       Name: setLimits
#
# Parameters:
#
# Decsription: Sets the limit values
#
#********************************************************************
proc setLimits { limit_type } \
{
  variable  State
  
  if { $limit_type == "high" } \
  {
    set temp  [expr $State(alerts) & 0x02]
    
    # If the limit alert is turned OFF, turn it ON
    if { $temp == 0 } \
    {
      set temp  [expr $State(alerts) | 0x02]
      
      FMDev alerts  $temp  -nowait
    }
    
    FMDev ulimit  $State(alertHi)  -nowait
  }

  if { $limit_type == "low" } \
  {
    set temp  [expr $State(alerts) & 0x01]
    
    # If the limit alert is turned OFF, turn it ON
    if { $temp == 0 } \
    {
      set temp  [expr $State(alerts) | 0x01]
      
      FMDev alerts  $temp  -nowait
    }
    
    FMDev llimit  $State(alertLo)  -nowait
  }

  return
}  


#********************************************************************
#
#       Name: toggleAlerts
#
# Parameters:
#
# Decsription: Toggles the state of the alerts
#
#********************************************************************
proc toggleAlerts { limit_type } \
 {
   variable  State
   
   if { $limit_type == "high" } \
   {
     set new  [expr $State(alerts) ^ 2]
    
     FMDev alerts  $new  -nowait
   }

   if { $limit_type == "low" } \
   {
     set new  [expr $State(alerts) ^ 1]
    
     FMDev alerts  $new  -nowait
   }
  
   return
 }


#********************************************************************
#
#       Name: toggleAlarm
#
# Parameters:
#
# Decsription: Toggles the state of the alarm
#
#********************************************************************
proc toggleAlarm { } \
{
   variable State
   FMDev alerts [expr $State(alerts) ^ 4] -nowait
}


#********************************************************************
#
#       Name: togglePeakEnable
#
# Parameters: 
#
# Decsription: 
#
#********************************************************************
proc togglePeakEnable { } \
{
  if { $::FMDev::State(peakEnable) == 1 } \
  {
    after idle "FMDev peak 0 -nowait"
  }
  
  if { $::FMDev::State(peakEnable) == 0 } \
  {
    after idle "FMDev phreset  -nowait"
    after idle "FMDev peak 1   -nowait"
  }

  return    
}


##********************************************************************
##
##       Name: setAverage
##
## Parameters: new_value   - requested new value 
##                           -1 - no new value requested
##
## Description: set the meter average
##
##********************************************************************
#proc setAverage { } \
# {
#   # Set the meter engine
#   after idle "FMDev  average  $::FMDev::State(average)  -nowait"
#    
#   return
# }


#****************************************************************************
# Name:        query_fmdev_state
# Parameters:  none
# Description: This function sends down a request to the FM Deviation engine to 
#            : obtain the engine state 
#****************************************************************************
 proc query_fmdev_state { } {

   FMDev run  
   FMDev math 
   FMDev avg
   FMDev en
   FMDev trig
   FMDev ulim
   FMDev llim
   FMDev alerts
   FMDev peak
   FMDev squ

   set ::FMDev::State(engineId) [format_version [FMDev ver 0] ]
   set ::FMDev::State(driverId) [format_version [FMDev ver 1] ]

 }

#****************************************************************************
# Name:        getSaveState
# Parameters:  none
# Description: This function returns a list of TCL commands which can be used
#              to set the FM Deviation engine to its current state
#****************************************************************************
 proc getSaveState { } {
   variable State
   
   set returnList \
   {
     FMDev run    $State(Engine_run)     -nowait 
     FMDev math   $State(Engine_math)    -nowait
     FMDev trig   $State(Engine_trigger) -nowait
     FMDev avg    $State(Engine_average) -nowait
     FMDev en     $State(Engine_enabled) -nowait
     FMDev peak   $State(peakEnable)     -nowait
     FMDev alerts $State(alerts)         -nowait
     FMDev ulim   $State(alertHi)        -nowait
     FMDev llim   $State(alertLo)        -nowait
     catch {FMDev squ    $State(use_squelch,raw) -nowait}

     set ::FMDev::State(BellEnabled) {$State(BellEnabled)}
     set ::FMDev::State(resolution)  {$State(resolution)}
   }            
           
   return [subst -nocommands $returnList]   
 }
 
#****************************************************************************
# Name:        Sync
# Parameters:  none
# Description: Write state variables to hardware
#****************************************************************************
 proc Sync { } {
    eval [getSaveState]
    update_mod_format
 }

#****************************************************************************
# Name:        DefaultSetup
# Parameters:  none
# Description: Sets the FM Deviation engine with default values
#****************************************************************************
 proc DefaultSetup { } {
    FMDev trig CONTINUOUS       ;# Set the meter for continuous readings
    FMDev math PEAK             ;# Set DSP to peak readings, not RMS
    FMDev avg 1000              ;# Set average - 200 to 10000 is typical
 }

#****************************************************************************
# Name:        QuickStart
# Parameters:  none
# Description: Turns on the FM Deviation engine but does not set up parameters 
#****************************************************************************
 proc QuickStart { } {
    FMDev en 3                  ;# Enable data to engine
    FMDev run ON                ;# Start the engine
 }

#****************************************************************************
# Name:        QuickStop
# Parameters:  none
# Description: Handles a normal stop of the FM Deviation engine 
#****************************************************************************
 proc QuickStop { } {
    FMDev en 1                  ;# Disable data to engine
    FMDev run OFF               ;# Stop the engine
 }


#****************************************************************************
# Name:        SyncAll
# Parameters:  none
# Description: Insures that TCL, the engine and hardware are in sync
#****************************************************************************
 proc SyncAll { } {
    set_fmdev_callbacks
    query_fmdev_state
    if ![interp issafe] \
    {
       after 100 {::FMDev::Sync}
    }
 }

#****************************************************************************
# Name:        Get_HTML_reading
# Parameters:  base_url - the base URL for the containing page
# Description: Return an string containing HTML showing the current state
#              of this system
#****************************************************************************
 proc Get_HTML_reading { {base_url ""} } \
 {
    variable State
    set x \
{<tr>
<td nowrap>$State(title)</td><td nowrap>$State(avg_value) $State(units)</td>
</tr>}
    return [subst $x]
 }
 
#****************************************************************************
# Initialize the namespace and activate the callbacks
#****************************************************************************
   Load::on_system_done { ::FMDev::SyncAll }

}  ;# End of the FM Deviation namespace

#****************************************************************************
# This registers the engine with the list for generating an HTML page
#****************************************************************************
namespace eval ::HTTPD \
{
   variable main_screen_readings
   lappend  main_screen_readings(1) FMDev
}
#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates {

   variable engines   
   lappend engines FMDev
}

#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
     Fmdev_Eng "FM Deviation Engine"
     Fmdev_Drv "FM Deviation Driver"
   }
   set version(Fmdev_Eng) [FMDev version 0]
   set version(Fmdev_Drv) [FMDev version 1]
}



