#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2003         DESIGN ENGINEERING     #
# --------------------------------------------------------------------------#
# FILE        : Ltr.tcl                                                     #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bradford Taylor                                             #
# DESCRIPTION : This file is sourced by TCL when the LTR    command is      #
#             : first used.  It loads the C++ interface file, provides a    #
#             : namespace for screen variables, and sets up callbacks to    #
#             : update the variables for the UI.                            #
#                                                                           #
#############################################################################

#****************************************************************************
# Handle a load of the C++ interface
#****************************************************************************
# Call the base class dummy command to force a load
if { "[info command Ltr]" != "" } { return }

package require Engtclbase

Engtclbase

# Load the object file
::load $::Ltr::object

package provide Ltr 1.0

#***************************************************************************
#  Add functions to Ltr namespace                                          *
#***************************************************************************

namespace eval ::Ltr \
{
  variable SaveSortOrder FIRST
  #***************************************************************************
  #
  #        Name: set_ltr_callbacks
  #
  #  Parameters: none
  #
  # Description: Sets up the callbacks to update the Ltr data
  #
  #***************************************************************************
  proc set_ltr_callbacks {} \
  {  
    Ltr -onchange reading  { ::Ltr::updateReading %V}

    Ltr -onchange mode     { ::Ltr::updateMode %V}
 
    Ltr -onchange band     { ::Ltr::updateBand %V}

    Ltr -onchange area     { ::Ltr::updateArea    %V}
    Ltr -onchange free     { ::Ltr::updateFree    %V}
    Ltr -onchange goto     { ::Ltr::updateGoto    %V}
    Ltr -onchange group    { ::Ltr::updateGroup   %V}
    Ltr -onchange riccode  { ::Ltr::updateRicCode %V}
    Ltr -onchange home     { ::Ltr::updateHome    %V}
    Ltr -onchange inuse    { ::Ltr::updateInUse   %V}

    Ltr -onchange rstate   { ::Ltr::updateRepeatSimState  %V}
    Ltr -onchange mstate   { ::Ltr::updateRadioSimState   %V}

    Ltr -onchange params   { ::Ltr::updateTxParams  %V}
  }
  
  
  #***************************************************************************
  #
  #        Name: Init
  #
  #  Parameters:
  #
  # Description: procedure to call to load the namespace
  #
  #***************************************************************************
  proc Init { } \
  {
  
  }
  
  
  #***************************************************************************
  #
  #        Name: updateReading       
  #
  #  Parameters: The layer 1 Ltr data packet
  #
  # Description: Updates the state parameters for Ltr
  #
  #***************************************************************************
  proc updateReading { readingList } \
  {
    #puts "Ltr::updateReading:  $readingList"

    variable  rlist
    variable  State

    set rl  $readingList
  
    set rlist  $rl

    # Get the data from the packet
    set State(sync)       [format "0x%03X" [lindex $rl 0]]
    set State(word)       [format "0x%06X" [lindex $rl 1]]
    set State(checksum)   [format "0x%02X" [lindex $rl 2]]
    set State(timestamp)  [lindex $rl 3]
    set State(status)     [lindex $rl 4]
    set State(inverted)   [expr $::Ltr::State(status) & 0x1]

    set State(rx_area)    [lindex $rl 5]
    set State(rx_in_use)  [lindex $rl 6];  # Same as GOTO
    set State(rx_goto)    [lindex $rl 6];  # Same as IN USE
    set State(rx_home)    [lindex $rl 7]
    set State(rx_group)   [lindex $rl 8]
    set State(rx_free)    [lindex $rl 9]

    return
  }
  
  
  # show data word
  proc showWord {} \
  {
  
  }
  
  
  #***************************************************************************
  #
  #        Name: repeaterSimTurnOn
  #
  #  Parameters:
  #
  # Description: Turns on the Repeater Simulator
  #
  #***************************************************************************
  proc repeaterSimTurnOn { } \
  {
    # Turn off any previous trunking state
    Generator::Trunking_mode  "::Ltr::ltrTurnOff"
   
    # Turn on Repeater Sim
    Ltr mode  4

    return
  }
 

  #***************************************************************************
  #
  #        Name: radioSimTurnOn
  #
  #  Parameters:
  #
  # Description: Turns on the Radio Simulator
  #
  #***************************************************************************
  proc radioSimTurnOn { } \
  {
    # Turn off any previous trunking state
    Generator::Trunking_mode  "::Ltr::ltrTurnOff"
   
    # Turn on Radio Sim
    Ltr mode  5

    return
  }
 

  #***************************************************************************
  #
  #        Name: monitorTurnOn
  #
  #  Parameters:
  #
  # Description: Turns on the Monitor
  #
  #***************************************************************************
  proc monitorTurnOn { } \
  {
    variable  State
    
    # Turn off any previous trunking state
    Generator::Trunking_mode  "::Ltr::ltrTurnOff"
   
    # Turn on Monitor
    switch -- $State(monitor_mode_text) \
    {
      "OFF"       { ltrTurnOff   }

      "REPEATER"  { Ltr  mode  1 }
    
      "RADIO"     { Ltr  mode  2 }
    }

    return
  }

  #***************************************************************************
  #
  #        Name: ltrTurnOff
  #
  #  Parameters:
  #
  # Description: Turns off the LTR engine
  #
  #***************************************************************************
  proc ltrTurnOff { } \
  {
    # Turn off the LTR engine
    Ltr mode  0

    return
  }


  #***************************************************************************
  #
  #        Name: updateMode       
  #
  #  Parameters: mode
  #
  # Description: Updates the MODE state variable
  #
  #***************************************************************************
  proc updateMode { mode } \
  {
    variable  State
    
    # Turn off all Mode options
    set State(monitor_mode_text)        "OFF"

    set State(find_repeat_on_off)       0
    set State(find_repeat_on_off_text)  "OFF"

    set State(repeat_sim_on_off)        0
    set State(repeat_sim_on_off_text)   "OFF"
           
    set State(radio_sim_on_off)         0
    set State(radio_sim_on_off_text)    "OFF"
    set State(mode) $mode
    switch -- $mode \
    {
      0  { }
      
      1  { # Monitor Repeater
           set State(monitor_mode_text)  "REPEATER" }
      
      2  { # Monitor Radio
           set State(monitor_mode_text)  "RADIO" }
      
      3  { # Find Repeater 
           set State(find_repeat_on_off)       1
           set State(find_repeat_on_off_text)  "ON" }
      
      4  { # Repeater Sim
           set State(repeat_sim_on_off)        1
           set State(repeat_sim_on_off_text)   "ON" }
        
      5  { # Radio Sim
           set State(radio_sim_on_off)         1
           set State(radio_sim_on_off_text)    "ON" }
    }

    return
  }


  #***************************************************************************
  #
  #        Name: updateBand       
  #
  #  Parameters: area
  #
  # Description: Updates the BAND state variable
  #
  #***************************************************************************
  proc updateBand { band } \
  {
    variable  State
    
    set State(band)  $band
    
    switch -- $band \
    {
      0        { set State(band_text)  "800"  }
      
      1        { set State(band_text)  "900"  }
      
      default  { set State(band_text)  "USER" }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: updateArea       
  #
  #  Parameters: area
  #
  # Description: Updates the AREA state variable
  #
  #***************************************************************************
  proc updateArea { area } \
  {
    variable  State
    
    set State(tx_area)  $area
    
    return
  }


  #***************************************************************************
  #
  #        Name: updateFree       
  #
  #  Parameters: free
  #
  # Description: Updates the FREE state variable
  #
  #***************************************************************************
  proc updateFree { free } \
  {
    variable  State
    
    set State(tx_free)  $free

    return
  }


  #***************************************************************************
  #
  #        Name: updateGoto       
  #
  #  Parameters: goto
  #
  # Description: Updates the GOTO state variable
  #
  #***************************************************************************
  proc updateGoto { goto } \
  {
    variable  State
    
    set State(tx_goto)  $goto

    return
  }


  #***************************************************************************
  #
  #        Name: updateGroup       
  #
  #  Parameters: group
  #
  # Description: Updates the GROUP state variable
  #
  #***************************************************************************
  proc updateGroup { group } \
  {
    variable  State
    
    set State(tx_group)  $group

    return
  }


  #***************************************************************************
  #
  #        Name: updateRicCode       
  #
  #  Parameters: ric_code
  #
  # Description: Updates the RIC Code state variable
  #
  #***************************************************************************
  proc updateRicCode { ric_code } \
  {
    variable  State
    
    set State(tx_riccode)  $ric_code

    return
  }


  #***************************************************************************
  #
  #        Name: updateHome       
  #
  #  Parameters: home
  #
  # Description: Updates the HOME state variable
  #
  #***************************************************************************
  proc updateHome { home } \
  {
    variable  State
    
    set State(tx_home)  $home

    return
  }
  
  

  #***************************************************************************
  #
  #        Name: updateInUse       
  #
  #  Parameters: in_use
  #
  # Description: Updates the IN USE state variable
  #
  #***************************************************************************
  proc updateInUse { in_use } \
  {
    #puts "updateInUse:  $in_use"
    
    variable  State
    
    set State(tx_in_use)  $in_use

    return
  }


  #***************************************************************************
  #
  #        Name: updateRepeatSimState
  #
  #  Parameters: rstate
  #
  # Description: Updates the State variable of the Repeater Sim
  #
  #***************************************************************************
  proc updateRepeatSimState { rstate } \
  {
    variable  State
    
    switch -- $rstate \
    {
      0  { set State(tx_rstate)  "IDLE"   }
      
      1  { set State(tx_rstate)  "SYS BUSY"   }
      
      2  { set State(tx_rstate)  "REP BUSY"   }
      
      3  { set State(tx_rstate)  "CONT"   }
      
      4  { set State(tx_rstate)  "R INIT" }
      
      5  { set State(tx_rstate)  "RIC"    }
      
      6  { set State(tx_rstate)  "WAIT..."   }
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: updateRadioSimState
  #
  #  Parameters: mstate
  #
  # Description: Updates the State variable of the Radio Sim
  #
  #***************************************************************************
  proc updateRadioSimState { mstate } \
  {
    variable  State
    
    switch -- $mstate \
    {
      0  { set State(tx_mstate)  "IDLE"   }
      
      1  { set State(tx_mstate)  "PTT"   }
      
      2  { set State(tx_mstate)  "LISTEN" }
    }
    
    return
  }


  
  #***************************************************************************
  #
  #        Name: updateTxParams
  #
  #  Parameters: params - DEVIATION and INVERT values
  #
  # Description: Updates the DEVIATION and INVERT state variables
  #
  #***************************************************************************
  proc updateTxParams { params } \
  {
    #puts "updateTxParams:  $params"
    
    variable  State

    set State(tx_deviation)  [lindex $params  0]

    set State(tx_invert)     [lindex $params  1]

    set State(tx_sync)       [lindex $params  2]
    
    set State(tx_sync,hex) [format %X $State(tx_sync)]

    return
  }


  #****************************************************************************
  #
  #        Name: copyRxDataToTxData
  #
  #  Parameters:
  #
  # Description: Copies the received data to the transmit data fields
  #
  #****************************************************************************
  proc copyRxDataToTxData { } \
  {
    variable  State
    # check if we HAVE RX data to copy...
    if {$State(rx_area) != "" }  { Ltr area  $State(rx_area) }
    if {$State(rx_home) != "" }  { Ltr home  $State(rx_home) }
    if {$State(rx_group) != "" } { Ltr group $State(rx_group) }

    return
  }

  
    
  #****************************************************************************
  #
  #        Name: create_fcc_tables
  #
  #  Parameters: none
  #
  # Description: Creates two arrays for the FCC Channel number to Frequency 
  #              lookup
  #
  #****************************************************************************
  proc create_fcc_tables { } \
  {
    variable  State
    
    # Set the band
    foreach { band  band_label } { 0  800  1  900 } \
    {
      Ltr band  $band

      for { set i  1 } { $i < 1024 } { incr i  1 } \
      {
        # Get the frequency for each FCC Channel number
        #set temp [format "%10.6f" [Ltr  fcctx  $i] ]
        #set State($band_label,fcc,$i)  $temp
        set State($band_label,fcc,$i)  [Ltr  fcctx  $i]
      }
    }

    return
  }


  #****************************************************************************
  #
  # Name:        query_ltr_state
  #
  # Parameters:  none
  #
  # Description: This function sends down a request to the LTR engine to 
  #              obtain the engine state 
  #
  #****************************************************************************
  proc query_ltr_state { } \
  {
    Ltr  mode
    
    Ltr  params
    
    set ::Ltr::State(engineId) [format_version [Ltr ver 0]]
    set ::Ltr::State(driverId) [format_version [Ltr ver 1]]
    
    return
  }


  #****************************************************************************
  #
  #        Name: getSaveState
  #
  #  Parameters: none
  #
  # Description: This function returns a list of TCL commands which can be used
  #              to set the Ltr engine to its current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    variable  State
  
    Ltr::Init
   
    set returnList \
    {
      if { [IFR_options is_installed LTR] } \
      {
        Ltr band     $State(band)       -nowait
        Ltr area     $State(tx_area)    -nowait
        Ltr free     $State(tx_free)    -nowait
        Ltr home     $State(tx_home)    -nowait
        Ltr goto     $State(tx_goto)    -nowait
        Ltr group    $State(tx_group)   -nowait
        Ltr riccode  $State(tx_riccode) -nowait
        Ltr inuse    $State(tx_in_use)  -nowait
        Ltr params   $State(tx_deviation)  $State(tx_invert)  $State(tx_sync) -nowait
      }
    }

    for { set i  1 } { $i <= 20 } { incr i  1 } \
    {
      # Call to set the frequencies directly
      set temp [Ltr  txfreq  $i]
      append returnList "Ltr  txfreq  $i $temp -nowait\n"
      set temp [Ltr  rxfreq  $i]
      append returnList "Ltr  rxfreq  $i $temp -nowait\n"
    }

    switch -- $State(mode) \
    {
       0 { append returnList "SavedStates::Finally ::Ltr::ltrTurnOff\n"}
       1 { append returnList "set ::Ltr::State(monitor_mode_text) REPEATER\n"
           append returnList "SavedStates::Finally ::Ltr::monitorTurnOn\n"}
       2 { append returnList "set ::Ltr::State(monitor_mode_text) RADIO\n"
           append returnList "SavedStates::Finally ::Ltr::monitorTurnOn\n"}
       4 { append returnList "SavedStates::Finally ::Ltr::repeaterSimTurnOn\n"}
       5 { append returnList "SavedStates::Finally ::Ltr::radioSimTurnOn\n"}
       default { append returnList "SavedStates::Finally ::Ltr::ltrTurnOff\n"}
    }

    #puts "$returnList"
   
    return [subst -nocommands $returnList]
  }


  #****************************************************************************
  #
  #        Name: Sync
  #
  #  Parameters: none
  #
  # Description: Write state variables to hardware
  #
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }
  
  
  #****************************************************************************
  #
  #        Name: SyncAll
  #
  #  Parameters: none
  #
  # Description: Insures that TCL, the engine and hardware are in sync
  #
  #****************************************************************************
  proc SyncAll { } \
  {
    if { ![IFR_options is_installed LTR] } \
    { 
      return 
    }
     
    set_ltr_callbacks
    
    create_fcc_tables
   
    if { ![interp issafe] } \
    {
      ::Ltr::Sync
    }

    return
  }


  #***************************************************************************
  # Radio Setups
  #***************************************************************************
  
  # Standard brand radio
  proc Standard {} \
  {
    Ltr band 0
    Ltr border 0
    Ltr rxfreq 8 [Ltr fccrx 521]
    Ltr txfreq 8 [Ltr fcctx 521]
    Ltr rxfreq 4 [Ltr fccrx 561]
    Ltr txfreq 4 [Ltr fcctx 561]
    Ltr area 1
    Ltr home 8
    Ltr goto 8
    Ltr free 4
    Ltr group 1
    Ltr ric -1
    Ltr txfreq 8
  }
  	
  # King radio
  proc King {} \
  {
    Ltr band 0
    Ltr border 0
    Ltr rxfreq 5 [Ltr fccrx 258];  # Example of 'band' usage
    Ltr txfreq 5 [Ltr fcctx 258]
    Ltr rxfreq 9 [Ltr fccrx 298]
    Ltr txfreq 9 [Ltr fcctx 298]
    Ltr area 0
    Ltr home 5
    Ltr goto 5
    Ltr free 9
    Ltr group 1
    Ltr ric   2
    Ltr txfreq 5
  }
  
  # Uniden Radio
  proc Uniden {} \
  {
    Ltr band 0
    Ltr border 0
    Ltr rxfreq 5 [Ltr fccrx 343]
    Ltr txfreq 5 [Ltr fcctx 343]
    Ltr rxfreq 9 [Ltr fccrx 303]
    Ltr txfreq 9 [Ltr fcctx 303]
    Ltr area 0
    Ltr home 5
    Ltr goto 5
    Ltr free 9
    Ltr group 151
    Ltr ric   201
    Ltr txfreq 5
  }
  
  # EF Johnson Radio
  proc Jradio {} \
  {
    Ltr band 2
    Ltr border 0
    Ltr txfreq 1  461.412500
    Ltr rxfreq 1  466.412500
    Ltr txfreq 11 462.062500
    Ltr rxfreq 11 467.062500
    Ltr txfreq 3  463.487500
    Ltr rxfreq 3  468.487500
    Ltr txfreq 13 463.787500
    Ltr rxfreq 13 468.787500
    Ltr txfreq 18 463.937500
    Ltr rxfreq 18 468.937500
    Ltr area 0
    Ltr home 1
    Ltr goto 1
    Ltr free 1
    Ltr ric  201
    Ltr group 2
    Ltr txfreq 1
  }

  # EF Johnson Repeater
  proc Johnson {} \
  {
    Ltr band 1
    Ltr border 0
    Ltr rxfreq 1 [Ltr fccrx 248]
    Ltr txfreq 1 [Ltr fcctx 248]
    Ltr area 0
    Ltr home 1
    Ltr goto 1
    Ltr free 1
    Ltr txfreq 1
  }
  

  proc Test1 {} \
  {
    set ::Ltr::abort  0
  
    for { set i  1 } { $i <= 255 } { incr i  1 } \
    {
      if { $::Ltr::abort == 1 } \
      {
        set i  300
        
        break
      }
    
      puts "Radio Sim group:  $i"
    
      after idle "Ltr group $i"
    
      after 2000
  
      puts "Repeater Sim group:  $::Ltr::State(rx_group)"
    
      if { $i == $::Ltr::State(rx_group) } \
      {
        set i  300

        break
      }
    
      update
    }
    
    return
  }


  #****************************************************************************
  # Initialize the namespace and activate the callbacks
  #****************************************************************************
  Load::on_system_done { ::Ltr::SyncAll }
  
  
};  # End of the LTR namespace (Ltr)


#****************************************************************************
# This registers the engine with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable engines   

  lappend engines  Ltr
}

#****************************************************************************
# This registers the engine with the list for the version screen
#****************************************************************************
namespace eval ::Version_Screen \
{
   variable version
   variable names
   
   array set names \
   {
     Ltr_Eng "LTR Engine"
     Ltr_Drv "LTR Driver"
   }
   set version(Ltr_Eng) [Ltr version 0]
   set version(Ltr_Drv) [Ltr version 1]
}

#****************************************************************************
# Define the option namespace, and set up our HTML option info
#****************************************************************************
namespace eval ::LTR \
{
  variable InfoString

  set LTR_version [string trimleft [Ltr version 0] "LtrEng"]
  set InfoString [subst \
  {
         <br>
         (using LTR version - $LTR_version)
         <br>
         This option provides features to test radios and base stations which
         use Logic Trunked Radio (LTR) protocol.
  }]
}
