#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2000.Screens.tcl                                            #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file contains the Screens namespace, which provides    #
#             : common functions for setting up the instrument view part    #
#             : of the screen                                               #
#############################################################################
#****************************************************************************
#
#  Screens Namespace  
#
#****************************************************************************
#
#      $toplevel.main           Name of main instrument display area
#
#****************************************************************************
#
#  Public Functions:
#
#     GetTopName                This function accepts a parent and a child
#                               suffix name, and returns a window name which
#                               is a child of the toplevel of the parent
#
#     ShowAllTiles
#
#     ShowRequiredTiles         When a screen is a simple collection of tiles,
#                               this function will display all of the tiles
#
#     ShowOptionTiles
#
#     Clear                     Removes the tiles from the screen which have
#                               been displayed using ShowRequiredTiles
#
#     Build_std_options         This function generates an options menu for
#                               a screen made up of standard tiles
#
#****************************************************************************
#
#  Private Functions:  These functions should be considered as private members
#      of the "Screens" class and not used to get private information 
#
#****************************************************************************
#
#     InitFreeSpace             This function initializes the MainFreeSpace 
#                               when the main screen is erased
#
#     GetStdOptionsList         This function merges information from several
#                               screen "lists" to get the list tiles to display
#                               for the standard options menu.
#
#     GetDesiredDisplayList     This function merges information from several
#                               screen "lists" to get the list of tiles which
#                               the user currently wants displayed.
#
#     AllocateSpace             This function determines if space for a tile is
#                               available in the MainFreeSpace array, and 
#                               allocates space by updating that array and
#                               returning an available tile corner location.
#
#     AddTile                   This function adds the namespace of a tile to the
#                               ActiveTile list of a screen if the tile is not
#                               a required or illegal tile
#
#     RemoveTile                This function removes a tile namespace from the
#                               ActiveTile list of a screen
#
#
#****************************************************************************

namespace eval Screens \
{
  variable  CurrentNamespace  ""
  
  variable  MainFreeSpace

  array set MainFreeSpace {}
  
  variable  MenuSelect

  set MenuSelect(0)  1
  
  # This array contains the parms for the local screen:
  # i.e. the 2975's own display (".")
  variable  Local_screen

  set Local_screen  Intro_Screen


  #****************************************************************************
  #
  # Name:        GetTopName
  #
  # Parameters:  parent    - parent of child window
  #              childName - name suffix for child window 
  #
  # Description: Generates the name of a window which is the child of the
  #              toplevel window of the parent.  Returns "" if parent does not
  #              exist.
  #
  #****************************************************************************
  proc GetTopName { parent  childName } \
  {
    Debug::Puts Screens 2 "GetTopName:  $parent,  $childName"

    if { ![winfo exists $parent] } \
    {
      return  ""
    }

    set topLevelWin  [winfo toplevel  $parent]
    
    return  [name_child  $topLevelWin  $childName]   
  }

  
  #****************************************************************************
  #
  # Name:        Clear
  #
  # Parameters:  topLevelWin - toplevel window for screen to be cleared
  #
  # Description: Ungrids the screen tiles
  #
  #****************************************************************************
  proc Clear { topLevelWin  { menuName "" } } \
  {
    Debug::Puts Screens 2 "Clear:  $topLevelWin,  $menuName"

    # Insure we are talking about the top level window
    set topLevelWin  [winfo toplevel  $topLevelWin]

    set Screen  [View::GetMainView  $topLevelWin]

    if { ![winfo exists $Screen] } \
    {
      return
    }

    # Grid remove unmap slave, but remember its configuration
    # Fixed tiles sizing problem when tiles are re-mapped
    foreach { widget } [grid slaves $Screen] \
    {
      grid remove  $widget
    }
   
    foreach { widget } [pack slaves $Screen] \
    {
      pack forget  $widget
    }
   
    if { $menuName != "" } \
    {
      tkMenuUnpost  $menuName
    }
    
    return
  }


  #****************************************************************************
  #
  # Name:        InitFreeSpace
  #
  # Parameters:  topLevelWin - toplevel window for screen to be displayed
  #
  # Description: Resets all free space variables for a given toplevel
  #
  #****************************************************************************
  proc InitFreeSpace { topLevelWin } \
  {
    Debug::Puts Screens 2 "InitFreeSpace,  $topLevelWin"

    # Insure we are talking about the top level window
    set topLevelWin  [winfo toplevel $topLevelWin]

    set ::Screens::MainFreeSpace($topLevelWin,Full)      { 0 0 }
    set ::Screens::MainFreeSpace($topLevelWin,Half_v)    0
    set ::Screens::MainFreeSpace($topLevelWin,Half_h)    0
    set ::Screens::MainFreeSpace($topLevelWin,Quarter)   0
    set ::Screens::MainFreeSpace($topLevelWin,Eighth_v)  0
    set ::Screens::MainFreeSpace($topLevelWin,Eighth_h)  0
    set ::Screens::MainFreeSpace($topLevelWin,Sixteenth) 0
    
    return
  }


  #****************************************************************************
  #
  # Name:        GetStdOptionsList
  #
  # Parameters:  ScreenNamespace - namespace of screen having standard tiles
  #
  # Description: Returns a list of standard options
  #
  #****************************************************************************
  proc GetStdOptionsList { ScreenNamespace } \
  {
    Debug::Puts Screens 2 "GetStdOptionsList:  $ScreenNamespace"

    # Start first with the required tiles
    set tempList  {}
    set usedList  [subst $\::${ScreenNamespace}\::RequiredTiles]
    set badList   [subst $\::${ScreenNamespace}\::NotAllowedTiles] 

    # Add the recommended tiles which are allowed and not used
    foreach { nameSpace } [subst $\::${ScreenNamespace}\::RecommendedTiles] \
    {
      if { ( [lsearch -exact $usedList $nameSpace] == -1 ) && 
           ( [lsearch -exact $badList $nameSpace] == -1 ) &&
           ( [namespace children :: $nameSpace] != "")
         } \
      {
        lappend tempList $nameSpace   
      }
    } 
      
    return $tempList
  }
  

  #****************************************************************************
  # 
  # Name:        GetDesiredDisplayList
  #
  # Parameters:  ScreenNamespace - namespace of screen having standard tiles
  #
  # Description: Returns a list of standard options
  #
  #****************************************************************************
  proc GetDesiredDisplayList { ScreenNamespace } \
  {
    Debug::Puts Screens 2 "GetDesiredDisplayList:  $ScreenNamespace"

    # Start the 'tempList' with the required tiles first
    set tempList  [subst $\::${ScreenNamespace}\::RequiredTiles]
    
    # Get the list of not allowed tiles
    set badList   [subst $\::${ScreenNamespace}\::NotAllowedTiles] 

    # Add the active tiles, which are allowed, to the required tiles
    foreach { nameSpace } [subst $\::${ScreenNamespace}\::ActiveTiles] \
    {
      if { ([lsearch -exact $tempList $nameSpace] == -1) && 
           ([lsearch -exact $badList $nameSpace]  == -1) &&
           ([namespace children :: $nameSpace]    != "") } \
      {
        lappend tempList $nameSpace
      }
    } 
      
    Debug::Puts Screens 2 "tempList:  $tempList"
    
    return $tempList
  }


  #****************************************************************************
  #
  # Name:        AllocateSpace
  #
  # Parameters:  topLevelWin - toplevel window for screen to be displayed
  #              size        - size of tile to be allocated
  #
  # Description: Determines whether space is available and adjusts the free space
  #              accordingly.
  #
  # Returns:     0 = Allocation failure, or a grid specification list in the
  #              form of {ULC_x, ULC_y, Span_x, Span_y}
  #
  #****************************************************************************
  proc AllocateSpace { topLevelWin  size } \
  {
    Debug::Puts Screens 2 "AllocateSpace:  $topLevelWin,  $size"
    
    variable  MainFreeSpace

    # Insure we are talking about the top level window
    set topLevelWin  [winfo toplevel $topLevelWin]

    switch $size \
    {
      Full { if { $MainFreeSpace($topLevelWin,Full) != 0 } \
             {
               set MainFreeSpace($topLevelWin,Full)  0
               
               return [list 0 0 4 4] 
             } \
             else \
             {
               return 0 
             }
           }
           
      Half_v { if { $MainFreeSpace($topLevelWin,Half_v) != 0 } \
               {
                 set loc  $MainFreeSpace($topLevelWin,Half_v)
               
                 set MainFreeSpace($topLevelWin,Half_v)  0
               
                 return [list [lindex $loc 0] [lindex $loc 1] 2 4] 
               } \
               else \
               {
                 set loc  [Screens::AllocateSpace  $topLevelWin  Full]
               
                 if { $loc == 0 } \
                 {
                   return 0
                 } \
                 else \
                 {
                   set x [lindex $loc 0]
                   
                   incr x  2
                   
                   set MainFreeSpace($topLevelWin,Half_v)  [list $x [lindex $loc 1]]
                   
                   return [list [lindex $loc 0] [lindex $loc 1] 2 4]
                 }
               }
             }

      Half_h { if { $MainFreeSpace($topLevelWin,Half_h) != 0 } \
               {
                 set loc  $MainFreeSpace($topLevelWin,Half_h)
                 
                 set MainFreeSpace($topLevelWin,Half_h)  0
                 
                 return [list [lindex $loc 0] [lindex $loc 1] 4 2] 
               } \
               else \
               {
                 set loc  [Screens::AllocateSpace  $topLevelWin  Full]
               
                 if { $loc == 0 } \
                 {
                   return 0
                 } \
                 else \
                 {
                   set y [lindex $loc 1]
                   
                   incr y  2
                   
                   set MainFreeSpace($topLevelWin,Half_h)  [list [lindex $loc 0] $y]
                   
                   return [list [lindex $loc 0] [lindex $loc 1] 4 2]
                 }
               }
             }

      Quarter { if { $MainFreeSpace($topLevelWin,Quarter) != 0 } \
                {
                  set loc  $MainFreeSpace($topLevelWin,Quarter)
                  
                  set MainFreeSpace($topLevelWin,Quarter)  0
                  
                  return [list [lindex $loc 0] [lindex $loc 1] 2 2] 
                } \
                else \
                {
                  set loc  [Screens::AllocateSpace  $topLevelWin  Half_h]
                  
                  set x [lindex $loc 0]
                  
                  set y [lindex $loc 1]
                
                  if { $loc == 0 } \
                  {
                    set loc  [Screens::AllocateSpace  $topLevelWin  Half_v]
                    
                    set x [lindex $loc 0]
                    
                    set y [lindex $loc 1]
                  
                    if { $loc == 0 } \
                    {
                      return 0
                    } \
                    else \
                    {
                      incr y  2
                      
                      set MainFreeSpace($topLevelWin,Quarter)  [list $x $y]
                      
                      return [list [lindex $loc 0] [lindex $loc 1] 2 2]
                    }
                  } \
                  else \
                  {
                    incr x  2
                    
                    set MainFreeSpace($topLevelWin,Quarter)  [list $x $y]
                    
                    return [list [lindex $loc 0] [lindex $loc 1] 2 2]
                  }
                }
              }

      Eighth_v { if { $MainFreeSpace($topLevelWin,Eighth_v) != 0 } \
                 {
                   set loc  $MainFreeSpace($topLevelWin,Eighth_v)
                   
                   set MainFreeSpace($topLevelWin,Eighth_v)  0
                   
                   return [list [lindex $loc 0] [lindex $loc 1] 1 2] 
                 } \
                 else \
                 {
                   set loc [Screens::AllocateSpace  $topLevelWin  Quarter]
                 
                   if { $loc == 0 } \
                   {
                     return 0
                   } \
                   else \
                   {
                     set x [lindex $loc 0]
                     
                     incr x  1
                     
                     set MainFreeSpace($topLevelWin,Eighth_v)  [list $x [lindex $loc 1]]
                     
                     return [list [lindex $loc 0] [lindex $loc 1] 1 2]
                   }
                 }
               }

      Eighth_h { if { $MainFreeSpace($topLevelWin,Eighth_h) != 0 } \
                 {
                   set loc  $MainFreeSpace($topLevelWin,Eighth_h)
                   
                   set MainFreeSpace($topLevelWin,Eighth_h)  0
                   
                   return [list [lindex $loc 0] [lindex $loc 1] 2 1] 
                 } \
                 else \
                 {
                   set loc [Screens::AllocateSpace  $topLevelWin  Quarter]
                   
                   if { $loc == 0 } \
                   {
                     return 0
                   } \
                   else \
                   {
                     set y [lindex $loc 1]
                     
                     incr y  1
                     
                     set MainFreeSpace($topLevelWin,Eighth_h)  [list [lindex $loc 0] $y]
                     
                     return [list [lindex $loc 0] [lindex $loc 1] 2 1]
                   }
                 }
               }

      Sixteenth { if { $MainFreeSpace($topLevelWin,Sixteenth) != 0 } \
                  {
                    set loc  $MainFreeSpace($topLevelWin,Sixteenth)
                    
                    set MainFreeSpace($topLevelWin,Sixteenth)  0
                    
                    return [list [lindex $loc 0] [lindex $loc 1] 1 1] 
                  } \
                  else \
                  {
                    set loc  [Screens::AllocateSpace  $topLevelWin  Eighth_h]
                  
                    if { $loc == 0 } \
                    {
                      set loc  [Screens::AllocateSpace  $topLevelWin  Eighth_v]
                    
                      if { $loc == 0 } \
                      {
                        return 0
                      } \
                      else \
                      {
                        set y [lindex $loc 1]
                        
                        incr y  1
                        
                        set MainFreeSpace($topLevelWin,Sixteenth)  [list [lindex $loc 0] $y]
                        
                        return [list [lindex $loc 0] [lindex $loc 1] 1 1]
                      }
                    } \
                    else \
                    {
                      set x [lindex $loc 0]
                      
                      incr x  1
                      
                      set MainFreeSpace($topLevelWin,Sixteenth)  [list $x [lindex $loc 0]]
                      
                      return [list [lindex $loc 0] [lindex $loc 1] 1 1]
                    }
                  }
                }
                
      default { return 0 }
     
    }
   
    return  
  }


  #****************************************************************************
  #
  # Name:        ShowAllTiles
  #
  # Parameters:  topLevelWin     - toplevel to pack tiles into
  #              ScreenNamespace - namespace of screen to be shown
  #
  # Description: Displays both the RequiredTiles and the ActiveTiles 
  #              for the selected screen
  #
  # Returns:     
  #
  #****************************************************************************
  proc ShowAllTiles { topLevelWin  ScreenNamespace } \
  {
    Debug::Puts Screens 2 "ShowAllTiles:  $topLevelWin,  $ScreenNamespace"
    
    return
  }


  #****************************************************************************
  #
  # Name:        ShowRequiredTiles
  #
  # Parameters:  topLevelWin     - toplevel to pack tiles into
  #              ScreenNamespace - namespace of screen to be shown
  #
  # Description: Displays the RequiredTiles for the selected screen
  #
  # Returns:     A list of tiles that did not fit on the screen
  #
  #****************************************************************************
  proc ShowRequiredTiles { topLevelWin  ScreenNamespace } \
  {
    Debug::Puts Screens 2 "ShowRequiredTiles:  $topLevelWin,  $ScreenNamespace"
    
    variable  CurrentNamespace
    
    # Initialize at no missing tiles
    set MissedTiles  {}

    # Get current screen
    set CurrentNamespace  $ScreenNamespace

    # Do basic sanity checks before we try to run
    if { ![winfo exists $topLevelWin] } \
    {
      return  $MissedTiles
    }
    
    # insure we are talking about the top level window
    set topLevelWin  [winfo toplevel  $topLevelWin]

    # Get the list of active tiles from the screen
    set ActiveList  [Screens::GetDesiredDisplayList  $ScreenNamespace]
    
    if { [llength $ActiveList] == 0 } \
    {
      return $MissedTiles
    } 

    # Initialize free space array 
    InitFreeSpace  $topLevelWin 
    
    set mainWin  [View::GetMainView  $topLevelWin]

    # Remove all tiles from the screen
    # Grid remove unmap slave, remember its configuration
    foreach  tile  [grid slaves $mainWin] \
    {
      grid remove $tile
    }

    Debug::Puts Screens 2 "  ActiveList:  $ActiveList"
    
    # Attempt to place all tiles on the screen
    foreach { tile } $ActiveList \
    {
      # Get the tile size and name (create if needed)
      set size  [subst $\::${tile}\::Size]
      
      set name  [eval \::${tile}\::Create  $topLevelWin]

      Debug::Puts Screens 2 "name:  $name"
      
      if { ![winfo exists $name] } \
      {
        # If tile does not exist, add to list of missing
        lappend MissedTiles $tile
      } \
      else \
      {
        set loc  [Screens::AllocateSpace  $topLevelWin  $size]
        
        if { $loc == 0 } \
        {
          # If no space exists, add tile to list of missing
          lappend MissedTiles $tile
        } \
        else \
        {
          grid $name -in $mainWin                 \
                     -col [lindex $loc 0]         \
                     -row [lindex $loc 1]         \
                     -columnspan [lindex $loc 2]  \
                     -rowspan [lindex $loc 3]     \
                     -sticky news

          raise $name
        }
      }
    }

    
    # Force a screen area with a 4 x 4 grid with equal weighting
    for { set i 0 } { $i < 4 } { incr i 1 } \
    {
      grid rowconfigure     $mainWin $i  -weight 1 -minsize 110
      grid columnconfigure  $mainWin $i  -weight 1 -minsize 130
    }
    
    grid propagate $mainWin 0
    
    return
  }


  #****************************************************************************
  #
  # Name:        ShowOptionTiles
  #
  # Parameters:  topLevelWin     - toplevel to pack tiles into
  #              ScreenNamespace - namespace of screen to be shown
  #
  # Description: Displays the ActiveTiles for the selected screen
  #
  # Returns:     
  #
  #****************************************************************************
  proc ShowOptionTiles { topLevelWin  ScreenNamespace } \
  {
    Debug::Puts Screens 2 "ShowOptionTiles:  $topLevelWin,  $ScreenNamespace"
    
    return
  }


  #****************************************************************************
  #
  # Name:        Menu_accept
  #
  # Parameters:  widget          - menu making the request
  #              ScreenNamespace - namespace of screen to be shown
  #
  # Description: Handles the 'Accept Options' command of the Options menu
  #
  # Returns:     none
  #
  #****************************************************************************
  proc Menu_accept { menuName  ScreenNamespace } \
  {
    Debug::Puts Screens 2 "Menu_accept:  $menuName,  $ScreenNamespace"

    # Copy the new state to the active state
    upvar #0 ${ScreenNamespace}::New_menu    new
     
         
    # We are a menu, which is it's own toplevel. get the toplevel of our parent
    set topWin  [winfo toplevel [winfo parent $menuName]]
     
    set \::${ScreenNamespace}\::ActiveTiles  {}
     
    # Iterate over the menu items
    set menubutton  [View::GetOptionMenuName  $topWin]
    
    set max_index  [${menubutton}.menu  index  end]
    
    for { set index  1 } { $index < $max_index } { incr index  1 } \
    {      
      set var  [${menubutton}.menu  entrycget  $index  -variable]
      
      upvar #0 $var var_value
      
      Debug::Puts Screens 2 "var_value:  $var_value"
      if {("$var_value" != "") && ($var_value != 0)} \
      {
          AddTile  $ScreenNamespace  $var_value
      }
    }

    tkMenuUnpost $menuName

    # Now pack the tiles on the screen
    ShowRequiredTiles  $topWin  $ScreenNamespace
    
    return
  }
  

  #****************************************************************************
  #
  # Name:        Menu_escape
  #
  # Parameters:  widget          - menu making the request
  #              ScreenNamespace - namespace of screen to be shown
  #
  # Description: Handles the 'Escape' command of the Options menu
  #
  # Returns:     none
  #
  #****************************************************************************
  proc Menu_escape { menuName  ScreenNamespace } \
  {
    Debug::Puts Screens 2 "Menu_escape:  $menuName,  $ScreenNamespace"

    # Restore the present selection of Active Tiles
    set ActiveTiles  [subst $\::${ScreenNamespace}\::ActiveTiles]
    
    foreach { Tile } $ActiveTiles \
    {
      set ::${ScreenNamespace}::New_menu($Tile)  $Tile
    }

    tkMenuUnpost $menuName

    return
  }


  #****************************************************************************
  #
  # Name:        Menu_select
  #
  # Parameters:  widget          - menu making the request
  #              ScreenNamespace - namespace of screen to be shown
  #
  # Description: Handles the selections of the Options menu.
  #              Used to determine if there is enough romm on the screen
  #              for the selected Options.
  #
  # Returns:     none
  #
  #****************************************************************************
  proc Menu_select { menuName  ScreenNamespace  index  onval  offval } \
  {
    Debug::Puts Screens 2 "Menu_select:  $menuName,  $ScreenNamespace,  \
                                                   $index,  $onval,  $offval"

    # This procedure is bypassed
    return
    # This procedure is bypassed
    
    
    set AvailableSpace  1.0
    
    set RequiredTiles  [subst $\::${ScreenNamespace}\::RequiredTiles]

    # Subtract the available space for the Required Tiles
    foreach { TileNamespace } $RequiredTiles \
    {
      set TileSize  [subst $\::${TileNamespace}\::Size]
      
      switch -- $TileSize \
      {
        "Full"       { set size  1.0    }
        "Half_v"     { set size  0.5    }
        "Half_h"     { set size  0.5    }
        "Quarter"    { set size  0.25   }
        "Eighth_v"   { set size  0.125  }
        "Eighth_h"   { set size  0.125  }
        "Sixteenth"  { set size  0.0625 }
      }
      
      set AvailableSpace  [expr  $AvailableSpace - $size]
    }
    
    Debug::Puts Screens 2 "  AvailableSpace:  $AvailableSpace"
    
    # Go through the menu items and subtract the space of the enabled
    # options from the available space
    # If the available space is less than 0.0, set the menu item to the
    # 'offvalue' and return
    set NumberOptions  [$menuName  index  end]
    
    # Start from index 1 to skip the 'Accept Options' command button
    # End with last index value minus 1 to avoid the 'Escape' command button
    for { set i  1 } { $i < [expr $NumberOptions - 1] } { incr i  1 } \
    {
      # Get the menu item variable
      set IndexVariable   [$menuName  entrycget  $i  -variable]
 
      # Get the value menu item of the variable
      set IndexVarValue  [subst $$IndexVariable]
      
      # Get the menu item ON value
      set IndexOnValue  [$menuName  entrycget  $i  -onvalue]
      
      # The namespace of the menu item is the same as the ON value
      set IndexNamespace  $IndexOnValue

      set TileSize  [subst $\::${IndexNamespace}\::Size]
      
      switch -- $TileSize \
      {
        "Full"       { set size  1.0    }
        "Half_v"     { set size  0.5    }
        "Half_h"     { set size  0.5    }
        "Quarter"    { set size  0.25   }
        "Eighth_v"   { set size  0.125  }
        "Eighth_h"   { set size  0.125  }
        "Sixteenth"  { set size  0.0625 }
      }
      
      # If the menu item is set the ON value, subtract the size of the
      # tile from the available space
      if { $IndexVarValue == $IndexOnValue } \
      {
        set AvailableSpace  [expr  $AvailableSpace - $size]
      }

      Debug::Puts Screens 2 "  AvailableSpace:  $AvailableSpace"

      # The available space is less than 0.0 with this menu item selected
      if { $AvailableSpace < 0.0 } \
      {
        # Set the presently selected menu item to the 'offvalue'
        set ::${ScreenNamespace}::New_menu($index)  $offval
        
        # Display an error dialog box
        set title  "Operator Error"

        set prompt_text  "Too many items have been selected."
    
        set window_name  ${menuName}.error

        set prompt_return  \
               [tk_dialog  $window_name  $title  $prompt_text  error  0  "OK"]
        
        return
      }
    }
    

    return
  }


  #****************************************************************************
  #
  # Name:        Menu_config
  #
  # Parameters:  widget          - Screen window needing the options menu
  #              ScreenNamespace - namespace of screen to be shown
  #
  # Description: Configures the state of the Options menu entries to the 
  #              ActiveTiles list
  #
  # Returns:     none
  #
  #****************************************************************************
  proc Menu_config { menuName  ScreenNamespace } \
  {
    
    Debug::Puts Screens 2 "Menu_config:  $menuName,  $ScreenNamespace"

    # Get the screen's variables into the local scope
    upvar #0 ${ScreenNamespace}::New_menu     new
    upvar #0 ${ScreenNamespace}::ActiveTiles  active

    Debug::Puts Screens 2 "  active:  $active"
    
    # Get the option menu list for this screen
    set optionsMenuList  [Screens::GetStdOptionsList  $ScreenNamespace]
        
    Debug::Puts Screens 2 "  optionsMenuList:  $optionsMenuList"
    
    # Iterate over the state, find the active tiles
    foreach { menuItem } $optionsMenuList \
    {
      Debug::Puts Screens 2 "  menuItem:  $menuItem"
      
      set index   $menuItem
      set onval   $menuItem
      set offval  0
      
      if { [lsearch -exact $active $menuItem] == -1 } \
      {
        if { "$offval" != "" } \
        {
          set new($index)  $offval
        } \
        else \
        {
           if { ![info exists new($index)] } \
           {
              set new($index) ""
           }
        }
      } \
      else \
      {
        set new($index)  $onval
      }
    }
        
    return
  }


  #****************************************************************************
  #
  # Name:        Build_std_options
  #
  # Parameters:  widget          - Screen window needing the options menu
  #              ScreenNamespace - namespace of screen to be shown
  #
  # Description: Builds an options menu for a screen
  #
  # Returns:     The name of an options menu
  #
  #****************************************************************************
  proc Build_std_options { widget  ScreenNamespace } \
  {
    Debug::Puts Screens 2 "Build_std_options:  $widget,  $ScreenNamespace"

    # Do basic sanity checks before we try to run
    if { ![winfo exists $widget] } \
    {
      return 0
    }

    # Generate the menu as a child of the toplevel, 
    # and using the name of the screen space
    set topWin  [winfo toplevel $widget]
    
    # Get the Options menubutton path
    set buttonName  [View::GetOptionMenuName  $topWin]

    # Setup the help for the selected mode
    option add  *$buttonName*helpUrl  index.html  widgetDefault

    # Get the Options menu path
    set menuName  ${buttonName}.menu

    # Destroy the current menu
    destroy  $menuName
 
    # Create the new Options menu
    menu $menuName -tearoff 0  -postcommand ""

    # Bind button release so default bindings do not unpost the menu
    # bind to the Return and Escape hardkeys
    bind $menuName <KeyRelease-F8>  \
                          "Screens::Menu_accept  %W  $ScreenNamespace; break"
                          
    bind $menuName <Key-F8>  "break"
        
    bind $menuName <KeyRelease-Return> { MenuInvokeRemain %W 0; break }
    bind $menuName <ButtonRelease-1>   { MenuInvokeRemain %W 1; break }
    
    bind $menuName <Destroy>  \
                  "Screens::Cleanup_std_options  $menuName  $ScreenNamespace"


    # Add the LogKeyStrokes bindtag
    add_bindtag  $menuName  LogKeyStrokes


    Trace_std_options  $menuName  $ScreenNamespace

    Rebuild_std_options_core  $menuName  $ScreenNamespace
    
    return $buttonName
  }


  proc Trace_std_options { menuName  ScreenNamespace } \
  {
    Debug::Puts Screens 2 "Trace_std_options:  $menuName,  $ScreenNamespace"

    trace variable ::${ScreenNamespace}::RequiredTiles     w  \
                    "Screens::Rebuild_std_options $menuName $ScreenNamespace"
                    
    trace variable ::${ScreenNamespace}::NotAllowedTiles   w  \
                    "Screens::Rebuild_std_options $menuName $ScreenNamespace"
                   
    trace variable ::${ScreenNamespace}::RecommendedTiles  w  \
                    "Screens::Rebuild_std_options $menuName $ScreenNamespace"
    
    return
  }


  proc Cleanup_std_options { menuName  ScreenNamespace } \
  {
    Debug::Puts Screens 2 "Cleanup_std_options:  $menuName,  $ScreenNamespace"

    trace vdelete ::${ScreenNamespace}::RequiredTiles     w  \
                    "Screens::Rebuild_std_options $menuName $ScreenNamespace"
                    
    trace vdelete ::${ScreenNamespace}::NotAllowedTiles   w  \
                    "Screens::Rebuild_std_options $menuName $ScreenNamespace"
                   
    trace vdelete ::${ScreenNamespace}::RecommendedTiles  w  \
                    "Screens::Rebuild_std_options $menuName $ScreenNamespace"

    return
  }


  proc Rebuild_std_options { menuName  ScreenNamespace  args } \
  {
    Debug::Puts Screens 2 "Rebuild_std_options:  $menuName,  $ScreenNamespace"

    set cmd  "Screens::Rebuild_std_options_core $menuName $ScreenNamespace"
    
    after cancel $cmd
    after idle $cmd

    return
  }


  proc Rebuild_std_options_core { menuName  ScreenNamespace } \
  {
    Debug::Puts Screens 2 "Rebuild_std_options_core:  $menuName,  $ScreenNamespace"
    
    set optionsMenuList  [Screens::GetStdOptionsList  $ScreenNamespace]
    set selectedList     [subst $\::${ScreenNamespace}\::ActiveTiles]
    
    Debug::Puts Screens 2 "  selectedList:  $selectedList"
    
    # Set the acceleration index based on the list size
    set listSize  [llength $optionsMenuList]
    # add RETURN and ESC entries
    incr listSize 2
    
    if { $listSize < 10 } \
    {
      set accelNum 0
      set format %1d
      set max_col_items 10
    } \
    else \
    {
      set accelNum 00
      set format %02d

      # Added to allow room for a long options list
      $menuName config -font menu_small
      # split roughly in half if possible
      set max_col_items [expr int(($listSize+1)/2)]
      if {$max_col_items > 20} \
      {
         set max_col_items 20
      }
    }

    # Add ENTER key options
    $menuName add command -accel  RETURN            \
                          -label  "Accept Options"  \
                          -command  [list after idle "Screens::Menu_accept  $menuName  $ScreenNamespace"]


    upvar #0 ${ScreenNamespace}::New_menu    new
   
    set columnbreak        false
    set columnbreak_index  1
    
    foreach { menuItem } $optionsMenuList \
    {
      set subMenuName  [subst "$\::${menuItem}\::ShortName"]
      
      set var  ${ScreenNamespace}::New_menu($menuItem)
      
      set index   $menuItem
      set onval   $menuItem
      set offval  0
      
      if { $columnbreak_index == $max_col_items } \
      {
        set columnbreak        true
        set columnbreak_index  0
      } \
      else \
      {
        set columnbreak  false
      }
      set ac [format $format $accelNum]
      $menuName add check                                          \
             -accel        $ac                                     \
             -label        $subMenuName                            \
             -variable     ::${ScreenNamespace}::New_menu($index)  \
             -onvalue      $onval                                  \
             -offvalue     $offval                                 \
             -columnbreak  $columnbreak                            \
             -command      "Screens::Menu_select  $menuName  $ScreenNamespace \
                                                       $index  $onval  $offval"
     
      set end  [$menuName index end]

      bind $menuName $ac "%W invoke $end; event generate %W ,"
 
      incr  accelNum
      incr  columnbreak_index
    }

    # Add ESCAPE functionality
    $menuName add command -accel    ESC              \
                          -label    "Escape"         \
                          -command  "Screens::Menu_escape  $menuName  $ScreenNamespace"

    # Initialize the 'New_menu' elements to the 'ActiveTiles' elements
    Screens::Menu_config  $menuName  $ScreenNamespace

    return
  }
  
  
  proc Cross_check { cross_check_cmd  ScreenNamespace  args } \
  {
    set cmd  "$cross_check_cmd  $ScreenNamespace"

    after cancel $cmd
    after idle $cmd
    
    return
  }

  
  #****************************************************************************
  #
  # Name:        AddTile
  #
  # Parameters:  ScreenNamespace - namespace of screen having standard tiles
  #              TileNamespace   - namespace of tile being added
  #
  # Description: Adds the tile to the ActiveTile list in the screen namespace
  #
  #****************************************************************************
  proc AddTile { ScreenNamespace  TileNamespace } \
  {
    Debug::Puts Screens 2 "AddTile:  $ScreenNamespace,  $TileNamespace"

    # Verify that the tile is allowed
    set tempList  [subst $\::${ScreenNamespace}\::NotAllowedTiles] 
  
    if { [lsearch -exact $tempList $TileNamespace] != -1 } \
    {
      return
    }

    set tempList  [subst $\::${ScreenNamespace}\::RequiredTiles] 
    
    if { [lsearch -exact $tempList $TileNamespace] != -1 } \
    {
      return
    }

    set tempList  [subst $\::${ScreenNamespace}\::ActiveTiles] 
    
    if { ([lsearch -exact $tempList $TileNamespace] == -1 ) &&
         ([namespace children :: $TileNamespace] != "") } \
    {
      lappend \::${ScreenNamespace}\::ActiveTiles  $TileNamespace
    }
    
    return
  }


  #****************************************************************************
  #
  # Name:        RemoveTile
  #
  # Parameters:  ScreenNamespace - namespace of screen having standard tiles
  #              TileNamespace   - namespace of tile being added
  #
  # Description: Removes the tile to the ActiveTile list in the screen namespace
  #
  #****************************************************************************
  proc RemoveTile { ScreenNamespace  TileNamespace } \
  {
    Debug::Puts Screens 2 "RemoveTile:  $ScreenNamespace,  $TileNamespace"

    set tempList  [subst $\::${ScreenNamespace}\::ActiveTiles]
    
    set i  [lsearch -exact $tempList $TileNamespace]
  
    if { $i != -1 } \
    {
      set \::${ScreenNamespace}\::ActiveTiles  [lreplace $tempList $i $i]
    }
    
    return
  }
  

  #****************************************************************************
  #
  # Name:        Show
  #
  # Parameters:  topLevelWin     - toplevel window for screen
  #              ScreenNamespace - namespace of screen having standard tiles 
  #
  # Description: Cleans the current screen and displays the new screen
  #
  #****************************************************************************
  proc Show { topLevelWin  ScreenNamespace } \
  {
    variable show_running
    if {[info exists show_running] && $show_running} { return } 
    set show_running 1
    
    Busy
    oldCatch \
    {
        Debug::Puts Screens 2 "Show:  $topLevelWin,  $ScreenNamespace"
        
        # Do basic sanity checks on the window and namespace
        if { ![winfo exists $topLevelWin] } \
        {
          return
        }
        
        # Insure we are talking about the top level window
        set topLevelWin  [winfo toplevel $topLevelWin]
      
        # Is this a "transient" entry? 
        # If so, just invoke it and exit
        if { [info exists \::${ScreenNamespace}\::Is_transient] } \
        {
          set child  [\::${ScreenNamespace}\::Setup  $topLevelWin]

          return
        }
        update idle
        # Set the screen name in the menu
        upvar 0 ::${ScreenNamespace}::ShortName  shortname

        # Set the screen name in the menu
        set menu  [View::GetMainMenuName  $topLevelWin]

        $menu configure -text $shortname
            
        # Set the focus to the menu while the screens are changing
        safe_focus_force $menu
        
        if { ![info exists \::${ScreenNamespace}\::RequiredTiles] } \
        {
          return
        }

        Screens::Clear  $topLevelWin

        View::ClearAllSoftkeys  $topLevelWin
        
        # Erase the options menu. If they want an options menu, let them say so
        View::PackOptions  $topLevelWin

        set child  [\::${ScreenNamespace}\::Setup  $topLevelWin]

        if { $child != "" } \
        {
          set parent  [View::GetMainView  $topLevelWin]

          pack propagate $parent false

          pack $child -in $parent -expand true -fill both
        }
        
        # Update our local state if needed...
        if { "$topLevelWin" == "." } \
        {
          variable  Local_screen

          set Local_screen  $ScreenNamespace
        }

        # kick the clock, in case it went stupid
        ::TimeDate_Screen::Clock

    }
    # Go not busy after half a second
    set show_running 0
    after 500 NotBusy
  }


  #****************************************************************************
  #
  # Name:        Show_local
  #
  # Parameters:  none
  #
  # Description: sets the current local screen to be whatever Local_screen is
  #
  #****************************************************************************
  proc Show_local { } \
  {
    Debug::Puts Screens 2 "Show_local:  "

    after cancel Screens::Show_local

    variable Local_screen

    if { $Local_screen == "" } \
    {
      set Local_screen Intro_Screen
    }

    catch { Show . $Local_screen } err 
    
    return
  }
  
  #****************************************************************************
  # Name:        Show_remote
  #
  # Parameters:  remote_widget  The widget invoking a recall of a setup
  # Description: sets the remote screen to be whatever Local_screen is
  #     This function is used by remote screens to force an update of their
  #     screens when a "setup" is recalled or "factory default" is pressed
  #****************************************************************************
  proc Show_remote { remote_widget } \
  {
    Debug::Puts Screens 2 "Show_remote: $remote_widget"
puts "Hit Show_remote: $remote_widget"

    after cancel Screens::Show_remote

    variable Local_screen

    if { $Local_screen == "" } \
    {
      set Local_screen Intro_Screen
    }

    set remote_screen [Get_main_screen $remote_widget]
    if { $remote_screen != "." } \
    {
puts "Show_remote: activating local screen $Local_screen"
       catch { Show $remote_screen $Local_screen } err 
    }
  }

  #****************************************************************************
  #
  # Name:        getSaveState
  #
  # Parameters:  none
  #
  # Description: This function returns a list of TCL commands which can be used
  #              to set the Screen to its current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    variable Local_screen
   
    if { $Local_screen == "" } { return }
   
    set returnList \
    {
      set ::Screens::Local_screen  $Local_screen
      
      Screens::Show_local
    }
    
    return [subst $returnList]
  }
 
 
  #****************************************************************************
  #
  # Name:        Get_main_screen
  #
  # Parameters:  widget - path of the invoking widget
  #
  # Description: Locate the main screen of that widget
  #
  #****************************************************************************
  proc Get_main_screen { widget } \
  {
    Debug::Puts Screens 2 "Get_main_screen:  $widget"
    
    while { [winfo exists $widget] } \
    {
      if { "[winfo class $widget]" == "IFR-2975" } \
      {
        return $widget
      }

      set widget [winfo parent $widget]
    }
  
    return .
  }
 
 
};  # End of the Screens namespace

# we don't hook into the saved screens list - we must be the last item done.
