#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2002      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2002.softkeys.tcl                                           #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file contains the definition for a collection of       #
#             : soft key widgets.                                           #
#############################################################################
#  Softkey design
#     o  Soft keys will be defined as widgets which are packed into a frame
#        on the softkey bar.  They should have their own frame that all parts
#        of the key are packed into, and that frame needs to be a child of 
#        the top level window which owns the softkey bar.  
#     o  All softkeys will be identified by NAME, rather than position.  This
#        allows them to be reused by other screens and to have their position 
#        controlled by a softkey manager.  The softkey create function must be
#        capable of finding the toplevel of the softkey bar, and merging the
#        generic name with this to create an instance name.
#     o  All soft keys will be assigned a priority by the function which 
#        creates them.  1 = Low priority, 2-3 = Medium priority, 4-5 high priority
#############################################################################
#  Softkey objects
#     o  "DualSkey" is a two button softkey, with a label field.  It is 
#        connected with a TCL variable and automatically changes states based
#        on the state of that variable (which normally is a two-state variable)
#############################################################################

namespace eval SoftKeyUtils \
{
  proc Locate_keybar { widget } \
  {
     set toplevel [View::FindSoftKeyOwner $widget]
     if ![winfo exists $toplevel] { return }
     # locate the currently active softkey strip
     foreach item [pack slaves $toplevel] \
     {
        if {[winfo class $item] == "KEY_BAR"} { return $item }
     }
     return -code return
  }
  
  #****************************************************************************
  # Name:        Regenerate
  # Parameters:  widget		Path for the frame of the softkey
  # Description: This routine handles housecleaning when the softkey is unmapped
  #****************************************************************************
  proc Regenerate { widget key action } \
  {
     set keybar [Locate_keybar $widget]
     if ![winfo exists $keybar] { return }
     set frame $keybar.$key
     set softFrame [pack slaves $frame]
     foreach item $softFrame \
     {
        event generate $item $action
     }
     event generate $frame $action
  }
  
  for { set i  2 } { $i < 8 } { incr i  1 } \
  {
      bind all <F$i>            "::SoftKeyUtils::Regenerate %W frm$i <<Press>>"
      bind all <KeyRelease-F$i> "::SoftKeyUtils::Regenerate %W frm$i <<Release>>"
    }
  
    bind all <F8>            "::SoftKeyUtils::Regenerate %W return <<Press>>   "
    bind all <KeyRelease-F8> "::SoftKeyUtils::Regenerate %W return <<Release>> "


};  # End of the SoftKeyUtils namespace








#****************************************************************************
#
#  DualSkey Namespace  
#
#****************************************************************************
#  Private Variables:
#
#     RefVariable       TCL variable used to control which button is depressed.
#                       Normally it should only have two states
#     RefValue          The value of the RefVariable which should cause the
#                       LEFT button to be depressed.
#     Trace             Contains a string used to create and remove a trace on
#                       a TCL variable which triggers a display change
#     Action            Contains a string used to bind to the button
#****************************************************************************
#  Public Functions:
#     Assign            Primary call to create a dual softkey widget
#     GetFrameName      Obtains softkey widget frame name
#     Map		Called when softkey widget is mapped on screen
#     Unmap		Called when softkey widget is unmapped
#****************************************************************************
#  Private Functions:
#     Create		Creates a dual softkey
#     Update		Updates the softkey display based on variable status
#****************************************************************************

namespace eval DualSkey \
{
  variable RefVariable
  variable RefValue
  variable Trace
  variable Action
  variable Priority

  #****************************************************************************
  # Name:        GetFrameName
  # Parameters:  widget		Path for widget controlling the softkey
  #              name		Name which identifies the softkey
  # Description: This routine defines the path name for the Dual Key frame
  #****************************************************************************
  proc GetFrameName { widget name } \
  {
     set topLevelWin [winfo toplevel $widget]
     set fname [name_child $topLevelWin dualsk_$name]
     return $fname
  } 

  #****************************************************************************
  # Name:        Create
  # Parameters:  widget		Path for widget controlling the softkey
  #              name		Name which identifies the softkey
  # Description: This routine creates the Dual Key frame, the label fields and
  #              buttons, and returns its name
  #****************************************************************************
  proc Create { widget name } \
  {
     set fname [GetFrameName $widget $name]
     if { ![winfo exists $fname] } \
     {
        frame $fname -class DUAL_SK_FRAME
        label $fname.label -justify center
        button $fname.sk_left -takefocus 0
        button $fname.sk_right -takefocus 0
        pack $fname.label    -side top 
        pack $fname.sk_left $fname.sk_right -side left -expand 1 -fill x -ipady 2
     }

     return $fname
  } 

  #****************************************************************************
  # Name:        Assign
  # Parameters:  widget		Path for widget controlling the softkey
  #              name		Name of the softkey
  #              priority       Numeric value of key importance
  #              label          General button label
  #              left_label     Text on left button
  #              right_label    Text on right button
  #              action         What to do when the softkey is pressed, normally
  #                             this should command a hardware change
  #              color          Optional background color
  # Description: Defines the parameters for a DUAL softkey, creating the widget
  #              if needed
  #****************************************************************************
  proc Assign { widget name priority label left_label right_label variable left_value \
                sk_action lk_action rk_action { color grey } } \
  {
     variable 	RefVariable
     variable   RefValue
     variable   Trace
     variable   Action
     variable   Priority

     set fname [Create $widget $name]
     $fname.label    configure -text $label
     $fname.sk_left  configure -text $left_label  -com $lk_action -bg $color -activebackground $color -font dual_key
     $fname.sk_right configure -text $right_label -com $rk_action -bg $color -activebackground $color -font dual_key

     set RefVariable($fname) $variable
     set RefValue($fname) $left_value
     set Trace($fname) "$variable w \"::DualSkey::Update $fname\""
     set Action($fname) $sk_action
     set Priority($fname) $priority

     bind $fname <<Press>>    ""
     bind $fname <<Release>>  "$::DualSkey::Action($fname)"
     bind $fname <Map> "::DualSkey::Map $fname"
     bind $fname <Unmap> "::DualSkey::Unmap $fname"

     return $fname
  }

  #****************************************************************************
  # Name:        Update
  # Parameters:  fname		Path for the frame of the softkey
  #              args		Any callback information
  # Description: This routine adjusts the dual button setting based on the 
  #              current status of the button variable
  #****************************************************************************
  proc Update { fname args } \
  {
     variable 	RefVariable
     variable   RefValue

     upvar #0 $RefVariable($fname) value

     if { $value == $RefValue($fname) } \
     {
        $fname.sk_left   config -relief sunken
        $fname.sk_right  config -relief raised
     } else \
     {
        $fname.sk_left   config -relief raised
        $fname.sk_right  config -relief sunken
     }
  } 

  #****************************************************************************
  # Name:        Map
  # Parameters:  fname		Path for the frame of the softkey
  #              args		Any callback information
  # Description: This routine adjusts the dual button setting based on the 
  #              current status of the button variable
  #****************************************************************************
  proc Map { widget } \
  {
     variable   Trace

     eval trace variable $Trace($widget)
     Update $widget
  }

  #****************************************************************************
  # Name:        Unmap
  # Parameters:  widget		Path for the frame of the softkey
  # Description: This routine handles housecleaning when the softkey is unmapped
  #****************************************************************************
  proc Unmap { widget } \
  {
     variable   Trace

     eval trace vdelete $Trace($widget)
  }

  #****************************************************************************
  # Name:        AssignRcvInput
  # Parameters:  widget		Path for the widget desiring the softkey
  #              id		Softkey position to use
  #              priority       Importance for the key to be displayed
  #              color          Background color for the softkey
  # Description: This routine handles housecleaning when the softkey is unmapped
  #****************************************************************************
  proc AssignRcvInput { widget id {priority 1} { color grey } } \
  {
     set fullname [::DualSkey::Assign $widget RcvInPort $priority "Input" "ANT" "T/R"  \
                            ::Receiver::State(rfinput) "ANT" "::Receiver::toggle_input" \
                            "Receiver rfinput ANT" "Receiver rfinput T/R" $color]
     ::View::AssignDualSoftkey $widget $id $fullname
  }

  #****************************************************************************
  # Name:        AssignGenOutput
  # Parameters:  widget		Path for the widget desiring the softkey
  #              id		Softkey position to use
  #              priority       Importance for the key to be displayed
  #              color          Background color for the softkey
  # Description: This routine handles housecleaning when the softkey is unmapped
  #****************************************************************************
  proc AssignGenOutput { widget id {priority 1} { color grey } } \
  {
     set fullname [::DualSkey::Assign $widget GenOutPort $priority "TG Output" "GEN" "T/R"  \
                            ::Generator::State(rfout) "GEN" "::Generator::toggleRFoutput" \
                            "Generator rfoutput GEN" "Generator rfoutput T/R" $color]
     ::View::AssignDualSoftkey $widget $id $fullname
  }

  #****************************************************************************
  # Name:        AssignTraceRun
  # Parameters:  widget		Path for the widget desiring the softkey
  #              id		Softkey position to use
  #              priority       Importance for the key to be displayed
  #              color          Background color for the softkey
  # Description: This routine handles housecleaning when the softkey is unmapped
  #****************************************************************************
  proc AssignTraceRun { widget id {priority 1} { color grey } } \
  {
     set fullname [::DualSkey::Assign $widget AnlzTraceRun $priority "Trace" "STOP" "RUN"  \
                   ::Analyzer::State(run) "STOP" "::AnalyzerUI::toggle_start_stop $widget" \
                   "::AnalyzerUI::set_start_stop $widget STOP" \
                   "::AnalyzerUI::set_start_stop $widget START" $color]
     ::View::AssignDualSoftkey $widget $id $fullname
  }









};  # End of the DualSkey namespace



