#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2020.rec.tcl                                                #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This namespace manages the Receiver Screen                  #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName        	  Name of screen                                    #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Dummy Functions:                                                          #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval Rec_Screen \
{
  variable LongName
  variable ShortName
  variable RecommendedTiles
  variable NotAllowedTiles
  variable RequiredTiles
  variable ActiveTiles
  variable MenuFlags
  variable SubmenuScreens
  variable SubDirPath

  set LongName  "receiver_Screen"
  set ShortName "Receiver (TX Test)"

  set RequiredTiles [ list ReceiverUI ]

  set RecommendedTiles \
  {
     QtrScopeUI 
     Analyzer_QtrUI
     SinadUI 
     DistortionUI 
     CounterUI 
     DvmUI 
     Dev_C4fmUI 
     PowerUI 
     RssiUI 
     RferrUI
     BerMtrUI
     MultimeterUI 
     FGen_SmallUI 
     UplinkUI
     Tone_Signal_DecodeUI
  }

  # Put EVM first since it should always be installed on new systems
  if { [IFR_options  is_installed  EVM] } \
  {
    # Add the EVM option
    lappend RecommendedTiles  EVM_Plot_QtrUI  EVM_DataUI
  }
  
  if { [IFR_options  is_installed  Audio_Analyzer] } \
  {
    lappend RecommendedTiles  Audio_Analyzer_QtrUI
  }

    if { [IFR_options  is_installed  P25_trunk] || [IFR_options  is_installed  P25_OBT] } \
  {
    lappend RecommendedTiles  P25_Repeat_SimUI
  }

  if { [IFR_options  is_installed  P25_CTRL_LOG] }  \
  {
    lappend RecommendedTiles  P25_LoggerUI
  }

  if { [IFR_options  is_installed  SzSnet] } \
  {
    # Add Smartnet option
    lappend RecommendedTiles  SzSnet_Repeat_SimUI  SzSnet_ScannerUI
  }
  
  # Call the procedure to check for the SmartNet Monitor option
  if { [SzSnet_MonitorUI::check_for_option] } \
  {
    lappend RecommendedTiles  SzSnet_MonitorUI
  }


  if { [IFR_options  is_installed  LTR] } \
  {
    lappend RecommendedTiles  LTR_Repeat_SimUI  LTR_Radio_SimUI  LTR_MonitorUI
  }


  if { [IFR_options  is_installed  PASSPORT] } \
  {
    lappend RecommendedTiles  Passport_Repeat_SimUI  Passport_Radio_SimUI
  }
  
  if { [IFR_options  is_installed  PASSPORT_DEVELOP] } \
  {
    lappend RecommendedTiles  Passport_MonitorUI
  }

  set NotAllowedTiles  { }
  set ActiveTiles      { Analyzer_QtrUI SinadUI CounterUI Dev_C4fmUI RssiUI }
  set SubmenuScreens   { }
  set SubDirPath       { }
  
  set MenuFlags 0


  #****************************************************************************
  # Name:        Setup
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  # Returns:     Indicator that no tile is generated
  #****************************************************************************
  proc Setup { parent } \
  {
    # Use the Screen functions to manage our tiles
    set topWin  [winfo toplevel $parent]
     
    # Use the Screen functions to manage our options menu
    set dropMenu  [Screens::Build_std_options  $topWin  Rec_Screen]
     
    View::PackOptions  $topWin  $dropMenu
    
    View::PackKeyBar  $topWin

    Screens::ShowRequiredTiles  $topWin  Rec_Screen

    return  ""
  }


  #****************************************************************************
  # Name:        Map
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Map { name } \
  {
    
  }


  #****************************************************************************
  # Name:        Unmap
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Unmap { name } \
  {
    
  }


  #****************************************************************************
  # Name:        Destroy
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Destroy { name } \
  {
    
  }


  #****************************************************************************
  # Name:        getSaveState
  # Parameters:  none
  # Description: This function returns a list of TCL commands which can be used
  #              to set the Rec_Screen screen to its current state
  #****************************************************************************
  proc getSaveState { } \
  {
    set returnList \
    {
      set Rec_Screen::ActiveTiles {$Rec_Screen::ActiveTiles}
    }
   
    return [subst $returnList]
  }


  #****************************************************************************
  # Functions executed when the namespace is sourced
  #****************************************************************************
  # Add this on the menu
  ::View::RegisterScreen  Rec_Screen


};  # End of the Rec_Screen



#****************************************************************************
# This registers the screens with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable screens   

  lappend screens Rec_Screen
}



