#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001-2002      DESIGN ENGINEERING   #
# --------------------------------------------------------------------------#
# FILE        : 2099.reload.tcl                                             #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This namespace is a dummy "screen" that sources and holds   #
#             : the submenu namespace list for the "Systems" droplist       #
#############################################################################
#                                                                           #
# Required Variables:                                                       #
#    LongName		        Name of screen                              #
#    ShortName          Name of screen for menus                            #
#    RecomendedTiles    List of tiles available for this screen. It is not  #
#                       necessary to put a tile on this list that is on the #
#                       RequiredTiles list                                  #
#    NotAllowedTiles    List of tiles NEVER on this screen                  #
#    RequiredTiles      List of tiles ALWAYS on this screen                 #
#    ActiveTiles        List of tiles currently displayed on this screen    #
#    MenuFlags          0 = Active Screen                                   #
#                       1 = Currently functioning as a submenu parent       #
#                                                                           #
# Required Variables for submenu "Screens":                                 #
#    SubmenuScreens     List of all tiles (or submenus) for droplist        #
#    SubDirPath         Path to subdirectory holding the screens for the    #
#                       submenu of this "screen"                            #
#                                                                           #
#############################################################################
#                                                                           #
# Dummy Functions:                                                          #
#    Setup              Called by the screen manager to create the screen.  #
#    Map                Called when screen is mapped on window              #
#    Unmap              Called when screen is removed from display          #
#    Destroy            Called when screen is destroyed                     #
#                                                                           #
#############################################################################

namespace eval Reload_Screen \
{
  variable LongName
  variable ShortName
  variable RecommendedTiles
  variable NotAllowedTiles
  variable RequiredTiles
  variable ActiveTiles
  variable MenuFlags
  variable SubmenuScreens
  variable SubDirPath

  set LongName Reload_Screen
  set ShortName "Reload"
  set RequiredTiles { }
  set NotAllowedTiles { }
  set RecommendedTiles { }
  set ActiveTiles { }
  set MenuFlags 0
  set SubmenuScreens { }
  set SubDirPath { }


  variable tile_path      ""
  variable tile_size      "full"
  variable tile_class     "RELOAD"

  variable files
  array set files \
  {
     path  /usr/reload
     names {reload.tar.gz reload.tgz}
     local /usr/reload/reload.tar.gz
     post  /usr/reload/post.tcl
     tarfile /usr/reload/reload.tar
     local_option /usr/reload/options.new
  }
  variable status
  array set status \
  {
     main ""
     bottom ""
  }

  variable reload
  array set reload \
  {
     gzip_color    blue
     verify_color  blue
     untar_color   blue
     file_color    blue
     decrypt_bytes 0
     decrypt_done  0
     decrypt_msg   ""
     decrypt_color  blue
     post_count    1
     post_done     0
     post_msg      ""
     where         /
  }

   
  #*************************************************
  #       Name: parse_firmware_version
  # Parameters: version to parse, name of array to get values 
  #Description: Parses a version string 
  #*************************************************
  proc parse_firmware_version { ver array_name } {

    upvar $array_name array

    set array(type) other

    # step 1: convert underscores to periods

    regsub -all _ $ver . ver

    # Now, see what form the value takes
    set array(version) $ver

    if {[regexp {^V([\.0-9]+)(/.*)*$} $ver match main extra]} {
      set array(type) version
      set array(base_version) $main
  
    } elseif {[regexp {^DVT([\.0-9]+)(/.*)*$} $ver match main extra]} {
      set array(type) dvt
      set array(base_version) $main
  
    } elseif {[regexp {^([\.0-9]+)(/.*)*$} $ver match main extra]} {
      set array(type) numeric
      set array(base_version) $main
    } else {
      set array(base_version) $ver
    }
  }


  #*************************************************
  #       Name: compare_firmware_version
  # Parameters: 
  #Description: compares passed version to current version
  #       : returns text for html warning frame
  #*************************************************
  proc compare_firmware_version {new_version} {
  
    set current(version) [firmware_version]
    set new(version) $new_version
    set frame_text ""
  
    parse_firmware_version $current(version) current
    parse_firmware_version $new(version) new
    
    # CHECK FOR NEW BEING OLDER THAN CURRENT
    if {($new(type) != "other"        && 
         $new(type) == $current(type) && 
         $new(base_version) < $current(base_version)
        )
       } \
    {
         set frame_text "WARNING:\n$new(version) firmware is an earlier version than the currently loaded $current(version) firmware."
         upvar downgrade downgrade
         set downgrade 1
         return $frame_text
    }
  
    # CHECK FOR UNRELEASED VERSION
    if {$new(type) != "version"} {
      append frame_text "WARNING:\n$new(version) firmware is an unreleased firmware version."  
    }
  
    if {$frame_text == ""} {
      set frame_text "$new(version) firmware has been selected for downloading." 
    }
  
    return $frame_text
  }
  
  #*************************************************
  #       Name: download_show_progress
  # Parameters: token, 
  #       : total,
  #       : current
  #Description: updates progress bar
  #*************************************************
  proc download_show_progress { frame token total current } {

    if {$total == 0 } { return }

    update_progress_bar $frame.progress [expr $current*100.0/$total] blue
    update idle
  }
  
  #*************************************************
  #       Name: download_ok
  # Parameters: token, 
  #Description: show download successful message
  #*************************************************
  proc download_ok { frame {next_url {}} } {

     # normal completion, continue

     set top_frame  [::HtmlFrame::frame_of [winfo parent $frame] ]
  
     # user abort
     $top_frame.b6 configure -text "BACK" -command "::HtmlFrame::do_back $top_frame" -state disabled
  
     # DISPLAY DONE
     # if we have a next URL, take them there. Otherwise, show done msg here
     if {"$next_url" != ""} {
         if {![catch {::HtmlFrame::do_url $top_frame $next_url
                   } err ]} { return }
                  
     }

     $frame.text configure -text "DOWNLOAD COMPLETE\nPlease re-read the instructions\
                                    then press the RETURN key to continue"
     pack forget $frame.bar
  }
  
  #*************************************************
  #       Name: download_bad
  # Parameters: token, 
  #Description: show download unsuccessful message
  #*************************************************
  proc download_bad { frame errmsg } {

     set top_frame  [::HtmlFrame::frame_of [winfo parent $frame] ]
  
     set errmsg [::HtmlFrame::encode $errmsg]
     
     ::HtmlFrame::do_url $top_frame \
     $::HtmlFrame::err_base/reload/error.tcl?$errmsg
  }
  
  #*************************************************
  #       Name: download_cancel
  # Parameters: token
  #Description: show download cancelled message
  #*************************************************
  proc download_cancel { frame  } {

     set top_frame  [::HtmlFrame::frame_of [winfo parent $frame] ]
  
     # user abort
     $top_frame.b6 configure -text "BACK" -command "::HtmlFrame::do_back $top_frame" -state disabled
     ::HtmlFrame::do_url $top_frame \
     $::HtmlFrame::err_base/reload/cancel.html
     catch { source /system/default_reloadstate.tcl }
  }
  
  #*************************************************
  #       Name: download_get_done
  # Parameters: token, 
  #Description: closes file, provides done message
  #*************************************************
  proc download_get_done { frame next_url token } {
     variable files
     upvar #0 $token status

     ::http::wait $token

     set top_frame  [::HtmlFrame::frame_of [winfo parent $frame] ]
  
     bind $frame <Destroy> ""
     bind $frame <<DOWNLOAD_DONE>> {}

     $top_frame.b1 configure -text "" -command ""
  
     set filehandle $status(-channel)
     set reload_err $::HtmlFrame::err_base/reload/

     set http_status      [split $status(http)]
     set http_status_code [lindex $http_status 1]
  
     if {$http_status_code == 401 } { 
          # file moved error. The system will automatically go to the new
          # location. All we have to do is rewind the file pointer
           seek $filehandle 0 start
           ::Reload_Screen::download_show_progress $frame token 100 0
           return
     }

     switch -exact -- $status(status) {
       "ok"     { ::Reload_Screen::download_ok     $frame $next_url
                  return
                }
       "cancel" { ::Reload_Screen::download_cancel $frame 
                  close $filehandle
                  file delete $files(local)
                  return

                }
     }
    
     # other error. Stop the transfer, and show the error
     close $filehandle
  

     ::Reload_Screen::download_bad $frame \
     code=$http_status_code&url=$status(url)&err=$status(status)&status=$status(http)
     
  }
  #*************************************************
  #       Name: transfer_option_file
  # Parameters:
  #Description: 
  #*************************************************
  proc transfer_option_file {frame url {next_url ""}} \
  {
     variable files
     $frame.progress.text configure -text  "Downloading option file"
     set cmd [namespace code "transfer_file_core $frame $url $next_url $files(local_option) 8192"]
     after cancel $cmd
     after idle $cmd
  }
  proc transfer_file_core {frame url {next_url ""} filename blocksize} \
  {
    variable files

    if ![winfo exists $frame] { return }
    set top_frame  [::HtmlFrame::frame_of [winfo parent $frame] ]
    
    # Do a fast shut down of most hardware systems, to give TCL more time
#    quick_hardware_reset

    # set the base URL for error messages
    set reload_err $::HtmlFrame::err_base/reload/cant_create_file.tcl
   
    # CREATE DIR IN CASE IT DOES NOT EXISTS
    if [catch {file mkdir $files(path)} err] {
        set err [::HtmlFrame::encode $err]
        set name [::HtmlFrame::encode $files(path)]
        ::HtmlFrame::do_url $top_frame  $reload_err?err=$err&type=directory&name=$name
      return
    }
  
    if [catch { set file [open $filename w]} err] {
       set err [::HtmlFrame::encode $err]
       set name [::HtmlFrame::encode $filename]
       ::HtmlFrame::do_url $top_frame  \
            $reload_err?err=$err&type=file&name=$name
       return
    }
    fconfigure $file -translation binary 
  
    # ALLOW SYSTEM TO DEFER WRITES TO HARD DRIVE

    fconfigure $file -buffersize [expr 10*$blocksize]
   
    ::HtmlFrame::split_url $url URL
   
    switch -exact -- $URL(proto) {
     "http" {
        set token [::HtmlFrame::geturl_with_auth $url -channel $file \
                                    -progress "::Reload_Screen::download_show_progress $frame" \
                                    -command "::Reload_Screen::download_done_event $frame" \
                                    -blocksize $blocksize]

        bind $frame <<DOWNLOAD_DONE>> "::Reload_Screen::download_get_done $frame \"$next_url\" $token"
        $top_frame.b1 configure -text "CANCEL DNLOAD" \
                          -command "::Reload_Screen::cancel_download $frame $token"
        $top_frame.b6 configure -text "BACK" \
                          -command "::Reload_Screen::cancel_download $frame $token"

        if [winfo exists $frame] {bind $frame <Destroy> "+http::reset $token cancel"}
        update idle
     }
     "ftp" {

       if [catch {
            set ftp [::ftp::create $url -channel $file \
                                        -progress "::Reload_Screen::download_show_progress $frame" \
                                        -blocksize $blocksize
                    ]
            $top_frame.b1 configure -text "CANCEL DNLOAD" \
                          -command "::ftp::done $ftp cancel"
            if [winfo exists $frame] {bind $frame <Destroy> "+::ftp::done $ftp cancel"}
            update idle
            upvar #0 $ftp ftp_data
            ::ftp::Open $ftp
            ::ftp::get $ftp
            ::ftp::wait $ftp
            if {"$ftp_data(status)" == "cancel" } {
               ::Reload_Screen::download_cancel $frame
            } else {
              $top_frame.b1 configure -text "" \
                          -command ""
              if [winfo exists $frame] { bind $frame <Destroy> "" }
              ::Reload_Screen::download_ok $top_frame $next_url
            }
            ::ftp::done $ftp
            unset $ftp
           } err ] {
             ::Reload_Screen::download_bad  $frame err=$err&type=file&name=$filename
             close $file
             catch {file delete $files(local_option) }
             return;
          }
      }

      default {
      }
    }
     
  }  
  #**************************************************************************
  # Name: quick_hardware_reset
  # Parameters:   none
  # Description: This function attempts to shut down the basic 2975 hardware
  #     systems to avoid interference during the reload process
  #**************************************************************************
  proc quick_hardware_reset { } \
  {
     # Do a fast shut down of most hardware systems, to give TCL more time
     if [IFR_options is_installed SzSnet]  { ::SzSnet::Sim_turn_off }
     if [IFR_options is_installed LTR]  { ::Ltr::ltrTurnOff }
     if { [IFR_options is_installed EVM] }  {
        Modacc enable  0 -nowait
        Modacc run  0 -nowait
     }
     P25 decode run 0 -nowait
     P25 decode enable 1  -nowait
     Power enable 0  -nowait
     FGen::setMode  1  0 
     FGen::setMode  2  0
     FGen::setMode  3  0
     FGen::setMode  4  0
     C4fm en 0 -nowait
     C4fm run 0 -nowait
     RFError run 0 -nowait
     RFError en 0 -nowait
     FMDev run 0  -nowait 
     FMDev en 0 -nowait
     RLmeter en 0 -nowait
     RLmeter run 0 -nowait
     AFCounter run 0 -nowait
     AFCounter run2 0 -nowait
     AFCounter en 0 -nowait
     AFCounter en2 0 -nowait
     Sinad run 0 -nowait
     Sinad en 0 -nowait
     Scope enable1 0 -nowait
     Scope run1 0 -nowait
     Scope run2 0 -nowait
     Scope run3 0 -nowait
     Analyzer run 0 -nowait
     Generator ptt OFF -nowait
     Dvm run 0 -nowait
     Dvm en 0 -nowait
     Ber run 0 -nowait
     Ber en 0 -nowait
  }

  #**************************************************************************
  # Name: download_done_event
  # Parameters:   frame     widget generating the event
  #               token     auto provided by HTTP
  # Description: This function is needed to generate an event, and force the
  #     token, which is automatically appended by the HTTP library, to be
  #     thrown away
  #**************************************************************************
  proc download_done_event {frame token} \
  {
     event generate $frame <<DOWNLOAD_DONE>>
  }

  #**************************************************************************
  # Name: cancel_download
  # Parameters:   frame     widget displaying the information
  #               token     HTTP connection token
  # Description: This function cancels the download of a file
  #**************************************************************************
  proc cancel_download {frame token} \
  {
     variable files
     upvar #0 $token status

     # Handle cleanup of the parent frame
     bind $frame <<DOWNLOAD_DONE>> {}
     set top_frame  [::HtmlFrame::frame_of [winfo parent $frame] ]
     bind $frame <Destroy> ""
     $top_frame.b1 configure -text "" -command ""

     # Close and delete the files
     set filehandle $status(-channel)
     close $filehandle
     file delete $files(local)

     # Shutdown the socket
     http::reset $token cancel

     # Display a frame to inform user that download was canceled
     ::Reload_Screen::download_cancel $frame 
  }

  #*************************************************
  #       Name: transfer_reload_file
  # Parameters:
  #Description: 
  #*************************************************
  proc transfer_reload_file {frame version url {next_url ""}} \
  {
     variable files
     set cmd [namespace code "transfer_file_core $frame $url $next_url $files(local) 65536"]
     after cancel $cmd
     after idle $cmd
  }
  #*************************************************
  #       Name: do_reload_frame
  # Parameters: frame, frame widget name of frame inside html widget
  #       : array_name, name of array holding data (version and url)
  #Description: displays version message on html widget, displays yes/no softkeys
  #*************************************************
  proc do_reload_frame {frame array_name} {
  
    upvar $array_name APPLET_ARGS

    # we are a subframe of the HTML widget, which is a child
    # of an HtmlFrame.
    set top_frame  [::HtmlFrame::frame_of [winfo parent $frame] ]
  
    # if a next URL isn't provided, set it to NULL
    if {![info exists APPLET_ARGS(next_url)]} \
    {
       set APPLET_ARGS(next_url) {}
    }
    
    # set the downgrade variable to FALSE
    set downgrade 0
    # if there is an override frame text provided, just use that and assume
    # no risk of downgrade
    if [info exists APPLET_ARGS(frame_text)] \
    {
       set frame_text $APPLET_ARGS(frame_text)
    } \
    else \
    {
       set frame_text [compare_firmware_version $APPLET_ARGS(version)]
    }

    # Check for "unsafe downgrade", and show link if needed
    if {$downgrade} \
    {
      label $frame.warning -text $frame_text \
      -wrap [$frame cget -width] \
      -font Text
      pack $frame.warning -fill both -expand true
      $top_frame.b1 configure -text "READ MORE" \
                              -command "::Reload_Screen::downgrade_warning \
                                   $top_frame                \
                                   $APPLET_ARGS(version) \
                                   $APPLET_ARGS(url)     \
                                   $APPLET_ARGS(next_url)"

    } \
    else \
    {
      progress_bar $frame.progress
      pack $frame.progress  -fill both -expand true
      $frame.progress.text configure -text $frame_text
      # Add our softkeys to the window
      $top_frame.b1 configure -text "GET FILE" \
                            -command "::Reload_Screen::transfer_reload_file \
                                   $frame                \
                                   $APPLET_ARGS(version) \
                                   $APPLET_ARGS(url)     \
                                   $APPLET_ARGS(next_url)"
    }
    bind $frame <Destroy> "$top_frame.b1 configure -text \"\" -command \"\""
    
  }
  #*************************************************
  #       Name: downgrade_warning
  # Parameters: frame,version,url,next url
  #Description: Warn user he is about to make the biggest mistake
  #             of his life
  #*************************************************
  proc downgrade_warning { top_frame version url next_url} \
  {
     ::HtmlFrame::do_url $top_frame "file://localhost/sysdata/html/downgrade.shtml?version=$version&url=$url&next_url=$next_url"
  }
  
  #*************************************************
  #       Name: do_option_frame
  # Parameters: frame, frame widget name of frame inside html widget
  #       : array_name, name of array holding data (version and url)
  #Description: displays version message on html widget, displays yes/no softkeys
  #*************************************************
  proc do_option_frame {frame array_name} {
  
    upvar $array_name APPLET_ARGS

    # we are a subframe of the HTML widget, which is a child
    # of an HtmlFrame.
    set top_frame  [::HtmlFrame::frame_of [winfo parent $frame] ]
  
    # if a next URL isn't provided, set it to NULL
    if {![info exists APPLET_ARGS(next_url)]} {
       set APPLET_ARGS(next_url) {}
    }
    progress_bar $frame.progress

    $frame.progress.text configure -text "Option file is ready to download"

    pack $frame.progress  -fill both -expand true

    # Add our softkeys to the window
    $top_frame.b1 configure -text "GET FILE" \
                            -command "::Reload_Screen::transfer_option_file \
                                   $frame                \
                                   $APPLET_ARGS(url)     \
                                   $APPLET_ARGS(next_url)"

    bind $frame <Destroy> "$top_frame.b1 configure -text \"\" -command \"\""
    
  }  
  #*************************************************
  #     Name: build_html_screen
  # Parameters: URL to show, toplevel to be a child of
  #Description: builds html screen
  #*************************************************
  proc build_html_screen { url widget } {

    set html [::help::show $widget $url "Download new firmware"]
    bind $html <Destroy> "+::Reload_Screen::reload_file_check $widget"
  }
   
  #*************************************************
  #       Name: reload_floppy_check
  # Parameters: none
  #Description: firmware reload from the floppy drive
  #*************************************************
  proc reload_floppy_check { tile } {
     variable files
     variable status
     
     file mkdir $files(path)
     if [catch {
	     if [file exists /fd0/OPTIONS.NEW] \
	     {
	        set status(main) "Copying option file to hard drive"
	        update idle
	        file copy -force /fd0/OPTIONS.NEW $files(local_option)
	        reload_file_check $tile 
	     } else \
             {
	        foreach file $files(names) \
	        {
	           if [file exists /fd0/$file] \
	           {
  	              set status(main) "Copying update file to hard drive"
	              update idle
	              file copy -force /fd0/$file $files(local)
	              reload_file_check $tile 
	              break
	           }
	        } 
	     } 
	 } err ] \
	 {
	 	set status(main) "Error reading floppy"
	 }
  }
  
  #*************************************************
  #       Name: reload_file_check
  # Parameters: 
  #Description: checks for firmware reloading file
  #*************************************************
  proc reload_file_check { tile } \
  {
    variable files
    variable status
    
    # find the parent tile
    if ![winfo exists $tile] { return }
    
    while {"[winfo class $tile]" != "RELOAD" } \
    {
       set tile [winfo parent $tile]
    }
    
    pack forget $tile.proxycfg
    pack forget $tile.loading
 
    # ASSIGN SOFTKEYS
    ::View::ClearAllSoftkeys  $tile
    $tile.status config -fg black
    $tile.bottom config -fg black
    set status(bottom) ""

    ::View::AssignSoftkey $tile 3 "Check Floppy"    "::Reload_Screen::reload_floppy_check $tile"
    ::View::AssignSoftkey $tile 4 "Download System" "::Reload_Screen::build_reload_download $tile"
    if [file exists $files(local)] \
    {
       set status(main) "System update file found at $files(local)"
       ::View::AssignSoftkey $tile  2  "Reload System"  "::Reload_Screen::Begin_install $tile"
    } \
    elseif [file exists $files(local_option)] \
    {
       set status(main) "Option file found at $files(local_option)"
       ::View::AssignSoftkey $tile  2  "Install Option File"  "::Reload_Screen::Begin_install $tile"
    } \
    else \
    {
       set status(main) "System update not found"
    }
    View::AssignReturn $tile 

  }
  
  #*************************************************
  #       Name: reload_install_done
  # Parameters: 
  #Description: restores firmware install screen
  #*************************************************
  proc reload_install_done_callback { tile args } \
  {
     set cmd "::Reload_Screen::reload_install_done $tile"
     after cancel $cmd
     after idle $cmd
  }
  proc reload_install_done { tile } \
  {
    variable files
    variable status
    variable reload


    # walk up until we have found the top level tile
    while { "[winfo class $tile]" != "RELOAD" } \
    {
       set tile [winfo parent $tile]
    }
    
    
    trace vdelete ::Untar::State(tardone) w "::Reload_Screen::reload_install_done_callback $tile"

    trace vdelete ::Gzip::State(gziperrcode) w "::Reload_Screen::unzip_error $tile"

    trace vdelete ::Gzip::State(gzip_done) w "::Reload_Screen::start_untar $files(local)"
 
    trace vdelete ::Untar::State(tarerrcode) w "::Reload_Screen::untar_error $tile"


    # CHANGE CURRENT DIRECTORY BACK AFTER UNTARING
    cd /system
  
    # restore the config file
    if [file exists /system/etc/ifcfg_eth0.bak] \
    {
       catch {file copy -force /system/etc/ifcfg_eth0.bak /system/etc/ifcfg_eth0}
    }
    
    # if no error, then finish up
    if {($reload(file_color) == "blue" && $reload(gzip_color) == "blue") && \
        ($reload(decrypt_color) == "blue")} \
    {

      # SOURCE post.tcl IF EXISTS
      if {[file exists $files(post)]} \
      {
        trace variable ::IFR_options::progress w [namespace code update_post]
        set reload(post_msg) "Post install script"
        Screens::position_widget $tile.loading.post
        catch { source $files(post) } 
        trace vdelete ::IFR_options::progress w [namespace code update_post]
        file delete $files(post)
      }
      # finish the post-install bar, if needed.
      set reload(post_done) $reload(post_count)
      file delete $files(post)
      file delete $files(local)
      file delete $files(local_option)
      file delete $files(tarfile)
    
      grid forget $tile.loading.warning
      grid forget $tile.loading.warning2
      Screens::position_widget $tile.loading.alldone
    }

    set ::sys(powerdown_allowed) 1
    set menu [View::GetMainMenuName $tile]
    $menu configure -state normal
    set ::Slave::local_lockout {}
    View::AssignReturn $tile "::Reload_Screen::reload_file_check $tile"
  }

  proc update_decrypt { args } \
  {
     variable reload
     set reload(decrypt_done) $::IFR_options::progress
  }
  
  proc update_post { args } \
  {
     variable reload
     set reload(post_count) 100
     set reload(post_done) $::IFR_options::progress
  }
  

  #*************************************************
  #       Name: reload_do_install
  # Parameters: 
  #Description: preforms firmware install
  #*************************************************
  proc reload_do_install { tile } {
     variable status
     variable files
     global color
     variable reload
     
     # remove the softkeys
     ::View::ClearSoftkey $tile  2
     ::View::ClearSoftkey $tile  3
     ::View::ClearSoftkey $tile  7

     set reload(decrypt_color) blue
     set reload(file_color) blue
     set reload(gzip_color) blue
     set reload(verify_color) blue
    
     # Check for options file - install it first
     if [file exists $files(local_option)] \
     {
        set reload(decrypt_msg) "Decrypting options file"
        set reload(decrypt_bytes) 100
        set reload(decrypt_done) 0
        Screens::position_widget $tile.decrypt
        
        trace variable ::IFR_options::progress w [namespace code update_decrypt]

        if [catch {IFR_options apply} err ] \
        {
           puts "Error decrypting file: $err"
           set reload(decrypt_msg) "Error decrypting file: $err"
           set reload(decrypt_bytes) 100
           set reload(decrypt_done) 100
           set reload(decrypt_color) red
           file delete $files(local_option)
           set err_window [name_child $tile err]
           set options_url file://localhost/sysdata/html/help/Options/
           after 100
           switch -- $err \
           {
              "File corrupt" \
              {
                 ::HtmlFrame::make_toplevel $err_window ${options_url}corrupt.shtml
              }
              "Not for us" \
              {
                 ::HtmlFrame::make_toplevel $err_window ${options_url}not_me.shtml
              }
              default \
              {
                 ::HtmlFrame::make_toplevel $err_window ${options_url}err.shtml
              }
           }
           reload_file_check $tile
           trace vdelete ::IFR_options::progress w [namespace code update_decrypt]
           return
        }
        trace vdelete ::IFR_options::progress w [namespace code update_decrypt]
        set reload(decrypt_msg) "Decryption done"
        set reload(decrypt_bytes) 100
        set reload(decrypt_done) 100
        set reload(decrypt_color) blue

        file delete $files(local_option)
     }
     
     # now, is there an update to install?
     if ![file exists $files(local)] \
     {
        # no install, so exit.
        reload_install_done $tile
        return
     }
  
     # load Gzip & Untar modules
     set ::Gzip::State(gzippercent)  0
     set ::Untar::State(tarfilesize) 0

     set reload(gzip_color)   blue
     set reload(file_color)   blue
     set reload(untar_color)  blue
     set reload(verify_color) blue

     foreach bar { unzip verify untar copy} \
     {
        Screens::position_widget $tile.$bar
     }

     set ::sys(powerdown_allowed) 0
     set menu [View::GetMainMenuName $tile]
     $menu configure -state disabled
     set ::Slave::local_lockout RELOAD

     View::AssignReturn $tile

     trace variable ::Untar::State(tarerrcode) w "::Reload_Screen::untar_error $tile"

     trace variable ::Untar::State(tardone) w "::Reload_Screen::reload_install_done_callback $tile"
     
     trace variable ::Gzip::State(gziperrcode) w "::Reload_Screen::unzip_error $tile"

     trace variable ::Gzip::State(gzip_done) w "::Reload_Screen::start_untar $files(local)"

     unzip $files(local)
  }


  #*************************************************
  #       Name: start_untar
  # Parameters: file - file to untar
  #Description: start untar file
  #*************************************************
  proc start_untar { file args } \
  {
     if {$::Gzip::State(gzip_done)} \
     {
        set cmd "::Reload_Screen::start_untar_core $file"
        after cancel $cmd
        after idle $cmd
     }
  }
  proc start_untar_core { filename } \
  {
     set newfile [file rootname $filename]
     after 1000 "::Reload_Screen::untar_core $newfile"
  }
  proc untar_core { newfile } \
  {
     variable reload

     if {$reload(gzip_color) == "blue"} \
     {
        if [catch {exec untar $newfile $reload(where)}] \
	{
	   ::Untar::updateUntarError 5
	}
     }
  }


  #*************************************************
  #        Name: unzip
  #  Parameters: file - file to unzip
  # Description: star unzip file
  #*************************************************
  proc unzip { file } \
  {
     # save our current network config
     catch \
     {
        file copy -force /system/etc/ifcfg_eth0 /system/etc/ifcfg_eth0.bak
     }

     if [catch {Gzip unzip $file}] \
     {
        ::Gzip::updateGzipError 7
     }
  }


  #*************************************************
  #       Name: untar_error
  # Parameters: tile - tile path
  #Description: display error on untar file
  #*************************************************
  proc untar_error { tile args } \
  {
     set cmd "::Reload_Screen::untar_error_core $tile"
     after cancel $cmd
     after idle $cmd
  }
  proc untar_error_core { tile } \
  {
     variable reload
     global color
     variable status

     set reload(gzip_color)   red
     set reload(file_color)   red
     set reload(untar_color)  red
     set reload(verify_color) red
     if ![winfo exists $tile] { return }
     set parent [winfo parent $tile]
     $parent.status config -fg $color(messagecaution)
     set status(main) "UNTAR ERROR: $::Untar::State(tarerrstr)"
     $parent.bottom config -fg $color(messagecaution)
     set status(bottom) "UNTAR Failed"
     reload_install_done $tile
  }


  #*************************************************
  #       Name: unzip_error
  # Parameters: tile - tile path
  #Description: display error on unzip file
  #*************************************************
  proc unzip_error { tile args } \
  {
     set cmd "::Reload_Screen::unzip_error_core $tile"
     after cancel $cmd
     after idle $cmd
  }
  proc unzip_error_core { tile } \
  {
     global color
     variable reload
     variable status
     
     if ![winfo exists $tile] { return }

     set reload(gzip_color) red
     set parent [winfo parent $tile]
     $parent.status config -fg $color(messagecaution)
     set status(main) "GZIP ERROR: $::Gzip::State(gziperrstr)"
     $parent.bottom config -fg $color(messagecaution)
     set status(bottom) "GZIP Failed"
     reload_install_done $tile
  }


  #*************************************************
  #       Name: Begin_install
  # Parameters: 
  #Description: builds firmware installation screen 
  #*************************************************
  proc Begin_install { tile } \
  {
    variable status
    # remove the other items from the screen
    ::View::ClearAllSoftkeys  $tile
    grid forget $tile.proxycfg
  
    foreach bar { decrypt unzip verify untar copy alldone post } \
    {
       grid forget $tile.loading.$bar
    }
    set status(main) ""
    set status(bottom) ""
    Screens::position_widget $tile.loading
    Screens::position_widget $tile.loading.warning
    Screens::position_widget $tile.loading.warning2

    # ASSIGN SOFTKEYS
    ::View::ClearAllSoftkeys  $tile
    
    ::View::AssignSoftkey $tile 2 "Start Install" "::Reload_Screen::reload_do_install $tile.loading"
    View::AssignReturn $tile "::Reload_Screen::reload_file_check $tile"

  }
   
  #*************************************************
  #       Name: Use_proxy_user
  # Parameters: none
  #Description: toggle need for proxy user & password
  #*************************************************
  proc Use_proxy_user {} \
  {
     switch -- $::sys(proxyneedpw) \
     {
        "NO"   { set ::sys(proxyneedpw) "YES"}
        default {set ::sys(proxyneedpw) "NO"}
     }
  }
  
  #*************************************************
  #       Name: Proxy_user
  # Parameters: tile to update
  #Description: handles display of proxy uid & pw
  #*************************************************
  proc Proxy_user {tile args} \
  {
    switch -- $::sys(proxyneedpw) \
    {
       "YES"   { Screens::position_widget $tile.password }
       default { grid forget $tile.password }
    }
  }
  #*************************************************
  #       Name: Map_proxy_setup
  # Parameters: tile to update
  #Description: tie into proxy needed var when mapped
  #*************************************************
  proc Map_proxy_setup {tile} \
  {
     trace variable ::sys(proxyneedpw) w "Reload_Screen::Proxy_user $tile"
     Proxy_user $tile
  }
  
  #*************************************************
  #       Name: Unmap_proxy_setup
  # Parameters: tile to update
  #Description: remove watch on proxy needed state
  #*************************************************
  proc Unmap_proxy_setup {tile args} \
  {
    trace vdelete ::sys(proxyneedpw) w "Reload_Screen::Proxy_user $tile"
  }
  
  #*************************************************
  #       Name: use_proxy
  # Parameters: none
  #Description: toggle use of proxy server
  #*************************************************
  proc use_proxy { } {   
    if {$::sys(useproxy) == "NO"} \
    {
      set ::sys(useproxy) YES
    } \
    else \
    {
      set ::sys(useproxy) NO
    }
  }
  
  #*************************************************
  #       Name: proxy
  # Parameters: tile
  #Description: map or unmap the proxy setup as needed
  #*************************************************
  proc proxy { tile args } \
  {
     switch -- $::sys(useproxy) \
     {
        "YES"   {Screens::position_widget $tile.proxy_setup}
        default {grid forget $tile.proxy_setup}
     }
  }
  #*************************************************
  #       Name: Map_proxy
  # Parameters: tile to update
  #Description: tie into proxy needed var when mapped
  #*************************************************
  proc Map_proxy {tile} \
  {
     trace variable ::sys(useproxy) w "Reload_Screen::proxy $tile"
     proxy $tile
  
     # ASSIGN SOFTKEYS
     ::View::ClearAllSoftkeys  $tile
     set alt_url $::sys(alt_firmware_url)
    
     ::View::AssignSoftkey $tile  2  "GO TO IFR"  "::Reload_Screen::build_html_screen $::sys(firmware_url) $tile"
     ::View::AssignSoftkey $tile  3  ""           "::Reload_Screen::build_html_screen $alt_url $tile"
     View::AssignReturn $tile "::Reload_Screen::reload_file_check $tile"
  }
  
  #*************************************************
  #       Name: Unmap_proxy
  # Parameters: tile to update
  #Description: remove watch on proxy needed state
  #*************************************************
  proc Unmap_proxy {tile args} \
  {
    trace vdelete ::sys(useproxy) w "Reload_Screen::proxy $tile"
  }
  #*************************************************
  #       Name: build_reload_download
  # Parameters: 
  #Description: builds initial firmware download screen 
  #*************************************************
  proc build_reload_download { tile } \
  {
    # remove the reload page if needed
    grid forget $tile.loading
    Screens::position_widget $tile.proxycfg
  }
  
  #****************************************************************************
  # Name:        Setup
  # Parameters:  parent		parent of child window
  # Description: Create the screen
  # Returns:     Tile path to be packed into the parent frame
  #****************************************************************************
  proc Setup { parent } \
  {
    set top_level  [winfo toplevel $parent]
   
    set tile_path  [name_child $top_level reload]
    
    set tile_class  $::Reload_Screen::tile_class

    if { ![winfo exists $tile_path] } \
    {
      set tile_path [::Screens::create_tile  $tile_path  reload  $tile_class]
      grid rowconfigure $tile_path.proxycfg 0 -weight 0
      grid rowconfigure $tile_path.proxycfg 1 -weight 1
    }

    pack forget $tile_path.loading
    pack forget $tile_path.proxycfg
    
    return $tile_path
  }

  
  #****************************************************************************
  # Name:        Map
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Map { name } \
  {
    after cancel "::Reload_Screen::Cleanup_core $name"
    # remove the options - we don't have them.
    View::PackOptions $name
    reload_file_check $name
  }

  #****************************************************************************
  # Name:        Cleanup_core
  # Parameters:  parent object
  # Description: Dummy function
  #****************************************************************************
  proc Cleanup_core { tile_path } \
  {
     catch {destroy $tile_path}
  }

  #****************************************************************************
  # Name:        Cleanup
  # Parameters:  parent object
  # Description: Dummy function
  #****************************************************************************
  proc Cleanup { tile_path } \
  {
     after 1000 "::Reload_Screen::Cleanup_core $tile_path"
  }
  #****************************************************************************
  # Name:        Unmap
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Unmap { name } \
  { 
     #Cleanup $name
     View::AssignReturn $name
     return
  }

 
  #****************************************************************************
  # Name:        Destroy
  # Parameters:  parent		parent of child window
  # Description: Dummy function
  #****************************************************************************
  proc Destroy { name } \
  { 
     Cleanup $name
  }


  #****************************************************************************
  # Functions executed when the namespace is sourced
  #****************************************************************************
  ::View::RegisterScreen Reload_Screen;  # Add this on the menu


};  # End of the Reload_Screen name space

