#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001        DESIGN ENGINEERING      #
# ------------------------------------------------------------------------- #
# FILE        : testMain.tcl                                                #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : FITS Built In Test main program code                        #
#############################################################################
#                                                                           #
# Test Window Design:                                                       #
#    o Options menu allows selection of parameters for all tests            #
#    o Test selection menu allows execution of individual or all tests      #
#    o Progress Bar - updated by number of tests executed                   #
#    o Test Status - Yellow=Running, Red=Failed, Green=Passed               #
#    o Text window - displays results of tests                              #
#    o Widget window(s) - allows tiles to be packed into window, and shows  #
#      results of resizing and cursor movement                              #
#                                                                           #
# ------------------------------------------------------------------------- #
#                                                                           #
# Automated Testing                                                         #
#    o testCompare   This function executes a list of test cases.           #
#                    It is designed to execute a TCL command, get a result  #
#                    from the command, compare this result to a known value #
#                    and indicate errors.                                   #
#    o testReportData   This provides a simple way to print test data which #
#                    needs to be manually verified. When this function is   #
#                    used to print data, the print can be masked off by a   #
#                    user that only wants to see main test results.         #
#                                                                           #
#############################################################################

#############################################################################
# FUNCTION    : testMainStart                                               #
# ARGUMENTS   : none                                                        #
# RETURNS     : none                                                        #
# PURPOSE     : Generates the main test window and manages the user test    #
#             : interface                                                   #
#############################################################################
proc testMainStart { } {

    set testParent ".testMainWin"
    if {![winfo exists $testParent]} {
        toplevel $testParent
    }

    createMainWinFrames $testParent 
}

#############################################################################
# FUNCTION    : createMainWinFrames                                         #
# ARGUMENTS   : parent		Parent container window path                #
# RETURNS     : none                                                        #
# PURPOSE     : Create the main window frames and the primary menu bar      #
#             : buttons and menus                                           #
#############################################################################
proc createMainWinFrames { parent } {
    global t_color
    global testMode
    global autoTestList
    global testStatus

    initTestVar
    # Force the test window to be slightly smaller than the full screen to
    # insure access to its controls
    set screen_x [winfo screenwidth .]
    set screen_x [expr ($screen_x - 200)]
    set screen_y [winfo screenheight .]
    set screen_y [expr ($screen_y - 150)]
    $parent config -width $screen_x -height $screen_y
    pack propagate $parent false
 
    # Create a title bar, and a main test client area
    frame $parent.wTitleBar -height 30 -bg $t_color(title)
    pack $parent.wTitleBar -side top -fill x

    frame $parent.client -bg $t_color(bkgnd)
    pack $parent.client -side top -fill both -expand true

    # Create title bar name and test buttons
    label $parent.wTitleBar.name -text "2795 Built In Test" -bg $t_color(title) -fg $t_color(text) -justify left -anchor w   

    #
    # Generate the options menu
    #
    menubutton $parent.wTitleBar.options -text  "Options" -menu $parent.wTitleBar.options.menu \
                            -bg $t_color(menu) -relief raised
    menu $parent.wTitleBar.options.menu
    $parent.wTitleBar.options.menu add command -label "Reset Errors" -command resetTestVar     
    $parent.wTitleBar.options.menu add cascade -label "Results Detail" -menu $parent.wTitleBar.options.menu.detail
    menu $parent.wTitleBar.options.menu.detail
    $parent.wTitleBar.options.menu.detail add radiobutton -label "None" \
           -command "set testStatus(reportLevel) 0" -value 0 -variable testStatus(reportLevel)   
    $parent.wTitleBar.options.menu.detail add radiobutton -label "Minimal" \
           -command "set testStatus(reportLevel) 1" -value 1 -variable testStatus(reportLevel)   
    $parent.wTitleBar.options.menu.detail add radiobutton -label "Moderate" \
           -command "set testStatus(reportLevel) 2" -value 2 -variable testStatus(reportLevel)   
    $parent.wTitleBar.options.menu.detail add radiobutton -label "Verbose" \
           -command "set testStatus(reportLevel) 3" -value 3 -variable testStatus(reportLevel)   
    $parent.wTitleBar.options.menu add cascade -label "Target Device" -menu $parent.wTitleBar.options.menu.device
    menu $parent.wTitleBar.options.menu.device
    $parent.wTitleBar.options.menu.device add radiobutton -label "Screen" \
           -command "set testStatus(device) 0" -value 0 -variable testStatus(device)   
    $parent.wTitleBar.options.menu.device add radiobutton -label "Terminal" \
           -command "set testStatus(device) 1" -value 1 -variable testStatus(device)    
    $parent.wTitleBar.options.menu.device add radiobutton -label "File" \
           -command "set testStatus(device) 2" -value 2 -variable testStatus(device)   
    $parent.wTitleBar.options.menu add cascade -label "Visual Tests" -menu $parent.wTitleBar.options.menu.visual
    menu $parent.wTitleBar.options.menu.visual
    $parent.wTitleBar.options.menu.visual add radiobutton -label "Disable test data" \
           -command "set testStatus(visual) 0" -value 0 -variable testStatus(visual)   
    $parent.wTitleBar.options.menu.visual add radiobutton -label "Enable test data" \
           -command "set testStatus(visual) 1" -value 1 -variable testStatus(visual)   

    #
    # Generate the test menu
    #
    menubutton $parent.wTitleBar.tests -text  "Tests" -menu $parent.wTitleBar.tests.menu \
                            -bg $t_color(menu) -relief raised
    menu $parent.wTitleBar.tests.menu
    $parent.wTitleBar.tests.menu add command -label "All" -command "dummy_cmd ALL"     
    $parent.wTitleBar.tests.menu add separator     
    makeOptionMenu $parent.wTitleBar.tests.menu autoTestList

    button $parent.wTitleBar.exit -text  "Exit" -command "destroy $parent" \
                            -bg $t_color(menu) -relief raised

    set btags [bindtags $parent.wTitleBar.exit]
    set temp [linsert $btags 1 "Cursor"]
    bindtags $parent.wTitleBar.exit $temp


    pack $parent.wTitleBar.name $parent.wTitleBar.options $parent.wTitleBar.tests \
         $parent.wTitleBar.exit -side left -expand true -fill both
}

#############################################################################
# FUNCTION    : testCompare                                                 #
# ARGUMENTS   : testList	List of test cases to execute               #
#             : debug           Set to "1" to get extra debug data          #
# RETURNS     : none                                                        #
# PURPOSE     : To execute a number of test cases, comparing the results    #
#             : with expected values, and updating the test status          #
#############################################################################
proc testCompare { testListName {debug 0} } {
   global testStatus

   upvar #0 $testListName testList
   global $testList

   if { $testStatus(reportLevel) == 3 } {
      set debug 1
   }


   set testLocalErrors 0
   set testLocalCases 0

   foreach test_run $testList {
      set testCmd     [lindex $test_run 0]
      set testDesired [lindex $test_run 1]
      set testType    [lindex $test_run 2]
      set testRange   [lindex $test_run 3]

      incr testLocalCases 1
      if {$debug == 1 } {
         testReportPrint "-- Test Case $testLocalCases: Cmd = $testCmd"
      }
      set testResults [eval $testCmd]
      if {$debug == 1 } {
         testReportPrint "-- Desired = $testDesired, Actual = $testResults"
      }
      if {$testType == "INT"} {
         if {$testResults != $testDesired} {
            incr testLocalErrors 1
            testReportErrors "** Test Error: $testLocalErrors $testCmd, actual=$testResults"
         }
      } elseif {$testType == "DBL"} {
         if {$testResults > $testDesired } {
            set delta [expr $testResults - $testDesired]               
         } else {
            set delta [expr $testDesired - $testResults]               
         }
         if {$delta > $testRange} {
            incr testLocalErrors 1
            testReportErrors "** Test Error: $testLocalErrors $testCmd, actual=$testResults"
         }
      } elseif {$testType == "STR"} {
         if {$testResults != $testDesired} {
            incr testLocalErrors 1
            testReportErrors "** Test Error: $testLocalErrors $testCmd, actual=$testResults"
         }
      } elseif {$testType == "DELAY"} {
      } else {
         testReportErrors "Unknown test compare type = $testType"
      }
   }

   set testStatus(moduleCases) $testLocalCases
   set testStatus(moduleErrors) $testLocalErrors
   set testStatus(totalCases) [expr $testStatus(totalCases) + $testLocalCases]
   set testStatus(totalErrors) [expr $testStatus(totalErrors) + $testLocalErrors]
   testReport $testListName
}

#############################################################################
# FUNCTION    : testReport                                                  #
# ARGUMENTS   : testList	List of test cases to execute               #
#             : reportLevel     Identifies the information provided         #
# RETURNS     : none                                                        #
# PURPOSE     : This prints various amounts of information about the test   #
#             : results                                                     #
#############################################################################
proc testReport { testList } {
   global testStatus

   if { $testStatus(reportLevel) == 0 } {return}

   if { $testStatus(reportLevel) == 1 } {
      if { $testStatus(moduleErrors) == 0 } {
         testReportPrint "Module passed: $testList"
      } else {
         testReportPrint "**Error: Module $testList failed $testStatus(moduleErrors) of $testStatus(moduleCases) cases"
      }       
   }

   if { $testStatus(reportLevel) >= 2 } {
      if { $testStatus(moduleErrors) == 0 } {
         testReportPrint "Module passed: $testList"
      } 
      testReportPrint "Executed Module: $testList, Failures=$testStatus(moduleErrors)/$testStatus(totalErrors), Cases=$testStatus(moduleCases)/$testStatus(totalCases)"
   }
}

#############################################################################
# FUNCTION    : testReportErrors                                            #
# ARGUMENTS   : printLine	Text line to be printed                     #
# RETURNS     : none                                                        #
# PURPOSE     : This allows control of printing test error messages which   #
#             : is independent of test data or reports                      #
#############################################################################
proc testReportErrors { printLine } {
   # Currently print all errors
   testReportPrint $printLine
}

#############################################################################
# FUNCTION    : testReportData                                              #
# ARGUMENTS   : printLine	Text line to be printed                     #
# RETURNS     : none                                                        #
# PURPOSE     : This allows control of whether data which must be manually  #
#             : verified by the user is printed.                            #
#############################################################################
proc testReportData { printLine } {
   global testStatus

   if { $testStatus(visual) } {
      testReportPrint $printLine
   }
}

#############################################################################
# FUNCTION    : testReportPrint                                             #
# ARGUMENTS   : printLine	Text line to be printed                     #
# RETURNS     : none                                                        #
# PURPOSE     : This function allows printing of test results to a selected #
#             : destination. While it currently only prints to the terminal #
#             : the goal is to allow a testStatus variable to specify the   #
#             : target to be the test text window, terminal or a file.      #
#############################################################################
proc testReportPrint { printLine } {
   puts $printLine
}

#############################################################################
# FUNCTION    : resetTestVar                                                #
# ARGUMENTS   : none                                                        #
# RETURNS     : none                                                        #
# PURPOSE     : Resets the test variables                                   #
#############################################################################
proc resetTestVar { } {
   global testStatus

   set testStatus(moduleCases) 0
   set testStatus(moduleErrors) 0
   set testStatus(totalCases) 0
   set testStatus(totalErrors) 0
}

#############################################################################
# FUNCTION    : autoLoadRunModuleTest                                       #
# ARGUMENTS   : testList	List of module tests to run                 #
# RETURNS     : none                                                        #
# PURPOSE     : Switch between test modes                                   #
#############################################################################
proc autoLoadRunModuleTest { testList menuName} {
   global testStatus

   upvar #0 $testList testList1
   global $testList1

   set tempTotal $testStatus(totalErrors)
   set tested 0
   foreach testModule $testList1 {
      set menuIndex [lindex $testModule 0]
      if {$menuIndex == 0 } {continue}
      set testName [lindex $testModule 1]
      if {$testName != $menuName } {continue}
      set testFunct [lindex $testModule 2]
      set testPath  [lindex $testModule 3]
      source $testPath
      $testFunct
      set tested 1
   }

   if { $tempTotal == $testStatus(totalErrors) } {
      if {$tested > 0} {
          testReportPrint "Test $menuName completed without errors"
      } else {
          testReportPrint "Test $menuName had no script"
      }
   } else {
      set temp [expr $testStatus(totalErrors) - $tempTotal]
      testReportPrint "Test $menuName completed with $temp errors"
   }
}


#############################################################################
# FUNCTION    : makeOptionMenu                                              #
# ARGUMENTS   : parentMenu	Window of parent menu                       #
#             : menulist        List which contains data for setting up     #
#             :                 both the menu and auto test execution       #
# RETURNS     : none                                                        #
# PURPOSE     : Set up the test menu and link commands to it                #
#############################################################################
proc makeOptionMenu { parentMenu menulist } {
   upvar #0 $menulist testList
   global $testList


   set subMenuIndex 0
   foreach value $testList {
      set menuIndex [lindex $value 0] 
      if {$menuIndex == 0} {
         set subMenu [lindex $value 1]
         incr subMenuIndex 1 
         $parentMenu add cascade -label $subMenu -menu $parentMenu.$subMenuIndex
         menu $parentMenu.$subMenuIndex     
      } else {
         set menuName [lindex $value 1] 
         set cmdValue "autoLoadRunModuleTest $menulist {$menuName}"  
         if {$subMenuIndex == 0} {
            $parentMenu add command -label $menuName -command $cmdValue     
         } else {
            $parentMenu.$subMenuIndex add command -label $menuName -command $cmdValue     
         }
      }
   }
}





proc dummy_cmd { value } {
    puts "Hit dummy command: $value"
}


