#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : startup.tcl                                                 #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file contains functions used during the initial system #
#             : boot process. The functions which source other .tcl files   #
#             : cannot be part of a namespace, or any namespace sourced     #
#             : becomes a child namespace.                                  #
#############################################################################

namespace eval Startup \
{
   variable file_count
   variable loaded_count
   set file_count 0
   set loaded_count 0
   variable file_name
   set file_name ""
   
   variable encr_file_count
   set encr_file_count 0
 
#****************************************************************************
# Name:        CountDir
# Parameters:  current_dir          path to screen directory to source
# Description: This counts all of the .tcl and .rc files in a specific 
#              directory
#****************************************************************************
  proc CountDir { current_dir } \
  {
     variable file_count
     # Scan the directory to source all files
     foreach screen_file [lsort [glob -nocomplain $current_dir/*]] \
     {
        # IGNORE CVS DIRECTORYS
        if {"$screen_file" == "CVS"} \
        {
           continue
        }
        # Ignore anything which is not a file
        if {[file isfile $screen_file]} \
        {
           switch -exact -- [string tolower [file extension $screen_file ]] \
           {
              ".tcl,enc" -
              ".rc,enc" -
              ".tcl" -
              ".rc" \
              {
                 incr file_count
              }
           }
        }
        # count subdirectories, too
        if [file isdir $screen_file] \
        {
          CountDir $screen_file
        }
        Debug::Puts Startup 20 "Countdir $screen_file"
        update idle
     }
  }
  
#****************************************************************************
# Name:        SourceDir
# Parameters:  current_dir          path to screen directory to source
# Description: This sources all of the .tcl and .rc files in a specific 
#              directory
# NOTE: This function is indirectly recursive!
#       When this routine is called by SourceScreenDir to source screen 
#       directories, some of the screen namespaces will call SourceScreenDir
#       while they are sourced to load subdirectories. This creates a recursive
#       call to this function.
#****************************************************************************
  proc Process_encr_tcl_file { orig_file file_name } \
  {
     Debug::Puts Startup 1 "post decryption source $orig_file $file_name $::result $::code"
     if {$::result == "OK"} \
     {
        BGSource $file_name
     }
     BGCommand "file delete $file_name"
     BGCommand "incr Startup::loaded_count"
  }
  proc Process_encr_rc_file { orig_file file_name } \
  {
     Debug::Puts Startup 1 "post decryption option read $orig_file $file_name $::result $::code"
     if {$::result == "OK"} \
     {
        catch {option read $file_name}
     }
     BGCommand "file delete $file_name"
     BGCommand "incr Startup::loaded_count"
  }
  
  proc Process_a_file { file_name } \
  {
     Debug::Puts Startup 20 "Loading $file_name"
     set dir [file dirname $file_name]
     
     switch -exact -- [string tolower [file extension $file_name ]] \
     {
        ".tcl" \
        {
           # TCL file defining a screen or screen component
           BGSource $file_name
           BGCommand "incr Startup::loaded_count"
        }
        ".rc" \
        {
           # screen resource file
           option read $file_name
           incr Startup::loaded_count
           Debug::Puts Startup 20 "loaded $file_name"
           update idle
        }
        ".tcl,enc" \
        {
           # encrypted TCL file
           variable encr_file_count
           set fname [file join /usr/foo.$encr_file_count]
           incr encr_file_count
           IFR_options decrypt $file_name $fname "Startup::Process_encr_tcl_file $file_name $fname"
        }
        ".rc,enc" \
        {
            # encrypted rc file
           variable encr_file_count
           set fname [file join /usr/foo.$encr_file_count]
           incr encr_file_count
           IFR_options decrypt $file_name $fname "Startup::Process_encr_rc_file $file_name $fname"
        }
    }
    Debug::Puts Startup 20 "Done Loading $file_name"
    update idle
  }
  
  proc SourceDir { current_dir } \
  {
     variable loaded_count
     variable file_name
     set flist [concat [lsort [glob -nocomplain $current_dir/*.rc]] \
                     [lsort [glob -nocomplain $current_dir/*.rc,enc]]\
                     [lsort [glob -nocomplain $current_dir/*.tcl]]\
                     [lsort [glob -nocomplain $current_dir/*.tcl,enc]]\
               ]
     # Scan the directory to source all files
     foreach file_name $flist \
     {
        # Ignore anything which is not a file
        if {[file isfile $file_name]} \
        {
           Process_a_file $file_name
        }
     }
  }

#****************************************************************************
# Name:        SourceScreenDir
# Parameters:  current_dir          path to screen directory to source
#              nsList               list of menu/submenu screen namespaces  
# Description: This routine is used to source all of the .tcl and .rc files 
#              in screen directories
#****************************************************************************
# NOTE: This routine is indirectly recursive
#****************************************************************************
# Design Info: There is a need to build a menu of the list of available screens,
#    and a flexible way to build that list is when the screens are loaded. Each
#    screen file when sourced will register itself with the ::View:: manager.
#    Submenus on the screen menu will be handled by having a "Screen" file which
#    does little more than maintain a list of the namespaces for the submen, and
#    which will call this function to source a specific directory.  NOTE: While
#    this function is not recursively load subdirectories, the sourcing of the
#    submenu "Screen" files WILL cause a recursive call to this function.
# Assumptions:
#    1) All screens for a specific menu are in one directory
#    2) If a menu has a submenu, then directory having the screens for that 
#       menu will also have a submenu "screen" file which knows where its children
#       screens are located
#****************************************************************************
  proc SourceScreenDir { current_dir nsList } \
  {
     set ::View::MenuListPtr($current_dir) $nsList
     SourceDir $current_dir
  }

} ;# End of the Startup namespace

