#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C) 2003      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
#                                                                           #
# FILE        : 3001.SincUI.tcl                                             #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : Preston McCormick                                           #
#                                                                           #
# DESCRIPTION : Code for the SINC waveform                                  #
#                                                                           #
#############################################################################

package require FGen


namespace eval SincUI \
{
  variable  last_focus
  variable  please_wait  ""
  
  variable  sinc_data
  
  variable  StateUI
  
  set StateUI(level,default)   1.000
  set StateUI(dev,default)     5.00
  set StateUI(rate,default)    100000
  set StateUI(omega,default)   200.00
  

  #**************************************************************************
  #
  #        Name: setup_sinc_waveform
  # 
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Starts the SINC waveform
  #
  #**************************************************************************
  proc setup_sinc_waveform { lun } \
  {
    #puts "setup_sinc_waveform:  $lun"

    # Set the Level
    FGen::setLevel  $lun  $::FGen::State($lun,SINC,level)
      
    # Set the Rate (the shape must be set first to avoid clipping the rate
    FGen shape${lun}  ARBITRARY  -nowait
    FGen::setFreq  $lun  [expr $::FGen::State($lun,SINC,rate) / 10.0]
    
    # Recalculate the waveform using the existing Omega value
    calculate_sinc  $lun

    # Set up the function generator parameters for SINC
    ::FGen::send_sinc_seq  $lun
  }
  
  
  #**************************************************************************
  #
  #        Name: set_sinc_level
  # 
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the level of the SINC waveform
  #
  #**************************************************************************
  proc set_sinc_level { lun } \
  {
    #puts "set_sinc_level:  $lun"

    if { $lun == 1 || $lun == 2 } \
    {
      # Set the level to the present value
      FGen::setLevel  $lun  $::FGen::State($lun,SINC,level)
    
      # Procedure 'SincUI::send_sinc_seq' called in 'FGen::setLevel'
    }
    
    return
  }
    
  
  #**************************************************************************
  #
  #        Name: set_sinc_dev
  # 
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the deviation or percent deviation of the SINC 
  #              waveform
  #
  #**************************************************************************
  proc set_sinc_dev { lun } \
  {
    #puts "set_sinc_dev:  $lun"

    # Set the deviation entry depending on the Deviation units
    if { $lun == 3 || $lun == 4 } \
    {
      switch -- $::FGen::State(dev_units) \
      {
        "kHz"  { # Set the Deviation to the present value
                 FGen::setDeviation  $lun  $::FGen::State($lun,SINC,dev)
      
                 # Procedure 'SincUI::send_sinc_seq' called in 'FGen::setDeviation' 
               }
      
        "%"    { # Set the Percent Deviation to the present value
                 FGen::setPercent  $lun  $::FGen::State($lun,SINC,percent)
      
                 # Procedure 'SincUI::send_sinc_seq' called in 'FGen::setPercent' 
               }
      }
    }

    return
  }
    
  
  #**************************************************************************
  #
  #        Name: set_sinc_rate
  # 
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the rate of the SINC waveform
  #
  #**************************************************************************
  proc set_sinc_rate { lun } \
  {
    #puts "set_sinc_rate:  $lun"

    # Set the rate to the present value
    FGen::setFreq  $lun  [expr $::FGen::State($lun,SINC,rate) / 10.0]
    
    # Procedure 'SincUI::send_sinc_seq' called in 'FGen::setFreq'

    return
  }
  
  
  #**************************************************************************
  #
  #        Name: set_sinc_omega
  # 
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the Omega value of the SINC waveform
  #
  #**************************************************************************
  proc set_sinc_omega { lun } \
  {
    #puts "set_sinc_omega:  $lun"

    # Perform the calculation of the SINC arbitrary waveform
    calculate_sinc  $lun

    # Set up the function generator parameters for SINC
    ::FGen::send_sinc_seq  $lun
  }


  #**************************************************************************
  #
  #        Name: calculate_sinc
  # 
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Calculates the SINC waveform 1024 point array from the 
  #              present OMEGA value, and sends this to the engine.
  #
  #**************************************************************************
  proc calculate_sinc { lun } \
  {
    #puts "calculate_sinc:  $lun"
     
    variable  sinc_data

    # Initialize the sinc_data
    set sinc_data  ""
    
    # Get the OMEGA parameter value
    set len 16384
    set half_len [expr $len/2]
    
    # Avoid a divide by zero condition
    if { $::FGen::State($lun,SINC,omega) < 0.01 } \
    {
       set ::FGen::State($lun,SINC,omega) 0.01
    }

    ##set omega [expr $f * 2 * 3.14159256 / $len]
    
    set omega  [expr $::FGen::State($lun,SINC,omega) * 2 * 3.14159256 / $len]

    for { set i [expr -$half_len] } { $i < $half_len } { incr i  1 } \
    {
      if { $i == 0 } \
      {
        # First value of the SINC is 1.0
        set y  1.0
      } \
      else \
      {
        # Calculate the rest SINC values
        set y  [expr sin( $omega * $i ) / ( $omega * $i )]
      }
 
      # Append the present value to the list of values
      ##lappend sinc_data  [expr $y + $dc]
      lappend  sinc_data  $y
    }

    # Download the data to the Function Generator engine by sending 
    # the variable name.
    FGen  defarb  ::SincUI::sinc_data
  }


  #**************************************************************************
  #
  #        Name: display_sinc_config
  # 
  #  Parameters: tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number for the function generator
  #
  # Description: Displays the SINC mode configuration toplevel 
  #
  #**************************************************************************
  proc display_sinc_config { tile_path  lun } \
  {
    #puts "display_sinc_config:  $tile_path,  $lun"
    
    variable  last_focus
      
    set last_focus  $tile_path

    set top_level  [winfo toplevel $tile_path]

    set tile_path  [name_child  $top_level  sinc_${lun}]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  SINC_CONFIG]
    }
    
    # Configure the title to reflect the LUN number
    wm title  $tile_path  "SINC Config #$lun"

    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8>  "destroy [winfo toplevel $tile_path]"
    
    # Bind the <Destroy> event for cleanup
    bind $tile_path <<Destroy>>  "SincUI::destroy_sinc_config  $tile_path  $lun"

    # Set the deviation entry depending on the Deviation units
    if { $lun == 3 || $lun == 4 } \
    {
      switch -- $::FGen::State(dev_units) \
      {
        "kHz"  { ${tile_path}.ent_level config  -textvariable ::FGen::State($lun,SINC,dev)     }
      
        "%"    { ${tile_path}.ent_level config  -textvariable ::FGen::State($lun,SINC,percent) }
      }
    }

    after idle  "focus -force  ${tile_path}.btn_return"
    
    return
  }


  #**************************************************************************
  #
  #        Name: destroy_sinc_config
  # 
  #  Parameters: tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number for the function generator
  #
  # Description: Handles the <Destroy> event for the SINC configuration 
  #              toplevel 
  #
  #**************************************************************************
  proc destroy_sinc_config { tile_path  lun } \
  {
    #puts "destroy_sinc_config:  $tile_path,  $lun"
     
    variable  last_focus
    variable  please_wait
    

    after idle  "focus -force $last_focus"

    return
  }


  #**************************************************************************
  #
  #        Name: sinc_restore_def
  # 
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Restore defaults
  #
  #**************************************************************************
  proc sinc_restore_def { lun } \
  {
    #puts "sinc_restore_def:  $lun"
     
    variable  StateUI
    
    switch -- $lun \
    {
      1  -
      2  { FGen::setLevel  $lun  $StateUI(level,default)   }
      
      3  -
      4  { FGen::setDeviation  $lun  $StateUI(dev,default) }
    }
    
    # Set the Rate
    FGen::setFreq  $lun  [expr $StateUI(rate,default) / 10.0]
    
    # Set the Omega value
    set ::FGen::State($lun,SINC,omega)  $StateUI(omega,default)

    # Perform the calculation of the SINC arbitrary waveform
    calculate_sinc  $lun

    # Set up the function generator parameters for SINC
    ::FGen::send_sinc_seq  $lun
  }


  #**************************************************************************
  #
  #        Name: getSaveState
  #
  #  Parameters: none
  #
  # Description: This function returns a list of TCL commands which can be 
  #              used to set the SINC mode to its current state
  #
  #**************************************************************************
  proc getSaveState { } \
  {
    set returnList { }
    

    return  [subst -nocommands $returnList]
  }


  #**************************************************************************
  #
  #        Name: Sync
  #
  #  Parameters: none
  #
  # Description: Write state variables to SincUI
  #
  #****************************************************************************
  proc Sync { } \
  {
    eval  [getSaveState]
  }


};  # End of the SincUI namespace




#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles   

  lappend tiles  SincUI
}



