#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C) 2003      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
#                                                                           #
# FILE        : 3002.ArbitraryUI.tcl                                        #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : Preston McCormick                                           #
#                                                                           #
# DESCRIPTION : Code for the ARBITRARY waveform                             #
#                                                                           #
#############################################################################

package require FGen


namespace eval ArbitraryUI \
{
  variable  last_focus
  variable  please_wait  ""
  
  variable  shape_names
  
  set shape_names(0)  ""
  
  variable  shape_data

  variable  file_ext      ".arb"
  variable  waveform_dirs  { "/system/waveforms"  "/usr/waveforms" }
    
  
  
  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: widget - any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "GEN"         -
        "SMALL_FGEN"  -
        "QTR_FGEN"    -
        "ARB_CONFIG"  { return $widget }
      }
       
      set widget [winfo parent $widget]
      if {$widget == .} { return }
    }
    
    return
  }


  #**************************************************************************
  #
  #        Name: setup_arb_waveform
  # 
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Starts the Arbitrary waveform
  #
  #**************************************************************************
  proc setup_arb_waveform { lun } \
  {
    Debug::Puts ARB 10 "setup_arb_waveform:  $lun"

    # Set the Level
    FGen::setLevel  $lun  $::FGen::State($lun,ARB,level)
      
    # Set the Rate
    FGen::setFreq  $lun  [expr $::FGen::State($lun,ARB,rate) / 10.0]
    
    set_arb_shape  $lun
  }


  #**************************************************************************
  #
  #        Name: get_arb_file_names
  # 
  #  Parameters: tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number for the function generator
  #
  # Description: Get the available shape names and add the names of the 
  #              data to the shape menu, then invoke the first waveform.
  #              Called by the quarter function generator tile and the 
  #              configuration toplevel.
  #
  #**************************************************************************
  proc get_arb_file_names { menu lun } \
  {
    Debug::Puts ARB 10 "get_arb_file_names:  $menu,  $lun"

    variable  file_ext
    variable  shape_names
    variable  shape_data
    variable  waveform_dirs
    
    get_arb_files

    # Delete existing menu items
    $menu  delete  0  end

    # Get the names of the waveforms
    set max_index  [array  size  shape_names]
    set format %1d
    if {$max_index >= 100} \
    {
       set format %03d
    } \
    elseif {$max_index >= 10} \
    {
       set format %02d
    }
    for { set i  0 } { $i < $max_index } { incr i  1 } \
    {
      set name  $shape_names($i)
      
      # Add the shape name to the shape menu as a radio button
      $menu  add  radio                               \
                          -label  $name                             \
                          -value  $name                             \
                          -accel [format $format $i]                \
                          -variable  ::FGen::State($lun,ARB,shape)  \
                          -command  "ArbitraryUI::set_arb_shape  $lun"
      bind $menu [format $format $i] "do_menu_key %W $i"
    }
  }


  #**************************************************************************
  #
  #        Name: get_arb_files
  # 
  #  Parameters:   #
  # Description: Get the available shape files, store the data descriptions
  #              and data in local variables.
  #
  #**************************************************************************
  proc get_arb_files { args } \
  {
    variable  file_ext
    variable  shape_names
    variable  shape_data
    variable  waveform_dirs
    variable waveform_files
    
    Debug::Puts ARB 10 "get_arb_files"
    # Initialize the shape_name array index
    set name_index  0
      
    # Check all waveform directories
    foreach { waveform_dir } $waveform_dirs \
    {
      # Change to the system waveform directory
      # If there is an error, return
      if ![file exists $waveform_dir] \
      {
         file mkdir $waveform_dir
      }
      if ![file isdir $waveform_dir] \
      {
        puts "'$waveform_dir' is not a directory"
        
        continue
      }
        
      # Get the waveform files that exist in the directory
      foreach { file_name } [glob -nocomplain  $waveform_dir/*.arb] \
      {
        set file_size  [file size  $file_name]
        set file_mtime [file mtime $file_name]
        
        # do I need to reload?
        if [info exists waveform_files($file_name,file_size)] \
        {
           if {($waveform_files($file_name,file_size) == $file_size) &&
               ($waveform_files($file_name,file_mtime) == $file_mtime)
              } \
           {
              continue
           }
        }
        set waveform_files($file_name,file_size)  $file_size
        set waveform_files($file_name,file_mtime) $file_mtime
        # Open the file
        set file_id  [open $file_name r]
        set data [read $file_id]
        close $file_id
        array set shape $data
        
        # Convert the shape name to all upper case letters
        set shape_names($name_index)  [string toupper  $shape(NAME)]
        
        
        # Assign the description to a variable referenced by the shape name
        set shape_data($shape_names($name_index),desc)  $shape(DESCRIPTION)
        
        # Assign the data to a variable referenced by the shape name
        set shape_data($shape_names($name_index),data)  $shape(data)
        
        if [info exists shape(RATE)] \
        {
           set shape_data($shape_names($name_index),rate)  $shape(RATE)
        } \
        else \
        {
           set shape_data($shape_names($name_index),rate) ""
        }
       
        incr  name_index  1
      }
    }
  }


  #**************************************************************************
  #
  #        Name: set_arb_level
  # 
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the Level of the Arbitrary Waveform
  #
  #**************************************************************************
  proc set_arb_level { lun } \
  {
    Debug::Puts ARB 10 "set_arb_level:  $lun"

    if { $lun == 1 || $lun == 2 } \
    {
      # Set the Level
      FGen::setLevel  $lun  $::FGen::State($lun,ARB,level)
    
      # Procedure 'ArbitraryUI::send_arb_seq' called in 'FGen::setLevel'
    }
  }
  
  
  #**************************************************************************
  #
  #        Name: set_arb_dev
  # 
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the Deviation of the Arbitrary Waveform
  #
  #**************************************************************************
  proc set_arb_dev { lun } \
  {
    Debug::Puts ARB 10 "set_arb_dev:  $lun"

    # Set the deviation entry depending on the Deviation units
    if { $lun == 3 || $lun == 4 } \
    {
      switch -- $::FGen::State(dev_units) \
      {
        "kHz"  { # Set the Deviation to the present value
                 FGen::setDeviation  $lun  $::FGen::State($lun,ARB,dev)
      
                 # Procedure 'SincUI::send_sinc_seq' called in 'FGen::setDeviation' 
               }
      
        "%"    { # Set the Percent Deviation to the present value
                 FGen::setPercent  $lun  $::FGen::State($lun,ARB,percent)
      
                 # Procedure 'SincUI::send_sinc_seq' called in 'FGen::setPercent' 
               }
      }
    }
  }
  
  
  #**************************************************************************
  #
  #        Name: set_arb_rate
  # 
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the Rate (Frequency) of the Arbitrary Waveform
  #
  #**************************************************************************
  proc set_arb_rate { lun } \
  {
    Debug::Puts ARB 10 "set_arb_rate:  $lun"

    # Set the Rate
    FGen::setFreq  $lun  [expr $::FGen::State($lun,ARB,rate) / 10.0]
    
    # Procedure 'ArbitraryUI::send_arb_seq' called in 'FGen::setFreq'

    return
  }
  
  
  #**************************************************************************
  #
  #        Name: set_arb_shape
  # 
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Sets the Arbitrary Waveform shape of the Function Generator
  #
  #**************************************************************************
  proc set_arb_shape { lun } \
  {
    Debug::Puts ARB 10 "set_arb_shape:  $lun"
     
    variable  file_desc
    variable  shape_data
    
    set shape_name  $::FGen::State($lun,ARB,shape)
    if ![info exists shape_data($shape_name,desc)] { return }
    
    # Display the description for the file
    set file_desc($lun)  $shape_data($shape_name,desc)
    
    # Download the data to the Function Generator engine by sending 
    # the variable name.
    FGen  defarb  ::ArbitraryUI::shape_data($shape_name,data)
    
    if {[info exists ::ArbitraryUI::shape_data($shape_name,rate)] &&
        $::ArbitraryUI::shape_data($shape_name,rate) != ""
       }\
    {
       set ::FGen::State($lun,ARB,rate) $shape_data($shape_name,rate) 
       set_arb_rate $lun
    }
    
  }


  #**************************************************************************
  #
  #        Name: send_arb_seq
  # 
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Send out the Arbitrary Waveform sequence
  #
  #**************************************************************************
  proc send_arb_seq { lun } \
  {
    Debug::Puts ARB 10 "send_arb_seq:  $lun"

    # Turn on the Function Generator in Arbitrary mode
    after idle "FGen::Send  $lun  $::FGen::State($lun,ARB,freq)  \
                                  0                              \
                                  ARB                            \
                                  0                              \
                                  $::FGen::State($lun,ARB,level)"
    
  }
  

  #**************************************************************************
  #
  #        Name: display_arb_config
  # 
  #  Parameters: tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number for the function generator
  #
  # Description: Displays the Arbitrary Waveform mode configuration toplevel 
  #
  #**************************************************************************
  proc display_arb_config { tile_path  lun } \
  {
    Debug::Puts ARB 10 "display_arb_config:  $tile_path,  $lun"
    
    variable  last_focus
      
    set last_focus  $tile_path

    set top_level  [winfo toplevel $tile_path]

    set tile_path  [name_child  $top_level  arb_${lun}]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  ARB_CONFIG]
    }
    
    # Configure the title to reflect the LUN number
    wm title  $tile_path  "Arbitrary Waveform Config #$lun"

    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8>  "destroy [winfo toplevel $tile_path]"
    
    bind $tile_path <<Destroy>>  "ArbitraryUI::destroy_arb_config  $tile_path  $lun"

    # Set the deviation entry depending on the Deviation units
    if { $lun == 3 || $lun == 4 } \
    {
      switch -- $::FGen::State(dev_units) \
      {
        "kHz"  { ${tile_path}.ent_level config  -textvariable ::FGen::State($lun,ARB,dev)     }
      
        "%"    { ${tile_path}.ent_level config  -textvariable ::FGen::State($lun,ARB,percent) }
      }
    }
    
    # Define the menubutton to be manipulated
    if { [winfo class $tile_path] == "QTR_FGEN" } \
    {
      set menu_button  ${tile_path}.fgen${lun}.frm_arb.mnu_arb_shape
    }
    
    if { [winfo class $tile_path] == "ARB_CONFIG" } \
    {
      set menu_button  ${tile_path}.mnu_shape
    }
    if [winfo exists $menu_button] \
    {
       set menu [$menu_button cget -menu]
       $menu config -postcommand "ArbitraryUI::get_arb_file_names  $menu  $lun"
    }
    safe_focus_force  ${tile_path}.btn_return"
  }


  #**************************************************************************
  #
  #        Name: destroy_arb_config
  # 
  #  Parameters: tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number for the function generator
  #
  # Description: Handles the <Destroy> event for the Arbitrary Waveform
  #              configuration toplevel 
  #
  #**************************************************************************
  proc destroy_arb_config { tile_path  lun } \
  {
    Debug::Puts ARB 10 "destroy_arb_config:  $tile_path,  $lun"
     
    variable  last_focus
    safe_focus_force $last_focus
  }


  #**************************************************************************
  #
  #        Name: arb_restore_def
  # 
  #  Parameters: lun - Logical Unit Number for the function generator
  #
  # Description: Restore defaults
  #
  #**************************************************************************
  proc arb_restore_def { lun } \
  {
    Debug::Puts ARB 10 "arb_restore_def:  $lun"
  }


  #**************************************************************************
  #
  #        Name: getSaveState
  #
  #  Parameters: none
  #
  # Description: This function returns a list of TCL commands which can be 
  #              used to set the Arbitrary Waveform mode to its current 
  #              state
  #
  #**************************************************************************
  proc getSaveState { } \
  {
    set returnList \
    {
      foreach lun { 1 2 3 4 } \
      {
         if {$::FGen::State($lun,fgen_mode) == "ARB"} \
         {
            ArbitraryUI::set_arb_shape $lun
            send_arb_seq  $lun
         }
      }
    }

    return $returnList
  }


  #**************************************************************************
  #
  #        Name: Sync
  #
  #  Parameters: none
  #
  # Description: Write state variables to ArbitraryUI
  #
  #****************************************************************************
  proc Sync { } \
  {
    get_arb_files
    eval [getSaveState]
  }
  Sync

};  # End of the ArbitraryUI namespace




#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles   

  lappend tiles  ArbitraryUI
}


