#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2003      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 3004.DcsUI.tcl                                              #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Preston McCormick                                           #
# DESCRIPTION : Code for the DCS Tone Signaling mode                        #
#                                                                           #
#############################################################################


namespace eval DCS_UI \
{
  variable  last_focus   ""
  variable  please_wait  ""
  

  
  #***************************************************************************
  #
  # Name:        get_tile
  #
  # Parameters:  widget - any child widget of the tile
  #
  # Description: get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "GEN"          -
        "SMALL_FGEN"   -
        "QTR_FGEN"     -
        "DCS_CONFIG"   { return $widget}
      }
        
      set widget [winfo parent $widget]
    }
  }


  #***************************************************************************
  #
  # Name:        dcs_key_return
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Handles the <Return> key press
  #
  #***************************************************************************
  proc dcs_key_return { tile_path  lun } \
  {
    #puts "dcs_key_return:  $tile_path,  $lun"

    set entry_path  $tile_path

    # Move the cursor to the end of the string
    $entry_path icursor end
    
    # Clear the selection
    $entry_path select clear

    # Reset the foreground color of the entry widget to black
    $entry_path config -fg black
    
    dcs_validate  $lun

    return
  }
  
  #***************************************************************************
  #
  # Name:        dcs_validate
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Validates the DCS code
  #
  #***************************************************************************
  proc dcs_validate { lun } \
  {
    #puts "dcs_validate:  $lun"
    
    set dcs_code  $::FGen::State($lun,DCS,code)

    
    # Format the string as a valid octal number
    set str_length  [string length  $dcs_code]
    
    switch -- $str_length \
    {
      0       { set dcs_code  "000" }
      
      1       { set pad  "00"
                
                set dcs_code  [append pad $dcs_code]  }
      
      2       { set pad  "0"
      
                set dcs_code  [append pad $dcs_code]  }
      
      default { # If the string has 3 or more characters, 
                # take the last three characters
                set dcs_code  [string range $dcs_code  [expr $str_length - 3] $str_length] }
    }
    
    set FGen::State($lun,DCS,code)  $dcs_code

    
    # Validate the code value by determining if the value entered
    # is in the menu of valid values
    set found  [lsearch -exact  $::DCS::codes  $dcs_code]

    #puts "found:  $found"
    
    if { $found == -1 } \
    {
      set ::FGen::dcs_invalid($lun)  1
    } \
    else \
    {
      set ::FGen::dcs_invalid($lun)  0
    }
    
    # Send out the DCS sequence if it is ON
    if { $::FGen::State($lun,DCS,state) == "ON" } \
    {
      dcs_start  $lun
    }
    
    return
  }


  #***************************************************************************
  #
  # Name:        dcs_toggle_state
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Toggles the DCS state between ON and OFF
  #
  #***************************************************************************
  proc dcs_toggle_state { lun } \
  {
    #puts "dcs_toggle_state:  $lun"
    
    if { $::FGen::State($lun,DCS,state) == "OFF" } \
    {
      set ::FGen::State($lun,DCS,state)  "ON"
      
      dcs_start  $lun
    } \
    else \
    {
      set ::FGen::State($lun,DCS,state)  "OFF"
      
      dcs_stop  $lun
    }

    return
  }


  #***************************************************************************
  #
  # Name:        dcs_toggle_invert
  #
  # Parameters:  lun       - Logical Unit Number of the function generator
  #
  # Description: Toggles the DCS state between INVERT and NORMAL
  #
  #***************************************************************************
  proc dcs_toggle_invert { lun } \
  {
    #puts "dcs_toggle_invert:  $lun,  $new_state"
    
    # Value '172' does not have an invert
    if { $::FGen::State($lun,DCS,code) == "172" } \
    {
      return
    }
    
    if { $::FGen::State($lun,DCS,invert) == "INVERT" } \
    {
      set ::FGen::State($lun,DCS,invert)  "NORMAL"
    } \
    else \
    {
      set ::FGen::State($lun,DCS,invert)  "INVERT"
    }
    
    # Get the invert code
    set dcs_code  $::FGen::State($lun,DCS,code)
      
    if { [info exists ::DCS::invert_codes($dcs_code)] } \
    {
      set ::FGen::State($lun,DCS,code)  $::DCS::invert_codes($dcs_code)
    }

    # Send out the DCS sequence if it is ON
    # to send out the new invert value
    if { $::FGen::State($lun,DCS,state) == "ON" } \
    {
      dcs_start  $lun
    }

    return
  }


  #***************************************************************************
  #
  # Name:        dcs_start
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Sends a DCS code
  #
  #***************************************************************************
  proc dcs_start { lun } \
  {
    #puts "dcs_start:  $lun"

    set dcs_code  $FGen::State($lun,DCS,code)
    
    set ::FGen::State($lun,DCS,seq)  ""
    set ::FGen::State($lun,DCS,seq)  [DCS::Compose  $dcs_code]

    # Send the sequence
    dcs_send  $lun 
    
    return
  }
  
  
  #***************************************************************************
  #
  # Name:        dcs_stop
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Stops the DCS code burst
  #
  #***************************************************************************
  proc dcs_stop { lun } \
  {
    #puts "dcs_stop:  $lun"
    
    # Set up the Unkey sequence
    set ::FGen::State($lun,DCS,seq)  ""
    set ::FGen::State($lun,DCS,seq)  [DCS::Compose_unkey]

    # Send the unkey
    dcs_send  $lun

    return
  }


  #***************************************************************************
  #
  # Name:        dcs_send
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Sends the DCS code sequence
  #
  #***************************************************************************
  proc dcs_send { lun } \
  {
    #puts "dcs_send:  $lun"
    
    #puts "dcs_seq:  $::FGen::State($lun,DCS,seq)"

    # Send the Tone items
    # Break the string into a valid list
    FGen::Send  $lun  [lindex $::FGen::State($lun,DCS,seq) 0] \
                      [lindex $::FGen::State($lun,DCS,seq) 1] \
                      [lindex $::FGen::State($lun,DCS,seq) 2] \
                      [lindex $::FGen::State($lun,DCS,seq) 3]

    return
  }


  #***************************************************************************
  #
  # Name:        dcs_config
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Displays the DCS configuration toplevel
  #
  #***************************************************************************
  proc dcs_config { tile_path  lun } \
  {
    #puts "dcs_config:  $tile_path,  $lun"
    
    variable  last_focus

    set last_focus  $tile_path

    set top_level  [winfo toplevel $tile_path]

    set tile_path  [name_child  $top_level  dcs_${lun}]

    if { ![winfo exists  $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  DCS_CONFIG]
    }
    
    # Configure the title to reflect the LUN number
    wm title  $tile_path  "DCS Config #$lun"

    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8>  "destroy [winfo toplevel $tile_path]"

    bind $tile_path <<Destroy>>  "DCS_UI::dcs_config_return  $tile_path  $lun"

    # Trace on the dcs_invalid variable to turn the foreground of the
    # code entry widget yellow if the code is invalid
    # At this time the ColorTrack is not working
    trace variable  ::FGen::dcs_invalid($lun)  w  \
                                 "DCS_UI::update_code_valid  $tile_path  $lun"
                                 
    update_code_valid  $tile_path  $lun                                 
    

    after idle  "focus -force  ${tile_path}.btn_return"

    return
  }


  #***************************************************************************
  #
  # Name:        dcs_config_return
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Handles the <Destroy> event for the DCS configuration
  #              toplevel
  #
  #***************************************************************************
  proc dcs_config_return { tile_path  lun } \
  {
    #puts "dcs_config_return:  $tile_path,  $lun"

    variable  last_focus
    variable  please_wait
    
    set please_wait  "Please Wait ..."
    
    update
    
    # Stop the trigger event
    FGen::setResponse  $lun  1  {}

    set please_wait  ""
    
    update
    
    # Delete the trace on the dcs_invalid variable 
    # At this time the ColorTrack is not working
    trace vdelete  ::FGen::dcs_invalid($lun)  w  \
                                 "DCS_UI::update_code_valid  $tile_path  $lun"


    after idle  "focus -force $last_focus"
    
    return
  }


  #***************************************************************************
  #
  # Name:        update_code_valid
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Turns the code foreground yellow if the code is invalid
  #
  #***************************************************************************
  proc update_code_valid { tile_path  lun  args } \
  {
    #puts "update_code_valid:  $tile_path,  $lun,  $args"
    
    set tile_path  [get_tile  $tile_path]
        
    if { $::FGen::dcs_invalid($lun) == 0 } \
    {
      ${tile_path}.ent_dcs_code  config  -fg black
    } \
    else \
    {
      ${tile_path}.ent_dcs_code  config  -fg yellow
    }
    
    return
  }


};  # End of DCS_UI namespace
