#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C) 2003      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
#                                                                           #
# FILE        : 3006.DtmfUI.tcl                                             #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : Preston McCormick                                           #
#                                                                           #
# DESCRIPTION : Code for the DTMF Tone Signaling mode                       #
#                                                                           #
#############################################################################


namespace eval DTMF_UI \
{
  variable  last_focus   ""
  variable  please_wait  ""
  

  #***************************************************************************
  #
  # Name:        get_tile
  #
  # Parameters:  widget - any child widget of the tile
  #
  # Description: get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
     
     while { 1 } \
     {
       switch -- [winfo class $widget] \
       {
         "GEN"          -
         "SMALL_FGEN"   -
         "QTR_FGEN"     -
         "DTMF_CONFIG"  { return $widget}
       }
        
       set widget [winfo parent $widget]
     }
     
  }


  #***************************************************************************
  #
  # Name:        dtmf_set_mode
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Sets the DTMF mode
  #
  #***************************************************************************
  proc dtmf_set_mode { lun  args } \
  {
    #puts "dtmf_set_mode:  $lun,  $args"
    
    # Stop the existing tone sequence
    FGen::setResponse  $lun  1  {}
        
    ## Stop any existing tone
    #FGen::Send  $lun  [FGen::Off]

    set dtmf_code  $::FGen::State($lun,DTMF,code)
    
    if { $dtmf_code == "" } \
    {
      return
    }

    switch -- $::FGen::State($lun,DTMF,mode) \
    {
      "CONT"    -
      "BURST"   { dtmf_send_seq  $lun }
      
      "KEY PAD" { }
    }


    return
  }


  #***************************************************************************
  #
  # Name:        dtmf_key_return
  #
  # Parameters:  tile_path - tile path of the invoking widget (%W)
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Handles the <Return> key press
  #
  #***************************************************************************
  proc dtmf_key_return { tile_path  lun } \
  {
    #puts "dtmf_key_return:  $tile_path,  $lun"

    set entry_path  $tile_path
    
    # Move the cursor to the end of the string
    $entry_path icursor end
    
    # Clear the selection
    $entry_path select clear

    switch -- $::FGen::State($lun,DTMF,mode) \
    {
      "CONT"    -
      "BURST"   { dtmf_send_seq  $lun }
      
      "KEY PAD" { }
    }

    return
  }
  
  
  #***************************************************************************
  #
  # Name:        dtmf_key_backspace
  #
  # Parameters:  tile_path - tile path of the invoking widget (%W)
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Handles the <BackSpace> key press
  #
  #***************************************************************************
  proc dtmf_key_backspace { tile_path  lun } \
  {
    #puts "dtmf_key_backspace:  $tile_path,  $lun"

    set entry_path  $tile_path
    
    # Move the cursor to the end of the string
    $entry_path icursor end
    
    # Clear the selection
    $entry_path select clear

    # Delete the character just before the insert cursor
    $entry_path delete  [expr [$entry_path index end] - 1]  end

    return
  }
  
  
  #***************************************************************************
  #
  # Name:        dtmf_key_press
  #
  # Parameters:  tile_path - tile path of the invoking widget (%W)
  #              key_value - the value of the key that was pressed (%A)
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Handles the <KeyPress> event
  #
  #***************************************************************************
  proc dtmf_key_press { tile_path  key_value  lun } \
  {
    
    switch -- $key_value \
    {  
      "0"  -
      "1"  -
      "2"  -
      "3"  -
      "4"  -
      "5"  -
      "6"  -
      "7"  -
      "8"  -
      "9"  -
      "a"  -
      "b"  -
      "c"  -
      "d"  -
      "A"  -
      "B"  -
      "C"  -
      "D"  -
      "*"  -
      "#"  { set value  $key_value }

      default  { return }
    }

    switch -- $::FGen::State($lun,DTMF,mode) \
    {
      "KEY PAD"  \
      {
         set ::FGen::State($lun,DTMF,digit) $value
         after idle "DTMF_UI::dtmf_send_seq  $lun"
       }
    }

  }
  
  
  #***************************************************************************
  #
  # Name:        dtmf_key_release
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Handles the <KeyRelease> event
  #
  #***************************************************************************
  proc dtmf_key_release { lun } \
  {
    #puts "dtmf_key_release:  $lun"
    
    switch -- $::FGen::State($lun,DTMF,mode) \
    {
       "KEY PAD"
       {
          # Stop any existing tone
           FGen::Send  $lun  [FGen::Off]
           set ::FGen::State($lun,DTMF,code) ""
       }
    }
  }


  #***************************************************************************
  #
  # Name:        dtmf_send_seq
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Send the DTMF code sequence
  #
  #***************************************************************************
  proc dtmf_send_seq { lun } \
  {
    #puts "dtmf_send_seq:  $lun"
    
    set dtmf_code  $::FGen::State($lun,DTMF,code)
    
    if { $dtmf_code == "" } \
    {
      return
    }
    
    #puts "dtmf_code:  $dtmf_code"
    
    # Clear the present sequence value list
    set ::FGen::State($lun,DTMF,seq)  {}
      
    switch -- $::FGen::State($lun,DTMF,mode) \
    {
      "CONT"     { # Compose the Tone list
                   set ::FGen::State($lun,DTMF,seq)  [DTMF::Compose  $dtmf_code]
      
                   # Append the Event item to the list
                   lappend  ::FGen::State($lun,DTMF,seq)  [FGen::Event 1]

                   # Setup the trigger event trigger
                   FGen::setResponse  $lun  1  "::DTMF_UI::dtmf_burst_seq  $lun"

                   # Send the first burst
                   dtmf_burst_seq  $lun }
    
      "BURST"    { # Compose the Tone list
                   set ::FGen::State($lun,DTMF,seq)  [DTMF::Compose  $dtmf_code]
      
                   # Append the Event item to the list
                   lappend  ::FGen::State($lun,DTMF,seq)  [FGen::Event 1]

                   # Stop the trigger event
                   FGen::setResponse  $lun  1  {}
 
                   # Send the burst
                   dtmf_burst_seq  $lun }

      "KEY PAD"  { # Compose the digit Tone
                   if { $::FGen::State($lun,DTMF,digit) == "" } \
                   {
                     return
                   }
                   
                   set dtmf_digit   $::FGen::State($lun,DTMF,digit)
                   
                   set ::FGen::State($lun,DTMF,seq)  [DTMF::Compose_digit  $dtmf_digit]
      
                   # Send the burst
                   dtmf_burst_seq  $lun }
    }

    return
  }


  #***************************************************************************
  #
  # Name:        dtmf_burst_seq
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Sends a DTMF burst
  #
  #***************************************************************************
  proc dtmf_burst_seq { lun } \
  {
    #puts "dtmf_burst_seq:  $lun"
    
    #puts "dtmf_seq:  $::FGen::State($lun,DTMF,seq)"
    
    # Send the Tone items
    FGen::Send  $lun  $::FGen::State($lun,DTMF,seq)

    
    return
  }


  #***************************************************************************
  #
  # Name:        dtmf_config
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #              value     - the value of the key that was pressed
  #
  # Description: Displays the DTMF configuration toplevel
  #
  #***************************************************************************
  proc dtmf_config { tile_path  lun } \
  {
    #puts "dtmf_config:  $tile_path,  $lun"
    
    variable  last_focus
      
    set last_focus  $tile_path

    set top_level [winfo toplevel $tile_path]

    set tile_path   [name_child  $top_level  dtmf_$lun]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  DTMF_CONFIG]
    }

    # Configure the title to reflect the LUN number
    wm title  $tile_path  "DTMF Config #$lun"

    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8> "destroy [winfo toplevel $tile_path]"

    bind $tile_path <<Destroy>> "DTMF_UI::dtmf_config_return  $lun"

    after idle  "focus -force  ${tile_path}.btn_return"

    return
  }


  #***************************************************************************
  #
  # Name:        dtmf_config_return
  #
  # Parameters:  value     - the value of the key that was pressed
  #
  # Description: Handles the <Destroy> event for the DTMF configuration
  #              topelevel
  #
  #***************************************************************************
  proc dtmf_config_return { lun } \
  {
    #puts "dtmf_config_return:  $lun"

    variable  last_focus
    variable  please_wait
    
    set please_wait  "Please Wait ..."

    # Stop the trigger event
    FGen::setResponse  $lun  1  {}

    set please_wait  ""

    safe_focus_force $last_focus
  }


  #***************************************************************************
  #
  # Name:        dtmf_restore_def
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Restores the default DTMF timing values
  #
  #***************************************************************************
  proc dtmf_restore_def { lun } \
  {
    #puts "dtmf_restore_def:  $lun"
    
    set ::DTMF::Timing(MARK)   $::DTMF::Timing(MARK_DEFAULT)
    set ::DTMF::Timing(SPACE)  $::DTMF::Timing(SPACE_DEFAULT)
    set ::DTMF::Timing(END)    $::DTMF::Timing(END_DEFAULT)

  }


};  # End of DTMF_UI namespace
