#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C) 2003      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
#                                                                           #
# FILE        : 3007.ToneRemoteUI.tcl                                       #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : Preston McCormick                                           #
#                                                                           #
# DESCRIPTION : Code for the Tone Remote Tone Signaling mode                #
#                                                                           #
#############################################################################


namespace eval ::ToneRemoteUI \
{
  variable  please_wait  ""

  
  #***************************************************************************
  #
  # Name:        get_tile
  #
  # Parameters:  widget - any child widget of the tile
  #
  # Description: get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
     while { 1 } \
     {
       switch -- [winfo class $widget] \
       {
         "GEN"          -
         "SMALL_FGEN"   -
         "QTR_FGEN"     -
         "TREM_CONFIG"  { return $widget}
       }
        
       set widget [winfo parent $widget]
     }
     
  }


  #***************************************************************************
  #
  # Name:        trem_send_seq
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Sends the Tone Remote code sequence
  #
  #***************************************************************************
  proc trem_send_seq { lun } \
  {
    #puts "trem_send_seq:  $lun"
    
    # Clear the present sequence value list
    set ::FGen::State($lun,TREM,seq)  {}
      
    # If the selected code is 'USER' get the user entry value
    if { $::FGen::State($lun,TREM,code) == "USER" } \
    {
      # Compose the Tone list
      set ::FGen::State($lun,TREM,seq)  \
                    [ToneRemote::Compose  $::FGen::State($lun,TREM,user_code)]
    } \
    else \
    {    
      # Compose the Tone list
      set ::FGen::State($lun,TREM,seq)  \
                         [ToneRemote::Compose  $::FGen::State($lun,TREM,code)]
    }
      
    # Append the Event item to the list
    lappend  ::FGen::State($lun,TREM,seq)  [FGen::Event 1]

    # Clear the trigger event
    FGen::setResponse  $lun  1  {}

    # Send the first burst
    trem_burst_seq  $lun
  
    return
  }


  #***************************************************************************
  #
  # Name:        trem_burst_seq
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Sends the Tone Remote code burst
  #
  #***************************************************************************
  proc trem_burst_seq { lun } \
  {
    #puts "trem_burst_seq:  $lun"
    
    #puts "t_rem_seq:  $::FGen::State($lun,TREM,seq)"

    # Send the Tone items
    after idle "FGen::Send  $lun  $::FGen::State($lun,TREM,seq)"

    return
  }


  #***************************************************************************
  #
  # Name:        trem_set_code
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Updates the display to the Tone Remote code in the 
  #              FUNCTION FREQ box on the config toplevel
  #
  #***************************************************************************
  proc trem_set_code { lun  args } \
  {
    #puts "trem_set_code:  $lun,  $args"
    
    set t_rem_code       $::FGen::State($lun,TREM,code)
    set t_rem_user_code  $::FGen::State($lun,TREM,user_code)
    
    #puts "t_rem_code:  $t_rem_code"
    
    if { $t_rem_code == "USER" } \
    {
      set ::ToneRemote::Freqs(FUNCTION)  $t_rem_user_code
    } \
    else \
    {
      set ::ToneRemote::Freqs(FUNCTION)  $t_rem_code
    }          
  
    return
  }
  
  
  #***************************************************************************
  #
  # Name:        trem_config
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Displays the Tone Remote configuration toplevel
  #
  #***************************************************************************
  proc trem_config { tile_path  lun } \
  {
    #puts "trem_config:  $tile_path,  $lun"
    
    variable  last_focus
      
    set last_focus  $tile_path

    set top_level [winfo toplevel $tile_path]

    set tile_path   [name_child  $top_level  t_rem_$lun]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_toplevel_tile  $tile_path  TREM_CONFIG]
    }
    
    # Configure the title to reflect the LUN number
    wm title  $tile_path  "Tone Remote Config #$lun"

    # Bind the F8 key for a return
    bind $tile_path <KeyRelease-F8> "destroy [winfo toplevel $tile_path]"
    
    bind $tile_path <<Destroy>> \
                         "ToneRemoteUI::trem_config_return  $tile_path  $lun"

    trace variable  ::ToneRemote::GuardDur  w  \
                      "ToneRemoteUI::trem_update_guard_dur  $tile_path  $lun"

    trem_update_guard_dur  $tile_path  $lun
    
    trace variable  ::FGen::State($lun,TREM,code)  w  \
                           "ToneRemoteUI::trem_update_user  $tile_path  $lun"

    trem_update_user  $tile_path  $lun

    safe_focus_force ${tile_path}.btn_return
    
    return
  }

  
  #***************************************************************************
  #
  # Name:        trem_config_return
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Handles the <Destroy> event for  the Tone Remote 
  #              configuration toplevel
  #
  #***************************************************************************
  proc trem_config_return { tile_path  lun } \
  {
    #puts "trem_config_return:  $tile_path,  $lun"
    
    variable  last_focus
    variable  please_wait
    
    set please_wait  "Please Wait ..."
        
    trace vdelete  ::ToneRemote::GuardDur  w  \
                      "ToneRemoteUI::trem_update_guard_dur  $tile_path  $lun"

    trace vdelete  ::FGen::State($lun,TREM,code)  w  \
                           "ToneRemoteUI::trem_update_user  $tile_path  $lun"

    # Stop the trigger event
    FGen::setResponse  $lun  1  {}

    set please_wait  ""
    
    safe_focus_force $last_focus

  }


  #***************************************************************************
  #
  # Name:        trem_update_guard_dur
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Enables or disables the GUARD DUR entry widget depending on
  #              the Guard mode
  #
  #***************************************************************************
  proc trem_update_guard_dur { tile_path  lun  args } \
  {
    #puts "trem_update_guard_dur:  $tile_path,  $lun,  $args"
    
    set tile_path  [get_tile  $tile_path]
    
    set entry  ${tile_path}.ent_guard_dur
    
    if { ![winfo exists $entry] } \
    {
      return
    }
    
    if { $::ToneRemote::GuardDur == "CONT" } \
    {
      $entry config -state disabled  -foreground white  -selectforeground white
    } \
    else \
    {
      $entry config -state normal  -foreground black  -selectforeground black
    }
    
    return
  }


  #***************************************************************************
  #
  # Name:        trem_update_user
  #
  # Parameters:  tile_path - tile path of the invoking widget
  #              lun       - Logical Unit Number of the function generator
  #
  # Description: Updates the display by showing the USER entry widget if
  #              the CODE is set to USER
  #
  #***************************************************************************
  proc trem_update_user { tile_path  lun  args } \
  {
    #puts "trem_update_user:  $tile_path,  $lun,  $args"
    
    set tile_path  [get_tile  $tile_path]
    
    if { ![winfo exists  ${tile_path}.ent_trem_user_code] } \
    {
      return
    }
    
    if { $::FGen::State($lun,TREM,code) == "USER" } \
    {
      Screens::position_widget  ${tile_path}.ent_trem_user_code
      Screens::position_widget  ${tile_path}.lbl_trem_user_code
      Screens::position_widget  ${tile_path}.lbl_trem_user_code_units
    } \
    else \
    {
      grid remove  ${tile_path}.ent_trem_user_code
      grid remove  ${tile_path}.lbl_trem_user_code
      grid remove  ${tile_path}.lbl_trem_user_code_units
    }
    
    return
  }


  #***************************************************************************
  #
  # Name:        trem_restore_def
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Restores the default Tone Remote timing values
  #
  #***************************************************************************
  proc trem_restore_def { lun } \
  {
    #puts "trem_restore_def:  $lun"
    
    set ::ToneRemote::Timing(FUNCTION)  $::ToneRemote::Timing(FUNCTION_DEFAULT)
    set ::ToneRemote::Timing(MAX)       $::ToneRemote::Timing(MAX_DEFAULT)
    set ::ToneRemote::Timing(GUARD)     $::ToneRemote::Timing(GUARD_DEFAULT)

    set ::ToneRemote::Freqs(FUNCTION)   $::ToneRemote::Freqs(FUNCTION_DEFAULT)
    set ::ToneRemote::Freqs(MAX)        $::ToneRemote::Freqs(MAX_DEFAULT)
    set ::ToneRemote::Freqs(GUARD)      $::ToneRemote::Freqs(GUARD_DEFAULT)

    set ::ToneRemote::Levels(FUNCTION)  $::ToneRemote::Levels(FUNCTION_DEFAULT)
    set ::ToneRemote::Levels(MAX)       $::ToneRemote::Levels(MAX_DEFAULT)
    set ::ToneRemote::Levels(GUARD)     $::ToneRemote::Levels(GUARD_DEFAULT)

  }


  #***************************************************************************
  #
  # Name:        trem_toggle_guard_dur
  #
  # Parameters:  lun - Logical Unit Number of the function generator
  #
  # Description: Toggles the Guard duration between CONT and BURST
  #
  #***************************************************************************
  proc trem_toggle_guard_dur { lun } \
  {
    #puts "trem_toggle_guard_dur"
    
    if { $::ToneRemote::GuardDur == "CONT" } \
    {
      set ::ToneRemote::GuardDur "BURST"
      
      # Cancel any existing tones
      FGen::Send  $lun  0  0  SINE  0
    } \
    else \
    {
      set ::ToneRemote::GuardDur "CONT"
    }

    return
  }


};  # End of ToneRemoteUI namespace
