#############################################################################
#                                                                           #
# IFR SYSTEMS, INC        COPYRIGHT (C)1998-2001    DESIGN ENGINEERING      #
#                                                                           #
# --------------------------------------------------------------------------#
#                                                                           #
# FILE        : 3533.AnalyzerUI.tcl                                         #
#                                                                           #
# PRODUCT     : FITS                                                        #
#                                                                           #
# AUTHOR      : Preston McCormick                                           #
#                                                                           #
# DESCRIPTION : Spectrum Analyzer User Interface                            #
#                                                                           #
#############################################################################

package require Analyzer

namespace eval Analyzer_FullUI \
{
  # Global variables
  variable LongName       "fullanlz"
  variable ShortName      "Spectrum Analyzer"
  variable Size           Full

  variable tile_class     FULL_ANLZ


  #********************************************************************
  #
  #         Name: Analyzer_FullUI::Create
  #
  #   Parameters:
  #
  #  Description: 
  #
  #********************************************************************
  proc Create { tile_path } \
  {
    Debug::Puts AnalyzerUI 2 "Analyzer_FullUI::Create"

    variable LongName
    variable tile_class

    set top_level  [winfo toplevel  $tile_path]
   
    set tile_path  [name_child  $top_level  $LongName]
  
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  anlz  $tile_class]
      
      # set the check for the sweep menu
      $tile_path.mnubtn_sweep.menu config -postcommand "AnalyzerUI::Check_sweep_menu $tile_path.mnubtn_sweep.menu"
    }  
    return $tile_path
  }


  #**************************************************************************
  #
  #        Name: Analyzer_FullUI::Map
  #
  #  Parameters: tile_path  Analyzer widget path
  #
  # Description: This function is called when the full screen analyzer needs
  #              to be mapped on the screen.  It uses the generic setup of 
  #              the analyzer and then customizes the tile based on whether 
  #              it is a MAIN or CHANNEL (zoomed) analyzer.
  #
  #**************************************************************************
  proc Map { tile_path } \
  {
     Debug::Puts AnalyzerUI 2 "Analyzer_FullUI::Map:  $tile_path"

     # Handle generic setup of the analyzer
     ::AnalyzerUI::Map  $tile_path

     if { $::AnalyzerUI::StateUI(bw_type) == "MAIN" } \
     {
        #********************************************************************
        # Handle the setup for a main analyzer
        #********************************************************************

        # Configure the menus for the allowed options
        ::AnalyzerUI::rbw_active_menu  $tile_path  MAIN
        ::AnalyzerUI::span_active_menu  $tile_path  MAIN
        ::AnalyzerUI::sweep_active_menu  $tile_path  MAIN
        
        # Stop the Generator SWEEP mode and the Receiver SCAN mode
        GeneratorUI::gen_freq_sweep  "OFF"

        ReceiverUI::set_scan  "STOP"
     } \
     else \
     {
        #********************************************************************
        # Handle the setup for a channel analyzer
        #********************************************************************

        # Configure the menus for the allowed options
        ::AnalyzerUI::rbw_active_menu  $tile_path  CHANNEL
        ::AnalyzerUI::span_active_menu  $tile_path  CHANNEL
        ::AnalyzerUI::sweep_active_menu  $tile_path  CHANNEL

        # Set up the return key
        set widget  [winfo toplevel $tile_path]
        
        Debug::Puts AnalyzerUI 2 "::AnalyzerUI::prev_screen $::AnalyzerUI::prev_screen \
        #             ::Screens::CurrentNamespace $::Screens::CurrentNamespace"
        
        View::AssignReturn $tile_path "::Screens::Show $widget $::AnalyzerUI::prev_screen"
     }
     AnalyzerUI::update_mode_core $tile_path
  }


  #**************************************************************************
  #
  #        Name: Analyzer_FullUI::Unmap
  #
  #  Parameters: tile_path  - Analyzer widget path
  #
  # Description: This function is called when the quarter screen analyzer 
  #             needs to be removed from the screen.  
  #             We need to be in OFF mode at this point.
  #
  #**************************************************************************
  proc Unmap { tile_path } \
  {
    # disable RETURN softkey
    ::View::AssignReturn [winfo parent $tile_path] ""

    ::AnalyzerUI::Unmap  $tile_path
  }
  

  #********************************************************************
  #
  #        Name: Analyzer_FullUI::Destroy
  #
  #  Parameters: tile_path - Analyzer widget path
  #
  # Description: 
  #
  #********************************************************************
  proc Destroy { tile_path } \
  {
    ::AnalyzerUI::Destroy  $tile_path
  }

}




namespace eval View \
{
   lappend PreCreate Analyzer_FullUI
}


namespace eval Analyzer_SplitUI \
{
  # Global variables
  variable LongName       "splitanlz"
  variable ShortName      "Spectrum Analyzer"
  variable Size           Half_v

  variable tile_path      ""
  variable tile_class     SPLIT_ANLZ


  #********************************************************************
  #
  #         Name: Analyzer_SplitUI::Create
  #
  #   Parameters:
  #
  #  Description: 
  #
  #********************************************************************
  proc Create { tile_path } \
  {
    Debug::Puts AnalyzerUI 2 "Analyzer_SplitUI::Create"
    
    variable LongName
    
    set LongName  $::Analyzer_SplitUI::LongName
    
    variable tile_class

    set tile_class  $::Analyzer_SplitUI::tile_class


    set top_level [winfo toplevel $tile_path]

    set tile_path [name_child $top_level $LongName]

    Debug::Puts AnalyzerUI 2 "tile_path:  $tile_path"
    Debug::Puts AnalyzerUI 2 "tile_class:  $tile_class"
    
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  anlz  $tile_class]
    }

    return $tile_path
  }
  

  #**************************************************************************
  #
  #        Name: Analyzer_SplitUI::Unmap
  #
  #  Parameters: tile_path  - Analyzer widget path
  #
  # Description: This function is called when the quarter screen analyzer 
  #              needs to be removed from the screen.
  #
  #**************************************************************************
  proc Unmap { tile_path } \
  {
    ::AnalyzerUI::Unmap  $tile_path
    
    Analyzer run 3 -nowait
  }
  

  #********************************************************************
  #
  #        Name: Analyzer_SplitUI::Destroy
  #
  #  Parameters: tile_path - Analyzer widget path
  #
  # Description: 
  #
  #********************************************************************
  proc Destroy { tile_path } \
  {
    ::AnalyzerUI::Destroy $tile_path
  }  

}


namespace eval Analyzer_QtrUI \
{
  # Global variables
  variable LongName       "qtranlz"
  variable ShortName      "Spectrum Analyzer"
  variable Size           Quarter

  variable tile_path      ""
  variable tile_class     QUARTER_ANLZ


  #********************************************************************
  #
  #         Name: Analyzer_QtrUI::Create
  #
  #   Parameters:
  #
  #  Description: 
  #
  #********************************************************************
  proc Create { tile_path } \
  {
    Debug::Puts AnalyzerUI 2 "Analyzer_QtrUI::Create"
    
    variable LongName
    
    set LongName  $::Analyzer_QtrUI::LongName
    
    variable tile_class

    set tile_class  $::Analyzer_QtrUI::tile_class

    set top_level [winfo toplevel $tile_path]

    set tile_path [name_child $top_level $LongName]
 
    Debug::Puts AnalyzerUI 2 "tile_path:  $tile_path"
    Debug::Puts AnalyzerUI 2 "tile_class:  $tile_class"
    
    if { ![winfo exists $tile_path] } \
    {
      set tile_path  [Screens::create_tile  $tile_path  anlz  $tile_class]
    }
    
    return $tile_path
  }


  #**************************************************************************
  #
  #        Name: Analyzer_QtrUI::Map
  #
  #  Parameters: tile_path  Analyzer widget path
  #
  # Description: This function is called when the quarter screen analyzer 
  #              needs to be mapped on the screen.  
  #              It uses the generic setup of the analyzer and then 
  #              customizes the tile for a CHANNEL analyzer.
  #
  #**************************************************************************
  proc Map { tile_path } \
  {
     Debug::Puts AnalyzerUI 2 "Analyzer_QtrUI::Map:  $tile_path"

     if { $::Analyzer::State(span_f1) > 5.0 } \
     {
        Analyzer span 5.0 -1.0 -nowait
     }

     set ::AnalyzerUI::StateUI(bw_type)  "CHANNEL"

     # Handle generic setup of the analyzer
     AnalyzerUI::Map  $tile_path

     # Configure the menus for the allowed options
     AnalyzerUI::span_active_menu  $tile_path  CHANNEL
  }


  #**************************************************************************
  #
  #        Name: Analyzer_QtrUI::Unmap
  #
  #  Parameters: tile_path  - Analyzer widget path
  #
  # Description: This function is called when the quarter screen analyzer 
  #              needs to be removed from the screen.
  #
  #**************************************************************************
  proc Unmap { tile_path } \
  {
    AnalyzerUI::Unmap  $tile_path

    # need this line when switching between Full and Ch Mode
    #  BUT when zooming in channel mode, don't touch
    if { $::AnalyzerUI::StateUI(bw_type) != "CHANNEL" } \
    {
      Analyzer run 3 -nowait
    }
  }
 
  
  #********************************************************************
  #
  #        Name: Analyzer_QtrUI::Destroy
  #
  #  Parameters: tile_path - Analyzer widget path
  #
  # Description: 
  #
  #********************************************************************
  proc Destroy { tile_path } \
  {
    ::AnalyzerUI::Destroy $tile_path
  }

}


namespace eval View \
{
   lappend PreCreate Analyzer_QtrUI
}




namespace eval AnalyzerUI \
{
  # Global variables
  variable  LongName       "anlz"
  variable  ShortName      "Spectrum Analyzer"
  variable  Size           ""

  variable  tile_path      ""
  variable  tile_class     ""
  variable  prev_screen    ""

  variable  StateUI
  
  set  StateUI(units)     "dBm"
  set  StateUI(mode)      "LIVE"
  set  StateUI(acp_mode)  "OFF"
  set  StateUI(compare)   "OFF"
  set  StateUI(bw_type)   "MAIN"
  
  # Trigger variables
  set  StateUI(rbw_menu)  { "300 Hz" "3 kHz" "30 kHz" "60 kHz" "300 kHz" "6 MHz" }
  
  foreach vbw $::Analyzer::State(vbw_list) \
  {
     if {$vbw == 0} \
     {
        lappend StateUI(vbw_menu) "NONE"
     } \
     elseif {$vbw < 1000} \
     {
        lappend StateUI(vbw_menu) "$vbw Hz"
     } \
     elseif {$vbw < 1000000 } \
     {
        lappend StateUI(vbw_menu) "[expr int($vbw/1000)] kHz"
     } \
     else \
     {
        lappend StateUI(vbw_menu) "[expr int($vbw/1000000)] MHz"
     }
  }
  foreach sweep $::Analyzer::State(sweep_list) \
  {
     if {$sweep < .001 } \
     {
        lappend StateUI(sweep_menu) "[expr int($sweep*1000000)] us"
     } \
     elseif {$sweep < 1 } \
     {
        lappend StateUI(sweep_menu) "[expr int($sweep*1000)] ms"
     } \
     else \
     {
        lappend StateUI(sweep_menu) "$sweep s"
     }
  }
   
  set  StateUI(scale_menu)  { "10"  "5"  "2" }
  
  #********************************************************************
  #
  # Name: AnalyzerUI::Check_sweep_menu
  #
  #  Parameters: menu to check
  #
  # Description: 
  # marks as invalid/valid the sweep speeds in the menu
  #********************************************************************
  proc Check_sweep_menu { menu } \
  {
     set range [Analyzer sweep -range]
     set min [lindex $range 0]
     set max [lindex $range 2]
     
     set end [$menu index end]
     for {set i 0} { $i < $end} { incr i} \
     {
         set value [parse_sweep_value [$menu entrycget $i -value]]
         if { ($value < $min) || ($value > $max)} \
         {
            $menu entryconfig $i -state disabled
         } \
         else \
         {
            $menu entryconfig $i -state normal
         }
     }
  }
  
  #********************************************************************
  #
  #        Name: AnalyzerUI::Create
  #
  #  Parameters:
  #
  # Description: 
  #
  #********************************************************************
  proc Create { tile_path } \
  {
    Debug::Puts AnalyzerUI 2 "AnalyzerUI::Create"
    Debug::Puts AnalyzerUI 2 "tile_path:  $tile_path"
    Debug::Puts AnalyzerUI 2 "tile_class:  $tile_class"


    return $tile_path
  }

 
  #******************************************************************** 
  #
  #        Name: AnalyzerUI::Show_normalizing
  #
  #  Parameters: tile on who's behave we should show the alert
  #
  # Description: Show the analyzer normalizing dialog
  #
  #********************************************************************
  proc Show_normalizing { tile_path args } \
  {
     set cmd "AnalyzerUI::Show_normalizing_core $tile_path"

     after cancel $cmd
     after idle $cmd
  }
  
  proc Show_normalizing_core { tile_path } \
  {
     set alert [name_child $tile_path normalizing]
     
     switch -- $::Analyzer::State(normalize_state) \
     {
       0  -
       8  { # Normalize is stopped - kill the alert
            destroy $alert
          
            ::View::RestoreSoftkeys $tile_path
          
            return
          }
         
       9  -
       14 -
       15 -
       16 -
       17 -
       10 { # User abort or cal error - cancel after a second...
            after 1000 "destroy $alert; ::View::RestoreSoftkeys $tile_path"
          
            return
          }
          
       default { # Normalize is running - create alert as needed
                 if [winfo exists $alert] { return }
                 
                 ::View::StoreSoftkeys $tile_path
                 ::Screens::create_toplevel_tile $alert ANLZ_NORMALIZE
               }
     }
     
     return
  }

  
  #******************************************************************** 
  #
  #        Name: AnalyzerUI::Map
  #
  #  Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Map { tile_path } \
  {
    Debug::Puts AnalyzerUI 2 "AnalyzerUI::Map:  $tile_path"

    variable StateUI

    set_scale       $tile_path
    set_mode        $tile_path
    set_start_stop  $tile_path  START
    set_compare     $tile_path

    trace variable ::Analyzer::State(enable)  w  \
                                         "AnalyzerUI::update_mode $tile_path"
                                         
    trace variable ::Analyzer::State(cal)  w  \
                                            "AnalyzerUI::show_cal $tile_path"

    if { [winfo class $tile_path] == "FULL_ANLZ" } \
    {
      set_softkeys $tile_path

      trace variable ::Analyzer::State(tgstate)  w  \
                                "AnalyzerUI::set_track_gen_on_off $tile_path"

      set_track_gen_on_off_core $tile_path
    }

    trace variable ::Analyzer::State(normalize_state)  w  \
                                    "AnalyzerUI::Show_normalizing $tile_path"

    trace variable ::Analyzer::State(span)  w  \
                                   "AnalyzerUI::update_zero_span  $tile_path"
    
    AnalyzerUI::update_zero_span_core  $tile_path
    
    
    # Markers
    foreach { num } $::Anlz_MarkersUI::marker_list \
    {
      trace variable ::Anlz_MarkersUI::StateUI(marker,$num,state)  w  \
                     "Anlz_MarkersUI::update_marker_on_off  $tile_path  $num"
                             
      trace variable ::Anlz_MarkersUI::StateUI(marker,$num,state_acp)  w  \
                     "Anlz_MarkersUI::update_marker_on_off  $tile_path  $num"

      trace variable ::Anlz_MarkersUI::StateUI(marker,$num,orient)  w  \
                     "Anlz_MarkersUI::update_marker_on_off  $tile_path  $num"

      # Update the Markers for every trace
      trace variable ::Anlz_MarkersUI::StateUI(array)  w  \
                   "Anlz_MarkersUI::update_marker_position  $tile_path  $num"

      ::Anlz_MarkersUI::update_marker_on_off_core  $tile_path  $num
    }
    
    AnalyzerUI::update_mode_core $tile_path
    show_cal_core $tile_path
  }

  #********************************************************************
  #
  #        Name: AnalyzerUI::Cleanup
  #
  #  Parameters: tile_path
  #
  # Description: 
  #
  #********************************************************************
  proc Cleanup { tile_path } \
  {
    Debug::Puts AnalyzerUI 2 "AnalyzerUI::Cleanup:  $tile_path"

    trace vdelete ::AnalyzerUI::StateUI(compare)  w  \
                                        "AnalyzerUI::compare_mode $tile_path"
                                        
    trace vdelete ::AnalyzerUI::StateUI(mode)  w  \
                                            "AnalyzerUI::set_mode $tile_path"
                                            
    trace vdelete ::Analyzer::State(cal)  w  \
                                            "AnalyzerUI::show_cal $tile_path"
                                            
    trace vdelete ::Analyzer::State(tgstate)  w  \
                                "AnalyzerUI::set_track_gen_on_off $tile_path"
                                
    trace vdelete ::Analyzer::State(normalize_state)  w  \
                                    "AnalyzerUI::Show_normalizing $tile_path"
                                    
    trace vdelete ::Analyzer::State(enable)  w  \
                                         "AnalyzerUI::update_mode $tile_path"
    
    trace vdelete ::Analyzer::State(span)  w  \
                                   "AnalyzerUI::update_zero_span  $tile_path"

    # Markers
    foreach { num } $::Anlz_MarkersUI::marker_list \
    {
      trace vdelete ::Anlz_MarkersUI::StateUI(marker,$num,state)  w  \
                     "Anlz_MarkersUI::update_marker_on_off  $tile_path  $num"
                             
      trace vdelete ::Anlz_MarkersUI::StateUI(marker,$num,state_acp)  w  \
                     "Anlz_MarkersUI::update_marker_on_off  $tile_path  $num"

      trace vdelete ::Anlz_MarkersUI::StateUI(marker,$num,orient)  w  \
                     "Anlz_MarkersUI::update_marker_on_off  $tile_path  $num"

      trace vdelete ::Anlz_MarkersUI::StateUI(array)  w  \
                   "Anlz_MarkersUI::update_marker_position  $tile_path  $num"
    }
  }
  
  
  #********************************************************************
  #
  #        Name: AnalyzerUI::Unmap
  #
  #  Parameters: tile_path
  #
  # Description: 
  #
  #********************************************************************
  proc Unmap_full { tile_path } \
  {
    Cleanup $tile_path
  }


  #********************************************************************
  #
  #        Name: AnalyzerUI::Unmap
  #
  #  Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Unmap { tile_path } \
  {
    Cleanup $tile_path
  }


  #********************************************************************
  #
  #        Name: AnalyzerUI::Destroy
  #
  #  Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Destroy { tile_path } \
  {
    Cleanup $tile_path
  }


  #********************************************************************
  #
  #        Name: AnalyzerUI::Register
  #
  #  Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc Register { tile_path } \
  {
  }

  
  #********************************************************************
  #
  #        Name: AnalyzerUI::show_cal
  #
  #  Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc show_cal { tile_path args } \
  {
     set cmd "AnalyzerUI::show_cal_core $tile_path"
     after cancel $cmd
     after idle $cmd
  }
  
  proc show_cal_core { tile_path } \
  {
     if {$::Analyzer::State(cal) == 1 } \
     {
        place forget $tile_path.uncal
     } \
     else \
     {
        ::Screens::position_widget $tile_path.uncal
     }
  }
  
  
  ################################################################  
  #
  #  User Interface support procedures
  #
  ################################################################

  #***************************************************************************
  #
  #        Name: AnalyzerUI::get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
     while { 1 } \
     {
       switch -- [winfo class $widget] \
       {
         "QUARTER_ANLZ"  -
         "FULL_ANLZ"     { return $widget}
       }
        
       set widget [winfo parent $widget]
     }
  }
  
  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_freq
  #
  #  Parameters: none
  #
  # Description: Calls Generator and Receiver engine to set the center 
  #              frequency for Analyzer
  #
  #***************************************************************************
  proc set_freq { } \
  {
    Receiver freq $::Receiver::State(freq_raw) MHz -nowait
  } 
  
  #***************************************************************************
  #
  #        Name: update_zero_span
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Updates display for the '0 SPAN' setting
  #
  #***************************************************************************
  proc update_zero_span { tile_path args } \
  {
    set cmd [namespace code "update_zero_span_core $tile_path"]
    after cancel $cmd
    after idle $cmd
  }
  proc update_zero_span_core { tile_path } \
  {
    Debug::Puts AnalyzerUI 2 "update_zero_span  $tile_path"
    
    set tile_path  [get_tile  $tile_path]
    
    # Display changes only apply to the Full Analyzer
    if { [winfo class $tile_path] != "FULL_ANLZ" } \
    {
      return
    }
    
   
    # Determine of the Analyzer is in 0 SPAN
    if { $::Analyzer::State(span) == "0 SPAN" } \
    {
      # Remove the Center Magnitude
      place forget  ${tile_path}.lbl_ctr
      place forget  ${tile_path}.lbl_value
      place forget  ${tile_path}.lbl_ctrunit

      # Turn OFF the Tracking Generator if it is ON
      if { $::Analyzer::State(tgstate) == 1 } \
      {
        Analyzer tgstate 0
      }

      # Set the analyzer mode to LIVE
      set ::AnalyzerUI::StateUI(mode)  "LIVE"
      
      # Display the trigger fields
      Screens::position_widget  ${tile_path}.lbl_trigger
      Screens::position_widget  ${tile_path}.mnu_trigger
      Screens::position_widget  ${tile_path}.lbl_trig_level
      Screens::position_widget  ${tile_path}.ent_trig_level
      Screens::position_widget  ${tile_path}.lbl_trig_level_units
      Screens::position_widget  ${tile_path}.lbl_trig_slope
      Screens::position_widget  ${tile_path}.btn_trig_slope
      Screens::position_widget  ${tile_path}.lbl_trig_hpos
      Screens::position_widget  ${tile_path}.ent_trig_hpos
      Screens::position_widget  ${tile_path}.lbl_trig_hpos_units
    } \
    else \
    {
     
      # Remove the trigger fields
      place forget  ${tile_path}.lbl_trigger
      place forget  ${tile_path}.mnu_trigger
      place forget  ${tile_path}.lbl_trig_level
      place forget  ${tile_path}.ent_trig_level
      place forget  ${tile_path}.lbl_trig_level_units
      place forget  ${tile_path}.lbl_trig_slope
      place forget  ${tile_path}.btn_trig_slope
      place forget  ${tile_path}.lbl_trig_hpos
      place forget  ${tile_path}.ent_trig_hpos
      place forget  ${tile_path}.lbl_trig_hpos_units
    
      # Display the Center Magnitude fields
      Screens::position_widget  ${tile_path}.lbl_ctr
      Screens::position_widget  ${tile_path}.lbl_value
      Screens::position_widget  ${tile_path}.lbl_ctrunit
    }
  }


  #############################################################################
  # 
  #  Trigger Support procedures
  # 
  #############################################################################
  #***************************************************************************
  #
  #        Name: set_trig_mode
  #
  #  Parameters:
  #
  # Description: Sets the trigger mode of the Analyzer
  #
  #***************************************************************************
  proc set_trig_mode { } \
  {
    Debug::Puts AnalyzerUI 2 "set_trig_mode"
    
    variable  StateUI
    
    switch -- $::Analyzer::State(trig_mode_text) \
    {
      "AUTO"      { Analyzer  trigmode  0 }
      
      "NORMAL"    { Analyzer  trigmode  1 }
      
      "ONE SHOT"  { Analyzer  trigmode  2 }
    }
  }


  #***************************************************************************
  #
  #        Name: set_trig_level
  #
  #  Parameters:
  #
  # Description: Sets the trigger level of the Analyzer
  #
  #***************************************************************************
  proc set_trig_level { } \
  {
    Debug::Puts AnalyzerUI 2 "set_trig_level"
    
    variable  StateUI
    
    Analyzer  triglevel  $::Analyzer::State(trig_level)  -nowait
  }
    
  
  #***************************************************************************
  #
  #        Name: toggle_trig_slope
  #
  #  Parameters:
  #
  # Description: Toggles the trigger slope of the Analyzer between RISING
  #              and FALLING
  #
  #***************************************************************************
  proc toggle_trig_slope { } \
  {
    Debug::Puts AnalyzerUI 2 "toggle_trig_slope"
    
    variable  StateUI

    if { $::Analyzer::State(trig_slope_text) == "RISING" } \
    {
      # Set trigger slope to falling
      Analyzer  trigedge  2 -nowait
    } \
    else \
    {
      # Set trigger slope to rising
      Analyzer  trigedge  1 -nowait
    }
  }


  #***************************************************************************
  #
  #        Name: set_trig_hpos
  #
  #  Parameters:
  #
  # Description: Sets the trigger horizontal position of the Analyzer
  #
  #***************************************************************************
  proc set_trig_hpos { } \
  {
    Debug::Puts AnalyzerUI 2 "set_trig_hpos"
    
    variable  StateUI
    
    Analyzer  trigdepth  $::Analyzer::State(trig_hpos)  -nowait
  }
    
  

  #***************************************************************************
  #
  #        Name: AnalyzerUI::rbw_active_menu
  #
  #  Parameters:  tile_path  Tile path information
  #               tile_type  MAIN | CHANNEL
  #
  # Description: The channel analyzer does not allow some of the frequencies
  #              which are available on the full analyzer.  This function 
  #              disables the menu so that they cannot be selected
  #
  #***************************************************************************
  proc rbw_active_menu { tile_path  tile_type } \
  {
     # Obtain the current menu
     set tile_path  [get_tile $tile_path]

     set menu  ${tile_path}.mnubtn_rbw.menu

     # Disable some of the options if in the Channel (zoomed) analyzer 
     if { $tile_type == "MAIN" } \
     {
        $menu entryconfig "300 Hz" -state normal
        $menu entryconfig "3 kHz" -state normal
        $menu entryconfig "30 kHz" -state normal
        $menu entryconfig "60 kHz" -state normal
        $menu entryconfig "300 kHz" -state normal
        $menu entryconfig "6 MHz" -state normal
     } \
     else \
     {
        $menu entryconfig "300 Hz" -state normal
        $menu entryconfig "3 kHz" -state normal
        $menu entryconfig "30 kHz" -state disabled
        $menu entryconfig "60 kHz" -state normal
        $menu entryconfig "300 kHz" -state disabled
        $menu entryconfig "6 MHz" -state disabled
     }
  }
  
  #***************************************************************************
  #
  #        Name: AnalyzerUI::sweep_active_menu
  #
  #  Parameters:  tile_path  Tile path information
  #               tile_type  MAIN | CHANNEL
  #
  # Description: The channel analyzer does not allow some of the frequencies
  #              which are available on the full analyzer.  This function 
  #              disables the menu so that they cannot be selected
  #
  #***************************************************************************
  proc sweep_active_menu { tile_path  tile_type } \
  {
     return

  }
  
  
  #***************************************************************************
  #
  #        Name: AnalyzerUI::span_active_menu
  #
  #  Parameters:  tile_path  Tile path information
  #               tile_type  MAIN | CHANNEL
  #
  # Description: The channel analyzer does not allow some of the spans
  #              which are available on the full analyzer.  This function 
  #              disables the menu so that they cannot be selected
  #
  #***************************************************************************
  proc span_active_menu { tile_path tile_type } \
  {
     # Obtain the current menu
     set tile_path  [get_tile $tile_path]
     
     set menu  ${tile_path}.mnubtn_span.menu

     # Enable the span values common between the Main and Channel analyzers
     $menu entryconfig "0 SPAN" -state normal
     $menu entryconfig "1 kHz" -state normal
     $menu entryconfig "2 kHz" -state normal
     $menu entryconfig "5 kHz" -state normal
     $menu entryconfig "10 kHz" -state normal
     $menu entryconfig "20 kHz" -state normal
     $menu entryconfig "50 kHz" -state normal
     $menu entryconfig "100 kHz" -state normal
     $menu entryconfig "200 kHz" -state normal
     $menu entryconfig "500 kHz" -state normal
     $menu entryconfig "1 MHz" -state normal
     $menu entryconfig "2 MHz" -state normal
     $menu entryconfig "5 MHz" -state normal

     # Disable some of the options if in the Channel (zoomed) analyzer 
     if { $tile_type == "MAIN" } \
     {
        $menu entryconfig "10 MHz" -state normal
        $menu entryconfig "20 MHz" -state normal
        $menu entryconfig "50 MHz" -state normal
        $menu entryconfig "100 MHz" -state normal
        $menu entryconfig "200 MHz" -state normal
        $menu entryconfig "500 MHz" -state normal
        $menu entryconfig "1 GHz" -state normal
        $menu entryconfig "2 GHz" -state normal
        $menu entryconfig "3 GHz" -state normal
     } \
     else \
     {
        $menu entryconfig "10 MHz" -state disabled
        $menu entryconfig "20 MHz" -state disabled
        $menu entryconfig "50 MHz" -state disabled
        $menu entryconfig "100 MHz" -state disabled
        $menu entryconfig "200 MHz" -state disabled
        $menu entryconfig "500 MHz" -state disabled
        $menu entryconfig "1 GHz" -state disabled
        $menu entryconfig "2 GHz" -state disabled
        $menu entryconfig "3 GHz" -state disabled
     }
  }
    
    
  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_rbw
  #
  #  Parameters: n/a
  #
  # Description: provides UI selection to engine rbw translation
  #
  #***************************************************************************
  proc set_rbw {  } \
  {
     switch -- $::Analyzer::State(rbw) \
     {
        "300 Hz"  { Analyzer rbw 300 -nowait }
        "3 kHz"   { Analyzer rbw 3000 -nowait }
        "60 kHz"  { Analyzer rbw 60000 -nowait}
        "30 kHz"  { Analyzer rbw 30000 -nowait}
        "300 kHz" { Analyzer rbw 300000 -nowait}
        "6 MHz"   { Analyzer rbw 6000000 -nowait}
     }
  }
  
  
  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_rbw_auto
  #
  #  Parameters: span_string
  #
  # Description: Set recommended rbw for given span
  #
  #***************************************************************************
  proc set_rbw_auto { } \
  {
    variable StateUI

    set rbw 300

    switch $::Analyzer::State(span) \
    {
      "0 SPAN"   { set rbw 60000 }

      "1 kHz"   -
      "2 kHz"   -
      "5 kHz"   -
      "10 kHz"  -
      "20 kHz"  { set rbw 300 }

      "50 kHz"  -
      "100 kHz" -
      "200 kHz" { set rbw 3000 }

      "500 kHz" -
      "1 MHz"   { set rbw 30000 }

      "2 MHz"   -
      "5 MHz"   { set rbw 60000 }

      "10 MHz"  -
      "20 MHz"  -
      "50 MHz"  { set rbw 300000 }

      "100 MHz" -
      "200 MHz" -
      "500 MHz" -
      "1 GHz"   -
      "2 GHz"   -
      "3 GHz"   { set rbw 6000000 }
    }

    # exceptions for channel analyzer, some rbw's aren't available
    if { $StateUI(bw_type) == "CHANNEL" } \
    {
      switch $::Analyzer::State(span) \
      {
        "500 kHz" { set rbw 3000 }
        "1 MHz"   { set rbw 60000 }
      }
    }

    Analyzer rbw $rbw -nowait
  }


  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_vbw
  #
  #  Parameters: new_value -
  #
  # Description: sets Analyzer video bandwidth
  #
  #***************************************************************************
  proc set_vbw {  } \
  {
    
    switch $::Analyzer::State(vbw) \
    {
      "10 Hz"   { set vbw  10      }
      "30 Hz"   { set vbw  30      }
      "100 Hz"  { set vbw  100     }
      "300 Hz"  { set vbw  300     }
      "1 kHz"   { set vbw  1000    }
      "3 kHz"   { set vbw  3000    }
      "10 kHz"  { set vbw  10000   }
      "30 kHz"  { set vbw  30000   }
      "100 kHz" { set vbw  100000  }
      "300 kHz" { set vbw  300000  }
      "1 MHz"   { set vbw  1000000 }
      "3 MHz"   { set vbw  3000000 }
      "NONE"    { set vbw  0       }
      
      default   { set vbw 0 }
    }   
  
    Analyzer vbw $vbw -nowait
  }


  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_vbw_active
  #
  #  Parameters: tile_path
  #
  # Description:
  #
  #***************************************************************************
  proc set_vbw_active { tile_path } \
  {
    Debug::Puts AnalyzerUI 2 "::set_vbw_active:  $span_string"

    set span_string ::Analyzer::State(span)
    
    set menu  ${tile_path}.mnubtn_vbw.menu

    # Disable all menu items
    set max_index  [$menu index end]
    
    for { set i 0 } { $i <= $max_index } { incr i 1 } \
    {
      $menu entryconfig $i -state disabled
    }
    
    # Enable selected menu items    
    switch $span_string \
    {
      "0 SPAN"   -
      "1 kHz"    -
      "2 kHz"    -
      "5 kHz"    { $menu entryconfig "300 Hz" -state normal
                   $menu entryconfig "1 kHz" -state normal
                   $menu entryconfig "3 kHz" -state normal
                   $menu entryconfig "10 kHz" -state normal
                   $menu entryconfig "30 kHz" -state normal
                   $menu entryconfig "100 kHz" -state normal
                   $menu entryconfig "300 kHz" -state normal
                   $menu entryconfig "1 MHz" -state normal
                   $menu entryconfig "3 MHz" -state normal
                   $menu entryconfig "NONE" -state normal
                 }
                 
      "10 kHz"   -
      "20 kHz"   -
      "50 kHz"   { $menu entryconfig "1 kHz" -state normal
                   $menu entryconfig "3 kHz" -state normal
                   $menu entryconfig "10 kHz" -state normal
                   $menu entryconfig "30 kHz" -state normal
                   $menu entryconfig "100 kHz" -state normal
                   $menu entryconfig "300 kHz" -state normal
                   $menu entryconfig "1 MHz" -state normal
                   $menu entryconfig "3 MHz" -state normal
                   $menu entryconfig "NONE" -state normal
                 }
                 
      "100 kHz"  -
      "200 kHz"  { $menu entryconfig "3 kHz" -state normal
                   $menu entryconfig "10 kHz" -state normal
                   $menu entryconfig "30 kHz" -state normal
                   $menu entryconfig "100 kHz" -state normal
                   $menu entryconfig "300 kHz" -state normal
                   $menu entryconfig "1 MHz" -state normal
                   $menu entryconfig "3 MHz" -state normal
                   $menu entryconfig "NONE" -state normal
                 }
                 
      "500 kHz"  -
      "1 MHz"    { $menu entryconfig "10 kHz" -state normal
                   $menu entryconfig "30 kHz" -state normal
                   $menu entryconfig "100 kHz" -state normal
                   $menu entryconfig "300 kHz" -state normal
                   $menu entryconfig "1 MHz" -state normal
                   $menu entryconfig "3 MHz" -state normal
                   $menu entryconfig "NONE" -state normal
                 }
                 
      "2 MHz"    -
      "5 MHz"    { $menu entryconfig "30 kHz" -state normal
                   $menu entryconfig "100 kHz" -state normal
                   $menu entryconfig "300 kHz" -state normal
                   $menu entryconfig "1 MHz" -state normal
                   $menu entryconfig "3 MHz" -state normal
                   $menu entryconfig "NONE" -state normal
                 }
                 
      "10 MHz"   { $menu entryconfig "100 kHz" -state normal
                   $menu entryconfig "300 kHz" -state normal
                   $menu entryconfig "1 MHz" -state normal
                   $menu entryconfig "3 MHz" -state normal
                   $menu entryconfig "NONE" -state normal
                 }
                 
      "20 MHz"   -
      "50 MHz"   { $menu entryconfig "300 kHz" -state normal
                   $menu entryconfig "1 MHz" -state normal
                   $menu entryconfig "3 MHz" -state normal
                   $menu entryconfig "NONE" -state normal
                 }
                 
      "100 MHz"  -
      "200 MHz"  -
      "500 MHz"  -
      "1 GHz"    -
      "2 GHz"    -
      "3 GHz"    { $menu entryconfig "1 MHz" -state normal
                   $menu entryconfig "3 MHz" -state normal
                   $menu entryconfig "NONE" -state normal
                 }
  
      default    { puts "::set_vbw_active err: $span_string"; return }
    }
  
    # Determine the calibration state
    set vbw_string  $::Analyzer::State(vbw)
    
    set vbw_state  [$menu entrycget ${vbw_string}* -state]
    
    if { $vbw_state == "disabled" } \
    {
      ${tile_path}.mnubtn_vbw config -foreground yellow
      ${tile_path}.mnubtn_vbw config -activeforeground yellow
      
      #set ::AnalyzerUI::StateUI(vbw_calstate)  "UNCAL"
    }
    
    if { $vbw_state == "normal" || $vbw_state == "active" } \
    {
      ${tile_path}.mnubtn_vbw config -foreground black
      ${tile_path}.mnubtn_vbw config -activeforeground black

      #set ::AnalyzerUI::StateUI(vbw_calstate)  "CAL"
    }
  }
  
  
  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_vbw_auto
  #
  #  Parameters: 
  #
  # Description: Set recommended vbw for given span
  #
  #***************************************************************************
  proc set_vbw_auto {  } \
  {
    switch $::Analyzer::State(span) \
    {
      "0 SPAN"   { Analyzer vbw 10000 -nowait }

      "1 kHz"   -
      "2 kHz"   -
      "5 kHz"   { Analyzer vbw 300 -nowait }
      
      "10 kHz"  -
      "20 kHz"  -
      "50 kHz"  { Analyzer vbw 1000 -nowait }
      
      "100 kHz" -
      "200 kHz" { Analyzer vbw 3000 -nowait }
      
      "500 kHz" -
      "1 MHz"   { Analyzer vbw 10000 -nowait }
      
      "2 MHz"   -
      "5 MHz"   { Analyzer vbw 30000 -nowait }
      
      "10 MHz"  { Analyzer vbw 100000 -nowait }
      
      "20 MHz"  -
      "50 MHz"  { Analyzer vbw 300000 -nowait }
      
      "100 MHz" -
      "200 MHz" -
      "500 MHz" -
      "1 GHz"   -
      "2 GHz"   -
      "3 GHz"   { Analyzer vbw 1000000 -nowait }
      
      default   {  }
    }
  }


  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_sweep
  #
  #  Parameters: new_value - 
  #
  # Description: sets Analyzer sweep time
  #
  #***************************************************************************
  proc parse_sweep_value { value  } \
  {
    
    regexp {([0-9]*) (.*)} $value foo value units
    
    switch -- [string tolower $units] \
    {
       "us" { }
       "ms" { set value [expr $value * 1000] }
       "s"  { set value [expr $value * 1000000] }
    }
    return $value
  }
  proc set_sweep {   } \
  {
    set value [parse_sweep_value $::Analyzer::State(sweep) ]
    Analyzer sweep $value $::Analyzer::State(sweep_type)  -nowait
  }


  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_sweep_active
  #
  #  Parameters: span_string
  #
  # Description:
  #
  #***************************************************************************
  proc set_sweep_active { tile_path } \
  {
    return
    # DDH disabled for now. Need to think about this....
    
    set span_string $::Analyzer:State(span)
    
    Debug::Puts AnalyzerUI 2 "::set_sweep_active:  $span_string"
    
    set menu  ${tile_path}.mnubtn_sweep.menu

    # Disable all menu items
    set max_index  [$menu index end]
    
    for { set i 0 } { $i <= $max_index } { incr i 1 } \
    {
      $menu entryconfig $i -state disabled
    }
    
    # Enable selected menu items    
    switch $span_string \
    {
      "0 SPAN"  -
      "1 kHz"   -
      "2 kHz"   -
      "5 kHz"   -
      "10 kHz"  -
      "20 kHz"  -
      "50 kHz"  -
      "100 kHz" -
      "200 kHz" { $menu entryconfig "1 s" -state normal
                  $menu entryconfig "2 s" -state normal
                  $menu entryconfig "3 s" -state normal
                  $menu entryconfig "4 s" -state normal
                  $menu entryconfig "5 s" -state normal
                  $menu entryconfig "6 s" -state normal }
      
      "500 kHz" -
      "1 MHz"   { $menu entryconfig "200 ms" -state normal
                  $menu entryconfig "500 ms" -state normal
                  $menu entryconfig "750 ms" -state normal
                  $menu entryconfig "1 s" -state normal
                  $menu entryconfig "2 s" -state normal
                  $menu entryconfig "3 s" -state normal
                  $menu entryconfig "4 s" -state normal
                  $menu entryconfig "5 s" -state normal
                  $menu entryconfig "6 s" -state normal }
      
      "2 MHz"   { $menu entryconfig "500 ms" -state normal
                  $menu entryconfig "750 ms" -state normal
                  $menu entryconfig "1 s" -state normal
                  $menu entryconfig "2 s" -state normal
                  $menu entryconfig "3 s" -state normal
                  $menu entryconfig "4 s" -state normal
                  $menu entryconfig "5 s" -state normal
                  $menu entryconfig "6 s" -state normal }
      
      "5 MHz"   -
      "10 MHz"  { $menu entryconfig "750 ms" -state normal
                  $menu entryconfig "1 s" -state normal
                  $menu entryconfig "2 s" -state normal
                  $menu entryconfig "3 s" -state normal
                  $menu entryconfig "4 s" -state normal
                  $menu entryconfig "5 s" -state normal
                  $menu entryconfig "6 s" -state normal }
      
      "20 MHz"  { $menu entryconfig "750 ms" -state normal
                  $menu entryconfig "1 s" -state normal
                  $menu entryconfig "2 s" -state normal
                  $menu entryconfig "3 s" -state normal
                  $menu entryconfig "4 s" -state normal
                  $menu entryconfig "5 s" -state normal
                  $menu entryconfig "6 s" -state normal }
      
      "50 MHz"  -
      "100 MHz" { $menu entryconfig "500 ms" -state normal
                  $menu entryconfig "750 ms" -state normal
                  $menu entryconfig "1 s" -state normal
                  $menu entryconfig "2 s" -state normal
                  $menu entryconfig "3 s" -state normal
                  $menu entryconfig "4 s" -state normal
                  $menu entryconfig "5 s" -state normal
                  $menu entryconfig "6 s" -state normal }
      
      "200 MHz" -
      "500 MHz" { $menu entryconfig "1 s" -state normal
                  $menu entryconfig "2 s" -state normal
                  $menu entryconfig "3 s" -state normal
                  $menu entryconfig "4 s" -state normal
                  $menu entryconfig "5 s" -state normal
                  $menu entryconfig "6 s" -state normal }
      
      "1 GHz"   { $menu entryconfig "2 s" -state normal
                  $menu entryconfig "3 s" -state normal
                  $menu entryconfig "4 s" -state normal
                  $menu entryconfig "5 s" -state normal
                  $menu entryconfig "6 s" -state normal }
      
      "2 GHz"   -
      "3 GHz"   { $menu entryconfig "3 s" -state normal
                  $menu entryconfig "4 s" -state normal
                  $menu entryconfig "5 s" -state normal
                  $menu entryconfig "6 s" -state normal }
      

      default   { puts "::set_sweep_active err: $span_string"; return } 
    }
  
    # Determine the calibration state
    set sweep_string  $::Analyzer::State(sweep)
    
    set sweep_state  [$menu entrycget ${sweep_string}* -state]
    
    if { $sweep_state == "disabled" } \
    {
      ${tile_path}.mnubtn_sweep config -foreground yellow
      ${tile_path}.mnubtn_sweep config -activeforeground yellow
      
      #set ::AnalyzerUI::StateUI(sweep_calstate)  "UNCAL"
    }
    
    if { $sweep_state == "normal" || $sweep_state == "active" } \
    {
      ${tile_path}.mnubtn_sweep config -foreground black
      ${tile_path}.mnubtn_sweep config -activeforeground black

      #set ::AnalyzerUI::StateUI(sweep_calstate)  "CAL"
    }
  }

  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_tos
  #
  #  Parameters:  none
  #
  # Description: sets Analyzer top of screen with state value
  #
  #***************************************************************************
  proc set_tos { } \
  {
    after idle "Analyzer topofscreen $::Analyzer::State(tos) -nowait"
  }


  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_scale
  #
  #  Parameters: tile_path
  #              new_value  - 2 dB or 5 dB
  #
  # Description: sets vertical scale (dB/div), default is 10 dB
  #
  #***************************************************************************
  proc set_scale { tile_path { new_value "" } } \
  {
    Debug::Puts AnalyzerUI 2 "::set_scale:  $new_value"

    if { "$new_value" == "" } \
    {
      set scale  $::Analyzer::State(scale)
    } \
    else \
    {
      set scale  $new_value
    }

    if { $scale == "" } \
    {
      set scale  10
    }

    Analyzer scale $scale -nowait
  }


  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_mode
  #
  #  Parameters: tile_path
  #              args
  #
  # Description: This function will configure the hardware as described by
  #              option menu.  LIVE, AVG, PEAK are independent functions -
  #              compare and TG are modifiers that may (or may not) be used 
  #              with LIVE, AVG, PEAK.
  #              This procedure also sets a psuedo-mode, ACP, which sets the
  #              Analyzer to the LIVE mode then tracks the measurement with
  #              the Markers.
  #
  #***************************************************************************
  proc set_mode { tile_path  args } \
  {
    set cmd  "AnalyzerUI::set_mode_core $tile_path"
  
    after cancel $cmd
    after idle $cmd
  }

  proc set_mode_core { tile_path } \
  {
    variable StateUI

    if { [winfo class $tile_path] == "FULL_ANLZ" } \
    {
      place forget  ${tile_path}.lbl_avg
      place forget  ${tile_path}.ent_avg
    }

    # Clear SOFTKEY 6:
    View::AssignSoftkey $tile_path  6  ""  ""

    if { $::Analyzer::State(enable) & 0x08 } \
    {
      if { [winfo class $tile_path] == "FULL_ANLZ" } \
      {
        # SOFTKEY 6: CLEAR PEAK: Clear Peak Hold Trace
        View::AssignSoftkey  $tile_path  6  "CLEAR\nTRACE"  \
                   "Analyzer::Toggle PEAK; after 750 {Analyzer::Toggle PEAK}"
      }
    }
    
    if { $::Analyzer::State(enable) & 0x10 } \
    {
      # SOFTKEY 6: CLEAR AVG: Clear Average Trace
      View::AssignSoftkey  $tile_path  6  "CLEAR\nTRACE"  \
             "Analyzer::Toggle AVERAGE; after 750 {Analyzer::Toggle AVERAGE}"

      if { [winfo class $tile_path] == "FULL_ANLZ" } \
      {
        Screens::position_widget  ${tile_path}.lbl_avg
        Screens::position_widget  ${tile_path}.ent_avg
      }
    }
  }
  
  proc update_mode { tile_path  args } \
  {
    set cmd  [namespace code "update_mode_core $tile_path"]
  
    after cancel $cmd
    after idle $cmd
  }
  
  proc update_mode_core { tile_path } \
  {
    update_traces $tile_path
  
    set_compare_core $tile_path
  }
  
  proc update_traces { tile_path } \
  {
    variable StateUI

    if { $::Analyzer::State(enable) & 0x02 } \
    {
      ${tile_path}.anlz.live  config  -active 1

      set StateUI(live)  1
    } \
    else \
    {
      ${tile_path}.anlz.live  config  -active 0
    }

    if { $::Analyzer::State(enable) & 0x08 } \
    {
      ${tile_path}.anlz.peak  config  -active 1
    } \
    else \
    {
      ${tile_path}.anlz.peak  config  -active 0
    }

    if { $::Analyzer::State(enable) & 0x10 } \
    {
      ${tile_path}.anlz.average  config  -active 1
    } \
    else \
    {
      ${tile_path}.anlz.average  config  -active 0
    }

    if { $::Analyzer::State(enable) & 0x80 } \
    {
      ${tile_path}.anlz.compare  config  -active 1
    } \
    else \
    {
      ${tile_path}.anlz.compare  config  -active 0
    }
  }

  trace variable ::Analyzer::State(enable) w [namespace code update_modevars]

  proc update_modevars { args } \
  {
    set cmd [namespace code update_modevars_core]
  
    after cancel $cmd
    after idle $cmd
  }
  
  proc update_modevars_core { } \
  {
    variable StateUI
    
    if { $::Analyzer::State(enable) & 0x02 } \
    {
      set StateUI(live)  1
    } \
    else \
    {
      set StateUI(live)  0
    }
    if { $::Analyzer::State(enable) & 0x08 } \
    {
      set StateUI(peak)  1
    } \
    else \
    {
      set StateUI(peak)  0
    }
    if { $::Analyzer::State(enable) & 0x10 } \
    {
      set StateUI(avg)  1
    } \
    else \
    {
      set StateUI(avg)  0
    }
    if { $::Analyzer::State(enable) & 0x80 } \
    {
      set StateUI(compare)  1
    } \
    else \
    {
      set StateUI(compare)  0
    }
  }
  
  update_modevars_core
  
  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_track_gen_on_off
  #
  #  Parameters: 
  #
  # Description: sets tracking gen level 
  #
  #***************************************************************************
  proc set_track_gen_on_off { tile_path  args } \
  {
    set cmd "AnalyzerUI::set_track_gen_on_off_core $tile_path"
     
    after cancel $cmd
    after idle $cmd
  }

  proc set_track_gen_on_off_core { tile_path } \
  {
    variable StateUI

    if { $StateUI(bw_type) == "CHANNEL" } \
    {
      place forget  ${tile_path}.lbl_rflvl
      place forget  ${tile_path}.ent_rflvl
      place forget  ${tile_path}.lbl_rflvl_units
     
      return
    }

    if { $::Analyzer::State(tgstate) == 1} \
    {
      Screens::position_widget  ${tile_path}.lbl_rflvl
      Screens::position_widget  ${tile_path}.ent_rflvl
      Screens::position_widget  ${tile_path}.lbl_rflvl_units
    
      ::DualSkey::AssignGenOutput $tile_path 4 
    } \
    else \
    {
      place forget  ${tile_path}.lbl_rflvl
      place forget  ${tile_path}.ent_rflvl
      place forget  ${tile_path}.lbl_rflvl_units
    
      View::AssignSoftkey $tile_path  4  ""  ""
    }
  }

  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_track_gen_level
  #
  #  Parameters: new_value
  #
  # Description: sets tracking gen level 
  #
  #***************************************************************************
  proc set_track_gen_level { } \
  {
    set cmd "AnalyzerUI::set_track_gen_level_core"

    after cancel $cmd
    after idle $cmd
  }
  
  proc set_track_gen_level_core { } \
  {
    Generator rflevel $::Generator::State(rflev,dBm) -nowait
  }

  
  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_compare
  #
  #  Parameters: tile_path
  #             args
  #
  # Description:  StateUI(compare) describes if we are in compare mode or
  #               some sort of live mode.  We can save traces in live mode
  #               and we can recall traces for compare in compare mode.
  #
  #               StateUI(compare)   ON     compare mode is on
  #               StateUI(compare)   OFF    compare mode is off
  #
  #***************************************************************************
  proc set_compare { tile_path args } \
  {
    set cmd "AnalyzerUI::set_compare_core $tile_path"

    after cancel $cmd
    after idle $cmd
  }

  proc set_compare_core { tile_path } \
  {
    Debug::Puts AnalyzerUI 2 "::set_compare:"

    variable StateUI

    set key "SAVE TRACE"
    set cmd {Analyzer save 1 1 -nowait ; Analyzer save 0 3 -nowait}

    if { [winfo class $tile_path] == "FULL_ANLZ" } \
    {
      View::AssignSoftkey $tile_path  5  $key $cmd
    }

    set_mode_core $tile_path
  }


  #***************************************************************************
  #
  #        Name: AnalyzerUI::toggle_start_stop
  #
  #  Parameters: new_value
  #
  # Description:
  #
  #***************************************************************************
  proc toggle_start_stop { tile_path } \
  {
    if { $::Analyzer::State(run) == "STOP" } \
    {
      set_start_stop $tile_path  "START"
    } \
    else \
    {
      set_start_stop $tile_path  "STOP"
    }
  }


  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_start_stop
  #
  #  Parameters: new_value
  #
  # Description:
  #
  #***************************************************************************
  proc set_start_stop { tile_path  { start_stop "" } } \
  {
    Debug::Puts AnalyzerUI 2 "::set_start_stop: $tile_path $start_stop"

    variable StateUI
    
    if { $start_stop == "START" } \
    {
      if { [winfo class $tile_path] == "QUARTER_ANLZ" } \
      {
        # Start Channel Analyzer
        Analyzer run 1 -nowait
      }

      if { [winfo class $tile_path] == "FULL_ANLZ" } \
      {
        if { $StateUI(bw_type) == "MAIN" } \
        {
          # Start Full Analyzer
          Analyzer run 2 -nowait
        } \
        else \
        {
          # Start Channel Analyzer
          Analyzer run 1 -nowait
        }
      }
    }

    if { $start_stop == "STOP" } \
    {
      # Command the analyzer to stop
      Analyzer run 3 -nowait
    }
  }



  ################################################################
  #
  #             Softkey Support
  #
  ################################################################

  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_softkeys
  #
  #  Parameters: type        FULL, CHANNEL
  #              tile_path   
  #
  # Description: assigns Analyzer softkeys
  #
  #***************************************************************************
  proc set_softkeys { tile_path } \
  {
    Debug::Puts AnalyzerUI 2 "AnalyzerUI::set_softkeys:  $tile_path"

    variable StateUI

    set tile_path  [get_tile $tile_path]

    
    # SOFTKEY 1 ( HELP )


    # SOFTKEY 2 ( Trace: Start, Stop )
    ::DualSkey::AssignTraceRun $tile_path 2 

    # SOFTKEY 3 ( Input: ANT, T/R )
    ::DualSkey::AssignRcvInput $tile_path 3 

    # SOFTKEY 4 ()
    # This is set up based on the Tracking Generator mode
  
    # SOFTKEY 5 ()
    set_compare $tile_path
  
    # SOFTKEY 6 ()
    View::AssignSoftkey $tile_path  6  ""  ""
   
    # SOFTKEY 7 ( Normalize )
    View::AssignSoftkey $tile_path  7  "NORMAL"  "Analyzer normalize 1"
 
     
    # SOFTKEY 8 ( RETURN )
  }

  ################################################################  
  #
  #  Menu Dropdown Interface Support
  #
  ################################################################


  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_options_menu
  #
  #  Parameters: 
  #
  # Description: assigns Analyzer options menu
  #
  #***************************************************************************
  proc set_options_menu { widget  ScreenNamespace } \
  {
    Debug::Puts AnalyzerUI 2 "set_options_menu:  $widget,  $ScreenNamespace"

    variable StateUI

    set topWin  [winfo toplevel $widget]
    
    # Get the Options menubutton path
    set buttonName  [View::GetOptionMenuName  $topWin]
    
    option add  *$buttonName*helpUrl  index.html  widgetDefault
    
    # Get the Options menu path
    set menuName  ${buttonName}.menu
    
    # Destroy the existing Options menu
    destroy  $menuName
    
    # Create a new Options menu
    menu $menuName -tearoff 0  
    
    # Add ENTER key options
    $menuName add command -accel RETURN            \
                          -label "Accept Options"  \
                          -com "tkMenuUnpost $menuName"
                          
    $menuName add separator

    
    $menuName add checkbutton  -label     "Live"  \
                         -accel     1       \
                         -variable  ::AnalyzerUI::StateUI(live) \
                         -command "Analyzer::Toggle LIVE"
                         
    bind $menuName <KeyRelease-1> {Analyzer::Toggle LIVE }
                               
    $menuName add checkbutton  -label     "Average"  \
                         -accel     2          \
                         -variable  ::AnalyzerUI::StateUI(avg) \
                         -command "Analyzer::Toggle AVERAGE"
                         
    bind $menuName <KeyRelease-2> { Analyzer::Toggle AVERAGE }

    $menuName add checkbutton  -label     "Peak"  \
                         -accel     3       \
                         -variable  ::AnalyzerUI::StateUI(peak) \
                         -command "Analyzer::Toggle PEAK"
                         
    bind $menuName <KeyRelease-3> {Analyzer::Toggle PEAK }



    $menuName add check  -label     "Compare"  \
                         -accel     5          \
                         -variable  ::AnalyzerUI::StateUI(compare) \
                         -command "Analyzer::Toggle REF"

    bind $menuName <KeyRelease-5> { Analyzer::Toggle REF }

    $menuName add separator
    
    # exceptions for channel analyzer, some rbw's aren't available
    if { $StateUI(bw_type) == "MAIN"} \
    {
      $menuName add check  -label     "Tracking Gen."  \
                           -accel     6                \
                           -onvalue   1                \
                           -offvalue  0                \
                           -variable  ::Analyzer::State(tgstate) \
                           -command {Analyzer tgstate $::Analyzer::State(tgstate)}

      bind $menuName <KeyRelease-6> { ::Analyzer::toggle_tgen }

      $menuName add separator
    }
    
    # Markers
    $menuName add command -label    "Configure Markers"       \
                          -accel    7                         \
                          -command  "tkMenuUnpost $menuName;  \
                                     ::Anlz_MarkersUI::config_marker  $widget"
                                     
    bind $menuName <KeyRelease-7>  "tkMenuUnpost $menuName;  \
                                     ::Anlz_MarkersUI::config_marker  $widget"

    $menuName add separator

    # Add ESCAPE functionality
    $menuName add command -accel ESC       \
                          -label "Escape"  \
                          -com  "tkMenuUnpost $menuName"

    # Bind button release so default bindings do not unpost the menu
    # bind to the Return and Escape hardkeys
    bind $menuName <KeyRelease-F8>     "tkMenuUnpost $menuName; break"
    bind $menuName <KeyRelease-Escape> "tkMenuUnpost $menuName; break"
    
    bind $menuName <KeyRelease-Return> { MenuInvokeRemain %W 0; break }
    bind $menuName <ButtonRelease-1>   { MenuInvokeRemain %W 1; break }

    return $buttonName
  } 
  
  #***************************************************************************
  #
  #        Name: set_tgen
  #
  #  Parameters: none
  #
  # Description: This function is used as a menu command to force the 
  #              evaluation of the variable to occur when the menu is 
  #              activated and not when the menu is created.
  #
  #***************************************************************************
  proc set_tgen { } \
  {
     Analyzer  tgstate  $Analyzer::State(tgstate) -nowait
  }
  

  #***************************************************************************
  #
  #        Name: toggle
  #
  #  Parameters: 
  #
  # Description: toggle the state of something....
  #
  #***************************************************************************
  proc toggle { item } \
  {
     variable StateUI
  
     switch -- $StateUI($item) \
     {
       "OFF"   { set StateUI($item)  "ON"  }
        
       default { set StateUI($item)  "OFF" }
     }

  }
  
  #############################################################################
  # 
  #  Adjacent Channel Power  (ACP)  Support procedures
  # 
  #############################################################################
  #***************************************************************************
  #
  #        Name: set_acp_mode
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              mode - ON or OFF
  #
  # Description: Sets the state of the ACP mode
  #
  #***************************************************************************
  proc set_acp_mode { tile_path  acp_mode } \
  {
    Debug::Puts AnalyzerUI 2 "set_acp_mode:  $tile_path,  $acp_mode"
    
    variable StateUI
    
    set StateUI(acp_mode)  $acp_mode
    
    set tile_path  [get_tile  $tile_path]
    
    # Get the toplevel
    set top_level  [winfo toplevel $tile_path]
    
    # Get the Options menubutton path
    set option_menubtn  [View::GetOptionMenuName  $top_level]
    
    # Get the Options menu path
    set option_menu  ${option_menubtn}.menu
    

    if { $acp_mode == "ON" } \
    { 
      # Set the SPAN to 50 kHz
      after idle "Analyzer  span  0.050  -1.0 -nowait"
      
      # Disable configuring the Markers
      # Disable the ability to bring up the Marker Config toplevel
      # for the Full Analyzer only
      if { [winfo class  $tile_path] == "FULL_ANLZ" } \
      {
        # Perform the following only if the Full Analyzer is displayed
        if { [winfo ismapped  $tile_path] } \
        {
          set index  [$option_menu  index  "Configure Markers"]
        
          $option_menu  entryconfig  $index  -state disabled
        }
      }
        
      # Turn Marker 1 ACP state ON
      set ::Anlz_MarkersUI::StateUI(marker,1,state_acp)  "ON"
      
      # Turn Marker 2 ACP state ON
      set ::Anlz_MarkersUI::StateUI(marker,2,state_acp)  "ON"

      # Turn Marker 3 ACP state ON
      set ::Anlz_MarkersUI::StateUI(marker,3,state_acp)  "ON"

      # Turn Marker 4 ACP state OFF
      set ::Anlz_MarkersUI::StateUI(marker,4,state_acp)  "OFF"

      # Turn Marker 5 ACP state OFF
      set ::Anlz_MarkersUI::StateUI(marker,5,state_acp)  "OFF"

      # Turn Marker 6 ACP state OFF
      set ::Anlz_MarkersUI::StateUI(marker,6,state_acp)  "OFF"

      trace variable  ::Receiver::State(freq)  w  \
                         "Anlz_MarkersUI::update_marker_freq_acp  $tile_path"
      
      trace variable  ::Power::State(reading,dBm)  w  \
                                     "AnalyzerUI::update_tos_acp  $tile_path"
      
      Anlz_MarkersUI::update_marker_freq_acp   $tile_path
      
      update_tos_acp  $tile_path
    } \
    else \
    {
      # Enable configuring the Markers
      # Enable the ability to bring up the Marker Config toplevel
      # for the Full Analyzer only
      if { [winfo class  $tile_path] == "FULL_ANLZ" } \
      {
        # Perform the following only if the Full Analyzer is displayed
        if { [winfo ismapped  $tile_path] } \
        {
          set index  [$option_menu  index  "Configure Markers"]
        
          $option_menu  entryconfig  $index  -state normal
        }
      }

      # Turn Marker 1 ACP state OFF
      set ::Anlz_MarkersUI::StateUI(marker,1,state_acp)  "OFF"
      
      # Turn Marker 2 ACP state OFF
      set ::Anlz_MarkersUI::StateUI(marker,2,state_acp)  "OFF"
      
      # Turn Marker 3 ACP state OFF
      set ::Anlz_MarkersUI::StateUI(marker,3,state_acp)  "OFF"

      # Turn Marker 4 ACP state OFF
      set ::Anlz_MarkersUI::StateUI(marker,4,state_acp)  "OFF"

      # Turn Marker 5 ACP state OFF
      set ::Anlz_MarkersUI::StateUI(marker,5,state_acp)  "OFF"

      # Turn Marker 6 ACP state OFF
      set ::Anlz_MarkersUI::StateUI(marker,6,state_acp)  "OFF"


      trace vdelete  ::Receiver::State(freq)  w  \
                         "Anlz_MarkersUI::update_marker_freq_acp  $tile_path"
      
      trace vdelete  ::Power::State(reading,dBm)  w  \
                                     "AnalyzerUI::update_tos_acp  $tile_path"
    }
  }


  #***************************************************************************
  #
  #        Name: update_tos_acp
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Updates the Top Of Scale in the ACP mode
  #
  #***************************************************************************
  proc update_tos_acp { tile_path  args } \
  {
    Debug::Puts AnalyzerUI 2 "update_tos_acp:  $tile_path,  $args"
    
    variable StateUI
    
    if { $StateUI(mode) == "ACP" && $StateUI(acp_mode) == "ON" } \
    {
      # Get the Power Meter reading, in dBm
      set power_reading  $::Power::State(reading,dBm)
      
      if { $power_reading < 0.0 } \
      {
        return
      }
      
      # Adjust by 13.0 dBm
      set power_reading  [expr $power_reading - 5.0]
      
      # Set the Analyzer Top Of Scale
      set ::Analyzer::State(tos)  $power_reading
      
      AnalyzerUI::set_tos
    }
  }
   

  #############################################################################
  # 
  #               Normalizing Support
  # 
  #############################################################################
  
  #***************************************************************************
  #
  #        Name: AnalyzerUI::anlz_normalizing_message
  #
  #  Parameters: none
  #
  # Description: Analyzer normalizing dialog message
  #
  #***************************************************************************
  proc anlz_normalizing_mess {} \
  {
    Debug::Puts AnalyzerUI 2 "::anlz_normalizing_mess"
  }

  
  #***************************************************************************
  #
  #        Name: AnalyzerUI::anlz_norm_message
  #
  #  Parameters: 
  #
  # Description: displays anlz normalize message
  #
  #***************************************************************************
  proc anlz_norm_message { } \
  {
    Debug::Puts AnalyzerUI 2 "::anlz_norm_message"
  
    message_box .anlznormal INFORM
  }
  




  #############################################################################
  # 
  #   Analyzer Subsystem Control/Interface Routines
  # 
  #############################################################################
  
  #***************************************************************************
  #
  #        Name: AnalyzerUI::set_offending_hardware_off
  #
  #  Parameters: none
  #
  # Description: shut off for extra DSP MF time..
  #
  #***************************************************************************
  proc set_offending_hardware_off { } \
  {
    Debug::Puts AnalyzerUI 2 "::set_offending_hardware_off"
  }
  

  

  ################################################################
  #
  #      Coupling Cal Display Summary
  #
  ################################################################
  
  #***************************************************************************
  #
  #        Name: AnalyzerUI::do_calsummary
  #
  #  Parameters: name element op
  #
  # Description: update calsummary when calstate changes
  #              to be used as a trace operation
  #
  #***************************************************************************
  proc do_calsummary { lun } \
  {
    Debug::Puts AnalyzerUI 2 "::do_calsummary:  $lun"

    variable StateUI
     
    foreach { el } $StateUI(calstate.$lun) \
    {
      if { [string tolower $el] == "uncal"} \
      {
        set cal 0
      }
    }
  
    if { $cal } \
    {
      set anlz(calsummary.$lun) CAL
      
      return REMOVE
    } \
    else \
    {
      set anlz(calsummary.$lun) UnCAL
      
      return DISPLAY
    }
  }
  

  #***************************************************************************
  #
  #        Name: AnalyzerUI::display_cal
  #
  #  Parameters: state - ON,OFF
  #              path  - remember display path
  #
  # Description: place the uncalibration widget on screen
  #
  #***************************************************************************
  global display_cal_save

  set display_cal_save(path)    {}
  set display_cal_save(options) {}
  
  proc display_cal { state {path invalid} {lun invalid}} \
  {
    Debug::Puts AnalyzerUI 2 "::display_cal:  $state  $path  $lun"
    
    global display_cal_save
  
    if { $path == "" } \
    {
      puts "::display_cal path err: $path"
      return
    }
  
    if { $path != "invalid" } \
    {
      set display_cal_save(path)    $path
      set display_cal_save(options) [place info $path]
    } \
    else \
    {
      set path $display_cal_save(path)
    }
  
    if { $state == "INIT" } \
    {
      if { $lun == "invalid" } \
      {
        puts "::display_cal err: state $state, lun $lun"

        return
      } \
      else \
      {
        set state [::AnalyzerUI::do_calsummary $lun]
      }
    }
  
    switch $state \
    {
      DISPLAY -
      ON  -
      1   { ::AnalyzerUI::place_arg $path "-in"     $display_cal_save(options)
            ::AnalyzerUI::place_arg $path "-relx"   $display_cal_save(options)
            ::AnalyzerUI::place_arg $path "-rely"   $display_cal_save(options)
            ::AnalyzerUI::place_arg $path "-width"  $display_cal_save(options)
            ::AnalyzerUI::place_arg $path "-height" $display_cal_save(options) }
    
      REMOVE -
      OFF    -
      0      { place forget $path }
    
      default { puts "::display_cal state err: $state" }
    }
  }
  

  #***************************************************************************
  #
  #        Name: AnalyzerUI::update_calsummary
  #
  #  Parameters: name element op
  #
  # Description: update calsummary when calstate changes
  #              to be used as a trace operation
  #
  #***************************************************************************
  proc update_calsummary { name element op } \
  {
    Debug::Puts AnalyzerUI 2 "::update_calsummary:  $name  $element  $op"
      
    catch {
      scan $element "calstate.%d" lun
      set name ${name}($element)
  
      upvar #0 $name list
  
      set state [::AnalyzerUI::do_calsummary $lun]
      ::AnalyzerUI::display_cal $state
    } result
  }


  #***************************************************************************
  #
  #        Name: AnalyzerUI::zoom_analyzer_screen
  #
  #  Parameters: none 
  #
  # Description: zoom/switch to full scope screen 
  #
  #***************************************************************************
  proc zoom_analyzer_screen { widget } \
  {
     variable StateUI
     variable prev_screen

     set StateUI(bw_type) "CHANNEL"
     set prev_screen $::Screens::CurrentNamespace

     ::Screens::Show $widget Analyzer_Channel_Screen
  }


  #***************************************************************************
  #
  #        Name: set_freq_mouse
  #
  #  Parameters: widget - invoking widget path
  #              x_coord - present x coordinate of the mouse relative to the 
  #                        Analyzer trace widget
  #
  # Description: Sets the Analyzer center frequency to the frequency 
  #              represented by the position of the mouse on the Analyzer
  #              trace widget when double clicked.
  #
  #***************************************************************************
  proc set_freq_mouse { widget  x_coord } \
  {
    Debug::Puts AnalyzerUI 2 "set_freq_mouse:  $widget,  $x_coord"
    
    set tile_path  [get_tile  $widget]
    
    # Convert to a 'double' value for calculations
    set x_coord  [expr double($x_coord)]
    
    # Get the width of the Analyzer trace widget
    set width  [${tile_path}.anlz cget -width]
    
    # Convert to a 'double' value for calculations
    set width  [expr double($width)]
    
    # Determine the center of the Analyzer trace widget
    set center  [expr $width / 2.0]
    
    # Determine the x offset of the mouse position
    set offset  [expr $x_coord - $center]
    
    # Get the Analyzer span
    set span  $::Analyzer::State(span,MHz)
        
    # Set the offset frequency to the ratio of the x offset value
    set offset_freq  [expr ($offset / $center) * ($span / 2.0)]
    
    # Get the center frequency
    set center_freq  $::Receiver::State(freq)
    
    # Set the new center frequency to the present center freq plus the offset
    set new_freq  [expr $center_freq + $offset_freq]
    
    
    # Adjust the Analyzer if the center freq is less than half of the span
    if { $::Receiver::State(freq) <= [expr $span / 2.0] } \
    {
      # Get the present index of the menu
      set pres_index  [${tile_path}.mnubtn_span.menu index $::Analyzer::State(span)]
      
      # If the index value is greater than 0
      if { $pres_index > 0 } \
      {
        # Select the next lower index on the menu
        ${tile_path}.mnubtn_span.menu invoke [expr $pres_index - 1]
 
        set pres_index  [${tile_path}.mnubtn_span.menu index $::Analyzer::State(span)]
      }
    }
 
    # Set the Analyzer to the new center frequency 
    Receiver freq $new_freq -nowait
  }
  
 
  #***************************************************************************
  #
  #        Name: set_span_mouse
  #
  #  Parameters: widget  - invoking widget path
  #              x_coord - present x coordinate of the mouse relative to the 
  #                        Analyzer trace widget
  #              mode    - a flag to determine which maouse button action
  #                        was received to capture the start and stop
  #                        mouse coordinates.
  #
  # Description: Sets the Analyzer span to a span represented by the 
  #              movement of the mouse on the Analyzer trace widget when 
  #              the left mouse button is held down while in motion.
  #
  #***************************************************************************
  proc set_span_mouse { widget  x_coord  mode } \
  {
    Debug::Puts AnalyzerUI 2 "set_span_mouse:  $widget,  $x_coord,  $mode"
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]
    
    if { $mode == "START" } \
    {
      # Convert to a 'double' value for calculations
      set StateUI(x_start)  [expr double($x_coord)]
    }
    
    if { $mode == "STOP" } \
    {
      # Convert to a 'double' value for calculations
      set StateUI(x_stop)  [expr double($x_coord)]
      
      set StateUI(x_span)  [expr abs($StateUI(x_stop) - $StateUI(x_start))]
      
      set StateUI(x_center)  [expr ($StateUI(x_stop) + $StateUI(x_start)) / 2.0]
      
      # Convert to an 'int' value for display
      set StateUI(x_center)  [expr int($StateUI(x_center))]

      # Reset the center frequency to the center of the selected area
      set_freq_mouse  $tile_path  $StateUI(x_center)

      set trace_width   [${tile_path}.anlz cget -width]
  
      # Convert to a 'double' value for calculations
      set trace_width   [expr double($trace_width)]

      set span  $::Analyzer::State(span,MHz)
         
      set span_selected  [expr ($StateUI(x_span) / $trace_width) * $span]
      
      foreach span [lsort -real [array names ::Analyzer::span_strings]] \
      {
         if {$span_selected <= $span } \
         {
            break;
         }
      }
              
      after idle "Analyzer  span  $span  -1.0  -nowait"
    }
  }


  #****************************************************************************
  #
  #        Name: AnalyzerUI::getSaveState
  #
  # Parameters:  none
  #
  # Description: This function returns a list of TCL commands which can be used
  #              to set the AnalyzerUI tiles to its current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    variable StateUI
    variable prev_screen
 
    set returnList \
    {
      set ::AnalyzerUI::StateUI(units)     {$StateUI(units)}
      set ::AnalyzerUI::StateUI(mode)      {$StateUI(mode)}
      set ::AnalyzerUI::StateUI(bw_type)   {$StateUI(bw_type)}
      set ::AnalyzerUI::prev_screen        {$prev_screen}
    }
       
    return [subst $returnList]
  }


  #****************************************************************************
  #
  #        Name: AnalyzerUI::Sync
  #
  #  Parameters: none
  #
  # Description: Write state variables to AnalyzerUI
  #
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }


};  # End of AnalyzerUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles   

  lappend tiles  AnalyzerUI
}

  
  
  
namespace eval Anlz_MarkersUI \
{
  variable  marker_list  { 0  1  2  3  4  5  6  7 }; # List of logical Marker numbers
  
  variable  marker_nums  { 1  2  3  4  5  6  7  8 }; # Used for Marker names and
                                                     # Difference and Cable Fault
                                                     # menus
  
  array set marker_color_list  { 0  "gold"    \
                                 1  "green"   \
                                 2  "cyan"    \
                                 3  "tan"     \
                                 4  "orange"  \
                                 5  "red"     \
                                 6  "yellow"  \
                                 7  "gray70" }

  variable  StateUI
  
  set StateUI(marker,cable_flt,1)         1
  set StateUI(marker,cable_flt,2)         2
  set StateUI(marker,cable_flt,start)     0
  set StateUI(marker,cable_flt,stop)      0
  set StateUI(marker,cable_flt,value,m)   0.00
  set StateUI(marker,cable_flt,value,ft)  0.00
  set StateUI(marker,cable_flt,units)     "m"

  set StateUI(vel_factor)  70.00
  # Set the Marker defaults
  proc update_marker_freq { num args } \
  {
     variable StateUI
     
     # update cable fault info
     set marker_1  [expr  $StateUI(marker,cable_flt,1) - 1]
     set marker_2  [expr  $StateUI(marker,cable_flt,2) - 1]
    
     if {$num == $marker_1} \
     {
        set StateUI(marker,cable_flt,start) $StateUI(marker,$num,freq)
     }
     if {$num == $marker_2} \
     {
        set StateUI(marker,cable_flt,stop) $StateUI(marker,$num,freq)
     }
  }
  
  foreach { num } $marker_list \
  {
    set StateUI(marker,$num,state)       "OFF"
    set StateUI(marker,$num,state_acp)   "OFF"
    set StateUI(marker,$num,orient)      "VERT"
    set StateUI(marker,$num,color)       $marker_color_list($num)
    set StateUI(marker,$num,freq)        0.000000
    set StateUI(marker,$num,mag)         0.00
    set StateUI(marker,$num,time)        0.00
    trace variable StateUI(marker,$num,freq) w [namespace code "update_marker_freq $num"]
  }

  set StateUI(marker,diff1,1)     1
  set StateUI(marker,diff1,2)     1
  set StateUI(marker,diff1,db)    0.00
  set StateUI(marker,diff1,freq)  0.000000
  set StateUI(marker,diff1,time)  0.00

  set StateUI(marker,diff2,1)     1
  set StateUI(marker,diff2,2)     1
  set StateUI(marker,diff2,db)    0.00
  set StateUI(marker,diff2,freq)  0.000000
  set StateUI(marker,diff2,time)  0.00

  set StateUI(diff_units,1)  "dB"
  set StateUI(diff_units,2)  "dB"
  
 
  set StateUI(cable_material_menu)  { "Solid Polyethylene"      \
                                      "Foam Polyethylene"       \
                                      "Foam Polystyrene"        \
                                      "Air Space Polyethylene"  \
                                      "Solid Teflon"            \
                                      "Air Space Teflon"        \
                                      "RG-58 A/U w/solid poly"  \
                                      "User defined"             }
  
  
  proc Update_trace { args } \
  {
     Analyzer trace -nowait
  }
  
  Analyzer -onchange trace { set ::Anlz_MarkersUI::StateUI(array)  %V }
  
  # remove this when analyzer engine provides marker support like audio analyzer
  trace variable ::Analyzer::State(center_dBm)  w  "Anlz_MarkersUI::Update_trace"
  

  #***************************************************************************
  #
  #        Name: get_tile
  #
  #  Parameters: any child widget of the tile
  #
  # Description: Get the name of the tile's parent frame
  #
  #***************************************************************************
  proc get_tile { widget } \
  {
    while { 1 } \
    {
      switch -- [winfo class $widget] \
      {
        "QUARTER_ANLZ"     -
        "FULL_ANLZ"        -
        "ANLZ_MARKER_CFG"  { return $widget }
      }
        
      set widget [winfo parent $widget]
    }
  }


  #***************************************************************************
  #
  #        Name: config_marker
  #
  #  Parameters: tile_path - tile path of the widget
  #
  # Description: Displays the configuration toplevel for the markers.
  #
  #***************************************************************************
  proc config_marker { tile_path } \
  {
    Debug::Puts AnalyzerUI 2 "config_marker:  $tile_path"
    
    variable  marker_list
    variable  marker_color_list
    
    set top_level  [winfo toplevel $tile_path]
   
    set tile_path  [name_child  $top_level  anlz_marker_cfg]

    if { ![winfo exists $tile_path] } \
    {
      set tile_path  \
                 [Screens::create_toplevel_tile  $tile_path  ANLZ_MARKER_CFG]
    }
    
    bind $tile_path <KeyRelease-F8>  "destroy [winfo toplevel $tile_path]"
    
    bind $tile_path <<Destroy>>  \
                        "Anlz_MarkersUI::destroy_anlz_marker_cfg  $tile_path"
                       
    # Remove the Frequency and Time fields
    foreach { num } $marker_list \
    {
      grid remove  ${tile_path}.ent_mark${num}_freq
      grid remove  ${tile_path}.lbl_mark${num}_freq_units
      grid remove  ${tile_path}.ent_mark${num}_time
      grid remove  ${tile_path}.lbl_mark${num}_time_units
      
      ${tile_path}.lbl_mark${num}  config  -bg $marker_color_list($num)
      
      trace variable ::Anlz_MarkersUI::StateUI(marker,$num,orient)  w  \
                      "Anlz_MarkersUI::update_config_fields  $tile_path  $num"
                     
      trace variable ::Analyzer::State(span)  w  \
                      "Anlz_MarkersUI::update_config_fields  $tile_path  $num"

      update_config_fields  $tile_path  $num
    }
    
    trace variable ::Anlz_MarkersUI::StateUI(diff_units,1)  w  \
                            "Anlz_MarkersUI::update_diff_units  $tile_path  1"

    trace variable ::Anlz_MarkersUI::StateUI(diff_units,2)  w  \
                            "Anlz_MarkersUI::update_diff_units  $tile_path  2"

    update_diff_units  $tile_path  1
    update_diff_units  $tile_path  2

    trace variable ::Anlz_MarkersUI::StateUI(marker,cable_flt,units)  w  \
                          "Anlz_MarkersUI::update_cable_flt_units  $tile_path"

    update_cable_flt_units  $tile_path
    
    safe_focus_force ${tile_path}.btn_return
  }


  #***************************************************************************
  #
  #        Name: destroy_anlz_marker_cfg
  #
  #  Parameters: tile_path - tile path of the invoking widget 
  #
  # Description: Performs a clean up when the marker config toplevel is 
  #              destroyed
  #
  #***************************************************************************
  proc destroy_anlz_marker_cfg { tile_path } \
  {
    Debug::Puts AnalyzerUI 2 "destroy_anlz_marker_cfg:  $tile_path"
    
    variable  marker_list

    foreach { num } $marker_list \
    {
      trace vdelete ::Anlz_MarkersUI::StateUI(marker,$num,orient)  w  \
                      "Anlz_MarkersUI::update_config_fields  $tile_path  $num"

      trace vdelete ::Analyzer::State(span)  w  \
                      "Anlz_MarkersUI::update_config_fields  $tile_path  $num"
    }

    trace vdelete ::Anlz_MarkersUI::StateUI(diff_units,1)  w  \
                            "Anlz_MarkersUI::update_diff_units  $tile_path  1"

    trace vdelete ::Anlz_MarkersUI::StateUI(diff_units,2)  w  \
                            "Anlz_MarkersUI::update_diff_units  $tile_path  2"
    
    trace vdelete ::Anlz_MarkersUI::StateUI(marker,cable_flt,units)  w  \
                          "Anlz_MarkersUI::update_cable_flt_units  $tile_path"
  }


  #***************************************************************************
  #
  #        Name: set_marker_on_off
  #
  #  Parameters: real_num - real marker number (starts at 1 for Marker 1)
  #              state    - either ON or OFF
  #
  # Description: Sets the state of the marker ON and OFF
  #              NOTE: This procedure is used as a remote command
  #
  #***************************************************************************
  proc set_marker_on_off { real_num  { state  "OFF" } } \
  {
    Debug::Puts AnalyzerUI 2 "set_marker_on_off:  $real_num,  $state"
    
    variable  StateUI
    
    set num  [expr  $real_num - 1]
    
    switch -- $state \
    {
      "ON"   { set StateUI(marker,$num,state)  "ON"  }
      
      "OFF"  { set StateUI(marker,$num,state)  "OFF" }
    }   
  }


  #***************************************************************************
  #
  #        Name: toggle_marker_on_off
  #
  #  Parameters: num - marker number (starts at 0 for Marker 1)
  #
  # Description: Toggles the state of the marker ON and OFF
  #
  #***************************************************************************
  proc toggle_marker_on_off { num } \
  {
    Debug::Puts AnalyzerUI 2 "toggle_marker_on_off:  $num"
    
    variable  StateUI
    
    if { $StateUI(marker,$num,state) == "ON" } \
    {
      set StateUI(marker,$num,state)  "OFF"
    } \
    else \
    {
      set StateUI(marker,$num,state)  "ON"
    }
  }


  #***************************************************************************
  #
  #        Name: set_marker_orient
  #
  #  Parameters: real_num - real marker number (starts at 1 for Marker 1)
  #              orient   - either ON or OFF
  #
  # Description: Sets the orientation of the marker VERT or HORIZ
  #              NOTE: This procedure is used as a remote command
  #
  #***************************************************************************
  proc set_marker_orient { real_num  { orient  "VERT" } } \
  {
    Debug::Puts AnalyzerUI 2 "set_marker_orient:  $real_num,  $orient"
    
    variable  StateUI
    
    set num  [expr  $real_num - 1]
    
    switch -- $orient \
    {
      "HORIZ"  { set StateUI(marker,$num,orient)  "HORIZ" }

      "VERT"   { set StateUI(marker,$num,orient)  "VERT"  }
    }   
  }


  #***************************************************************************
  #
  #        Name: toggle_marker_orient
  #
  #  Parameters: num - marker number
  #
  # Description: Toggles the orientation of the marker VRT and HORIZ
  #
  #***************************************************************************
  proc toggle_marker_orient { num } \
  {
    Debug::Puts AnalyzerUI 2 "toggle_marker_orient:  $num"
    
    variable  StateUI
    
    if { $StateUI(marker,$num,orient) == "VERT" } \
    {
      set StateUI(marker,$num,orient)  "HORIZ"
    } \
    else \
    {
      set StateUI(marker,$num,orient)  "VERT"
    }
  }


  #***************************************************************************
  #
  #        Name: update_config_fields
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #              num       - marker number
  #
  # Description: Updates the display for changes in the Marker orientation.
  #              For updating the fields on the Marker configuration 
  #              toplevel.
  #
  #***************************************************************************
  proc update_config_fields { tile_path  num  args } \
  {
    Debug::Puts AnalyzerUI 2 "update_config_fields:  $tile_path,  $num,  $args"
    
    variable  StateUI
    
    set tile_path  [get_tile  $tile_path]
    
    # Update fields for the Marker configuration toplevel only
    if { [winfo class  $tile_path] != "ANLZ_MARKER_CFG" } \
    {
      return
    }
    
    # Remove the Frequency entry and label fields, the Time entry and 
    # label fields and the Magnitude entry and label fields
    grid remove  ${tile_path}.ent_mark${num}_freq
    grid remove  ${tile_path}.lbl_mark${num}_freq_val
    grid remove  ${tile_path}.lbl_mark${num}_freq_units
    grid remove  ${tile_path}.ent_mark${num}_time
    grid remove  ${tile_path}.lbl_mark${num}_time_val
    grid remove  ${tile_path}.lbl_mark${num}_time_units
    grid remove  ${tile_path}.ent_mark${num}_mag
    grid remove  ${tile_path}.lbl_mark${num}_mag_val
    
    switch -- $StateUI(marker,$num,orient) \
    {
      "VERT"   { if { $::Analyzer::State(span) == "0 SPAN" } \
                 {
                   # Display the Time entry field, the Time units label and 
                   # the Magnitude label field
                   Screens::position_widget  ${tile_path}.ent_mark${num}_time
                   Screens::position_widget  ${tile_path}.lbl_mark${num}_time_units
                   Screens::position_widget  ${tile_path}.lbl_mark${num}_mag_val
                 } \
                 else \
                 {
                   # Display the Frequency entry field, the Frequency units label and 
                   # the Magnitude label field
                   Screens::position_widget  ${tile_path}.ent_mark${num}_freq
                   Screens::position_widget  ${tile_path}.lbl_mark${num}_freq_units
                   Screens::position_widget  ${tile_path}.lbl_mark${num}_mag_val 
                 }
               }
      
      "HORIZ"  { if { $::Analyzer::State(span) == "0 SPAN" } \
                 {
                   # Display the Time label field, the Time units label and 
                   # the Magnitude entry field
                   Screens::position_widget  ${tile_path}.lbl_mark${num}_time_val
                   Screens::position_widget  ${tile_path}.lbl_mark${num}_time_units
                   Screens::position_widget  ${tile_path}.ent_mark${num}_mag
                 } \
                 else \
                 {
                   # Display the Frequency label field, the Frequency units label and 
                   # the Magnitude entry field
                   Screens::position_widget  ${tile_path}.lbl_mark${num}_freq_val
                   Screens::position_widget  ${tile_path}.lbl_mark${num}_freq_units
                   Screens::position_widget  ${tile_path}.ent_mark${num}_mag
                 }
               }
    }
  }


  #***************************************************************************
  #
  #        Name: update_marker_on_off
  #
  #  Parameters: widget - invoking widget path
  #              num    - marker number
  #
  # Description: Displays a marker on the Analyzer widget, or removes it from 
  #              the display
  #
  #***************************************************************************
  proc update_marker_on_off { widget  num  args } \
  {
     set cmd [namespace code "update_marker_on_off_core $widget $num"]
     after cancel $cmd
     after idle $cmd
  }
  proc update_marker_on_off_core { widget  num } \
  {

    Debug::Puts AnalyzerUI 2 "update_marker_on_off:  $widget,  $num"
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]
    
    if { ![winfo exists ${tile_path}.anlz] } \
    {
      return
    }

    set trace_height  [winfo height ${tile_path}.anlz]
    set trace_width   [winfo width ${tile_path}.anlz]
    
    # Destroy the Marker if it exists so that the bindings and settings
    # are initialized for either the normal or the ACP mode
    if { [winfo exists ${tile_path}.anlz.marker$num] } \
    {
      destroy  ${tile_path}.anlz.marker$num
      destroy  ${tile_path}.anlz.marker_blk$num
      destroy  ${tile_path}.anlz.marker_name$num
      destroy  ${tile_path}.anlz.marker_freq$num
      destroy  ${tile_path}.anlz.marker_mag$num
    }
      
    # Don't need Markers 4, 5, or 6 for the ACP measurement, 
    # so don't allow them to be displayed when in the ACP mode
    if { ($::AnalyzerUI::StateUI(mode) == "ACP") && ($num > 3) } \
    {
      return
    }

    # Vertically oriented markers and ACP markers
    if { $StateUI(marker,$num,orient) == "VERT" || $::AnalyzerUI::StateUI(mode) == "ACP" } \
    { 
      # Marker line
      canvas ${tile_path}.anlz.marker$num -height $trace_height   \
                                          -width 3                \
                                          -bg $StateUI(marker,$num,color)
    
      if { $::AnalyzerUI::StateUI(mode) == "ACP" } \
      {
        # No bindings in the ACP mode
      } \
      else \
      {
        # Create bindings
        bind ${tile_path}.anlz.marker$num <ButtonPress-1>  \
                          "Anlz_MarkersUI::set_marker_color  %W  $num  white"
      
        bind ${tile_path}.anlz.marker$num <ButtonRelease-1>                 \
                       "Anlz_MarkersUI::set_marker_color  %W  $num  normal"
      }
     
      # Marker block - to help the user click on the marker for cursor movement
      canvas ${tile_path}.anlz.marker_blk$num                           \
                                        -height 10                      \
                                        -width 23                       \
                                        -bg $StateUI(marker,$num,color) \
                                        -takefocus  1                   \
                                        -highlightthickness  3          \
                                        -highlightbackground  blue
      
      if { $::AnalyzerUI::StateUI(mode) == "ACP" } \
      {
        # Change the marker block to not take focus
        ${tile_path}.anlz.marker_blk$num  config  -takefocus  0           \
                                                  -highlightthickness  0  \
                                                  -highlightbackground  blue
     
        # No bindings for the ACP mode
      } \
      else \
      {
        # Create bindings
        bind ${tile_path}.anlz.marker_blk$num <ButtonPress-1>  \
                          "Anlz_MarkersUI::set_marker_color  %W  $num  white"
      
        bind ${tile_path}.anlz.marker_blk$num <ButtonRelease-1>             \
                       "Anlz_MarkersUI::set_marker_color  %W  $num  normal"
                                
        bind ${tile_path}.anlz.marker_blk$num <Return>  \
                             "Anlz_MarkersUI::set_marker_spin_move  %W  $num"
     
        add_bindtag  ${tile_path}.anlz.marker_blk$num  Cursor
      }
     
      # Marker name
      canvas ${tile_path}.anlz.marker_name$num -height 15 -width 45 -bg blue
     
      set ref_num  [expr  $num + 1]

      ${tile_path}.anlz.marker_name$num create text 0 0                      \
                                          -text "MRKR${ref_num}"             \
                                          -fill $StateUI(marker,$num,color)  \
                                          -font analyzer_tiny                \
                                          -anchor nw                         \
                                          -tags marker_name
     
      # Marker freq
      canvas ${tile_path}.anlz.marker_freq$num -height 15 -width 85 -bg blue
     
      ${tile_path}.anlz.marker_freq$num create text 0 0                      \
                                          -text ""                           \
                                          -fill $StateUI(marker,$num,color)  \
                                          -font analyzer_tiny                \
                                          -anchor nw                         \
                                          -tags marker_freq
     
      # Marker magnitude
      canvas ${tile_path}.anlz.marker_mag$num -height 15 -width 75 -bg blue
     
      ${tile_path}.anlz.marker_mag$num create text 0 0                       \
                                          -text ""                           \
                                          -fill $StateUI(marker,$num,color)  \
                                          -font analyzer_tiny                \
                                          -anchor nw                         \
                                          -tags marker_mag
    } \
    elseif { $StateUI(marker,$num,orient) == "HORIZ" } \
    { 
      # Horizontally oriented markers
      # Marker line
      canvas ${tile_path}.anlz.marker$num -height 3             \
                                          -width  $trace_width  \
                                          -bg $StateUI(marker,$num,color)
    
      # Create bindings
      bind ${tile_path}.anlz.marker$num <ButtonPress-1>  \
                          "Anlz_MarkersUI::set_marker_color  %W  $num  white"
    
      bind ${tile_path}.anlz.marker$num <ButtonRelease-1>  \
                       "Anlz_MarkersUI::set_marker_color  %W  $num  normal"

      # Marker block - to help the user click on the marker for cursor movement
      canvas ${tile_path}.anlz.marker_blk$num                           \
                                        -height 23                      \
                                        -width  10                      \
                                        -bg $StateUI(marker,$num,color) \
                                        -takefocus  1                   \
                                        -highlightthickness  3          \
                                        -highlightbackground  blue
    
      # Create bindings
      bind ${tile_path}.anlz.marker_blk$num <ButtonPress-1>  \
                          "Anlz_MarkersUI::set_marker_color  %W  $num  white"
    
      bind ${tile_path}.anlz.marker_blk$num <ButtonRelease-1>  \
                       "Anlz_MarkersUI::set_marker_color  %W  $num  normal"
                              
      bind ${tile_path}.anlz.marker_blk$num <Return>  \
                             "Anlz_MarkersUI::set_marker_spin_move  %W  $num"

      add_bindtag  ${tile_path}.anlz.marker_blk$num  Cursor

      # Marker name
      canvas ${tile_path}.anlz.marker_name$num -height 15 -width 45 -bg blue

      set ref_num  [expr $num + 1]

      ${tile_path}.anlz.marker_name$num create  text  0  0                   \
                                          -text "MRKR${ref_num}"             \
                                          -fill $StateUI(marker,$num,color)  \
                                          -font analyzer_tiny                \
                                          -anchor nw                         \
                                          -tags marker_name

      # Marker freq
      # Not displayed for a horizontal marker
      
      # Marker magnitude
      canvas ${tile_path}.anlz.marker_mag$num -height 15 -width 75 -bg blue

      ${tile_path}.anlz.marker_mag$num create  text  0  0                    \
                                          -text ""                           \
                                          -fill $StateUI(marker,$num,color)  \
                                          -font analyzer_tiny                \
                                          -anchor nw                         \
                                          -tags marker_mag

    }
      
    if { $StateUI(marker,$num,state) == "ON" || \
                                    $StateUI(marker,$num,state_acp) == "ON" } \
    {
      update_marker_mag  $tile_path  $num
    }
    
    # Call update to 'place' or 'place forget' the marker
    update_marker_position  $tile_path  $num
  }


  #***************************************************************************
  #
  #        Name: set_marker_x
  #
  #  Parameters: widget   - tile path of the widget
  #              num      - marker number
  #              x_coord  - x coordinate of the cursor
  #              x_offset - offset used to compensate for the marker block 
  #                         area
  #
  # Description: Sets the 'x' value of the marker
  #
  #***************************************************************************
  proc update_marker_x {widget  num  x_coord } \
  {
    variable  StateUI

    set StateUI(marker,$num,mouse_x)  $x_coord

    set cmd "::Anlz_MarkersUI::update_marker_x_core $widget $num"

    after cancel $cmd
    after idle   $cmd
  }
  
  proc update_marker_x_core {widget  num  } \
  {
    variable  StateUI

    set_marker_x  $widget  $num  $StateUI(marker,$num,mouse_x)
  }
  
  proc set_marker_x { widget  num  x_coord { x_offset  0 } } \
  {
    Debug::Puts AnalyzerUI 2 "set_marker_x:  $widget,  $num,  $x_coord,  $x_offset"

    variable  StateUI
    
    set tile_path  [get_tile  $widget]

    if { ![winfo exists ${tile_path}.anlz] } \
    {
      return
    }
    
    set root_x  [winfo rootx ${tile_path}.anlz]
    
    if {$x_coord == "" } \
    {
      if { $::Analyzer::State(span) == "0 SPAN" } \
      {
        # Get the x position of the present marker time
        set time_m  $StateUI(marker,$num,time)

        # Get the X position of the present marker time
        set x_coord  [Analyzer::time_to_X  ${tile_path}.anlz  $time_m]
      } \
      else \
      {
        # Get the x position of the present marker frequency
        set freq_m  $StateUI(marker,$num,freq)
      
        set x_coord  [Analyzer::freq_to_X  ${tile_path}.anlz  $freq_m]
      }

      set x_coord [expr  int($x_coord + $root_x)]
    }
    
    # Get the dimensions of the trace widget
    set trace_width   [${tile_path}.anlz cget -width]

    # Calculate the X position delta
    set x_move  [expr $x_coord - $x_offset - $root_x] 
   
    # Limit the X position
    if { $x_move < 0 } \
    {
      set x_move 0
    } \
    elseif { $x_move > $trace_width } \
    {
      set x_move  $trace_width
    }

    # Use the marker time value in Zero span
    if { $::Analyzer::State(span) == "0 SPAN" } \
    {
      # Get the frequency of the X position
      set time_m  [Analyzer::X_to_time  ${tile_path}.anlz  $x_move]

      # Set the time of the marker
      set StateUI(marker,$num,time)  [format "%.2f" $time_m]
    } \
    else \
    {
      # Get the frequency of the X position
      set freq_m  [Analyzer::X_to_freq  ${tile_path}.anlz  $x_move]

      # Set the frequency of the marker
      set StateUI(marker,$num,freq)  [format "%.6f" $freq_m]
    }
    
    # Update the position of the marker
    update_marker_position  $tile_path  $num
  }


  #***************************************************************************
  #
  #        Name: set_marker_y
  #
  #  Parameters: widget   - tile path of the widget
  #              num      - marker number
  #              y_coord  - y coordinate of the cursor
  #              y_offset - offset used to compensate for the marker block 
  #                         area
  #
  # Description: Sets the 'y' value of the marker
  #
  #***************************************************************************
  proc update_marker_y {widget  num  y_coord } \
  {
    variable  StateUI
    set StateUI(marker,$num,mouse_y) $y_coord
    set cmd "::Anlz_MarkersUI::update_marker_y_core $widget $num"
    after cancel $cmd
    after idle $cmd
  }
  proc update_marker_y_core {widget  num  } \
  {
    variable  StateUI
    
    set_marker_y  $widget  $num  $StateUI(marker,$num,mouse_y)
  }
  proc set_marker_y { widget  num  y_coord  { y_offset  0 } } \
  {
    Debug::Puts AnalyzerUI 2 "set_marker_y:  $widget,  $num,  $y_coord,  $y_offset"
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]

    if { ![winfo exists ${tile_path}.anlz] } \
    {
      return
    }

    set root_y  [winfo rooty ${tile_path}.anlz]

    # Get the present marker level
    set db_m  $StateUI(marker,$num,mag)
   
    if { $y_coord == "" } \
    {
      set y_coord  [Analyzer::db_to_Y  ${tile_path}.anlz  $db_m]
      
      set y_coord [expr  int($y_coord + $root_y)]
    }
    
    # Get the dimensions of the trace widget
    set trace_height  [${tile_path}.anlz cget -height]

    # Calculate the Y position delta
    set y_m  [expr $y_coord - $y_offset - $root_y]
    
    # Limit the Y position
    if { $y_m < 0 } \
    {
      set y_m  0
    }
    
    if { $y_m > $trace_height } \
    {
      set y_m  $trace_height
    }

    # Get the level of the Y position
    set db_m  [Analyzer::Y_to_db  ${tile_path}.anlz  $y_m]

    # Set the marker to the new level
    set StateUI(marker,$num,mag)  [format  "%.2f"  $db_m]
    
    # Update the position of the marker
    update_marker_position  $tile_path  $num
  }


  #***************************************************************************
  #
  #        Name: update_marker_position
  #
  #  Parameters: widget  - tile path of the widget
  #              num     - marker number
  #
  # Description: Moves the marker
  #
  #***************************************************************************
  proc update_marker_position { widget  num  args } \
  {
    set cmd "Anlz_MarkersUI::update_marker_position_core $widget $num"
    
    after cancel  $cmd
    after idle  $cmd
  }
  
  proc update_marker_position_core { widget  num } \
  {
    Debug::Puts AnalyzerUI 2 "update_marker_position:  $widget,  $num"
    
    variable  StateUI

    if { ![winfo exists $widget] } { return }
    
    set tile_path  [get_tile  $widget]

    if { ![winfo exists ${tile_path}.anlz.marker$num] } \
    {
      return
    }
    
 
    # Move the marker only if it is ON
    if { $StateUI(marker,$num,state) == "ON" || $StateUI(marker,$num,state_acp) == "ON" } \
    {
      # Vertically oriented markers and ACP markers
      if { $StateUI(marker,$num,orient) == "VERT" || $::AnalyzerUI::StateUI(mode) == "ACP" } \
      { 
        # if we are in the process of switching from horizontal to vertical the marker
        # may not exist yet - if so, skip the update
        if ![winfo exists ${tile_path}.anlz.marker_name$num] { return }
        if ![winfo exists ${tile_path}.anlz.marker_freq$num] { return }
        if ![winfo exists ${tile_path}.anlz.marker_mag$num] { return }
        # Get the dimensions of the trace widget
        set trace_height  [${tile_path}.anlz cget -height]
        set trace_width   [${tile_path}.anlz cget -width]
        
        # Use the marker time value in Zero span
        if { $::Analyzer::State(span) == "0 SPAN" } \
        {
          # Get the x position of the present marker time
          set time_m  $StateUI(marker,$num,time)

          # Get the X position of the present marker time
          set marker_x  [Analyzer::time_to_X  ${tile_path}.anlz  $time_m]
        } \
        else \
        {
          # Get the x position of the present marker frequency
          set freq_m  $StateUI(marker,$num,freq)
        
          set marker_x  [Analyzer::freq_to_X  ${tile_path}.anlz  $freq_m]
        }
        
        # Readjust for the marker width at each side of the trace widget
        if { $marker_x >= 3 } \
        {
          set marker_x  [expr $marker_x - 2]
        }
        
        if { $marker_x >= [expr $trace_width - 3] } \
        {
          set marker_x  [expr $trace_width - 3]
        }
        
        # Move the 'name' to the left side of the marker line if the marker 
        # is closer than 80 pixels to the right side of the Audio Analyzer 
        # widget, else move it to the right
        if { $marker_x > [expr $trace_width - 80] } \
        {
          set marker_name_offset  -60
        } \
        else \
        {
          set marker_name_offset  5
        }

        # Place the marker line
        place ${tile_path}.anlz.marker$num  -x $marker_x  -y 0
        
        if { $::AnalyzerUI::StateUI(mode) == "ACP" } \
        {
          # Adjust the Marker block for the ACP mode display
          # i.e. compensate the horizontal position for no highlightthickness
          # Place the marker block
          place ${tile_path}.anlz.marker_blk$num   \
                         -x [expr $marker_x - 10]  \
                         -y 0
        } \
        else \
        {
          # Place the marker block
          place ${tile_path}.anlz.marker_blk$num   \
                         -x [expr $marker_x - 13]  \
                         -y 0
        }
        
        # Adjust the 'y' position for the two different Analyzer sizes: 
        # Quarter and Full
        set offset  [expr  $num + 1]
        set adjust  0
        
        if { [winfo class  $tile_path] == "QUARTER_ANLZ" } \
        {
          # Adjust the offset of the name, freq, and magnitude text
          if { $num > 2 } \
          {
            set offset  3
          }

          set adjust  7
        } \
        elseif { [winfo class  $tile_path] == "FULL_ANLZ" } \
        {
          # Adjust the offset of the name, freq, and magnitude text
          if { $num > 3 } \
          {
            set offset  4
          }
        }
        
        # Display the Marker
        # Place the marker name
        place ${tile_path}.anlz.marker_name$num                   \
                       -x [expr $marker_x + $marker_name_offset]  \
                       -y [expr $trace_height - ((5 - $offset) * 45) + $adjust]
        
        # Place the marker frequency label
        place ${tile_path}.anlz.marker_freq$num                   \
                       -x [expr $marker_x + $marker_name_offset]  \
                       -y [expr ($trace_height - ((5 - $offset) * 45)) + 15 + $adjust]
        
        # Place the marker magnitude label
        place ${tile_path}.anlz.marker_mag$num                    \
                       -x [expr $marker_x + $marker_name_offset]  \
                       -y [expr ($trace_height - ((5 - $offset) * 45)) + 30 + $adjust]
        
        
        # Use the marker time value in Zero span
        if { $::Analyzer::State(span) == "0 SPAN" } \
        {
          # Display the time value
          ${tile_path}.anlz.marker_freq$num itemconfig marker_freq  \
                                       -text "$StateUI(marker,$num,time) msec"
        } \
        else \
        {
          # Display the frequency value
          ${tile_path}.anlz.marker_freq$num itemconfig marker_freq  \
                                        -text "$StateUI(marker,$num,freq) MHz"
        }
      } \
      elseif { $StateUI(marker,$num,orient) == "HORIZ" } \
      {
        # if we are in the process of switching from horizontal to vertical the marker
        # may not exists yet - if so, skip the update
        if ![winfo exists ${tile_path}.anlz.marker_name$num] { return }
        if ![winfo exists ${tile_path}.anlz.marker_mag$num] { return }

        # Horizontally oriented markers
        # Get the dimensions of the trace widget
        set trace_height  [${tile_path}.anlz cget -height]
        set trace_width   [${tile_path}.anlz cget -width]

        # Get the y position of the present marker magnitude
        set db_m  $StateUI(marker,$num,mag)

        set marker_y  [Analyzer::db_to_Y  ${tile_path}.anlz  $db_m]
        
        # Readjust for the marker width at the top and bottom of the 
        # trace widget
        if { $marker_y >= 3 } \
        {
          set marker_y  [expr $marker_y - 2]
        }
        
        if { $marker_y >= [expr $trace_height - 3] } \
        {
          set marker_y  [expr $trace_height - 3]
        }
       
        # Move the 'name' below the marker line if the marker is
        # closer than 30 pixels to the top of the Audio Analyzer 
        # widget, else move it above the marker line
        if { $marker_y < 30 } \
        {
          set marker_name_offset  5
        } \
        else \
        {
          set marker_name_offset  -30
        }
        
        # Place the marker components
        place ${tile_path}.anlz.marker$num  -x 0  -y $marker_y
       
        place ${tile_path}.anlz.marker_blk$num  \
                         -x 0                   \
                         -y [expr $marker_y - 13]
       
        # Adjust the 'y' position for the two different Analyzer sizes: 
        # Quarter and Full
        set adjust  0
        
        if { [winfo class  $tile_path] == "QUARTER_ANLZ" } \
        {
          #set adjust  5
        } \
        elseif { [winfo class  $tile_path] == "FULL_ANLZ" } \
        {
          
        }

        place ${tile_path}.anlz.marker_name$num    \
                       -x [expr ($num * 55) + 18]  \
                       -y [expr $marker_y + $marker_name_offset + $adjust]
       
        place ${tile_path}.anlz.marker_mag$num     \
                       -x [expr ($num * 55) + 18]  \
                       -y [expr $marker_y + $marker_name_offset + 15 + $adjust]
      }
      
      # Update the marker magnitude
      update_marker_mag  $widget  $num
    }

    if { $StateUI(marker,$num,state) == "OFF" && $StateUI(marker,$num,state_acp) == "OFF" } \
    {
      foreach element "${tile_path}.anlz.marker_mag$num ${tile_path}.anlz.marker_freq $num${tile_path}.anlz.marker$num ${tile_path}.anlz.marker_blk$num ${tile_path}.anlz.marker_name$num" \
      {
           if [winfo exists $element] { place forget $element}
      }
    }
  }


  #***************************************************************************
  #
  #        Name: update_marker_mag
  #
  #  Parameters: widget - tile path of the widget
  #              num    - marker number
  #
  # Description: Gets the Marker magnitude values from the trace data
  #
  #***************************************************************************
  proc update_marker_mag { widget  num  args } \
  {
     set cmd "::Anlz_MarkersUI::update_marker_mag_core $widget $num"
     after cancel $cmd
     after idle $cmd
  }
  proc update_marker_mag_core { widget  num  } \
  {
    Debug::Puts AnalyzerUI 2 "update_marker_mag:  $widget,  $num"
    
    variable  StateUI
        
    set tile_path  [get_tile  $widget]

    if { ![winfo exists ${tile_path}.anlz.marker_mag$num] } \
    {
      return
    }
    
    if { $StateUI(marker,$num,state) == "ON" || $StateUI(marker,$num,state_acp) == "ON" } \
    {
      # Vertically oriented markers
      if { $StateUI(marker,$num,orient) == "VERT" || $::AnalyzerUI::StateUI(mode) == "ACP" } \
      { 
        # Get the trace widget width
        set trace_width  [${tile_path}.anlz cget -width]

        # Convert to a 'double' value for calculations
        set trace_width  [expr double($trace_width)]
        
        # Determine the position of the Marker
        if { $::Analyzer::State(span) == "0 SPAN" } \
        {
          # Use the marker time value in Zero span
          set time_m  $StateUI(marker,$num,time)
          
          set x_m  [Analyzer::time_to_X  ${tile_path}.anlz  $time_m]
        } \
        else \
        {
          # Use the marker frequency value otherwise
          set freq_m  $StateUI(marker,$num,freq)
       
          set x_m  [Analyzer::freq_to_X  ${tile_path}.anlz  $freq_m]
        }
        
        # Convert to a 'double' value for calculations
        set x_m  [expr double($x_m)]
        
        # Get the trace data array from the Analyzer
        upvar 0 StateUI(array) array
        
        # Get the number of elements in the array, minus 1
        set max_index  [llength  $array]
        
        set max_index  [expr $max_index - 1]

        # Convert the X position to an array index
        set index_m  [expr ($x_m / $trace_width) * $max_index]
        
        # Convert to an 'int' value and add 1
        set index_m  [expr int($index_m)]
        
        set index_m  [expr $index_m + 1]

        if { $index_m > $max_index } \
        {
          set index_m  $max_index
        }
        
        # Set the magnitude value
        set x  [lindex  $array  $index_m]
        
        if { $x == "" } \
        {
          set StateUI(marker,$num,mag)  $x
        } \
        else \
        {
          set StateUI(marker,$num,mag)  [format  %4.1f  $x]
        }
          
        if { $::AnalyzerUI::StateUI(mode) == "ACP" } \
        {
          update_marker_mag_acp  $tile_path
        } \
        else \
        {
          ${tile_path}.anlz.marker_mag$num itemconfig marker_mag  \
                                         -text "$StateUI(marker,$num,mag) dBm"

          update_marker_diff
        }
      }
      
      if { $StateUI(marker,$num,orient) == "HORIZ" } \
      {
        # Horizontally oriented markers
        ${tile_path}.anlz.marker_mag$num itemconfig marker_mag  \
                                         -text "$StateUI(marker,$num,mag) dBm"

        update_marker_diff
      }
    }
    
    if { $StateUI(marker,$num,state) == "OFF" && $StateUI(marker,$num,state_acp) == "OFF" } \
    {
      set StateUI(marker,$num,mag)  0.00
    }
  }


  #***************************************************************************
  #
  #        Name: get_marker_mag
  #
  #  Parameters: real_num - real marker number (starts at 1 for Marker 1)
  #
  # Description: Gets the presnt Marker magnitude value
  #              NOTE: This procedure is used as a remote command
  #
  #***************************************************************************
  proc get_marker_mag { real_num  } \
  {
    Debug::Puts AnalyzerUI 2 "get_marker_mag:  $real_num"
    
    variable  StateUI
        
    set num  [expr  $real_num - 1]
    
    set magnitude  $StateUI(marker,$num,mag)
 
    return  $magnitude
  }


  #***************************************************************************
  #
  #        Name: set_marker_spin_move
  #
  #  Parameters: widget - tile path of the Marker block area
  #              num    - marker number
  #
  # Description: Changes the Marker block color, and takes Spin knob events
  #              to move the marker
  #
  #***************************************************************************
  proc set_marker_spin_move { widget  num  } \
  {
    Debug::Puts AnalyzerUI 2 "set_marker_spin_move:  $widget,  $num"
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]
    
    set marker_blk  ${tile_path}.anlz.marker_blk$num

    set present_color  [$marker_blk cget  -bg]
    
    if { $present_color == $StateUI(marker,$num,color) } \
    {
      set_marker_color  $tile_path  $num  white
      
      # Vertically oriented markers
      if { $StateUI(marker,$num,orient) == "VERT" } \
      { 
        # Bind the <Right> event to decrement the Marker X position by 2  (includes <<Spin_up>>)
        bind  $marker_blk  <Right>  "Anlz_MarkersUI::set_marker_x  $marker_blk  $num {} -2; break"
                        
        # Bind the <Left> event to increment the Marker X position by 2  (includes <<Spin_down>>)
        bind  $marker_blk  <Left>  "Anlz_MarkersUI::set_marker_x  $marker_blk  $num {} 2; break"
      }
      
      # Horizontally oriented markers
      if { $StateUI(marker,$num,orient) == "HORIZ" } \
      {
        # Bind the <Right> event to increment the Marker Y position by 2  (includes <<Spin_up>>)
        bind  $marker_blk  <Right>  "Anlz_MarkersUI::set_marker_y  $marker_blk  $num {} 2; break"
                        
        # Bind the <Left> event to decrement the Marker Y position by 2  includes <<Spin_down>>)
        bind  $marker_blk  <Left>  "Anlz_MarkersUI::set_marker_y  $marker_blk  $num {} -2; break"
      }
    } \
    else \
    {
      set_marker_color  $tile_path  $num  normal

      # Unbind the <Right> and <Left> events from the Marker block
      bind  $marker_blk  <Right>  {}
      bind  $marker_blk  <Left>   {}
    }
  }
  
  
  #***************************************************************************
  #
  #        Name: set_marker_color
  #
  #  Parameters: widget - tile path of the Marker block area
  #              num    - marker number
  #              mode   - mode of the color change
  #                       white
  #                       normal - StateUI color value
  #
  # Description: Changes the Marker color
  #
  #***************************************************************************
  proc set_marker_color { widget  num  mode } \
  {
    Debug::Puts AnalyzerUI 2 "set_marker_color:  $widget,  $num,  $mode"
    
    variable  StateUI

    set tile_path  [get_tile  $widget]
    
    if { ![winfo exists ${tile_path}.anlz.marker$num] } \
    {
      return
    }

    # Change the marker only if it is ON
    if { $StateUI(marker,$num,state) == "ON" || \
                                    $StateUI(marker,$num,state_acp) == "ON" } \
    {
      # Vertically oriented markers
      if { $StateUI(marker,$num,orient) == "VERT" } \
      { 
        set marker       ${tile_path}.anlz.marker$num
        set marker_blk   ${tile_path}.anlz.marker_blk$num
        set marker_name  ${tile_path}.anlz.marker_name$num
        set marker_freq  ${tile_path}.anlz.marker_freq$num
        set marker_mag   ${tile_path}.anlz.marker_mag$num
       
        switch -- $mode \
        {
          "white"    { $marker      config  -bg  white
                       $marker_blk  config  -bg  white
       
                       $marker_name  itemconfig  marker_name  -fill white
                       $marker_freq  itemconfig  marker_freq  -fill white
                       $marker_mag   itemconfig  marker_mag   -fill white
                       focus $marker_blk
                       bind $marker_blk <Motion> "Anlz_MarkersUI::update_marker_x  %W  $num  %X"
                       bind $marker <Motion> "Anlz_MarkersUI::update_marker_x  %W  $num  %X"
                     }
                       
          "normal"   { $marker      config  -bg  $StateUI(marker,$num,color)
                       $marker_blk  config  -bg  $StateUI(marker,$num,color)
       
                       $marker_name  itemconfig  marker_name  \
                                             -fill $StateUI(marker,$num,color)
                                                
                       $marker_freq  itemconfig  marker_freq  \
                                             -fill $StateUI(marker,$num,color)
                                                
                       $marker_mag   itemconfig  marker_mag  \
                                             -fill $StateUI(marker,$num,color) 
                                             
                       # Call the 'rebuildCursor' procedure to remap the
                       # cursor focus.
                       # This is done in case one Marker is mapped directly
                       # over another.
                       # When the covering Marker is moved, the Marker below
                       # will be able to get focus.
                       Cursor::rebuildControl  $tile_path  MAP
                       bind $marker_blk <Motion> {}
                       bind $marker <Motion> {}
                     }
        }
      }
      
      # Horizontally oriented markers
      if { $StateUI(marker,$num,orient) == "HORIZ" } \
      {
        set marker       ${tile_path}.anlz.marker$num
        set marker_blk   ${tile_path}.anlz.marker_blk$num
        set marker_name  ${tile_path}.anlz.marker_name$num
        set marker_mag   ${tile_path}.anlz.marker_mag$num

        switch -- $mode \
        {
          "white"    { $marker      config  -bg  white
                       $marker_blk  config  -bg  white
     
                       $marker_name  itemconfig  marker_name  -fill white
                       $marker_mag   itemconfig  marker_mag   -fill white
                       focus $marker_blk
                       bind $marker_blk <Motion> "Anlz_MarkersUI::set_marker_y  %W  $num  %Y"
                       bind $marker <Motion> "Anlz_MarkersUI::set_marker_y  %W  $num  %Y"
                      }
                     
          "normal"   { $marker      config  -bg  $StateUI(marker,$num,color)
                       $marker_blk  config  -bg  $StateUI(marker,$num,color)
      
                       $marker_name  itemconfig  marker_name  \
                                           -fill $StateUI(marker,$num,color)
                                              
                       $marker_mag   itemconfig  marker_mag  \
                                           -fill $StateUI(marker,$num,color) 
                                           
                       # Call the 'rebuildCursor' procedure to remap the
                       # cursor focus.
                       # This is done in case one Marker is mapped directly
                       # over another.
                       # When the covering Marker is moved, the Marker below
                       # will be able to get focus.
                       bind $marker_blk <Motion> {}
                       bind $marker <Motion> {}
                       Cursor::rebuildControl  $tile_path  MAP }
        }
      }
    }
  }


  #***************************************************************************
  #
  #        Name: toggle_diff_units
  #
  #  Parameters: diff - index of difference field
  #
  # Description: Toggles the units of the difference readings.
  #
  #***************************************************************************
  proc toggle_diff_units { diff } \
  {
    Debug::Puts AnalyzerUI 2 "toggle_diff_units:  $diff"
    
    variable  StateUI
    
    if { $StateUI(diff_units,$diff) == "dB" } \
    {
      set StateUI(diff_units,$diff)  "MHz"
    } \
    elseif { $StateUI(diff_units,$diff) == "MHz" } \
    {
      set StateUI(diff_units,$diff)  "msec"
    } \
    else \
    {
      set StateUI(diff_units,$diff)  "dB"
    }
  }


  #***************************************************************************
  #
  #        Name: update_diff_units
  #
  #  Parameters: widget - tile path of the invoking widget
  #              diff   - index of difference field
  #
  # Description: Update the display to the present difference units
  #
  #***************************************************************************
  proc update_diff_units { widget  diff  args } \
  {
    Debug::Puts AnalyzerUI 2 "update_diff_units:  $widget,  $diff,  $args"
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]
    
    grid remove  ${tile_path}.lbl_diff${diff}_db
    grid remove  ${tile_path}.lbl_diff${diff}_freq
    grid remove  ${tile_path}.lbl_diff${diff}_msec
    
    switch -- $StateUI(diff_units,$diff) \
    {
      "dB"    { Screens::position_widget  ${tile_path}.lbl_diff${diff}_db   }
    
      "MHz"   { Screens::position_widget  ${tile_path}.lbl_diff${diff}_freq }
    
      "msec"  { Screens::position_widget  ${tile_path}.lbl_diff${diff}_msec }
    }
  }


  #***************************************************************************
  #
  #        Name: update_marker_diff
  #
  #  Parameters: 
  #
  # Description: Displays between two markers that are selected
  #
  #***************************************************************************
  proc update_marker_diff { } \
  {
    Debug::Puts AnalyzerUI 2 "update_marker_diff"
    
    variable  StateUI

    # First difference
    foreach  { diff }  { diff1  diff2 } \
    {
      # Subtract 1 to get the actual marker number from the menu selection
      set num_1  [expr  $StateUI(marker,$diff,1) - 1]
      set num_2  [expr  $StateUI(marker,$diff,2) - 1]
    
      if {$StateUI(marker,$num_1,mag) != "" && $StateUI(marker,$num_2,mag) != "" } \
      {
        set result_db  [expr $StateUI(marker,$num_1,mag) - $StateUI(marker,$num_2,mag)]
          
        set StateUI(marker,$diff,db)  [format "%.2f"  $result_db]
      } \
      else \
      {
        set result_db  ""

        set StateUI(marker,$diff,db)  ""
      }
    
      set result_freq  [expr $StateUI(marker,$num_1,freq) - $StateUI(marker,$num_2,freq)]

      set result_time  [expr $StateUI(marker,$num_1,time) - $StateUI(marker,$num_2,time)]

      set StateUI(marker,$diff,freq)  [format "%.6f"  $result_freq]
      
      set StateUI(marker,$diff,time)  [format "%.2f"  $result_time]
    }
  }
  

  #***************************************************************************
  #
  #        Name: toggle_cable_flt_units
  #
  #  Parameters: 
  #
  # Description: Toggles the units of the cable fault reading.
  #
  #***************************************************************************
  proc toggle_cable_flt_units { } \
  {
    Debug::Puts AnalyzerUI 2 "toggle_cable_flt_units"
    
    variable  StateUI
    
    if { $StateUI(marker,cable_flt,units) == "ft" } \
    {
      set StateUI(marker,cable_flt,units)  "m"
    } \
    else \
    {
      set StateUI(marker,cable_flt,units)  "ft"
    }
  }


  #***************************************************************************
  #
  #        Name: update_cable_flt_units
  #
  #  Parameters: widget - tile path of the invoking widget
  #
  # Description: Update the display to the present cable fault units
  #
  #***************************************************************************
  proc update_cable_flt_units { widget  args } \
  {
    Debug::Puts AnalyzerUI 2 "update_cable_flt_units:  $widget,  $args"
    
    variable  StateUI
    
    set tile_path  [get_tile  $widget]
    
    grid remove  ${tile_path}.lbl_cable_flt_ft
    grid remove  ${tile_path}.lbl_cable_flt_m

    
    switch -- $StateUI(marker,cable_flt,units) \
    {
      "ft"  { Screens::position_widget  ${tile_path}.lbl_cable_flt_ft }
    
      "m"   { Screens::position_widget  ${tile_path}.lbl_cable_flt_m  }
    }
  }


  #***************************************************************************
  #
  #        Name: set_velocity_factor
  #
  #  Parameters: 
  #
  # Description: Sets the velocity factor based on the cable type selected
  #
  #***************************************************************************
  proc set_velocity_factor { } \
  {
    Debug::Puts AnalyzerUI 2 "set_velocity_factor"
    
    variable  StateUI
    
    switch -- $StateUI(cable_material) \
    {
      "Solid Polyethylene"       { set StateUI(vel_factor)  65.9 }

      "Foam Polyethylene"        { set StateUI(vel_factor)  80.0 }

      "Foam Polystyrene"         { set StateUI(vel_factor)  91.0 }

      "Air Space Polyethylene"   { set StateUI(vel_factor)  86.0 }

      "Solid Teflon"             { set StateUI(vel_factor)  69.4 }

      "Air Space Teflon"         { set StateUI(vel_factor)  87.0 }

      "RG-58 A/U w/solid poly"   { set StateUI(vel_factor)  65.9 } 
      
      default                    { set StateUI(vel_factor)  0.00 }
    }
  }


  #***************************************************************************
  #
  #        Name: update_cable_fault
  #
  #  Parameters: 
  #
  # Description: Calculates the cable fault for the selected markers
  #
  #***************************************************************************
  proc update_cable_fault { } \
  {
    Debug::Puts AnalyzerUI 2 "update_cable_fault"
    
    variable  StateUI
    
    set ft_to_meter  0.3047879
    
    set vel_factor   $StateUI(vel_factor)
    
    if { $vel_factor >= 1.0 } \
    {
      set vel_factor  [expr $vel_factor / 100.0]
    }

    # Subtract 1 to get the actual marker number from the menu selection
    
    set freq_1  $StateUI(marker,cable_flt,start)
    set freq_2  $StateUI(marker,cable_flt,stop)

    set freq_delta  [expr abs($freq_1 - $freq_2)]
    
    if { $freq_delta == 0.0 } \
    { 
      set StateUI(marker,cable_flt,value,ft)  0.00
    
      set StateUI(marker,cable_flt,value,m)   0.00
    } \
    else \
    {
      set cable_flt_ft  [expr (492.0 * $vel_factor) / $freq_delta]
    
      set cable_flt_m   [expr $cable_flt_ft * $ft_to_meter]

    
      set StateUI(marker,cable_flt,value,ft)  [format "%.2f" $cable_flt_ft]
    
      set StateUI(marker,cable_flt,value,m)   [format "%.2f" $cable_flt_m]
    }
  }
  proc update_cable_fault_trace { args } \
  {
     set cmd [namespace code update_cable_fault]
     after cancel $cmd
     after 100 $cmd
  }
  proc update_cable_fault_markers { args } \
  {
     set cmd [namespace code update_cable_fault_markers_core]
     after cancel $cmd
     after idle $cmd
  }
  proc update_cable_fault_markers_core {} \
  {
     variable StateUI
     set marker_1  [expr  $StateUI(marker,cable_flt,1) - 1]
     set marker_2  [expr  $StateUI(marker,cable_flt,2) - 1]
    
     set StateUI(marker,cable_flt,start) $StateUI(marker,$marker_1,freq)
     set StateUI(marker,cable_flt,stop)  $StateUI(marker,$marker_2,freq)
     
  }
  proc hook_up { } \
  {
     set ns [namespace current]
     set cmd  ${ns}::update_cable_fault_trace
     set cmd2 ${ns}::update_cable_fault_markers
     trace variable ${ns}::StateUI(vel_factor)             w $cmd
     trace variable ${ns}::StateUI(marker,cable_flt,start) w $cmd
     trace variable ${ns}::StateUI(marker,cable_flt,stop)  w $cmd
     trace variable ${ns}::StateUI(marker,cable_flt,1)     w $cmd2
     trace variable ${ns}::StateUI(marker,cable_flt,2)     w $cmd2
  }
  hook_up

  #***************************************************************************
  #
  #  Marker ACP mode procedures
  #
  #***************************************************************************
  #***************************************************************************
  #
  #        Name: update_marker_freq_acp
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Updates the Marker positions in the ACP mode
  #
  #***************************************************************************
  proc update_marker_freq_acp { tile_path  args } \
  {
    Debug::Puts AnalyzerUI 2 "update_marker_freq_acp:  $tile_path,  $args"
    
    variable StateUI
    
    if { $::AnalyzerUI::StateUI(mode) == "ACP" &&  \
                                  $::AnalyzerUI::StateUI(acp_mode) == "ON" } \
    {
      # Get the Analyzer center freq
      set center_freq  $::Receiver::State(freq)
      
      # If the value is nulled out because of the Spinentry behavior return
      if { $center_freq == "" } \
      {
        return
      }

      # Configure Marker 1: freq at the center freq
      set StateUI(marker,0,freq)  $center_freq
      
      # Configure Marker 2: freq at the center freq + 12.5 kHz
      set StateUI(marker,1,freq)  [format "%.6f" [expr $center_freq + 0.0125]]
      
      # Configure Marker 3: freq at the center freq - 12.5 kHz
      set StateUI(marker,2,freq)  [format "%.6f" [expr $center_freq - 0.0125]]
      
      # Update the position of the markers
      update_marker_position  $tile_path  0
      update_marker_position  $tile_path  1
      update_marker_position  $tile_path  2
    }
  }


  #***************************************************************************
  #
  #        Name: update_marker_mag_acp
  #
  #  Parameters: tile_path - tile path of the invoking widget
  #
  # Description: Updates the Marker magnitudes in the ACP mode
  #
  #***************************************************************************
  proc update_marker_mag_acp { tile_path  args } \
  {
    Debug::Puts AnalyzerUI 2 "update_marker_mag_acp:  $tile_path,  $args"
    
    variable StateUI
    
    if { $::AnalyzerUI::StateUI(mode) == "ACP" &&  $::AnalyzerUI::StateUI(acp_mode) == "ON" } \
    {
      set reference  $StateUI(marker,0,mag)
      
      # Configure Marker 1: magnitude 0 dBc
      set StateUI(marker,0,mag)  [expr $StateUI(marker,0,mag) - $reference]

      ${tile_path}.anlz.marker_mag0 itemconfig marker_mag  \
                                             -text "$StateUI(marker,0,mag) dB"

      # Configure Marker 2: magnitude relative to Marker 1 and units in dBc
      set StateUI(marker,1,mag)  [expr $StateUI(marker,1,mag) - $reference]
      
      ${tile_path}.anlz.marker_mag1 itemconfig marker_mag  \
                                            -text "$StateUI(marker,1,mag) dBc"
      
      # Configure Marker 3: magnitude relative to Marker 1 and units in dBc
      set StateUI(marker,2,mag)  [expr $StateUI(marker,2,mag) - $reference]

      ${tile_path}.anlz.marker_mag2 itemconfig marker_mag  \
                                            -text "$StateUI(marker,2,mag) dBc"
    }
  }


  #****************************************************************************
  #
  #        Name: getSaveState
  #
  # Parameters:  none
  #
  # Description: This function returns a list of TCL commands which can be used
  #              to set the AnalyzerUI tiles to its current state
  #
  #****************************************************************************
  proc getSaveState { } \
  {
    variable  StateUI
    variable  marker_list

    set returnList  {}
    
    # Markers
    foreach { num } $marker_list \
    {
      append returnList  "set ::Anlz_MarkersUI::StateUI(marker,$num,state)  \
                                              {$StateUI(marker,$num,state)}\n"
                                              
      append returnList  "set ::Anlz_MarkersUI::StateUI(marker,$num,orient)  \
                                             {$StateUI(marker,$num,orient)}\n"
                                             
      append returnList  "set ::Anlz_MarkersUI::StateUI(marker,$num,freq)  \
                                               {$StateUI(marker,$num,freq)}\n"
                                               
      append returnList  "set ::Anlz_MarkersUI::StateUI(marker,$num,mag)  \
                                                {$StateUI(marker,$num,mag)}\n"

      append returnList  "set ::Anlz_MarkersUI::StateUI(marker,$num,time) \
                                               {$StateUI(marker,$num,time)}\n"
    }    
    
    return [subst $returnList]
  }


  #****************************************************************************
  #
  #        Name: Sync
  #
  #  Parameters: none
  #
  # Description: Write state variables to AnalyzerUI
  #
  #****************************************************************************
  proc Sync { } \
  {
    eval [getSaveState]
  }


};  # End of Anlz_MarkersUI namespace


#****************************************************************************
# This registers the tiles with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable tiles   

  lappend tiles  Anlz_MarkersUI
}

