#!/usr/local/bin/wish
#****************************************************************************
#
# Current Cursor Movement Functions
#     Basic Concepts:
#        - A list of all available cursor locations will be obtained by
#          recursively searching through all child windows which are packed,
#          grided or placed into the widget. This process must be activated
#          AFTER the cursors become visible
#        - The screen will be divided into a coarse grid which can be set by
#          the parameters: ScreenGrid(num_cols) and ScreenGrid(num_rows)
#        - The list of cursors will be searched to set up two arrays. One array
#          will be a link from each screen grid area to a list of cursors located
#          in that grid. The second will be an array which converts a widget name
#          to a grid area coordinate.
#        - All curser locations need to be bound to cause the cursor list to be
#          erased if they become unmapped
#        - If there is no "hard-wired" movement defined for a cursor, the cursor
#          keys will cause a virtual cursor event, which will then trigger a
#          virtual default cursor event. The default cursor event will search for
#          the next best cursor location, and once located, will assign an event
#          on the cursor which will bypass the lookup process. This binding will
#          be erased when the screen changes. This provides a significant speed
#          increase once the movement has been determined.
#     Movement Analysis process:
#        - All cursors in the same grid as the cursor losing focus are checked to
#          see if they are legal candidates for the next cursor. If any qualify,
#          the best cursor of this group is chosen.
#        - If no cursors in the original grid are acceptable, then the grid is
#          walked by rows or columns until a non-empty grid is found. The best cursor
#          in this grid becomes selected as the next cursor.
#     Issues:
#        - Screen mapping may be modified by a top level window or menu covering
#          a currently displayed window. This may cause an undesirable release 
#          of bindings.
#
# Alternative methods
#     Simple Cursor Lists
#        - Build a list of all focusable items
#        - From full list, build  LEFT/RIGHT/UP/DOWN lists
#        - When cursor movement occurs, find the current widget's position
#          in the list and simply move to the next widget in the list
#     Window oriented enhancements
#        - When a screen is created or modified, it knows the placement of
#          main window panes. Building a list of focusable items from these
#          panes may provide a subgrouping of cursor targets which allow a
#          faster identification of where to move to.
#     Smart Widgets
#        - It may be desirable for a widget or pane to control all of the
#          bindings within that window.
#     Cursor DEFAULT movements
#        - It may be desirable to define a DEFAULT set of movements for a 
#          widget which would be set up automatically, and have the curser
#          movements generate both an <<UP>> and an <<UP_DEFAULT>> event.
#          This would allow smart widgets to define bindings to the <<UP>>  
#          and an automatic process to define bindings to <<UP_DEFAULT>>. The
#          concept is that any <<UP>> bindings would block the activation of
#          the <<UP_DEFAULT>> bindings, and the automatic process would only
#          clear the <<_DEFAULT>> bindings.
#
# Regeneration of cursor arrays
#     Automatic regeneration:  All screen widgets which can be made the focus of
#          the cursor are supposed to be assigned the "Cursor" bindtag. When these
#          items MAP or UNMAP from the screen, this signals that the cursor arrays
#          are out of date. It is not advisable to immediately update the arrays
#          since a lot of these signals occur when there are major screen changes.
#          It is desirable to set up a fixed time delay to allow screen events to
#          complete (and reset the delay time with each MAP/UNMAP event), and then
#          update the list during idle time. This blocks the list generation from
#          consuming time while the display is updating, and yet will still try to
#          get the list updated before the user needs it.
#     Manual regeneration:  If the user attempts to move the cursor, and either the
#          widget he is moving from, or the widget he is moving to does not exist,
#          that is an indication that the list needs to be rebuilt. If a screen
#          change event has not occurred recently, this should trigger an immediate
#          rebuild of the list so that the cursor movement can occur. 
#     Manual regeneration issues:  If the user attempts to move the cursor WHILE the
#          screen is being regenerated, it is NOT desirable to try to update the 
#          cursor list. This just wastes time and creates a problem with the stack
#          depth of TCL evaluation objects.
#     Possible Solutions:
#       1) If we could insure that all cursor objects had the "Cursor" bindtag, we
#          could eliminate the manual regeneration. Simply take no action if the list
#          was bad. PROBLEM: Pop up messages may not have the bindtag, and it is not
#          easy to catch the problem.
#       2) We could create a rebuild process that would have master control
#          o  It would need a delay to start when screens rebuild
#          o  If manual rebuild is needed, it would be delayed only if the screen 
#             rebuild process had been started (even if it gets delayed seconds)
#          o  If moving the cursor, and a move to is not available, ignore the move
#
# Additional desired features
#     Cursor History
#        - User Problem: When the user moves from one screen location 
#          to another, he expects the opposite cursor movement to return him to 
#          the original position. An automated screen cursor positioning algorithm
#          does not provide this due to screen layout issues.
#        - Solution Concept: Maintain a short (1-5) list of the last cursor
#          movements (both the cursor key AND screen window). If the user hits
#          the opposite key, then the list is walked backward. If the opposite 
#          key is not hit, then it is saved as a new sequence.
#
#****************************************************************************

# Delete the namespace if it exists to insure that it is clean
foreach ns [namespace children ::] \
{
    if {$ns == "::Cursor"} {
        namespace delete ::Cursor
        break
    }
}

namespace eval ::Cursor {

   variable ScreenGrid
   variable CursorArray_Win
   variable CursorArray_Grid  
   variable cursorlist
   variable recalc_delay 100
   variable BuildState

   set ScreenGrid(num_cols) 8
   set ScreenGrid(num_rows) 20
   set BuildState(.) 0
 
#****************************************************************************
# Name:        defaultSetupCursorArrays
# Parameters:  name   A window on the screen that needs to be checked
#                               for cursor movement
#              direction        cursor direction
# Description: This function is hit as a default when there is a cursor movement
#              made away from a window that is not marked with the "Cursor" 
#              bindtag. The window may be a MENU, or it may be a new window on
#              the screen. No action is taken for MENUs, but a new window will
#              trigger a reset of the default cursor movement process.
#****************************************************************************
 proc defaultSetupCursorArrays { name direction } {
   
   # Basic error checking. If we have a cursor movement away from an illegal
   # window, no default behavior has yet been defined
   if {![winfo exists $name]} {
      if {![winfo exists [focus]]} \
      {
         return
      }
      set name [focus]
   }

   # Avoid processing menus 
   set btags [bindtags $name]
   if { [lsearch -exact $btags "Menu"] != -1 } {
      return
   }

   # Since we are trying to move from a window that does not have a "Cursor"
   # bindtag, the screen has been updated. Force the list to update
   rebuildControl $name MOVEMENT
 }

#****************************************************************************
# Name:        checkCursorArrays
# Parameters:  name   A window on the screen that needs to be checked
#                               for cursor movement
# Description: Check the curser to see if in our list. If not, rebuild list
#****************************************************************************
 proc checkCursorArrays { name } {
   
   # If the name does not exist in the array, we need to wipe out all of the
   # cursor information and start again.
   if { ![info exists ::Cursor::CursorArray_Grid($name)] } {
      set top [winfo toplevel $name]
      rebuildControl $top MOVEMENT

      return ""
   }
   return $name
 }

#****************************************************************************
# Name:        setCursorArrays
# Parameters:  topWin   Toplevel window to set cursors for
# Description: Locate all current cursor locations, and set up arrays to allow
#              quick access to the information
#****************************************************************************
 proc setCursorArrays { topWin } {
   variable BuildState
   
   # Exit if the top level has been removed before we had time to make the list
   if { ![winfo exists $topWin] } {return}

   # Exit if the array is already set up
   if { [info exists ::Cursor::CursorArray_Win] } {
      return
   }

   set cursors [get_all_cursors $topWin]

   # Setup column width and height
   set grid_width [expr int(([winfo screenwidth $topWin] / $::Cursor::ScreenGrid(num_cols)) + 1)]
   set grid_height [expr int(([winfo screenheight $topWin] / $::Cursor::ScreenGrid(num_rows)) + 1)]
   set corner_x [winfo rootx .]
   set corner_y [winfo rooty .]

   foreach widget $cursors {
      set win_x [expr [winfo rootx $widget] - $corner_x]
      set win_y [expr [winfo rooty $widget] - $corner_y]
      set x [expr int($win_x / $grid_width)]
      set y [expr int($win_y / $grid_height)]
      lappend ::Cursor::CursorArray_Win($x.$y) $widget
      set ::Cursor::CursorArray_Grid($widget) [list $x $y $win_x $win_y]
   }

   if {[focus -lastfor $topWin] == $topWin} {
      after idle  "focus [lindex $cursors 0]"
   }

   # Indicate that the arrays are set properly
   set BuildState($topWin) 0
 }

#****************************************************************************
# Name:        get_contained_widgets
# Parameters:  name   Widget name
# Description: Locate all widgets packed/placed/gridded into the given one
#****************************************************************************
 proc get_contained_widgets { name } {
   return "[pack slaves $name] [place slaves $name] [grid slaves $name]"
 }

#****************************************************************************
# Name:        clear_relations
# Parameters:  widget   Widget name
# Description: Clear the list of cursors on this toplevel, and unbind all 
#              cursor relations for this object
#****************************************************************************
 proc clear_relations { widget args } {

   if {![winfo exists $widget]} { return }
   set top [winfo toplevel $widget]

   # zap all cached relations, as they are invalid
   if {[info exists ::Cursor::cursorlist($top)]} {
      foreach c $::Cursor::cursorlist($top) {
         if {[winfo exists $c]} {
            bind $c <<cursor_right>> {}
            bind $c <<cursor_left>>  {}
            bind $c <<cursor_up>>    {}
            bind $c <<cursor_down>>  {}
         }
      }
   }

   # now clear the cached list
   set ::Cursor::cursorlist($top) {}

   if {[info exists ::Cursor::CursorArray_Win]} {
      unset ::Cursor::CursorArray_Win
   }

   if {[info exists ::Cursor::CursorArray_Grid]} {
      unset ::Cursor::CursorArray_Grid
   }
 }

#****************************************************************************
# Name:        get_cursors
# Parameters:  name   Widget name
#              _list    List to put screen cursors in
# Description: Locate all focusable elements (a.k.a. cursors) in this widget. 
#              Also sets the bind tags for the widget
#****************************************************************************
 proc get_cursors { name _list} {
   upvar #0 $_list list

   set children [get_contained_widgets $name]
   foreach widget $children {
      if {[tkFocusOK $widget]}  {
         lappend list $widget
         set btags [bindtags $widget]
         if { [lsearch -exact $btags "Cursor"] == -1 } {
             set temp [linsert $btags 1 "Cursor"]
             bindtags $widget $temp
         }
      }
      get_cursors $widget $_list
   }
 }

#****************************************************************************
# Name:        get_all_cursors 
# Parameters:  toplevel   Name of top level window
# Description: Locate all focusable elements (a.k.a. cursors) in this toplevel
#****************************************************************************
 proc get_all_cursors { toplevel } {

   if [info exists ::Cursor::cursorlist($toplevel)] {
      if {"$::Cursor::cursorlist($toplevel)" != ""} {
         return $::Cursor::cursorlist($toplevel)
      }
   }
   set ::Cursor::cursorlist($toplevel) {}
   get_cursors $toplevel ::Cursor::cursorlist($toplevel)
   
   return $::Cursor::cursorlist($toplevel)
 }

#****************************************************************************
# Name:        move_cursor_up
# Parameters:  name   Widget Name
# Description: Locate the next cursor UP movement location from widget
#****************************************************************************
 proc move_cursor_up {name} {

   # Check and see if the cursor arrays are set up. If we do not get our name
   # back, the arrays are rebuilding and so we should exit
   set temp [checkCursorArrays $name]
   if { $temp != $name } return;

   # Get information about the current cursor
   set arrayVal $::Cursor::CursorArray_Grid($name)
   set grid_x [lindex $arrayVal 0]
   set grid_y [lindex $arrayVal 1]
   set win_x  [lindex $arrayVal 2]
   set win_y  [lindex $arrayVal 3]
   set best_name ""

   set gridList $::Cursor::CursorArray_Win($grid_x.$grid_y)

   # Attempt to find a match in the current grid. A match has to be ABOVE the
   # current location, or at the exact same level and to the LEFT
   set found 0
   set delta_y 99999
   set delta_x 99999
   foreach tWin $gridList {
      if {$tWin == $name } { continue }
      set arrayVal $::Cursor::CursorArray_Grid($tWin)
      set test_y  [lindex $arrayVal 3]
      set temp [expr $win_y - $test_y]
      if { $temp == 0 } {
         set test_x  [lindex $arrayVal 2]
         set temp [expr $test_x - $win_x]
         if { ($temp > 0) && ($temp < $delta_x) } {
            set delta_y 0
            set delta_x $temp
            set found 1
            set best_name $tWin
         }
         
      } elseif { ($temp > 0) && ($temp < $delta_y) } {
         set delta_y $temp
         set found 1
         set best_name $tWin
      }
   }

   if { $found == 0 } {
      # Since there was no match in current grid, the next available grid with
      # a cursor has a legal position
      set index_x $grid_x
      set index_y $grid_y
      while {1} {
         while {1} {
            incr index_y -1
            if {$index_y < 0} {
               break
            }
            if {[info exists ::Cursor::CursorArray_Win($index_x.$index_y)]} {
               set found 1
               set gridList $::Cursor::CursorArray_Win($index_x.$index_y)
               break
            } 
         }
         if { $found } {break}
         set index_y $::Cursor::ScreenGrid(num_rows)
         incr index_x 1
         if { $index_x >= $::Cursor::ScreenGrid(num_cols) } {
            set index_x 0
         }
      }

      # We have a list of possible cursors. Get the stats on the first one
      # and then try to improve on it.
      set best_name [lindex $gridList 0]
      set arrayVal $::Cursor::CursorArray_Grid($best_name)
      set best_x [lindex $arrayVal 2]      
      set best_y [lindex $arrayVal 3]      
      foreach tWin $gridList {
         if {$tWin == $best_name } { continue }
         set arrayVal $::Cursor::CursorArray_Grid($tWin)
         set test_y  [lindex $arrayVal 3]
         if { $test_y == $best_y } {
            set test_x  [lindex $arrayVal 2]
            if {$test_x < $best_x} {
               set best_x $test_x
               set best_name $tWin
            }
         } elseif { $test_y > $best_y } {
            set best_name $tWin
            set best_y $test_y      
         }
      }
   }
  

   if {[string length $best_name] != 0 } {
      if { [tabTestToWindow $best_name] } {
         # set this up as a binding on the widget
         bind $name <<cursor_up>> "::Cursor::tabTestToWindow $best_name; break"
      }
   }
 }

#****************************************************************************
# Name:        move_cursor_down
# Parameters:  name   Widget Name
# Description: Locate the next cursor DOWN movement location from widget
#****************************************************************************
 proc move_cursor_down {name} {

   # Check and see if the cursor arrays are set up. If we do not get our name
   # back, the arrays are rebuilding and so we should exit
   set temp [checkCursorArrays $name]
   if { $temp != $name } return;

   # Get information about the current cursor
   set arrayVal $::Cursor::CursorArray_Grid($name)
   set grid_x [lindex $arrayVal 0]
   set grid_y [lindex $arrayVal 1]
   set win_x  [lindex $arrayVal 2]
   set win_y  [lindex $arrayVal 3]
   set best_name ""

   set gridList $::Cursor::CursorArray_Win($grid_x.$grid_y)

   # Attempt to find a match in the current grid. A match has to be BELOW the
   # current location
   set found 0
   set delta_y 99999
   set delta_x 99999
   foreach tWin $gridList {
      if {$tWin == $name } { continue }
      set arrayVal $::Cursor::CursorArray_Grid($tWin)
      set test_y  [lindex $arrayVal 3]
      set temp [expr $test_y - $win_y]
      if { $temp == 0 } {
         set test_x  [lindex $arrayVal 2]
         set temp [expr $win_x - $test_x]
         if { ($temp > 0) && ($temp < $delta_x) } {
            set delta_y 0
            set delta_x $temp
            set found 1
            set best_name $tWin
         }
         
      } elseif { ($temp > 0) && ($temp < $delta_y) } {
         set delta_y $temp
         set found 1
         set best_name $tWin
      }
   }

   if { $found == 0 } {
      # Since there was no match in current grid, the next available grid with
      # a cursor has a legal position
      set index_x $grid_x
      set index_y $grid_y
      while {1} {
         while {1} {
            incr index_y 1
            if {$index_y >= $::Cursor::ScreenGrid(num_rows)} {
               break
            }
            if {[info exists ::Cursor::CursorArray_Win($index_x.$index_y)]} {
               set found 1
               set gridList $::Cursor::CursorArray_Win($index_x.$index_y)
               break
            } else {
            }
         }
         if { $found } {break}
         set index_y -1
         incr index_x -1
         if { $index_x < 0 } {
            set index_x $::Cursor::ScreenGrid(num_cols)
         }
      }

      # We have a list of possible cursors. Get the stats on the first one
      # and then try to improve on it.
      set best_name [lindex $gridList 0]
      set arrayVal $::Cursor::CursorArray_Grid($best_name)
      set best_x [lindex $arrayVal 2]      
      set best_y [lindex $arrayVal 3]      
      foreach tWin $gridList {
         if {$tWin == $best_name } { continue }
         set arrayVal $::Cursor::CursorArray_Grid($tWin)
         set test_y  [lindex $arrayVal 3]
         if { $test_y == $best_y } {
            set test_x  [lindex $arrayVal 2]
            if {$test_x > $best_x} {
               set best_x $test_x
               set best_name $tWin
            }
         } elseif { $test_y < $best_y } {
            set best_name $tWin
            set best_y $test_y      
         }
      }
   }
  

   if {[string length $best_name] != 0 } {
      if { [tabTestToWindow $best_name] } {
         # set this up as a binding on the widget
         bind $name <<cursor_down>> "::Cursor::tabTestToWindow $best_name; break"
      }
   }
 }


#****************************************************************************
# Name:        move_cursor_right
# Parameters:  name   Widget Name
# Description: Locate the next cursor RIGHT movement location from widget
#****************************************************************************
 proc move_cursor_right {name} {

   # Check and see if the cursor arrays are set up. If we do not get our name
   # back, the arrays are rebuilding and so we should exit
   set temp [checkCursorArrays $name]
   if { $temp != $name } return;

   # Get information about the current cursor
   set arrayVal $::Cursor::CursorArray_Grid($name)
   set grid_x [lindex $arrayVal 0]
   set grid_y [lindex $arrayVal 1]
   set win_x  [lindex $arrayVal 2]
   set win_y  [lindex $arrayVal 3]
   set best_name ""

   set gridList $::Cursor::CursorArray_Win($grid_x.$grid_y)

   # Attempt to find a match in the current grid. A match has to be RIGHT the
   # current location
   set found 0
   set delta_y 99999
   set delta_x 99999
   foreach tWin $gridList {
      if {$tWin == $name } { continue }
      set arrayVal $::Cursor::CursorArray_Grid($tWin)
      set test_x  [lindex $arrayVal 2]
      set test_y  [lindex $arrayVal 3]

      set temp [expr $test_x - $win_x]
      if { $temp == 0 } {
         set temp [expr $win_y - $test_y]
         if { ($temp > 0) && ($temp < $delta_y) } {
            set delta_x 0
            set delta_y $temp
            set found 1
            set best_name $tWin
         }
         
      } elseif { ($temp > 0) && ($temp < $delta_x) } {
         set delta_x $temp
         set found 1
         set best_name $tWin
      }
   }

   if { $found == 0 } {
      # Since there was no match in current grid, the next available grid with
      # a cursor has a legal position
      set index_x $grid_x
      set index_y $grid_y
      while {1} {
         while {1} {
            incr index_x 1
            if {$index_x >= $::Cursor::ScreenGrid(num_cols)} {
               break
            }
            if {[info exists ::Cursor::CursorArray_Win($index_x.$index_y)]} {
               set found 1
               set gridList $::Cursor::CursorArray_Win($index_x.$index_y)
               break
            } else {
            }
         }
         if { $found } {break}
         set index_x -1
         incr index_y 1
         if { $index_y > $::Cursor::ScreenGrid(num_rows) } {
            set index_y 0
         }
      }

      # We have a list of possible cursors. Get the stats on the first one
      # and then try to improve on it.
      set best_name [lindex $gridList 0]
      set arrayVal $::Cursor::CursorArray_Grid($best_name)
      set best_x [lindex $arrayVal 2]      
      set best_y [lindex $arrayVal 3]      
      foreach tWin $gridList {
         if {$tWin == $best_name } { continue }
         set arrayVal $::Cursor::CursorArray_Grid($tWin)
         set test_x  [lindex $arrayVal 2]
         if { $test_x == $best_x } {
            set test_y  [lindex $arrayVal 3]
            if {$test_y > $best_y} {
               set best_y $test_y
               set best_name $tWin
            }
         } elseif { $test_x < $best_x } {
            set best_name $tWin
            set best_x $test_x      
         }
      }
   }
  

   if {[string length $best_name] != 0 } {
      if { [tabTestToWindow $best_name] } {
         # set this up as a binding on the widget
         bind $name <<cursor_right>> "::Cursor::tabTestToWindow $best_name; break"
      }
   }
 }

#****************************************************************************
# Name:        move_cursor_left
# Parameters:  name   Widget Name
# Description: Locate the next cursor LEFT movement location from widget
#****************************************************************************
 proc move_cursor_left {name} {

   # Check and see if the cursor arrays are set up. If we do not get our name
   # back, the arrays are rebuilding and so we should exit
   set temp [checkCursorArrays $name]
   if { $temp != $name } return;

   # Get information about the current cursor
   set arrayVal $::Cursor::CursorArray_Grid($name)
   set grid_x [lindex $arrayVal 0]
   set grid_y [lindex $arrayVal 1]
   set win_x  [lindex $arrayVal 2]
   set win_y  [lindex $arrayVal 3]
   set best_name ""

   set gridList $::Cursor::CursorArray_Win($grid_x.$grid_y)

   # Attempt to find a match in the current grid. A match has to be LEFT of the
   # current location
   set found 0
   set delta_y 99999
   set delta_x 99999
   foreach tWin $gridList {
      if {$tWin == $name } { continue }
      set arrayVal $::Cursor::CursorArray_Grid($tWin)
      set test_x  [lindex $arrayVal 2]
      set test_y  [lindex $arrayVal 3]
      set temp [expr $win_x - $test_x]
      if { $temp == 0 } {
         set temp [expr $test_y - $win_y]
         if { ($temp > 0) && ($temp < $delta_y) } {
            set delta_x 0
            set delta_y $temp
            set found 1
            set best_name $tWin
         }
         
      } elseif { ($temp > 0) && ($temp < $delta_x) } {
         set delta_x $temp
         set found 1
         set best_name $tWin
      }
   }

   if { $found == 0 } {
      # Since there was no match in current grid, the next available grid with
      # a cursor has a legal position
      set index_x $grid_x
      set index_y $grid_y
      while {1} {
         while {1} {
            incr index_x -1
            if {$index_x < 0} {
               break
            }
            if {[info exists ::Cursor::CursorArray_Win($index_x.$index_y)]} {
               set found 1
               set gridList $::Cursor::CursorArray_Win($index_x.$index_y)
               break
            } else {
            }
         }
         if { $found } {break}
         set index_x $::Cursor::ScreenGrid(num_cols)
         incr index_y -1
         if { $index_y < 0 } {
            set index_y $::Cursor::ScreenGrid(num_rows)
         }
      }

      # We have a list of possible cursors. Get the stats on the first one
      # and then try to improve on it.
      set best_name [lindex $gridList 0]
      set arrayVal $::Cursor::CursorArray_Grid($best_name)
      set best_x [lindex $arrayVal 2]      
      set best_y [lindex $arrayVal 3]      
      foreach tWin $gridList {
         if {$tWin == $best_name } { continue }
         set arrayVal $::Cursor::CursorArray_Grid($tWin)
         set test_x  [lindex $arrayVal 2]
         if { $test_x == $best_x } {
            set test_y  [lindex $arrayVal 3]
            if {$test_y < $best_y} {
               set best_y $test_y
               set best_name $tWin
            }
         } elseif { $test_x > $best_x } {
            set best_name $tWin
            set best_x $test_x      
         }
      }
   }
  

   if {[string length $best_name] != 0 } {
      if { [tabTestToWindow $best_name] } {
         # set this up as a binding on the widget
         bind $name <<cursor_left>> "::Cursor::tabTestToWindow $best_name; break"
      }
   }
 }

#****************************************************************************
# Name:        tabTestToWindow
# Parameters:  target   Desired target cursor location
# Description: If target cursor exists, move to it. Otherwise print out error
#              information.
#****************************************************************************
 proc tabTestToWindow { target } {
   
   # If cursor exists, we are done
   if { [winfo exists $target] } {
      tkTabToWindow $target
      return 1
   }

   rebuildControl [focus] MOVEMENT

   return 0
 }

#****************************************************************************
# Name:        rebuildControl
# Parameters:  widget   Cursor widget which has changed its map status
#                       or which is being moved from
#              reason   { MAP | MOVEMENT }
# Description: This function destroys current cursor movement bindings and
#              sets up an idle proc to rebuild new ones
#****************************************************************************
 proc rebuildControl { widget reason } {

   variable BuildState

   # Attempt to find the widget. If we cannot find the widget, attempt to find
   # the widget that has the focus. If neither are available, we do not know
   # what to rebuild
   if {![winfo exists $widget]} \
   {
      if {![winfo exists [focus]]} \
      {
         puts "Unable to rebuild cursor list for $widget"
         return
      }
      set widget [focus]
   } 
   set top [winfo toplevel $widget]
   if {![info exists BuildState($top)]} \
   {
      set BuildState($top) 0
   }


   # Deal with situation where the user has moved the cursor and either the
   # source or target position does not exist
   if {$reason == "MOVEMENT" } \
   { 
      if { $BuildState($top) == 0 } \
      {
         set BuildState($top) 1
         ::Cursor::clear_relations $top
         after cancel ::Cursor::setCursorArrays $top
         after idle ::Cursor::setCursorArrays $top
      }
      return
   }

   # Deal with situation where one or more cursor targets have either been
   # mapped or unmapped from the screen. Since if this happens it typically
   # will be part of a number of this type of calls, we want to delay doing
   # much until a fixed time has passed after the last screen change.
   if {$reason == "MAP" } \
   { 
      if { $BuildState($top) == 0 } \
      {
         set BuildState($top) 1
         ::Cursor::clear_relations $top
      }
      after cancel "::Cursor::setCursorArrays $top"
      after cancel "::Cursor::cleanRebuild $top"
      after 500 "::Cursor::cleanRebuild $top"
   }
 }

#****************************************************************************
# Name:        cleanRebuild
# Parameters:  top      Top level window to have cursor list rebuilt
# Description: This function should ONLY be called from rebuildControl, since
#              that function handles the processing delay when the screen is
#              updated. This calls clear_relations even though there should
#              not be active bindings at this point, to insure that everything
#              is clean before we start.
#****************************************************************************
 proc cleanRebuild { top args } {
    after idle ::Cursor::clear_relations $top
    after idle ::Cursor::setCursorArrays $top
 }


}
# End of Cursor Namespace


bind Cursor <Unmap>                   { ::Cursor::rebuildControl %W MAP }
bind Cursor <Map>                     { ::Cursor::rebuildControl %W MAP }
bind Cursor <<cursor_right_default>>  { after idle "::Cursor::move_cursor_right %W"; break }
bind Cursor <<cursor_left_default>>   { after idle "::Cursor::move_cursor_left  %W"; break }
bind Cursor <<cursor_up_default>>     { after idle "::Cursor::move_cursor_up    %W"; break }
bind Cursor <<cursor_down_default>>   { after idle "::Cursor::move_cursor_down  %W"; break }

bind all <<cursor_right>>  { event generate %W <<cursor_right_default>> }
bind all <<cursor_left>>   { event generate %W <<cursor_left_default>> }
bind all <<cursor_up>>     { event generate %W <<cursor_up_default>> }
bind all <<cursor_down>>   { event generate %W <<cursor_down_default>> }

bind all <<cursor_right_default>>  { ::Cursor::defaultSetupCursorArrays %W RIGHT }
bind all <<cursor_left_default>>   { ::Cursor::defaultSetupCursorArrays %W LEFT }
bind all <<cursor_up_default>>     { ::Cursor::defaultSetupCursorArrays %W UP }
bind all <<cursor_down_default>>   { ::Cursor::defaultSetupCursorArrays %W DOWN }



