#****************************************************************************
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,01      DESIGN ENGINEERING      *
#****************************************************************************
# FILE        : 1151.bindings.tcl                                           *
# PRODUCT     : 2975-P25                                                    *
# DESCRIPTION : Practical Programming in Tcl and Tk, Brent B. Welch         *
#               Chapter 43, A user interface to widget bindings.            *
#                                                                           *
#  A "Return" on the "Event" or "Command" edit lines will save the binding  *
#  changes used by the screen.                                              *
#
#  After deleting a binding or editing a binding, the display widget will   *
#  display "old" stuff till the widget is dismissed and restarted.          *
#                                                                           *
#  A file of changes ("SAVE" button) is kept in /sysdata/bindings.          *
#                                                                           *
#****************************************************************************

namespace eval ::Bind {
   variable count 0
   
#*************************************************
#       Name: Interface
# Parameters: w - widget pathname
#Description: launch routine from here
#*************************************************
 proc Interface { w } {
  variable count
  incr count
  
  # Set a class used for resource specifications
  set screen [option get . debugScreen {} ]
  set frame [toplevel .bindui$count -class Bindui -screen $screen]
  wm title $frame "Bindings for $w"
  pack propagate $frame false

  # A labeled entry at the top to hold the current
  # widget name or class.
  set        t [frame $frame.top -bd 2]
  label      $t.l
  menubutton $t.e -menu $t.e.bindtags
  
  # set up the menu for the widget and it's bindings
  menu $t.e.bindtags
  foreach tag [bindtags $w] {
     $t.e.bindtags add command -label $tag \
        -command [namespace code "Display $frame $tag"]
  }

  pack $t.l -side left
  pack $t.e -side left -fill x -expand true
  pack $t   -side top  -fill x

  # Command buttons
  button $t.quit -command "destroy $frame"
  button $t.save -command [namespace code "Save $frame"]
  button $t.edit -command [namespace code "Edit $frame"]
  button $t.new  -command [namespace code "New  $frame"]

  pack $t.quit $t.save $t.edit $t.new -side right

  # A pair of listboxes and a scrollbar
  scrollbar $frame.s -orient vertical \
      -command [namespace code "Yview $frame"]
  listbox $frame.key  -yscrollcommand "$frame.s set"
  listbox $frame.cmd  -yscrollcommand "$frame.s set"

  pack $frame.s   -side left -fill y
  pack $frame.key -side left -fill both -expand false
  pack $frame.cmd -side left -fill both -expand true

  bind $frame <B1-Motion> [namespace code "Dragto %W %x %y"]
  bind $frame <Button-2>  [namespace code "Mark   %W %x %y"]
  bind $frame <Button-1>  [namespace code "Select %W %y   "]
  bind $frame <Shift-B1-Motion> {}
  bind $frame <Shift-Button-1> {}

  # Initialize the display
  Display $frame $w
 }


#*************************************************
#       Name: Display
# Parameters: frame to show in
#             tag being queried
#Description: 
#*************************************************
 proc Display { frame {tag {}}} {

  if {"$tag" != "" } {
    # show which binding is being examined.
    $frame.top.e config -text $tag
  } else {
    set tag [$frame.top.e cget -text]
  }
  # clear the display of bindings
  $frame.key delete 0 end
  $frame.cmd delete 0 end

  foreach seq [lsort [bind $tag]] {
    $frame.key insert end $seq
    $frame.cmd insert end [bind $tag $seq]
  }
 }

#*************************************************
#       Name: Yview
# Parameters: 
#Description: 
#*************************************************
 proc Yview { frame args } {
   foreach w "$frame.key $frame.cmd" {
    if [ catch {
      eval {$w yview} $args
    } err ] {
      puts "Yview Error: $err"
    }
  }
 }

#*************************************************
#       Name: Dragto
# Parameters: 
#Description: 
#*************************************************
 proc Dragto { widget x y } {
  set frame [winfo toplevel $widget]
  foreach w "$frame.key $frame.cmd " {
    $w scan dragto $x $y
  }
 }

#*************************************************
#       Name: Mark
# Parameters: 
#Description: 
#*************************************************
 proc Mark { widget x y } {
  set frame [winfo toplevel $widget]
  foreach w "$frame.key $frame.cmd " {
     $w scan mark $x $y
  }
 }
 
#*************************************************
#       Name: Select
# Parameters: 
#Description: 
#*************************************************
 proc Select { widget y } {
  set frame [winfo toplevel $widget]
  if {"[winfo class $widget]" != "Listbox" } { return }
  foreach w "$frame.key $frame.cmd " {
    $w select clear 0 end
    $w select anchor     [$w nearest $y]
    $w select set anchor [$w nearest $y]
  }
 }


#*************************************************
#       Name: New
# Parameters: 
#Description: 
#*************************************************
 proc New { frame } {
  set f $frame.edit
  if {![winfo exists $f] } {
    frame $f

    foreach x {key cmd} {
      set f2 [frame $f.$x]

      pack  $f2 -fill x -padx 2
      label $f2.l -width 11 -anchor e
      pack  $f2.l -side left
      entry $f2.e
      pack  $f2.e -side left -fill x -expand true

      bind $f2.e <Return> [namespace code "Define $frame"]
    }
    $f.key.l config -text Event:
    $f.cmd.l config -text Command:
  }
  
  pack $frame.edit -after $frame.top -fill x
 }

#*************************************************
#       Name: Edit
# Parameters: 
#Description: 
#*************************************************
 proc Edit { frame } {
  New $frame

  set line [$frame.key curselection]

  if {$line == {}} {
    return
  }

  $frame.edit.key.e delete 0 end
  $frame.edit.key.e insert 0 [$frame.key get $line]
  $frame.edit.cmd.e delete 0 end
  $frame.edit.cmd.e insert 0 [$frame.cmd get $line]
 }

#*************************************************
#       Name: Define
# Parameters: 
#Description: 
#*************************************************
 proc Define { f } {
  # Remove the edit window
  pack forget $f.edit
  
  set widget [$f.top.e cget -text]
  set tag    [$f.edit.key.e get]
  set cmd    [$f.edit.cmd.e get]
  
  if [catch { bind $widget $tag $cmd } err] {
    error "Define Error: $err"
  }
  Display $f
 }

#*************************************************
#       Name: Save
# Parameters: 
#Description: 
#*************************************************
 proc Save { f } {
   set dotfile [option get $f filename {}]
   file mkdir [file dirname $dotfile]
   if [ catch {
     set out [open $dotfile w]
   } err ]  {
     error "Save failure: $err"
   }

   set w [$f.top.e cget -text]
   
   set bindtags [bindtags $w]
   foreach tag $bindtags {
     set bindings [bind $tag]
     foreach binding $bindings {
      # get the current binding
      set cmd [bind $tag $binding]
      # Output a Tcl command
      set cmd "bind $tag $binding $cmd"
      puts $out "$cmd"
    }
  }

  close $out
 }

#*************************************************
#       Name: Read
# Parameters: 
#Description: 
#*************************************************
 proc Read { } {
  set dotfile [option get $f filename {}]
  if {![file exists $dotfile]} {
     return
  }
  
  if [catch {
      # Read the saved Tcl commands
      source $dotfile
     } err] {
     error "Bind::Read failed: $err"
  }
 }
 
#*************************************************
#       Name: Path
# Parameters: 
#Description: set widget bindings for mouse operation
#*************************************************
  proc Path { } {
    bind all <Button-2> {puts "Path: %W"}
    bind all <Button-3> {::Bind::Interface %W}
  }
# end of Bind namespace
}
