#*****************************************************************************************
# newentry.tcl  this file changes the bindings on a specific entry 
# widget so to imulate a test set data entry field.  this imulation
# is detailed as follows:
#     if numeric keys start edit:
#        background color is changed
#        field is blanked
#        entered keys are added to the displayed value
#        key entries are only limited to the number of digits allowed
#           at end of edit, value is reprinted and then limits are
#           enforced.
#        quanity of digits allowed before decimal point are enforced
#           as well as the quanity of digits after the decimal point 
#        current entered value is not used in hardware until the edit
#           is finished and the value's limits enforced
#        background color is restored
#        Enter ends edit and reprints formatted value
#        Arrow keys end this type of edit and then move cursor to next
#           field
#     if Enter starts edit:
#        background color is changed
#        one digit is highlighted
#        left and right arrow keys change which digit is highlighted
#        up and down keys change the currently highlighted digit
#        spinner had same affect as up and down keys
#        currently edited value changes hardware for each changed value
#           without requiring edit to end
#        limits are enforced all during edit
#        if numeric keys are pressed, the first edit type is begun
#
#     GLOBALS USED:
#*****************************************************************************************

namespace eval ::Entry \
{
  namespace export  Highlight  Clear  Insert
  
  proc Entry_up { w } \
  {
     upvar #0 ::Entry::entry_data($w,editon)  editon
                    
     if { [info exists editon] && $editon } \
     {
       event generate $w  <<edit_up>>
     } \
     else \
     {
       event generate $w <<cursor_up>>
     }
  }
  proc Entry_down { w } \
  {
     upvar #0 ::Entry::entry_data($w,editon)  editon
  
     if { [info exists editon] && $editon } \
     {
        event generate $w  <<edit_down>>
     } \
     else \
     {
        event generate $w  <<cursor_down>>
     }
  }
  proc Entry_right { w } \
  {
     upvar #0 ::Entry::entry_data($w,editon)  editon
                      
     if { [info exists editon] && $editon } \
     {
        event generate $w  <<edit_right>>
     } \
     else \
     {
        event generate $w  <<cursor_right>>
     }
  }
  proc Entry_left { w } \
  {
     upvar #0 ::Entry::entry_data($w,editon)  editon
  
     if { [info exists editon] && $editon } \
     {
        event generate $w <<edit_left>>
     } \
     else \
     {
        event generate $w  <<cursor_left>>
     }
  }
  
  # SPLIT ARROW KEYS INTO edit_ AND cursor_ EVENTS
  bind all <Up>        "::Entry::Entry_up %W"
  bind all <Down>      "::Entry::Entry_down %W"
  bind all <Right>     "::Entry::Entry_right %W"
  bind all <Left>      "::Entry::Entry_left %W"
  bind all <KP_Up>     "::Entry::Entry_up %W"
  bind all <KP_Down>   "::Entry::Entry_down %W"
  bind all <KP_Right>  "::Entry::Entry_right %W"
  bind all <KP_Left>   "::Entry::Entry_left %W"


  #**************************************************************************
  #        Name: Highlight
  #  Parameters: widget
  # Description: shows currently selected char 
  #**************************************************************************
  proc Highlight { widget } \
  {
    #puts "Highlight:  $widget"
  
    # now, get where the cursor really is
    set sel  [$widget index insert]
  
    set endsel  $sel
  
    incr endsel
  
    $widget selection  range  $sel  $endsel
    
    return
  }


  #**************************************************************************
  #        Name: Clear
  #  Parameters: widget,
  # Description: clear the edit field
  #**************************************************************************
  proc Clear { widget } \
  {
    set var  [$widget cget -textvariable]
   
    upvar #0 $var  value
   
    set value  ""
   
    $widget icursor end
   
    Highlight  $widget
    
    return
  }
   
  
  #**************************************************************************
  #        Name: Insert
  #  Parameters: widget, key, insert point
  # Description: insert a char into the value at
  #             current insert point
  #**************************************************************************
  proc Insert { widget key index } \
  {
    upvar #0 [$widget cget -textvariable]  value
  
    set first  [expr $index - 1]
    set last   [expr $index + 1]
    
    set newvalue  {}
  
    append newvalue  [string range $value 0 $first]  $key  [string range $value $last end]
   
    return  $newvalue
  }


};  # End of Entry namespace



namespace eval ::SpinEntry \
{
  namespace import  ::Entry::Highlight  ::Entry::Clear  ::Entry::Insert

  # Pitch the keypad number key events over to standard keys
  foreach key { 0 1 2 3 4 5 6 7 8 9 A B C D E F period minus plus } \
  {
    bind SpinEntry <Key-$key>  "SpinEntry::keypress  %W %A %K"
  }
   
  bind SpinEntry <Destroy>    "SpinEntry::Unbind    %W"
  
  #bind SpinEntry <KeyRelease-Return>    "SpinEntry::Return  %W; break"
  #bind SpinEntry <KeyRelease-KP_Enter>  "SpinEntry::Return  %W; break"
  bind SpinEntry <Key-Return>    "SpinEntry::Return  %W"
  bind SpinEntry <Key-KP_Enter>  "SpinEntry::Return  %W"
  
  
  bind SpinEntry <<Spin_up>>      { if {[SpinEntry::Spin_up   %W] == 1} { break } }
  bind SpinEntry <<Spin_down>>    { if {[SpinEntry::Spin_down %W] == 1} { break } }
  
  bind SpinEntry <<edit_left>>    "SpinEntry::left      %W"
  bind SpinEntry <<edit_right>>   "SpinEntry::right     %W"
  bind SpinEntry <<edit_up>>      "SpinEntry::up        %W"
  bind SpinEntry <<edit_down>>    "SpinEntry::down      %W"
  
  bind SpinEntry <Escape>         "SpinEntry::Escape       %W"
  bind SpinEntry <BackSpace>      "SpinEntry::Backspace    %W"
  bind SpinEntry <<EditOpen>>     "SpinEntry::start_edit   %W"
  bind SpinEntry <<EditClose>>    "SpinEntry::stop_edit    %W"
  bind SpinEntry <<EditEscape>>   "SpinEntry::stop_edit    %W ESC"
  bind SpinEntry <<EditClear>>    "SpinEntry::Clear        %W"
  bind SpinEntry <<Validate>>     "SpinEntry::Validate     %W"
  bind SpinEntry <Button-1>       "SpinEntry::Click        %W %x; break"
  bind SpinEntry <Double-1>       "SpinEntry::DoubleClick  %W %x; break"

  variable show_count 0
  variable Widget_cleanups ""
   
  
  
  #**************************************************************************
  #        Name: Create
  #              widget
  # Description: Creates an entry 
  #**************************************************************************
  proc Create { widget } \
  {
    entry  $widget

    Bind  $widget
      
    return  $widget
  }


  #**************************************************************************
  #        Name: Bind
  #              widget,
  #              upper_max, number of digits allowed before decimal point
  #              lesser_max, number of digits allowed after decimal point
  #              max, maximum value allowed
  #              min, minimum value allowed
  #              comm, command line to execute when edit finished
  #              editbgcolor, background color during edit
  # Description: binds entry 
  #**************************************************************************
  proc Bind {widget {upper_max {}} {lesser_max {}} {max {}} {min {}} \
                              {comm_list {}} {editbgcolor {}} {spinupdate {}}} \
  {
    #puts "SpinEntry::Bind:  $widget,  $upper_max,  $lesser_max,  $max,  $min"
  
    if { ![winfo exists $widget] } \
    {
      return
    }
  
    # IF EDIT IS ACTIVE FOR THIS WIDGET, DONT ALLOW NEW EDIT BINDING
    if { [info exists ::Entry::entry_data($widget,editon)] && \
                                          $::Entry::entry_data($widget,editon) } \
    {
      return
    }
  
    if { $upper_max   == "" } { set upper_max   [option get $widget upper_max  {}] }
    if { $lesser_max  == "" } { set lesser_max  [option get $widget lesser_max {}] }
    if { $max         == "" } { set max         [option get $widget max        {}] }
    if { $min         == "" } { set min         [option get $widget min        {}] }
    if { $comm_list   == "" } { set comm_list   [option get $widget comm_list  {}] }
    if { $editbgcolor == "" } { set editbgcolor [option get $widget editbg     {}] }

    if { $spinupdate  == "" } { set spinupdate  [option get $widget spinupdate {}] }
    if { $comm_list   == "" } { set comm_list   [option get $widget command    {}] }
   
    set editfgcolor [option get $widget editfg     {}]
    set format  [option get $widget format {}]

    # Replace any '%W' designations with the widget tile path
    regsub -all "%W"  $comm_list  $widget  comm_list
  
    set comm  [list $comm_list]
    
    # Initialize the widgets variables
    set ::Entry::entry_data($widget,upper_max)    $upper_max
    set ::Entry::entry_data($widget,lesser_max)   $lesser_max
    set ::Entry::entry_data($widget,max)          $max
    set ::Entry::entry_data($widget,min)          $min
    set ::Entry::entry_data($widget,comm)         $comm_list
    set ::Entry::entry_data($widget,editbgcolor)  $editbgcolor
    set ::Entry::entry_data($widget,editfgcolor)  $editfgcolor
    set ::Entry::entry_data($widget,editon)       0
    set ::Entry::entry_data($widget,spinupdate)   $spinupdate
    set ::Entry::entry_data($widget,format)       $format
        
    # Clear all current bindings
    del_bindtag $widget Entry
       
    # Add the necessary bindings
    add_bindtag $widget  SpinEntry
    
    eval $widget configure -takefocus 1 [option get $widget editops {}]
    
    # Stop the insert point from showing when not in edit.
    $widget configure -insertofftime 1000
    $widget configure -insertontime  0
    $widget configure -cursor xterm
    
    return
  }


  #**************************************************************************
  #        Name: Unbind
  #  Parameters: 
  # Description: removes our data from the array
  #**************************************************************************
  proc Unbind { widget } \
  {
    variable  Widget_cleanups
     
    lappend Widget_cleanups  $widget

    after cancel  SpinEntry::Unbind_after
    after idle    SpinEntry::Unbind_after
    
    return
  }
  
  proc Unbind_after { } \
  {
    variable  Widget_cleanups
    
    set widget  [lindex $Widget_cleanups 0]
    
    set Widget_cleanups  [lrange $Widget_cleanups 1 end]
       
    foreach  thing  [Find_vars $widget] \
    {
      unset  ::Entry::entry_data($thing)
    }
    
    if { [llength $Widget_cleanups] != 0 } \
    {
      after idle  SpinEntry::Unbind_after
    }
  }


  #**************************************************************************
  #        Name: Escape
  #  Parameters: widget,
  # Description: binds Escape key for entry 
  #**************************************************************************
  proc Escape { widget } \
  {
    upvar ::Entry::entry_data($widget,editon)  editon
   
    # If not set up or not open, do nothing
    if { ![info exists editon] || !$editon } { return }
    
    upvar ::Entry::entry_data($widget,original)  original
    
    upvar #0 [$widget cget -textvariable]  value
  
    set value  $original
    
    event generate  $widget  <<EditEscape>>
    
    return
  }
  
  
  #**************************************************************************
  #        Name: Backspace
  #  Parameters: widget,
  # Description: binds backspace key for entry, actually does nothing
  #              so that backspace binding is not killed by do_entry_keypress
  #**************************************************************************
  proc Backspace { widget } \
  {   
    # If we aren't enabled, bail right now
    if { [$widget cget -state] == "disabled" } { return }
  
    upvar ::Entry::entry_data($widget,editon)  editon

    if { ![info exists editon] } { set editon  0 }

    # If the edit isn't open, generate the edit open event
    if { !$editon } \
    {
      event generate  $widget  <<EditOpen>>
      
      # Continue editing
      ##event generate  $widget  <BackSpace>
      ##return
    }

    tkEntryBackspace  $widget

    Highlight  $widget
    
    return
  }          
  
  
  #**************************************************************************
  #        Name: return
  #  Parameters: widget
  # Description: handles binding for ENTER key
  #**************************************************************************
   proc Return { widget } \
   {
     #puts "Return:  $widget"
     
     upvar ::Entry::entry_data($widget,editon)  editon
     
     if { ![info exists editon] } { set editon  0 }
     
     #puts "  editon:  $editon"
     
     if { $editon } \
     {
       event generate  $widget  <<EditClose>>
       
       return
     }
  
     # If '-state' option is 'disabled', 
     # don't start edit,
     # return
     if { [$widget cget -state] == "disabled" } \
     { 
       return 
     }
    
     # Open the edit
     event generate  $widget  <<EditOpen>>
     
     return
   }
  
  
  #**************************************************************************
  #        Name: Format
  #  Parameters: widget, new value
  # Description: formats the data for display
  #**************************************************************************
  proc Format { widget  newvalue } \
  {
    upvar #0 [$widget cget -textvariable]  value
    
    upvar ::Entry::entry_data($widget,upper_max)   upper_max
    upvar ::Entry::entry_data($widget,lesser_max)  lesser_max
    upvar ::Entry::entry_data($widget,max)         max
    upvar ::Entry::entry_data($widget,min)         min
    upvar ::Entry::entry_data($widget,format)      format
    
    switch $format \
    {
       "hex" \
       {
          scan $newvalue %x newvalue
          set value [format %.*X $upper_max $newvalue]
       }
       "octal" \
       {
          scan $newvalue %o newvalue
          set value [format %.*o $upper_max $newvalue]
       }
       default \
       {
          scan $newvalue %f newvalue
          set value [format %1.*f $lesser_max $newvalue]
       }
    }
  }
  
  
  #**************************************************************************
  #        Name: compute_exponent
  #  Parameters: widget
  # Description: computes the delta value to adjust by
  #**************************************************************************
  proc compute_exponent { widget } \
  {
    # Get access to the value
    upvar #0 [$widget cget -textvariable]  value
    
    set selected  [$widget index insert]
    
    # If the selection is past the end of the value,
    # move in to the end of the value
    if { $selected == [$widget index end] } \
    {
      incr selected  -1
    }
     
    # Locate the decimal point, and compute the
    # exponent based upon our location relative to it.
    set dec_index  [string first "." $value]
    
    if { $dec_index == -1 } \
    {
      # No decimal in field. 
      # compute where we are relative the end
      set end  [$widget index end]
     
      if { $end == $selected } \
      { 
        return  0 
      }
      
      return  [expr [$widget index end] - $selected - 1]
    }
     
    set incr_exponent  [expr $dec_index - $selected - 1]
    
    if { $selected > $dec_index } \
    {
      # On right of decimal point. allow for that 1 char
      incr incr_exponent
    }
     
    return  $incr_exponent
  }
  
  
  #**************************************************************************
  #        Name: compute_insert_point
  #  Parameters: widget, exponent value
  # Description: computes location of insert point
  #**************************************************************************
  proc compute_insert_point { widget  exponent } \
  {
    #puts "compute_insert_point"
    
    # Get access to the value
    upvar #0 [$widget cget -textvariable]  value
  
    # Locate the decimal point, and compute the
    # exponent based upon our location relative to it.
    set dec_index  [string first "."  $value]
  
    if { $dec_index == -1 } \
    {
      # No decimal point, so relate from end of string
      set dec_index  [string length  $value]
    }
    
    if { $exponent >= 0 } \
    {
      # Allow a char for the decimal point itself
      set cursor  [expr $dec_index - $exponent - 1]
    } \
    else \
    {
      set cursor  [expr $dec_index - $exponent]
    }
    
    $widget icursor  $cursor
    
    Highlight  $widget
    
    return
  }
  
  
  #**************************************************************************
  #        Name: Spin_up
  #  Parameters: widget
  # Description: handle spinner up events
  #**************************************************************************
  proc Spin_up { widget } \
  {
    #puts "Spin_up:  $widget"
    
    # If the widget's gone away, bail
    if { ![winfo exists $widget] } { return  0 }
  
    # If we aren't enabled, bail right now
    if { [$widget cget -state] == "disabled" } { return  0 }
  
    # If the edit isn't open, return 0
    upvar ::Entry::entry_data($widget,editon)  editon
  
    if { ![info exists editon] } { set editon  0 }
  
    if { !$editon } { return  0 }
  
    # Generate an edit up event.
    event generate $widget  <<edit_up>>
  
    return  1
  }
  
  
  #**************************************************************************
  #        Name: Spin_down
  #  Parameters: widget
  # Description: handle spinner down events
  #**************************************************************************
  proc Spin_down { widget } \
  {
    #puts "Spin_down:  $widget"
  
    # If the widget's gone away, bail
    if { ![winfo exists $widget ] } { return  0 }
  
    # If we aren't enabled, bail right now
    if { [$widget cget -state] == "disabled" } { return  0 }
  
    # If the edit isn't open, return 0
    upvar ::Entry::entry_data($widget,editon)  editon
  
    if { ![info exists editon] } { set editon  0 }
  
    if { !$editon } { return  0 }
  
    # Generate an edit up event.
    event generate  $widget  <<edit_down>>
  
    return  1
  }
     
  
  #**************************************************************************
  #        Name: up
  #  Parameters: widget
  # Description: binds up to entry
  #**************************************************************************
  proc up { widget } \
  {
    set cmd  "SpinEntry::updown_core  $widget up"
      
    after cancel  $cmd
      
    after idle    $cmd
      
    return
  }
  
  proc updown_core { widget dir } \
  {
    #puts "up_core:  $widget"
    
    # if we aren't enabled, bail right now
    if { [$widget cget -state] == "disabled" } { return }
   
    # if the edit isn't open, generate the edit open event, and
    # regenerate this event.
    upvar ::Entry::entry_data($widget,editon)  editon
   
    if { ![info exists editon] } { set editon 0 }
    
    if { !$editon } \
    {
      return
    }
  
    upvar ::Entry::entry_data($widget,min)         min
    upvar ::Entry::entry_data($widget,max)         max
    upvar ::Entry::entry_data($widget,comm)        comm
    upvar ::Entry::entry_data($widget,spinupdate)  spinupdate
    upvar ::Entry::entry_data($widget,lesser_max)  lesser_max
    upvar ::Entry::entry_data($widget,format)      format
    set after_name  ::Entry::entry_data($widget,after)
    
    upvar $after_name  after
    
    upvar base  base
  
    # access the variable and limits
    upvar #0 [$widget cget -textvariable]  value
     
    set selected   [$widget index  insert]
  
    set length     [string length  $value]
  
    set exponent   [compute_exponent  $widget]
    
    set increment  [expr pow(10, $exponent)]
  
    # convert to an integer
    set newvalue 0
    switch -- $format \
    {
       "octal" { scan $value %o newvalue}
       "hex"   { scan $value %x newvalue}
       default { scan $value %f newvalue}
    }
    switch -- $dir \
    {
      "up" \
      {
        set newvalue [expr $newvalue + $increment]
        if { $newvalue > $max } \
        {
          return 
        }
      }
      default
      {
         set newvalue [expr $newvalue - $increment]
         if { $newvalue < $min } \
         { 
            return 
         }
      }
   }
    switch -- $format \
    {
       "octal" { set newvalue [format %o [expr int($newvalue)]]}
       "hex"   { set newvalue [format %X [expr int($newvalue)]]}
       default { set newvalue [format %f $newvalue]}
    }
    # FORMAT VALUE
    Format  $widget  $newvalue
    
    # adjust Highlight as needed
    compute_insert_point  $widget  $exponent
   
    if { $comm != "" && $spinupdate } \
    {
      after idle "eval $comm"
    }
  
    return
  }
  
  
  #**************************************************************************
  #        Name: down
  #  Parameters: widget
  # Description: binds down to entry 
  #**************************************************************************
  proc down { widget } \
  {
    set cmd  "SpinEntry::updown_core  $widget down"
      
    after cancel  $cmd
      
    after idle    $cmd
      
    return
  }
   
  
  #**************************************************************************
  #
  #        Name: left
  #
  #  Parameters: widget
  #
  # Description: move selected char left 
  #
  #**************************************************************************
  proc left { widget } \
  {
    #puts "left:  $widget"
    
    # If we aren't enabled, bail right now.
    if { [$widget cget -state] == "disabled" } { return }
  
    # If the edit isn't open, generate the edit open event, and
    # regenerate this event.
    upvar #0 ::Entry::entry_data($widget,editon)  editon
  
    if { ![info exists editon] } { set editon 0 }
  
    if { !$editon } \
    {
      return
    }
    
    upvar #0 [$widget cget -textvariable]  value
    
    set sel  [$widget index insert]
  
    incr sel  -1
  
    # Is the cursor at the decimal point? if so, move again.
    if { [string index $value $sel] == "." } \
    {
      incr sel  -1
    }
  
    # If the selection index is 0, put a '0' at the beginning and move the 
    # cursor to the left.
    if { $sel == -1 } \
    {
      $widget insert  $sel  "0"
      
      incr sel  -1
    }
  
    $widget icursor  $sel
  
    Highlight $widget
  }
  
  
  #**************************************************************************
  #
  #        Name: right
  #
  #  Parameters: widget
  #
  # Description: binds entry 
  #
  #**************************************************************************
  proc right { widget } \
  {
    #puts "right:  $widget"
  
    # If we aren't enabled, bail right now
    if { [$widget cget -state] == "disabled" } { return }
    
    # If the edit isn't open, generate the edit open event, and
    # regenerate this event.
    upvar #0 ::Entry::entry_data($widget,editon)  editon
    
    if { ![info exists editon] } { set editon  0 }
    
    if { !$editon } \
    {
      return
    }
    
    upvar #0 [$widget cget -textvariable]  value
    
    set sel  [$widget index insert]
    
    incr sel  1
    
    # Is the cursor at the decimal point? 
    # if so, move again.
    if { [string index $value $sel] == "." } \
    {
      incr sel  1
    }
    
    # Remove the leading zero that may exist
    if { [string index $value 0] == "0" } \
    {
      set value  [string range  $value  1  end]
      
      incr sel  -1
    }
  
    $widget icursor  $sel
    
    Highlight $widget
  }
   
   
  #**************************************************************************
  #        Name: keypress
  #  Parameters: widget
  # Description: binds entry 
  #**************************************************************************
  proc keypress { widget  key  keysym } \
  {
    #puts "SpinEntry::keypress:  $widget,  $key,  $keysym"
    
    # If we aren't enabled, bail right now
    if { [$widget cget -state] == "disabled" } { return }
  
    # If the edit isn't open, generate the edit open event, and
    # regenerate this event.
    upvar #0 ::Entry::entry_data($widget,editon)  editon
  
    if { ![info exists editon] } \
    {
       set editon 0
    }
    
    if { !$editon } \
    { 
      event generate $widget <<EditOpen>>
      event generate $widget <<EditClear>>
      
      # Continue editing
    }
    
    set fmt_string "%f"
    
    # ignore invalid keypresses for other modes
    upvar ::Entry::entry_data($widget,format)       format
    switch -- $format \
    {
       "octal" \
       {
          set fmt_string "%o"
          if {[string first $key "01234567"] == -1} { return }
       }
       "hex" \
       {
          set fmt_string "%x"
          if {[string first $key "0123456789ABCDEF"] == -1} { return }
       }
       default \
       {
          if {[string first $key "0123456789+-."] == -1} { return }
       }
    }
    
    # access the variable and limits
    upvar ::Entry::entry_data($widget,var)          var
    upvar ::Entry::entry_data($widget,upper_max)    upper_max
    upvar ::Entry::entry_data($widget,lesser_max)   lesser_max
    upvar ::Entry::entry_data($widget,max)          max
    upvar ::Entry::entry_data($widget,min)          min
    upvar ::Entry::entry_data($widget,back)         back
    upvar ::Entry::entry_data($widget,comm)         comm
    upvar ::Entry::entry_data($widget,editbgcolor)  editbgcolor      
    upvar ::Entry::entry_data($widget,upper)        upper      
    upvar ::Entry::entry_data($widget,lesser)       lesser      
    upvar ::Entry::entry_data($widget,insert)       insert      
    upvar ::Entry::entry_data($widget,dp_index)     dp_index      
    upvar ::Entry::entry_data($widget,newvalue)     newvalue   
        
    upvar #0 [$widget cget -textvariable]  value
    
    set newvalue $value
     
    # clear the insert point value
    set exp  {}
    
    set insert    [$widget index insert]
    set dp_index  [string first "." $value]
    
    # compute where we are in relation to the decimal point
    if { $dp_index == -1 } \
    {
      # no decimal point, all chars are in the upper part
      set upper $insert
    } \
    else \
    {
      # are we above or below the decimal?
      if { $insert >= $dp_index } \
      {
        # below the DP:
        set lesser [expr $insert - $dp_index - 1]
      } \
      else \
      {
        set upper [expr $insert + 1]
      }
    }
  
    set minus_flag 0
    
    switch -- $key \
    {
        "." { # DECIMAL POINT KEY
              # are we allowed to have a decimal point?
              if {$lesser_max <= 0 } { return }
              # get the location of an existing dp.
              if { $dp_index == -1 } \
              {
                 set newvalue [Insert $widget $key $insert]
              } \
              elseif { $insert < $dp_index } \
              {
                # insert decimal point ahead of current, and
                # remove all other chars
                set last [expr $insert - 1]
                set newvalue [string range $value 0 $last]
                append newvalue $key
              } \
              else \
              {
                # this DP is after the current. Ignore key
                return
              }
            }
    
        "-" { # MINUS KEY
              # have to handle 0 (or blank) specially
              if {$value == "" } { set value 0}
              set newvalue [expr $value + 0] ;# convert to numeric
              if {$newvalue == 0} \
              {
                 # since -0 is 0, we need to flag that we need to add
                 # a - to the number at the end of the routine.
                 set minus_flag 1
              } \
              else \
              {
                set newvalue [expr -$newvalue]
              }
            }
        "+" { # PLUS KEY
              if {$value > 0 } { return }
              set exp [compute_exponent $widget]
              set newvalue [expr 0-$value + 0]
              #puts "newvalue is $newvalue"
            }
       
        default { # are we before or after the decimal point?
                  if {($dp_index == -1) || ($insert < $dp_index)} \
                  {
                     # before decimal point. See if we are exceeding the
                     # maximum # of digits...
                     # allow for minus if needed
                     set new_uppermax $upper_max
                     if {"[string index $newvalue 0]"=="-"} \
                     {
                        incr new_uppermax
                        #incr insert
                     }
                     
                     if {$upper <= $new_uppermax} \
                     {
                        set newvalue [Insert $widget $key $insert]
                     } \
                     else \
                     {
                        # not allowed, so ignore the digit.
                        return
                     }
                  } \
                  else \
                  {
                     # after decimal point
                     if {$lesser < $lesser_max} \
                     {
                        set newvalue [Insert $widget $key $insert]
                     } \
                     else \
                     {
                        # not allowed, so ignore the digit.
                        return
                     }
                  }
           }
    }
    
    # if the first char is a ., prepend a 0
    set p1 {^(-?)0*((0\.)?.+\.*)}
    set p2 {^(-)?(\.)(.*)}
    regsub $p1 $newvalue {\1\2} newvalue
    regsub $p2 $newvalue {\10.\3} newvalue
    
    scan $newvalue $fmt_string newvalue_numeric
    
    # ok, if $max is < 0, we have to allow 0, so override that here.
    # equally, if $min >0, we have to allow 0.
    set newmin $min
    set newmax $max
    
    if {$newmax  < 0 } \
    {
       if {$newvalue == 0 } \
       {
           set minus_flag 1
       } \
       elseif {$newvalue > 0 } \
       {
          set newvalue [expr 0-$newvalue]
       }
       
       set newmax 0
    }
    
    if {$newmin > 0 } \
    { 
       set newmin 0
    }
   
    if { $newvalue_numeric >= $newmin && $newvalue_numeric <= $newmax } \
    {
        if {$minus_flag} \
        {
           set value "-0"
        } \
        else \
        {
           set value $newvalue
        }
    }
    
    if {"$exp" != "" } \
    {
        # if we need to restore the insert point, do so
        compute_insert_point $widget $exp
    } \
    else \
    {
       # move the insert right 1 place
       incr insert
       if {[string index $value $insert] == "."} \
       {
          incr insert
       }
       $widget icursor $insert
    }
    Highlight $widget
  }
  
  
  #**************************************************************************
  #        Name: start_edit
  #  Parameters: widget
  # Description: start the edit 
  #**************************************************************************
  proc start_edit { widget } \
  {
    #puts "start_edit:  $widget"

    # If -state IS disabled, DONT START EDIT, RETURN
    if { [$widget cget -state] == "disabled" } { return }
  
    # See if this objects data fields have been set up. if not,
    # then see if they were set up in the resource file.
    if { ![info exists ::Entry::entry_data($widget,lesser_max)] } \
    {
      Bind  $widget
    }
  
    # Access the variable and limits
    upvar ::Entry::entry_data($widget,lesser_max)   lesser_max
    upvar ::Entry::entry_data($widget,upper_max)    upper_max
    upvar ::Entry::entry_data($widget,max)          max
    upvar ::Entry::entry_data($widget,min)          min
    upvar ::Entry::entry_data($widget,comm)         comm
    upvar ::Entry::entry_data($widget,editbgcolor)  editbgcolor
    upvar ::Entry::entry_data($widget,editfgcolor)  editfgcolor
    upvar ::Entry::entry_data($widget,back)         back
    upvar ::Entry::entry_data($widget,fg)           fg
    upvar ::Entry::entry_data($widget,upper)        upper
    upvar ::Entry::entry_data($widget,lesser)       lesser
    upvar ::Entry::entry_data($widget,original)     original
    upvar ::Entry::entry_data($widget,editon)       editon
    
    upvar #0 [$widget cget -textvariable]  value
  
    # BIND FOCUSOUT TO RETURN PROCEDURE
    bind $widget  <FocusOut>  "event generate %W <<EditClose>>"
  
    # START EDIT
    set upper     0
    set lesser    0
    set original  $value
    
    set back  [$widget cget -bg]
    set fg    [$widget cget -fg]
    set selbg [option get $widget edisel {}]
    if {$selbg == ""}       { set selbg yellow}
    if {$editfgcolor == ""} { set editfgcolor black}
    if {$editbgcolor == ""} { set editbgcolor white} 
    
    set editon  1
  
    $widget config  -bg $editbgcolor -fg $editfgcolor    \
                    -selectbackground $selbg
         
    $widget select  clear
    
    # Determine if the insert cursor is over the '.'
    set sel  [$widget index insert]
    
    # If it is, move it to the right
    if { [string index $value $sel] == "." } \
    {
      incr sel  1
      
      $widget icursor  $sel
    }
    
    # Move the insert cursor
    Highlight  $widget
    
    $widget configure  -insertofftime 500
    $widget configure  -insertontime  500
  }
  
  #**************************************************************************
  # Name: rebind_start_edit
  # Parameters: widget
  # Description: This routine is used to change the <FocusOut> binding on the
  #    edit field so that it does not do the normal <<EditClose>> call. This
  #    is needed if the desired response on losing focus is to completely 
  #    cancel any activity. 
  #**************************************************************************
  proc rebind_start_edit { widget } \
  {
     SpinEntry::start_edit $widget
     bind $widget <FocusOut>  "SpinEntry::cancel_edit $widget; break"
     bind $widget <1> { 
        grab release %W
        set min_x [winfo rootx %W]
        set max_x [expr $min_x + [winfo width %W] ]
        set min_y [winfo rooty %W]
        set max_y [expr $min_y + [winfo height %W] ]
        set mouse_x [winfo pointerx %W]
        set mouse_y [winfo pointery %W]
        set back [option get %W back  {}]
        %W config  -bg $back 

        if { ($mouse_x >= $min_x) && ($mouse_x <= $max_x) && \
             ($mouse_y >= $min_y) && ($mouse_y <= $max_y) } \
        {
            SpinEntry::stop_edit %W
        } else \
        {
            SpinEntry::cancel_edit %W
        }
        bind %W <1> {}
        break;
     }
     grab -global $widget
  }
  
  #**************************************************************************
  #        Name: Validate
  #  Parameters: widget
  # Description: validate the value & send to hardware
  #**************************************************************************
  proc Validate { widget } \
  {
    #puts "Validate:  $widget"
    
    upvar ::Entry::entry_data($widget,lesser_max)  lesser_max
    upvar ::Entry::entry_data($widget,max)         max
    upvar ::Entry::entry_data($widget,min)         min
    upvar ::Entry::entry_data($widget,comm)        comm
    upvar ::Entry::entry_data($widget,original)    original
    upvar ::Entry::entry_data($widget,format)      format
    upvar #0 [$widget cget -textvariable]  value
  
    # Check for bogus entries, and restore the original if needed
    if { $value == "-" || $value == "." || $value == "" } \
    {
      set value  $original
    }
  
    # Strip any leading zeros
    set value  [string trimleft  $value  0]
    
    # Keep at least 1 zero
    if { $value == "" } { set value 0 }
    
    switch -- $format \
    {
      "hex" {}
      "octal" {}
      default \
      {
         set value  [::format_numeric  $value  $lesser_max  $max  $min]
      }
    }
    
    # Now, handle any final command
    if { $comm != "" } \
    {
      namespace eval :: " eval $comm "
    }
    
    return
  }
  
  
  #**************************************************************************
  #        Name: stop_edit
  #  Parameters: widget
  # Description: close the edit field 
  #**************************************************************************
  proc stop_edit { widget {escape {}} } \
  {
    #puts "stop_edit:  $widget,  $escape"

    upvar ::Entry::entry_data($widget,editon) editon

    if {![info exists editon] || !$editon} { return }
    
    upvar ::Entry::entry_data($widget,back) back
    upvar ::Entry::entry_data($widget,fg) fg
    bind $widget <FocusOut> { }
    
    # stop the insert point from showing
    $widget configure -insertofftime 1000
    $widget configure -insertontime  0
  
    set editon 0
    
    $widget select clear   
    $widget config -bg $back -selectbackground $back -fg $fg
  
    if {($escape == "") || ([option get $widget esc_does_nothing {}] == "")} \
    {
       event generate $widget <<Validate>>
    }
  } 

  #**************************************************************************
  # Name: cancel_edit
  # Parameters: widget
  # Description: close the edit field and kills any action 
  # This is currently only used by the Quick Setup Recall edit field
  #**************************************************************************
  proc cancel_edit { widget } \
  {
    #puts "cancel_edit:  $widget"

    upvar ::Entry::entry_data($widget,editon) editon
    if {![info exists editon] || !$editon} { return }
    
    set back [option get $widget back  {}]
    upvar ::Entry::entry_data($widget,fg) fg
    bind $widget <FocusOut> { }
    
    # stop the insert point from showing
    $widget configure -insertofftime 1000
    $widget configure -insertontime  0
  
    set editon 0
    
    $widget select clear   

    $widget config -bg $back -selectbackground $back -fg $fg

    upvar ::Entry::entry_data($widget,original)    original
    upvar ::Entry::entry_data($widget,lesser_max)  lesser_max
    upvar ::Entry::entry_data($widget,max)         max
    upvar ::Entry::entry_data($widget,min)         min
    ::format_numeric  $original  $lesser_max  $max  $min
  } 

  #**************************************************************************
  #        Name: Find_vars
  #  Parameters: 
  # Description: get a list of our state variables
  #**************************************************************************
  proc Find_vars { widget } \
  {
    # now, find all edit state variables associated with this widget
    return [array names ::Entry::entry_data $widget*]
  }
  
  
  #**************************************************************************
  #        Name: At_exit
  #  Parameters: 
  # Description: Handles cleanup when we are powering down
  #**************************************************************************
  proc At_exit { } \
  {
     bind SpinEntry <Destroy> ""
  }
  
  
  at_exit SpinEntry::At_exit
   
  
  #**************************************************************************
  #        Name: Click
  #  Parameters: 
  # Description: Handles a user click
  #**************************************************************************
  proc Click { widget x } \
  {
     upvar #0 ::Entry::entry_data($widget,editon) editon
     # if this variable isn't set up, or this widet's gone, bail.
     if {![info exists editon] || ![winfo exists $widget]} { return }
     
     if {$editon } \
     {
         # edit is open. Position the selection point
         $widget icursor @$x
         Highlight $widget
     }\
     else \
     {
        # is this widget focused? if so, open it. Else focus it
        if {[focus -displayof $widget] != $widget} \
        {
           focus $widget
        } \
        else \
        {
           event generate $widget <<EditOpen>>
        }
     }
  }
  
  
  proc DoubleClick { widget x } \
  {
     upvar #0 ::Entry::entry_data($widget,editon) editon
     # if this variable isn't set up, or this widet's gone, bail.
     if {![info exists editon] || ![winfo exists $widget]} { return }
     
     if {$editon } \
     {
         event generate $widget <<EditClose>>
     } \
     else \
     {
         event generate $widget <<EditOpen>>
     }
  }
  
  
  #**************************************************************************
  #        Name: Show
  #  Parameters: 
  # Description: Creates a diagnostic window with
  #             the edit parameters in it
  #**************************************************************************
  proc Show { {widget ""} } \
  {
    if {"$widget"==""} \
    {
       set widget [focus]
    }
    # create our toplevel
    variable show_count
    set screen [option get . debugScreen {} ]
    set frame [toplevel .edit$show_count -screen $screen -width 100 -height 100 -bd 0]
    incr show_count
    
    wm title $frame "Edit status of $widget"
    label $frame.title -text "Edit status of $widget" -font Helvetica
    grid  $frame.title -row 0 -column 0 -sticky news -columnspan 2
    
    scrollbar $frame.xscroll -orient horiz \
                             -command "$frame.view xview"
    grid  $frame.xscroll -row 2 -column 0 -sticky news
    
    scrollbar $frame.yscroll -orient vert \
                             -command "$frame.view yview"
    grid $frame.yscroll -row 1 -column 1 -sticky news -rowspan 2
    
    frame $frame.f
    
    canvas $frame.view -width 0 -height 0 \
                          -xscrollcommand "$frame.xscroll set" \
                          -yscrollcommand "$frame.yscroll set"
    grid $frame.view -row 1 -column 0 -sticky news
    lower $frame.view
    
    grid rowconfigure $frame 0 -weight 0
    grid rowconfigure $frame 1 -weight 1
    grid rowconfigure $frame 2 -weight 0
  
    grid columnconfigure $frame 0 -weight 1
    grid columnconfigure $frame 1 -weight 0
    
    frame $frame.members
    $frame.view create window 0 0 -anchor nw -window $frame.members -tag members
    raise $frame.members $frame.view
    
    # now, find all edit state variables associated with this widget
    set l [Find_vars $widget]
   
    # now we have our drawing surface. Create the items
    grid propagate $frame.members true
    set row 0
    foreach member [lsort $l] \
    {
      set v ::Entry::entry_data($member)
      set l $frame.members.name$row
      set d $frame.members.value$row
      regexp -- "$widget,(.*)" $member foo piece
      
      label $l -text         $piece  -font Helvetica -relief ridge -bd 1
      label $d -textvariable $v      -font Helvetica -relief ridge -bd 1
      grid $l -in $frame.members -row $row -col 0 -sticky news
      grid $d -in $frame.members -row $row -col 1 -sticky news
      incr row
    }
    # add the text variable
    set l $frame.members.name$row
    set d $frame.members.value$row
    label $l -text   "Text variable"  -font Helvetica -relief ridge -bd 1
    label $d -text [$widget cget -textvariable] -font Helvetica -relief ridge -bd 1
    grid $l -in $frame.members -row $row -col 0 -sticky news
    grid $d -in $frame.members -row $row -col 1 -sticky news
    incr row
    
    # add the value of the text variable
    set l $frame.members.name$row
    set d $frame.members.value$row
    label $l -text   "Value:"  -font Helvetica -relief ridge -bd 1
    label $d -textvariable [$widget cget -textvariable] -font Helvetica -relief ridge -bd 1
    grid $l -in $frame.members -row $row -col 0 -sticky news
    grid $d -in $frame.members -row $row -col 1 -sticky news
    incr row
    
    grid columnconfig $frame.members 0 -weight 0
    grid columnconfig $frame.members 1 -weight 1
    bind $frame.members <Configure> \
    {
       set parent [winfo parent %W]
       set width  [winfo reqwidth  %W]
       set height [winfo reqheight %W]
       set vwidth  [winfo width  $parent.view]
       set vheight [winfo height $parent.view]
       if { $vwidth > $width   }   { set width $vwidth }
       if { $vheight > $height } { set height $vheight}
       $parent.view config -scrollregion "0 0 $width $height"
    }
  }
  
  
};  # End of namespace SpinEntry



# make a version of the bind command visible
proc bind_numeric_spin { widget upper_max lesser_max max min comm_list  \
                                         {editbgcolor {}} {spin_update {1}} } \
{
  #puts "bind_numeric_spin:  $widget,  $upper_max,  $lesser_max,  $max \
  #                             $min,  $comm_list,  $editbgcolor,  $spin_update"
  
  if {"$editbgcolor" == "0" } \
  {
     set editbgcolor ""
     puts "Bogus color for bind_numeric_spin:"
     CallTrace
  }
  
  SpinEntry::Bind $widget "$upper_max" "$lesser_max" "$max" "$min" \
                                     "$comm_list" "$editbgcolor" "$spin_update"
}

