#****************************************************************************
# hexstringentry.tcl  This file changes the bindings on a specific entry 
#                     widget so to imulate a test set data entry field 
#                     allowing hexadecimal digits only.  
#
# This entry differs from the standard hex entry in that the data is
# treated as a string with the upper maximum value limiting the string length.
#
# This entry was created specifically for the use of entering long
# hexidecmal strings for P25 keys.
#
# No arrow digit spin, no negative entry.  The only
# characters allowed are "0123456789ABCDEF".  This imulation
# is detailed as follows:
#     Enter key starts edit:
#        background color is changed
#        redraw softkeys
#        field is blanked
#        entered keys are added to the displayed value
#        key entries are only limited to the number of digits allowed
#           at end of edit, value is reprinted and then limits are
#           enforced.
#        quanity of digits are limited to passed value
#        current entered value is not used in hardware until the edit
#           is finished and the value's limits enforced
#        background color is restored
#        original softkeys are replaced
#        Enter ends edit and reprints formatted value
#        Arrow keys end edit and then move cursor to next field
#
#     GLOBALS USED:
#****************************************************************

#  ENTRY EDIT BINDING VARIABLES


namespace eval ::HexStringEntry \
{
  namespace import ::Entry::Highlight ::Entry::Insert

  bind HexStringEntry <<edit_up>>    "HexStringEntry::Up          %W; break"
  bind HexStringEntry <<edit_down>>  "HexStringEntry::Down        %W; break"
  bind HexStringEntry <<EditOpen>>   "HexStringEntry::start_edit  %W"
  bind HexStringEntry <<EditClose>>  "HexStringEntry::stop_edit   %W"
  bind HexStringEntry <<Validate>>   "HexStringEntry::Validate    %W; break"
  bind HexStringEntry <<EditClear>>  "HexStringEntry::Clear       %W; break"
  bind HexStringEntry <FocusIn>      "HexEntry::FocusIn     %W"
  bind HexStringEntry <FocusOut>     "HexEntry::FocusOut    %W"

  # Add a 'break' at the end of the command in order to limit the
  # binding action to 'HexStringEntry::keypress', and not continue
  # to the 'SpinEntry::keypress'
  foreach  key  { 0 1 2 3 4 5 6 7 8 9 A B C D E F 
                      KP_0 KP_1 KP_2 KP_3 KP_4 KP_5 KP_6 KP_7 KP_8 KP_9 } \
  {
    bind HexStringEntry <Key-$key> "HexStringEntry::keypress %W %A %K; break"
  }

  foreach { key  up } { a A b B c C d D e E f F } \
  {
    bind HexStringEntry <Key-$key> "HexStringEntry::keypress %W $up $up; break"
  }

  foreach { skey  letter } { F2 A F3 B F4 C F5 D F6 E F7 F } \
  {
    bind HexStringEntry <Key-$skey> "event generate %W <Key-$letter>; break"

    bind HexStringEntry <KeyRelease-$skey> "break"
  }


  #*************************************************
  #       Name: Create
  # Parameters: widget
  #Description: Create a HexStringEntry widget
  #*************************************************
  proc Create { widget } \
  {
    #puts "Create:  $widget"

    entry $widget

    Bind $widget
  }
 
 
  #*************************************************
  #       Name: Bind
  # Parameters: widget
  #Description: binds entry widget to handle hex entries 
  #*************************************************
  proc Bind { widget { upper_max "" } { comm_list "" } { editbgcolor "" } } \
  {
    #puts "Bind:  $widget,  $upper_max,  $comm_list,  $editbgcolor"

    if {$upper_max == ""}   { set upper_max   [option get $widget upper_max  {}] }
    if {$comm_list == ""}   { set comm_list   [option get $widget comm_list  {}] }
    if {$comm_list == ""}   { set comm_list   [option get $widget command    {}] }
    if {$editbgcolor == ""} { set editbgcolor [option get $widget editbg     {}] }
  
    # Replace any '%W' designations with the widget tile path
    regsub -all "%W"  $comm_list  $widget  comm_list

    set var  [$widget cget -textvariable]

    set backcolor  [$widget cget -bg]

    if { "$editbgcolor" == "" } \
    {
      set editbgcolor [option get $widget editbg {}]
    }
    set editfgcolor [option get $widget editfg {}]
      
    set ::Entry::entry_data($widget,var)         $var
    set ::Entry::entry_data($widget,upper_max)   $upper_max
    set ::Entry::entry_data($widget,lesser_max)  0
    set ::Entry::entry_data($widget,comm)        [list $comm_list]
    set ::Entry::entry_data($widget,back)        $backcolor
    set ::Entry::entry_data($widget,editbgcolor) $editbgcolor
    set ::Entry::entry_data($widget,editfgcolor) $editfgcolor
    set ::Entry::entry_data($widget,editon)      0
  
    # Clear all current bindings, then set the ones I want
    del_bindtag $widget Entry

    eval $widget configure -takefocus 1 [option get $widget editops {}]

    # Add the necassary bindtags
    add_bindtag $widget  HexStringEntry  SpinEntry

    $widget configure -takefocus 1  -cursor xterm

    
    return
  }


  #*************************************************
  #       Name: Format
  # Parameters: widget to format
  #Description: pads hex value with zeros if needed, limits value if needed
  #*************************************************
  proc Format { widget  value } \
  {
    #puts "Format:  $widget,  $value"

    upvar #0 ::Entry::entry_data($widget,upper_max)  upper_max

    if {"$value" == "" } \
    {
      set value  0
    }
    
    set str_length  [string length $value]
   
    if { $str_length >= $upper_max } \
    {
      # Truncate the value
      set start_index  [expr $str_length - $upper_max]

      set value  [string range  $value  $start_index  $str_length]
    } \
    else \
    {
      # Pad with zeros
      set value  [format "%0${upper_max}s"  "$value"]
    }

    return $value
  }


  #*************************************************
  #       Name: stop_edit
  # Parameters: widget
  #Description: close the edit on the widget
  #*************************************************
  proc stop_edit { widget } \
  {
    #puts "stop_edit:  $widget"

    upvar ::Entry::entry_data($widget,editon) editon
 
    if { [info exists editon] && $editon == 0 } \
    {
      # if not editing then exit
      return
    }

        # The rest of the work is done in SpinEdit
    
    return
  }
 
 
  #*************************************************
  #       Name: Validate
  # Parameters: widget
  #Description: validate the value & commit to HW
  #*************************************************
  proc Validate { widget } \
  {
    #puts "Validate:  $widget"

    upvar #0 ::Entry::entry_data($widget,comm) comm

    # The next 4 lines go away when we start editing
    # The live value rather than a copy
    upvar #0 [$widget cget -textvariable] current
    upvar #0 ::Entry::entry_data($widget,var)  var
    upvar #0 $var value
 
    set value  $current
      
    # Now, handle any final command
    namespace eval :: " eval $comm "
    
    return
  }
 
 
  #*************************************************
  #       Name: start_edit
  # Parameters: widget
  #Description: Open the field for edit 
  #*************************************************
  proc start_edit { widget } \
  {
    #puts "start_edit:  $widget"

    # Do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } { return }

    # Pre-format the value
    upvar #0 ::Entry::entry_data($widget,var)  var
    upvar #0 $var value
   
    set value  [Format $widget $value]
    
    $widget icursor 0

    # The rest of the work is done in SpinEdit
    
    return
  }


  #*************************************************
  #       Name: Clear
  # Parameters: widget,
  #Description: clear the edit field
  #*************************************************
  proc Clear { widget } \
  {
    #puts "Clear:  $widget"

    set var  [$widget cget -textvariable]

    upvar #0 $var value

    set value  [Format $widget 0 ]

    $widget icursor 0

    ::Entry::Highlight $widget
    
    return
  }


  #*************************************************
  #       Name: Down
  # Parameters: widget
  #Description: binds down to entry 
  #*************************************************
  proc Down { widget} \
  {
    #puts "Down:  $widget"

    # Do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } { return }
    
    upvar ::Entry::entry_data($widget,editon) editon
   
    # If not open, open and re-issue
    if { $editon == 0 } \
    {
      event generate $widget <<EditOpen>>

      # Continue editing
      ##event generate $widget <<edit_down>>
      ##return
    }


    upvar #0 ::Entry::entry_data($widget,upper_max) upper_max

    # Get the insert point
    set insert  [$widget index insert]

    # IF MAX NUMBER OF DIGITS ALREADY ENTERED, DO NOTHING
    if { $insert >= $upper_max } \
    {
      return
    }

    upvar #0 [$widget cget -textvariable] value

    set after_name  ::Entry::entry_data($widget,after)

    upvar $after_name after
  
    # Keep it simple. Hex fields act like thumbwheels - they
    # don't carry over.
    set c  [string toupper [string index $value $insert]]

    switch -exact -- $c \
    {
      "0" { set c A }
      "1" { set c 0 } 
      "2" { set c 1 }
      "3" { set c 2 }
      "4" { set c 3 }
      "5" { set c 4 }
      "6" { set c 5 }
      "7" { set c 6 }
      "8" { set c 7 }
      "9" { set c 8 }
      "A" { set c 9 }
      "B" { set c A }
      "C" { set c B }
      "D" { set c C }
      "E" { set c D }
      "F" { set c E }
      default { set c F}
    }
   
    set value  [Format $widget [Entry::Insert $widget $c $insert]]
   
    # If the event isn't scheduled to happen, schedule an after idle
    if { ![info exists after] } \
    {
      # no event scheduled, schedule it
      set after  [ after 100 "unset $after_name; event generate $widget <<Validate>>" ]
    }
    
    return
  }


  #*************************************************
  #       Name: Up
  # Parameters: widget
  #Description: binds up to entry
  #*************************************************
  proc Up { widget } \
  {
    #puts "Up:  $widget"

    # Do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } { return }

    upvar ::Entry::entry_data($widget,editon) editon

    # If not open, open and re-issue
    if { $editon == 0 } \
    {
      event generate $widget <<EditOpen>>

      # Continue editing
      ##event generate $widget <<edit_up>>
      ##return
    }

    upvar #0 ::Entry::entry_data($widget,upper_max) upper_max

    # Get the insert point
    set insert  [$widget index insert]

    # IF MAX NUMBER OF DIGITS ALREADY ENTERED, DO NOTHING
    if { $insert >= $upper_max } \
    {
      return
    }

    upvar #0 [$widget cget -textvariable] value

    set after_name  ::Entry::entry_data($widget,after)

    upvar $after_name after

    # Keep it simple. Hex fields act like thumbwheels - they
    # don't carry over.
    set c  [string toupper [string index $value $insert]]

    switch -exact -- $c \
    {
      "0" { set c 1 }
      "1" { set c 2 } 
      "2" { set c 3 }
      "3" { set c 4 }
      "4" { set c 5 }
      "5" { set c 6 }
      "6" { set c 7 }
      "7" { set c 8 }
      "8" { set c 9 }
      "9" { set c A }
      "A" { set c B }
      "B" { set c C }
      "C" { set c D }
      "D" { set c E }
      "E" { set c F }
      "F" { set c 0 }
      default { set c 0}
    }
   
    set value  [Format $widget [Entry::Insert $widget $c $insert]]
    
    # If the event isn't scheduled to happen, schedule an after idle
    if { ![info exists after] } \
    {
      # No event scheduled, schedule it
      set after  [ after 100 "unset $after_name; event generate $widget <<Validate>>" ]
    }
    
    return
  }


  #*************************************************
  #       Name: keypress
  # Parameters: widget
  #Description: binds entry 
  #*************************************************
  proc keypress { widget  key  keysym } \
  {
    #puts "HexStringEntry::keypress:  $widget,  $key,  $keysym"

    # If we aren't enabled, bail right now
    if { [$widget cget -state] == "disabled" } { return }

    # If the edit isn't open, generate the edit open event, and
    # regenerate this event.
    upvar ::Entry::entry_data($widget,editon) editon

    if { !$editon } \
    {
      event generate $widget <<EditOpen>>
      event generate $widget <<EditClear>>

      # Continue editing
      #event generate $widget <Key-$keysym>
      #return
    }
   
    upvar #0 [$widget cget -textvariable] value
    upvar #0 ::Entry::entry_data($widget,upper_max) upper_max

    # Get the insert point
    set insert  [$widget index insert]

    # IF MAX NUMBER OF DIGITS ALREADY ENTERED, DO NOTHING
    if { $insert >= $upper_max } \
    {
      return
    }
   
    set value  [Format $widget [Entry::Insert $widget $key $insert]]

    $widget icursor [expr $insert + 1]

    Entry::Highlight $widget
    
    return
  }
  
  
};  # End of HexStringEntry namespace


