#****************************************************************
# hexentry  this file changes the bindings on a specific entry 
# widget so to imulate a test set data entry field allowing hexadecimal
# digits only.  No arrow digit spin, no negative entry.  The only
# characters allowed are "0123456789ABCDEF".  This imulation
# is detailed as follows:
#     Enter key starts edit:
#        background color is changed
#        redraw softkeys
#        field is blanked
#        entered keys are added to the displayed value
#        key entries are only limited to the number of digits allowed
#           at end of edit, value is reprinted and then limits are
#           enforced.
#        quanity of digits are limited to passed value
#        current entered value is not used in hardware until the edit
#           is finished and the value's limits enforced
#        background color is restored
#        original softkeys are replaced
#        Enter ends edit and reprints formatted value
#        Arrow keys end edit and then move cursor to next field
#
#     GLOBALS USED:
#****************************************************************

#  ENTRY EDIT BINDING VARIABLES

namespace eval ::HexEntry \
{
  namespace import  ::Entry::Highlight  ::Entry::Clear  ::Entry::Insert

  bind HexEntry <<edit_up>>    "HexEntry::Up          %W; break"
  bind HexEntry <<edit_down>>  "HexEntry::Down        %W; break"
  bind HexEntry <<edit_left>>  "HexEntry::left        %W; break"
  bind HexEntry <<edit_right>> "HexEntry::right       %W; break"
  bind HexEntry <<EditOpen>>   "HexEntry::start_edit  %W"
  bind HexEntry <<Validate>>   "HexEntry::Validate    %W; break"
  bind HexEntry <<EditClear>>  "HexEntry::Clear       %W; break"
  
  bind HexEntry <FocusIn>      "HexEntry::FocusIn     %W"
  bind HexEntry <FocusOut>     "HexEntry::FocusOut    %W"
  bind HexEntry <Key-Return>   "HexEntry::Return      %W"
  bind HexEntry <Key-KP_Enter> "HexEntry::Return      %W"
  
  # auto-cleanup tag to clean up the softkeys on an active HexEntry
  bind HexEntry_focused <Destroy> "HexEntry::FocusOut %W"
  bind HexEntry_focused <Unmap>   "HexEntry::FocusOut %W"
  
  # Bind the numbers and capital letters A, B, C, D, E, F
  # Add a 'break' at the end of the command in order to limit the
  # binding action to 'HexEntry::keypress', and not continue
  # to the 'SpinEntry::keypress'
  foreach key { 0 1 2 3 4 5 6 7 8 9 A B C D E F } \
  { 
     bind HexEntry <Key-$key> "HexEntry::keypress %W %A %K; break"
  }

  # Lower case letters
  # Send the equivalent upper case letter as an arguement
  foreach {key up } { a A b B c C d D e E f F } \
  {
     bind HexEntry <Key-$key> "HexEntry::keypress %W $up $up; break"
  }
  bind HexEntry <Key-period> "break"
  bind HexEntry <Key-minus>  "break"
  bind HexEntry <Key-plus>   "break"

   
  #*************************************************
  #       Name: Create
  # Parameters: widget
  #Description: Create a HexEntry widget
  #*************************************************
   proc Create { widget } \
   {
      entry $widget
      
      Bind $widget
   }
   
  #*************************************************
  #       Name: Bind
  # Parameters: widget
  #Description: binds entry widget to handle hex entries 
  #*************************************************
   proc Bind { widget {upper_max ""} {max ""} {min ""} {comm_list ""} {editbgcolor ""} } \
   {
     if {$upper_max == ""}   { set upper_max   [option get $widget upper_max  {}] }
     if {$max == ""}         { set max         [option get $widget max        {}] }
     if {$min == ""}         { set min         [option get $widget min        {}] }
     if {$comm_list == ""}   { set comm_list   [option get $widget comm_list  {}] }
     if {$comm_list == ""}   { set comm_list   [option get $widget command    {}] }
     if {$editbgcolor == ""} { set editbgcolor [option get $widget editbg     {}] }
    
     # Replace any '%W' designations with the widget tile path
     regsub -all "%W"  $comm_list  $widget  comm_list
  
     set var [$widget cget -textvariable]
     set backcolor [$widget cget -bg]
     if {"$editbgcolor" == ""} {
       set editbgcolor [option get $widget editbg {}]
     }
     set editfgcolor [option get $widget editfg {}]
     set max 0x0$max
     set min 0x0$min
     
     set ::Entry::entry_data($widget,var)         $var
     set ::Entry::entry_data($widget,upper_max)   $upper_max
     set ::Entry::entry_data($widget,lesser_max)  0
     set ::Entry::entry_data($widget,max)         [expr $max]
     set ::Entry::entry_data($widget,min)         [expr $min]
     set ::Entry::entry_data($widget,comm)        [list $comm_list]
     set ::Entry::entry_data($widget,back)        $backcolor
     set ::Entry::entry_data($widget,editbgcolor) $editbgcolor
     set ::Entry::entry_data($widget,editfgcolor) $editfgcolor
     set ::Entry::entry_data($widget,editon)      0
    
     # clear all current bindings, then set the ones I want
     del_bindtag $widget Entry
     
     eval $widget configure -takefocus 1 [option get $widget editops {}]
     
     add_bindtag $widget  HexEntry  SpinEntry
     
     $widget configure -takefocus 1  -cursor xterm
   }
   
   #**************************************************************************
   #        Name: Return
   #  Parameters: widget
   # Description: handles binding for ENTER key
   #**************************************************************************
   proc Return { widget } \
   {
     #puts "HexEntry::Return:  widget = $widget"
     
     upvar ::Entry::entry_data($widget,editon)  editon
     
     if { ![info exists editon] } \
     { 
        set editon  0 
     }
     
     #puts "HexEntry::Return:  editon:  $editon"
     
     # If the <Return> key is hit when an edit is active, reformat the data
     # fields, padding them with leading zeros as needed
     if { $editon } \
     {
        upvar #0 ::Entry::entry_data($widget,var)  var
        upvar #0 $var  value
        set value [Format $widget $value] 
        return
     }
     
     # The rest of the processing is handled in the SpinEntry
   }
  
 

  #*************************************************
  #       Name: FocusIn
  # Parameters: widget receiving focus
  #Description: Show the softkeys for this widget
  #*************************************************
   proc FocusIn { widget } \
   {
      Debug::Puts HexEntry 3 "FocusIn $widget"
      add_bindtag $widget HexEntry_focused
      set hex_keys [Create_hex_keys $widget]
      View::UseSoftkey $hex_keys
   }
   
  #*************************************************
  #       Name: FocusOut
  # Parameters: widget receiving focus
  #Description: Restore the softkeys
  #*************************************************
   proc FocusOut { widget } \
   {
      Debug::Puts HexEntry 3 "FocusOut $widget"
      del_bindtag $widget HexEntry_focused
      set topLevelWin [View::FindSoftKeyOwner $widget]
      set keybar  [name_child $topLevelWin key_bar]
      View::UseSoftkey $keybar
   }
    
  #*************************************************
  #       Name: Create_hex_keys
  # Parameters: widget that needs keys
  #Description: Creates a hex key strip for the
  #             toplevel this widget is a child of
  #             and returns that widget's name
  #*************************************************
   proc Create_hex_keys { widget } \
   {
     set topLevelWin  [View::FindSoftKeyOwner $widget]
     set hex_keys     [name_child $topLevelWin hex_keys]
     if [winfo exists $hex_keys] { return $hex_keys }
      
     set hex_keys [View::CreateSoftkeys $topLevelWin hex_keys]
     
     foreach { key letter } { 2 A 3 B 4 C 5 D 6 E 7 F } \
     {
        set up_cmd   [subst -nocommands {event generate [focus] <KeyRelease-$letter>}]
        set down_cmd [subst -nocommands {event generate [focus] <Key-$letter>}]
        ::View::AssignSoftkey $hex_keys $key $letter $up_cmd $down_cmd
     }
     # map return key identically to normal return.
     set returnkey  [name_child $topLevelWin key_bar.return]
     set return_cmd [$returnkey cget -command]
     ::View::AssignReturn $hex_keys $return_cmd
     return $hex_keys
   }
   
  #*************************************************
  #       Name: Format
  # Parameters: widget to format
  #Description: pads hex value with zeros if needed, limits value if needed
  #*************************************************
   proc Format {widget value } \
   {
     upvar #0 ::Entry::entry_data($widget,upper_max) upper_max
     upvar #0 ::Entry::entry_data($widget,min)       min
     upvar #0 ::Entry::entry_data($widget,max)       max
  
     if {"$value" == "" } \
     {
        set value 0x0
     } \
     else \
     {
        set value 0x$value
     }
     set value [limit_value $value $max $min]
     
     # now make it hex again
     set value [format "%0${upper_max}X" $value]
  
     return $value
   }
   
  #*************************************************
  #       Name: editFormat
  # Parameters: widget to format
  # Description: Handles the format of a HEX number
  #    but it does NOT pad with leading zeros
  #*************************************************
   proc editFormat {widget value } \
   {
     upvar #0 ::Entry::entry_data($widget,upper_max) upper_max
     upvar #0 ::Entry::entry_data($widget,min)       min
     upvar #0 ::Entry::entry_data($widget,max)       max
  
     if {"$value" == "" } \
     {
        set value 0x0
     } \
     else \
     {
        set value 0x$value
     }
     set value [limit_value $value $max $min]
     
     # now make it hex again
     set value [format "%X" $value]
  
     return $value
   }

  #*************************************************
  #       Name: UpdateInsert
  # Parameters: widget
  # Description: Updates the Insert location when an
  #   entry field is converted from a raw entry to
  #   a position entry number, which may add leading
  #   zeros.
  #*************************************************
   proc UpdateInsert { widget } \
   {
      upvar #0 ::Entry::entry_data($widget,upper_max) upper_max
      set var [$widget cget -textvariable]
      upvar #0 $var value

      set len [string length $value]

      # get the insert point
      set insert [$widget index insert]

      # If the lengths do not match, we will be padding the number
      # with leading zeros. We need to adjust the entry and highlight
      # positions after we reformat the number.
      if { $upper_max != $len } \
      {
         set $var [Format $widget $value]
         upvar #0 $var value
         set insert [expr $insert + $upper_max - $len]
         set ::Entry::entry_data($widget,insert) $insert      
         $widget icursor  $insert
         Highlight $widget
      }
   return $insert
   }
   
  #*************************************************
  #       Name: Validate
  # Parameters: widget
  #Description: validate the value & commit to HW
  #*************************************************
   proc Validate { widget } \
   {
     upvar #0 ::Entry::entry_data($widget,comm)  comm
  
     # the next 4 lines go away when we start editing
     # the live value rather than a copy
     upvar #0 [$widget cget -textvariable]  current
     
     upvar #0 ::Entry::entry_data($widget,var)  var
     
     upvar #0 $var  value
     
     set value  $current
        
     # now, handle any final command
     namespace eval :: " eval $comm "
   }
   
  #*************************************************
  #       Name: start_edit
  # Parameters: widget
  #Description: Open the field for edit 
  #*************************************************
  proc start_edit { widget } \
  {
     # do nothing if not enabled for edit
     if {[$widget cget -state] == "disabled"} { return }
  
     upvar #0 ::Entry::entry_data($widget,editbgcolor)  editbgcolor
  
     # pre-format the value
     upvar #0 ::Entry::entry_data($widget,var)  var
     #edit vars not set up? Not, good, try to work around
     if ![info exists var]  \
     {
         set var [$widget cget -textvariable]
     }
     # variable not specified/doesn't exists? try what the widget is set to
     if ![info exists $var] \
     {
         set var [$widget cget -textvariable]
     }
     # last call. Do we have a var to edit?
     if ![info exists $var] \
     {
         puts "Invalid var"
         return
     }
     upvar #0 $var value
     
     set value  [Format $widget $value]
     
     $widget icursor 0
  
     # the rest of the work is done in SpinEntry::start_edit
   }
  
  #*************************************************
  #       Name: Clear
  # Parameters: widget,
  #Description: clear the edit field
  #*************************************************
   proc Clear { widget } \
   {
     set var  [$widget cget -textvariable]

     upvar #0 $var  value

     set value  [editFormat $widget 0 ]

     $widget icursor 0

     Highlight  $widget
   }
  
  #*************************************************
  #       Name: Down
  # Parameters: widget
  #Description: binds down to entry 
  #*************************************************
  proc Down { widget} \
  {
     # do nothing if not enabled for edit
     if {[$widget cget -state] == "disabled"} { return }
     upvar ::Entry::entry_data($widget,editon) editon
     
     # if not open, open and re-issue
     if { $editon == 0 } \
     {
        event generate $widget <<EditOpen>>
        
        # Continue editing
        ##event generate $widget <<edit_down>>
        ##return
     }
  
     upvar #0 [$widget cget -textvariable] value
     # Get the insert point. Exit if we are after the entered text
     set insert [UpdateInsert $widget]
     set len [string length $value]
     if { $len <= $insert } \
     {
        return
     }

     set after_name ::Entry::entry_data($widget,after)
     upvar $after_name after
    
     # keep it simple. Hex fields act like thumbwheels - they
     # don't carry over.
     set c [string toupper [string index $value $insert]]
     
     switch -exact -- $c \
     {
       "0" { set c F }
       "1" { set c 0 } 
       "2" { set c 1 }
       "3" { set c 2 }
       "4" { set c 3 }
       "5" { set c 4 }
       "6" { set c 5 }
       "7" { set c 6 }
       "8" { set c 7 }
       "9" { set c 8 }
       "A" { set c 9 }
       "B" { set c A }
       "C" { set c B }
       "D" { set c C }
       "E" { set c D }
       "F" { set c E }
       default { set c F}
     }

     set value [Format $widget [Insert $widget $c $insert]]

     # if the event isn't scheduled to happen, schedule an after idle
     if {![info exists after] } {
        # no event scheduled, schedule it
        set after [ after 100 "unset $after_name; event generate $widget <<Validate>>" ]
     }
  }
  
  #*************************************************
  #       Name: Up
  # Parameters: widget
  #Description: binds up to entry
  #*************************************************
  proc Up { widget } \
  {
     # do nothing if not enabled for edit
     if {[$widget cget -state] == "disabled"} { return }
     
     upvar ::Entry::entry_data($widget,editon) editon
     
     # if not open, open and re-issue
     if {$editon == 0 } \
     {
        event generate $widget <<EditOpen>>

        # Continue editing
        ##event generate $widget <<edit_up>>
        ##return
     }
  
     upvar #0 [$widget cget -textvariable] value
     # Get the insert point. Exit if we are after the entered text
     set insert [UpdateInsert $widget]
     set len [string length $value]
     if { $len <= $insert } \
     {
        return
     }
     
     set after_name ::Entry::entry_data($widget,after)
     
     upvar $after_name after
  
     # keep it simple. Hex fields act like thumbwheels - they
     # don't carry over.
     set c [string toupper [string index $value $insert]]
     
     switch -exact -- $c {
       "0" { set c 1 }
       "1" { set c 2 } 
       "2" { set c 3 }
       "3" { set c 4 }
       "4" { set c 5 }
       "5" { set c 6 }
       "6" { set c 7 }
       "7" { set c 8 }
       "8" { set c 9 }
       "9" { set c A }
       "A" { set c B }
       "B" { set c C }
       "C" { set c D }
       "D" { set c E }
       "E" { set c F }
       "F" { set c 0 }
       default { set c 0}
     }
     
     set value [Format $widget [Insert $widget $c $insert]]
     
     # if the event isn't scheduled to happen, schedule an after idle
     if {![info exists after] } \
     {
        # no event scheduled, schedule it
        set after [ after 100 "unset $after_name; event generate $widget <<Validate>>" ]
     }
  }
  #**************************************************************************
  #
  #        Name: left
  #
  #  Parameters: widget
  #
  # Description: move selected char left 
  #
  #**************************************************************************
  proc left { widget } \
  {
    #puts "left:  $widget"
    
    # If we aren't enabled, bail right now.
    if { [$widget cget -state] == "disabled" } { return }
  
    # If the edit isn't open, generate the edit open event, and
    # regenerate this event.
    upvar #0 ::Entry::entry_data($widget,editon)  editon
  
    if { ![info exists editon] } { set editon 0 }
  
    if { !$editon } \
    {
      return
    }
    
    upvar #0 [$widget cget -textvariable]  value
    
    set sel  [$widget index insert]
  
    incr sel  -1
  
    # If the selection index is 0, put a '0' at the beginning and move the 
    # cursor to the left if we have not exceeded the field size.
    if { $sel == -1 } \
    {
       upvar #0 ::Entry::entry_data($widget,upper_max) upper_max
       set len [string length $value]
       if {$len < $upper_max} \
       {
          $widget insert  $sel  "0"
          incr sel  -1
       }
    }
  
    $widget icursor  $sel
  
    Highlight $widget
  }
  
  #**************************************************************************
  #
  #        Name: right
  #
  #  Parameters: widget
  #
  # Description: binds entry 
  #
  #**************************************************************************
  proc right { widget } \
  {
    #puts "right:  $widget"
  
    # If we aren't enabled, bail right now
    if { [$widget cget -state] == "disabled" } { return }
    
    # If the edit isn't open, generate the edit open event, and
    # regenerate this event.
    upvar #0 ::Entry::entry_data($widget,editon)  editon
    
    if { ![info exists editon] } { set editon  0 }
    
    if { !$editon } \
    {
      return
    }
    
    # Move the insert point one to the right if we have not exceeded
    # the field size
    set sel  [$widget index insert]
#    upvar #0 ::Entry::entry_data($widget,upper_max) upper_max
    upvar #0 [$widget cget -textvariable]  value
    set max [string length $value]
    if { $sel < $max } \
    {
       incr sel  1
       $widget icursor  $sel
    }

    Highlight $widget
  }
   

  
  #*************************************************
  #       Name: keypress
  # Parameters: widget
  #Description: binds entry 
  #*************************************************
  proc keypress { widget key keysym } \
  {
     #puts "HexEntry::keypress:  $widget,  $key,  $keysym"
  
     # if we aren't enabled, bail right now
     if {[$widget cget -state] == "disabled"} { return }
     
     # if the edit isn't open, generate the edit open event, and
     # regenerate this event.
     upvar ::Entry::entry_data($widget,editon) editon
     
     if {!$editon} \
     { 
       event generate $widget <<EditOpen>>
       event generate $widget <<EditClear>>
       
       # Continue editing
     }
     
     upvar #0 [$widget cget -textvariable] value
     upvar #0 ::Entry::entry_data($widget,upper_max) upper_max
  
     # get the insert point
     set insert [$widget index insert]
  
     # IF MAX NUMBER OF DIGITS ALREADY ENTERED, DO NOTHING
     if {$insert >= $upper_max} \
     {
        return
     }
     
     # If we are editing a field which is currently padded with
     # leading zeros, do not change the edit
     set len [string length $value]
     if { $len == $upper_max } \
     {
        set value [Format $widget [Insert $widget $key $insert]]
     } else \
     {
        set value [editFormat $widget [Insert $widget $key $insert]]
     }
  
     $widget icursor [expr $insert + 1]
  
     Highlight $widget
  
     #Validate $widget
  }


};  # End of HexEntry namespace
  
  
  
# Phone digit entry for P25 downlink, LCO 07
namespace eval ::DigEntry \
{
  namespace import ::Entry::Highlight ::Entry::Clear ::Entry::Insert

  bind DigEntry <<edit_up>>    "DigEntry::Up          %W; break"
  bind DigEntry <<edit_down>>  "DigEntry::Down        %W; break"
  bind DigEntry <<EditOpen>>   "DigEntry::start_edit  %W"
  bind DigEntry <<EditClose>>  "DigEntry::stop_edit   %W"
  bind DigEntry <<Validate>>   "DigEntry::Validate    %W; break"
  bind DigEntry <<EditClear>>  "DigEntry::Clear       %W; break"

  foreach key { 0 1 2 3 4 5 6 7 8 9 A B C D KP_Multiply numbersign } \
  {
    bind DigEntry <Key-$key> "DigEntry::keypress %W %A %K; break"
  }

  foreach { skey letter } { F2 A F3 B F4 C F5 D } \
  {
    bind DigEntry <Key-$skey> "event generate %W <Key-$letter>; break"
    bind DigEntry <KeyRelease-$skey> "break"
  }


  #*************************************************
  #       Name: Create
  # Parameters: widget
  #Description: Create a DigEntry widget
  #*************************************************
   proc Create { widget } \
   {
      entry $widget

      Bind $widget
      
      return  $widget
   }
  
  
  #*************************************************
  #       Name: Bind
  # Parameters: widget
  #Description: binds entry widget to handle entries 
  #*************************************************
   proc Bind { widget {upper_max ""} {comm_list ""} {editbgcolor ""} } \
   {
     if {$upper_max == ""}   { set upper_max   [option get $widget upper_max  {}] }
     if {$comm_list == ""}   { set comm_list   [option get $widget command  {}] }
     if {$editbgcolor == ""} { set editbgcolor [option get $widget editbg   {}] }
  
     # Replace any '%W' designations with the widget tile path
     regsub -all "%W"  $comm_list  $widget  comm_list
  
     set var [$widget cget -textvariable]
     set backcolor [$widget cget -bg]
  
     if {"$editbgcolor" == ""} \
     {
       set editbgcolor [option get $widget editbg {}]
     }
  
     set ::Entry::entry_data($widget,var)         $var
     set ::Entry::entry_data($widget,upper_max)   $upper_max
     set ::Entry::entry_data($widget,comm)        [list $comm_list]
     set ::Entry::entry_data($widget,back)        $backcolor
     set ::Entry::entry_data($widget,editbgcolor) $editbgcolor
     set ::Entry::entry_data($widget,editon)      0
    
     # clear all current bindings, then set the ones I want
     del_bindtag $widget Entry
     
     eval $widget configure -takefocus 1 [option get $widget editops {}]
     
     add_bindtag $widget  DigEntry  SpinEntry  Cursor
     
     $widget configure -takefocus 1
   }


  #*************************************************
  #       Name: Format
  # Parameters: widget to format
  #Description: limits value if needed
  #*************************************************
   proc Format {widget value } \
   {
     upvar #0 ::Entry::entry_data($widget,upper_max) upper_max
  
     set con ""
     set len [string length $value]
  
     if {$len > 10} \
     {
        for { set i 0 } { $i < 10 } { incr i 1 } \
        {
           set val [string index $value $i]
           set con $con$val
        }
  
        set value $con
     }
  
     return $value
   }
  
  
  #*************************************************
  #       Name: stop_edit
  # Parameters: widget
  #Description: close the edit on the widget
  #*************************************************
  proc stop_edit { widget } \
  {
     upvar ::Entry::entry_data($widget,editon) editon
  
     if {[info exists editon] && $editon == 0 } \
     {
        # if not editing then exit
        return
     }
  
     # the rest of the work is done in SpinEdit
   }
  
  
  #*************************************************
  #       Name: Validate
  # Parameters: widget
  #Description: validate the value & commit to HW
  #*************************************************
   proc Validate { widget } \
   {
  
     upvar #0 ::Entry::entry_data($widget,comm) comm
  
     # the next 4 lines go away when we start editing
     # the live value rather than a copy
     upvar #0 [$widget cget -textvariable] current
     upvar #0 ::Entry::entry_data($widget,var)  var
     upvar #0 $var value
     set value $current
        
     # now, handle any final command
     namespace eval :: " eval $comm "
   }
   
  #*************************************************
  #       Name: start_edit
  # Parameters: widget
  #Description: Open the field for edit 
  #*************************************************
  proc start_edit { widget } \
   {
  
     # do nothing if not enabled for edit
     if {[$widget cget -state] == "disabled"} { return }
  
     upvar #0 ::Entry::entry_data($widget,editbgcolor) editbgcolor
  
     # pre-format the value
     upvar #0 ::Entry::entry_data($widget,var)  var
     upvar #0 $var value
  
     set value [Format $widget $value]
  
     $widget icursor 0
  
     # the rest of the work is done in SpinEdit
   }
  
  
  #*************************************************
  #       Name: Clear
  # Parameters: widget,
  #Description: clear the edit field
  #*************************************************
   proc Clear { widget } \
   {
      set var [$widget cget -textvariable]
      upvar #0 $var value
      set value 0
      $widget icursor 0
      Highlight $widget
   }
  
  
  #*************************************************
  #       Name: keypress
  # Parameters: widget
  #Description: binds entry 
  #*************************************************
  proc keypress { widget key keysym} \
  {
     # if we aren't enabled, bail right now
     if {[$widget cget -state] == "disabled"} { return }
     # if the edit isn't open, generate the edit open event, and
     # regenerate this event.
     upvar ::Entry::entry_data($widget,editon) editon
  
     if {!$editon} \
     { 
       event generate $widget <<EditOpen>>
       event generate $widget <<EditClear>>
       
       # Continue editing
       ##event generate $widget <Key-$keysym>
       ##return
     }
  
     upvar #0 [$widget cget -textvariable] value
     upvar #0 ::Entry::entry_data($widget,upper_max) upper_max
  
     # get the insert point
     set insert [$widget index insert]
  
     # IF MAX NUMBER OF DIGITS ALREADY ENTERED, DO NOTHING
     if {$insert >= $upper_max} \
     {
        return
     }
     
     set value [Format $widget [Insert $widget $key $insert]]
  
     $widget icursor [expr $insert + 1]
     Highlight $widget
     Validate $widget
  }
  
  
  #*************************************************
  #       Name: Down
  # Parameters: widget
  #Description: binds down to entry 
  #*************************************************
  proc Down { widget} \
  {
     # do nothing if not enabled for edit
     if {[$widget cget -state] == "disabled"} { return }
  
     upvar ::Entry::entry_data($widget,editon) editon
     
     # if not open, open and re-issue
     if { $editon == 0 } \
     {
       event generate $widget <<EditOpen>>

       # Continue editing
       ##event generate $widget <<edit_down>>
       ##return
     }
  
     # get the insert point
     set insert [$widget index insert]
     upvar #0 [$widget cget -textvariable] value
     set after_name ::Entry::entry_data($widget,after)
     upvar $after_name after
    
     # keep it simple. Act like thumbwheels
     set c [string toupper [string index $value $insert]]
  
     switch -exact -- $c {
       "#" { set c D }
       "*" { set c # }
       "0" { set c * }
       "1" { set c 0 } 
       "2" { set c 1 }
       "3" { set c 2 }
       "4" { set c 3 }
       "5" { set c 4 }
       "6" { set c 5 }
       "7" { set c 6 }
       "8" { set c 7 }
       "9" { set c 8 }
       "A" { set c 9 }
       "B" { set c A }
       "C" { set c B }
       "D" { set c C }
       default { set c D}
     }
  
     set value [Format $widget [Insert $widget $c $insert]]
     # if the event isn't scheduled to happen, schedule an after idle
     if {![info exists after] } \
     {
        # no event scheduled, schedule it
        set after [ after 100 "unset $after_name; event generate $widget <<Validate>>" ]
     }
  }
  
  
  #*************************************************
  #       Name: Up
  # Parameters: widget
  #Description: binds up to entry
  #*************************************************
  proc Up { widget } \
  {
     # do nothing if not enabled for edit
     if {[$widget cget -state] == "disabled"} { return }
  
     upvar ::Entry::entry_data($widget,editon) editon
  
     # if not open, open and re-issue
     if {$editon == 0 } \
     {
       event generate $widget <<EditOpen>>

       # Continue editing
       ##event generate $widget <<edit_up>>
       ##return
     }
  
     # get the insert point
     set insert [$widget index insert]
     upvar #0 [$widget cget -textvariable] value
     set after_name ::Entry::entry_data($widget,after)
     upvar $after_name after
  
     # keep it simple. Act like thumbwheels 
     set c [string toupper [string index $value $insert]]
  
     switch -exact -- $c {
       "#" { set c * }
       "*" { set c 0 }
       "0" { set c 1 }
       "1" { set c 2 } 
       "2" { set c 3 }
       "3" { set c 4 }
       "4" { set c 5 }
       "5" { set c 6 }
       "6" { set c 7 }
       "7" { set c 8 }
       "8" { set c 9 }
       "9" { set c A }
       "A" { set c B }
       "B" { set c C }
       "C" { set c D }
       "D" { set c # }
       default { set c #}
     }
  
     set value [Format $widget [Insert $widget $c $insert]]
  
     # if the event isn't scheduled to happen, schedule an after idle
     if {![info exists after] } \
     {
        # no event scheduled, schedule it
        set after [ after 100 "unset $after_name; event generate $widget <<Validate>>" ]
     }
  }
  
  
};  # End of DigEntry namespace



# make an alias for the old command
proc hex_entry {widget upper_max max min comm_list {editbgcolor ""} } \
{
   ::HexEntry::Bind $widget "$upper_max" "$max" "$min" "$comm_list" "$editbgcolor"
}
