#****************************************************************************  
#
# alphanumentry.tcl:  This file changes the bindings on a specific entry 
#                     widget so to imulate an alphanumeric entry field.  
#                     This imulation is detailed as follows:
#
#     - If numeric keys start edit:
#
#       - Background color is changed.
#
#       - Entered keys are inserted at the cursor position.
#
#       - Enter ends edit, and the background color is restored.
#
#       - Left and Right Arrow keys move the cursor to next field, 
#         left or right respectively.
#
#       - Up and Down keys change the character in the currently highlighted 
#         field, circulating through all 26 letters (A-Z), and 10 numbers (0-9).
#
#       - The spinner has same affect as Up and Down keys.
#
#     - If Enter starts edit:
#
#       - Background color is changed.
#
#       - The last field on the right is highlighted.
#
#       - Left and Right Arrow keys end move the cursor to next field, 
#         left or right respectively.
#
#       - Up and Down keys change the character in the currently highlighted 
#         field, circulating through all 26 letters (A-Z), and 10 numbers (0-9).
#
#       - The spinner has same affect as Up and Down keys.
#
#     - If keyboard presses starts edit:
#
#       - Background color is changed.
#
#       - Typed characters are entered as typed
#
#       - The last field on the right is highlighted.
#
#       - Enter ends edit, and the background color is restored.
#
#       - Left and Right Arrow keys move the cursor to next field, 
#         left or right respectively.
#
#       - Up and Down keys change the character in the currently highlighted 
#         field, circulating through all 26 letters (A-Z), and 10 numbers (0-9).
#
#       - The spinner has same affect as Up and Down keys.
#
#     GLOBALS USED:
#
#****************************************************************************

namespace eval ::AlphaNumEntry \
{
  variable show_count  0
 
  variable Widget_cleanups  ""
   
  namespace import ::Entry::Highlight ::Entry::Clear ::Entry::Insert ::Entry::Find_vars

  # define the characters we will support
  variable supported_chars \
  {
    0 1 2 3 4 5 6 7 8 9
    A B C D E F G H I J K L M N O P Q R S T U V W X Y Z
    a b c d e f g h i j k l m n o p q r s t u v w x y z
  }
  # Need to bind to valid keypresses.
  foreach  { key }  $supported_chars \
  {
    bind AlphaNumEntry  <Key-$key>  "AlphaNumEntry::Keypress  %W  %A  %K"
  }
  # now add the oddballs
  append supported_chars { ( ) - +  _ , . : ; " " / ~ \\ ! \" \$ # & * < = > ? @}
  
  foreach keysym \
    {
       parenleft parenright minus plus underscore comma period colon semicolon space slash
       asciitilde backslash exclam quotedbl dollar numbersign ampersand asterisk less equal
       greater question at
    } \
  {
     bind AlphaNumEntry <Key-$keysym> "AlphaNumEntry::Keypress  %W  %A  %K"
  }
 
  
  bind AlphaNumEntry <Return>     "AlphaNumEntry::Return     %W; break"
  bind AlphaNumEntry <KP_Enter>   "AlphaNumEntry::Return     %W; break"
  
  bind AlphaNumEntry <Escape>     "AlphaNumEntry::Escape     %W; break"
  bind AlphaNumEntry <BackSpace>  "AlphaNumEntry::Backspace  %W; break"
  bind AlphaNumEntry <Delete>     "AlphaNumEntry::Delete     %W; break"
  bind AlphaNumEntry <Home>       "AlphaNumEntry::Home       %W; break"
  bind AlphaNumEntry <End>        "AlphaNumEntry::End        %W; break"

  bind AlphaNumEntry <Destroy>    "AlphaNumEntry::Destroy  %W"

  bind AlphaNumEntry <Button-1>   "AlphaNumEntry::Button_1  %W  %x"


  bind AlphaNumEntry <<edit_up>>     "AlphaNumEntry::Up     %W"
  bind AlphaNumEntry <<edit_down>>   "AlphaNumEntry::Down   %W"
  bind AlphaNumEntry <<edit_left>>   "AlphaNumEntry::Left   %W"
  bind AlphaNumEntry <<edit_right>>  "AlphaNumEntry::Right  %W"

  bind AlphaNumEntry <<EditOpen>>    "AlphaNumEntry::Start_edit  %W"
  bind AlphaNumEntry <<EditClose>>   "AlphaNumEntry::Stop_edit   %W"

  bind AlphaNumEntry <<EditClear>>   "AlphaNumEntry::Clear  %W"
  bind AlphaNumEntry <<EditEscape>>  "AlphaNumEntry::Stop_edit  %W"
 
  bind AlphaNumEntry <<Spin_up>>      { if {[AlphaNumEntry::Spin_up   %W] == 1} { break } }
  bind AlphaNumEntry <<Spin_down>>    { if {[AlphaNumEntry::Spin_down %W] == 1} { break } }
  bind AlphaNumEntry <<Validate>>     { AlphaNumEntry::Validate  %W}



  #***************************************************************************
  #
  #        Name: Create
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Create an AlphaNumEntry widget
  #
  #***************************************************************************
  proc Create { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::Create:  $widget"
    
    entry  $widget
    
    Bind  $widget
    
    return  $widget
  }

   
  #***************************************************************************
  #
  #        Name: Bind
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Bind the entry widget to handle alphanumeric entries
  #
  #***************************************************************************
  proc Bind { widget { comm_list "" } { editbgcolor "" } } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::Bind:  $widget,  $comm_list,  $editbgcolor"

    if { $comm_list   == "" } { set comm_list   [option get $widget comm_list  {}] }
    if { $comm_list   == "" } { set comm_list   [option get $widget command    {}] }
    if { $editbgcolor == "" } { set editbgcolor [option get $widget editbg     {}] }
    
    # Replace any '%W' designations with the widget tile path
    regsub -all "%W"  $comm_list  $widget  comm_list

    set var  [$widget cget -textvariable]

    set backcolor  [$widget cget -bg]
    
    if { "$editbgcolor" == "" } \
    {
      set editbgcolor  [option get $widget editbg {}]
    }
    set editfgcolor  [option get $widget editfg {}]
    
    set ::Entry::entry_data($widget,var)          $var
    set ::Entry::entry_data($widget,comm)         [list $comm_list]
    set ::Entry::entry_data($widget,back)         $backcolor
    set ::Entry::entry_data($widget,editbgcolor)  $editbgcolor
    set ::Entry::entry_data($widget,editfgcolor)  $editfgcolor
    set ::Entry::entry_data($widget,editon)       0
    
    # Clear all current bindings
    del_bindtag $widget Entry
    
    eval $widget configure -takefocus 1 [option get $widget editops {}]
    
    # Add the necessary bindtags
    add_bindtag  $widget  AlphaNumEntry
    
    $widget configure -takefocus 1  -cursor xterm
    
    return
  }

  
  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: widget - invoking widget
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::Destroy:  $widget"

    return
  }


  #***************************************************************************
  #
  #        Name: Clear
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <<EditClear>> event for the entry widget
  #
  #***************************************************************************
  proc Clear { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::Clear:  $widget"

    if { [info exists editon] && $editon == 0 } \
    {
      return
    }
    
    set var  [$widget cget -textvariable]
  
    upvar #0 $var  value
  
    $widget icursor 0
    
    Highlight $widget
    
    return
  }

  
  #***************************************************************************
  #
  #        Name: Return
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <Return> event for the entry widget
  #
  #***************************************************************************
  proc Return { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::Return:  $widget"

    # do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } \
    { 
      return 
    }
    
    upvar ::Entry::entry_data($widget,editon)  editon
    
    if { ![info exists editon] } \
    {
      return
    }
    
    if { $editon } \
    {
      # If open, stop editting
      event generate $widget <<EditClose>>
    } \
    else \
    {
      # If not open, open for editting
      event generate $widget <<EditOpen>>
    }

    return
  }
  
  
  #***************************************************************************
  #
  #        Name: Escape
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <Escape> event for the entry widget
  #
  #***************************************************************************
  proc Escape { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::Escape:  $widget"

    # do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } \
    { 
      return 
    }
    
    upvar ::Entry::entry_data($widget,editon)  editon
    
    # If is open, close
    if { [info exists editon] && $editon == 0 } \
    {
      return
    }

    event generate $widget <<EditClose>>

    return
  }


  #***************************************************************************
  #
  #        Name: Backspace
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <BackSpace> event for the entry widget
  #
  #***************************************************************************
  proc Backspace { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::Backspace:  $widget"

    # do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } \
    { 
      return 
    }
    
    upvar ::Entry::entry_data($widget,editon)  editon
    
    # If not open, return
    if { [info exists editon] && $editon == 0 } \
    {
      return
    }

    upvar #0 [$widget cget -textvariable]  value
    
    set insert  [$widget index insert]

    set str_length  [string length $value]
    
    set first_str_half   [string range $value  0  [expr $insert - 2]]
    set second_str_half  [string range $value  $insert  $str_length]
      
    set new_string  ""
      
    append  new_string  $first_str_half  $second_str_half

    set value  "$new_string"

    $widget icursor [expr $insert - 1]
    
    Highlight $widget

    return
  }


  #***************************************************************************
  #
  #        Name: Delete
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <Delete> event for the entry widget
  #
  #***************************************************************************
  proc Delete { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::Delete:  $widget"

    # do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } \
    { 
      return 
    }
    
    upvar ::Entry::entry_data($widget,editon)  editon
    
    # If is open, perform a delete
    if { [info exists editon] && $editon == 0 } \
    {
      return
    }

    upvar #0 [$widget cget -textvariable]  value
    
    set insert  [$widget index insert]

    set str_length  [string length $value]
    
    set first_str_half   [string range $value  0  [expr $insert - 1]]
    set second_str_half  [string range $value  [expr $insert + 1]  $str_length]
    
    set new_string  ""
    
    append  new_string  $first_str_half  $second_str_half

    set value  "$new_string"

    $widget icursor $insert
    
    Highlight $widget

    return
  }


  #***************************************************************************
  #
  #        Name: End
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <End> event for the entry widget
  #
  #***************************************************************************
  proc End { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::End:  $widget"

    # do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } \
    { 
      return 
    }
    
    upvar ::Entry::entry_data($widget,editon)  editon
    
    # If is open, move to the end of the string
    if { [info exists editon] && $editon == 0 } \
    {
      return
    }

    upvar #0 [$widget cget -textvariable]  value
    
    set str_length  [string length $value]
    
    set insert  $str_length

    $widget icursor $insert
    
    Highlight $widget

    return
  }


  #***************************************************************************
  #
  #        Name: Home
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <Home> event for the entry widget
  #
  #***************************************************************************
  proc Home { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::Home:  $widget"

    # do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } \
    { 
      return 
    }
    
    upvar ::Entry::entry_data($widget,editon)  editon
    
    # If is open, move to the end of the string
    if { [info exists editon] && $editon == 0 } \
    {
      return
    }

    $widget icursor 0
    
    Highlight $widget

    return
  }


  #***************************************************************************
  #
  #        Name: Up
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <<edit_up>> event for the entry widget
  #
  #***************************************************************************
  proc Up { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::Up:  $widget"
    variable supported_chars
    # do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } \
    { 
      return 
    }
    
    upvar ::Entry::entry_data($widget,editon)  editon
    
    # get the insert point
    set insert  [$widget index insert]
    
    upvar #0 [$widget cget -textvariable]  value
    
    set after_name  ::Entry::entry_data($widget,after)
    
    upvar $after_name  after
  
    #upvar ::Entry::entry_data($widget,after)  after_name

    set str_length    [string length $value]
    set present_char  [string index $value $insert]
    
    set first_str_half   [string range $value  0  [expr $insert - 1]]
    set second_str_half  [string range $value  [expr $insert + 1]  $str_length]
    
    set c  $present_char
    set index [lsearch -exact $supported_chars $c]
    incr index
    if {$index > [llength $supported_chars]} \
    {
       set index 0
    }
    set c [lindex $supported_chars $index]

    # Insert the new character
    set new_string  ""

    append  new_string  "$first_str_half"  "$c"  "$second_str_half"
    
    set value  "$new_string"

    # if the event isn't scheduled to happen, schedule an after idle
    if {![info exists after] } \
    {
      # no event scheduled, schedule it
      set after  [after 100 "unset $after_name"]
    }
    
    Highlight $widget

    return
  }


  #***************************************************************************
  #
  #        Name: Down
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <<edit_down>> event for the entry widget
  #
  #***************************************************************************
  proc Down { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::Down:  $widget"
    variable supported_chars
    # do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } \
    { 
      return 
    }
    
    upvar ::Entry::entry_data($widget,editon)  editon
      
    # get the insert point
    set insert  [$widget index insert]
    
    upvar #0 [$widget cget -textvariable]  value
    
    set after_name  ::Entry::entry_data($widget,after)
    
    upvar $after_name  after
    
    #upvar ::Entry::entry_data($widget,after)  after_name

    set str_length    [string length $value]
    set present_char  [string index $value $insert]
    
    set first_str_half   [string range $value  0  [expr $insert - 1]]
    set second_str_half  [string range $value  [expr $insert + 1]  $str_length]
    
    set c  $present_char
    set index [lsearch -exact $supported_chars $c]
    incr index -1
    if {$index < 0} \
    {
       set index end
    }
    set c [lindex $supported_chars $index]

    # Insert the new character
    set new_string  ""

    append  new_string  "$first_str_half"  "$c"  "$second_str_half"
    
    set value  "$new_string"

    # if the event isn't scheduled to happen, schedule an after idle
    if {![info exists after] } \
    {
      # no event scheduled, schedule it
      set after  [after 100 "unset $after_name"]
    }
    
    Highlight $widget

    return
  }
  
  
  #***************************************************************************
  #
  #        Name: Left
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <<edit_left>> event for the entry widget
  #
  #***************************************************************************
  proc Left { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::Left:  $widget"

    # do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } \
    { 
      return 
    }
    
    upvar ::Entry::entry_data($widget,editon)  editon
     
    # If not open, return
    if { [info exists editon] && $editon == 0 } \
    {
      return
    }

    set sel  [$widget index insert]

    incr sel  -1
    
    if { $sel < 0 } \
    {
      set sel  0
      
      #upvar #0 [$widget cget -textvariable]  value
      # 
      #set new_string  ""
      #
      #append  new_string  " "  "$value"
      #
      #set value  "$new_string"
      #
      #set sel  0
    }
      
    $widget icursor $sel
  
    Highlight $widget

    return
  }
  
  
  #***************************************************************************
  #
  #        Name: Right
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <<edit_right>> event for the entry widget
  #
  #***************************************************************************
  proc Right { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::Right:  $widget"

    # do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } \
    { 
      return 
    }
    
    upvar ::Entry::entry_data($widget,editon)  editon
     
    # If not open, return
    if { [info exists editon] && $editon == 0 } \
    {
      return
    }

    set sel  [$widget index insert]

    incr sel  1
  
    upvar #0 [$widget cget -textvariable]  value
    
    set str_length  [string length $value]
    
    if { $sel >= $str_length } \
    {
      # Insert a space at the end
      append  value  " "
    }

    $widget icursor $sel
  
    Highlight $widget

    return
  }
  
  
  #***************************************************************************
  #
  #        Name: Keypress
  #
  #  Parameters: widget - invoking widget
  #             key    -
  #             keysym -
  #
  # Description: Handle the <KeyPress> event for the entry widget
  #
  #***************************************************************************
  proc Keypress { widget  key  keysym } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::Keypress:  $widget,  $key"

    # if we aren't enabled, bail right now
    if { [$widget cget -state] == "disabled" } \
    { 
      return
    }
    
   
    # If the edit isn't open, generate the edit open event, and
    # regenerate this event.
    upvar ::Entry::entry_data($widget,editon)  editon

    if { [info exists editon] && $editon == 0 } \
    {
      # clear any current valie
      $widget delete 0 end
      event generate $widget <<EditOpen>>
      
      # Continue with the editting
      event generate $widget <Key-$keysym>
      return
    }
    
    # Get the insert point
    set insert  [$widget index insert]
  

    upvar #0 [$widget cget -textvariable]  value
    
    set str_length  [string length $value]
    
    set first_str_half   [string range $value  0  [expr $insert - 1]]
    set second_str_half  [string range $value  $insert  $str_length]
    
    # Insert the new character
    set new_string  ""

    append  new_string  $first_str_half  $key  $second_str_half
    
    set value  "$new_string"


    $widget icursor  [expr $insert + 1]
    
    Highlight $widget
    
    return -code break
  }
  
  
  #***************************************************************************
  #
  #        Name: Spin_up
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <<Spin_up>> event for the entry widget
  #              Generates the <<edit_up>> event for changes in the spin dial.
  #
  #***************************************************************************
  proc Spin_up { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "Spin_up:  $widget"
    
    # If the widget's gone away, bail
    if { ![winfo exists $widget ] } \
    { 
      return 0
    }
    
    # If we aren't enabled, bail right now
    if { [$widget cget -state] == "disabled" } \
    { 
      return 0
    }
    
    # If the edit isn't open, return 0
    upvar ::Entry::entry_data($widget,editon)  editon

    if { ![info exists editon] } \
    { 
      set editon  0
    }
    
    if { !$editon } \
    { 
      return 0 
    }

    # Generate an <<edit_up>> event.
    event generate $widget <<edit_up>>
    
    return 1
  }


  #***************************************************************************
  #
  #       Name: Spin_down
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <<Spin_up>> event for the entry widget
  #              Generates the <<edit_down>> event for changes in the spin dial.
  #
  #***************************************************************************
  proc Spin_down { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "Spin_down:  $widget"

    # If the widget's gone away, bail
    if { ![winfo exists $widget ] } \
    { 
      return 0
    }
    
    # If we aren't enabled, bail right now
    if { [$widget cget -state] == "disabled" } \
    { 
      return 0
    }
    
    # If the edit isn't open, return 0
    upvar ::Entry::entry_data($widget,editon)  editon
    
    if { ![info exists editon] } \
    { 
      set editon  0
    }
    
    if { !$editon } \
    { 
      return  0 
    }

    # Generate an <<edit_down>> event.
    event generate $widget <<edit_down>>
    
    return  1
  }


  #**************************************************************************
  #
  #        Name: Stop_edit
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <<EditClose>> event for the entry widget.
  #              Returns the background and selectbackground colors
  #              to the orignal state, and sets the edit mode to 0 (OFF).
  #
  #**************************************************************************
  proc Stop_edit { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::Stop_edit:  $widget"

    if { [info exists editon] && $editon == 0 } \
    {
      # If not editing then exit
      return
    }

    upvar ::Entry::entry_data($widget,editon)  editon
    upvar ::Entry::entry_data($widget,back)    backcolor
  
    bind $widget <FocusOut> {}

    # Stop the insert point from showing
    $widget configure -insertofftime 1000
    $widget configure -insertontime  0

    $widget select clear   

    $widget config -bg $backcolor -selectbackground $backcolor
    
    set editon  0
    
    # Trim any space off of both ends of the string
    upvar #0 [$widget cget -textvariable]  value
    
    set value  [string trim  $value]
    event generate $widget <<Validate>>
    return
  }
  #**************************************************************************
  #        Name: Validate
  #  Parameters: widget
  # Description: validate the value & send to hardware
  #**************************************************************************
  proc Validate { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "Validate:  $widget"
    
    upvar ::Entry::entry_data($widget,lesser_max)  lesser_max
    upvar ::Entry::entry_data($widget,max)         max
    upvar ::Entry::entry_data($widget,min)         min
    upvar ::Entry::entry_data($widget,comm)        comm
    upvar ::Entry::entry_data($widget,original)    original
  
    upvar #0 [$widget cget -textvariable]  value

    # Now, handle any final command
    if { $comm != "" } \
    {
      namespace eval :: " eval $comm "
    }
    
    return
  }
  
  #**************************************************************************
  #
  #        Name: Start_edit
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <<EditOpen>> event for the entry widget
  #              Sets the background and selectbackground colors
  #              to the edit colors, and sets the edit mode to 1 (ON).
  #
  #***************************************************************************
  proc Start_edit { widget } \
  {
    Debug::Puts AlphaNumEntry 2 "AlphaNumEntry::Start_edit:  $widget"

    # do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } \
    { 
      return 
    }
  
    upvar ::Entry::entry_data($widget,editon)  editon
    upvar #0 ::Entry::entry_data($widget,editbgcolor)  editbgcolor
  
    set editon  1

    bind $widget <FocusOut> "event generate %W <<EditClose>>"

    $widget config -bg $editbgcolor \
                              -selectbackground [option get $widget edisel {}]

    $widget configure -insertofftime 500
    $widget configure -insertontime  500
  
    Highlight $widget

    return
  }

 
  #***************************************************************************
  #
  #        Name: Button_1
  #
  #  Parameters: widget - invoking widget
  #             x      - mouse position x value
  #
  # Description: Handle the <<Button-1>> event for the entry widget
  #              Sets the insert cursor to the position of the mouse x
  #              coordinate, then generates the <<EditOpen>> event if the
  #              edit mode is 0 (OFF)
  #
  #***************************************************************************
  proc Button_1 { widget  x } \
  {
    Debug::Puts AlphaNumEntry 2 "Button_1:  $widget,  $x"
    
    upvar #0 ::Entry::entry_data($widget,editon)  editon
  
    # if this variable isn't set up, or this widet's gone, bail.
    if { ![info exists editon] || ![winfo exists $widget] } \
    { 
      return 
    }
    
    # Position the selection point
    $widget icursor @$x
    
    
    # If the insert cursor is to the far right,
    # adjust it to the last visible character
    set insert  [$widget index insert]

    upvar #0 [$widget cget -textvariable]  value
    
    set str_length  [string length $value]
    
    if { $insert == $str_length } \
    {
      incr insert  -1
      
      $widget icursor  $insert
    }

    Highlight $widget
      

    if { $editon == 0 } \
    {
      # Start the edit
      # Generate an <<EditOpen>> event
      event generate $widget <<EditOpen>>
    }
    
    return
  }
  

  #***************************************************************************
  #
  #        Name: Highlight
  #
  #  Parameters: widget - invoking widget
  #
  # Description: shows currently selected char 
  #
  #***************************************************************************
  proc Highlight { widget } \
  {
    # now, get where the cursor really is
    set sel  [$widget index insert]
 
    $widget selection range $sel [expr $sel + 1]
  }


};  # End of AlphaNumEntry

 
 
 
 
 
 
 
 
