#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998,99,00      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : 2110.lineplot.tcl
# PRODUCT     : FITS                                                        #
# AUTHOR      : Steve Layton                                               #
# DESCRIPTION : mimics a plot widget.
#        NOTE : Does not plot multiple sets of data, one set at a time
#############################################################################


#############################################################################
#
#  lineplot documentation
#
#  Lineplot plots one set of data at a time.
#
#  To create a plot:
#
#     lineplot name ?option? ?value? ...
#
#  lineplot operations
#
#     $w add {data}                 Adds data to plot
#
#     $w cget option                Return the value of the specified
#                                   attribute
#
#     $w configure ?option? ?value? Query or manipulate the configuration
#                                   information of the lineplot
#
#     $w erase {data}               omits data from plot
#
#
#
#  lineplot attributes
#
#     autosizex         If "true", calculates maximum and minimum X
#                       values to plot based on current data plotted.
#                       If minimum value is greater than or equal to 
#                       zero, zero is used. 
#                       If "false", maxx is used as the maximum X
#                       value plotted, minx is used as the minimum
#                       value plotted.
#
#     autosizey         If "true", calculates maximum and minimum Y
#                       values to plot based on current data plotted.
#                       If minimum value is greater than or equal to 
#                       zero, zero is used.
#                       If "false", maxy is used as the maximum Y
#                       value plotted, miny is used as the minimum
#                       value plotted.
#
#     axiscolor         Color of X axis, Y axis and tic marks on both
#                       axes.
#
#     background        Color of background.
#
#     font              Font used for legend text.
#
#     height            Height of plot widget.
#
#     legendx           List printed in X axis legend.  If legendx is
#                       an empty set, actual plot minimum and maximum
#                       values are used. If ticspacex is implicit, a
#                       tic mark is generated for each legend value
#                       except first (origin value). Not used when 
#                       autosizex is true.
#
#     legendy           List printed in Y axis legend.  If legendy is
#                       an empty set, actual plot minimum and maximum
#                       values are used. If ticspacey is implicit, a 
#                       tic mark is generated for each legend value
#                       except first (origin value). Not used when 
#                       autosizey is true.
#
#     line              Plots line connecting points if "true".
#
#     maxx              Maximum X value plotted if autosizey is false.
#
#     maxy              Maximum Y value plotted if autosizey is false.
#
#     minx              Minimum X value plotted if autosizex is false
#
#     miny              Minimum Y value plotted if autosizex is false
#
#     originplacement   Placement of x axis.  "bottom" for x axis along
#                       bottom. "middle" for x axis across middle of plot.
#
#     pointcolor        Color of points and line connecting points.
#
#     pointshape        Shape of points plotted {x, square, circle, dot}
#
#     smooth            "true" for spline curve, "false" for point to
#                       point line.
#     splinesteps       Number of spline steps used when line smooth is
#                       true.
#
#     ticcountx         Number of tic marks drawn along the X axis
#                       between the origin and the maxx or largest x
#                       value if ticspacex is explicit.
#
#     ticcounty         Number of tic marks drawn along the Y axis
#                       between the origin and the maxy or largest y
#                       value if ticspacey is explicit.
#
#     ticspacex         If "explicit", the ticcountx number of tic marks
#                       are used.  If "implicit" and autosizex is false,
#                       a tic mark is drawn for each ledgendx value.
#
#     ticspacey         If "explicit", the ticcounty number of tic marks
#                       are used.  If "implicit" and autosizey is false,
#                       a tic mark is drawn for each ledgendy value.
#
#     unitx             Text printed at end of the X axis legend.
#
#     unity             Text printed at end of the Y axis legend.
#
#     width             Width of plot widget.
#
#############################################################################

namespace eval lineplot \
{

   variable Created
   variable LastCreated
   
   array set Created {}
   set LastCreated 1
   
#*************************************************
#       Name: Find_largest
# Parameters: name, name of lineplot
#Description: finds maximum x and y values of lineplot data
#           : sets actual_max values
#*************************************************
proc Find_largest {name} \
{
   variable Created
   upvar #0 $Created($name) array_var

   # FIND LARGEST X AND LARGEST Y AND WIDEST Y
   set temp_x 0
   set temp_y 0
   set wide_y 0
   
   if {[llength $array_var(data)] > 0} {
      set low_x [lindex $array_var(data) 0]
      set low_y [lindex $array_var(data) 1]
   } else {
      set low_x 0
      set low_y 0
   }  
   
   set data_list $array_var(data)

   foreach {xvalue yvalue} $data_list  {
      if {$xvalue > $temp_x} {
         set temp_x $xvalue
      }
      if {$yvalue > $temp_y} {
         set temp_y $yvalue
      }
      if {$xvalue < $low_x} {
         set low_x $xvalue
      }
      if {$yvalue < $low_y} {
         set low_y $yvalue
      }
   }

   set array_var(largest_x) $temp_x     
   set array_var(largest_y) $temp_y
   set array_var(smallest_x) $low_x
   set array_var(smallest_y) $low_y  

   if {[font measure $array_var(font) $array_var(unity)] > $wide_y} {
      set wide_y [font measure $array_var(font) $array_var(unity)]
   }

   set legend_list $array_var(legendy)
   foreach value $legend_list {
      set legend_width [font measure $array_var(font) $value]
      if {$legend_width > $wide_y} {
         set wide_y $legend_width
      }
   }

   # SET actual_maxx AND actual_maxy
   if {$array_var(autosizex) == "true"} {
      set array_var(actual_maxx) [expr 1.05*$array_var(largest_x)]
      if {$low_x < 0} {
         set array_var(actual_originx) [expr floor(1.05*$array_var(smallest_x))]
      } else {
         set array_var(actual_originx) 0
      }
   } else {
      set array_var(actual_maxx) [expr $array_var(maxx)*1.05]
      set array_var(actual_originx) $array_var(minx)
   }     
   if {$array_var(autosizey) == "true"} {
      set array_var(actual_maxy) [expr $array_var(largest_y)*1.05]
      # KEEP LARGEST Y FOR Y LEGEND WIDTH COMPARISONS
      set biggest_y $array_var(largest_y)

      if {$low_y < 0} {
         set array_var(actual_originy) [expr floor(1.05*$array_var(smallest_y))]
      } else {
         set array_var(actual_originy) 0
      }
   } else {
      # KEEP LARGEST Y FOR Y LEGEND WIDTH COMPARISONS
      set biggest_y $array_var(maxy)

      set array_var(actual_maxy) [expr 1.05*$array_var(maxy)]
      set array_var(actual_originy) $array_var(miny)
   }     

   # CHECK ORIGIN Y AND ACTUAL_MAXY FOR WIDTH
   if {($array_var(autosizey) == "true" || [llength $array_var(legendy)] < 2) && [font measure $array_var(font) $array_var(actual_originy)] > $wide_y} {
      set wide_y [font measure $array_var(font) $array_var(actual_originy)]
   }
   if {([llength $array_var(legendy)] < 1 || $array_var(autosizey) == "true") && [font measure $array_var(font) $biggest_y] > $wide_y} {
      set wide_y [font measure $array_var(font) $biggest_y]
   }

   set array_var(widest_y) $wide_y         
}

#*************************************************
#       Name: Sort
# Parameters: list of data
#Description: sorts list by x values, deletes same points
#*************************************************
proc Sort {data} {

   set length [llength $data]
      
   for {set pass 1} {$pass <= [expr $length-2]} {incr pass 2} {
      for {set index 0} {$index <= [expr $length-4]} {incr index 2} {
         if {[lindex $data $index] > [lindex $data [expr $index+2]]} {
            set tempx [lindex $data $index]
            set tempy [lindex $data [expr $index+1]]
            set data [lreplace $data $index [expr $index+1] [lindex $data [expr $index+2]] [lindex $data [expr $index+3]]]
            set data [lreplace $data [expr $index+2] [expr $index+3] $tempx $tempy]
         }
      }
   }
   
   # DELETE SAME POINTS
   set do_again 1
   while {$do_again} {
      set do_again 0
      set length [llength $data]
      for {set index 0} {$index <= [expr $length-4]} {incr index 2} {
         if {[lindex $data $index] == [lindex $data [expr $index+2]] && [lindex $data [expr $index+1]] == [lindex $data [expr $index+3]]} {
            set data [lreplace $data [expr $index+2] [expr $index+3]]
            set do_again 1
            break
         }
      }
   }

   return $data
}

#*************************************************
#       Name: Omit_same
# Parameters: data, list of data to omit out of
#           : items, list of data to look for
#Description: finds items's elements inside data, omits them from
#           : within data
#************* ************************************
proc Omit_same {data items} {

   if {$data != ""} {
      foreach {xcheck ycheck} $items {
         set data_index 0
         foreach {xvalue yvalue} $data {
            if {$xvalue == $xcheck && $yvalue == $ycheck} {
               set data [lreplace $data $data_index [expr $data_index+1]]
               continue
            }
            incr data_index 2
         }
      }
   }
   return $data
}


#*************************************************
#       Name: Create_frames
# Parameters: name
#Description: draws lineplot frames
#*************************************************
proc Create_frames {name } {

   variable Created
   upvar #0 $Created($name) array_var


   $name config -height $array_var(height) -width $array_var(width)

   # DESTROY OUTER FRAME IF PRESENT
   destroy $name.up
   destroy $name.low

   frame $name.up -height [expr $array_var(height)-$array_var(legendheightx)] -width $array_var(width) -bg $array_var(background)
   pack propagate $name.up false
   pack $name.up -expand true -fill both

   frame $name.low -height $array_var(legendheightx) -width $array_var(width) -bg $array_var(background)
   pack propagate $name.low false
   pack $name.low -expand true -fill both
   
   frame $name.up.side -height [expr $array_var(height)-$array_var(legendheightx)] -width $array_var(legendwidthy) -bg $array_var(background)
   pack propagate $name.up.side false
   pack $name.up.side -expand true -fill both -side left

   frame $name.up.plot -height [expr $array_var(height)-$array_var(legendheightx)] -width [expr $array_var(width)-$array_var(legendwidthy)] -bg $array_var(background)
   pack propagate $name.up.plot false
   pack $name.up.plot -expand true -fill both -side right

}

#*************************************************
#       Name: Create_array
# Parameters: name, name of lineplot
#Description: create lineplotX array with default values
#*************************************************
proc Create_array {name} {
   variable Created
   variable LastCreated

   # IF NAME IS IN Created, REMOVE IT AND THE ARRAY NAME FROM
   # LIST OF CREATED LINEPLOTS
   if [info exists Created($name)] \
   {
      puts "found existing lineplot $name - removing"
      unset $Created($name)
      unset Created($name)
   }
   
   # DECIDE NEW ARRAY NAME, ADD TO Created
   incr LastCreated
   set array_name ::lineplot::$LastCreated
   set Created($name) $array_name
   
   puts "creating lineplot array $array_name"
   upvar #0 $Created($name) array_var
      
   
   # VALUES THAT CAN BE CONFIG AND CGET
   set array_var(width) 540       ;# total width of lineplot
   set array_var(height) 400      ;# total height of lineplot
   set array_var(background) white;# plot background color
   set array_var(axiscolor) black ;# axis and tic color
   set array_var(pointcolor) red  ;# color of points plotted and connecting line
   set array_var(autosizex) true  ;# size plot to 5% below lowest x to 5% above highest x, lower 5% ommitted if zero fits
   set array_var(autosizey) true  ;# size plot to 5% below lowest x to 5% above highest x, lower 5% ommitted if zero fits
   set array_var(maxx) 10         ;# if autosizex is false, marks max x value of plot scale
   set array_var(maxy) 10         ;# if autosizey is false, marks max y value of plot scale
   set array_var(originplacement) bottom   ;# {middle bottom} placement of x origin
   set array_var(minx) 0          ;# value of x origin
   set array_var(miny) 0          ;# value of y origin 
   set array_var(line) true       ;# {true false} connects points plotted
   set array_var(smooth) true     ;# {true false} line smoothness that connects points
   set array_var(splinesteps) 20  ;# number of splinesteps used in smoothing line
   set array_var(pointshape) x    ;# {dot x circle square} graphical representation of each point of data
   set array_var(legendx) {}      ;# list of values along axis evenly distributed
   set array_var(legendy) {}      ;# list of values along axis evenly distributed
   set array_var(font) lineplot_font ;# font of legend
   set array_var(unitx) {}        ;# unit displayed at end of axis
   set array_var(unity) {}        ;# unit displayed at end of axis
   set array_var(ticcountx) 0     ;# number of tics for x axis
   set array_var(ticcounty) 0     ;# number of tics for y axis
   set array_var(ticspacex) implicit ;# {explicit implicit} explicit uses specified values, implicit is one for each unit
   set array_var(ticspacey) implicit ;# {explicit implicit} explicit uses specified values, implicit is one for each unit

   # INTERNAL VALUES
   set array_var(class) lineplot  ;# class, lineplot 
   set array_var(legendwidthy) 0  ;# width of y legend
   set array_var(legendheightx) 0 ;# height of x legend   
   set array_var(largest_x) 0     ;# largest x value among currently plotted data 
   set array_var(largest_y) 0     ;# largest y value among currently plotted data
   set array_var(smallest_x) 0    ;# smallest x value among currently plotted data   
   set array_var(smallest_y) 0    ;# smallest y value among currently plotted data
   set array_var(widest_y) 0      ;# widest text in Y legend
   set array_var(data) {}         ;# data list
   set array_var(actual_maxx) 0   ;# actual maximum x value, used when autosizex is true
   set array_var(actual_maxy) 0   ;# actual maximum y value, used when autosizey is true   
   set array_var(actual_originx) 0;# actual maximum x value, used when autosizex is true
   set array_var(actual_originy) 0;# actual maximum y value, used when autosizey is true   
   set array_var(line_tag) 0      ;# actual maximum y value, used when autosizey is true
   
   set array_var(takefocus) 0
   set array_var(state) normal

}


#*************************************************
#       Name: Default
# Parameters: index, array index
#Description: returns default value for lineplot array index
#*************************************************
proc Default {index} {

   switch -exact -- $index {
   
      width {return 540}            
      height {return 400}     
      background {return white}
      axiscolor {return black}
      pointcolor {return red}
      autosizex {return true}    
      autosizey {return true}       
      maxx {return 15}        
      maxy {return 10}        
      originplacement {return bottom}
      minx {return 0} 
      miny {return 0} 
      line {return true}
      smooth {return true} 
      splinesteps {return 20}     
      pointshape {return x}      
      legendx {return ""}        
      legendy {return ""}        
      font {return lineplot_font}
      unitx {return ""}          
      unity {return ""}          
      ticcountx {return 0}       
      ticcounty {return 0}       
      ticspacex {return implicit}
      ticspacey {return implicit}
      class {return lineplot}    
      legendwidthy {return 0}     
      legendheightx {return 0}   
      largest_x {return 0}        
      largest_y {return 0}       
      smallest_x {return 0}      
      smallest_y {return 0}      
      widest_y {return 0}        
      data {return ""}           
      actual_maxx {return 0}     
      actual_maxy {return 0}     
      actual_originx {return 0}     
      actual_originy {return 0}     
      line_tag {return 0}        
   }

   return "no default for $index"
}


#*************************************************
#       Name: Redraw_all?
# Parameters: name, lineplot name
#           : arg_list, argument list received when config was called
#Description: checks lineplot arguments to see if redraw of all 3
#           : regions is needed
#*************************************************
proc Redraw_all? {name arg_list} {

   set redraw_list [list -width -height -autosizex -autosizey -originplacement -unity -legendy -font -minx -miny]

   foreach option $arg_list {
      if {[lsearch -exact $redraw_list $option] != -1} {
         return 1
      }
   }

   variable Created
   upvar #0 $Created($name) array_var

   if {[lsearch -exact $arg_list -data] != -1 && ($array_var(autosizex) == "true" || $array_var(autosizey) == "true")} {
      return 1
   }

   if {[lsearch -exact $arg_list -maxx] != -1 && $array_var(autosizex) != "true"} {
      return 1
   }

   if {[lsearch -exact $arg_list -maxy] != -1 && $array_var(autosizey) != "true"} {
      return 1
   }

   if {[lsearch -exact $arg_list -minx] != -1 && $array_var(autosizex) != "true"} {
      return 1
   }

   if {[lsearch -exact $arg_list -miny] != -1 && $array_var(autosizey) != "true"} {
      return 1
   }

#  if {[lsearch -exact $arg_list -legendx] != -1 && $array_var(ticspacex) != ""} {
#     return 1
#  }

#  if {[lsearch -exact $arg_list -legendy] != -1 && $array_var(ticspacey) != ""} {
#     return 1
#  }

   if {[lsearch -exact $arg_list -ticcountx] != -1 && $array_var(ticspacex) != "implicit"} {
      return 1
   }

   if {[lsearch -exact $arg_list -ticcounty] != -1 && $array_var(ticspacey) != "implicit"} {
      return 1
   }

   return 0
}

#*************************************************
#       Name: Redraw_plot?
# Parameters: name, lineplot name
#           : arg_list, argument list received when config was called
#Description: checks lineplot arguments to see if redraw of plot area
#           : is needed
#*************************************************
proc Redraw_plot? {name arg_list} {

   set redraw_list [list -axiscolor -pointcolor -pointshape -ticspacex -ticspacey]

   foreach option $arg_list {
      if {[lsearch -exact $redraw_list $option] != -1} {
         return 1
      }
   }

   variable Created
   upvar #0 $Created($name) array_var

   if {[lsearch -exact $arg_list -legendx] != -1 && $array_var(ticspacex) != "explicit"} {
      return 1
   }

   if {[lsearch -exact $arg_list -legendy] != -1 && $array_var(ticspacey) != "explicit"} {
      return 1
   }

   if {[lsearch -exact $arg_list -ticcountx] != -1 && $array_var(ticspacex) != "implicit"} {
      return 1
   }

   if {[lsearch -exact $arg_list -ticcounty] != -1 && $array_var(ticspacey) != "implicit"} {
      return 1
   }

   return 0
}

#*************************************************
#       Name: Config_value_check
# Parameters: element, option name without dash
#           : v alue, passed value for option
#Description: checks value to a legal value for the option
#           : returns 1 if illegal value, 0 otherwise
#*************************************************
proc Config_value_check {element value} {

   upvar name name

   switch -exact -- $element {
      autosizex {if {$value != "true" && $value != "false"} {
                  puts "bad autosizex value \"$value\": must be true or false"
                  return 1}}
      autosizey {if {$value != "true" && $value != "false"} {
                  puts "bad autosizey value \"$value\": must be true or false"                  
                  return 1}}
      line        {if {$value != "true" && $value != "false"} {
                  puts "bad line value \"$value\": must be true or false"                 
                  return 1}} 
      smooth      {if {$value != "true" && $value != "false"} {
                  puts "bad smooth value \"$value\": must be true or false"                  
                  return 1}}
      originplacement  {if {$value != "middle" && $value != "bottom"} {
                  puts "bad originplacement type \"$value\": must be bottom or middle"                
                  return 1}}
                  
      ticspacex {if {$value != "explicit" && $value != "implicit"} {
                  puts "bad ticspacex type \"$value\": must be implicit or explicit"                  
                  return 1}}
      ticspacey {if {$value != "explicit" && $value != "implicit"} {
                  puts "bad ticspacey type \"$value\": must be implicit or explicit"
                  return 1}}     
      pointshape {if {$value != "x" && $value != "circle" && $value != "square" && $value != "dot"} {
                  puts "bad pointshape type \"$value\": must be x, square, circle or dot"
                  return 1}}
      font      {if {[lsearch -exact [font names] $value] == -1} {
                  puts "bad font \"$value\": current avaialble fonts are [font names]"
                  return 1}}
      axiscolor {set parent [string range $name 0 [expr [string last "." $name]-1]]
                 if {$parent == ""} {set parent .}
                 if {[string first "unknown color" [winfo rgb $parent $value]] != -1} {
                  puts "unknown color name \"$value\""                  
                  return 1}}
      pointcolor {set parent [string range $name 0 [expr [string last "." $name]-1]]
                 if {$parent == ""} {set parent .}
                 if {[string first "unknown color" [winfo rgb $parent $value]] != -1} {
                  puts "unknown color name \"$value\""                  
                  return 1}}
      background {set parent [string range $name 0 [expr [string last "." $name]-1]]
                 if {$parent == ""} {set parent .}
                 if {[string first "unknown color" [winfo rgb $parent $value]] != -1} {
                  puts "unknown color name \"$value\""                  
                  return 1}}
   }

   return 0
}

#*************************************************
#       Name: Check_data
# Parameters: data, data list
#Description: checks data for even number of values and
#           : that each value is a number
#*************************************************
proc Check_data {data} {

   # CHECK DATA FOR NON NUMERICS
   if {[lsearch $data {*[!-':-~/]*} ] != -1} {
      puts "bad data value \"[lindex $data [lsearch $data {*[!-/:-~]*} ]]\": must be numeric values"
      return 1
   }

   # CHECK FOR EVEN NUMBER OF VALUES
   if {[expr floor([llength $data]/2.0)] != [expr [llength $data]/2.0]} {
      puts "odd # of values in data \{$data\}"
      return 1
   }

   return 0
}


#*************************************************
#       Name: Line
# Parameters: name, lineplot name
#Description: draws line connecting points
#*************************************************
proc Line {name} {

   variable Created
   upvar #0 $Created($name) array_var

   set plotheight [expr $array_var(height)-$array_var(legendheightx)-1]
   set plotwidth  [expr $array_var(width) -$array_var(legendwidthy)   ]

   # IF ONLY ONE POINT, DONT ATTEMPT LINE
   #if {[llength $array_var(data)] < 4} {
   #  set array_var(line_tag) 0
   #  return
   #}
   
   if {$array_var(line) == "true"} {
      set total_points $array_var(data)
      set coor_list {}

      foreach {xvalue yvalue} $total_points {
         lappend coor_list [expr $plotwidth*($xvalue-$array_var(actual_originx))/($array_var(actual_maxx)-$array_var(actual_originx))]
         if {[expr $array_var(actual_maxy)-$array_var(actual_originy)] == 0} {
            set ycoor 0
         } else {
            set ycoor [expr $plotheight-($plotheight*($yvalue-$array_var(actual_originy)))/($array_var(actual_maxy)-$array_var(actual_originy))]
         }
         if {$array_var(originplacement) == "middle"} {
            set ycoor [expr $ycoor/2]
         }
         lappend coor_list $ycoor
      }

      $name.up.plot.can delete $array_var(line_tag)

      if {[llength $coor_list] > 3} {
         set array_var(line_tag) [eval {$name.up.plot.can create line} $coor_list {-smooth $array_var(smooth) -splinesteps $array_var(splinesteps) -width 1 -fill $array_var(pointcolor)}]
      }
   }


}

#*************************************************
#       Name: Points 
# Parameters: name, lineplot name
#           : point_list
#Description: plots or unplots a set of points
#*************************************************
proc Points  { name point_list action} {

   variable Created
   upvar #0 $Created($name) array_var

   set plotheight [expr $array_var(height)-$array_var(legendheightx)-1]
   set plotwidth  [expr $array_var(width)-$array_var(legendwidthy)]
 
   foreach {xvalue yvalue} $point_list {
      # PROTECT FROM DIVIDE BY ZERO
      if {[expr (double($array_var(actual_maxx))-double($array_var(actual_originx)))] == 0} {
         set xcoor 0
      } else {
         set xcoor [expr $plotwidth*(double($xvalue)-double($array_var(actual_originx)))/(double($array_var(actual_maxx))-double($array_var(actual_originx)))]
      }
      # PROTECT FROM DIVIDE BY ZERO
      set denom [expr $array_var(actual_maxy)-$array_var(actual_originy)]
      if {$denom == 0} {
         set ycoor 0
      } else {    
         set ycoor [expr $plotheight-($plotheight*($yvalue-$array_var(actual_originy)))/$denom]
      }
      if {$array_var(originplacement) == "middle"} {
         set ycoor [expr $ycoor/2]
      }

      if {$action == "erase"} {
         set draw_color $array_var(background)
      } else {
         set draw_color $array_var(pointcolor)
      }
      

      # DRAW POINTS
      if {$array_var(pointshape) == "dot"} {
         $name.up.plot.can create oval [expr $xcoor-2] [expr $ycoor-2] [expr $xcoor+2] [expr $ycoor+2] -fill $draw_color -outline $draw_color

      } elseif {$array_var(pointshape) == "circle"} {
         $name.up.plot.can create oval [expr $xcoor-2] [expr $ycoor-2] [expr $xcoor+2] [expr $ycoor+2] -outline $draw_color

      } elseif {$array_var(pointshape) == "square"} {
         $name.up.plot.can create rect [expr $xcoor-2] [expr $ycoor-2] [expr $xcoor+2] [expr $ycoor+2] -outline $draw_color 

      } else {
         $name.up.plot.can create line [expr $xcoor-2] [expr $ycoor-2] [expr $xcoor+3] [expr $ycoor+3] -width 1 -fill $draw_color 
         $name.up.plot.can create line [expr $xcoor-2] [expr $ycoor+2] [expr $xcoor+3] [expr $ycoor-3] -width 1 -fill $draw_color 
      }
   }

}

#*************************************************
#       Name: Axis
# Parameters: name, lineplot name
#Description: creates new lineplot axis
#*************************************************
proc Axis { name args } {

   variable Created
   upvar #0 $Created($name) array_var
   
   set framewidth  $array_var(width)
   set frameheight $array_var(height)
   set plotheight  [expr $frameheight-$array_var(legendheightx)-1]
   set plotwidth   [expr $framewidth -$array_var(legendwidthy)   ]


   # DESTROY CANVAS IF PRESENT
   destroy $name.up.plot.can

   # CREATE CANVAS WIDGET
   canvas $name.up.plot.can -height [expr $frameheight-$array_var(legendheightx)] -width [expr $framewidth-$array_var(legendwidthy)] -bg $array_var(background) -highlightbackground $array_var(background) -bd 0 -highlightthickness 0
   pack propagate $name.up.plot.can false
   pack $name.up.plot.can -expand true -fill both

   # Y AXIS 
   $name.up.plot.can create line 0 0 0 $plotheight -width 1 -fill $array_var(axiscolor) 

   # FOR BOTTOM VALUE OF originplacement
   if {$array_var(originplacement) != "middle"} {

      # Y AXIS TICS
      if {$array_var(ticspacey) == "explicit"} {      
         if {$array_var(ticcounty) > 0} {
            if {$array_var(autosizey) == "false"} {
               set tic_space [expr ($plotheight*($array_var(maxy)-$array_var(miny)))/($array_var(ticcounty)*($array_var(actual_maxy)-$array_var(actual_originy)))]
            } else {
               set tic_space [expr double($plotheight)/$array_var(ticcounty)]           
            }
            for {set i 1} {[expr $i*$tic_space] <= $plotheight} {incr i} {
                  $name.up.plot.can create line 0 [expr $plotheight-double($i)*$tic_space] 4 [expr $plotheight-double($i)*$tic_space] -width 1 -fill $array_var(axiscolor)    
            }
         }
      } else {
         # IMPLICIT TIC SPACE
         if {$array_var(autosizey) == "false"} {
            if {[llength $array_var(legendy)] < 2} {     
               set tic_number 1
            } else {
               set tic_number [expr [llength $array_var(legendy)]-1]    
            }
            set tic_space [expr ($plotheight*($array_var(maxy)-$array_var(miny)))/($tic_number*($array_var(actual_maxy)-$array_var(actual_originy)))]             
            for {set i 1} {[expr $i*$tic_space] <= $plotheight} {incr i} {
               $name.up.plot.can create line 0 [expr $plotheight-double($i)*$tic_space] 4 [expr $plotheight-double($i)*$tic_space] -width 1 -fill $array_var(axiscolor)    
            }
         } else {
            set tic_number 8
            set tic_space [expr double($plotheight)/$tic_number]
            for {set i 1} {$i <= $tic_number} {incr i} {
               $name.up.plot.can create line 0 [expr $plotheight-double($i)*$tic_space] 4 [expr $plotheight-double($i)*$tic_space] -width 1 -fill $array_var(axiscolor)    
            }
         }
      }  

   } else {
      # MIDDLE originplacement
      
      # Y AXIS TICS
      if {$array_var(ticspacey) == "explicit"} {      
         if {$array_var(ticcounty) > 0} {
            if {$array_var(autosizey) == "false"} {
               set tic_space [expr (.5*$plotheight*($array_var(maxy)-$array_var(miny)))/($array_var(ticcounty)*($array_var(actual_maxy)-$array_var(actual_originy)))]
            } else {
               set tic_space [expr (.5*$plotheight-1)/$array_var(ticcounty)]]            
            }
            for {set i 1} {$i <= $array_var(ticcounty)} {incr i} {
               $name.up.plot.can create line 0 [expr $plotheight/2-double($i)*$tic_space] 4 [expr $plotheight/2-double($i)*$tic_space] -width 1 -fill $array_var(axiscolor)   
               $name.up.plot.can create line 0 [expr $plotheight/2+double($i)*$tic_space] 4 [expr $plotheight/2+double($i)*$tic_space] -width 1 -fill $array_var(axiscolor)   
            }
         }
      } else {
         # IMPLICIT TIC SPACE
         if {$array_var(autosizey) == "false"} {
            if {[llength $array_var(legendy)] < 2} {     
               set tic_number 1
            } else {
               set tic_number [expr [llength $array_var(legendy)]-1]    
            }
            set tic_space [expr (.5*$plotheight*($array_var(maxy)-$array_var(miny)))/($tic_number*($array_var(actual_maxy)-$array_var(actual_originy)))]             
            for {set i 1} {$i <= $tic_number} {incr i} {
               $name.up.plot.can create line 0 [expr $plotheight*.5-double($i)*$tic_space] 4 [expr $plotheight*.5-double($i)*$tic_space] -width 1 -fill $array_var(axiscolor)    
               $name.up.plot.can create line 0 [expr $plotheight*.5+double($i)*$tic_space] 4 [expr $plotheight*.5+double($i)*$tic_space] -width 1 -fill $array_var(axiscolor)    
            }
         } else {
            set tic_number 4
            set tic_space [expr .5*$plotheight/$tic_number]
            for {set i 1} {$i <= $tic_number} {incr i} {
               $name.up.plot.can create line 0 [expr $plotheight/2-double($i)*$tic_space] 4 [expr $plotheight/2-double($i)*$tic_space] -width 1 -fill $array_var(axiscolor)   
               $name.up.plot.can create line 0 [expr $plotheight/2+double($i)*$tic_space] 4 [expr $plotheight/2+double($i)*$tic_space] -width 1 -fill $array_var(axiscolor)   
            }
         }
      }
   }


   # X LOWER LINE
   $name.up.plot.can create line 0 $plotheight $plotwidth $plotheight -width 1 -fill $array_var(axiscolor)  
   set tic_top [expr $plotheight-4]
   set tic_bottom $plotheight

   # MIDDLE ORIGIN PLACEMENT
   if {$array_var(originplacement) == "middle"} {
      $name.up.plot.can create line 0 [expr $plotheight/2] $plotwidth [expr $plotheight/2] -width 1 -fill $array_var(axiscolor)  
      set tic_top [expr ($plotheight/2)-4]
      set tic_bottom [expr ($plotheight/2)+4]
   }
         

   # X AXIS TICS

   if {$array_var(ticspacex) == "explicit"} {
      if {$array_var(ticcountx) > 0} {
         if {$array_var(autosizex) == "false"} {
            set tic_space [expr ($plotwidth*($array_var(maxx)-$array_var(minx)))/($array_var(ticcountx)*($array_var(actual_maxx)-$array_var(actual_originx)))]
         } else {
            set tic_space [expr double($plotwidth-1)/$array_var(ticcountx)]             
         }
         for {set i 1} {[expr $i*$tic_space] <= $plotwidth} {incr i} {
            $name.up.plot.can create line [expr double($i)*$tic_space] $tic_top [expr double($i)*$tic_space] $tic_bottom -width 1 -fill $array_var(axiscolor)  
         }
      }
   } else {
      # IMPLICIT TIC SPACING
      if {$array_var(autosizex) == "false"} {
         if {[llength $array_var(legendx)] < 2} {     
            set tic_number 1
         } else {
            set tic_number [expr [llength $array_var(legendx)]-1]    
         }
         set tic_space [expr ($plotwidth*($array_var(maxx)-$array_var(minx)))/($tic_number*($array_var(actual_maxx)-$array_var(actual_originx)))]
         for {set i 1} {[expr $i*$tic_space] <= $plotwidth} {incr i} {
            $name.up.plot.can create line [expr double($i)*$tic_space] $tic_top [expr double($i)*$tic_space] $tic_bottom -width 1 -fill $array_var(axiscolor)  
         }
      } else {
         set tic_number 8
         set tic_space [expr double($plotwidth-1.0)/$tic_number]
         for {set i 1} {[expr $i*$tic_space] <= $plotwidth} {incr i} {
            $name.up.plot.can create line [expr double($i)*$tic_space] $tic_top [expr double($i)*$tic_space] $tic_bottom -width 1 -fill $array_var(axiscolor)  
         }
      }
   }

}


#*************************************************
#       Name: Legendx
# Parameters: name, lineplot name
#Description: creates lineplot x legend
#*************************************************
proc Legendx { name } {

   variable Created
   upvar #0 $Created($name) array_var

   set framewidth $array_var(width)
   set frameheight $array_var(height)
   set font_height [expr 5+.5*[font metrics $array_var(font) -ascent]]
   set plotwidth [expr $framewidth-$array_var(legendwidthy)]
   set text_width 0


   # DESTROY CANVAS IF PRESENT
   destroy $name.low.can

   # CREATE CANVAS WIDGET
   canvas $name.low.can -height $array_var(legendheightx) -width $array_var(width) -bg $array_var(background) -highlightbackground $array_var(background) -bd 0 -highlightthickness 0
   pack propagate $name.low.can false
   pack $name.low.can -expand true -fill both


   # FILL LOWER CANVAS WITH X LEGEND
   if {$array_var(autosizex) == "false"} {
      set many [llength $array_var(legendx)]
      if {$many < 2} {
         # PRINT actual_originx FOR ORIGIN X VALUE
         set text_value $array_var(minx)
         $name.low.can create text $array_var(legendwidthy) $font_height -text $text_value -font $array_var(font)
      }
      if {$many == 0} {
         # PRINT maxx VALUE AT END OF X LEGEND
         set text_value $array_var(maxx)
         set lastwidth [font measure $array_var(font) $text_value]
         set xcoor [expr $array_var(legendwidthy)+($plotwidth*($array_var(maxx)-$array_var(minx))/($array_var(actual_maxx)-$array_var(actual_originx)))]
         # CHANGE X COOR IF VALUE WILL GO OFF PLOT AREA
         if {[expr $framewidth-$xcoor] < .5*$lastwidth} {
            set xcoor [expr $framewidth-.5*$lastwidth]
         }
         $name.low.can create text $xcoor $font_height -text $text_value -font $array_var(font)

      } elseif {$many == 1} {
         # USE ONLY LEGEND VALUE FOR END OF X LEGEND
         set lastwidth [font measure $array_var(font) $array_var(legendx)]
         set xcoor [expr $array_var(legendwidthy)+($plotwidth*($array_var(maxx)-$array_var(minx))/($array_var(actual_maxx)-$array_var(actual_originx)))]
         # CHANGE X COOR IF VALUE WILL GO OFF PLOT AREA
         if {[expr $framewidth-$xcoor] < .5*$lastwidth} {
            set xcoor [expr $framewidth-.5*$lastwidth]
         }
         $name.low.can create text $xcoor $font_height -text $array_var(legendx) -font $array_var(font)

      } else {
         # DISPLAY LEGEND SUPPLIED
         set text_width [expr ( $plotwidth*($array_var(maxx)-$array_var(minx)) / 
                                ( ($array_var(actual_maxx)-$array_var(actual_originx))*
                                  ([llength $array_var(legendx)]-1)
                                )
                              )
                        ]
         foreach value $array_var(legendx) {
            # LAST VALUE
            if {[lindex $array_var(legendx) end] == $value} {
               set lastwidth [font measure $array_var(font) $value]
               $name.low.can create text [expr $array_var(legendwidthy)+($plotwidth*($array_var(maxx)-$array_var(minx))/($array_var(actual_maxx)-$array_var(actual_originx)))] $font_height -text $value -justify right -font $array_var(font)
            } else {
               $name.low.can create text [expr $array_var(legendwidthy)+[lsearch -exact $array_var(legendx) $value]*$text_width] $font_height -text $value -font $array_var(font)
            }
         }
      }
   
   } else {
      # AUTOSIZE TRUE, PRINT LEGEND VALUES IF ROOM
      set room_used 0
      set value $array_var(actual_maxx)
      set value_width [font measure $array_var(font) $value]
      set lastwidth [font measure $array_var(font) "W"]
      if {$value_width < [expr $plotwidth/2]} {
         $name.low.can create text [expr $framewidth-.5*$value_width-5] $font_height -text $value -font $array_var(font)
         set room_used $value_width
         set lastwidth $value_width
         
         # ORIGIN VALUE
         set value $array_var(actual_originx)
         set value_width [font measure $array_var(font) $value]
         if {[expr $room_used+$value_width] < [expr $plotwidth/2]} {
            $name.low.can create text [expr $array_var(legendwidthy)-($value_width/2)] $font_height -text $value -font $array_var(font)
            set room_used [expr $room_used+$value_width]

            # MIDPOINT VALUE
            set value [expr $array_var(actual_originx)+($array_var(actual_maxx)-$array_var(actual_originx))/2]
            if {[expr int($value)] == $value && ($array_var(ticspacex) != "explicit" || ($array_var(ticspacex) == "explicit" && [expr int(.5*$array_var(ticcountx))] == [expr .5*$array_var(ticcountx)]))} {
               set value_width [font measure $array_var(font) $value]
               if {[expr $room_used+$value_width] < [expr $plotwidth/2]} {
                  $name.low.can create text [expr $framewidth-($plotwidth)/2] $font_height -text $value -font $array_var(font)
                  set room_used [expr $room_used+$value_width]
         
                  # BOTH QUARTER POINT VALUES
                  set value [expr $array_var(actual_originx)+($array_var(actual_maxx)-$array_var(actual_originx))/4]
                  if {[expr int($value)] == $value && ($array_var(ticspacex) != "explicit" || ($array_var(ticspacex) == "explicit" && [expr int(.25*$array_var(ticcountx))] == [expr .25*$array_var(ticcountx)])) } {
                     set value2 [expr $array_var(actual_originx)+($array_var(actual_maxx)-$array_var(actual_originx))*3/4]          
                     set value_width [font measure $array_var(font) $value]
                     set value_width2 [font measure $array_var(font) $value2]          
                     if {[expr $room_used+$value_width+$value_width2] < [expr $plotwidth/2]} {
                        $name.low.can create text [expr $array_var(legendwidthy)+($plotwidth/4)] $font_height -text $value -font $array_var(font)
                        $name.low.can create text [expr $array_var(legendwidthy)+($plotwidth*3/4)] $font_height -text $value2 -font $array_var(font)
                     }
                  }
               }
            }
         }
      }
   }

   # ADD UNITS TO X LEGEND LOWER CANVAS
   if {$array_var(unitx) != ""} {   
      # IF LEGEND CONTAINS AT LEAST 2 ITEMS AND UNITS FITS BETWEEN LAST 2 ITEMS
      set widthtest [font measure $array_var(font) "$array_var(unitx)WW"]    

      if {$array_var(autosizex) == "false"} {
         set maxx_point [expr ($plotwidth*($array_var(maxx)-$array_var(minx))/($array_var(actual_maxx)-$array_var(actual_originx)))]
         set unit_coor [expr $array_var(legendwidthy)+$maxx_point-.5*$lastwidth-.5*$widthtest]
      } else {
         set unit_coor [expr $framewidth-$lastwidth-.5*$widthtest]
      }
      $name.low.can create text $unit_coor $font_height -text $array_var(unitx) -font $array_var(font)      
   }

}

#*************************************************
#       Name: lineplot_legendy
# Parameters: name, lineplot name
#Description: creates lineplot y legend
#*************************************************
proc lineplot_legendy { name } {

   variable Created
   upvar #0 $Created($name) array_var
   
   set framewidth $array_var(width)
   set frameheight $array_var(height)
   set font_height [expr 5+[font metrics $array_var(font) -ascent]]
   set plotheight [expr $frameheight - $array_var(legendheightx)]


   # DESTROY CANVAS IF PRESENT
   destroy $name.up.side.can

   # CREATE CANVAS WIDGET
   canvas $name.up.side.can -height [expr $frameheight-$array_var(legendheightx)] -width $array_var(legendwidthy) -bg $array_var(background) -highlightbackground $array_var(background) -bd 0 -highlightthickness 0
   pack propagate $name.up.side.can false
   pack $name.up.side.can -expand true -fill both


   if {$array_var(autosizey) == "false"} {

      # FOR BOTTOM VALUE OF originplacement
      if {$array_var(originplacement) != "middle"} {
         set max_dist [expr $plotheight*($array_var(maxy)-$array_var(miny))/($array_var(actual_maxy)-$array_var(miny))]
         set max_point [expr $plotheight-$max_dist]

         # FILL SIDE CANVAS WITH Y LEGEND
         set many [llength $array_var(legendy)]
         if {$many < 2} {
            # PRINT actual_originy FOR ORIGIN Y VALUE
            set text $array_var(actual_originy)
            set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$text]]
            $name.up.side.can create text $textxcoor [expr $frameheight-$array_var(legendheightx)-(5+[font metrics $array_var(font) -descent])] -text $array_var(actual_originy) -font $array_var(font)
         }
         if {$many == 0} {
            # PRINT maxy VALUE AT TOP OF Y LEGEND
            set text $array_var(maxy)
            set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$text]]
            $name.up.side.can create text $textxcoor $max_point -text $array_var(maxy) -font $array_var(font)

         } elseif {$many == 1} {
            # USE ONLY LEGEND VALUE FOR TOP OF Y LEGEND
            set text $array_var(legendy) 
            set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$text]]
            $name.up.side.can create text $textxcoor $max_point -text $array_var(legendy) -font $array_var(font)

         } else {
            # DISPLAY LEGEND SUPPLIED
            set text_width [expr double($max_dist)/([llength $array_var(legendy)]-1)]
            set i 0
            foreach value $array_var(legendy) {
               if {[lindex $array_var(legendy) 0] == $value} {
                  # FIRST VALUE OF LEGEND
                  set textycoor [expr $frameheight - $array_var(legendheightx)-.5*$font_height] 

               } elseif {[lindex $array_var(legendy) end] == $value} {
                  # LAST VALUE OF LEGEND
                  set textycoor $max_point
               
               } else {
                  set textycoor [expr $frameheight-$array_var(legendheightx) -$i*$text_width]
               }
               set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$value]]
               $name.up.side.can create text $textxcoor $textycoor -text $value -font $array_var(font) -justify right
               incr i
            }  
         }
      
      } else {
         # MIDDLE ORIGIN PLACEMENT
         set max_dist [expr $plotheight*.5*($array_var(maxy)-$array_var(miny))/($array_var(actual_maxy)-$array_var(miny))]
   
         # FILL SIDE CANVAS WITH Y LEGEND
         set many [llength $array_var(legendy)]
         if {$many < 2} {
            # PRINT actual_originy FOR ORIGIN Y VALUE
            set text $array_var(actual_originy)
            set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$text]]
            $name.up.side.can create text $textxcoor [expr $plotheight/2] -text $array_var(actual_originy) -font $array_var(font)
         }
         if {$many == 0} {
            # PRINT maxy VALUE AT TOP OF Y LEGEND
            set text $array_var(maxy)
            set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$text]]
            $name.up.side.can create text $textxcoor [expr .5*$plotheight-$max_dist] -text $array_var(maxy) -font $array_var(font)
            set text [expr 2*$array_var(miny)-$array_var(maxy)]
            set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$text]]
            $name.up.side.can create text $textxcoor [expr .5*$plotheight+$max_dist] -text $text -font $array_var(font)
   

         } elseif {$many == 1} {
            # USE ONLY LEGEND VALUE FOR TOP OF Y LEGEND
            set text $array_var(legendy) 
            set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$text]]
            $name.up.side.can create text $textxcoor [expr .5*$plotheight-$max_dist] -text $array_var(legendy) -font $array_var(font)
            set text [expr 2*$array_var(miny)-$array_var(maxy)]
            set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$text]]
            $name.up.side.can create text $textxcoor [expr .5*$plotheight+$max_dist] -text $array_var(legendy) -font $array_var(font)

         } else {
            # DISPLAY LEGEND SUPPLIED
            set text_width [expr .5*($frameheight-$array_var(legendheightx))/([llength $array_var(legendy)]-1)]
            set i 0
            foreach value $array_var(legendy) {
               if {[lindex $array_var(legendy) 0] == $value} {
                  # FIRST VALUE OF LEGEND
                  set textycoor [expr $plotheight/2] 
                  set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$value]]
                  $name.up.side.can create text $textxcoor $textycoor -text $value -font $array_var(font) -justify right
   
               } elseif {[lindex $array_var(legendy) end] == $value} {
                  # LAST VALUE OF LEGEND
                  set textycoor [expr .5*$plotheight-$max_dist]
                  set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$value]]
                  $name.up.side.can create text $textxcoor $textycoor -text $value -font $array_var(font) -justify right
                  
                  set textycoor [expr .5*$plotheight-$max_dist]
                  set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i[expr -1*$value]]]
                  $name.up.side.can create text $textxcoor $textycoor -text $value -font $array_var(font) -justify right
                              
               } else {
                  set textycoor [expr .5*$plotheight-$i*$text_width]
                  set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$value]]
                  $name.up.side.can create text $textxcoor $textycoor -text $value -font $array_var(font) -justify right

                  set textycoor [expr .5*$plotheight+$i*$text_width]
                  set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i[expr -1*$value]]]
                  $name.up.side.can create text $textxcoor $textycoor -text $value -font $array_var(font) -justify right
               }
               incr i
            }
         }
      }
   
   } else {
      # AUTOSIZE TRUE, PRINT LEGEND VALUES IF ROOM
      if {$array_var(originplacement) != "middle"} {

         # BOTTOM ORIGIN PLACEMENT
         set lines_used 0
         set value $array_var(actual_maxy)
         set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$value]]
         if {$textxcoor < .5*[font measure $array_var(font) $value]} {set textxcoor [expr .5*[font measure $array_var(font) $value]]}
         if {$font_height < [expr $plotheight/2]} {
            $name.up.side.can create text $textxcoor [expr .5*$font_height] -text $value -font $array_var(font)
            incr lines_used
         
            # ORIGIN VALUE
            set value $array_var(actual_originy)
            set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$value]]
            if {$textxcoor < .5*[font measure $array_var(font) $value]} {set textxcoor [expr .5*[font measure $array_var(font) $value]]}
            if {[expr $font_height*($lines_used+1)] < [expr $plotheight/2]} {
               $name.up.side.can create text $textxcoor [expr $plotheight-.5*$font_height] -text $value -font $array_var(font)
               incr lines_used 

               # MIDPOINT VALUE
               set value [expr $array_var(actual_originy)+($array_var(actual_maxy)-$array_var(actual_originy))/2]
               if {[expr int($value)] == $value && ($array_var(ticspacey) != "explicit" || ($array_var(ticspacey) == "explicit" && [expr int(.5*$array_var(ticcounty))] == [expr .5*$array_var(ticcounty)]))} {
                  set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$value]]
                  if {$textxcoor < .5*[font measure $array_var(font) $value]} {set textxcoor [expr .5*[font measure $array_var(font) $value]]}
                  if {[expr $font_height*($lines_used+1)] < [expr $plotheight/2]} {
                     $name.up.side.can create text $textxcoor [expr $plotheight*.5] -text $value -font $array_var(font)
                     incr lines_used 
         
                     # BOTH QUARTER POINT VALUES
                     set value [expr $array_var(actual_originy)+($array_var(actual_maxy)-$array_var(actual_originy))*3/4]
                     if {[expr int($value)] == $value && ($array_var(ticspacey) != "explicit" || ($array_var(ticspacey) == "explicit" && [expr int(.25*$array_var(ticcounty))] == [expr .25*$array_var(ticcounty)]))} {
                     set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$value]]
                     if {$textxcoor < .5*[font measure $array_var(font) $value]} {set textxcoor [expr .5*[font measure $array_var(font) $value]]}
                        set value2 [expr $array_var(actual_originy)+($array_var(actual_maxy)-$array_var(actual_originy))/4]            
                        set textxcoor2 [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$value2]]
                        if {$textxcoor2 < 0} {set textxcoor2 0}

                        if {[expr $font_height*($lines_used+2)] < [expr $plotheight/2]} {
                           $name.up.side.can create text $textxcoor [expr .25*$plotheight] -text $value -font $array_var(font)
                           $name.up.side.can create text $textxcoor2 [expr .75*$plotheight] -text $value2 -font $array_var(font)
                        }
                     }
                  }  
               }
            }
         }
      } else {
         # MIDDLE ORIGIN PLACEMENT
         # PRINT TOP AND BOTTOM VALUES ON Y AXIS
         set lines_used 0
         set value $array_var(actual_maxy)
         set value2 [expr 2*$array_var(actual_originy)-$array_var(actual_maxy)]
         set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$value]]
         if {$textxcoor < .5*[font measure $array_var(font) $value]} {set textxcoor [expr .5*[font measure $array_var(font) $value]]}
         set textxcoor2 [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$value2]]         
         if {$textxcoor2 < .5*[font measure $array_var(font) $value2]} {set textxcoor2 [expr .5*[font measure $array_var(font) $value2]]}
         if {[expr 2*$font_height] < [expr $plotheight/2]} {

            $name.up.side.can create text $textxcoor [expr .5*$font_height] -text $value -font $array_var(font)
            $name.up.side.can create text $textxcoor2 [expr $plotheight-.5*$font_height] -text $value2 -font $array_var(font)
            incr lines_used 2
         
            # PRINT ORIGIN VALUE ON Y AXIS
            set value $array_var(actual_originy)
            set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$value]]
            if {$textxcoor < .5*[font measure $array_var(font) $value]} {set textxcoor [expr .5*[font measure $array_var(font) $value]]}
            if {[expr $font_height*($lines_used+1)] < [expr $plotheight/2]} {
               $name.up.side.can create text $textxcoor [expr .5*$plotheight] -text $value -font $array_var(font)
               incr lines_used 

               # MIDPOINT VALUE
               set value [expr .5*$array_var(actual_maxy)+.5*$array_var(actual_originy)]
               if {[expr int($value)] == $value && ($array_var(ticspacey) != "explicit" || ($array_var(ticspacey) == "explicit" && [expr int(.5*$array_var(ticcounty))] == [expr .5*$array_var(ticcounty)]))} {
                  set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$value]]
                  if {$textxcoor < .5*[font measure $array_var(font) $value]} {set textxcoor [expr .5*[font measure $array_var(font) $value]]}
                  set value2 [expr 1.5*($array_var(actual_originy)-.5*$array_var(actual_maxy)]              
                  set textxcoor2 [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$value2]]
                  if {$textxcoor2 < .5*[font measure $array_var(font) $value2]} {set textxcoor2 [expr .5*[font measure $array_var(font) $value2]]}
                  if {[expr $font_height*($lines_used+2)] < [expr $plotheight/2]} {
                     $name.up.side.can create text $textxcoor  [expr $plotheight*.25] -text $value  -font $array_var(font)
                     $name.up.side.can create text $textxcoor2 [expr $plotheight*.75] -text $value2 -font $array_var(font)
                  }
               }
            }
         }
      }
   }
   
   # ADD UNITS TO Y LEGEND
   if {$array_var(unity) != ""} {   
      set text $array_var(unity)
      set textxcoor [expr $array_var(legendwidthy)-.5*[font measure $array_var(font) i$text]]
      if {$textxcoor < .5*[font measure $array_var(font) $text]} {set textxcoor [expr .5*[font measure $array_var(font) $text]]}
      if {$array_var(autosizey) == "false" && $array_var(originplacement) != "middle"} {
         set textycoor [expr $plotheight-$max_dist+$font_height]
      } elseif {$array_var(autosizey) == "false"} {
         set textycoor [expr .5*$plotheight-$max_dist+$font_height]
      } else {
         set textycoor [expr 1.5*$font_height]
      }
      $name.up.side.can create text $textxcoor $textycoor -text $text -font $array_var(font)
   }  

}

#*************************************************
#       Name: Create_new
# Parameters: name, lineplot name
#           : configuring arguments of lineplot
#Description: creates new lineplot widget proc
#*************************************************
proc Create_new { name args } \
{
   puts "Create_new $name $args"
   
   variable Created
   upvar #0 $Created($name) array_var
          
   # PARSE ARGS AND SET ARRAY VALUES
   foreach {option value} $args {

      # CHECK TO SEE IF A LEGAL option
      set element [string trimleft $option "-"]
      if {$element == "bg"} {set element background}

      if {![info exists array_var($element)]} {
         puts "lineplot::Create_new - unknown option \"$option\""
         return 1
      }
   
      # DO NOT ALLOW CHANGING OF INTERNAL VALUES
      if {[lsearch -exact {class legendwidthy legendheightx largest_x largest_y smallest_x smallest_y widest_y actual_maxx actual_maxy line_tag} $element] != -1} {
         puts "cant modify \"$option\" option"
         return 1
      }

      # CHANGE NEGATIVE VALUES TO ZERO WHEN NEGATIVE NOT ALLOWED
      if {[lsearch -exact {width height splinesteps ticcountx ticcounty} $element] != -1} {
         if {$value < 0} {
            set neg_index [lsearch -exact $args $option]
            set args [lreplace $args [expr $neg_index+1] [expr $neg_index+1] 0]
         }
      }

      # RESTRICT ILLEGAL OPTION VALUES
      if {[Config_value_check $element $value]} {
         return 1
      }

      # CHECK VALIDITY OF NEW DATA
      if {$option == "-data" && [Check_data $value]} {
         return 1
      }

      # OMIT CONFIGS THAT ARE ALREADY SAME VALUE
      if {$array_var($element) == $value} {
         set index [lsearch -exact $args $option]
         set args [lreplace $args $index [expr $index+1]]
      }
            
      # CHANGE VALUES FOR PASSED OPTIONS
      set array_var($element) $value                      
   }

   # DETERMINE LARGEST X,Y AND ACTUAL_MAXX, ACTUAL_MAXY VALUES
   Find_largest $name

   # DETERMINE DEFAULT LEGENDWIDTHY AND LEGENDHEIGHTX
   set array_var(legendheightx) [expr 5+[font metrics $array_var(font) -linespace]]
   set array_var(legendwidthy)  [expr 5+$array_var(widest_y)]
   
   puts "Create_new creating parent frame"

   # CREATE HOLDING FRAMES
   frame $name -height $array_var(height) -width $array_var(width) -bg $array_var(background)
   bind $name <Destroy> "lineplot::Destroy %W"
   pack propagate $name false
   pack $name 

   Create_frames $name

   # DRAW X AXIS LEGEND
   Legendx $name
   
   # DRAW Y AXIS LEGEND
   lineplot_legendy $name
   
   # DRAW PLOT AREA
   Axis $name  

   # DRAW DATA POINTS
   Points  $name $array_var(data) draw

   # SORT DATA LIST
   set total_points $array_var(data)
   set array_var(data) [Sort $total_points]

   # DRAW CONNECTING LINE
   Line $name

   return 0
}
#*************************************************
#       Name: Destroy
# Parameters: name
#Description: Clean-up after the destruction of a lineplot
#*************************************************
proc Destroy { name } \
{
   puts "Destroying $name"
   variable Created
   unset $Created($name)
   unset Created($name)
}

#*************************************************
#       Name: lineplot
# Parameters: name, lineplot name
#           : configuring arguments of lineplot
#Description: creates new lineplot widget proc
#*************************************************
proc lineplot { name args } \
{   
   # CHECK TO SEE IF $NAME WIDGET ALREADY EXISTS, IF SO, RETURN
   if {[winfo exists $name]} {
      puts "widget \"$name\" already exists"
      return
   }

   # CREATE NEW LINEPLOT ARRAY
   Create_array $name

   # CREATE NEW LINEPLOT WIDGET
   if {[eval Create_new $name $args]} {
      return
   }
   # hide the command for the widget
   append widget_name $name widget
   interp alias {} $name {} lineplot::cmd $name
    
   return $name
}


   # CREATE $NAME PROC
   proc cmd { name action args} \
   {
      variable Created
      upvar #0 $Created($name) array_var

      # CHECK TO SEE THAT CANVAS WIDGET EXISTS
      if {![winfo exists $name]} {
         puts "lineplot \"$name\" does not exist"
         return
      }

      # CHECK TO SEE IF LEGAL OPERATION
      set valid {cget config configure add erase}
      if {[lsearch -exact $valid $action] == -1} {
         puts "bad option \"$action\": must be cget, configure, add or erase"
         return
      }

      # HANDLE CGET
      if {$action == "cget"} {
         if {[llength $args] == 0} {
            puts "wrong # args: should be $name cget option"
            return
         }
         set element [string trimleft $args "-"]
         if {![info exists array_var($element)]} {
            puts "$name cget - unknown option \"$element\""
            return
         }
         return $array_var($element)
      }

      # HANDLE ADD
      if {$action == "add"} {
         if {[llength $args] == 1} {
            set point_list [lindex $args 0]
         } else {
            puts "wrong # args: should be \"$name add \{data\}\""
            return
         }

         # CHECK VALIDITY OF NEW DATA
         if {[Check_data $point_list]} {
            return
         }

         # OMIT POINTS ALREADY INCLUDED IN DATA
         set total_points $array_var(data)
         set array_var(data) [Omit_same $total_points $point_list]

         # ADD DATA TO TOTAL DATA
         set array_var(data) [concat $array_var(data) $point_list]
      
         # SORT DATA LIST
         set total_points $array_var(data)
         set array_var(data) [Sort $total_points]

         # DETERMINE LARGEST X,Y AND ACTUAL_MAXX, ACTUAL_MAXY VALUES
         Find_largest $name
         
         # IF autosizex OR autosizey IS TRUE, REDRAW ALL
         if {$array_var(autosizex) == "true" || $array_var(autosizey) == "true"} {
            
            # REDETERMINE DEFAULT LEGENDWIDTHY
            set array_var(legendwidthy) [expr 5+$array_var(widest_y)]
   
            # DRAW X AXIS LEGEND
            Legendx $name
   
            # DRAW Y AXIS LEGEND
            lineplot_legendy $name
   
            # DRAW PLOT AREA
            Axis $name  
         }
         

         if {$array_var(autosizex) == "true" || $array_var(autosizey) == "true"} {
            Points  $name $array_var(data) draw
         } else { 
            Points  $name $point_list add
         }

         # REDRAW CONNECTING LINE
         Line $name

         return $array_var(data)
      }

      # HANDLE ERASE
      if {$action == "erase"} {
         if {[llength $args] == 1} {
            set point_list [lindex $args 0]
         } else {
            puts "wrong # args: should be \"$name erase \{data\}\""
            return
         }

         # CHECK VALIDITY OF NEW DATA
         if {[Check_data $point_list]} {
            return
         }

         # ERASE, OMIT PASSED DATA FROM TOTAL DATA
         set total_points $array_var(data)
         set array_var(data) [Omit_same $total_points $point_list]

         # DETERMINE LARGEST X,Y AND ACTUAL_MAXX, ACTUAL_MAXY VALUES
         Find_largest $name

         # IF autosizex OR autosizey IS TRUE, REDRAW ALL
         if {$array_var(autosizex) == "true" || $array_var(autosizey) == "true"} {
            
            # REDETERMINE DEFAULT LEGENDWIDTHY
            set array_var(legendwidthy) [expr 5+$array_var(widest_y)]

            # DRAW X AXIS LEGEND
            Legendx $name
   
            # DRAW Y AXIS LEGEND
            lineplot_legendy $name
         }

         # DRAW PLOT AREA
         Axis $name  
         
         # PLOT REMAINING POINTS
         Points  $name $array_var(data) draw
   
         # REDRAW CONNECTING LINE
         Line $name

         return $array_var(data)
      }



      # HANDLE CONFIG
      if {$action == "config" || $action == "configure"} {

         # HANDLE NO ARGS
         if {[llength $args] == 0} {
            foreach {index value} [array get array_var] {
               set default [Default $index]
               if {$default == ""} {set default "{}"} 
               if {$value == ""} {set value "{}"} 
               puts -nonewline "\{-$index $default $value\} "
            }
         }
         
         # PARSE AND QUALIFY CONFIG OPTIONS PASSED
         foreach {option value} $args {
            set element [string trimleft $option "-"]
            if {$element == "bg"} {set element background}

            # RETURN ON UNKNOWN OPTIONS
            if {![info exists array_var($element)]} {
               puts "$name config - unknown option \"$element\""
               return
            }

            # RESTRICT OPTIONS NOT ALLOWED TO CONFIG
            if {[lsearch {class legendwidthy legendheightx largest_x largest_y smallest_x smallest_y widest_y actual_maxx actual_maxy actual_originx actual_originy line_tag} $option] != -1} {
               puts "cant modify \"$option\" option"
               return
            }

            # CHANGE NEGATIVE VALUES TO ZERO WHEN NEGATIVE NOT ALLOWED
            if {[lsearch -exact {width height splinesteps ticcountx ticcounty} $element] != -1} {
               if {$value < 0} {
                  set neg_index [lsearch -exact $args $option]
                  set args [lreplace $args [expr $neg_index+1] [expr $neg_index+1] 0]
               }
            }

            # RESTRICT ILLEGAL OPTION VALUES
            if {[Config_value_check $element $value]} {
               return
            }

            # CHECK VALIDITY OF NEW DATA
            if {$option == "-data" && [Check_data $value]} {
               return
            }

            # OMIT CONFIGS THAT ARE ALREADY SAME VALUE
            if {$array_var($element) == $value} {
               set index [lsearch -exact $args $option]
               set args [lreplace $args $index [expr $index+1]]
            }
            
            # CHANGE VALUES FOR PASSED OPTIONS
            set array_var($element) $value                      
         }

         # IF DATA IS CONFIGGED, SORT NEW DATA
         if {[lsearch -exact $args -data] != -1} {
            set total_points $array_var(data)
            set array_var(data) [Sort $total_points]
         }

         # CONFIGS THAT NEED TO REFIND LARGEST X, Y
         if {[lsearch -exact $args -unity] != -1 || [lsearch -exact $args -font] != -1 || [lsearch -exact $args -legendy] != -1 || [lsearch -exact $args -autosizex] != -1 || [lsearch -exact $args -autosizey] != -1 || [lsearch -exact $args -maxx] != -1 || [lsearch -exact $args -maxy] != -1 || [lsearch -exact $args -data] != -1 || [lsearch -exact $args -minx] != -1 || [lsearch -exact $args -miny] != -1} {

            Find_largest $name   
            
            # REDETERMINE DEFAULT LEGENDWIDTHY AND LEGENDHEIGHTX
            set array_var(legendheightx) [expr 5+[font metrics $array_var(font) -linespace]]
            set array_var(legendwidthy) [expr 5+$array_var(widest_y)]
         }
         
         # CONFIGS THAT NEED TO RESIZE FRAMES
         if {[lsearch -exact $args -width] != -1 || [lsearch -exact $args -height] != -1 || [lsearch -exact $args -unity] != -1 || [lsearch -exact $args -font] != -1 || [lsearch -exact $args -legendy] != -1} {
            Create_frames $name
         }

         set redraw_all [Redraw_all? $name $args]

         # CONFIGS THAT NEED TO REDRAW X AXIS LEGEND
         if {$redraw_all || [lsearch -exact $args -legendx] != -1 || [lsearch -exact $args -unitx] != -1} {
            Legendx $name
         }

         # CONFIGS THAT NEED TO REDRAW Y AXIS LEGEND
         if {$redraw_all || [lsearch -exact $args -legendy] != -1} {
            lineplot_legendy $name
         }

         # CONFIGS THAT NEED TO REDO AXIS OR PLOT
         if {$redraw_all || [Redraw_plot? $name $args]} {         
            Axis $name
            Points  $name $array_var(data) draw
            Line $name
         }

         # -background CONFIG
         if {[lsearch -exact $args -background] != -1 || [lsearch -exact $args -bg] != -1} {
            set newbg $array_var(background)
            if [catch {$name.up.plot.can config -bg $newbg}] {
               puts "unknown color name \"$newbg\""
               return
            }
            $name.up.side.can config -bg $newbg
            $name.low.can config -bg $newbg
         }

         # -line CONFIG
         if {[lsearch -exact $args -line] != -1} {
            if {$array_var(line) == "true"} {
               Line $name
            } else {
               $name.up.plot.can delete $array_var(line_tag)
            }
         }

         # -smooth CONFIG
         if {[lsearch -exact $args -smooth] != -1 && $array_var(line) == "true" && $array_var(line_tag) != 0} {
            if {$array_var(smooth) == "true"} {
               $name.up.plot.can itemconfigure $array_var(line_tag) -smooth true
            } else {
               $name.up.plot.can itemconfigure $array_var(line_tag) -smooth false
            }           
         }

         # -splinesteps CONFIG
         if {[lsearch -exact $args -splinesteps] != -1 && $array_var(line) == "true" && $array_var(line_tag) != 0} {
            $name.up.plot.can itemconfigure $array_var(line_tag) -splinesteps $array_var(splinesteps)
         }
      }
   }

#*************************************************
#       Name: lineplot_debug_test
# Parameters: 
#Description: debug routine to test lineplot
#*************************************************
proc Debug_test { } {

   frame .bigplot
   pack .bigplot
   
   frame .bigplot.top
   pack .bigplot.top
   
   set plot_data { -16 -30 -8 -24 0 -16 8 -8 }

   lineplot .bigplot.top.plot -width 430 -height 370 -autosizex true -autosizey true -unitx dBm -smooth false -data $plot_data -smooth true -splinesteps 200

}

interp alias {} lineplot {} lineplot::lineplot

}
