#****************************************************************************  
#
# p25digitentry.tcl:  This file changes the bindings on a specific entry 
#                     widget so to imulate a P25 DIGITS entry field.  
#                     This imulation is detailed as follows:
#
#     - If numeric keys start edit:
#
#       - Background color is changed.
#
#       - Entered keys are inserted at the cursor position.
#
#       - Enter ends edit, and the background color is restored.
#
#     - If Enter starts edit:
#
#       - Background color is changed.
#
#       - The last field on the right is highlighted.
#
#     - If keyboard presses starts edit:
#
#       - Background color is changed.
#
#       - Typed characters are entered as typed
#
#       - The last field on the right is highlighted.
#
#       - Enter ends edit, and the background color is restored.
#
#     GLOBALS USED:
#
#****************************************************************************

namespace eval ::P25DigitEntry \
{
  variable show_count  0
 
  variable Widget_cleanups  ""
   
  namespace import ::Entry::Highlight ::Entry::Clear ::Entry::Insert ::Entry::Find_vars

  bind P25DigitEntry  <Return>        "P25DigitEntry::Return     %W"
  bind P25DigitEntry  <Escape>        "P25DigitEntry::Escape     %W"
  bind P25DigitEntry  <BackSpace>     "P25DigitEntry::Backspace  %W"
  bind P25DigitEntry  <Delete>        "P25DigitEntry::Delete     %W"

  bind P25DigitEntry  <Destroy>       "P25DigitEntry::Destroy  %W"

  bind P25DigitEntry  <Button-1>      "P25DigitEntry::Button_1  %W  %x"

  bind P25DigitEntry  <<EditOpen>>    "P25DigitEntry::Start_edit  %W"

  bind P25DigitEntry  <<EditClose>>   "P25DigitEntry::Stop_edit   %W"
  bind P25DigitEntry  <<EditEscape>>  "P25DigitEntry::Stop_edit   %W"

  bind P25DigitEntry  <<EditClear>>   "P25DigitEntry::Clear  %W"
  
  bind P25DigitEntry  <<edit_left>>   "P25DigitEntry::Left   %W"
  bind P25DigitEntry  <<edit_right>>  "P25DigitEntry::Right  %W"



  # Bind keys used explicitly by this entry widget
  foreach { key } { 0 1 2 3 4 5 6 7 8 9 A B C D KP_Multiply numbersign H P } \
  { 
    bind  P25DigitEntry  <Key-$key>  "P25DigitEntry::Keypress  %W  %A"
  }
  
  # Bind keys that may be entered as lower case, 
  # and need to be converted to upper case
  foreach { key  up } { a A b B c C d D h H p P } \
  {
    bind P25DigitEntry  <Key-$key>  "P25DigitEntry::Keypress  %W  $up"
  }

  # Bind the Softkeys used explicitly by this entry widget
  foreach { skey  letter } { F2 A F3 B F4 C F5 D F6 H F7 P } \
  {
    bind P25DigitEntry  <Key-$skey>  "event generate  %W  <Key-$letter>"
   
    bind P25DigitEntry  <KeyRelease-$skey>  "break"
  }
 



  #***************************************************************************
  #
  #        Name: Create
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Create an P25DigitEntry widget
  #
  #***************************************************************************
  proc Create { widget } \
  {
    #puts "P25DigitEntry::Create:  $widget"
    
    entry  $widget
    
    Bind  $widget
    
    return  $widget
  }

   
  #***************************************************************************
  #
  #        Name: Bind
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Bind the entry widget to handle alphanumeric entries
  #
  #***************************************************************************
  proc Bind { widget {upper_max ""} { comm_list "" } { editbgcolor "" } } \
  {
    #puts "P25DigitEntry::Bind:  $widget,  $comm_list,  $editbgcolor"

    if { $comm_list   == "" } { set comm_list   [option get $widget comm_list  {}] }
    if { $comm_list   == "" } { set comm_list   [option get $widget command    {}] }
    if { $editbgcolor == "" } { set editbgcolor [option get $widget editbg     {}] }
    if {$upper_max == ""}   { set upper_max   [option get $widget upper_max  {}] }
    
    # Replace any '%W' designations with the widget tile path
    regsub -all "%W"  $comm_list  $widget  comm_list

    set var  [$widget cget -textvariable]

    set backcolor  [$widget cget -bg]
    
    if { "$editbgcolor" == "" } \
    {
      set editbgcolor  [option get $widget editbg {}]
    }
    set editfgcolor  [option get $widget editfg {}]
    
    set ::Entry::entry_data($widget,var)          $var
    set ::Entry::entry_data($widget,comm)         [list $comm_list]
    set ::Entry::entry_data($widget,back)         $backcolor
    set ::Entry::entry_data($widget,editbgcolor)  $editbgcolor
    set ::Entry::entry_data($widget,editfgcolor)  $editfgcolor
    set ::Entry::entry_data($widget,editon)       0
    set ::Entry::entry_data($widget,upper_max)    $upper_max
    
    # Clear all current bindings
    del_bindtag  $widget  Entry
    
    eval $widget  configure  -takefocus 1 [option get $widget editops {}]
    
    # Add the necessary bindings
    add_bindtag  $widget  P25DigitEntry
    
    $widget  config  -takefocus 1  -cursor xterm
    
    return
  }

  
  #***************************************************************************
  #
  #        Name: Destroy
  #
  #  Parameters: widget - invoking widget
  #
  # Description: 
  #
  #***************************************************************************
  proc Destroy { widget } \
  {
    #puts "P25DigitEntry::Destroy:  $widget"

    return
  }


  #***************************************************************************
  #
  #        Name: Clear
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <<EditClear>> event for the entry widget
  #
  #***************************************************************************
  proc Clear { widget } \
  {
    if { [info exists editon] && $editon == 0 } \
    {
      return
    }
    
    set var  [$widget cget -textvariable]
  
    upvar #0 $var  value
    set value ""
  
    $widget icursor 0
    
    Highlight $widget
    
    return
  }

  
  #***************************************************************************
  #
  #        Name: Return
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <Return> event for the entry widget
  #
  #***************************************************************************
  proc Return { widget } \
  {
    #puts "P25DigitEntry::Return:  $widget"

    # do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } \
    { 
      return 
    }
    
    upvar ::Entry::entry_data($widget,editon)  editon
    
    if { ![info exists editon] } \
    {
      return
    }
    
    if { $editon } \
    {
      # If open, stop editting
      event generate  $widget  <<EditClose>>
    } \
    else \
    {
      # If not open, open for editting
      event generate  $widget  <<EditOpen>>
    }

    return
  }
  
  
  #***************************************************************************
  #
  #        Name: Escape
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <Escape> event for the entry widget
  #
  #***************************************************************************
  proc Escape { widget } \
  {
    #puts "P25DigitEntry::Escape:  $widget"

    # do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } \
    { 
      return 
    }
    
    upvar ::Entry::entry_data($widget,editon)  editon
    
    # If is open, close
    if { [info exists editon] && $editon == 0 } \
    {
      return
    }

    event generate  $widget  <<EditClose>>

    return
  }


  #***************************************************************************
  #
  #        Name: Backspace
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <BackSpace> event for the entry widget
  #
  #***************************************************************************
  proc Backspace { widget } \
  {
    #puts "P25DigitEntry::Backspace:  $widget"

    # do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } \
    { 
      return 
    }
    
    upvar ::Entry::entry_data($widget,editon)  editon
    
    # If not open, return
    if { [info exists editon] && $editon == 0 } \
    {
      return
    }

    upvar #0 [$widget cget -textvariable]  value
    
    set insert  [$widget index insert]

    set str_length  [string length $value]
    
    set first_str_half   [string range $value  0  [expr $insert - 2]]
    set second_str_half  [string range $value  $insert  $str_length]
      
    set new_string  ""
      
    append  new_string  $first_str_half  $second_str_half

    set value  "$new_string"

    $widget icursor [expr $insert - 1]
    
    Highlight $widget

    return
  }


  #***************************************************************************
  #
  #        Name: Delete
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <Delete> event for the entry widget
  #
  #***************************************************************************
  proc Delete { widget } \
  {
    #puts "P25DigitEntry::Delete:  $widget"

    # do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } \
    { 
      return 
    }
    
    upvar ::Entry::entry_data($widget,editon)  editon
    
    # If is open, perform a delete
    if { [info exists editon] && $editon == 0 } \
    {
      return
    }

    upvar #0 [$widget cget -textvariable]  value
    
    set insert  [$widget index insert]

    set str_length  [string length $value]
    
    set first_str_half   [string range $value  0  [expr $insert - 1]]
    set second_str_half  [string range $value  [expr $insert + 1]  $str_length]
    
    set new_string  ""
    
    append  new_string  $first_str_half  $second_str_half

    set value  "$new_string"

    $widget icursor  $insert
    
    Highlight  $widget

    return
  }
  
  
  #***************************************************************************
  #
  #        Name: Keypress
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <KeyPress> event for the entry widget
  #
  #***************************************************************************
  proc Keypress { widget  key } \
  {
    #puts "P25DigitEntry::Keypress:  $widget,  $key"

    # if we aren't enabled, bail right now
    if { [$widget cget -state] == "disabled" } \
    { 
      return
    }
    
    # If the edit isn't open, generate the edit open event, and
    # regenerate this event.
    upvar ::Entry::entry_data($widget,editon)  editon

    if { [info exists editon] && $editon == 0 } \
    { 
       event generate $widget <<EditOpen>>
       event generate $widget <<EditClear>>
      
      # Continue editing
      ##event generate  $widget  <KeyPress>
      ##return
    }
    
    upvar #0 [$widget cget -textvariable] value
    upvar #0 ::Entry::entry_data($widget,upper_max) upper_max

    # Get the insert point
    set insert  [$widget index insert]
  
    # IF MAX NUMBER OF DIGITS ALREADY ENTERED, DO NOTHING
    if {$insert >= $upper_max} \
    {
        return
    }
     
    upvar #0 [$widget cget -textvariable]  value
    
    set str_length  [string length $value]
    
    set first_str_half   [string range $value  0  [expr $insert - 1]]
    set second_str_half  [string range $value  [expr $insert + 1]  $str_length]
    
    set new_c  $key

    # Insert the new character
    set new_string  ""

    append  new_string  $first_str_half  $new_c  $second_str_half
    
    set value  "$new_string"


    $widget  icursor  [expr $insert + 1] 
    
    $widget  xview  insert

    Highlight  $widget
    
    return
  }
  
  
  #**************************************************************************
  #
  #        Name: Start_edit
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <<EditOpen>> event for the entry widget
  #              Sets the background and selectbackground colors
  #              to the edit colors, and sets the edit mode to 1 (ON).
  #
  #**************************************************************************
  proc Start_edit { widget } \
  {
    #puts "P25DigitEntry::Start_edit:  $widget"

    # Do nothing if not enabled for edit
    if { [$widget cget -state] == "disabled" } \
    { 
      return 
    }
  
    upvar ::Entry::entry_data($widget,editon)  editon
    
    upvar #0 ::Entry::entry_data($widget,editbgcolor)  editbgcolor
  
    set editon  1

    # SAVE CURRENT SOFTKEYS TO BE LATER RESTORED
    View::StoreSoftkeys  $widget

    set topLevelWin  [winfo toplevel  $widget]
    
    set tile_path  [name_child  $topLevelWin  key_bar]

    if {![winfo exists  $tile_path]} \
    {
      set topLevelWin  [winfo parent  $topLevelWin]
      
      set tile_path  [name_child  $topLevelWin  key_bar]
    }

    foreach { key letter } { 2 A 3 B 4 C 5 D } \
    {
      View::AssignSoftkey  $tile_path  $key  $letter                        \
                          "event generate $tile_path <Key-$letter>"         \
                          "event generate $tile_path <KeyRelease-$letter>"  \
                          $editbgcolor
    }

    # Softkey 6 - Hookflash
    View::AssignSoftkey  $tile_path  6  "HOOK\nFLASH"                 \
                          "event generate $tile_path <Key-H>"         \
                          "event generate $tile_path <KeyRelease-H>"  \
                          $editbgcolor

    # Softkey 7 - Pause
    View::AssignSoftkey  $tile_path  7  "PAUSE"                       \
                          "event generate $tile_path <Key-P>"         \
                          "event generate $tile_path <KeyRelease-P>"  \
                          $editbgcolor

    bind $widget  <FocusOut>  "event generate  %W  <<EditClose>>"

    $widget  config  -bg $editbgcolor \
                     -selectbackground [option get $widget edisel {}]

    $widget  config  -insertofftime 500
    $widget  config  -insertontime  500
  
    Highlight  $widget

    return
  }

 
  #**************************************************************************
  #
  #        Name: Stop_edit
  #
  #  Parameters: widget - invoking widget
  #
  # Description: Handle the <<EditClose>> event for the entry widget.
  #              Returns the background and selectbackground colors
  #              to the orignal state, and sets the edit mode to 0 (OFF).
  #
  #**************************************************************************
  proc Stop_edit { widget } \
  {
    #puts "P25DigitEntry::Stop_edit:  $widget"

    if { [info exists editon] && $editon == 0 } \
    {
      # If not editing then exit
      return
    }
    
    # Restore the softkeys
    View::RestoreSoftkeys  $widget


    upvar ::Entry::entry_data($widget,editon)  editon
    upvar ::Entry::entry_data($widget,back)    backcolor
  
    bind $widget  <FocusOut>  {}

    # Stop the insert point from showing
    $widget  config  -insertofftime 1000
    $widget  config  -insertontime  0

    $widget select clear   

    $widget  config  -bg $backcolor  -selectbackground $backcolor
    
    set editon  0

    return
  }

  
  #***************************************************************************
  #
  #        Name: Button_1
  #
  #  Parameters: widget - invoking widget
  #              x      - mouse position x value
  #
  # Description: Handle the <<Button-1>> event for the entry widget
  #              Sets the insert cursor to the position of the mouse x
  #              coordinate, then generates the <<EditOpen>> event if the
  #              edit mode is 0 (OFF)
  #
  #***************************************************************************
  proc Button_1 { widget  x } \
  {
    #puts "Button_1:  $widget,  $x"
    
    upvar #0 ::Entry::entry_data($widget,editon)  editon
  
    # if this variable isn't set up, or this widet's gone, bail.
    if { ![info exists editon] || ![winfo exists $widget] } \
    { 
      return 
    }
    
    # Position the selection point
    $widget icursor @$x
    
    
    # If the insert cursor is to the far right,
    # adjust it to the last visible character
    set insert  [$widget index insert]

    upvar #0 [$widget cget -textvariable]  value
    
    set str_length  [string length $value]
    
    if { $insert == $str_length } \
    {
      incr insert  -1
      
      $widget icursor  $insert
    }

    Highlight $widget
      

    if { $editon == 0 } \
    {
      # Start the edit
      # Generate an <<EditOpen>> event
      event generate  $widget  <<EditOpen>>
    }
    
    return
  }


  #***************************************************************************
  #
  #        Name: Left
  #
  #  Parameters: widget
  #
  # Description: Handles the <<edit_left>> event 
  #
  #***************************************************************************
  proc Left { widget } \
  {
    # If we aren't enabled, bail right now
    if { [$widget cget -state] == "disabled" } { return }
   
    # If the edit isn't open, generate the edit open event, and
    # regenerate this event.
    upvar #0 ::Entry::entry_data($widget,editon)  editon
    
    if { ![info exists editon] } { set editon 0 }
    
    if { !$editon } \
    {
      event generate $widget <<EditOpen>>

      # Continue editing
      ##event generate $widget <<edit_left>>
      ##return
    }
   
    upvar #0 [$widget cget -textvariable]  value
   
    set sel  [$widget index insert]
    
    incr sel  -1
    
    $widget  icursor  $sel
  
    $widget  xview  insert

    Highlight $widget
    
    return
  }


  #***************************************************************************
  #
  #        Name: Right
  #
  #  Parameters: widget
  #
  # Description: Handles the <<edit_right>> event 
  #
  #***************************************************************************
  proc Right { widget } \
  {
    # If we aren't enabled, bail right now
    if { [$widget cget -state] == "disabled" } { return }
   
    # If the edit isn't open, generate the edit open event, and
    # regenerate this event.
    upvar #0 ::Entry::entry_data($widget,editon)  editon
   
    if { ![info exists editon] } { set editon 0 }
    
    if { !$editon } \
    {
      event generate $widget <<EditOpen>>

      # Continue editing
      ##event generate $widget <<edit_right>>
      ##return
    }
   
    upvar #0 [$widget cget -textvariable]  value
   
    set sel  [$widget index insert]
    
    incr sel  1
   
    $widget  icursor  $sel
  
    $widget  xview  insert

    Highlight $widget
    
    return
  }
  

  #***************************************************************************
  #
  #        Name: Highlight
  #
  #  Parameters: widget - invoking widget
  #
  # Description: shows currently selected char 
  #
  #***************************************************************************
  proc Highlight { widget } \
  {
    # now, get where the cursor really is
    set sel  [$widget index insert]
 
    $widget selection  range  $sel  [expr $sel + 1]
  }


};  # End of P25DigitEntry

 
 
 
 
 
 
 
 
