##############################################################################
# IFR SYSTEMS, INC          COPYRIGHT (C)1998-2001        DESIGN ENGINEERING #
# -------------------------------------------------------------------------- #
# FILE        : 1400.units_convert.tcl                                       #
# PRODUCT     : FITS                                                         #
# AUTHOR      : David Hagood                                                 #
# DESCRIPTION : Automatically convert between various units                  #
#                                                                            #
# The purpose of this file is to define routines to convert from 1 or more   #
# displayed units to the format an engine requires.                          #
# The parameters to the setup function are:                                  #
# {engine var, units} this list contains the name of the engine var and the  #
# units the engine var is in (W, dBm, dBW, dBV, dBmV, dBuV)                  #
# command - this is the command to issue when the engine needs to be updated #
# {display_var, units} this list contains the name of the UI var and the     #
# units it is in (as per above).                                             #
#                                                                            #
# The function will then connect the two sets of variables, so than an update#
# to the UI variable will drive a call to the engine update function with the#
# new value passed in.                                                       #
#                                                                            #
# An update to the engine variable will update the UI value IF the new value #
# would be different by more than an epsilon value - this prevents conversion#
# from confusing the display                                                 #
# 
##############################################################################
# 
# Design assumptions:
#    o  For an engine parameter which can be displayed in several units, the
#       engine TCL/C++ interface will maintain several copies of the data and
#       convert the engine reply into unit specific variables.
#    o  For a spin entry field, the data IS sent to the engine periodically 
#       while the spinner is moving or if the UP/DOWN arrows are pressed  
#    o  When typing in numbers, the data is NOT sent to the engine while the
#       digits are being entered. Since the spin entry widget modifies the 
#       display variable during key entry, it creates a problem to trace on the
#       entry data field.
# 
# 
# Design requirements:
#    o  Data conversion must be made on raw data, and not formatted data, to
#       preserve accuracy
#    o  Apart from the update of a variable by an entry widget, the data
#       fields will only be updated as a response to a change in engine data,
#       not by a change in a corresponding data field in a different unit.
# 
# 
# 
##############################################################################

namespace eval Units_convert \
{

  #********************************************************************
  #
  #       Name: Connect
  #
  # Parameters: {engine_var, units}, engine_cmd, {display_var,units}...
  #
  # Description: connect zero or more display fields to an engine field
  #
  #********************************************************************
  proc Connect { engine cmd args } \
  {
     set engine_var   [lindex $engine 0]
     set engine_units [lindex $engine 1]
     
     foreach {display } $args \
     {
        set display_var    [lindex $display 0]
        set display_units  [lindex $display 1]
        set display_format [lindex $display 2]
        
        set convert_from_cmd ${display_units}_to_${engine_units}
        set convert_to_cmd   ${engine_units}_to_${display_units}
        
        set from_cmd "Units_convert::engine_to_display $engine_var $display_var $convert_to_cmd $display_format"
        set to_cmd   [subst "Units_convert::display_to_engine $display_var $convert_from_cmd {$cmd}"]
        
        puts "$engine_var $from_cmd"
        puts "$display_var $to_cmd"
        trace variable $engine_var  w "Units_convert::trap \"$from_cmd\""
        trace variable $display_var w "Units_convert::trap \"$to_cmd\""
        eval $from_cmd
     }
  }
  
  #********************************************************************
  #
  #       Name: trap
  #
  # Parameters: command to execute, args (ignored)
  #
  # Description: schedule an update
  #
  #********************************************************************
  proc trap { cmd args } \
  {
     after cancel $cmd
     after idle $cmd
  }
  
  #********************************************************************
  #
  #       Name: engine_to_display
  #
  # Parameters: engine var to read, display var to update
  #
  # Description: update the display var if the change is greater than
  #              the threshold
  #
  #********************************************************************
  proc engine_to_display { engine_var display_var convert_cmd format } \
  {
     upvar #0 $engine_var engine
     upvar #0 $display_var display
     
     if [catch {set new [$convert_cmd $engine] } err ] { return }
     
     set new [format $format $new]
     
     if {$new != $display } \
     {
        puts "engine_to_display: update $display_var to $new"
        set display $new
     }
  }
  
  #********************************************************************
  #
  #       Name: display_to_engine
  #
  # Parameters: display var to read, cmd to convert, engine command to execute
  #
  # Description: update the engine
  #
  #********************************************************************
  proc display_to_engine { display_var convert_cmd cmd } \
  {
     upvar #0 $display_var display
     
     if [catch {set new [$convert_cmd $display] } err ] { return }
     
     puts "display_to_engine - $cmd $new "
     eval $cmd $new -nowait
  }
   
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc to_dB { units } \
  {
     if {$units == 0 } { return -120 }
     return [expr 10*log10($units)]
  }
  
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc from_dB { units } \
  {
     return [expr pow(10,$units/10)]
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc W_to_W { watts } \
  {
     return $watts
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc W_to_mW { watts } \
  {
     return [expr $watts*1000]
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc W_to_uW { watts } \
  {
     return [expr $watts*1000000]
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc uW_to_W { watts } \
  {
     return [expr $watts/1000000]
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc mW_to_W { watts } \
  {
     return [expr $watts/1000]
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc dBm_to_dBm { x } \
  {
     return $x
  }
  
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc W_to_dBm { watts } \
  {
     return [to_dB [expr $watts*1000]]
  }
    
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc dBm_to_W { db } \
  {
     return [expr [from_dB $db] / 1000 ]
  }

  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc W_to_dBW { watts } \
  {
     return [to_dB $watts]
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc dBW_to_W { db } \
  {
     return [from_dB $db]
  }
    
    
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc W_to_V { watts } \
  {
     return [expr sqrt($watts*50)]
  }
  
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc V_to_W { volts } \
  {
     return [expr $volts*$volts/50]
  }
  
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc W_to_mV { watts } \
  {
     return [expr [W_to_V $watts] * 1000]
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc mV_to_W { mv } \
  {
     return [V_to_W [expr $mv/1000]]
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc W_to_uV { watts } \
  {
     return [expr [W_to_V $watts] *1000000]
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc uV_to_W { mv } \
  {
     return [V_to_W [expr $uv/1000000]]
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc W_to_dBV { watts } \
  {
    return [to_dB [W_to_V $watts]]
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc dBV_to_W { db } \
  {
    return [V_to_W [from_dB $db]]
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc W_to_dBmV { watts } \
  {
     return [to_dB [W_to_mV $watts]]
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc W_to_dBuV { watts } \
  {
     return [to_dB [W_to_uV $watts]]
  }
  #********************************************************************
  #
  #       Name: 
  #
  # Parameters: 
  #
  # Description: 
  #
  #********************************************************************
  proc dBuV_to_W { db } \
  {
     return [uV_to_W [from_db $db]]
  }

}
