#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1998-2001      DESIGN ENGINEERING    #
# --------------------------------------------------------------------------#
# FILE        : 1411.download_ftp.tcl                                       #
# PRODUCT     : FITS                                                        #
# AUTHOR      : David Hagood/Hieu Tran                                      #
# DESCRIPTION : Download files via FTP  (as per RFC 0542)                   #
#                                                                           #
#############################################################################

# Delete the namespace if it exists to insure that it is clean
foreach ns [namespace children ::] \
{
    if {$ns == "::ftp"} {
        namespace delete ::ftp
        break
    }
}

namespace eval ::ftp {
   namespace export create done configure open get file_done debug dir binary ascii
   namespace import ::HtmlFrame::split_url
   
   # count of # of FTP transfers done. Used to create FTP
   # state variables
   
   variable count 0
   variable debug_flag 0
   variable debug_screen :0

#*************************************************
#       Name: debug
# Parameters: ftp token
#Description: create a debug window for the process 
#*************************************************
 proc debug { token {frame {}}  } {
   variable debug_screen
   
   if {"$frame"==""} { 
     # make new toplevel for this, based on the name
     set frame .debug$token
     toplevel $frame -screen $debug_screen
     wm title $frame $token
   }
 
   if {![winfo exists $frame ]} {
      frame $frame
      pack $frame
   }
   
   upvar #0 $token state
   set state(debug_frame) $frame
   bind $frame <Destroy> "set $token\(debug_frame) {}"
   
   pack propagate $frame false
  
   label $frame.token -text $token -bd 1 -relief groove -justify left \
                      -font "Helvetica -10 bold" -highlightthickness 0
   grid  $frame.token -column 0 -row 0 -columnspan 2 -sticky news
   grid rowconfigure $frame 0 -weight 0
   grid rowconfigure $frame 0 -weight 1
   
   set items [array names $token]
   set row 1
   foreach thing [lsort $items] {
     label $frame.$thing\_title -text $thing -relief groove -justify left \
                                -font "Helvetica -10 bold" -bd 1 -highlightthickness 0
     label $frame.$thing -textvar $token\($thing) -relief groove    \
                         -justify left -font "Helvetica -10 bold"   \
                         -wraplength 5i -bd 1 -highlightthickness 0
     grid $frame.$thing\_title -column 0 -row $row -sticky news
     grid $frame.$thing        -column 1 -row $row -sticky news
     incr row
   }
}
 
#*************************************************
#       Name: login
# Parameters: ftp token
#Description: send username and password to FTP server 
#*************************************************
 proc login { token  } {
   upvar #0 $token state

   set state(status) "LOGGING IN"
   # send the login request, and wait til we get in
   reply $token "USER $state(user)"
   # try at most 10 times
   for {set i 1 } { $i < 10 } { incr i } {
     switch -glob -- $state(code) {
        33* { 
              set state(status) "PASSWORD"
              reply $token "PASS $state(password)"
            }
        2*  {
              set state(status) "LOGGED IN"
              return 1
            }
        5*  {
              set state(status) "ERROR"
              return 0
            }
        default {
              reply $token
            }
     }
   }
   set state(status) "ERROR RETRY EXCEEDED"
   return 0
 }


#*************************************************
#       Name: port
# Parameters: ftp token 
#Description: setup listen port for file transfer 
#*************************************************
 proc port { token } {
   upvar #0 $token state
   
   # close old port
   if {"$state(data_socket)" != "" } {
      close $state(data_socket)
      set state(data_socket) ""
   }
   
   set state(status) "SETTING PORT"
   # get local machine IP address
   regexp {([0-9]*).([0-9]*).([0-9]*).([0-9]*)} $state(my_addr) match a1 a2 a3 a4

   # set up an incoming server on a randomly chosen port
   set state(data_socket) [ socket -server "::ftp::data $token" 0 ]
   # find the port we are using
   set state(data_port) [ lindex [ fconfigure $state(data_socket) -sockname ] 2 ]

   # split 16 bit port into high and low bytes
   set serv_up [ expr "int($state(data_port)/256)"]
   set serv_lw [ expr "$state(data_port)-$serv_up*256"]

   # send the port command to the remote system
   reply $token "PORT $a1,$a2,$a3,$a4,$serv_up,$serv_lw"

   switch -glob -- $state(code) {
      2* { return 1 }
      5* { 
           set state(status) "Error creating receive port"
           return 0
         }
      default {
           set state(status) "unknown err"
           return 0
         }
   }
 }
 
#*************************************************
#       Name: configure
#Parameters: token name, arg list
#Description: parse command options
#Commands understood:
#    -filename <file> receive data into file
#    -command  <cmd>  command to execute when complete
#    -handler  <cmd>  command to execute when data avail
#*************************************************
 proc configure { token args } {
    upvar #0 $token state
    # parse the options
    if {"$args" == "{}" } { return }
    foreach {cmd arg } $args {
      switch -- [string tolower $cmd] {
         -filename  { set state(savefilename) $arg }
         -command   { set state(callback) $arg }
         -handler   { set state(handler)  $arg }
         -channel   { set state(channel)  $arg }
         -blocksize { set state(blocksize)  $arg }
         -progress  { set state(progress)  $arg }
         "{}"      { }
         default   { 
            error "invalid commands $cmd; should be -filename, -command or -handler"
         }
      }
    }
 }

#*************************************************
#       Name: create
# Parameters: FTP URL array name, arg list
#Description: open connection for FTP operations
#Commands understood:
#    -filename <file> receive data into file
#    -command  <cmd>  command to execute when complete
#    -handler  <cmd>  command to execute when data avail
# returns the FTP token - an array in our namespace
# containing the info on the item
#*************************************************
 proc create { url args } {
   variable count
   variable debug_flag
   
   # create the token name
   set token ::ftp::$count
   incr count
   
   # create the token
   variable $token
   upvar #0 $token state
   
   foreach item [lsort { anchor anchorNsep callback code    \
                  control data data_port data_socket \
                  debug_frame fd file_done file_socket filename  \
                  handler msg my_addr password path  \
                  port portNsep proto server         \
                  server_addr status url user savefilename \
                  channel blocksize total bytes progress 
                }] {
      set state($item) ""
   }
   
   set state(url)      $url
   split_url $url      state 
   set state(status)   "INIT"
   set state(handler)  "::ftp::copy"
   set state(blocksize) 8192
   
   if {"$state(user)"     == "" } { set state(user)     "anonymous"}
   if {"$state(password)" == "" } { set state(password) "2975@ifrsys.com"}
   if {"$state(port)"     == "" } { set state(port)     21}

   eval configure $token $args
   
   if {$debug_flag != 0 } { debug $token }

   return $token
 }
#*************************************************
#       Name: Open
# Parameters: FTP URL array name
#Description: open connection for FTP operations
#*************************************************
 proc Open { token } {
   upvar #0 $token state
   set state(status) "CONNECTING"
   # connect to FTP server
   if [ catch { set state(control) [socket $state(server) $state(port)]} err ] {
      error "Cannot connect to FTP server $state(server):$state(port) - $err"
   }
   # find my address from the socket
   set state(my_addr)    [ lindex [ fconfigure $state(control) -sockname ] 0 ]
   # get the remote site's address
   set state(server_addr) [ lindex [ fconfigure $state(control) -peername ] 0 ]
   # set the control port to line buffering
   fconfigure $state(control) -buffering line
   
   # get host login
   reply $token
   switch -glob -- $state(code) {
      2* {
           login $token
         }
      default {
         set state(status) "UNKNOWN RESPONSE"
         error $state(status)
      }
   } 
   
   switch -glob -- $state(code) {
      2* {
           set state(status) "READY"
         }
      default {
         set state(status) "UNKNOWN RESPONSE"
         error $state(status)
      }
   }
   return
 }


#*************************************************
#       Name: done
# Parameters: FTP token
#Description: close a connection to the server 
#*************************************************
 proc done { token {status "CLOSED"}} {
   upvar #0 $token state
   if {![info exists $token] } { return }
   if {"$state(file_socket)" != "" } {
      puts $state(control) "ABOR"
      file_done $token $status
   }
   if {"$state(control)" != "" } {
     reply $token "QUIT"
     close $state(control)
     set state(control) ""
   }
   set state(file_done) 1
   set state(status) $status
 }
 
#*************************************************
#       Name: wait
# Parameters: FTP token
#Description: wait until retrieve is done
#*************************************************
 proc wait { token {filename ""} } {
   upvar #0 $token state
    switch -glob -- $state(code) {
      4* {
         set state(status) "Cannot retrieve"
         error $state(status)
      }
      5* {
         set state(status) "Cannot retrieve"
         error $state(status)
      }
   }
   
   # if a callback command was registers, return right now.
   if {"$state(callback)" != "" } {
      return
   }

   # no callback registered. Wait until transfer done
   if {$state(file_done) == 0 } {
      set state(status) "WAIT FOR DOWNLOAD"
      append v $token (file_done)
      vwait $v
   }
   
   return
 }
 
#*************************************************
#       Name: binary
# Parameters: FTP token
#Description: set to binary mode 
#*************************************************
 proc binary { token } {
   upvar #0 $token state

   set state(status) "SET XFER TYPE"
   # set to binary transfer
   reply $token "TYPE I"
 }

#*************************************************
#       Name: ascii
# Parameters: FTP token
#Description: set to binary mode 
#*************************************************
 proc ascii { token } {
   upvar #0 $token state

   set state(status) "SET XFER TYPE"
   # set to binary transfer
   reply $token "TYPE A"
 }
#*************************************************
#       Name: get
# Parameters: FTP token
#Description: retrieve file from FTP server 
#*************************************************
 proc get { token {filename ""} } {
   upvar #0 $token state
   
   if {"$state(control)" == "" } { Open $token }

   binary $token

   set state(file_done) 0
   
   port $token
   
   if {"$filename" != "" } {
      set state(path) $filename
   }
       
   set state(status) "REQUESTING"
   set state(bytes) 0
   set state(total) 0
   reply $token "RETR $state(path)"
   
   if {$state(code) == 150 } {
     set state(total) {}
     #file retrieve started. see if we got a size
     if [regexp --  \[^(\]*\[\(\](\[0-9\]+).* $state(msg) foo size] {
        set state(total) $size
     }
   }
   wait $token
 }

#*************************************************
#       Name: dir
# Parameters: FTP token
#Description: retrieve directory listing from FTP server 
#*************************************************
proc dir { token {filename ""} } {
   upvar #0 $token state

   if {"$state(control)" == "" } { Open $token }

   ascii $token

   set state(file_done) 0
   
   port $token
   
   if {"$filename" != "" } {
      set state(path) $filename
   }
       
   set state(status) "REQUESTING"
   reply $token "LIST $state(path)"

   wait $token
}

#*************************************************
#       Name: data
# Parameters: token - FTP token
#             sock - socket channel
#             addr - IP address
#             port - port connection
#Description: call when socket become readable and
#             start download file
#*************************************************
 proc data { token sock addr port } {
   upvar #0 $token state

   # are we expecting this? In other words, did it
   # come from our server and the FTP data port?
   if { $addr != $state(server_addr) } {
      close $sock
      set state(status) "Bogus connect addr!"
   }
   if { $port != 20 } {
      close $sock
      set state(status) "Bogus connect port!"
   }
   set state(file_socket) $sock
   set state(status) "DOWNLOADING"
   # set the files for binary mode, big buffers
   fconfigure $state(file_socket) -buffersize $state(blocksize) \
                                  -translation binary           \
                                  -blocking 0

   # do we have a filename specified? if so open it
   if {"$state(savefilename)" != "" } {
      set state(fd) [open $state(savefilename) w]
      fconfigure $state(fd) -buffersize $state(blocksize) \
                            -translation binary 
   } else {
      if {"$state(channel)" != "" } {
        set state(fd) $state(channel)
      } else {
        set state(fd) ""
      }
   }

   # tie the socket to the handler
   fileevent $sock readable "$state(handler) $token"
   
 }
#*************************************************
#       Name: file_done
# Parameters: token - FTP token
#Description: called at end of transfer
#*************************************************
 proc file_done { token {status "FILE DONE"} } {
    upvar #0 $token state
   
    # close the files
    if {"$state(file_socket)" != "" } {
       fileevent $state(file_socket) readable ""
       close $state(file_socket)
       set state(file_socket) ""
    }
    if {"$state(fd)" != "" } {
       close $state(fd)
       set state(fd) ""
    }
    reply $token
    set state(status) $status
    set state(file_done) 1
    # if a callback has been specified, call it
    if {"$state(callback)" != "" } {
       eval $state(callback) $token
    }
 }

#*************************************************
#       Name: read_data
# Parameters: token - FTP token
#Description: copy a block of data
#*************************************************
 proc read_data { token bytes_read {err "" } } {
    upvar #0 $token state
    
    if {"$state(file_socket)" == "" } { return }
    
    if {"$err" != "" } {
       error $err
       return
    }
    incr state(bytes) $bytes_read
    
    # service update handler
    if {"$state(progress)" != "" } {
       eval $state(progress) $token \"$state(total)\" \"$state(bytes)\"
    }
    
    # check for EOF
    if [eof $state(file_socket)] {
      file_done $token
      return
    }
 }
 
#*************************************************
#       Name: copy
# Parameters: token - FTP token
#Description: default handler - copy the data
#*************************************************
 proc copy { token } {
   upvar #0 $token state
   
  if {"$state(file_socket)" == "" } { return }

   # if we have a file, just copy the data in.
   # if not, copy it into our (data) member
   if {"$state(fd)" == "" } {
      append state(data) [read $state(file_socket)]
      set state(bytes) [string length $state(data)]
      file_done $token
   } else {
      fcopy $state(file_socket) $state(fd) \
         -size $state(blocksize)           \
         -command "::ftp::read_data $token"
   }
 }

#*************************************************
#       Name: reply
# Parameters: FTP token 
#Description: get reply from FTP server 
#*************************************************
 proc reply { token {cmd {}}} {
   variable debug_flag
   upvar #0 $token state
     # if a command was provided, send it
   if {"$cmd" != "" } {
      if {$debug_flag > 1 } {
         puts "CMD: $cmd"
      }
      puts $state(control) $cmd
   }
   
   set state(msg) [ gets $state(control) ]
   regexp {([0-9]*)} $state(msg) state(code)
   if {$debug_flag > 1 } {
      puts "REPLY: $state(msg)"
   }
   return $state(code)
 }
 
}
