#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C) 2001      DESIGN ENGINEERING        #
# --------------------------------------------------------------------------#
# FILE        : 2001.View.tcl                                               #
# PRODUCT     : FITS                                                        #
# AUTHOR      : Bruce Butterfield                                           #
# DESCRIPTION : This file contains the View namespace, which is used        #
#             : to display the title bar and softkeys of the instrument,    #
#             : and controls screen swapping based on the options menu      #
#############################################################################

#****************************************************************************
#
#  General purpose for the View Namespace  
#
#     o  The "View" will:
#        -  Create and display the clock
#        -  Create, display and maintain the main screen mode menu
#        -  Provide funtions connected to the main screen mode menu to switch
#           the screens
#        -  Create, display and provide functions which modify softkey response
#        -  Provide functions to display and manage the RETURN softkey
#
#****************************************************************************



#****************************************************************************
#
#  View Namespace  
#
#****************************************************************************
#
#  View Private Window Names:
#
#     $toplevel.key_bar               Vertical window holding the softkeys
#     $toplevel.key_bar.space$i       Name for spacing windows
#     $toplevel.key_bar.softkey$i     Name for softkey "i"
#     $topLevel.title_bar
#     $topLevel.title_bar.clock
#     $topLevel.title_bar.clock.text
#     $topLevel.title_bar.mode
#     $topLevel.title_bar.mode.mnu_mode
#     $topLevel.title_bar.mode.mnu_options
#     $topLevel.title_bar.setup
#     $topLevel.title_bar.setup.lbl
#     $topLevel.title_bar.setup.num
#     $topLevel.title_bar.setup.name
#
#  View Private variables:
#
#     KeyBarMode		Array of flags indexed by toplevel and window
#                               which allows the clock and options key to be
#                               displayed or removed.
#
#****************************************************************************
#
#  Public Functions:
#
#     CreateView 		    Creates an instrument view for a toplevel
#     GetMainView 		  Returns the window for the main instrument area
#     PackOptions  		  Packs an option button on the keybar
#
#     AssignSoftkey 	  Sets up a standard Softkey by index {2 - 7}
#     AssignReturn        sets up the handling of the return key
#     ClearSoftkey 		  Removes a Softkey by index {2 - 7}
#     ClearAllSoftkeys 	Removes all Softkeys
#
#****************************************************************************
#
#  Private Functions:
#
#     CreateSoftkeys	Creates a softkey sidebar for a toplevel
#     PackKeyBar        Packs and unpacks the clock, options and main key
#
#****************************************************************************

namespace eval View \
{
  variable KeyBarMode
  variable AvailScreens
  variable ScreenCount
  variable ScreenList
  variable Skey
  
  array set Skey {}
  
  variable PreCreate
   
  set ScreenList {}
  set ScreenCount 0

  set ::View::KeyBarMode(.clock)   1
  set ::View::KeyBarMode(.options) 0
  set ::View::AvailScreens { }
  set ::View:::MenuListPtr(/system/screens) ::View::AvailScreens
  
  bind View <Destroy> { View::Destructor  %W }
  
  variable Alert_routines
  variable Clean_ups

  # remove any child toplevels of . from the screen
  proc ClearChildren { } \
  {
     set screen [winfo screen .]
     
     foreach child [winfo children .] \
     {
        if { [winfo toplevel $child] == $child } \
        {
           if { $screen == [winfo screen $child] } \
           {
               if [winfo ismapped $child] \
               {
                  destroy $child
               }
           }
        }
     }
  }
  #****************************************************************************
  #
  # Name:        Find_Main_toplevel
  #
  # Parameters:  widget
  #
  # Description: find the main toplevel given a child
  #
  #****************************************************************************
  proc Find_Main_toplevel { widget } \
  {
     while {[winfo class $widget] != "IFR-2975"} \
     {
        set newitem [winfo toplevel $widget]
        if {$newitem == $widget} \
        {
           set newitem [winfo parent $widget]
        }
        set widget $newitem
     }
     return $widget
  }
  
  #****************************************************************************
  #
  # Name:        Destructor
  #
  # Parameters:  topLevelWin - toplevel window being destroyed
  #
  # Description: clean up a toplevel that is going away
  #
  #****************************************************************************
  proc Destructor { toplevel } \
  {
    Debug::Puts UI 5 "Destructor:  $toplevel"

    # remove ourselves from the screen list
    variable  ScreenList
    variable  Clean_ups
     
    set index  [lsearch $ScreenList $toplevel]
    set ScreenList  [lreplace $ScreenList $index $index]
     
    # Now, call any registered cleanup routines
    if { [info exists Clean_ups] } \
    {
      foreach  cleanup  $Clean_ups \
      {
        eval $cleanup $toplevel
      }
    }

    cancel_at_exit "View::show_powerdown $toplevel"
    
    return
  }


  #****************************************************************************
  #
  # Name:        CreateSoftkeys
  #
  # Parameters:  topLevelWin - toplevel window for screen to be cleared
  #
  # Description: Creates the soft keys for a specific toplevel view
  #
  #****************************************************************************
  proc CreateSoftkeys { topLevelWin {name key_bar} } \
  {
    set topLevelWin  [winfo toplevel $topLevelWin]
    
    set keyBar  [name_child $topLevelWin $name]

    if { [winfo exists $keyBar] } \
    {
      return $keyBar
    }

    frame $keyBar -class KEY_BAR

    pack propagate $keyBar false


    # Create and place the VOL/SQL key
    button $keyBar.volsq -command "::VolSquelch::Show $keyBar.volsq" -padx 2

    pack $keyBar.volsq -side top -expand true -fill x

    add_bindtag $keyBar.volsq  Softkey Reflow
    $keyBar.volsq config -takefocus 0
    
    ::VolSquelch::Create $topLevelWin

    frame $keyBar.space0 -class SOFT_SPACER
    
    pack  $keyBar.space0 -side top -fill x

    # Create and place 6 softkeys
    for { set i  2 } { $i < 8 } { incr i  1 } \
    {
      set bfrm  [frame $keyBar.frm$i]
      
      pack $bfrm -expand true -side top -fill x
      
      set b  [button $bfrm.softkey$i  -takefocus 0 -padx 2]
      
      set ::View::Skey(KeyType${bfrm})   SINGLE
      set ::View::Skey(DualName${bfrm})  ""
      
      pack $bfrm.softkey$i -expand true -side top -fill x

      set s  [frame $keyBar.space$i -class SOFT_SPACER]

      pack  $s -side top -fill x


      add_bindtag $b Softkey Reflow
    }

    # Create and place the RETURN key
    button $keyBar.return -takefocus 0 -padx 2

    add_bindtag $keyBar.return Softkey Reflow
 
    pack $keyBar.return -side top -expand true -fill x
    
    # Default disable RETURN key
    $keyBar.return configure -state disabled

    return $keyBar
  }

  #****************************************************************************
  #
  # Name:        FindSoftKeyOwner
  #
  # Parameters:  A widget to start from
  #
  # Description: Locate the owner of the current softkey strip and return it
  #
  #****************************************************************************
  proc FindSoftKeyOwner { widget } \
  {
     if ![winfo exists $widget] { return }
     if {[winfo class $widget] == "KEY_BAR"} { return [winfo parent $widget] }
     
     while 1 \
     {
        set look [name_child $widget key_bar]
        if [winfo exists $look] \
        {
           return $widget
        }
        set widget [winfo parent $widget]
        if {$widget == ""} { return }
        set widget [winfo toplevel $widget]
     }
     return $widget
  }
  proc FindSoftKeyBar { widget } \
  {
     if ![winfo exists $widget] { return }
     if {[winfo class $widget] == "KEY_BAR"} { return $widget }
     
     while 1 \
     {
        set look [name_child $widget key_bar]
        if [winfo exists $look] \
        {
           return $look
        }
        set widget [winfo parent $widget]
        if {$widget == ""} { return }
        set widget [winfo toplevel $widget]
     }
     return
  }
  
  #****************************************************************************
  #
  # Name:       UseSoftkey
  #
  # Parameters:  softkey widget to use
  #
  # Description: Replace current softkey strip with a different one
  #
  #****************************************************************************
  proc UseSoftkey { widget } \
  {
     variable Pending_skey_changes
     
     if ![winfo exists $widget] { return }
     if {[winfo class $widget] != "KEY_BAR"} { return }
     set toplevel [FindSoftKeyOwner $widget]
     if [info exists Pending_skey_changes($toplevel)] \
     {
        after cancel $Pending_skey_changes($toplevel)
     }
     set Pending_skey_changes($toplevel) [after 250 [namespace code "UseSoftkeyCore $widget"]]
  }
  proc UseSoftkeyCore { widget } \
  {
     variable Pending_skey_changes
     if ![winfo exists $widget] { return }
     if {[winfo class $widget] != "KEY_BAR"} { return }
     set toplevel [FindSoftKeyOwner $widget]
     if ![winfo exists $toplevel] { return }
     # remove pending callback
     unset Pending_skey_changes($toplevel)

     # locate current key bar and remove it
     foreach slave [pack slaves $toplevel] \
     {
        if {[winfo class $slave] == "KEY_BAR"} \
        {
           if {$slave != $widget} \
           {
              pack forget $slave
              pack $widget -side right -fill y
           }
           return
        }
     }
     pack $widget -side right -fill y
  }
  
  #****************************************************************************
  #
  # Name:        AssignSoftkey
  #
  # Parameters:  topLevelWin - toplevel window for screen to be cleared
  #
  # Description: Creates the soft keys for a specific toplevel view
  #
  #****************************************************************************
  proc AssignSoftkey { widget  id  label  { comUp "" } \
                                         { comDown "" }  { color grey } } \
  {
    if ![winfo exists $widget] { return }
    
    Debug::Puts UI 10 "AssignSoftkey $widget $id $label $comUp $comDown $color"
    set keyFrame [FindSoftKeyBar $widget]
    if ![winfo exists $keyFrame] { return }
    set keyBar $keyFrame.frm$id
    set skey $keyBar.softkey$id
    
    Debug::Puts UI 10 "AssignSoftkey $keyBar $comUp $comDown $color"
    
    set ::View::Skey(KeyType${keyBar})   SINGLE
    set ::View::Skey(DualName${keyBar})  ""

    PackSoftkey $widget $id $skey
    
    # if label is the NAME of a variable, us it. Else assume a string
    if [info exists $label] \
    {
       $keyBar.softkey$id configure -textvar $label -com $comUp -bg $color -activebackground $color
    } \
    else \
    {
        $keyBar.softkey$id configure -text $label -textvar "" -com $comUp -bg $color -activebackground $color
    }
    
    bind $keyBar.softkey$id <<Press>> $comDown
    
    return
  }


  #****************************************************************************
  # Name:        AssignDualSoftkey
  # Parameters:  widget		Path for widget controlling the softkey
  #              id		Number of soft key { 2 - 7 }
  #              label          General button label
  #              left_label     Text on left button
  #              right_label    Text on right button
  # Description: Creates the soft keys for a specific toplevel view
  #
  #****************************************************************************
  proc AssignDualSoftkey { widget  id  keyName } \
  {
    if { [winfo exists $keyName] } \
    {
      set keyFrame [FindSoftKeyBar $widget]
      set keyBar $keyFrame.frm$id
      
      set ::View::Skey(KeyType${keyBar})   DUAL
      set ::View::Skey(DualName${keyBar})  $keyName
        
      PackSoftkey $widget $id $keyName
    }
    
    return
  }


  #****************************************************************************
  # Name:        PackSoftkey
  # Parameters:  widget		Path for widget controlling the softkey
  #              id		Number of soft key { 2 - 7 }
  # Description: This routine packs the desired softkey in the frame
  #****************************************************************************
  proc PackSoftkey { widget  id  name } \
  {
    set keyFrame [FindSoftKeyBar $widget]
    set keyBar $keyFrame.frm$id
    
    if ![winfo exists $keyBar] { return }
    # Remove any keys previously in the frame
    foreach { keyWidget } [pack slaves $keyBar] \
    {
      pack forget $keyWidget
    }

    set test  $::View::Skey(KeyType${keyBar})

    if { $test == "DUAL" } \
    {
      pack $name -in $keyBar -expand true -side top -fill both
    } \
    else \
    {
      pack $keyBar.softkey$id -expand true -side top -fill x
    }
    
    return
  }


  #****************************************************************************
  #
  # Name:        AssignReturn
  #
  # Parameters:  widget - widget making request (used to find the toplevel)
  #              command - command string to be executed (empty string == disable )
  #
  # Description: Set the function associated with the Return button
  #
  #****************************************************************************
  proc AssignReturn { widget  { command "" } } \
  {
    if { ![winfo exists $widget] } { return }

    set keyBar [FindSoftKeyBar $widget]
    set returnkey  [name_child $keyBar return]

    if { ![winfo exists $returnkey] } { return }

    $returnkey configure -command $command
    
    if { "$command" == "" } \
    {
      $returnkey configure -state disabled
    } \
    else \
    {
      $returnkey configure -state normal
    }
    
    return
  }


  #****************************************************************************
  #
  # Name:        ClearSoftkey
  #
  # Parameters:  none
  #
  # Description: Creates the soft keys for a specific toplevel view
  #****************************************************************************
  proc ClearSoftkey { widget  id  { color grey } } \
  {
    
    set keyBar    [FindSoftKeyBar $widget]
    set keyFrame  [name_child $keyBar frm$id]

    set key     $keyFrame.softkey$id
    
    if { ![winfo exists $key] } { return }

    $key configure -text "" -textvar "" -com "" -bg $color -activebackground $color
    
    bind $key <<Press>> ""
    
    set ::View::Skey(KeyType${keyBar})  SINGLE
    
    PackSoftkey $widget $id $key
    
    return
  }


  #****************************************************************************
  #
  # Name:        ClearAllSoftkey
  #
  # Parameters:  none
  #
  # Description: Removes all softkeys from the display
  #
  #****************************************************************************
  proc ClearAllSoftkeys { widget } \
  {
    for { set i 2 } { $i < 8 } { incr i 1 } \
    {
      ::View::ClearSoftkey $widget $i
    }
  }


  #****************************************************************************
  #
  # Name:        StoreSoftkeys
  #
  # Parameters:  widget - widget making request (used to find the toplevel) 
  #
  # Description: Stores the softkeys' setting 
  #
  #****************************************************************************
  proc StoreSoftkeys { widget } \
  {
    set keyBar    [FindSoftKeyBar $widget]
        
    for { set i  2 } { $i < 8 } { incr i } \
    {
      set ::View::Skey(txt${keyBar}.frm${i}.softkey${i})      [${keyBar}.frm${i}.softkey${i} cget -text]
      set ::View::Skey(commup${keyBar}.frm${i}.softkey${i})   [${keyBar}.frm${i}.softkey${i} cget -command]
      set ::View::Skey(hue${keyBar}.frm${i}.softkey${i})      [${keyBar}.frm${i}.softkey${i} cget -bg]
      set ::View::Skey(commdown${keyBar}.frm${i}.softkey${i}) [bind ${keyBar}.frm${i}.softkey${i} <<Press>>]
      set ::View::Skey(keytype${keyBar}.frm${i})              $::View::Skey(KeyType${keyBar}.frm${i})
      set ::View::Skey(dualname${keyBar}.frm${i})             $::View::Skey(DualName${keyBar}.frm${i})
    }
  }     


  #****************************************************************************
  #
  # Name:        RestoreSoftkeys   
  #
  # Parameters:  widget - widget making request (used to find the toplevel) 
  #
  # Description: Restores the softkeys' setting  
  #
  #****************************************************************************
  proc RestoreSoftkeys { widget } \
  {
    set keyBar    [FindSoftKeyBar $widget]
    
    for { set i  2 } { $i < 8 } { incr i } \
    {
      # Text
      if { [info exists ::View::Skey(txt${keyBar}.frm${i}.softkey${i})] } \
      {
        ${keyBar}.frm${i}.softkey${i} config -text $::View::Skey(txt${keyBar}.frm${i}.softkey${i})
      } \
      else \
      {
        ${keyBar}.frm${i}.softkey${i} config -text ""
      }

      # Command up
      if { [info exists ::View::Skey(commup${keyBar}.frm${i}.softkey${i}) ]} \
      {
        ${keyBar}.frm${i}.softkey${i} config -command $::View::Skey(commup${keyBar}.frm${i}.softkey${i})
      } \
      else \
      {
        ${keyBar}.frm${i}.softkey${i} config -command {}
      }

      # Command down
      if { [info exists ::View::Skey(commdown${keyBar}.frm${i}.softkey${i})] } \
      {
        bind ${keyBar}.frm${i}.softkey${i} <<Press>> $::View::Skey(commdown${keyBar}.frm${i}.softkey${i})
      } \
      else \
      {
        bind ${keyBar}.frm${i}.softkey${i} <<Press>> {}
      }

      # Softkey color
      if { [info exists ::View::Skey(hue${keyBar}.frm${i}.softkey${i})] } \
      {
        ${keyBar}.frm${i}.softkey${i} config -bg $::View::Skey(hue${keyBar}.frm${i}.softkey${i}) \
   	                           -activebackground $::View::Skey(hue${keyBar}.frm${i}.softkey${i})
      } \
      else \
      {
        ${keyBar}.frm${i}.softkey${i} config -bg $::color(softkeybg) \
	                             -activebackground $::color(softkeybg)
      }

      # Initialize to default values
      set ::View::Skey(KeyType${keyBar}.frm${i})   SINGLE
      set ::View::Skey(DualName${keyBar}.frm${i})  ""

      if { [info exists ::View::Skey(keytype${keyBar}.frm${i})] } \
      {
        set ::View::Skey(KeyType${keyBar}.frm${i})  $::View::Skey(keytype${keyBar}.frm${i})
      }

      if { [info exists ::View::Skey(dualname${keyBar}.frm${i})] } \
      {
        set ::View::Skey(DualName${keyBar}.frm${i}) $::View::Skey(dualname${keyBar}.frm${i})
      }
     
      PackSoftkey $keyBar $i $::View::Skey(DualName${keyBar}.frm${i})
    }
    
    return
  }

      
  #****************************************************************************
  #
  # Name:        Handle_clock
  #
  # Parameters:  widget to update
  #
  # Description: show or remove the clock as requested
  #
  #****************************************************************************
  proc Handle_clock { widget  args } \
  {
    variable  KeyBarMode
    if ![winfo exists $widget.clock] \
    {
       after idle "View::Handle_clock $widget"
       return
    }
    if { $KeyBarMode(.clock) != 0 } \
    {
      # Pack the CLock frame
      pack $widget.clock -side left -fill y -before $widget.mode
    } \
    else \
    {
      # Pack forget the CLock frame
      pack forget $widget.clock
    }
    
    return
  }


  #****************************************************************************
  #
  # Name:        CreateView
  #
  # Parameters:  topLevelWin - toplevel window for screen to be cleared
  #
  # Description: Creates the main view of the instrument
  #
  #****************************************************************************
  proc CreateView { { topLevelWin "" }  { screen ":0.0" }  { title "" } } \
  {
    Debug::Puts UI 5 "CreateView:  $topLevelWin,  $screen,  $title"
    
    # Quick check: 
    # if we don't have enough memory for a new screen, throw an error
    if { ![Memory::Enough_for_view] } \
    {
      set msg  "Not enough memory for a new display"

      return -code error -errorcode {POSIX ENOMEM $msg} $msg
    }
    
    # Let them know we are busy...
    Busy
    
    variable  ScreenCount
    
    # If we weren't given a name, make one up as '.screen<number>'
    if { "$topLevelWin" == "" } \
    {
      set topLevelWin  .screen$ScreenCount
       
      incr ScreenCount
       
      while { [winfo exists $topLevelWin] } \
      {
        set topLevelWin  .screen$ScreenCount
       
        incr ScreenCount
      }
    }
    
    # Create the toplevel, if it is not defined
    if { ![winfo exists $topLevelWin] } \
    {
      # create a new view
      if [catch {toplevel $topLevelWin -class IFR-2975 -screen $screen} err ] \
      {
         NotBusy
         return
      }
    }
    $topLevelWin configure -bg grey
     
    # Set the toplevel title and icon
    if { "$title" == "" } \
    {
      set title "[IFR_options man] [IFR_options model] [IFR_options ser]"
    }
    wm withdraw    $topLevelWin
    wm title       $topLevelWin  $title

    if {$topLevelWin != "."} \
    {
       toplevel $topLevelWin.wait
       label $topLevelWin.wait.wait -text "Creating display\nPlease wait" -padx 10 -pady 10
       pack $topLevelWin.wait.wait -fill both -expand true
       add_bindtag $topLevelWin PleaseWait

       after 100 [namespace code "CreateView_core $topLevelWin"]
    } \
    else \
    {
       CreateView_core $topLevelWin
    }
    return $topLevelWin
}
bind PleaseWait <Map> { del_bindtag %W PleaseWait; after 1000 "destroy %W.wait"}
proc CreateView_core { topLevelWin } \
{
    wm iconbitmap  $topLevelWin  @/system/logo/logo.xbm
    wm geometry    $topLevelWin  640x480
    set topLevelWin  [winfo toplevel $topLevelWin]

    set keyWin  [CreateSoftkeys $topLevelWin]
    set mainframe [name_child $topLevelWin mainframe]
    if ![winfo exists $mainframe] \
    {
       set mainframe [frame $mainframe]
       pack propagate $mainframe false
    }
    
    pack $keyWin -side right -fill y
    pack $mainframe -side left -fill both -expand true

    Debug::Puts UI 2 "Creating title bar"
    
    # Create the Title Bar
    set titleBar  [name_child  $topLevelWin  title_bar]

    if { ![winfo exists $titleBar] } \
    {
      if { $topLevelWin == "." } \
      {
        set ::View::KeyBarMode(.clock)    1
        set ::View::KeyBarMode(.options)  0
      } \
      else \
      {
        set ::View::KeyBarMode($topLevelWin.clock)    1
        set ::View::KeyBarMode($topLevelWin.options)  0
      }

      Screens::create_tile  $titleBar  title_bar  TITLE_BAR
      pack $titleBar -in $mainframe -side top -fill x
      raise $titleBar
      
      # set up the clock so it will show (or not) automatically
      trace variable ::View::KeyBarMode(.clock) w "::View::Handle_clock $titleBar"

      Handle_clock  $titleBar
    }
    Debug::Puts UI 2 "Create main menu"
    
    BuildMainMenu  $topLevelWin
    # Create the main instrument window
    set mainWin  [name_child  $topLevelWin  mainView]

    if { ![winfo exists $mainWin] } \
    {
      frame $mainWin
     
      pack propagate $mainWin false
      
      pack $mainWin -in $mainframe -side top -expand true -fill both
      raise $mainWin
    }

    PackKeyBar $topLevelWin
    
    
    # add ourselves to the toplevel list, and remove ourselves when we are destroyed
    variable  ScreenList
    
    lappend ScreenList $topLevelWin
    
    add_bindtag  $topLevelWin  Toplevel_cleanup  View
    
    after idle "Mouse_cursor::Hook  $topLevelWin"
    
    after idle  "safe_focus_force [GetMainMenuName $topLevelWin]"
    
    # hook in any alert monitoring routines we need
    Debug::Puts UI 2 "Set up alert routines"
    variable  Alert_routines
    
    if [info exists Alert_routines ] \
    {
      foreach alert $Alert_routines \
      {
        eval $alert $topLevelWin
      }
    }
    Debug::Puts UI 2 "Precreate tiles"
    # Pre Create selected tiles
    Do_PreCreate  $topLevelWin
    Debug::Puts UI 2 "Showing"
    if { $topLevelWin != "." } \
    {
      ::Screens::Show  $topLevelWin  Intro_Screen
    }
    if {$topLevelWin != "."} \
    {
       after idle [subst {wm deiconify $topLevelWin; after idle NotBusy}]
    } \
    else \
    {
       after idle NotBusy
    }
    wm protocol $topLevelWin WM_DELETE_WINDOW "View::Destroy_view $topLevelWin"
  }
  proc Destroy_view { topLevelWin }\
  {
     puts "Destroying $topLevelWin"
     # we want to let thing calm down before destroying, so
     wm withdraw $topLevelWin
     wm protocol $topLevelWin WM_DELETE_WINDOW {}
     # I want to make sure all idle procs for this window are cleared
     set cmd {after idle {after idle {after idle {destroy $topLevelWin}}}}
     after idle [subst $cmd]
  }

  #****************************************************************************
  #
  # Name:        Do_PreCreate
  #
  # Parameters:  topLevelWin - window to pre-create for
  #
  # Description: creates all "standard" tiles needed for operation
  #
  #****************************************************************************
  proc Do_PreCreate { topLevelWin } \
  {
    #puts "Do_PreCreate:  $topLevelWin"

    variable  PreCreate
        
    Debug::Puts UI 2 "Pre-Creating tiles"
    foreach item $PreCreate \
    {
       if [Memory::Enough_for view] {Precreate_one $item $topLevelWin}
       update idle
    }
  }
  proc Precreate_one { item topLevelWin }\
  {
     puts "Create $item for $topLevelWin"
     set now [clock clicks]
     ${item}::Create $topLevelWin
     set then [clock clicks]
     puts "creating $item took [expr ($then-$now)/1000000.0] sec"
  }

  #****************************************************************************
  #
  # Name:        PackKeyBar
  #
  # Parameters:  topLevelWin - toplevel window for screen to be cleared
  #
  # Description: Packs the keybar display based on what needs to be shown
  #
  #****************************************************************************
  proc PackKeyBar { topLevelWin } \
  {
    #puts "PackKeyBar:  $topLevelWin"

    if { ![winfo exists $topLevelWin] } \
    {
      return
    }
    
    set topLevelWin  [winfo toplevel  $topLevelWin]

    # Get the Title Bar name
    set titleBar  [name_child  $topLevelWin  title_bar]
    set options   [name_child  $topLevelWin  options]
    
    if { ![winfo exists  $titleBar] } \
    {
      return
    }

    if { $::View::KeyBarMode($options) } \
    {
      Screens::position_widget  $titleBar.mode.mnu_options
    }
    
    Handle_clock  $titleBar
    
    return
  }


  #****************************************************************************
  #
  # Name:        GetMainView
  #
  # Parameters:  topLevelWin          toplevel of instruments main view
  #
  # Description: Returns the window of the main instrument's view
  #
  #****************************************************************************
  proc GetMainView { topLevelWin } \
  {
    #puts "GetMainView:  $topLevelWin"

    return  [name_child  $topLevelWin  mainView]
  }


  #****************************************************************************
  #
  # Name:        GetMainMenuFrameName
  #
  # Parameters:  topLevelWin          toplevel of instruments main view
  #
  # Description: Returns the name of the frame for the button for the 
  #              main screens menu
  #
  #****************************************************************************
  proc GetMainMenuFrameName { topLevelWin } \
  {
    #puts "GetMainMenuFrameName:  $topLevelWin"

    set top  [winfo toplevel $topLevelWin]

    set mainMenu  [name_child  $top  title_bar.mode]

    return  $mainMenu
  }


  #****************************************************************************
  #
  # Name:        GetMainMenuName
  #
  # Parameters:  topLevelWin          toplevel of instruments main view
  #
  # Description: Returns the name of the button for the main screens menu
  #
  #****************************************************************************
  proc GetMainMenuName { topLevelWin } \
  {
    #puts "GetMainMenuName:  $topLevelWin"

    set top  [winfo toplevel $topLevelWin]

    set mainMenu  [name_child  $top  title_bar.mode.mnu_mode]

    return  $mainMenu
  }


  #****************************************************************************
  #
  # Name:        GetOptionsMenuFrameName
  #
  # Parameters:  topLevelWin          toplevel of instruments main view
  #
  # Description: Returns the name of the frame for the options menu
  #
  #****************************************************************************
  proc GetOptionsMenuFrameName { topLevelWin } \
  {
    #puts "GetOptionsMenuFrameName:  $topLevelWin"

    set topLevelWin  [winfo toplevel $topLevelWin]

    set opWin  [name_child  $topLevelWin  title_bar.mode]

    return  $opWin
  }


  #****************************************************************************
  #
  # Name:        GetOptionMenuName
  #
  # Parameters:  topLevelWin          toplevel of instruments main view
  #
  # Description: Returns the name of the options menu
  #
  #****************************************************************************
  proc GetOptionMenuName { topLevelWin } \
  {
    #puts "GetOptionMenuName:  $topLevelWin"

    set topLevelWin  [winfo toplevel $topLevelWin]

    set opMenuButton  [name_child  $topLevelWin  title_bar.mode.mnu_options]

    return  $opMenuButton
  }


  #****************************************************************************
  #
  # Name:        PackOptions
  #
  # Parameters:  topLevelWin - toplevel window for screen to be cleared 
  #
  # Description: Packs an options button in the title bar
  #
  #****************************************************************************
  proc PackOptions { topLevelWin  { optionsButton  "" } } \
  {
    #puts "PackOptions:  $topLevelWin,  $optionsButton"

    set topLevelWinop  [winfo toplevel $topLevelWin]

    set opMenu   [GetOptionMenuName  $topLevelWin]
    
    #puts "opMenu:  $opMenu"
    
    if { ![winfo exists $opMenu] } \
    {
      return
    }
    set clear_children ""
    if {$topLevelWin == "."} \
    {
       set clear_children "[namespace code ClearChildren];"
    }
        
    if { $optionsButton == "" } \
    {
      pack forget  $opMenu

      # Unbind the Options menu
      bind $topLevelWinop  <<OptionDown>>  {}
      bind $topLevelWinop  <<Option>>      $clear_children
    } \
    else \
    {
      Screens::position_widget  $optionsButton

      # Bind the Options menu
      bind $topLevelWinop  <<OptionDown>>  "View::OptionDown  $opMenu"
                              
      bind $topLevelWinop  <<Option>>  "$clear_children after idle \"View::Option  $opMenu\""
    }
    return
  }


  #****************************************************************************
  #
  # Name:        RegisterScreen
  #
  # Parameters:  NameSpace	Namespace of screen to be added
  #
  # Description: This function adds the namespace to the current active menu list
  #
  #****************************************************************************
  proc RegisterScreen { NameSpace  { index {} } } \
  {
    #puts "RegisterScreen:  $NameSpace,  $index"

    variable MenuListPtr
    upvar dir dir
    
    upvar #0 $MenuListPtr($dir) list
    
    if { "$index" == "" } \
    {
      lappend list $NameSpace
    } \
    else \
    {
      set list [linsert $list $index $NameSpace]
    }
    
    return
  }

  #****************************************************************************
  #
  # Name:        BuildMainMenu
  #
  # Parameters:  None
  #
  # Description: This function will build the main menu
  #              1) The top level menu is in ::View::AvailScreens
  #              2) Submenus are stored in ::xxx_Screen::SubmenuScreens 
  #              menu to AvailScreens. It is called when there is a request 
  #              to add a submenu item, and a submenu candidate has been found.
  #              This routine can go recursive if the submenu has submenus.
  #
  # Note:        A sublist is of the form:
  #              {1 parent1 {0 child1} {0 child2} {1 parent2 {0 child3} {0 child4}}}
  #
  #****************************************************************************
  proc BuildMainMenu { widget } \
  {
    #puts "BuildMainMenu:  $widget"

    # Do basic sanity checks before we try to run
    if { ![winfo exists $widget] } \
    {
      return
    }

    # Generate the menu as a child of the toplevel
    set topWin  [winfo toplevel $widget]

    set buttonName  [GetMainMenuName  $topWin]
        
    #puts "buttonName:  $buttonName"
    
    set menuName  $buttonName.menu
    
    if { ![winfo exists $buttonName] } \
    {
      menubutton $buttonName -menu $menuName -padx 20
    }
    
    # link this to the MENU button
    bind $topWin <<ModeDown>> "event generate $buttonName <<Press>>; break"
    if { $topWin == "."} \
    {
      bind $topWin <<Mode>>     "[namespace code ClearChildren]; after idle \"event generate $buttonName <<Release>>\"; break"
    } \
    else \
    {
       bind $topWin <<Mode>>     "event generate $buttonName <<Release>>; break"
    }

    add_bindtag  $buttonName  Softkey

    if { ![winfo exists $menuName] } \
    {
      menu $menuName -tearoff 0 -postcommand ""
    }

    set trace_cmd  [list after idle ::View::ReBuildMenu $menuName ::View::AvailScreens $topWin]

    bind $menuName <Destroy> [list trace vdelete ::View::AvailScreens w $trace_cmd]
    

    # Add the LogKeyStrokes bindtag
    add_bindtag  $menuName  LogKeyStrokes
    

    trace variable ::View::AvailScreens w $trace_cmd
    
    ReBuildMenu  $menuName  ::View::AvailScreens  $topWin
  }
  
  proc BuildMainSubMenu { nameSpace  submenuName  topWin } \
  {
    #puts "BuildMainSubMenu:  $nameSpace,  $submenuName,  $topWin"

    # compose the name of the menu list
    append sub_name :: $nameSpace ::SubmenuScreens
    
    if { ![winfo exists $submenuName] } \
    {
      # set up to rebuild automatically when changed
      menu $submenuName -tearoff 0
    }
    
    set trace_cmd  [list after idle ::View::ReBuildMenu $submenuName $sub_name $topWin]

    bind $submenuName <Destroy> [list trace vdelete $sub_name w $trace_cmd]


    # Add the LogKeyStrokes bindtag
    add_bindtag  $submenuName  LogKeyStrokes


    trace variable $sub_name w $trace_cmd
    
    ReBuildMenu  $submenuName  $sub_name  $topWin
    
    return
  }
  
  # sort numbers first, then oddballs
  proc sortCommand { a b } \
  {
     set a0 No
     set b0 No
     catch { set a0 [expr int($a)]}
     catch { set b0 [expr int($b)]}
     if {$a0 == "No"} \
     {
         if {$b0 == "No"}\
         {
           return [string compare $a $b]
         }
         return 1
     }
     if {$b0 == "No"} { return -1}
     if {$a0 > $b0} { return 1}
     if {$a0 < $b0} { return -1}
     return 0
  }
  
  proc ReBuildMenu { menuName  listname  topWin  args } \
  {  
    #puts  "ReBuildMenu:  $menuName,  $listname"

    $menuName delete 0 end
    
    foreach { binding } [bind $menuName] \
    {
      if { "$binding" != "<Destroy>" } \
      {
        bind $menuName $binding {}
      }
    }
   
    upvar #0 $listname menuList 

    set accel  1
    set index  0
    
    # 3 passes - two to assign menu #s, the second to build the menu.
    array set items {}
    # First pass - let any menu item with a hard-wired accelerator claim it
    foreach { screenNameSpace } $menuList \
    {
      upvar #0 ::${screenNameSpace}\::Accel      my_accel
      if { [info exists my_accel] } \
      {
        # sanity check - look for dups
        if [info exists items($my_accel)] \
        {
            ErrLog::Log "$screenNameSpace wants menu accel $my_accel, but $items($my_accel) owns it already"
        }
        set items($my_accel) $screenNameSpace
      }
    }
    # second pass - assign anybody who didn't get assigned first pass
    foreach { screenNameSpace } $menuList \
    {
      upvar #0 ::${screenNameSpace}\::Accel my_accel
      if { [info exists my_accel] && ($items($my_accel) == $screenNameSpace)} \
      {
          # already assigned - skip it
          continue
      }
      while [info exists items($accel)] \
      {
         incr accel
      }
      # found an accel that works - hopefully
      set items($accel) $screenNameSpace
      incr accel
    }
    # OK, final pass - build the menu, sorted by accelerator
    foreach _accel [lsort -command sortCommand [array names items]] \
    {
       set screenNameSpace $items($_accel)
       upvar #0 ::${screenNameSpace}\::ShortName  submenuTitle
       upvar #0 ::${screenNameSpace}\::MenuFlags  menu_flags
       upvar #0 ::${screenNameSpace}\::Show_cmd   show_cmd
      
       if { ![info exists submenuTitle] } \
       {
         set submenuTitle  $screenNameSpace
       }
      
       if { [info exists show_cmd] } \
       {
         set submenuCmd  "$show_cmd $topWin $screenNameSpace"
       } \
       else \
       {
         set submenuCmd  "::Screens::Show $topWin $screenNameSpace"
       }

       if { ![info exists menu_flags] } \
       {
         set menu_flags  0
       }

       switch -- $menu_flags \
       {
         "0"        { $menuName add command -accel $_accel \
                                            -label $submenuTitle  \
                                            -com   "after idle $submenuCmd"        
                    }
                   
         "1"        -
         "submenu"  { set subMenu  [name_child $menuName submenu$index]

                      $menuName add cascade -accel $_accel    \
                                            -label $submenuTitle \
                                            -menu  $subMenu

                      ::View::BuildMainSubMenu  $screenNameSpace $subMenu $topWin
                    }
         
         "special"  { Debug::Puts UI 4 "Create special menu"
                      
                      set subMenu  [name_child $menuName submenu$index]
                      
                      $menuName add cascade -accel $_accel    \
                                            -label $submenuTitle \
                                            -menu  $subMenu

                      ::${screenNameSpace}::BuildMainSubMenu $subMenu $topWin
                    }
       }
      
       # if the accelerator is "+", bind to both + and keypad +
       if { "$_accel" == "+" } \
       {
         bind $menuName <KP_Add>  "do_menu_key  %W  $index"
       }
      
       bind $menuName $_accel    "do_menu_key  %W  $index"

       incr index
    }
    
    # Add ESCAPE functionality
    $menuName add command -accel  ESC              \
                          -label  "Escape"         \
                          -command  "tkMenuUnpost $menuName"
    
    return
  }


  #****************************************************************************
  #
  # Name:        OptionDown
  #
  # Parameters:  widget to use
  #
  # Description: handles Option down events
  #
  #*****************************************************************************
  proc OptionDown { widget } \
  {
    #puts "OptionDown:  $widget"

    if { [winfo exists $widget] && [winfo ismapped $widget] } \
    {
      event generate $widget <<Press>>
    }
    
    return
  }
  

  #****************************************************************************
  #
  # Name:        Option
  #
  # Parameters:  widget to use
  #
  # Description: handles Option events
  #
  #*****************************************************************************
  proc Option { widget } \
  {
    #puts "Option:  $widget"

    if { [winfo exists $widget] && [winfo ismapped $widget]  } \
    {
      event generate $widget <<Release>>
    }
    
    return
  }


  #****************************************************************************
  # Name:        getSaveState
  # Parameters:  none
  # Description: This function returns a list of TCL commands which can be used
  #              to set the View screen to its current state
  #****************************************************************************
  proc getSaveState { } \
  {
    set returnList \
    {
      
    }
   
    return [subst $returnList]
  }


  #****************************************************************************
  # Name:        Register_alert
  # Parameters:  routine to register
  # Description: Register a function to be called when a new view is created.
  #              This function will be called with the name of the toplevel of
  #              the view. It is assumed the function will do some sort of
  #              "hookup" for monitoring and displaying alerts.
  #****************************************************************************
  proc Register_alert { func } \
  {
    #puts "Register_alert:  $func"

    variable  Alert_routines
    variable ScreenList
  
    lappend Alert_routines $func
    
    foreach screen $ScreenList \
    {
       eval $func $screen
    }
  }
 
 
  #****************************************************************************
  # Name:        Register_cleanup
  # Parameters:  routine to register
  # Description: Register a function to be called when a view is destroyed.
  #              This function will be called with the name of the toplevel of
  #              the view. 
  #****************************************************************************
  proc Register_cleanup { func } \
  {
    #puts "Register_cleanup:  $func"

    variable  Clean_ups
    variable ScreenList
  
    lappend Clean_ups $func
  }
 
 
  #****************************************************************************
  # Name:        hook_powerdown
  # Parameters:  toplevel window
  # Description: Hook up to show a power off message when we are turned off
  #****************************************************************************
  proc hook_powerdown { top } \
  {
    at_exit "View::show_powerdown $top"
  }
 
 
  #****************************************************************************
  # Name:        show_powerdown
  # Parameters:  toplevel window
  # Description: Hook up to show a power off message when we are turned off
  #**************************************************************************** 
  proc show_powerdown { top } \
  {
    set msg  [name_child $top powerdown]
   
    # DISPLAY "POWERING DOWN" MESSAGE
    message_box $msg INFORM
  }
 
  Register_alert View::hook_powerdown
 
 
};  # End of the View namespace



#****************************************************************************
# This registers the screens with the list for saving instrument states
#****************************************************************************
namespace eval ::SavedStates \
{
  variable  screens   

  lappend screens View
}
