##############################################################################
# Aeroflex Inc     COPYRIGHT (C)1998-2005                 DESIGN ENGINEERING #
# -------------------------------------------------------------------------- #
# FILE        : slave_interp.tcl                                             #
# PRODUCT     : FITS                                                         #
# AUTHOR      : Hieu Tran                                                    #
# DESCRIPTION : set up safe slave interpreter for network connection         #
#                                                                            #
##############################################################################
package require LineEdit



namespace eval ::Slave {

# all of our slave interps will be represented by a TCL array, with at least
# the following members:
# NAME   - name of the interpreter
# FD     - file descriptor for I/O
# PROMPT - current prompt string

variable count 0
variable slave_list {}

variable local_lockout ""

# define the commands that get "tossed up" to the master interp
# this list can be added to as the system runs (to allow apps to be
# invoked from remote...)
# an alias command is assumed to accept the slave's name as the first
# parameter

  variable Aliases

  array set Aliases \
  {
   format_version        ::Slave::Version
   Master                ::Slave::Master
   puts                  ::Slave::Puts
   pwd                   ::Slave::Pwd
   cd                    ::Slave::Cd
   package               ::Slave::Package
   *IDN?                 ::Slave::Idn
   *idn?                 ::Slave::Idn
   DISPLAY               ::Slave::Display
   REDIRECT              ::Slave::Redirect
   echo                  ::Slave::Echo
   ::Screens::Show       ::Slave::Show
   Load::on_system_done  ::Slave::Eval
   Load::on_hwinit_done  ::Slave::Eval
   Load::on_UI_done      ::Slave::Eval
   Recall                ::Slave::Recall
   DEFAULT               ::Slave::Default
   *RST                  ::Slave::Default
   *rst                  ::Slave::Default
   init_done             ::Slave::Init_done
   BGSource              ::Slave::BGSource
   at_exit               ::Slave::At_exit
   LOCKOUT               ::Slave::Do_lockout
   open                  ::Slave::Open
   file                  ::Slave::File
   Memory::Cleanup       ::Slave::Cleanup
   Memory::Add_cleanup_proc ::Slave::Add_cleanup_proc
  }
  variable hot_start
  set hot_start \
  {
     # force load of all pkgIndex files in the background
     proc preload_pkg { dir  } \
     {
        upvar files files
        if {$dir == "." } { return }
        set f [file join $dir pkgIndex.tcl]
        if [file exists $f] {lappend files $f}
        set f [file join $dir tclIndex]
        if [file exists $f]  {lappend files $f}
        foreach d [glob -nocomplain [file join $dir *]] \
        {
           if [file isdirectory $d] \
           {
              preload_pkg $d
           }
        }
     }
     foreach d $auto_path \
     {
        preload_pkg $d 
     }
     if [info exists files] \
     {
        foreach f $files \
        {
           BGSource $f
        }
        unset files
     }
     BGCommand init_done
  }
  
  proc BGSource { slavename file } \
  {
     upvar 0 $slavename  slve
     upvar 0 slve(NAME) slave
     puts "Sourcing $file for $slave"
     
     # get the real path from the virtual one.
     if {[catch {set file [::safe::TranslatePath $slave $file]} msg]} \
     {
         ::safe::Log $slave $msg;
         return -code error "permission denied"
     }
        
     # check that the path is in the access path of that slave
     if {[catch {::safe::FileInAccessPath $slave $file} msg]} {
         ::safe::Log $slave $msg;
         return -code error "permission denied"
     }

     # passed all the tests , lets source it:
     if {[catch {$slave invoke BGSource_bin $file} msg]} \
     {
         Log $slave $msg;
         set ::slave_errorInfo "$msg: $::errorInfo"
         return -code error "script error";
     }
  }
  
   # add a routine to be called when we are low on memory
   proc Add_cleanup_proc { slavename cmd } \
   {
      upvar 0 $slavename  slave
      upvar 0 slave(cleanup_list) cleanup_list
      lappend cleanup_list $cmd
   }
   
   # called when we are low on memory
   proc Cleanup { slavename } \
   {
      upvar 0 $slavename  slave
      upvar 0 slave(cleanup_list) cleanup_list
      upvar 0 slave(NAME) interp
      foreach cmd $cleanup_list \
      {
         oldCatch {$interp eval $cmd }
      }
   }
#*************************************************
#       Name: File
# Parameters: per file command
#Description: Open a file
#*************************************************
  proc File { slavename args } \
  {
     upvar 0 $slavename  slave
     upvar 0 slave(NAME) interp
     set subcmd [lindex $args 0]
     set otherargs [lrange $args 1 end]
     if {[string compare mkdir $subcmd] == 0} \
     {
        # can only work with files within /usr
        foreach dir $otherargs \
        {
           regsub -all {/[^/]*/\.\.} $dir "" dir
           if ![string match /usr* $dir] \
           {
             return -code error "Permission denied"
           }
           file mkdir $dir
        }
        return
     }
     if {[string compare delete $subcmd] == 0} \
     {
        set force ""
        # can only work with files within /usr
        if {[string compare -force [lindex $otherargs 0]] == 0} \
        {
           set otherargs [lrange $otherargs 1 end]
        }
        foreach file $otherargs \
        {
           regsub -all {/[^/]*/\.\.} $file "" file
           if ![string match /usr* $file] \
           {
             return -code error "Permission denied"
           }
           eval file delete $force $file
        }
        return
     }
 
     if {[string compare copy $subcmd] == 0} \
     {
        set force ""
        # can only work with files within /usr
        if {[string compare -force [lindex $otherargs 0]] == 0} \
        {
           set otherargs [lrange $otherargs 1 end]
        }
        foreach file $otherargs \
        {
           regsub -all {/[^/]*/\.\.} $file "" file
           if ![string match /usr* $file] \
           {
             return -code error "Permission denied"
           }
        }
        eval file copy $force $otherargs
        return
     }
  
     set allowed "^(is.*|exists|dir.*|join|root.*|ext.*|tail|path.*|split)"
     if {[regexp $allowed $subcmd]} \
     {
         return [eval {file $subcmd} $otherargs]
     }
     return -code error "Permission denied"
  }
  
#*************************************************
#       Name: Open
# Parameters: file name, mode
#Description: Open a file
#*************************************************
  proc Open { slavename file {mode "r"} {perms "0666"}} \
  {
      upvar 0 $slavename  slave
      upvar 0 slave(NAME) interp
      set file [file join $slave(PWD) $file]
      regsub -all {/[^/]*/\.\.} $file "" file
      if ![string match /usr* $file] \
      {
         return -code error "Permission denied"
      }
      ::safe::FileInAccessPath $interp $file
      set fd [open $file $mode $perms]
      interp transfer "" $fd $interp
      return $fd
  }
 
  
#*************************************************
#       Name: At_exit
# Parameters: command
#Description: Register command to call on slave exit
#*************************************************
  proc At_exit { slavename cmd} \
  {
      upvar 0 $slavename  slave
      lappend slave(AT_EXIT) $cmd
  }
  
#*************************************************
#       Name: Init_done
# Parameters: slave name invoking the command
#Description: finish hot-starting a slave
#*************************************************
  proc Init_done { slavename } \
  {
     
      upvar 0 $slavename  slave
      upvar 0 slave(NAME) name
      puts "Slave $slavename init done"
      $name hide BGCommand
      $name hide BGSource
      $name hide preload_pkg
      $name eval "package require ALL;ALL"
      $name hide glob
      interp alias $name glob "" ::Slave::Glob $slavename
      
      NotBusy
  }
#*************************************************
#       Name: Glob
# Parameters: slave name and glob parms
#Description: do "glob" safely
#*************************************************
  proc Glob { slavename args } \
  {
      upvar 0 $slavename  slave
      upvar 0 slave(NAME) name
      set nocomp ""
      set done 0
      set patterns ""
      foreach arg $args \
      {
         if {!$done} \
         {
            if {[string compare $arg -nocomplain] == 0} \
            {
              set nocomp $arg
              continue
            }
            if {$arg == "--"} { set done 1; continue}
         }
         if {[file dirname $arg] != "."} \
         {
            safe::InAccessPath $name $arg
         }
         lappend patterns $arg
      }
      set old_pwd [pwd]
      cd $slave(PWD)
      set x [eval glob $nocomp -- $patterns]
      cd $old_pwd
      return $x
  }
  
#*************************************************
#       Name: Default
# Parameters: slave name invoking the command
#Description: Force unit to factory default.
#*************************************************
  proc Default { slavename } \
  {
     return [restore_factory_defaults]
  }
  
#*************************************************
#       Name: Add_alias
# Parameters: slave command - command in slave
#             master command - command in master
#Description: Define a new command alias for all slaves
#*************************************************
  proc Add_alias { slave_cmd master_cmd } \
  {
     # first, add to the alias table for all new slaves
     variable Aliases
     set Aliases($slave_cmd) $master_cmd
     
     # now, apply that to all running slaves
     variable slave_list
     foreach slavename $slave_list \
     {
        upvar 0 $slavename slave
        interp alias $slave(INTERP) $slave_cmd {} $master_cmd $slavename
     }
  }
  
#*************************************************
#       Name: Eval
# Parameters: slave - name of slave variable
#Description: Returns the unit version info 
#*************************************************
  proc Eval { slavename args } \
  {
     upvar 0 $slavename    slave
     upvar 0 slave(NAME)   name
     return [eval $name eval $args]
  }
  
#*************************************************
#       Name: Version
# Parameters: slave - name of slave variable
#Description: Returns the unit version info 
#*************************************************
  proc Version { slavename args } \
  {
     return [eval format_version $args]
  }
  
#*************************************************
#       Name: Show
# Parameters: slave - name of slave variable
#             
#Description: Returns the unit version info 
#*************************************************
  proc Show { slavename args } \
  {
     return [eval ::Screens::Show $args]
  }
  
#*************************************************
#       Name: Puts 
# Parameters: slave - name of slave variable
#             arg1, arg2, arg3 - puts arguments
#Description: writes a string to socket channel 
#*************************************************
  proc Puts { slavename arg1 {arg2 ""} { arg3 ""} } \
  {
      upvar 0 $slavename slave
      upvar 0 slave(FD) file
      upvar 0 slave(NAME) interp
      if {"$arg3" != "" } \
      {
         if {"$arg2" == "stdout" } \
         {
            set arg2 $filep
         }
         return [$interp invokehidden puts $arg1 $arg2 "$arg3"]
      } 
      if { "$arg2" != "" } \
      {
         if {"$arg1" == "stdout" } \
         {
           return [$interp invokehidden puts $file "$arg2"]
         }
         if {"$arg1" == "-nonewline" } \
         {
           return [$interp invokehidden puts $arg1 $file "$arg2"]
         }
         return [$interp invokehidden puts $arg1 "$arg2"]
      }
      return [$interp invokehidden puts $file "$arg1"]
   }

#*************************************************
#       Name: Prompt
# Parameters: slave - name of slave variable
#Description: show the prompt for the slave
#*************************************************
   proc Prompt { slavename string } {
      upvar 0 $slavename  slave
      upvar 0 slave(FD)   fd
      upvar 0 slave(ECHO) echo
      upvar 0 slave(EDIT) edit
      catch {
       if {$echo} { 
         puts -nonewline $fd $string
         flush $fd
       }
      }
   }
#*************************************************
#       Name: Input
# Parameters: slave - name of slave variable
#             string  - line to interpret
#Description: Accumulates a command line for the slave
#*************************************************
   proc Input { slavename {string {}} } {
      upvar 0 $slavename  slave
      upvar 0 slave(FD)   file
      upvar 0 slave(LINE) line
      upvar 0 slave(NAME) interp
      
      if [eof $file] {
         $interp eval exit
         return
      }
      
      append line $string "\n"
      if {"$line"=="\n"} {
         Prompt $slavename $slave(PS1)
         set line ""
         return
      }
      if ![info complete $line] {
         Prompt $slavename $slave(PS2)
         return
      }
      set line [string trimright $line]
      Do_cmd $slavename
      if ![interp exists $interp] { return }
      
      Prompt $slavename $slave(PS1)
   }
#*************************************************
#       Name: Pwd
# Parameters: slave to access
#Description: get current dir for slave
#*************************************************
   proc Pwd { slavename } \
   {
      upvar 0 $slavename  slave
      return $slave(PWD)
   }
#*************************************************
#       Name: Cd
# Parameters: slave to access, directory
#Description: CD for a slave
#*************************************************
   proc Cd { slavename dir } \
   {
      upvar 0 $slavename  slave
      upvar 0 slave(NAME) interp
      set dir [::safe::TranslatePath $interp $dir]
      if ![string match /usr* $dir] \
      {
         return -code error "Permission denied"
      }
      if [catch {set x [::safe::interpFindInAccessPath $interp $dir]} err] \
      {
         return -code error "Bad path $dir"
      }
      if ![file isdir $dir] { return -code error "Bad path $dir"}
      set slave(PWD) $dir
   }
   
#*************************************************
#       Name: Package
# Parameters: slave to access, args to slave command
#Description: do the package command for a slave
#*************************************************
   proc Package { slavename args } {
     upvar 0 $slavename  slave
     upvar 0 slave(NAME) interp
     # see if we are requiring a package
     if {"[lindex $args 0]"=="require" } {
      set name [lindex $args 1]
      if {"$name"=="-exact"} {
         set name [lindex $args 2]
      }
      if {"$name"=="Tk"} {
        # OK, we've served our purpose - we've caught the load
        # of Tk. Unhook ourselves, load Tk, and exit
        interp eval $interp "lappend auto_path $::tk_library /system/tk/extensions"
        ::safe::interpAddToAccessPath $interp $::tk_library
        ::safe::interpAddToAccessPath $interp /system/tk/extensions
        $interp eval "set tk_library $::tk_library"
        interp alias  $interp package {}
        interp expose $interp package_slave package
        append slave(WIN) . $interp
        # OK, can we create a toplevel there?
        catch { toplevel $slave(WIN) -container 1 -screen $slave(SCREEN) -class IFR-2975} err
        if ![winfo exists $slave(WIN)] {
           toplevel $slave(WIN) -class IFR-2975 -container 1
        }
        set slave(SCREEN) [winfo screen $slave(WIN)]
        wm title      $slave(WIN) "IFR-2975 serial #[IFR_options ser]"
        wm iconbitmap $slave(WIN) @/system/logo/logo.xbm
           
        ::safe::loadTk $interp -use $slave(WIN)
        wm protocol $slave(WIN) WM_DELETE_WINDOW "$interp eval exit; destroy $slave(WIN)"
        interp expose $interp wm
        interp expose $interp toplevel
        interp expose $interp menu
        interp alias  $interp tk::hide_dot   {} wm withdraw $slave(WIN)
        interp alias  $interp tk::unhide_dot {} wm deiconify $slave(WIN)
        $interp eval {
           bindtags . {. Tk all MainTopLevel}
           bind MainTopLevel <Map> {tk::unhide_dot}
           bind MainTopLevel <Unmap> {tk::hide_dot}
        }
        
        # inform the focus watcher that it's possible he may lose focus,
        # and that's OK
        disable_verify_focus $slavename
        return
      }
     }
     # OK, either it's not a require or it's some other package
     eval $interp invokehidden package_slave $args
   }

#*************************************************
#       Name: Do_cmd
# Parameters: interp - interpreter name
#             fd - socket channel
#             line - slave command
#Description: run slave command  
#*************************************************
  proc Do_cmd { slavename } {
   upvar 0 $slavename    slave
   upvar 0 slave(FD)     fd
   upvar 0 slave(LINE)   line_in
   upvar 0 slave(NAME)   interp
   upvar 0 slave(RESULT) result
   upvar 0 slave(ERR)    err
   
   # grab the line, and clear it. the command we are about
   # to execute might do an update_idle, and we might get
   # another input line before we are done.
   set line $line_in
   set line_in {}
   
   set old_pwd [pwd]
   
   if {"$line"==""} {
      return
   }
   if [catch {
      if {"$interp"==""} {
         set result [namespace eval :: $line]
      } else {
         cd $slave(PWD)
         set result [$interp eval $line]
      }
      if {$slave(TRIM) && ($result != "")} \
      {
         set result [string trimright $result]
         if {$result == ""} { puts "Trimmed a result to nothing!"}
      }
      if {[interp exists $interp] && ($result != "")}\
      {
         catch {puts $fd $result }
      }
   } err ] {
      set slave(ERRINFO) $::errorInfo
      if ![interp exists $interp] {return }
      catch {puts $fd "$err"}
   }
   cd $old_pwd
  }
#*************************************************
#       Name: Master
# Parameters: slavename - slave name
#Description: Start passing commands on through to
#             master
#*************************************************
  proc Master { slavename {redirect 0}} {
     upvar 0 $slavename slave
     switch $redirect \
     {
       0 \
       {
         set slave(NAME) {}
         return "Now communicating with master... Be careful"
       }
       1 \
       {
         # we've been asked to really redirect the master...
         $slave(EDIT) redirect
         # kill the slave interp - we don't need him...
         interp delete $slave(INTERP)
         return "All system I/O redirected."
       }
       42 \
       {
         close stdin
         $slave(EDIT) redirect
         package require Shell
         Shell [$slave(EDIT) pty]
         # kill the slave interp - we don't need him...
         interp delete $slave(INTERP)
         return "Shell running"
       }
     }
     
     return
  }

#*************************************************
#       Name: Return
# Parameters: none
#Description: Return control to the slave's interp
# Assumption: this is being called from within
#             ::Slave::Do_cmd
#*************************************************
  proc Return { } {
     upvar 2 slave slave
     if ![info exists slave(NAME)] {
        return "You don't seem to be a slave..."
     }
     set slave(NAME) $slave(INTERP)
  }
#*************************************************
#       Name: DeleteEntry
# Parameters: slavename - slave name
#Description: delete servicing hook on slave 
#*************************************************
  proc DeleteEntry { interp } {
     append slavename ::Slave:: $interp
     upvar 0 $slavename  slave
     $slave(DELETE) $slavename

     variable slave_list
     set index [lsearch $slave_list $slavename]
     if {$index != -1 } \
     {
        set slave_list [lreplace $slave_list $index $index]
     }
  }
#*************************************************
#       Name: Delete
# Parameters: slavename
#Description: delete servicing hook on slave 
#*************************************************
  proc Delete { slavename } {
     upvar 0 $slavename  slave
     if [info exists slave(WIN)] {
        destroy $slave(WIN)
        enable_verify_focus $slavename
     }
     if [info exists slave(AT_EXIT)] \
     {
        upvar 0 slave(NAME) interp
        foreach cmd $slave(AT_EXIT) \
        {
           catch {$interp eval $cmd}
        }
     }
     after 1000 "unset $slavename"
  }
#*************************************************
#       Name: Event
# Parameters: slavename - slave name
#             event - event string
#             other args as per event
#Description: Handle events from slave 
#*************************************************
   proc Event { slavename event args } {
      upvar 0 $slavename  slave
      upvar 0 slave(NAME) interp
      if ![interp exists $interp] { return }
      switch -- $event {
        "EOF"       { $interp eval exit  }
        "DO_ECHO"   { set slave(ECHO) ON }
        "DONT_ECHO" { set slave(ECHO) OFF}
        default     { lappend slave(EVENTS) "$event $args" }
      }
   }
   
#*************************************************
#       Name: Connect 
# Parameters: slave name
#Description: connect slave input 
#*************************************************
  proc Connect { slavename {type RS232} {term Ansi} } {
    upvar 0 $slavename  slave
    upvar 0 slave(FD) fd
    upvar 0 slave(NAME) name
    append edit $name _editor
    catch {fconfigure $fd -buffering none}
    
    set slave(EDIT) [LineEdit $edit -fd $fd \
                     -cmd "::Slave::Input $slavename" \
                     -type $type -term $term]
    $slave(EDIT) event [list ::Slave::Event $slavename]
  }
#*************************************************
#       Name: Display 
# Parameters: slave name
#Description: Set the display for this slave
#*************************************************
  proc Display { slavename {screen {}} } {
    upvar 0 $slavename  slave
    if {"$screen" != ""} {
       set slave(SCREEN) screen
    }
    if ![info exists slave(SCREEN)] { 
      return -err "No display defined"
    }
    
    return $slave(SCREEN)
  }
#*************************************************
#       Name: Redirect 
# Parameters: slave name
#Description: Redirect main display to the slave's
#*************************************************
  proc Redirect { slavename { display  {} } } \
  {
        
    upvar 0 $slavename  slave
    
    if { $display == "" }\
    {
      set display  $slave(SCREEN)
    }

    if { $display == "" }\
    {
      return -err "No DISPLAY defined"
    }
    
    #puts "  display:  $display"
    
    View::CreateView "" $display
  }
  
#*************************************************
#       Name: Echo 
# Parameters: slave name
#Description: Set echo mode for slave
#*************************************************
  proc Echo { slavename {mode ""}} {
    upvar 0 $slavename  slave
    switch -- $mode {
      "ON" -
      "on" -
      "1"     {set slave(ECHO) ON}
      default {set slave(ECHO) OFF}
    }
    if [info exists slave(EDIT)] {
      catch {
       $slave(EDIT) echo $slave(ECHO)
      }
    }
    return $slave(ECHO)
  }

#*************************************************
#       Name: Recall
# Parameters: Name of file to recall (relative /usr/setups)
#Description: Recall the specified setup
#*************************************************
  proc Recall { slavename fname } \
  {
     set f [file join /usr/setups $fname]
     if ![file exists $f] \
     {
        # maybe just setup number?
        set f [file join /usr/setups SAVE${fname}.SAV]
        if ![file exists $f] \
        {
           return -code error "Specified setup does not exist"
        }
     }
     if [catch \
         {
            set result [source $f]
         } err
        ] \
     {
        return -code error "Error reading setup: $err"
     }
     return $result;
  }
#*************************************************
#       Name: New
# Parameters: fd - channel
#Description: create a new safe slave interpreter,
#             set up alias, and hide exec
#*************************************************
  proc New { fd {fullname ""} } {
   variable count
   variable slave_list

   variable hot_start
      
   # Note we are busy right now
   Busy
   
   append name slave [incr count]
   append slave_name ::Slave:: $name
   upvar 0 $slave_name slave
   set access_path "$::tcl_library /system/interface /system/Tone_signaling /usr ."
   set slave(INTERP)  [::safe::interpCreate $name -accessPath $access_path]
   $slave(INTERP) eval "set auto_path \"$access_path\""
   
   if {$fullname != ""} \
   {
      set slave(FULLNAME) $fullname
   } \
   else \
   {
      set slave(FULLNAME) $name
   }
   
   array set slave [subst \
   {
     NAME    "$slave(INTERP)"
     FD      "$fd"
     PS1     "% "
     PS2     "> "
     LINE    ""
     ERR     ""
     ERRINFO ""
     IDN     "[IFR_options man],[IFR_options model],[IFR_options ser],[firmware_version]"
     RESULT  ""
     ECHO    0
     DELETE  "::Slave::Delete"
     init 0
     TRIM 0
     PWD /usr
     cleanup_list {}
   }]
   $name  hide  puts
   interp share "" $fd $name
   interp alias $name debug_puts  {} Debug::Puts "" 0
   interp alias $name Debug::Puts {} Debug::Puts

   ::safe::interpConfigure $slave(NAME) -deleteHook "::Slave::DeleteEntry" -accessPath $access_path
   $name  hide  package package_slave
   $name  hide  exec

   # hot-start the package info - we need glob for this
   $name invoke load /system/interface/Callback/Callback.out
   $name invoke load /system/interface/Utils/Utils.out
   $name hide BGSource BGSource_bin
   $name expose glob
   
   variable Aliases
   foreach cmd [array names Aliases] \
   {
       interp alias $name $cmd {} $Aliases($cmd) $slave_name
   }
   
   # hot-start the package info - we need glob for this
   $name eval $hot_start

   lappend slave_list $slave_name
   return $slave_name
  }
  
#*************************************************
#       Name: Idn
# Parameters: slave
#Description: Handle *IDN?
#*************************************************
   proc Idn { slavename } {
     upvar 0 $slavename  slave
     upvar 0 slave(IDN) idn
     return $idn
   }
   
#*************************************************
#       Name: Show_lockout
# Parameters: screen
#Description: show or remove the local lockout on
#             all screens
#*************************************************
  proc Show_lockout { top args } \
  {
     set tile_path [name_child $top lockout]
     variable local_lockout
     set mode [::View::GetMainMenuFrameName $top]
     set menu [::View::GetMainMenuName $top]
     if {$local_lockout != ""} \
     {
        if ![winfo exists $tile_path] \
        {
           ::Screens::create_tile $tile_path "" LOCKOUT
        }
        pack forget $menu
        pack $tile_path -side left -in $mode -fill both -expand true
        $tile_path configure -takefocus 1
        safe_focus_force $tile_path
        tkwait visibility $tile_path
        grab $tile_path
     } \
     else \
     {
        grab release $tile_path
        destroy $tile_path
        Screens::position_widget $menu
     }
  }
  
#*************************************************
#       Name: Setup_lockout
# Parameters: screen
#Description: hook lockout handler
#*************************************************
  proc Setup_lockout { top } \
  {
     trace variable ::Slave::local_lockout w "::Slave::Show_lockout $top"
     Show_lockout $top
  }

#*************************************************
#       Name: Cleanup_lockout
# Parameters: screen
#Description: Unhook lockout handler
#*************************************************
  proc Cleanup_lockout { top } \
  {
     trace vdelete ::Slave::local_lockout w "::Slave::Show_lockout $top"
  }

#*************************************************
#       Name: Do_lockout
# Parameters: slave
#Description: Unhook lockout handler
#*************************************************
  proc Do_lockout { slavename mode} \
  {
     upvar 0 $slavename  slave
     variable local_lockout
     if {$mode} \
     {
        if {$local_lockout != ""} \
        {
           return "Lockout already activated by $local_lockout"
        }
        set local_lockout $slave(FULLNAME)
        lappend slave(AT_EXIT) "LOCKOUT 0"
        return 1
     } \
     else \
     {
        if {$local_lockout != $slave(FULLNAME)} \
        {
           return "Lockout activated by $local_lockout, not you"
        }
        set local_lockout ""
        return 0
     }
  }

  View::Register_alert   ::Slave::Setup_lockout
  View::Register_cleanup ::Slave::Cleanup_lockout
  bind LOCKOUT <<Power>> "break"
  bind LOCKOUT <<Help>>  "break"
  bind LOCKOUT <KeyPress> "break"
  bind LOCKOUT <KeyRelease> "break"
  bind LOCKOUT <Button> break
  bind LOCKOUT <ButtonRelease> break

} ;# end of Slave namespace

namespace eval ::Telnet \
{

#*************************************************
#       Name: Init_done
# Parameters: slave name invoking the command
#Description: finish hot-starting a slave
#*************************************************
  proc Init_done { slavename } \
  {
      ::Slave::Init_done $slavename
      upvar 0 $slavename  slave
      upvar 0 slave(NAME) interp
      
      ::Slave::Connect $slavename TELNET
      ::Slave::Echo $slavename ON
      ::Slave::Prompt $slavename "$slave(IDN) ready\n% "
  }
#*************************************************
#       Name: New
# Parameters: fd - socket channel
#             addr - IP address
#             port - port connection
#Description: create a new safe slave interpreter,
#             set up alias, and hide exec
#*************************************************
   proc New { fd addr port } \
   {
      if {$::Slave::local_lockout != "" } \
      {
         puts $fd "Sorry, lockout active on $::Slave::local_lockout"
         close $fd
         return
      }
      if ![Memory::Enough_for slave] \
      {
         puts $fd "Sorry, not enough memory to support another remote session"
         close $fd
         return
      }
      set name [::Slave::New $fd "Telnet from $addr:$port" ]
      upvar 0 $name slave
      upvar 0 slave(NAME) interp
      
      interp alias $interp init_done {} ::Telnet::Init_done $name

      set slave(ADDR) $addr
      set slave(PORT) $port
      append slave(SCREEN) $addr :0.0
      set slave(DELETE) ::Telnet::Delete
   }
#*************************************************
#       Name: Delete
# Parameters: slavename - slave name
#Description: delete servicing hook on slave 
#*************************************************
  proc Delete { slavename } {
     # get our file descriptor
     upvar 0 $slavename  slave
     set fd $slave(FD)
     
     ::Slave::Delete $slavename
     catch {close $fd}
  }
  
#*************************************************
#       Name: telnet_server 
# Parameters: port - port number 
#Description: set port # and rename puts 
#*************************************************
   proc Server { port } {
      socket -server ::Telnet::New $port
   }
   
   proc Init {} {
      if [catch {Server 23} err] \
      {
         puts "Failed to get Telnet, going for 71"
         catch {Server 71}
      }
   }
}

# define the final slave init routine...
namespace eval ::Slave {
  proc Init {} {
     package require GPIB
     package require Serial
     ::Telnet::Init
     ::GPIB::Init
     ::Serial::Init
  }
  Init
}

