#############################################################################
# IFR SYSTEMS, INC        COPYRIGHT (C)1999,00      DESIGN ENGINEERING      #
# --------------------------------------------------------------------------#
# FILE        : to_genca.tcl                                                #
# PRODUCT     : FITS                                                        #
# AUTHOR      :                                                 #
# DESCRIPTION : Interface to RF Gen Level via Message Bus (gen cals)        #
#                                                                           #
#############################################################################
package require Tclmsgbus

set RFGEN_ENGINE(SUB_CALCONFIG)       32
set RFGEN_ENGINE(SUB_DLOAD_LVEL_CAL)  28
set RFGEN_ENGINE(SUB_DLOAD_FREQ_CAL)  29

set Send_Flags  0x1

#############################################################################
#                                                                           #
#                       Engine Interfaces                                   #
#                                                                           #
#############################################################################


#############################################################################
#                                                                           #
#                       Calibration Interfaces                              #
#                                                                           #
#############################################################################


#
#  gencal_set_calconfig - resize engine calibration tables
#
#       type - 1 ALC Correction Curve
#              2 T/R Freq Correction Curve
#              3 GEN Freq Correction Curve
#
#   Special Note:
#   after calling this command, you will need to
#   re-send gencal_level_cal's and/or gencal_freq_cal's
#   to fill in cal tables.
#  
proc gencal_set_calconfig { type size } {
  global RFGEN_ENGINE
  global Send_Flags
  
   set msg " { $RFGEN_ENGINE(SYSTEM_ID) \
               $RFGEN_ENGINE(SUB_CALCONFIG) 1 $Send_Flags } \
             { inline dw $type $size } "

#puts $msg

   tclMsgbus send $msg
}


#
#  RF Gen ALC DAC level calibration
#
#  args:   index        0 to ... (255 max)
#          alclvl       ALC DAC LEVEL (0 to 4095)
#          lvlout       dB Level ( front panel level dBm)
#
#  special note: spline curve is updated on index 0 only.
#
proc gencal_level_cal { index alclvl lvlout {temp_comp 0} {do_temp_comp 0} } {
  global RFGEN_ENGINE
  global Send_Flags
  
  set temp_comp_flag -1.0
  if {$do_temp_comp != 0} { set temp_comp_flag 1.0}
    
   set msg " { $RFGEN_ENGINE(SYSTEM_ID) \
               $RFGEN_ENGINE(SUB_DLOAD_LVEL_CAL) 1 $Send_Flags } \
             { inline double $index $alclvl $lvlout $temp_comp $temp_comp_flag } "


   tclMsgbus send $msg
  
}


#
#  RF Gen frequency calibration
#
#  args:   index         composite index (0 to 40) & port (TR, Gen)
#          port          0 is TR, 256 is GEN
#          freq          in MHz
#          level0        0  dB atten  RF Level
#          level10       10 dB atten  RF Level
#          level20       20 dB atten  RF Level
#
#  special note: spline curve is updated on index 0 only.
#
proc gencal_freq_cal { index port freq level0 level10 level20 } {
  global RFGEN_ENGINE
  global Send_Flags

  switch $port {
    TR  { set x 0 }
    GEN { set x 256 }
    default { return }
  }

  # only 0 to 255 allowed
  set temp [ expr $index & 0xff]

  # combine index and port information
  set both [ expr $temp + $x ]
  
  set msg " { $RFGEN_ENGINE(SYSTEM_ID) \
              $RFGEN_ENGINE(SUB_DLOAD_FREQ_CAL) 1 $Send_Flags } \
            { inline double $both $freq $level0 $level10 $level20 } "

  tclMsgbus send $msg

}


#
#  Save GEN ALC Calibration Table
#
proc gencal_alc_save_cal { filename } {
   global GENCAL
   global RFGEN_ENGINE
 
   #
   #  Do the following operations 1st...
   #
   gencal_alc_normalize_dbm_2_db


  # VERIFY DIR EXISTS
  if {![file exists /system/calibration] || ![file isdirectory /system/calibration]} {
    if [catch {file mkdir /system/calibration}] {
put "Dir not exists. Could not save RF Generator ALC Calibrations."
      return
    }
  }

  if [catch { OpenCalFile $filename w } fileId] {
put "File Open failed. Could not save RF Generator ALC Calibrations."
    return
  }

   puts $fileId "#"
   puts $fileId "#  RF GEN ALC Level Calibration Table"
   puts $fileId "#===================================="

   puts $fileId "RFGEN_LEVEL_CHARACTERISTICS \
                                   MAX     $RFGEN_ENGINE(AlcMaxIndex) \
                                   REFLVL  $RFGEN_ENGINE(Normalize.Genalc) \
                                   FREQ    $GENCAL(AlcFreq) \
                                   DATE  $GENCAL(level.caldate) \
                                   TIME  $GENCAL(level.caltime) \
                                   TEMP  $::HW_control::State(if_temp) \
                                   HWVER [Generator ver 2]"

   for { set i 0 } { $i < $RFGEN_ENGINE(AlcMaxIndex) } { incr i } {
     puts $fileId [format "RFGEN_LEVEL_CAL INDEX %4d \
                           ALCLEVEL %5d \
                           LEVELOUT %7.3f \
                           PADETECT %5d \
                           STATUS   $GENCAL(level,status,$i)" \
                           $i \
                           $GENCAL(level,alc,$i) \
                           $GENCAL(level,normalize,$i) \
                           $GENCAL(level,padetect,$i)
                 ]
   }

   close $fileId
}


#
#  Read GEN ALC Calibration Table
#
proc gencal_alc_read_cal { filename } {
   global GENCAL 
   global RFGEN_ENGINE
 
  # OPEN CAL FILE
  if [catch { OpenCalFile $filename r } fileId] {
puts "CAUTION: RF Generator ALC Cals are not present"
    return
  }

   while { [ gets $fileId line ] >= 0} {

     if { [lindex $line 0] == "RFGEN_LEVEL_CHARACTERISTICS" } {
        set RFGEN_ENGINE(AlcMaxIndex)      [lindex $line 2]
        set RFGEN_ENGINE(Normalize.Genalc) [lindex $line 4]
        set GENCAL(AlcFreq)            [lindex $line 6]
        set GENCAL(level.caldate)      [lindex $line 8]
        set GENCAL(level.caltime)      [lindex $line 10]
        set GENCAL(level.caltemp)      [lindex $line 12]
        set GENCAL(hwversion)          [lindex $line 14]
        if {"$GENCAL(level.caltemp)" == "" } \
        {
           set GENCAL(level.caltemp) 60
        }
        if {"$GENCAL(hwversion)" == "" } \
        {
           set GENCAL(hwversion) 0
        }
        continue
     }

     if { [lindex $line 0] == "RFGEN_LEVEL_CAL" } {
        set index                          [lindex $line 2 ]
        set GENCAL(level,alc,$index)       [expr int([lindex $line 4]) ]
        set GENCAL(level,normalize,$index) [lindex $line 6 ]
        set GENCAL(level,padetect,$index)  [lindex $line 8 ]
        set GENCAL(level,status,$index)    [lindex $line 10]

        gencal_alc_abs_db_2_dbm $index
     }
   }

  close $fileId
}



#
#  upload level cals RF Gen engine
#
proc gencal_set_engine_level { temp_comp {temp_comp_flag ""} } {
   global RFGEN_ENGINE
   global GENCAL

   # update ALC curve in ENGINE
   gencal_set_calconfig 1 $RFGEN_ENGINE(AlcMaxIndex)
   
   if {$temp_comp_flag == ""} \
   {
      # not told whether to do temp comp or not - base on hardware version
      if {[Generator ver 2] >= 2} \
      {
         set temp_comp_flag 0
      } \
      else \
      {
         set temp_comp_flag 1
      }
   }

   set i $RFGEN_ENGINE(AlcMaxIndex)
   incr i -1
   
   # re-spline occures when sending index 0
   for {  } { $i >= 0 } { incr i -1 } {
       gencal_level_cal $i $GENCAL(level,alc,$i) $GENCAL(level,normalize,$i) $temp_comp $temp_comp_flag
   }
}


#
#  Save GEN Freq Calibration Table
#
proc gencal_freq_save_cal { filename rfport } {
   global GENCAL
   global RFGEN_ENGINE
 
   #
   #  Do the following operations 1st...
   #
   gencal_freq_normalize_dbm_2_db $rfport

   switch $rfport {
     GEN    {
             set token_chr RFGEN_GEN_FREQ_CHARACTERISTICS
             set token_ele RFGEN_GEN_FREQ_CAL
            }
     TR     {
             set token_chr RFGEN_TR_FREQ_CHARACTERISTICS
             set token_ele RFGEN_TR_FREQ_CAL
            }
     default { return }
   }


  # VERIFY DIR EXISTS
  if {![file exists /system/calibration] || ![file isdirectory /system/calibration]} {
    if [catch {file mkdir /system/calibration}] {
put "Dir does not exist. Could not save RF Generator Freq $rfport Calibrations."
      return
    }
  }

  if [catch { OpenCalFile $filename w } fileId] {
puts "File Open failed. Could not save RF Generator Freq $rfport Calibrations."
    return
  }

   puts $fileId "#"
   puts $fileId "#  RF GEN Freq Level Calibration Table ($rfport)"
   puts $fileId "#============================================="

   puts $fileId "$token_chr \
                           MAX       $RFGEN_ENGINE(FreqMaxIndex) \
                           REFLVL0   $RFGEN_ENGINE(Normalize.Genfreq.dB0) \
                           REFLVL10  $RFGEN_ENGINE(Normalize.Genfreq.dB10) \
                           REFLVL20  $RFGEN_ENGINE(Normalize.Genfreq.dB20) \
                           DATE    $GENCAL($rfport.caldate) \
                           TIME      $GENCAL($rfport.caltime) "

   for { set i 0 } { $i < $RFGEN_ENGINE(FreqMaxIndex) } { incr i } {
     puts $fileId [format "$token_ele INDEX %4d \
                           FREQ    %5f \
                           LEVEL0  %7.2f \
                           LEVEL10 %7.2f \
                           LEVEL20 %7.2f \
                           STATUS0  $GENCAL(freq,$rfport,dB0,status,$i) \
                           STATUS10 $GENCAL(freq,$rfport,dB10,status,$i) \
                           STATUS20 $GENCAL(freq,$rfport,dB20,status,$i)" \
                           $i \
                           $GENCAL(freq,$rfport,freq,$i) \
                           $GENCAL(freq,$rfport,dB0,normalize,$i) \
                           $GENCAL(freq,$rfport,dB10,normalize,$i) \
                           $GENCAL(freq,$rfport,dB20,normalize,$i) \

                 ]
   }

   close $fileId
}


#
#  Read GEN Freq Calibration Table
#
proc gencal_freq_read_cal { filename rfport } {
   global GENCAL
   global RFGEN_ENGINE
   
   switch $rfport {
     GEN    {
             set token_chr RFGEN_GEN_FREQ_CHARACTERISTICS
             set token_ele RFGEN_GEN_FREQ_CAL
            }
     TR     {
             set token_chr RFGEN_TR_FREQ_CHARACTERISTICS
             set token_ele RFGEN_TR_FREQ_CAL
            }
     default { return }
   }


  # OPEN CAL FILE
  if [catch { OpenCalFile $filename r } fileId] {
puts "CAUTION: RF Generator Freq Cals for $rfport are not present"
    return
  }

   while { [ gets $fileId line ] >= 0} {

     if { [lindex $line 0] == $token_chr } {
        set RFGEN_ENGINE(FreqMaxIndex)                 [lindex $line 2 ]
        set RFGEN_ENGINE(Normalize.Genfreq.dB0)        [lindex $line 4 ]
        set RFGEN_ENGINE(Normalize.Genfreq.dB10)       [lindex $line 6 ]
        set RFGEN_ENGINE(Normalize.Genfreq.dB20)       [lindex $line 8 ]
        set GENCAL($rfport.caldate)                   [lindex $line 10 ]
        set GENCAL($rfport.caltime)                [lindex $line 12 ]

        continue
     }

     if { [lindex $line 0] == $token_ele } {
        set index                                      [lindex $line 2 ]
        set GENCAL(freq,$rfport,freq,$index)           [lindex $line 4 ]
        set GENCAL(freq,$rfport,dB0,normalize,$index)  [lindex $line 6 ]
        set GENCAL(freq,$rfport,dB10,normalize,$index) [lindex $line 8 ]
        set GENCAL(freq,$rfport,dB20,normalize,$index) [lindex $line 10 ]
        set GENCAL(freq,$rfport,dB0,status,$index)     [lindex $line 12 ]
        set GENCAL(freq,$rfport,dB10,status,$index)    [lindex $line 14 ]
        set GENCAL(freq,$rfport,dB20,status,$index)    [lindex $line 16 ]

        gencal_freq_abs_db_2_dbm $index $rfport
        continue
     }
   }

#  for { set i 0 } { $i < $RFGEN_ENGINE(FreqMaxIndex) } { incr i } {
#           puts [format "  INDEX %4d \
#                           FREQ %5f \
#                           LEVEL0  %7.2f \
#                           LEVEL10 %7.2f \
#                           LEVEL20 %7.2f \
#                           ABS0  %7.2f \
#                           ABS10 %7.2f \
#                           ABS20 %7.2f \
#                           STATUS0  $GENCAL(freq,$rfport,dB0,status,$i) \
#                           STATUS10 $GENCAL(freq,$rfport,dB10,status,$i) \
#                           STATUS20 $GENCAL(freq,$rfport,dB20,status,$i)" \
#                           $i \
#                           $GENCAL(freq,$rfport,freq,$i) \
#                           $GENCAL(freq,$rfport,dB0,normalize,$i) \
#                           $GENCAL(freq,$rfport,dB10,normalize,$i) \
#                           $GENCAL(freq,$rfport,dB20,normalize,$i) \
#                           $GENCAL(freq,$rfport,dB0,abs,$i) \
#                           $GENCAL(freq,$rfport,dB10,abs,$i) \
#                           $GENCAL(freq,$rfport,dB20,abs,$i) \
#                ]
#   }

  close $fileId
}


#
#  upload freq cals RF Gen engine
#
proc gencal_set_engine_freq { rfport } {
   global RFGEN_ENGINE
   global GENCAL

   switch $rfport {
     GEN    {  set which 3 }
     TR     {  set which 2 }
     default { return }
   }

   # update ALC curve in ENGINE
   gencal_set_calconfig $which $RFGEN_ENGINE(FreqMaxIndex)

   gencal_freq_normalize_dbm_2_db $rfport

   set i $RFGEN_ENGINE(FreqMaxIndex)
   incr i -1
 
   # re-spline occures when sending index 0
   for {  } { $i >= 0 } { incr i -1 } {
      gencal_freq_cal $i $rfport $GENCAL(freq,$rfport,freq,$i) \
                                 $GENCAL(freq,$rfport,dB0,normalize,$i) \
                                 $GENCAL(freq,$rfport,dB10,normalize,$i) \
                                 $GENCAL(freq,$rfport,dB20,normalize,$i)
   }
}


#############################################################################
#                                                                           #
#                      GenCal Database Interfaces (ALC)                     #
#                                                                           #
#############################################################################
#
# gencal_alc_capture
#
# this will capture PA_DETECT level, ALC DAC setting and init
# calibration status to CALIBRATED.
#
#
proc gencal_alc_capture { index } {
  global RFGEN_STATUS
  global RFGEN_ENGINE
  global GENCAL

  set GENCAL(level,padetect,$index) $RFGEN_STATUS(yig_padetect)
  set GENCAL(level,alc,$index)      $RFGEN_ENGINE(AlcDac)
  set GENCAL(level,status,$index)   CAL
# set GENCAL(level,abs,$index)      // already set from screen
}



#
#  gencal_alc_set_max
# 
#  initialize ALC curve data
#  create a standard increment and set status to UNCAL
#
proc gencal_alc_set_max { } {
  global GENCAL
  global RFGEN_ENGINE

  # divide by 0 protection
  if { $RFGEN_ENGINE(AlcMaxIndex) == 0 } {
    set RFGEN_ENGINE(AlcMaxIndex) 1
  }
  
  set step [ expr 4096 / $RFGEN_ENGINE(AlcMaxIndex) ]
  set s 4095

  for { set i 0 } { $i < $RFGEN_ENGINE(AlcMaxIndex) } { incr i } {
     set GENCAL(level,status,$i)       UNCAL
     set GENCAL(level,padetect,$i)     0
     set GENCAL(level,abs,$i)          $RFGEN_ENGINE(Normalize.Genalc)
     set GENCAL(level,normalize,$i)    0.0
     set GENCAL(level,alc,$i)          $s

     set s [ expr $s - $step ]
  }
}



#
#  gencal_alc_normalize_dbm_2_db
#
#  
#
proc gencal_alc_normalize_dbm_2_db { } {
  global GENCAL
  global RFGEN_ENGINE

  for { set i 0 } { $i < $RFGEN_ENGINE(AlcMaxIndex) } { incr i } {
     set GENCAL(level,normalize,$i) \
      [ expr $GENCAL(level,abs,$i) - $RFGEN_ENGINE(Normalize.Genalc) ]
  }
}

#
#  gencal_alc_abs_db_2_dbm
#
#
proc gencal_alc_abs_db_2_dbm { index } {
   global GENCAL
   global RFGEN_ENGINE
   
   set GENCAL(level,abs,$index) \
    [ expr $GENCAL(level,normalize,$index) + $RFGEN_ENGINE(Normalize.Genalc) ]
}


#############################################################################
#                                                                           #
#                      GenCal Database Interfaces (Freq)                    #
#                                                                           #
#############################################################################
#
# gencal_freq_capture
#
# this will capture PA_DETECT level, RF Output Power and init
# calibration status to CALIBRATED.
#
#
proc gencal_freq_capture { index } {
  global RFGEN_STATUS
  global RFGEN_ENGINE
  global GENCAL

  set rfport $RFGEN_ENGINE(Port)
  set step $RFGEN_ENGINE(Step)

  set GENCAL(freq,$rfport,dB$step,status,$index)    CAL
  set GENCAL(freq,$rfport,dB$step,padetect,$index)  $RFGEN_STATUS(yig_padetect)

  if {$rfport == "TR"} {
     set GENCAL(notsaved_tr) 1
  } else {
     set GENCAL(notsaved_gen) 1
  }
}


#
#  gencal_freq_set_max
# 
#  initialize FREQ curve data
#  create a standard increment and set status to UNCAL
#
proc gencal_freq_set_max { rfport } {
  global GENCAL
  global RFGEN_ENGINE

  # divide by 0 protection
  if { $RFGEN_ENGINE(FreqMaxIndex) == 0 } {
    set RFGEN_ENGINE(FreqMaxIndex) 1
  }
  
  set steps { dB0 dB10 dB20 }
  set freq_step [ expr 2700 / $RFGEN_ENGINE(FreqMaxIndex) ]
  set f 0

  for { set i 0 } { $i < $RFGEN_ENGINE(FreqMaxIndex) } { incr i } {
      foreach s $steps {
        set GENCAL(freq,$rfport,freq,$i)         $f
        set GENCAL(freq,$rfport,$s,normalize,$i) 0.0
        set GENCAL(freq,$rfport,$s,abs,$i)       $RFGEN_ENGINE(Normalize.Genfreq.$s)
        set GENCAL(freq,$rfport,$s,padetect,$i)  0
        set GENCAL(freq,$rfport,$s,status,$i)    UNCAL

        incr f $freq_step
      }
  }
}


#
#  gencal_freq_normalize_dbm_2_db
#
#  
#
proc gencal_freq_normalize_dbm_2_db { rfport } {
  global GENCAL
  global RFGEN_ENGINE

  set steps { dB0 dB10 dB20 }

  for { set i 0 } { $i < $RFGEN_ENGINE(FreqMaxIndex) } { incr i } {
      foreach s $steps {
        set GENCAL(freq,$rfport,$s,normalize,$i) \
            [ expr $GENCAL(freq,$rfport,$s,abs,$i) - \
                   $RFGEN_ENGINE(Normalize.Genfreq.$s) ]

#puts "$i $GENCAL(freq,$rfport,$s,normalize,$i) \
#      $GENCAL(freq,$rfport,$s,abs,$i) \
#      $RFGEN_ENGINE(Normalize.Genfreq.$s) "
      }
  }
}


#
#  gencal_freq_abs_db_2_dbm
#
#
proc gencal_freq_abs_db_2_dbm { index rfport } {
   global GENCAL
   global RFGEN_ENGINE

   set steps { dB0 dB10 dB20 }

   foreach s $steps {
     set GENCAL(freq,$rfport,$s,abs,$index) \
         [ expr $GENCAL(freq,$rfport,$s,normalize,$index) + \
                $RFGEN_ENGINE(Normalize.Genfreq.$s) ]

#puts "$index $GENCAL(freq,$rfport,$s,normalize,$index) \
#      $GENCAL(freq,$rfport,$s,abs,$index) \
#      $RFGEN_ENGINE(Normalize.Genfreq.$s) "
   }
}

#############################################################################
#                                                                           #
#                      GenCal Engine Spline Curve                           #
#                                                                           #
#############################################################################
#
# gencal_spline_info
# 
# retrieve spline curve point from engine, this will send the outgoing
#  message, the response will appear in ???
#
#       type - 1 ALC Correction Curve
#              2 T/R Freq Correction Curve
#              3 GEN Freq Correction Curve
#
#       atten - for FREQ curve, use atten ( 0=0db, 16=10db, 32=20db )
#
#       thing1 - freq or dB
#              for ALC curve, use dB here
#              for FREQ curve, use Freq in MHz
#
#       thing2 - Spline curve result will be returned here
#
proc gencal_spline_info { type atten thing1 } {
  global RFGEN_ENGINE
  global Send_Flags

  set thing0 $type

  switch $atten {
    dB0   { set thing0 [ expr $type + 0  ] }
    dB10  { set thing0 [ expr $type + 16 ] }
    dB20  { set thing0 [ expr $type + 32 ] }
  }

put "spline info called: type of $type, atten is $atten, thing is $thing1"

  set msg " { $RFGEN_ENGINE(SYSTEM_ID) \
              $RFGEN_ENGINE(SUB_SPLINEINFO) 1 $Send_Flags } \
            { inline dw $thing0 } \
            { inline double $thing1 } \
            { inline double 0.0 } "

  tclMsgbus send $msg
}

